# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.iee-einstein.org/" target="_blank">www.iee-einstein.org</a>)
#
#------------------------------------------------------------------------------
#
#	GUI Tools
#
#------------------------------------------------------------------------------
#
#       Common functions used in sevaral GUI panels
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
import wx
from math import isnan, isinf
from constants import *    #not needed here, but in the Panels !!!
from modules.messageLogger import *
from auxiliary.auxiliary import toGettextAndUnicode as _U
from auxiliary.auxiliary import toUnicode as U
from warnings import warn

# EINSTEIN colours
ORANGE = '#FF6000'
SOFTORANGE = '#FFA066'
WHITE = '#FFFFFF'
LIGHTGREY = '#F6F6F6'
MIDDLEGREY = '#C0C0C0'
DARKGREY = '#000060'
BLACK = '#000000'

YELLOW = '#FFFF00'
LIGHTYELLOW = '#FFFF80'
RED = '#FF0000'
BLUE = '#0000FF'
LIGHTRED = '#FFA0A0'
DARKGREEN = '#00A000'
LIGHTGREEN = '#F0FFF0'

# general base colours for GUI layout
TITLE_COLOR = ORANGE                # used for titles in static boxes etc.
TITLE_FONTSIZE = 10                      

# colours for data entry fields (general)
COLOUR_DATAENTRY_BG = WHITE         # editable data entry fields
COLOUR_READONLY_BG = LIGHTGREY      # read-only data entry fields
COLOUR_OVERWRITE_FG = DARKGREY      # calculated data in data entry fields which may be overwritten

COLOUR_MISSING_BG = LIGHTRED        # highlight for missing parameters after consistency check
COLOUR_MISSING_FG = RED

COLOUR_CALCULATED_BG = LIGHTGREEN   # calculated parameters after consistency check
COLOUR_CALCULATED_FG = DARKGREEN

COLOUR_CONFLICT_BG = LIGHTYELLOW    # parameters in conflict after consistency check
COLOUR_CONFLICT_FG = RED

COLOUR_CONFLICTSRC_BG = YELLOW      # parameters source of conflict after consistency check
COLOUR_CONFLICTSRC_FG = BLACK

# specific parameters for tables (wx.grid)
GRID_BACKGROUND_COLOR = COLOUR_READONLY_BG # idem
GRID_BACKGROUND_COLOR_HIGHLIGHT = SOFTORANGE # idem

GRID_LETTER_SIZE = 8 #points
GRID_LABEL_SIZE = 8  # points
GRID_LETTER_COLOR = DARKGREY     # specified as hex #RRGGA
GRID_LETTER_COLOR_HIGHLIGHT = WHITE     # specified as hex #RRGGA

# specific parameters for plots
GRAPH_BACKGROUND_COLOR = WHITE # idem

ORANGECASCADE = ['#FF6000','#EE5000','#DD4000','#CC2000','#BB1000',\
                 LIGHTGREY,MIDDLEGREY,DARKGREY,BLACK]
LINETYPES = ['-','--',':','.','-.',]

MARKERTYPES = ['s', 'o', 'v', '^', '+', 'D']

#..............................................................................
#..............................................................................
#..............................................................................
#   Several auxiliary functions for GUI 
#..............................................................................
#..............................................................................
#..............................................................................

def graphSquareFromSaw(resolution, sawValues):
    """Extent to square wave from sawtooth function values
    
    :param resolution: Number of square values for one sawtooth value
    :type  resolution: Integer
    :param sawValues: Sawtooth function values
    :type  sawValues: List
    :returns: Square function values
    :rtype:   List
    """
    squareValues = []
    for y in sawValues:
        squareValues.extend([y]*resolution)
    return squareValues

#------------------------------------------------------------------------------
def fillChoice(choice,choiceList,nonePossible=True):
#------------------------------------------------------------------------------
#   fills the list of possible choices from a list of strings
#------------------------------------------------------------------------------
    choice.Clear()
    if nonePossible==True:choice.Append("None")
    for c in choiceList:choice.Append(U(c))
    choice.SetSelection(0)
#------------------------------------------------------------------------------		
#------------------------------------------------------------------------------		
def setChoice(choice,strChoice):
#------------------------------------------------------------------------------		
#   sets a choice to the string
#------------------------------------------------------------------------------		
    try:choice.SetSelection(choice.FindString(U(strChoice)))
    except:choice.SetSelection(-1)

def check(value, length=None, replacement="NULL"):
    """Substitute "", None and NaN by 'NULL' and truncate to length, if 
    it's a string longer than length and length is given.
    
    You may replace "NULL" by any string you like via the replacement 
    parameter. 

    (should be moved some day to a separate file with sql-tools ...)

    """
    if value != "" and value != "None" and value is not None:
        if isinstance(value, str) or isinstance(value, unicode):
##            logDebug("GuiTools (check): string or unicode instance detected: %r"%value)
            return U(value).encode("utf-8")[0:length]
        else:
            try:
                if isnan(value):
                    warn("Check encountered a NaN value. This should not happen.", stacklevel=2)
                    return str(replacement)
                elif isinf(value):
                    warn("Check encountered an inf value. This should not happen.", stacklevel=2)
                    return str(replacement)
            except TypeError: # is not a number type checking for NaN is senseless
                pass
            try:
                return str(value)[0:length]
            except:
                return value
    else:
        return str(replacement)
