#Boa:Frame:PreferencesFrame
# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       www.einstein-energy.net
#
#------------------------------------------------------------------------------
#
#	Preferences Frame
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2020
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
import wx
from GUI.status import *
from GUI.GUITools import TITLE_COLOR
import database.dbio as dbio
from constants import *
from GUI.displayClasses import FieldSizes, ChoiceEntry
from GUI.dialogOK import DialogOK
from GUI.fonts import FontProperties
from auxiliary.auxiliary import toGettextAndUnicode as _U
from globals import logger

# constants that control the default sizes
# 1. font sizes
TYPE_SIZE_LEFT    =   9
TYPE_SIZE_RIGHT   =   9
TYPE_SIZE_TITLES  =  10

# 2. field sizes
HEIGHT_LEFT        =  29
LABEL_WIDTH_LEFT   = 340
HEIGHT_RIGHT       =  26
LABEL_WIDTH_RIGHT  = 300
DATA_ENTRY_WIDTH   = 160
UNITS_WIDTH        = 100

def create(parent):
    return PreferencesFrame(parent)

[wxID_FRAME1, wxID_FRAME1NOTEBOOK1, wxID_FRAME1STATICTEXT1, 
 wxID_FRAME1STATICTEXT2, 
] = [wx.NewId() for _init_ctrls in range(4)]

class PreferencesFrame(wx.Dialog):
    def __init__(self, parent):
        self._init_ctrls(parent)
        self.__do_layout()
##        self.notebook.SetSelection(1)
##        self.page1.Hide()
        self.fillPage()

    def _init_ctrls(self, prnt):
        # generated method, don't edit
        wx.Dialog.__init__(self, id=wxID_FRAME1, name='', parent=prnt,
              pos=wx.Point(300, 140), size=wx.Size(640, 180),
              style=wx.DEFAULT_FRAME_STYLE, title='EINSTEINs preferences')
        self.SetClientSize(wx.Size(640, 180))

        # access to font properties object
        fp = FontProperties()
        fp.pushFont()
        fp.changeFont(size=9)

        self.notebook = wx.Notebook(id=wxID_FRAME1NOTEBOOK1, name='notebook',
              parent=self, pos=wx.Point(0, 0), size=wx.Size(640, 140), style=0)

##        self.page1 = wx.Panel(self.notebook)
        self.page3 = wx.Panel(self.notebook)

        self.notebook.SetFont(fp.getFont())
        self.page3.SetFont(fp.getFont())

##        self.boxDebug = wx.StaticBox(self.page1, -1, _U("Debugging options"))
##        self.boxDebug.SetForegroundColour(TITLE_COLOR)
##        self.boxDebug.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))
##
##        self.par1_1 = ChoiceEntry(self.page1,
##                               values=DEBUGMODES,
##                               label=_U("Debug mode for consistency check:"),
##                               tip=_U("This will create different volume of output for debugging:"))        

        self.boxHR = wx.StaticBox(self.page3, -1, _U("Heat recovery calculation"))
        self.boxHR.SetForegroundColour(TITLE_COLOR)
        self.boxHR.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        fs = FieldSizes(wHeight=HEIGHT_LEFT,wLabel=LABEL_WIDTH_LEFT,
                       wData=DATA_ENTRY_WIDTH,wUnits=UNITS_WIDTH)


        self.entryHRModes = ChoiceEntry(self.page3,
                               values=[TRANSHRMODES[mode] for mode in HRMODES[1:]],
                               label=_U("Method for calculation of heat exchanger network"),
                               tip=_U("estimate is based on HR potential and does not account for real HXs"),
                                        allowNones = False)

        self.buttonCancel = wx.Button(self,wx.ID_CANCEL,_U("Cancel"))
        self.Bind(wx.EVT_BUTTON, self.OnButtonCancel, self.buttonCancel)
        self.buttonCancel.SetFont(fp.getFont())

        self.buttonOK = wx.Button(self,wx.ID_OK,_U("OK"))
        self.Bind(wx.EVT_BUTTON, self.OnButtonOK, self.buttonOK)
        
        self.buttonOK.SetDefault()
        self.buttonOK.SetFont(fp.getFont())

        fp.popFont()

    def __do_layout(self):
        # generated method, don't edit
        sizer_frame = wx.BoxSizer(wx.VERTICAL)
        sizer_buttons = wx.BoxSizer(wx.HORIZONTAL)

##        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_2 = wx.BoxSizer(wx.VERTICAL)
        sizer_HR = wx.StaticBoxSizer(self.boxHR, wx.VERTICAL)
##        sizer_Debug = wx.StaticBoxSizer(self.boxDebug, wx.VERTICAL)

        flagLabel = wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM
        flagText = wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM

        sizer_buttons.Add(self.buttonCancel, 0, flagText, 1)
        sizer_buttons.Add(self.buttonOK, 0, flagText, 1)
        
        # Left tab. Processes description
##        sizer_Debug.Add(self.par1_1, 0, flagText, 1)
##
##        sizer_1.Add(sizer_Debug, 0, flagText, 1)

        sizer_HR.Add(self.entryHRModes, 0, flagText, 1)

        sizer_2.Add(sizer_HR, 0, flagText, 1)

##        self.page1.SetSizer(sizer_1)
        self.page3.SetSizer(sizer_2)
                
##        self.notebook.AddPage(self.page1, _U('debug options'))
        self.notebook.AddPage(self.page3, _U('calculation tools'))

        sizer_frame.Add(self.notebook, 0, flagText, 1)
        sizer_frame.Add(sizer_buttons)

        self.SetSizer(sizer_frame)

        self.Layout()

    def OnButtonCancel(self, event):
##        confirm =  DialogOK(self,_U("exit settings"),_U("do you want to exit without saving ?"))
##        if confirm.ShowModal() == wx.ID_OK: self.EndModal(wx.ID_CANCEL)
        self.EndModal(wx.ID_CANCEL)

    def OnButtonOK(self, event):

##        debugMode = self.par1_1.GetValue(text="True")
##        if debugMode in DEBUGMODES:
##            Status.Debug = debugMode
##        else:
##            logDebug("PreferencesFrame: error in selected debug Mode. Value chosen = %s"%debugMode)
            
        hrModeIndex = self.entryHRModes.GetValue()
        if hrModeIndex is None: hrModeIndex = 1   # HRMODES[0] no longer supported
        hrMode = HRMODES[hrModeIndex+1]

        if hrMode != Status.HRTool:
            ret = logger.askConfirmation(_("Method for calculation of heat exchanger network") + " " + \
                                         _("changed to") + " " + TRANSHRMODES[hrMode] + "\n" + \
                                         _("This change will be applied to all alternative proposals in the project"))

            if ret:
                Status.prj.setHRMode(hrMode)
            else:
                self.fillPage()
                return

        self.EndModal(wx.ID_OK)

    def fillPage(self):
        index = HRMODES[1:].index(Status.HRTool) if Status.HRTool in HRMODES[1:] else None
        self.entryHRModes.SetValue(index)

