#Boa:Frame:conflictFrame
# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	conflictFrame
#			
#------------------------------------------------------------------------------
#			
#	windows for editing conflicts between parameters in cross checking
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
import wx
import wx.grid
from GUI.status import Status
from modules.ccheck.moduleCC import *
from modules.messageLogger import *
from modules.fluids import Fuel
from GUITools import *
from auxiliary.auxiliary import toUnicode as U
from auxiliary.auxiliary import toGettextAndUnicode as _U
from conflictSourcesFrame import ConflictSourcesFrame

def create(parent):
    return conflictFrame(parent)

[wxID_CONFLICTFRAME, wxID_CONFLICTFRAMEBUTTON1, wxID_CONFLICTFRAMEBUTTON2, 
 wxID_CONFLICTFRAMEGRID, 
] = [wx.NewId() for _init_ctrls in range(4)]

#
# constants
#

MAXROWS = 150
COLNO = 3
tableKeys = ["1","2","3","4","5","6"]
labels_column = -1

class conflictFrame(wx.Dialog):
    def _init_ctrls(self, prnt):
        # generated method, don't edit
        wx.Dialog.__init__(self, id=wxID_CONFLICTFRAME, name=u'conflictFrame',
              parent=prnt, pos=wx.Point(600, 400), size=wx.Size(900, 660),
              style=wx.DEFAULT_FRAME_STYLE, title=self.title)
        self.SetClientSize(wx.Size(940, 660))

        self.grid = wx.grid.Grid(id=wxID_CONFLICTFRAMEGRID, name='grid',
              parent=self, pos=wx.Point(0, 0), size=wx.Size(900, 600),
              style=wx.WANTS_CHARS | wx.VSCROLL | wx.HSCROLL)

        self.Bind(wx.grid.EVT_GRID_EDITOR_SHOWN, self.OnGridEdit, id=wxID_CONFLICTFRAMEGRID)
        self.Bind(wx.grid.EVT_GRID_CELL_CHANGE, self.OnGridEditStore,
              id=wxID_CONFLICTFRAMEGRID)
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_CLICK, self.OnGridCellLeftClick,
              id=wxID_CONFLICTFRAMEGRID)
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.OnGridCellLeftDClick,
              id=wxID_CONFLICTFRAMEGRID)

        self.button1 = wx.Button(id=wx.ID_CANCEL, label=_('Cancel'),
              name='button1', parent=self, pos=wx.Point(772, 610),
              size=wx.Size(128, 32), style=0)

        self.Bind(wx.EVT_BUTTON, self.OnButtonCancel, id=wx.ID_CANCEL)

        self.button2 = wx.Button(id=wx.ID_OK, label=_('OK'), name='button2',
              parent=self, pos=wx.Point(632, 610), size=wx.Size(128, 32),
              style=0)

        self.Bind(wx.EVT_BUTTON, self.OnButtonOK, id=wx.ID_OK)

        self.Bind(wx.EVT_CLOSE, self.OnClosure)

    def __init__(self, parent):
        self.title = _U("conflicts in parameter specifications")
        self._init_ctrls(parent)
        self.prnt = parent

        #self.maxrow = 25
        self.rowcount = 0
        self.theId = -1
        self.grid.EnableEditing(False)
        self.backcolor = GRID_BACKGROUND_COLOR

        self.lastEditRow = 0
        self.lastEditCol = 0

        self.setupGrid()
        self.displayData()
        
    def setupGrid(self):
        self.grid.CreateGrid(MAXROWS, COLNO)

        for col in range(COLNO):
            self.grid.SetColLabelValue(col, tableKeys[col])

#        self.grid.AutoSizeColumns(setAsMin=True)
        self.grid.EnableEditing(False)
        self.grid.SetSelectionMode(wx.grid.Grid.wxGridSelectRows)

        self.grid.EnableGridLines(True)
        self.grid.SetDefaultRowSize(20)
        self.grid.SetRowLabelSize(30)
        self.grid.SetDefaultColSize(100)
        self.grid.SetColSize(0,200)
        self.grid.SetColSize(2,600)
#        self.grid.SetColSize(4,200)

        self.grid.SetLabelFont(wx.Font(9, wx.ROMAN, wx.ITALIC, wx.BOLD))

        self.grid.SetColLabelValue(0, _U("parameter\n(values in conflict)"))
        self.grid.SetColLabelValue(1, _U("data group\n(accuracy)"))
        self.grid.SetColLabelValue(2, _U("description\n(calculated from)"))

        for r in range(MAXROWS):

            if r%3 == 0:    #main row of entry
                attr = wx.grid.GridCellAttr()
                attr.SetTextColour(GRID_LETTER_COLOR_HIGHLIGHT)
                attr.SetBackgroundColour(GRID_BACKGROUND_COLOR_HIGHLIGHT)
                attr.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.BOLD))

                self.grid.SetRowAttr(r, attr)

                self.grid.SetCellAlignment(r, 0, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);
                self.grid.SetCellAlignment(r, 1, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE);
                self.grid.SetCellAlignment(r, 2, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);                
            else:
                attr = wx.grid.GridCellAttr()
                attr.SetTextColour(GRID_LETTER_COLOR)
                attr.SetBackgroundColour(GRID_BACKGROUND_COLOR)
                attr.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.NORMAL))

                self.grid.SetRowAttr(r, attr)

                self.grid.SetCellAlignment(r, 0, wx.ALIGN_RIGHT, wx.ALIGN_CENTRE);
                self.grid.SetCellAlignment(r, 1, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE);
                self.grid.SetCellAlignment(r, 2, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);                
                

        self.grid.SetGridCursor(0, 0)
    

    def displayData(self):

#..............................................................................
# display data

        try:
            data = Interfaces.GData["CC Conflict"]
            (rows,cols) = data.shape
            for r in range(rows):
                for c in range(cols):
                    self.grid.SetCellValue(r, c, U(data[r][c]))

            for r in range(rows,MAXROWS):
                for c in range(cols):
                    self.grid.SetCellValue(r, c, "")            
        except:
            pass


    def OnGridEdit(self, event):
        self.lastEditRow = self.grid.GetGridCursorRow()
        self.lastEditCol = self.grid.GetGridCursorCol()
        event.Skip()
        
    def OnGridEditStore(self, event):
        value = self.grid.GetCellValue(self.lastEditRow, self.lastEditCol)
#        row = self.table.select({self.table.keys()[0]:self.grid.GetCellValue(self.lastEditRow,0)})[0]

        if value <> "" and value <> "None":
            row[self.lastEditCol] = value
        else:
            row[self.lastEditCol] = 'NULL'
        self.lastEditRow = 0
        self.lastEditCol = 0
        event.Skip()

    def OnGridCellLeftClick(self, event):
##	try:
##	    self.theId = int(self.grid.GetCellValue(event.GetRow(), self.col_returned))
##	except:
##	    self.theId = -1

        event.Skip()


    def OnGridCellLeftDClick(self, event):
        row = event.GetRow()
        col = event.GetCol()

        if row%3 > 0:

            data = []
            baseRow = row - (row%3)
            dataGroup = toUTF8(self.grid.GetCellValue(baseRow, 1))
            items = dataGroup.split('[')
            objectKey = items[0]
            try:
                objectNo = int(items[1].strip(']'))
            except:
                objectNo = None

            sources = toUTF8(self.grid.GetCellValue(row, 2))

            sourceList = sources.split(";")
            displayedSources = []
            for sourceStr in sourceList:
                sourceStr = sourceStr.strip(" ")
                if sourceStr not in displayedSources:
                    displayedSources.append(sourceStr)
                    if len(sourceStr) > 0:
                        if sourceStr[0] == "#": #parameter comes from another object
                            sourceElements = sourceStr[1:].split('[')
                            source = sourceElements[0]
                            try:
                                objectNo = int(sourceElements[1].strip(']'))
                            except:
                                objectNo = None
    ##                        print "source %s objectNo %s"%(source, objectNo)
                            if source in ["UPH", "UPHProck"]:
                                objectKey = "Process"
                                objectType = _("Process")
                            elif source in ["UPHProcm","USHm"]:
                                objectKey = "Pipe/Duct"
                                objectType = _("Pipe/Duct")
                            elif source in ["USHj","FETj","FETfuel_j","FETFuel_j","FETel_j", "OandMfix", "OandMvar"]:
                                objectKey = "Equipment"
                                objectType = _("Equipment")
                            elif source in ["FETi"]:
                                objectKey = "FET"
                                objectType = _("Fuel")
                            elif source in ["QHX", "OMFix", "OMVar"]:
                                objectKey = "HX"
                                objectType = _("Heat exchanger")
                            else:
                                objectKey = "?"
                                objectType = "Unknown OT - report bug: %s"%source
                        else: #parameter is from the main indicated object
                            source = sourceStr

                        if objectKey == "Process":
                            objectType = _("Process")
                            if objectNo is not None:
                                processes = Status.prj.getProcesses()
                                process = processes[objectNo-1]
                                objectName = process.Process
                            else:
                                objectNo = "-"
                                objectName = _("All processes")
                        elif objectKey == "Pipe/Duct":
                            objectType = _("Pipe/Duct")
                            if objectNo is not None:
                                pipes = Status.prj.getPipes()
                                pipe = pipes[objectNo-1]
                                objectName = pipe.Pipeduct
                            else:
                                objectNo = "-"
                                objectName = _("All pipes")
                        elif objectKey == "Equipment":
                            objectType = _("Equipment")
                            if objectNo is not None:
                                eqs = Status.prj.getEquipments(cascade=True)
                                eq = eqs[objectNo-1]
                                objectName = eq.Equipment
                            else:
                                objectNo = "-"
                                objectName = _("All equipment")
                        elif objectKey in ["FET", "Fuel"]:
                            objectType = _("Fuel consumption")
                            if objectNo is not None:
                                fuels = Status.prj.getFuels()
                                fuelID = fuels[objectNo-1].DBFuel_id
                                fuel = Fuel(fuelID)
                                objectName = fuel.name
                            else:
                                objectNo = "-"
                                objectName = _("All equipment")
                        elif objectKey == "FETel":
                            objectType = _("Electricity consumption")
                            if objectNo is not None:
                                objectName = _("Electricity")
                            else:
                                objectNo = "-"
                                objectName = _("All equipment")
                        elif objectKey == "HX":
                            objectType = _("Heat exchanger")
                            if objectNo is not None:
                                hxes = Status.prj.getHXes()
                                hx = hxes[objectNo-1]
                                objectName = hx.HXName
                            else:
                                objectNo = "-"
                                objectName = _("All heat exchangers")
                        elif objectKey == "Totals":
                            objectType = _("Totals")
                            if objectNo is None:
                                objectNo = "-"
                            objectName = "-"
                        else:
##                            print "TEST TEST Unknown object key %s"%objectKey
                            objectType = "?"
                            objectNo = "?"
                            objectName = "?"

##                        print "t: %s no: %s name: %s"%(objectType, objectNo, objectName)

                        try:
                            parDescription = Status.mod.moduleCC.parameterList[source]
                        except:
                            parDescription = ""
                        
                        parInfo = [objectType, str(objectNo), objectName, source, parDescription]
##                        print parInfo
                        data.append(parInfo)

            self.csf = ConflictSourcesFrame(self, data)
                

    def OnButtonCancel(self,event):
        event.Skip()

    def OnButtonOK(self,event):
        event.Skip()

    def OnClosure(self,event):
        try:
            self.csf.Destroy()
        except:
            pass
        event.Skip()


    
