#Boa:Frame:conflictFrame
# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	conflictFrame
#			
#------------------------------------------------------------------------------
#			
#	windows for editing conflicts between parameters in cross checking
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
import wx
import wx.grid
from GUI.status import Status
from modules.ccheck.moduleCC import *
from modules.messageLogger import *
from modules.fluids import Fuel
from GUITools import *
from auxiliary.auxiliary import toUnicode as U
from auxiliary.auxiliary import toGettextAndUnicode as _U

MAXROWS_CS = 20
COLNO_CS = 5

def create(parent):
    return ConflictSourcesFrame(parent, None)

[wxID_CONFLICTSOURCESFRAME, wxID_CONFLICTSOURCESFRAMEBUTTON1, wxID_CONFLICTSOURCESFRAMEBUTTON2, 
 wxID_CONFLICTSOURCESFRAMEGRID, 
] = [wx.NewId() for _init_ctrls in range(4)]

class ConflictSourcesFrame(wx.Frame):
    """Secondary conflict dialog for the detail display of sources
    """
    
    def _init_ctrls(self, prnt):
        # generated method, don't edit
        wx.Frame.__init__(self, id=wxID_CONFLICTSOURCESFRAME, name=u'conflictSourcesFrame',
              parent=prnt, pos=wx.Point(300, 460), size=wx.Size(800, 480),
              style=wx.DEFAULT_FRAME_STYLE, title=self.title)
        self.SetClientSize(wx.Size(800, 440))

        self.grid = wx.grid.Grid(id=wxID_CONFLICTSOURCESFRAMEGRID, name='grid',
              parent=self, pos=wx.Point(0, 0), size=wx.Size(800, 440),
              style=wx.WANTS_CHARS | wx.VSCROLL | wx.HSCROLL)

        self.button1 = wx.Button(id=wx.ID_CANCEL, label=_('Cancel'),
              name='button1', parent=self, pos=wx.Point(772, 410),
              size=wx.Size(128, 32), style=0)

        self.Bind(wx.EVT_BUTTON, self.OnButtonCancel, id=wx.ID_CANCEL)

        self.button2 = wx.Button(id=wx.ID_OK, label=_('OK'), name='button2',
              parent=self, pos=wx.Point(632, 410), size=wx.Size(128, 32),
              style=0)

        self.Bind(wx.EVT_BUTTON, self.OnButtonOK, id=wx.ID_OK)

    def __init__(self, parent, data):
        self.title = _U("Sources of ")+_U("conflicts in parameter specifications")
        self._init_ctrls(parent)

        #self.maxrow = 25
        self.rowcount = 0
        self.theId = -1
        self.grid.EnableEditing(False)
        self.backcolor = GRID_BACKGROUND_COLOR

        self.lastEditRow = 0
        self.lastEditCol = 0

        self.setupGrid()
        
        self.data = data
        self.displayData()
        
    def setupGrid(self):
        self.grid.CreateGrid(MAXROWS_CS, COLNO_CS)

##        for col in range(COLNO_CS):
##            self.grid.SetColLabelValue(col, tableKeys[col])

#        self.grid.AutoSizeColumns(setAsMin=True)
        self.grid.EnableEditing(False)
        self.grid.SetSelectionMode(wx.grid.Grid.wxGridSelectRows)

        self.grid.EnableGridLines(True)
        self.grid.SetDefaultRowSize(20)
        self.grid.SetRowLabelSize(30)
        self.grid.SetDefaultColSize(160)
        self.grid.SetColSize(0,80)
        self.grid.SetColSize(1,60)
        self.grid.SetColSize(4,300)

        self.grid.SetLabelFont(wx.Font(9, wx.ROMAN, wx.ITALIC, wx.BOLD))

        self.grid.SetColLabelValue(0, _U("Object Type"))
        self.grid.SetColLabelValue(1, _U("No."))
        self.grid.SetColLabelValue(2, _U("Object Name"))
        self.grid.SetColLabelValue(3, _U("Short Name"))
        self.grid.SetColLabelValue(4, _U("Description"))

        for r in range(MAXROWS_CS):

                attr = wx.grid.GridCellAttr()
                attr.SetTextColour(GRID_LETTER_COLOR)
                attr.SetBackgroundColour(GRID_BACKGROUND_COLOR)
                attr.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.NORMAL))

                self.grid.SetRowAttr(r, attr)

                self.grid.SetCellAlignment(r, 0, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);
                self.grid.SetCellAlignment(r, 1, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE);
                self.grid.SetCellAlignment(r, 2, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);                
                self.grid.SetCellAlignment(r, 3, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);                
                self.grid.SetCellAlignment(r, 4, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);                
                

        self.grid.SetGridCursor(0, 0)
    

    def displayData(self):

#..............................................................................
# display data

        data = self.data
        rows = min(len(data), MAXROWS_CS)
##        (rows,cols) = data.shape
        for r in range(rows):
            for c in range(COLNO_CS):
                try:
                    self.grid.SetCellValue(r, c, U(data[r][c]))
                except:
                    pass

        for r in range(rows,MAXROWS_CS):
            for c in range(COLNO_CS):
                self.grid.SetCellValue(r, c, "")            

        self.Show()


    def OnButtonOK(self, event):
        self.Destroy()

    def OnButtonCancel(self, event):
        pass

    
