# -*- coding: utf-8 -*-
#==============================================================================
#
#   E I N S T E I N
#
#   Expert System for an Intelligent Supply of Thermal Energy in Industry
#   (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#   DialogGauge
#
#------------------------------------------------------------------------------
#
#   Gauge representation
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
import wx
from auxiliary.auxiliary import toGettextAndUnicode as _U
from auxiliary.auxiliary import toUnicode as U
from GUI.status import Status

# imports required for panel
import modules.matPanel as Mp
import matplotlib.font_manager as font
from matplotlib.ticker import FuncFormatter
import textwrap
from GUITools import *
from GUI.numCtrl import format_int_wrapper, convertDoubleToString
import numpy as np
from globals import Globals

axeslabel_fontsize = 10
axesticks_fontsize = 8
legend_fontsize = 10
spacing_left = 0.1
spacing_right = 0.9
spacing_bottom = 0.1
spacing_top = 0.85


[wxID_DIALOGGAUGE, wxID_STGAUGE, wxID_GAUGE, wxID_BUTTONCANCEL, wxID_BUTTONOK
] = [wx.NewId() for _init_ctrls in range(5)]

def drawFigure(self):
    """Define the figures to be plotted"""
    # if not hasattr(self, 'subplot'):
    data = Status.int.GData["GAUGE"]

    if not hasattr(self, 'subplot'):
        self.subplot = self.figure.add_subplot(1, 1, 1)
        
    self.figure.subplots_adjust(left=spacing_left, right=spacing_right, bottom=spacing_bottom, top=spacing_top)

    ncurves = len(data)
    
    self.subplot.clear()
    for j in range(1, ncurves):
        self.subplot.plot(data[0][1:],
                          data[j][1:],
                          LINETYPES[(j - 1) % len(LINETYPES)],
                          color=ORANGECASCADE[(j - 1) % len(ORANGECASCADE)],
                          label=data[j][0],
                          linewidth=2)

##    self.subplot.axis(ymin=0)
    self.subplot.legend(loc=0)   #4: left lower; 0: best

##    major_formatter = FuncFormatter(format_int_wrapper)
##    self.subplot.axes.xaxis.set_major_formatter(major_formatter)
##    self.subplot.axes.yaxis.set_major_formatter(major_formatter)
    fp = font.FontProperties(size=axeslabel_fontsize)

    label = _(' ')
    padding = 10 if len(label) > 40 else None
    self.subplot.axes.set_ylabel(textwrap.fill(U(label), 40, break_long_words=False),
                                 fontproperties=fp,
                                 horizontalalignment='center',
                                 labelpad=padding)
    self.subplot.axes.set_xlabel(_U(' '), fontproperties=fp)
##        self.subplot.axis(xmax=52)

    for label in self.subplot.axes.get_yticklabels():
#        label.set_color(self.params['ytickscolor'])
        label.set_fontsize(axesticks_fontsize)
#        label.set_rotation(self.params['yticksangle'])

    # properties of labels on the x axis
    for label in self.subplot.axes.get_xticklabels():
#        label.set_color(self.params['xtickscolor'])
        label.set_fontsize(axesticks_fontsize)
#        label.set_rotation(self.params['xticksangle'])

    try:
        lg = self.subplot.get_legend()
        ltext = lg.get_texts()             # all the text.Text instance in the legend
        for txt in ltext:
            txt.set_fontsize(legend_fontsize)  # the legend text fontsize
        # legend line thickness
        llines = lg.get_lines()             # all the lines.Line2D instance in the legend
        for lli in llines:
            lli.set_linewidth(1.5)          # the legend linewidth
        # color of the legend frame
        # this only works when the frame is painted (see below draw_frame)
        frame = lg.get_frame()             # the patch.Rectangle instance surrounding the legend
        frame.set_facecolor('#F0F0F0')      # set the frame face color to light gray
        # should the legend frame be painted
        lg.draw_frame(False)
    except:
        # no legend
        pass

    self.canvas.draw()
    
class DialogGauge(wx.Dialog):
    def __init__(self, parent, title, message, plot=False):
        self.plot = plot
        self.dialog = plot # create an own dialog only in case of a plot to be displayed. otherwise show on gauge on the status bar
        self.message = message
        self.title = title
        self._init_ctrls(parent)
        self.update(0)
        self.f = 0
        self.cancelled = False
    
    def _init_ctrls(self, prnt):

        if self.plot:
            xsize = 400
            ysize = 480
        else:
            xsize = 400
            ysize = 180
        self.xsize = xsize
        self.ysize = ysize
        
        # generated method, don't edit
        wx.Dialog.__init__(self, id=wxID_DIALOGGAUGE, name="dialogGauge", parent=None,
              pos=wx.Point(500, 300), size=wx.Size(xsize, ysize),
              style=wx.DEFAULT_DIALOG_STYLE, title=U(self.title))
        self.SetClientSize(wx.Size(xsize-10, ysize-20))

        self.stDialog = wx.StaticText(id=wxID_STGAUGE,
              label=U(self.message), name='stDialog', parent=self,
              pos=wx.Point(20, 20), size=wx.Size(xsize-40, 64), style=0)
        self.stDialog.Center(wx.HORIZONTAL)

        self.gauge1 = wx.Gauge(id=wxID_GAUGE, name='gauge1', parent=self,
              pos=wx.Point(20, 80), range=100, size=wx.Size(xsize-40, 20),
              style=wx.GA_HORIZONTAL)

        if self.plot:
            self.panelPlot = wx.Panel(id=-1, name='panelPlotGauge', parent=self,
                  pos=wx.Point(20, 160), size=wx.Size(xsize - 40, ysize - 200),
                  style=wx.TAB_TRAVERSAL | wx.SUNKEN_BORDER)


            paramList = {'labels'      : 0, # labels column
                       'data'        : 1, # data column for this graph
                       'key'         : 'GAUGE', # key for Interface
                       'title'       : self.message, # title of the graph
                       'backcolor'   : GRAPH_BACKGROUND_COLOR, # graph background color
                       'ignoredrows' : []}            # rows that should not be plotted

            dummyData = np.array([["-",0],["-",0],["-",0]])
            Status.int.setGraphicsData("GAUGE", dummyData)
            Mp.MatPanel(self.panelPlot, wx.Panel, drawFigure, paramList)

    def add(self, val):
        if self.f + val < 100:
            self.update(self.f + val)

    def update(self, f, message=None, data=None):
        self.f=f
        if f >= 100:
            try:
                self.EndModal(wx.ID_OK)
            except:
                pass
        else:
            self.message = self.message if message is None else message

            if self.dialog:
                self.stDialog.SetLabel(U(self.message) + 
                                       _U("\n(%6.2f percent completed)") % int(f))
                self.gauge1.SetValue(int(f))
            else:
                Globals.GUI.panelInfo.showProgress(U(self.message) + _(" [%6.2f %%]")%int(f), int(f))
        
        if self.plot and data is not None:
##            self.panelPlot.Destroy()
##            
##            self.panelPlot = wx.Panel(id=-1, name='panelPlotGauge', parent=self,
##                  pos=wx.Point(20, 160), size=wx.Size(self.xsize - 40, self.ysize - 200),
##                  style=wx.TAB_TRAVERSAL | wx.SUNKEN_BORDER)

            Status.int.setGraphicsData("GAUGE", data)
##            Mp.MatPanel(self.panelPlot, wx.Panel, drawFigure)
            self.panelPlot.draw()

        if self.dialog:
            self.Show()
        wx.SafeYield()

    def Destroy(self):
        """
        Personalised Destroy function as a hack for backward compatibility with old dialog implementation
        Before killing itself clears the gauge on the status bar
        """
        if not self.dialog:
            Globals.GUI.panelInfo.showProgress(" ", 0)
            
        super(DialogGauge, self).Destroy()

    def OnButtonCancel(self, event):
##        self.EndModal(wx.ID_CANCEL)
        self.cancelled = True

    def OnButtonOK(self, event):
##        self.EndModal(wx.ID_OK)
        self.cancelled = True

    def checkActivation(self):
        print "DIALOG.checkActivation returns ",self.cancelled
        return self.cancelled

