# -*- coding: utf-8 -*-

#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.iee-einstein.org/" target="_blank">www.iee-einstein.org</a>)
#
#------------------------------------------------------------------------------
#
#	dialogLanguage: Tool main menu -> language selection
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

import wx
from constants import updateConstants
from GUI.status import Status
import database.dbio as dbio
from auxiliary.auxiliary import toGettextAndUnicode as _U, toUnicode as U, toUTF8

class DialogLanguage(wx.Dialog):
    def _init_ctrls(self, prnt):
        # generated method, don't edit
        wx.Dialog.__init__(self, id=-1, name='DialogLanguage', parent=prnt,
              pos=wx.Point(433, 283), size=wx.Size(350, 300),
              style=wx.DEFAULT_DIALOG_STYLE, title=_U('Select language'))

        self.st1 = wx.StaticText(id=-1,
                                 label=_U('Available languages'),
                                 name='st1',
                                 parent=self,
                                 pos=wx.Point(10, 16),
                                 size=wx.Size(200, 32),
                                 style=0)

        self.listBoxLang = wx.ListBox(id=-1,
                                      choices=[],
                                      name='listBoxLang',
                                      parent=self,
                                      pos=wx.Point(10, 50),
                                      size=wx.Size(200, 200),
                                      style=wx.LB_SINGLE|wx.LB_SORT)
        self.listBoxLang.SetFont(wx.Font(10, wx.ROMAN, wx.NORMAL, wx.BOLD, False, 'Ne Times Roman'))
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.OnListBoxLangDclick, self.listBoxLang)

        self.buttonOK = wx.Button(id=wx.ID_OK,
                                  label=_U('OK'),
                                  name='buttonOK',
                                  parent=self,
                                  pos=wx.Point(220, 50),
                                  size=wx.Size(91, 23),
                                  style=0)
        self.buttonOK.Bind(wx.EVT_BUTTON, self.OnListBoxLangDclick, id=wx.ID_OK)

        self.buttonCancel = wx.Button(id=wx.ID_CANCEL,
                                      label=_U('Cancel'),
                                      name='buttonCancel',
                                      parent=self,
                                      pos=wx.Point(220, 80),
                                      size=wx.Size(91, 23),
                                      style=0)

    def __init__(self, parent):
        self._init_ctrls(parent)
        self.language = ''
        self.fillPage()
        
#------------------------------------------------------------------------------
#--- Eventhandlers
#------------------------------------------------------------------------------		
    def OnListBoxLangDclick(self, event):
        self.language = toUTF8(self.listBoxLang.GetStringSelection()).split("-")[0].strip()

        Status.LanguageTool = self.language

        stool = dbio.get("stool")
        stool.LanguageTool = self.language
        dbio.commit()

        self.EndModal(wx.ID_OK)

#------------------------------------------------------------------------------
#--- Public methods
#------------------------------------------------------------------------------		
    def SetLangList(self,list):
        self.listBoxLang.Clear()
        for lang in list:
            self.listBoxLang.Append(U(lang))
        self.listBoxLang.SetSelection(0)

    def clear(self):
        self.listBoxLang.Clear()

    def fillPage(self):
        self.SetLangList([('bg - Български'),
                          ('cat- Català'),
                          ('cz - čeština'),
                          ('de - Deutsch'),
                          ('en - English'),
                          ('es - Castellano'),
                          ('fi - Finnish'),
                          ('fr - Français'),
                          ('it - Italiano'),
                          ('pl - Polski'),
                          ('si - Slovensko'),
                          ('sk - Slovak')
                          ])

    def GetLanguage(self):
        return self.language
