# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       www.einstein-energy.net
#
#------------------------------------------------------------------------------
#
#	dialogUnits: Tool main menu -> language selection
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

import wx
from view.units import UNITSYSTEM
from GUI.status import Status
from GUI.displayClasses import TextEntry, ChoiceEntry, FieldSizes
from GUI.tools.dialogOKCancel import DialogOKCancel
from GUI.fonts import FontProperties
import database.dbio as dbio

# constants that control the default sizes
# 1. font sizes
TYPE_SIZE = 9
TYPE_SIZE_TITLES = 10

# 2. field sizes
HEIGHT = 24
LABEL_WIDTH = 80
DATA_ENTRY_WIDTH = 80
UNITS_WIDTH = 10

# 3. vertical separation between fields
VSEP_LEFT = 2
VSEP_RIGHT = 2

class DialogUnits(DialogOKCancel):
    """
    Edits the unit settings
    """
    def __init__(self, prnt):
        super(DialogUnits, self).__init__(prnt, title=_('Select default units'), size=wx.Size(200,160))
        self.prnt = prnt

##        self._init_ctrls(prnt)
        self._custom_ctrls(prnt)        
        self._custom_layout()

        self.name = None

        self.display()
        self.Show()

    def _custom_ctrls(self, prnt):

        fs = FieldSizes(wHeight=HEIGHT, wLabel=LABEL_WIDTH,
                       wData=DATA_ENTRY_WIDTH, wUnits=UNITS_WIDTH)

        # left tab controls
        # tab 0 - Heat recovery from thermal equipment
        fp = FontProperties()
        fp.pushFont()
        fp.changeFont(size=TYPE_SIZE)

        self.entryUnits = ChoiceEntry(self, values=UNITSYSTEM.keys(),
                                      label=_("Unit system"),
                                      allowNones=False)
        
        self.entryCurrency = TextEntry(self, maxchars=3, value='EUR',
                                       label=_("Currency unit"),
                                       tip=_("e.g. EUR, USD, ..."))
                   
    def _custom_layout(self):
##        flagText = wx.ALIGN_CENTER_VERTICAL|wx.TOP

        sizerGlobal = wx.BoxSizer(wx.VERTICAL)
        sizerGlobal.AddSpacer(10)
        sizerGlobal.Add(self.entryUnits, 0, wx.ALIGN_LEFT, 8)
        sizerGlobal.AddSpacer(10)
        sizerGlobal.Add(self.entryCurrency, 0, wx.ALIGN_LEFT, 8)
        self._do_layout(sizerGlobal)
 
    def display(self):
        self.entryUnits.SetValue(Status.Units)
        self.entryCurrency.SetValue(Status.ProjectCurrency)

        self.Show()

    def OnButtonOK(self, event):
        newUnitSystem = self.entryUnits.GetValue(text=True)

        if newUnitSystem in UNITSYSTEM.keys():
            self.unitSystem = newUnitSystem
            Status.Units = self.unitSystem

            stool = dbio.get("stool")
            stool.UnitsTool = self.unitSystem
            dbio.commit()

        currency = self.entryCurrency.GetValue()
        if currency is not None and len(currency)>0:
            Status.ProjectCurrency = str(currency)
            sproject = Status.prj.getProjectStatus()
            sproject.UnitsReport = str(Status.Units) + "|" + Status.ProjectCurrency
            dbio.commit()

            from globals import Globals
            Globals.unitConversion.setCurrency(currency)
        
        super(DialogUnits, self).OnButtonOK(event)

