# -*- coding: cp1252 -*-
#============================================================================== 				
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	displayCC (Consistency Check)
#			
#------------------------------------------------------------------------------
#			
#	Short description:
#	
#	Functions for displaying CCPar states in Panels Q1 - Q9
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 				

EPSILON = 1.e-3     # required accuracy for function "isequal"
INFINITE = 1.e99    # numerical value assigned to "infinite"

from math import *
from GUI.GUITools import *
from modules.ccheck.ccheckFunctions import *
from modules.messageLogger import *

#libraries necessary for SQL access:
from GUI.status import *

class PanelCCPars():
    """ List of CCPars associated to (Float-)Entries in a panel
    """
    def __init__(self, parDict = {}, dataGroup = None):
        """
        Works on the base of parDict, a dictionary formed by entries of the type

        {parName: ccPar, entry)

        where
        parName:    identifier of the parameter
        ccPar:      corresponding ccPar object in the consistency check module
        entry:      object in the GUI
        """
        
        self.parDict = parDict
        self.dataGroup = dataGroup
        self.conflictList = []
        self.conflictSourceList = []
    
    def add(self, key, ccPar, entry):
        if type(ccPar) == "CCPar" and type(entry) == "FloatEntry" and key is not None:
            self.parDict.update({key:(ccPar, entry)})
            
    def displayCC(self):

        """displays the value contained in the associated CCPar into the associated entry"""
            
        self.getConflicts() # compares the general conflict list with the current parameter list
        
        for parName in self.parDict.keys():
            ccPar, entry = self.parDict[parName]
            if isinstance(ccPar, list): continue
            
##            print "displayCC %s %s %s"%(parName, ccPar.val, ccPar.userDefined)

            if Status.ANo >= 0 and hasattr(entry, "SetValue"):
                entry.SetValue(ccPar.val) # do not modify values in GUI panel for original state (-> danger to get stored in SQL)

            if parName in self.conflictSourceList:
                entry.setColor(fgColor=COLOUR_CONFLICTSRC_FG, bgColor=COLOUR_CONFLICTSRC_BG)    
            elif parName in self.conflictList:
                entry.setColor(fgColor=COLOUR_CONFLICT_FG, bgColor=COLOUR_CONFLICT_BG)
            elif ccPar.screened == True:
                entry.setColor(fgColor=COLOUR_MISSING_FG, bgColor=COLOUR_MISSING_BG)
            elif ccPar.userDefined == False:
                entry.setColor(fgColor=COLOUR_CALCULATED_FG, bgColor=COLOUR_CALCULATED_BG)


    def getConflicts(self):
        self.conflictList = []
        self.conflictSourceList = []

##        print "KEYS: ",self.parDict.keys()
        for entry in conflict.conflictList:
            added = False
            parName = "---"
            extendedParName = None
            if entry[10] == self.dataGroup:
                
                splitters = entry[2].split('[')
                if len(splitters) >= 1:
                    parName = splitters[0]
                    if len(splitters) >= 2:
                        extendedParName = splitters[0]+"["+splitters[1]
                    else:
                        extendedParName = None
                    if parName in self.parDict.keys():
                        self.conflictList.append(parName)
                        added = True
                    elif extendedParName in self.parDict.keys():
                        self.conflictList.append(extendedParName)
                        added = True

                sources = entry[3] + entry[7]
                for source in sources:

##                    splitters = sourceEntry.split('[') #identify attributes (e.g. stream names)
##                    source = splitters[0]
##                    if len(splitters) > 1:
##                        attribute = "["+splitters[1]
##                        print "source %s with attribute %s"%(source,attribute)
##                    else:
##                        attribute = None
                    if source in self.parDict.keys():
                        self.conflictSourceList.append(source)
                        
##            print entry
##            print "entry10 %s dataGroup %s parName %s extendedParName %s"%(entry[10], self.dataGroup, parName, extendedParName)
##            
##        print "CONFLICTLIST: ",self.conflictList
##        print "CONFLICTSOURCELIST: ",self.conflictSourceList
##        print "general conflict list: ",conflict.conflictList

class CCScheme():
    """displays conflicts and other CC-Info on the connection editor
    """
    def __init__(self, fuelIcon, electricityIcon, equipmentIcon, pipeIcon, processIcon):
        self.conflictList = []
        self.conflictSourceList = []
        self.fuelIcon = fuelIcon
        self.electricityIcon = electricityIcon
        self.equipmentIcon = equipmentIcon
        self.pipeIcon = pipeIcon
        self.processIcon = processIcon
        
        self.parDict = {"#FETi":None, "#FETj":None, "#FETFuel_j":None, "#FETel_c_j":None, "#USHj":None, "#USHm":None, "#UPHProcm":None, "#UPHProck":None}
        try:
            self.getConflicts()
        except:
            pass

    def getConflicts(self):
        self.conflictList = []
        self.conflictSourceList = []

##        print "KEYS: ",self.parDict.keys()
        for entry in conflict.conflictList:
            added = False
            parName = "---"
            extendedParName = None
            
            datagroup = entry[10]
            splitters = entry[10].split('[')

            if len(splitters) >= 1:
                objectType = splitters[0]
                if len(splitters) >= 2:
                    try:
                        
                        objectNo = int(splitters[1].split(']')[0])-1
                    except:
                        objectNo = 0
                else:
                    objectNo = 0

                print splitters, objectType, objectNo
                self.conflictList.append((objectType,objectNo))

            sources = entry[3] + entry[7]
            for source in sources:
                splitters = source.split('[')
                if len(splitters) > 0:
                    if splitters[0] in self.parDict.keys():
                        if source not in self.conflictSourceList: self.conflictSourceList.append(source)

##        print self.conflictList
##        print self.conflictSourceList
    
    def display(self):

        for entry in self.conflictSourceList:
            splitters = entry.split('[')
            if len(splitters) >= 2:
                key = splitters[0]
                try:
                    no = int(splitters[1].split(']')[0]) - 1
                except:
                    no = 0

                try:
                    if key == "#FETi":
                        self.fuelIcon[no].setColour(COLOUR_CONFLICTSRC_BG)
                    elif key in ["#FETj", "#USHj", "#FETFuel_j", "#FETel_c_j"]:
                        self.equipmentIcon[no].setColour(COLOUR_CONFLICTSRC_BG)
                    elif key == "#USHm" or key == "#UPHProcm":
                        self.pipeIcon[no].setColour(COLOUR_CONFLICTSRC_BG)
                    elif key == "#UPHProck" or key == "#UPH":
                        self.processIcon[no].setColour(COLOUR_CONFLICTSRC_BG)
                except:
                    print "displayCC:CCScheme.display - cannot highlight key %s no %s"%(key,no)
                    

        for entry in self.conflictList:
            objectType, no = entry
            try:
                if objectType == "Fuel":
                    self.fuelIcon[no].setColour(COLOUR_CONFLICT_BG, fgcolour=COLOUR_CONFLICT_FG)
                elif objectType == "Equipment":
                    self.equipmentIcon[no].setColour(COLOUR_CONFLICT_BG, fgcolour=COLOUR_CONFLICT_FG)
                elif objectType == "Pipe/Duct":
                    self.pipeIcon[no].setColour(COLOUR_CONFLICT_BG, fgcolour=COLOUR_CONFLICT_FG)
                elif objectType == "Process":
                    self.processIcon[no].setColour(COLOUR_CONFLICT_BG, fgcolour=COLOUR_CONFLICT_FG)
            except:
                print "displayCC:CCScheme.display - cannot highlight object %s no %s"%(objectType,no)

class TableCellEntry():
    """
    Helper class in order to extend the applicability of the displayCC module
    to data in cells from TableEntry
    """
    
    def __init__(self, tableEntry, r, c):
        """
        tableEntry: associated TableEntry classs
        r, c: row and column of the entry
        """
        self.entry = tableEntry
        self.r = r
        self.c = c

    def setColor(self, fgColor=None, bgColor=None):
        """
        function for analogy with simple entries (FloatEntry, etc.)
        """
        self.entry.setAttribute("colour", fgColor, row=self.r, col=self.c)
        self.entry.setAttribute("background", bgColor, row=self.r, col=self.c)
        

def findRC(key, matrix):
    for r, row in enumerate(matrix):
        if key in row: return r, row.index(key)
    return None
    
