# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#	EditWindow: Frame for editing anything in a pop-up window
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================
import wx
from globals import logger

from constants import PLATFORM
WINDOWSIZE = (820, 620) if PLATFORM=="WIN" else (800, 600)
class EditWindow(wx.Frame):
    def __init__(self, prnt, size=WINDOWSIZE, pos=(240, 40), permanent=False):
        super(EditWindow, self).__init__(prnt, size=size, pos=pos, style=wx.FRAME_FLOAT_ON_PARENT|wx.DEFAULT_FRAME_STYLE)
        self.prnt = prnt

        if hasattr(self.prnt, "childFrames"):
            self.prnt.childFrames.append(self)

        self.permanent = permanent

        self.changed = False
        self.panel = None

        self.onClose = None     # function to be carried out when closing
        
        self.Bind(wx.EVT_CLOSE, self.OnClose)

    def setCloseFunction(self, onClose):
        """
        some external function to be called if the window is closed
        f = onClose(ret)
        """
        self.onClose = onClose

    def OnClose(self, event):

        if hasattr(self.panel, "getUserChange"):
            self.changed = self.panel.getUserChange()
##            print "EditWindow.OnClose: unsaved changes in panel ? %s"%self.changed

        if self.changed:
            ret = logger.askConfirmation(_("Do you want to save your changes ?"))
            if ret:
                if self.panel is not None and hasattr(self.panel, "OnButtonOK"):
                    self.panel.OnButtonOK(None)

                if hasattr(self.prnt, "updateDrawings"):
                    self.prnt.updateDrawings()

            else:
                if self.panel is not None and hasattr(self.panel, "OnButtonCancel"):
                    self.panel.OnButtonCancel(None)
                
            event.Skip()
            return
##        if event.CanVeto() and self.changed:
##            
##            pass
##
##            if wx.MessageBox("The file has not been saved... continue closing?",
##                             "Please confirm",
##                             wx.ICON_QUESTION | wx.YES_NO) != wx.YES:
##
##                event.Veto()
##                return
        if self.onClose is not None:
            if hasattr(self.panel, "selectedId"):
                self.selectedId = self.panel.selectedId
            else:
                self.selectedId = None

            self.onClose(self.ret if hasattr(self, "ret") else None,
                         self.selectedId)
            
        elif hasattr(self.prnt, "updateDrawings"):
            self.prnt.updateDrawings()
            self.prnt.unselectTree()
##            self.prnt.tree.UnselectAll()
##            wx.SafeYield()     ## Note: this SafeYield gave problems on Windows (e.g. TC2.02 B03). TODO: check again without this on Linux

        elif hasattr(self.prnt, "onPseudoModal"):
            """ a hack in order to use this window similar to a modal dialog
            """
            if hasattr(self.panel, "selectedId"):
                self.selectedId = self.panel.selectedId
            else:
                self.selectedId = None
                
            self.prnt.onPseudoModal(self.ret if hasattr(self, "ret") else None)
            self.prnt.Enable()

###### This event.Skip gives problems in TC2.04 -> cancelled out (15.01.2019)
##        if event: event.Skip()
        try:
            self.Destroy()
        except:
            pass

