#!/usr/bin/env python
# -*- coding: utf-8 -*-
#==============================================================================
#
#   E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#   EINSTEIN Main
#
#------------------------------------------------------------------------------
#
#   GUI Main routines
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
import sys
import os
import gettext
import wx
import wx.lib.scrolledpanel as scrolled

from constants import APPDATA, INTERACTIONLEVELS, PLATFORM, updateConstants, translateConstants
from globals import logger
from GUI.GUITools import ORANGE

BASEDIR = os.path.abspath('../../..')
try:
    os.mkdir(APPDATA)
except OSError:
    pass # directory already existed
from modules.messageLogger import *
##import einstein.GUI.pSQL as pSQL
##import einstein.GUI.pSQLite as pSQLite
from constants import DBSERVER
import xml

#--- popup frames
from modules.project import Project #functions for handling of PId/ANo

#--- popup frames
from GUI.PreferencesFrame import PreferencesFrame
##from dialogPOManageDB import DialogPOManageDB

#--- Module Panels
from editWindow import EditWindow

from panelCC import PanelCC
from panels.panelA import PanelA
##from panelPO import PanelPO
from panelHR import PanelHR
from panelCTRL import PanelCTRL
from panelST import PanelST
from panelHP import PanelHP
from panelBB import PanelBB
from panelCH import PanelCH
from panelCHP import PanelCHP
from panels.Energy.panelEnergy import PanelEnergy
#TS20080406 the new panels Q*
from panelPRJ import PanelPRJ
from panels.Q.panelQ1 import PanelQ1
from panels.panelMET import PanelMET
from panels.Q.panelQ2_el import PanelQ2_el
from panelQ2_fuel import PanelQ2_fuel
from panels.Q.panelQ3 import PanelQ3
from panels.Q.panelQ4 import PanelQ4
from panels.Q.panelQ5 import PanelQ5
from panelQ6 import PanelQ6
from panelQ6B import PanelQ6B
from panelQ7 import PanelQ7
from panelQ8 import PanelQ8
from panelQM import PanelQM
from flowCharts.editCon import EditCon
from flowCharts.editHXNet import EditHXNet
from flowCharts.editMF import EditMF

# backend modules for direct calling
from modules.guiModules.modulePRJ import ModulePRJ
from modules.control import autoRun

# DBEditor
from panels.DB.panelDBHeatPump import PanelDBHeatPump
from panels.DB.panelDBSolarThermal import PanelDBSolarThermal
from panels.DB.panelDBCHP import PanelDBCHP
from panels.DB.panelDBBoiler import PanelDBBoiler
from panels.DB.panelDBFuel import PanelDBFuel
from panels.DB.panelDBFluid import PanelDBFluid
from panels.DB.panelDBElectricityMix import PanelDBElectricityMix
from panels.DB.panelDBBenchmark import PanelDBBenchmark
from panels.DB.panelDBAuditor import PanelDBAuditor
from schedules.profileSelector import ProfileSelector

from panels.DB.panelDBAuditorEntry import AuditorPopUp

from panelEA1 import PanelEA1
from panelEA2 import PanelEA2
from panelEA3 import PanelEA3
from panelEA4a import PanelEA4a
from panels.Energy.panelEA4b import PanelEA4b
from panels.Energy.panelEA4c import PanelEA4c
from panelEA5 import PanelEA5
from panelEM1 import PanelEM1
from panelEM2 import PanelEM2
from panels.Energy.panelEH1 import PanelEH1
from panels.Energy.panelEH2 import PanelEH2

from panels.PA.panelPA1 import PanelPA1
from panels.PA.panelPA2 import PanelPA2

from panels.CS.panelCS1 import PanelCS1
from panels.CS.panelCS2 import PanelCS2
from panels.CS.panelCS3 import PanelCS3
from panels.CS.panelCS4 import PanelCS4
from panels.CS.panelCS5 import PanelCS5
from panels.CS.panelCS6 import PanelCS6
from panels.CS.panelCS7 import PanelCS7

from panelBM1 import PanelBM1
from panelBM2 import PanelBM2
from panelBM3 import PanelBM3

from panels.ECO.panelECO  import PanelECO
from panels.ECO.panelECO0 import PanelECO0
from panels.ECO.panelECO1 import PanelECO1

from panelInfo import PanelInfo
from userHelp import FrameHelpAbout, FrameHelpSupport
from dialogLanguage import DialogLanguage
from dialogUnits import DialogUnits

from panelReport import PanelReport

from modules.xmlIO import (ImportDB, ExportDB,
                                    ImportProject, ExportProject, VersionError)

PANELCLASS = {"PRJ":PanelPRJ,
              "Q0": None,
              "Q1": PanelQ1,
              "MET": PanelMET,
              "FUEL":PanelQ2_fuel,
              "EL": PanelQ2_el,
              "PROC": PanelQ3,
              "EQ": PanelQ4,
              "CTRL": PanelCTRL,
              "PIPE": PanelQ5,
              "HX": PanelQ6,
              "WHEE": PanelQ6B,
              "Q7": PanelQ7,
              "Q8": PanelQ8,
              "QM": PanelQM,
              "EA": None,
              "Energy": PanelEnergy,
              "EA1": PanelEA1,
              "EA2": PanelEA2,
              "EA3": PanelEA3,
              "EA4a": PanelEA4a,
              "EA4b": PanelEA4b,
              "EA4c": PanelEA4c,
              "EA5": PanelEA5,
              "EM1": PanelEM1,
              "EM2": PanelEM2,
              "EH1": PanelEH1,
              "EH2": PanelEH2,
              "PA1": PanelPA1,
              "PA2": PanelPA2,

              "ECO":    PanelECO,
              "ECO0":   PanelECO0,
              "ECO1":   PanelECO1,
              
              "AUTO": None,
              
              "BM": None,
              "BM1": PanelBM1,
              "BM2": PanelBM2,
              "BM3": PanelBM3,
              
              "A":      PanelA,
              "CS1":    PanelCS1,
              "CS2":    PanelCS2,
              "CS3":    PanelCS3,
              "CS4":    PanelCS4,
              "CS5":    PanelCS5,
              "CS6":    PanelCS6,
              "CS7":    PanelCS7,
              
              "DA":     None,
              "CC":     PanelCC,
              "HR":     PanelHR,
              "HC":     None,
              "BB":     PanelBB,
              "CH":     PanelCH,
              "CHP":    PanelCHP,
              "HP":     PanelHP,
              "ST":     PanelST,

              "Report": PanelReport}


##from einstein.modules.spreadsheetIO import ImportQ
#TS2008-06-18 fonts management
from fonts import FontProperties
#TS2008-06-30 database management
from dialogDatabase import DlgDatabase

from GUI.status import * #processing status of the tool
from auxiliary.auxiliary import toGettextAndUnicode as _U 
from auxiliary.auxiliary import toUnicode as U
        
#----- Constants
##KFramesize = (1024, 740)
minFramesize = (820, 620) if PLATFORM=="WIN" else (800, 600)

ex = None # Used to buffer information during import/export

class EinsteinFrame(wx.Frame):
#------------------------------------------------------------------------------
#   Main frame structure:
#
#   EinsteinFrame -------- menubar
#                      |
#                      |-- panelInfo
#                      |
#                      |-- mainArea---x-- treeArea ----- tree
#                      |              |
#                      |              |-- workArea  --x-- workPanel
#                      |                              |
#                      |                              |
#                      |                              |-- messagePanel
#                      |
#                      |-- statusbar
#------------------------------------------------------------------------------
    def __init__(self, parent, id, title, wxapp):
        wx.Frame.__init__(self, parent, id, title, pos=(0, 0))
        self.SetSizeHints(*minFramesize)
        self.treePermissions = {}
        self.parent = parent
        self.wxapp = wxapp
        self.Maximize(True)

        self.childFrames = []   # child frames open

        self.editWindow = None
        self.viewerWindow = None

    def createUI(self):
        self.activeQid = 0
        self.activePanel = None

        # special treatment for pages which are kept in memory and not destroyed when switching
##        self.panelCC = None
##        self.Page3 = None

        ############################################
        #
        # UI generation
        #
        ############################################

        #----- add statusbar

        #----- create splitters
        self.mainArea = wx.SplitterWindow(self, style=wx.CLIP_CHILDREN | wx.SP_LIVE_UPDATE | wx.SP_3D | wx.SP_3DSASH)

        self.workArea = wx.SplitterWindow(self.mainArea, -1,
                                           style=wx.CLIP_CHILDREN | wx.SP_LIVE_UPDATE | wx.SP_3D | wx.SP_3DSASH)
        self.workArea.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE))

        self.treeArea = wx.Panel(self.mainArea, -1)

        ####---- Upper info panel
        self.panelInfo = PanelInfo(self, self)
        self.SetStatusBar(self.panelInfo)


        ####---- panel containing drawings
        self.workPanel = scrolled.ScrolledPanel(self.workArea, -1, style=wx.WANTS_CHARS | wx.HSCROLL | wx.VSCROLL)
        self.workPanel.SetupScrolling()

        self.notebook = wx.Notebook(self.workPanel, -1, style=0)
        self.panelSC = EditCon(self.notebook, main=self)
        self.notebook.AddPage(self.panelSC, _U('System scheme'))

        self.panelHXNet = EditHXNet(self.notebook, main=self)
        self.notebook.AddPage(self.panelHXNet, _U('HX Network'))

        self.panelMF = EditMF(self.notebook, main=self)
        self.notebook.AddPage(self.panelMF, _U('Mass flows'))

        self.panelEnergyFrame = wx.Panel(self.notebook)
        self.panelEnergyChild = None
        self.notebook.AddPage(self.panelEnergyFrame, _U('Energy'))
        #----- add menu
        self.CreateMenu()
        self.changeAssistantMainMenu(INTERACTIONLEVELS.index(Status.UserInteractionLevel))

        #----- create tree control
        self.CreateTree()
        #----- error log window
        self.message = wx.TextCtrl(id= -1,
                   name='message',
                   parent=self.workArea,
                   style=wx.RAISED_BORDER | wx.TE_MULTILINE | wx.TE_RICH)
##                   style=wx.RAISED_BORDER | wx.LC_REPORT | wx.LC_NO_HEADER)
##        self.message.InsertColumn(0, _U("Message log"))
        self.message.SetBackgroundColour('white')
        self.message.SetForegroundColour('green')
        # show/hide is a main menu action
        # ini. state is hidden
        self.message.Show()

        self.DoLayout ()
        self.BindEvents()
##        self.CreateTitlePage()


    def changeUI(self):
        #----- add statusbar
#        self.CreateStatusBar()

        self.blockTreeEvents = True

        self.panelInfo.Destroy()
        self.panelInfo = PanelInfo(self, self)

        self.CreateMenu()
        self.changeAssistantMainMenu(INTERACTIONLEVELS.index(Status.UserInteractionLevel))

        self.tree.Destroy()
        self.CreateTree()

        self.DoLayout()

        #----- set binding events
        self.BindEvents()
        self.blockTreeEvents = False

    def _log(self, fcolor, bcolor, text):
        """Display the given text in the logging widget of the EINSTEIN GUI."""
        fp = FontProperties()
        font = fp.getFont()
        font.SetPointSize(9)
        attr = wx.TextAttr(fcolor, bcolor, font=font)
        
        self.message.SetDefaultStyle(attr)
        self.message.AppendText("\n" + U(text))

    def logMessage(self, text):
        self._log('#00A000', '#FFFFFF', text)
##        self.doLog.LogThis('Message:' + text)

    def logWarning(self, text):
        self._log('#0000FF', '#FFFF80', text)
##        self.doLog.LogThis('Warning:' + text)

    def logError(self, text):
        self._log('#FFFFFF', '#FF0000', text)
##        self.doLog.LogThis('Error:' + text)

    def showError(self, text):
        self.logError(text)
        if Status.UserInteractionLevel <> "automatic":
            dlg = wx.MessageDialog(None, U(text), _U("Error"), wx.OK | wx.ICON_ERROR)
            ret = dlg.ShowModal()
            dlg.Destroy()
##        self.doLog.LogThis('Error: ' + text)

    def showWarning(self, text):
        self.logWarning(text)

        if Status.UserInteractionLevel <> "automatic":
            dlg = wx.MessageDialog(None, U(text), _U("Warning"), wx.OK | wx.ICON_EXCLAMATION)
            ret = dlg.ShowModal()
        #dlg.Destroy()
##        self.doLog.LogThis('Warning: ' + text)

    def showMessage(self, text):
        self.logMessage(text)
        if Status.UserInteractionLevel <> "automatic":
            dlg = wx.MessageDialog(None, U(text), _U("Info"), wx.OK | wx.ICON_INFORMATION)
            ret = dlg.ShowModal()
            dlg.Destroy()
##        self.doLog.LogThis('Info: ' + text)

        
    def showInfo(self, text):
        """ kept for backward compatibility """
        self.showMessage(text)


    def askConfirmation(self, text, alwaysAsk=False):
        self.logMessage(text)
        if Status.UserInteractionLevel <> "automatic" or alwaysAsk:
            dlg = wx.MessageDialog(None, U(text), _U("Confirm"), wx.YES_NO | wx.ICON_QUESTION)
            ret = dlg.ShowModal()
            dlg.Destroy()
            return (ret==wx.ID_YES)
        else:
            return True

    def DoLayout(self):
        #       Layout of main frame and panels
        
        # set sizers

        # this sizer allows the horiz.expansion of the tree
        sizer1 = wx.BoxSizer(wx.VERTICAL)
        sizer1.Add(self.tree, 1, wx.EXPAND)
        self.treeArea.SetSizer(sizer1)

        sizer2 = wx.BoxSizer(wx.VERTICAL)
        sizer2.Add(self.workPanel, 1, wx.EXPAND, 0)
        self.workArea.SetSizer(sizer2)

        sizer3 = wx.BoxSizer(wx.VERTICAL)
        sizer3.Add(self.notebook, 1, wx.EXPAND, 0)
        self.workPanel.SetSizer(sizer3)

        panelsizer = wx.BoxSizer(wx.VERTICAL)
        #panelsizer.Add(self.panelInfo, 0, wx.FIXED_MINSIZE, 0)
##        panelsizer.Add(self.panelInfo, 0, wx.EXPAND)
        panelsizer.Add(self.mainArea, 1, wx.EXPAND, 0)

##        infobar = self.FindWindowByName("PanelInfo")
##        infobar.SetMinSize((0, 26))

        self.SetSizer(panelsizer)

        self.Layout()

        # set splitters
        self.mainArea.SplitVertically(self.treeArea, self.workArea, 200)
        self.workArea.SplitHorizontally(self.workPanel, self.message, -80) # initially open
        self.workArea.SetSashPosition(580)

        # find the width of message panel
##        w = self.workArea.GetWindow1()
##        (width, height) = w.GetClientSizeTuple()
##        self.message.SetColumnWidth(0, width - 10)
##        self.workPanel.SetupScrolling()

##    def CreateTitlePage(self):
##        ####----PAGE Title
##        self.pageTitle = wx.Panel(id= -1, name='pageTitle', parent=self.dummy, pos=wx.Point(0, 0), size=wx.Size(800, 600), style=0)
##        self.pageTitle.Show()
##        #..............................................................................
##        # grid for displaying alternatives
##        self.box1 = wx.StaticBox(self.pageTitle, -1, _U("Welcome to ..."),
##                                 pos=(10, 10), size=(780, 580))
##        
##        ORANGE = '#FF6000'
##        self.box1.SetForegroundColour(ORANGE)
##        self.box1.SetFont(wx.Font(10, wx.SWISS, wx.NORMAL, wx.BOLD))
##
##        self.staticBitmap1 = wx.StaticBitmap(bitmap=wx.Bitmap(os.path.join('GUI/img', 'brochureTitlePage.jpg'),
##                                             wx.BITMAP_TYPE_JPEG),
##                                             id= -1, 
##                                             parent=self.pageTitle,
##                                             pos=wx.Point(60, 100),
##                                             size=wx.Size(700, 480),
##                                             style=wx.SUNKEN_BORDER)
##        self.staticBitmap2 = wx.StaticBitmap(bitmap=wx.Bitmap(os.path.join('GUI/img', 'einstein_logo_small.jpg'),
##                                             wx.BITMAP_TYPE_JPEG),
##                                             id= -1, 
##                                             parent=self.pageTitle,
##                                             pos=wx.Point(280, 0),
##                                             size=wx.Size(478, 130),
##                                             style=0)

    def control(self, action, data, goToKey=None):
        """
        generic GUI controller allowing for executing GUI actions from any interface
        or automatically from the code
        (substitutes the tree control from EINSTEIN 2)

        action: action code corresponding to the GUI panel to be activated (e.g. Q0 ...  Q9, CC, etc.)
        data: data for specifying additional inputs (e.g. number of equipment to be selected)
        """

        self.hidePages()
        self.activePanel = action

        # preliminary checks for specific panels
        if action=="EM2" and Status.ANo <= 0:
            logger.showWarning(_("Sorry. This function is not available for the present state."))
            return

        elif action=="QM" and Status.ANo <> -1:
            logger.showWarning(_("Cannot open monitoring data in checked state"))
            return

        elif action in ["ECO1"] and Status.ANo < 0:
            logger.showWarning(_("This function is only accessible once You have checked your data"))
            return
                

        if action=="AUTO":
            ret = logger.askConfirmation(_("EINSTEIN - thermal energy audit at the speed of the light\n"+ \
                               "fasten your seat belt. put your seats in upright position\n"
                               "ready for take off ?"))
            if ret:
                autoRun(self)
            return


        # actions in work panel (global and project actions)
        if action in ["A"] or action.startswith("CS"):
            self.editWindow = self.workPanel
            self.notebook.Hide()
        elif action in ["EH1", "EH2", "EA1", "EA2", "EA3", "EA4a", "EA4b", "EA4c", "EA5", "EM1", "EM2", "Energy"]:
            self.editWindow = self.panelEnergyFrame
            self.notebook.SetSelection(3)
            self.notebook.Show()
            
        # actions without own panel (alternative-specific actions; notebook is shown)
        elif action in ["Q0", "EA", "EM", "EH", "AUTO", "HC", "DA"]:
            self.editWindow = None
            self.notebook.Show()

        # actions in editWindow (alternative-specific actions; notebook is shown)
        else:
            if action in ["PA1"]:
                self.editWindow = EditWindow(self, size=(1000, 480))
            elif action in ["PA2"]:
                self.viewerWindow = EditWindow(self, pos=(200, 200), permanent=True)
            else:
                self.editWindow = EditWindow(self)  # opens a new pop-up frame containing the panel for editing
            self.notebook.Show()

        activeWindow = self.viewerWindow if action in ["PA2"] else self.editWindow

        
        try:
            no = int(data)
        except:
            no = data
        
        if action in PANELCLASS.keys():

            # special commands for special actions
            if action in ["PRJ"]:
                if data=="new":
                    self.activeQid = 0

            if activeWindow:
                panelClass = PANELCLASS[action]

                if action in ["PA1"]:
                    self.editPanel = panelClass(self.editWindow, self, no, size=(980, 480))
                elif action in ["PA2"]:
                    self.editPanel = panelClass(self.viewerWindow, self)
                else:
                    # try-except: some panels use the action code (no) already at the stage of panel.init, others don't
                    if action in ["HX"]: # list of panels for which 3 inputs are explicitely specified, for the others use try/except
                        self.editPanel = panelClass(self.editWindow, self, no)
                    else:
                        try:    
                            self.editPanel = panelClass(self.editWindow, self, no)
                        except TypeError:
                            self.editPanel = panelClass(self.editWindow, self)

                # bookkeeping for panelEnergyFrame
                if self.editWindow==self.panelEnergyFrame:
                    if self.panelEnergyChild is not None:

# NOTE: try-except added in order to avoid crashes on Windows (TODO: further investigate where this comes from
#  File "C:\Users\cris\Documents\einstein.git\Python26Einstein\lib\site-packages\wx-2.8-msw-unicode\wx\_core.py", line 14590, in __getattr__
#    raise PyDeadObjectError(self.attrStr % self._name)
#PyDeadObjectError: The C++ part of the PanelEnergy object has been deleted, attribute access no longer allowed.
                        try:
                            self.panelEnergyChild.Destroy()
                        except:
                            pass
                    self.panelEnergyChild = self.editPanel
                        
                self.editPanel.display(no)
                if goToKey: self.editPanel.goTo(goToKey)
            
            if action in ["PRJ"]:
                if data=="new":
                    self.editPanel.NewProject()

        if self.editWindow: self.editWindow.Show()
        if hasattr(self, "viewerWindow") and self.viewerWindow: self.viewerWindow.Show()

        for frame in self.childFrames:
            frame.Show()

    def updateChildFrames(self, keys):
        for frame in self.childFrames:
            try:
                panel = frame.panel
                panelKey = panel.key
                print "EinsteinGUI.updateChildFrames: testing panelKey = %s"%panelKey
            except:
                panel = None

            if panel is not None and panelKey in keys and hasattr(panel, "display"):
                print "Displaying panel %s"%panelKey
                panel.display(None)
                
        
    def updateDrawings(self):
        """
        refreshes all drawings in workPanel
        """
        if self.panelSC: self.panelSC.display()
        if self.panelHXNet: self.panelHXNet.display()
        if self.panelMF: self.panelMF.display()
        self.notebook.Refresh()

    def display(self):
        """ dummy function - to be used e.g. for eventhandlers using instances of ModulePRJ
        which calls self.panel.display
        -> Panel-modules so can be instantiated directly from the main frame, which
        behaves as if it would be a panel
        """
        pass
    
    def OnMenuNewProject(self, event):
        """Add project on main menu Opens panel Q1"""
        Status.prj.createNewProject(-1, _("new project"), _("add description here"))
        self.control("Q1", None)

    def OnMenuOpenProject(self, event):
        self.control("PRJ", None)

    def OnMenuDeleteProject(self, event):
        if Status.PId is not None:
            mod = ModulePRJ(self)
            mod.deleteProject(Status.PId)
        
    def OnMenuCopyProject(self, event):
        if Status.PId is not None:
            mod = ModulePRJ(self)
            mod.copyProject(Status.PId)
        
    def OnMenuExit(self, event):
        """Ask before exiting"""
        if self.askConfirmation(_U("Do you really want to exit?")) == wx.ID_YES:
            wx.Exit()

    def OnMenuExportAll(self, event):
        ex = ExportDB("all")

    def OnMenuExportDBBenchmark(self, event):
        ex = ExportDB("dbbenchmark")
        
    def OnMenuExportDBNaceCode(self, event):
        ex = ExportDB("dbnacecode")
        
    def OnMenuExportDBUnitOperation(self, event):
        ex = ExportDB("dbunitoperation")
        
    def OnMenuExportAllEquipments(self, event):
        ex = ExportDB("all equipments")
        
    def OnMenuExportDBCHP(self, event):
        ex = ExportDB("dbchp")
        
    def OnMenuExportDBHeatPump(self, event):
        ex = ExportDB("dbheatpump")
        
    def OnMenuExportDBFluid(self, event):
        ex = ExportDB("dbfluid")
        
    def OnMenuExportDBFuel(self, event):
        ex = ExportDB("dbfuel")
        
    def OnMenuExportDBElectricityMix(self, event):
        ex = ExportDB("dbelectricitymix")
        
    def OnMenuExportDBBoiler(self, event):
        ex = ExportDB("dbboiler")
        
    def OnMenuExportDBSolarThermal(self, event):
        ex = ExportDB("dbsolarthermal")
        
##    def OnMenuExportDBChiller(self, event):
##        ex = ExportDB("dbchiller")
##        
    def OnMenuImportData(self, event):
        im = ImportDB()

    def OnMenuExportProject(self, event):
        ex = ExportProject(pid=Status.PId)

    def OnMenuImportProject(self, event):
        try:
            ex = ImportProject()
        except IndexError:
            showError(_U("The project file seems to be inconsistent."\
                         " The import was probably unsuccessful."))
            raise
        except xml.parsers.expat.ExpatError: #@UndefinedVariable
            showError(_U("The project file is no valid xml. "\
                         "The import was unsuccessful"))
            raise
        except VersionError:
            showError(_U("The project file seems to be from a newer version "\
                         "of EINSTEIN. The import has been aborted."))
        else:
            pids = ex.getPid()
            name = Status.prj.getProjectName(pids[1])
            if pids != (None, None):
                self.showInfo(_U("A project with the name '%s' has been imported")
                              % U(name) )

        self.panelInfo.update()
        self.showMainMenuAlternatives()
        self.updateDrawings()
        
##    def OnMenuImportQ(self, event):
##        ex = ImportQ()

# Scroll-up menu "VIEW"
# Dynamic alternatives menu
    def OnMenuAlternatives(self, event):
        """Switch to the selected state. One of original state, 
        checked state, alternative proposal #n.
    
        """
        i = event.GetId() - self.rangeId[0]
        al = Status.prj.getAlternativeList()
        ANo = al[i][0]
        if ANo == -1 and Status.StatusCC != 0:
            ret = askConfirmation(_U("Should in addition to switching to the original state it also be unblocked?\n")+
                                            _U("The unblocking would give you the possibility to modify it again, but ")+
                                            _U("it deletes _all_ alternative proposals."), alwaysAsk=True)
            if ret:
                Status.prj.copyQuestionnaire()

        Status.prj.setActiveAlternative(ANo)
        self.panelInfo.update()
        self.showMainMenuAlternatives()
        self.updateDrawings()
        for child in  self.workPanel.GetChildren(): # refresh the current panel
##            if child == self.panelCC and self.activePanel != "CC":
##                continue
##            elif child == self.Page3 and self.activePanel != "Q3":
##                continue
##            else:
            self.blockTreeEvents = True #security feature - in case that panel display functions activate some tree functions
            try:
                child.display()
            except:
                pass
            self.blockTreeEvents = False

    def OnMenuEditDBAdmin(self, event):
        dlgdb = DlgDatabase(parent=None, id= -1)
        dlgdb.ShowModal()
        if dlgdb.getChanges():
            self.showWarning(_('Any changes will be in effect next time you restart Einstein'))
            
    def OnMenuEditDBAuditor(self, event):
        frameEditDBAuditor = PanelDBAuditor(self, "Edit DBAuditor", False, selectedId=Status.AuditorId)
        frameEditDBAuditor.ShowModal()

    def OnMenuEditDBBenchmark(self, event):
        frameEditDBBenchmark = PanelDBBenchmark(self, "Edit DBBenchmark", False)
        frameEditDBBenchmark.ShowModal()

##    def OnMenuEditDBBAT(self, event):
##        dlg = DialogPOManageDB(self)
##        dlg.ShowModal()
##        event.Skip()

##    def OnMenuEditDBNaceCode(self, event):
##        frameEditDBNaceCode = DBEditFrame(None, "Edit DBNaceCode", 'dbnacecode', 0, True)
##        frameEditDBNaceCode.ShowModal()
##    
##    def OnMenuEditDBUnitOperation(self, event):
##        frameEditDBUnitOperation = DBEditFrame(None, "Edit DBUnitOperation", 'dbunitoperation', 0, True)
##        frameEditDBUnitOperation.ShowModal()
    
    def OnMenuEditDBCHP(self, event):
        frameEditDBCHP = PanelDBCHP(self, "Edit DBCHP", False)
        frameEditDBCHP.ShowModal()

    def OnMenuEditDBHeatPump(self, event):
        frameEditDBHeatPump = PanelDBHeatPump(self, "Edit DBHeatPump", False)
        frameEditDBHeatPump.ShowModal()

    def OnMenuEditDBFluid(self, event):
        frameEditDBFluid = PanelDBFluid(self, _("Data base of materials"), False)
        frameEditDBFluid.ShowModal()

    def OnMenuEditDBFuel(self, event):
        frameEditDBFuel = PanelDBFuel(self, _("Data base of fuels"), False)
        frameEditDBFuel.ShowModal()

    def OnMenuEditDBBoiler(self, event):
        frameEditDBBoiler = PanelDBBoiler(self, "Edit DBBoiler", False)
        frameEditDBBoiler.ShowModal()

    def OnMenuEditDBSolarThermal(self, event):
        frameEditDBSolarThermal = PanelDBSolarThermal(self, "Edit DBSolarThermal")
        frameEditDBSolarThermal.ShowModal()
#    def OnMenuEditDBChiller(self, event):
#        frameEditDBChiller = DBEditFrame(None, "Edit DBChiller", 'dbchiller', 0, True)
#        frameEditDBChiller.ShowModal()
    def OnMenuEditDBElectricityMix(self, event):
        frameEditDBElectricityMix = PanelDBElectricityMix(self, "Edit DBElectricityMix", False)
        frameEditDBElectricityMix.ShowModal()

    def OnMenuEditDBProfiles(self, event):
        ps = ProfileSelector(self, mode="edit")
        ps.display(data=[])
        ret = ps.ShowModal()

# Scroll-up menu "USER SELECT LEVEL 1 - 3"
    def OnMenuUserSelectLevel1(self, event):
        self.panelInfo.changeAssistant(0)
        Status.prj.setUserInteractionLevel(1)

    def OnMenuUserSelectLevel2(self, event):
        self.panelInfo.changeAssistant(1)
        Status.prj.setUserInteractionLevel(2)

    def OnMenuUserSelectLevel3(self, event):
        self.panelInfo.changeAssistant(2)
        Status.prj.setUserInteractionLevel(3)

    def OnMenuSettingsPreferences(self, event):
        framePreferences = PreferencesFrame(self)
        framePreferences.ShowModal()

    def OnMenuSettingsLanguage(self, event):
        dialogLang = DialogLanguage(self)
        if dialogLang.ShowModal() == wx.ID_OK:
            newlang = dialogLang.GetLanguage()
            language = gettext.translation("einstein", "locale",
                                           languages=['%s' % (newlang,)], fallback=True)
            language.install()
            self.showInfo(_U("Please restart Einstein to let the language change "\
                             "take full effect."))
        dialogLang.Destroy()
        Status.TRANS = updateConstants()
        translateConstants()
        self.changeUI()
        
    def OnMenuSettingsUnits(self, event):
        dialogUnits = DialogUnits(self)
        ret = dialogUnits.ShowModal()
        if ret: self.changeUI()
        
    def OnMenuSettingsAuditor(self, event):
        dlg = PanelDBAuditor(self, _U("Select an auditor identity."), showOk=True)
        dlg.editWindow.setCloseFunction(self.OnMenuSettingsAuditorReturn)
        ret = dlg.ShowModal()

    def OnMenuSettingsAuditorReturn(self, ret, auditorId):
        if ret and auditorId is not None:
            Status.prj.setAuditor(int(auditorId))
            logMessage(_("Auditor changed: %s"%(Status.prj.getAuditorName(Status.AuditorId))))

    def OnMenuSettingsViewMessages(self, event):
        if self.menuBar.IsChecked(event.GetId()):
            self.workArea.SetSashPosition(-50)
            self.message.Show()
        else:
            self.workArea.SetSashPosition(-1)
            self.message.Hide()

    def OnMenuSettingsManageFonts(self, event):
        fp = FontProperties()
        fp.chooseFont()
        
# Scroll-up menu "HELP" and "About ..."
##    def OnMenuHelpUserManual(self, event):
##        showUserManual('../../../docs/sphinx/_build/html/users/guide.html')

    def OnMenuHelpSupport(self, event):
        frameSupport = FrameHelpSupport(self)
        frameSupport.Show()

    def OnMenuHelpAbout(self, event):
        frameAbout = FrameHelpAbout(self)
        frameAbout.Show()

    def _interceptActivation(self, event):
        item = event.GetItem()

        try:
            label = self.tree.GetItemText(item)
        except:
            label = None
        if self.treePermissions.has_key(label):
            (allow, level, message) = self.treePermissions[label]
            if not allow:
                event.Veto()
                if not message:
                    message = _U('Cannot open this action')
                self.showWarning(message)
                
    def OnTreeItemExpanding(self, event):
        self._interceptActivation(event)

    def OnTreeSelChanging(self, event):
        self._interceptActivation(event)

    def OnTreeItemRightClick(self, event):
        self.item = event.GetItem()
##        if self.item in self.qQ3sub:
##            self.PopupMenu(TreePopUpMenu(self))
        
    def OnTreeBeginDrag(self, event):
        pass
##        item = event.GetItem()
##        label = self.tree.GetItemText(item)
##        self.dragDropStartLabel = label
##
##        if item in self.qQ3sub:
##            k = self.qQ3sub.index(item)
##            self.dragDropStartNo = k
##            event.Allow()
##        else:
##            event.Veto()
##            pass
            
    def OnTreeEndDrag(self, event):
        pass
##        item = event.GetItem()
##        label = self.tree.GetItemText(item)
##        targetList = [self.qPage3]+self.qQ3sub
##        if item in targetList:
##            k = targetList.index(item)
##            if k > self.dragDropStartNo: k -= 1
##            
##            self.Page3.shiftFromTo(self.dragDropStartNo, k)
            

    def OnTreeSelChanged(self, event):
        if self.blockTreeEvents: return # blocks recursive calls from Tree to Panel
        
        self.blockTreeEvents = True

        self.item = event.GetItem()
        try:    # some still unexplained errors occured on Windows with the GetItemText function. TODO: look for this in detail
            text = self.tree.GetItemText(self.item)
        except:
            text = "---"
            
        logger.logMessage("@%s"%(text))

        if text != "---":
            for key in PANELCLASS.keys():
                try:
                    treeItem = getattr(self, "q"+key)
                except:
                    treeItem = None
                if self.item==treeItem:
                    self.control(key, None)
                    break

        wx.SafeYield()
        self.blockTreeEvents = False

    def OnButtonDataCheck(self, event):
        if self.activeQid <> 0:
            #ret = self.ModBridge.FunctionOneDataCheck(Status.SQL, DB, self.activeQid)
            #self.showInfo("Return of FunctionOneDataCheck %s" %(ret))
            pass
        else:
            self.showError("Select Questionnaire first!")

    def hidePages(self):

        permanentFrames = []
        for frame in self.childFrames:
            try:
                if not frame.permanent:
                    frame.Close()
                else:
                    permanentFrames.append(frame)
            except:
                pass
            
        self.childFrames = permanentFrames

        try:
            self.editPanel.Destroy()
        except:
            pass

        return

    def CreateMenu(self):
        self.menuBar = wx.MenuBar()

        self.menuProject = wx.Menu()
        self.menuAlternatives = wx.Menu()
        self.menuDatabase = wx.Menu()
        self.menuSettings = wx.Menu()
        self.menuHelp = wx.Menu()

        self.submenuPrint = wx.Menu()
        self.submenuEditDB = wx.Menu()
        self.submenuExportDB = wx.Menu()

        self.submenuEquipments = wx.Menu()
        self.submenuExportEquipments = wx.Menu()
        self.submenuUserLevel = wx.Menu()
##        self.submenuClassification = wx.Menu()

        self.PrintFullReport = self.submenuPrint.Append(-1, _U("full report"))
        self.PrintQuestionnaire = self.submenuPrint.Append(-1, _U("questionnaire"))

        self.openProject = self.menuProject.Append(-1, _U("open"))
        self.copyProject = self.menuProject.Append(-1, _U("copy"))
        self.newProject = self.menuProject.Append(-1, _U("new"))
        self.menuProject.AppendSeparator()
        self.importProject = self.menuProject.Append(-1, _U("import"))
        self.exportProject = self.menuProject.Append(-1, _U("export"))
        self.menuProject.AppendSeparator()
        self.deleteProject = self.menuProject.Append(-1, _U("delete"))
        self.menuProject.AppendSeparator()
        self.ExitApp = self.menuProject.Append(-1, _U("exit"))

        self.EditDBCHP = self.submenuEquipments.Append(-1, _U("&CHP"))
        self.EditDBHeatPump = self.submenuEquipments.Append(-1, _U("&Heat pumps") + u'/' + _U("&Chillers"))
##        self.EditDBChiller = self.submenuEquipments.Append(-1, _U("&Chillers"))
        self.EditDBBoiler = self.submenuEquipments.Append(-1, _U("B&oilers"))
##        self.EditDBStorage = self.submenuEquipments.Append(-1, _U("Stora&ge"))
        self.EditDBSolarThermal = self.submenuEquipments.Append(-1, _U("&Solar Collectors"))

        self.EditSubDB = self.menuDatabase.AppendMenu(-1, _U("Equipments"), self.submenuEquipments)
        self.EditDBFuel = self.menuDatabase.Append(-1, _U("Fue&ls"))
        self.EditDBFluid = self.menuDatabase.Append(-1, _U("Flui&ds"))
        self.EditDBElectricityMix = self.menuDatabase.Append(-1, _U("Electricity mix"))
        self.EditDBProfiles = self.menuDatabase.Append(-1, _U("Time schedules (profiles)"))
        self.EditDBBenchmark = self.menuDatabase.Append(-1, _U("&Benchmarks"))
##        self.EditDBBAT = self.menuDatabase.Append(-1, _U("Best available technologies"))
        self.EditDBAuditor = self.menuDatabase.Append(-1, _U("Auditor"))
        self.menuDatabase.AppendSeparator()
        self.ImportData = self.menuDatabase.Append(-1, _U("Import data"))
        self.ExportData = self.menuDatabase.AppendMenu(-1, _U("Export data"), self.submenuExportDB)
        self.menuDatabase.AppendSeparator()
        self.EditDBAdmin = self.menuDatabase.Append(-1, _U("Database administration"))


##        self.EditDBNaceCode = self.submenuClassification.Append(-1, _U("&Nace code"))
##        self.EditDBUnitOperation = self.submenuClassification.Append(-1, _U("&Unit operation"))

        self.ExportAll = self.submenuExportDB.Append(-1, _U("all data bases"))
        self.ExportEquipments = self.submenuExportDB.AppendMenu(-1, _U("Equipments"), self.submenuExportEquipments)
        self.ExportDBFuel = self.submenuExportDB.Append(-1, _U("Fue&ls"))
        self.ExportDBFluid = self.submenuExportDB.Append(-1, _U("Flui&ds"))
        self.ExportDBElectricityMix = self.submenuExportDB.Append(-1, _U("Electricity mix"))
        self.ExportDBBenchmark = self.submenuExportDB.Append(-1, _U("&Benchmarks"))
##        self.ExportDBBAT = self.submenuExportDB.Append(-1, _U("Best available technologies"))

        self.ExportAllEquipments = self.submenuExportEquipments.Append(-1, _U("all equipment data bases"))
        self.ExportDBCHP = self.submenuExportEquipments.Append(-1, _U("&CHP"))
        self.ExportDBHeatPump = self.submenuExportEquipments.Append(-1, _U("&Heat pumps")+u'/'+_U("&Chillers"))
##        self.ExportDBChiller = self.submenuExportEquipments.Append(-1, _U("&Chillers"))
        self.ExportDBBoiler = self.submenuExportEquipments.Append(-1, _U("B&oilers"))
##        self.ExportDBStorage = self.submenuExportEquipments.Append(-1, _U("Stora&ge"))
        self.ExportDBSolarThermal = self.submenuExportEquipments.Append(-1, _U("&Solar Collectors"))

        self.interactionLevelList = []
        i = wx.NewId()
        self.interactionLevelList.append(i)
        self.UserSelectLevel1 = self.submenuUserLevel.AppendRadioItem(i, INTERACTIONLEVELS[0])
        i = wx.NewId()
        self.interactionLevelList.append(i)
        self.UserSelectLevel2 = self.submenuUserLevel.AppendRadioItem(i, INTERACTIONLEVELS[1])
        i = wx.NewId()
        self.interactionLevelList.append(i)
        self.UserSelectLevel3 = self.submenuUserLevel.AppendRadioItem(i, INTERACTIONLEVELS[2])
        self.UserLevel = self.menuSettings.AppendMenu(-1, _U("User &interaction level"), self.submenuUserLevel)
        self.Preferences = self.menuSettings.Append(-1, _U("&Preferences"))
##        self.Classification = self.menuSettings.AppendMenu(-1, _U("C&lassification"), self.submenuClassification)
        self.Language = self.menuSettings.Append(-1, _U("&Language"))
        self.Units = self.menuSettings.Append(-1, _U("&Unit System"))
        self.Auditor = self.menuSettings.Append(-1,_U("&Auditor"))

        i = wx.NewId()
        self.ViewMessages = self.menuSettings.AppendCheckItem(i, _U("View message log"))

        self.ManageFonts = self.menuSettings.Append(-1, _U("Fonts"))
        
##        self.HelpUserManual = self.menuHelp.Append(-1, _U("&Documentation"))
##        self.menuHelp.AppendSeparator()
        self.HelpSupport = self.menuHelp.Append(-1, _U("&Documentation and Support"))
        self.menuHelp.AppendSeparator()
        self.HelpAbout = self.menuHelp.Append(-1, _U("&About"))
        
        self.menuBar.Append(self.menuProject, _U("&Project"))
        self.showMainMenuAlternatives()

        self.menuBar.Append(self.menuDatabase, _U("&Database"))
        self.menuBar.Append(self.menuSettings, _U("&Settings"))
        self.menuBar.Append(self.menuHelp, _U("&Help"))

        self.SetMenuBar(self.menuBar)
        self.menuBar.Check(i, True)

    def showMainMenuAlternatives(self):
        checkedId = None
        self.menuAlternatives = wx.Menu()
        id0 = wx.NewId()
        id1 = id0
        for al in Status.prj.getAlternativeList():
            if al[1] == "":
                label = u" "
            else:
                label = U(al[1])
            self.menuAlternatives.AppendRadioItem(id1, label)
            # save the active option id for checking later
            if al[1] == Status.ActiveAlternativeName:
                checkedId = id1
            id1 = wx.NewId()

        self.rangeId = (id0, id1 - 1)
        self.Bind(wx.EVT_MENU_RANGE, self.OnMenuAlternatives, id=self.rangeId[0], id2=self.rangeId[1])
        pos = self.menuBar.FindMenu(_U('Alternatives'))
        if pos < 0:
            # the first time just append
            self.menuBar.Append(self.menuAlternatives, _U("Alternatives"))
        else:
            # first delete existing
            self.menuBar.Remove(pos)
            # then insert new
            self.menuBar.Insert(1, self.menuAlternatives, _U("Alternatives"))

        if checkedId is not None:
            self.menuBar.Check(checkedId, True)

    def changeAssistantMainMenu(self, level):
        try:
            id = self.interactionLevelList[level]
            self.menuBar.Check(id, True)
        except AttributeError:
            pass

    def CreateTree(self):
        # access to font properties object
        fp = FontProperties()
        self.tree = wx.TreeCtrl(self.treeArea, -1, wx.Point(0, 0), wx.Size(200, 740),
                                wx.TR_DEFAULT_STYLE | wx.TR_NO_LINES | wx.TR_HIDE_ROOT | \
                                wx.TR_FULL_ROW_HIGHLIGHT | wx.TR_HAS_VARIABLE_ROW_HEIGHT)

        DARKORANGE = '#DD5000'
        DARKGREY = '#0F0F0F'
        font = fp.getFont()
        font.SetPointSize(10)
        self.tree.SetFont(font)
        self.tree.SetForegroundColour(DARKGREY)
        
        self.qRoot = self.tree.AddRoot("ROOT")
        self.qCurrent = self.tree.AppendItem(self.qRoot, _U("Selected Alternative"), 0)

        self.qQ0 = self.tree.AppendItem (self.qCurrent, _U("Edit / view data"), 0)
        self.qQ1 = self.tree.AppendItem (self.qQ0, _U("General data"), 0)
        self.qMET = self.tree.AppendItem(self.qQ0, _U("Weather data"), 0)
##        self.qPage2 = self.tree.AppendItem (self.qQ0, _U("Energy consumption"), 0)
##        self.qPage3 = self.tree.AppendItem (self.qQ0, _U("Processes data"), 0)
##        self.qQ3sub = [] #space for allocation of subitems
##        
##        self.qPage4 = self.tree.AppendItem (self.qQ0, _U("Generation of heat and cold"), 0)
##        self.qPage5 = self.tree.AppendItem (self.qQ0, _U("Distribution of heat and cold"), 0)
        self.qCTRL = self.tree.AppendItem (self.qQ0, _U("Control model"))
##        self.qHX = self.tree.AppendItem (self.qQ0, _U("Heat recovery"), 0)
##        self.qWHEE = self.tree.AppendItem (self.qQ0, _U("Electrical Equipment"), 0)
        self.qQ7 = self.tree.AppendItem (self.qQ0, _U("Renewable energies"), 0)
        self.qQ8 = self.tree.AppendItem (self.qQ0, _U("Buildings"), 0)
        self.qQM = self.tree.AppendItem (self.qQ0, _U("Monitoring Data"), 0)
##        self.qMF = self.tree.AppendItem(self.qQ0, _U("Material flows"), 0)

        self.qCC = self.tree.AppendItem (self.qCurrent, _U("Consistency Check"))

        #Design
        self.qDesign = self.tree.AppendItem (self.qCurrent, _U("Assisted design"))
##        #Process optimization
##        self.qPO = self.tree.AppendItem (self.qDesign, _U("Process optimization"))
        #Process optimization interface 1
        self.qHR = self.tree.AppendItem (self.qDesign, _U("HX network"))
        #H&C Supply
        self.qHC = self.tree.AppendItem (self.qDesign, _U("H&C Supply"))
        #H&C Storage
        #self.qStorage = self.tree.AppendItem (self.qHC, _U("H&C Storage"))
##        #CHP
        self.qCHP = self.tree.AppendItem (self.qHC, _U("CHP"))
        #Solar Thermal
        self.qST = self.tree.AppendItem (self.qHC, _U("Solar Thermal"))
        #Heat Pumps
        self.qHP = self.tree.AppendItem (self.qHC, _U("Heat Pumps"))
        #Biomass
        #self.qBIO = self.tree.AppendItem (self.qHC, _U("Biomass"))
##        #Chillers
        self.qCH = self.tree.AppendItem (self.qHC, _U("Chillers"))
        #Boilers & burners
        self.qBB = self.tree.AppendItem (self.qHC, _U("Boilers & burners"))
##        #H&C Distribution
##        self.qDistribution = self.tree.AppendItem (self.qDesign, _U("H&C Distribution"))

        self.qAUTO = self.tree.AppendItem(self.qDesign, _U("Auto-Pilot"))
        #
        # Energy performance
        #

        self.qEnergy = self.tree.AppendItem (self.qCurrent, _U("Energy performance"))

##        self.qStatistics = self.tree.AppendItem (self.qCurrent, _U("Energy statistics"))
##        self.qEA = self.tree.AppendItem (self.qEnergy, _U("Annual data"))
        # annual statistics subtree
        self.qEA1 = self.tree.AppendItem (self.qEnergy, _U("Primary energy"))
        self.qEA2 = self.tree.AppendItem (self.qEnergy, _U("Final energy by fuels"))
        self.qEA3 = self.tree.AppendItem (self.qEnergy, _U("Final energy by equipment"))
        self.qEA4a = self.tree.AppendItem (self.qEnergy, _U("H&C demand (proc.)"))
        self.qEA4b = self.tree.AppendItem (self.qEnergy, _U("H&C demand (temp.)"))
        self.qEA4c = self.tree.AppendItem (self.qEnergy, _U("H&C demand (time)"))
        self.qEA5 = self.tree.AppendItem (self.qEnergy, _U("Energy intensity"))

        self.qEM = self.tree.AppendItem (self.qEnergy, _U("Monthly data"))
        self.qEH = self.tree.AppendItem (self.qEnergy, _U("Hourly performance data"))
        self.qPA = self.tree.AppendItem (self.qEnergy, _U("Pinch analysis"))

       # monthly statistics subtree
        self.qEM1 = self.tree.AppendItem (self.qEM, _U("Monthly demand"))
        self.qEM2 = self.tree.AppendItem (self.qEM, _U("Monthly supply"))
        
##        # hourly statistics subtree
        self.qEH1 = self.tree.AppendItem (self.qEH, _U("Hourly demand"))
        self.qEH2 = self.tree.AppendItem (self.qEH, _U("Hourly supply"))

        self.qPA1 = self.tree.AppendItem (self.qPA, _U("List of streams"))
        self.qPA2 = self.tree.AppendItem (self.qPA, _U("Composite curves"))
 

        self.qBM = self.tree.AppendItem (self.qCurrent, _U("Benchmark check"))
        self.qBM1 = self.tree.AppendItem (self.qBM, _U("Global energy intensity"))
        self.qBM2 = self.tree.AppendItem (self.qBM, _U("SEC by product"))
        self.qBM3 = self.tree.AppendItem (self.qBM, _U("SEC by process"))
        
        self.qA = self.tree.AppendItem (self.qRoot, _U("Alternative proposals"))


        
        #Total Cost Assessment
        self.qECO = self.tree.AppendItem (self.qCurrent, _U("Economic analysis"))  
        self.qECO0 = self.tree.AppendItem (self.qECO, _U("Economic parameters"), 0)
        self.qECO1 = self.tree.AppendItem (self.qECO, _U("Cost details"))

##        #Comparative analysis
##        self.qCS = self.tree.AppendItem (self.qA, _U("Comparative study"))
        #Comparative study – Detail Info 1
        self.qCS1 = self.tree.AppendItem (self.qA, _U("Primary energy"))
        #Comparative study – Detail Info 2
        self.qCS2 = self.tree.AppendItem (self.qA, _U("Process & supply heat"))
        #Comparative study – Detail Info 3
        self.qCS3 = self.tree.AppendItem (self.qA, _U("Environmental impact"))
        #Comparative study – Detail Info 4
        self.qCS4 = self.tree.AppendItem (self.qA, _U("Investment cost"))
        #Comparative study – Detail Info 5
        self.qCS5 = self.tree.AppendItem (self.qA, _U("Annual cost"))
        #Comparative study – Detail Info 6
        self.qCS6 = self.tree.AppendItem (self.qA, _U("Additional cost per saved energy"))
        #Comparative study – Detail Info 7
        self.qCS7 = self.tree.AppendItem (self.qA, _U("Internal rate of return"))

        self.qReport = self.tree.AppendItem (self.qRoot, _U("Report"))
        
        #self.tree.Expand(self.qRoot)
        self.tree.Expand(self.qCurrent)
        self.tree.Expand(self.qQ0)
        self.tree.Expand(self.qEnergy)
##        self.tree.Expand(self.qBM)
        self.tree.Expand(self.qA)
                
    def clearSubTree(self, subTree, mainElem):
        popBlocker = self.blockTreeEvents
        self.blockTreeEvents = True
        for elem in subTree:
            pass
            self.tree.DeleteChildren(elem)
        self.tree.DeleteChildren(mainElem)
        subTree = []
        self.blockTreeEvents = popBlocker
        
    def modifyTree(self, label, newItem=None, reset=False):
        """ for use from GUI panels
        Adds a new item to the tree under the branch specified by label

        :params label (codified identifier of the branch, e.g. EA, PO, etc.

        """
        pass # deactivated in GUI3. Kept as comment in case of being needed again
##        self.blockTreeEvents = True
##
##        if label == "Q3":
##            if reset:
##                self.clearSubTree(self.qQ3sub, self.qPage3)
##                self.qQ3sub = []
##            else:
##                self.qQ3sub.append(self.tree.AppendItem (self.qPage3, newItem))
##                self.tree.SetItemTextColour(self.qQ3sub[-1], ORANGE)
##        self.blockTreeEvents = False
            
    def unselectTree(self):
        popBlocker = self.blockTreeEvents
        self.blockTreeEvents = True
        self.tree.Unselect() # TODO (2020-04-28): UnselectAll gave problems on Windows -> some root item got selected, pop-up cycling ...
        wx.SafeYield()
        self.blockTreeEvents = popBlocker
        
    def selectTreeItem(self, code, n=None, blockEvents=True):
        """ for use from GUI panels
        sets selection on the currently active item on the GUI

        :params n: object number (numbering from 0 to N-1

        """
        popBlocker = self.blockTreeEvents
        self.blockTreeEvents = blockEvents

        try:
            item = getattr(self, "q" + code)
        except:
            item = None

        if item is not None:
            self.tree.SelectItem(item)
            
        self.blockTreeEvents = popBlocker

        self.control(code, n)
            
    def BindEvents(self):
        #--- binding the menu
        self.Bind(wx.EVT_MENU, self.OnMenuOpenProject, self.openProject)
        self.Bind(wx.EVT_MENU, self.OnMenuCopyProject, self.copyProject)
        self.Bind(wx.EVT_MENU, self.OnMenuNewProject, self.newProject)
        self.Bind(wx.EVT_MENU, self.OnMenuDeleteProject, self.deleteProject)
        self.Bind(wx.EVT_MENU, self.OnMenuExit, self.ExitApp)
        self.Bind(wx.EVT_MENU, self.OnMenuImportData, self.ImportData)
        self.Bind(wx.EVT_MENU, self.OnMenuImportProject, self.importProject)
        self.Bind(wx.EVT_MENU, self.OnMenuExportProject, self.exportProject)
##        self.Bind(wx.EVT_MENU, self.OnMenuImportQ, self.ImportQ)

        self.Bind(wx.EVT_MENU, self.OnMenuEditDBAdmin, self.EditDBAdmin)
        self.Bind(wx.EVT_MENU, self.OnMenuEditDBAuditor, self.EditDBAuditor)
        self.Bind(wx.EVT_MENU, self.OnMenuEditDBBenchmark, self.EditDBBenchmark)
##        self.Bind(wx.EVT_MENU, self.OnMenuEditDBBAT, self.EditDBBAT)
##        self.Bind(wx.EVT_MENU, self.OnMenuEditDBNaceCode, self.EditDBNaceCode)
##        self.Bind(wx.EVT_MENU, self.OnMenuEditDBUnitOperation, self.EditDBUnitOperation)
        self.Bind(wx.EVT_MENU, self.OnMenuEditDBCHP, self.EditDBCHP)
        self.Bind(wx.EVT_MENU, self.OnMenuEditDBHeatPump, self.EditDBHeatPump)
        self.Bind(wx.EVT_MENU, self.OnMenuEditDBFluid, self.EditDBFluid)
        self.Bind(wx.EVT_MENU, self.OnMenuEditDBFuel, self.EditDBFuel)
        self.Bind(wx.EVT_MENU, self.OnMenuEditDBElectricityMix, self.EditDBElectricityMix)
        self.Bind(wx.EVT_MENU, self.OnMenuEditDBBoiler, self.EditDBBoiler)
        self.Bind(wx.EVT_MENU, self.OnMenuEditDBSolarThermal, self.EditDBSolarThermal)
        #self.Bind(wx.EVT_MENU, self.OnMenuEditDBChiller, self.EditDBChiller)
        self.Bind(wx.EVT_MENU, self.OnMenuEditDBProfiles, self.EditDBProfiles)

        self.Bind(wx.EVT_MENU, self.OnMenuExportAll, self.ExportAll)
        self.Bind(wx.EVT_MENU, self.OnMenuExportDBBenchmark, self.ExportDBBenchmark)
        self.Bind(wx.EVT_MENU, self.OnMenuExportAllEquipments, self.ExportAllEquipments)
        self.Bind(wx.EVT_MENU, self.OnMenuExportDBCHP, self.ExportDBCHP)
        self.Bind(wx.EVT_MENU, self.OnMenuExportDBHeatPump, self.ExportDBHeatPump)
        self.Bind(wx.EVT_MENU, self.OnMenuExportDBFluid, self.ExportDBFluid)
        self.Bind(wx.EVT_MENU, self.OnMenuExportDBFuel, self.ExportDBFuel)
        self.Bind(wx.EVT_MENU, self.OnMenuExportDBElectricityMix, self.ExportDBElectricityMix)
        self.Bind(wx.EVT_MENU, self.OnMenuExportDBBoiler, self.ExportDBBoiler)
        self.Bind(wx.EVT_MENU, self.OnMenuExportDBSolarThermal, self.ExportDBSolarThermal)
##        self.Bind(wx.EVT_MENU, self.OnMenuExportDBChiller, self.ExportDBChiller)

        self.Bind(wx.EVT_MENU, self.OnMenuUserSelectLevel1, self.UserSelectLevel1)
        self.Bind(wx.EVT_MENU, self.OnMenuUserSelectLevel2, self.UserSelectLevel2)
        self.Bind(wx.EVT_MENU, self.OnMenuUserSelectLevel3, self.UserSelectLevel3)

        self.Bind(wx.EVT_MENU, self.OnMenuSettingsPreferences, self.Preferences)
        self.Bind(wx.EVT_MENU, self.OnMenuSettingsLanguage, self.Language)
        self.Bind(wx.EVT_MENU, self.OnMenuSettingsUnits, self.Units)
        self.Bind(wx.EVT_MENU, self.OnMenuSettingsAuditor, self.Auditor)
        self.Bind(wx.EVT_MENU, self.OnMenuSettingsViewMessages, self.ViewMessages)
        self.Bind(wx.EVT_MENU, self.OnMenuSettingsManageFonts, self.ManageFonts)

##        self.Bind(wx.EVT_MENU, self.OnMenuHelpUserManual, self.HelpUserManual)
        self.Bind(wx.EVT_MENU, self.OnMenuHelpSupport, self.HelpSupport)
        self.Bind(wx.EVT_MENU, self.OnMenuHelpAbout, self.HelpAbout)

        #--- binding the Tree
        self.blockTreeEvents = False

        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnTreeSelChanged, self.tree)
##        self.Bind(wx.EVT_TREE_SEL_CHANGING, self.OnTreeSelChanging, self.tree)
##        self.Bind(wx.EVT_TREE_ITEM_EXPANDING, self.OnTreeItemExpanding, self.tree)
##
##        self.Bind(wx.EVT_TREE_ITEM_RIGHT_CLICK, self.OnTreeItemRightClick, self.tree)
##        self.Bind(wx.EVT_TREE_BEGIN_DRAG, self.OnTreeBeginDrag, self.tree)
##        self.Bind(wx.EVT_TREE_END_DRAG, self.OnTreeEndDrag, self.tree)


    def handleLicense(self):
        """
        carries out interactive license handling once the GUI has been opened
        """

        if Status.License.user=="demo" and not Status.License.check:    # nothing to do, time expired
            logger.showError(_("Time expired for demo version. EINSTEIN will stop"))
            sys.exit()
            
        # make sure that an auditor is set
        if (Status.AuditorId is None and Status.License.user=="demo") or not Status.License.check:
            logger.showMessage(_("Your license is not valid for the currently selected auditor\n\n") + \
                               _("Please select a valid auditor and/or add your correct auditor informations") + \
                               _("Restart EINSTEIN afterwards"))

            self.OnMenuSettingsAuditor(None)
            
        # in case of running a demo version, request a key
        elif Status.License.check == 2:   # demo version
            chk = askConfirmation(_("Do You want to create a request for a license key for the active auditor ?"))
            if chk:
                showMessage(_("You will be requested to save a file for the request on your computer\n") +
                            _("Please send this request by e-mail to your EINSTEIN distributor"))

                ntry = 1
                while not Status.License.request() and ntry <= 3:
                    dlg = AuditorPopUp(self, auditorId = Status.AuditorId)
                    ret = dlg.ShowModal()
                    if not ret:
                        break
                    
                    ntry += 1            

        
#Pop-up menu for tree
class TreePopUpMenu(wx.Menu):

    def __init__(self, parent):
        super(TreePopUpMenu, self).__init__()
        
        self.parent = parent

        menuRename = self.Append(-1, _U("rename"))
        self.Bind(wx.EVT_MENU, self.OnRename, menuRename)

        menuCopy = self.Append(-1, _U("copy"))
        self.Bind(wx.EVT_MENU, self.OnCopy, menuCopy)

        menuCreateNew = self.Append(-1, _U("create new"))
        self.Bind(wx.EVT_MENU, self.OnCreateNew, menuCreateNew)

        self.AppendSeparator()

        menuDelete = self.Append(-1, _U("delete"))
        self.Bind(wx.EVT_MENU, self.OnDelete, menuDelete)


    def OnRename(self, event):
        self.parent.showWarning("Renaming of Processes not yet implemented. Use COPY as a workaround")

    def OnCopy(self, event):
        self.parent.Page3.copyProcess()

    def OnCreateNew(self, event):
        self.parent.Page3.createNewProcess()

    def OnDelete(self, event):
        self.parent.Page3.deleteProcess()



#   Application module
class EinsteinApp(wx.App):
    def __init__(self):
        wx.App.__init__(self, redirect=False)

    def OnInit(self):
        self.Bind(wx.EVT_QUERY_END_SESSION , self._onQueryEndSession)
        self.Bind(wx.EVT_END_SESSION       , self._onEndSession)
        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        
        self.frame = EinsteinFrame(parent=None, id= -1, title="Einstein", wxapp=self)
        # Status.main has to exist BEFORE instantiating project
        Status.main = self.frame

        self.SetTopWindow(self.frame)

##        #initialize fonts management
##        # (needs database connected)
        fp = FontProperties()
        fp.initializeFont()
        

        # create Einstein UI
        self.frame.createUI()
        self.frame.Show()
        self.frame.Bind(wx.EVT_CLOSE, self._onFrameClose)
        self.SetTopWindow(self.frame)

        return True
        
    def OnExit(self):
        sys.stdout.flush()

    def OnQueryEndSession(self):
        sys.stdout.flush()

    def _onQueryEndSession(self, event):
        sys.stdout.flush()

    def _onEndSession(self, event):
        sys.stdout.flush()

    def _onFrameClose(self, event):
        sys.stdout.flush()
        self.frame.Destroy()
        wx.Exit()

    def OnKeyDown(self, event):
        keycode = event.GetKeyCode()
##        print "EinsteinApp.OnKeyDown: keycode ", keycode
##        if keycode == wx.WXK_ESCAPE:
##            print "ESCAPE"
        event.Skip()


##if __name__ == '__main__':
##    PROFILE = True
##    if PROFILE is True:
##        try:
##            import cProfile
##            import pstats
##            cProfile.run("main()", "runstats.txt")
##            s = pstats.Stats("runstats.txt")
##            s.sort_stats("cumulative")
##            s.print_stats(200)
##        except ImportError:
##            print >> sys.stderr, "Profiling module could not be imported, starting normal." 
##            main()
##    else:
##        main()
    
