# -*- coding: utf-8 -*-
#==============================================================================
#
#    E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       www.einstein-energy.net
#
#------------------------------------------------------------------------------
#
#    editCon: Visualise the connection of generators, pipes and processes
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
import wx
from modules.energyStats.moduleEditCon import ModuleEditCon
from GUI.status import Status
from modules.fluids import Fuel
from constants import EPSILON, INFINITE
from GUI.GUITools import ORANGE
from globals import logger, Globals
from model.systemConnections import EnergyFlow
from modules.guiModules.moduleQ6 import ModuleQ6
from GUI.panelQ6 import EditConnection
from auxiliary.auxiliary import noneCheck
from GUI.displayCC import CCScheme

YPOSMARGIN = 20

from lib.GUI.drawing.drawingPanel import DrawingPanel
from lib.GUI.drawing.moduleIcon import ModuleIcon, connectIcons

POPUPMENU =[_("fuel"), _("process"), _("pipe/duct"), _("equipment"), _("heat exchanger"), _("other source of waste heat")]
POPUPMENUIDS = [wx.NewId() for item in POPUPMENU]

OCODE= {"PROC": 0, "PIPE": 1, "EQ": 2, "WHEE": 3, "HX": 4, "FUEL": None, "EL": None}

PREFIX = ".          "

class EditCon(DrawingPanel):

    def __init__(self, parent, main):
        super(EditCon, self).__init__(parent, ModuleEditCon())

        self.main = main
        self.prnt = parent

        self.conLabelWidth = 35
        self.conSize = (35, 12)
        self.conPosIn = (25, 0)
        self.conPosOut = (-25, 0)

##        self.display()
##        parent.Show()

        self.conHandler = 0     # 0: standard -> generates a direct connection between objects where possible
                                # 1: hx connection -> generates a heat exchanger from source to sink


    def buildDrawing(self):

        if not self.data: return # security feature if no data are loaded

        self.bitMaps = {"Fuel": wx.Bitmap("GUI/img/fuelIcon.png"),
                      "El": wx.Bitmap("GUI/img/electricityIcon.png"),
                      "BB": wx.Bitmap("GUI/img/equipmentIcon-BB.png"),  # maintained as dummy-icon
                      "BB-Boiler": wx.Bitmap("GUI/img/equipmentIcon-BB-Boiler.png"),
                      "BB-Heater": wx.Bitmap("GUI/img/equipmentIcon-BB-Heater.png"),
                      "BB-Burner": wx.Bitmap("GUI/img/equipmentIcon-BB-Burner.png"),
                      "CH-Comp": wx.Bitmap("GUI/img/equipmentIcon-CH-Comp.png"),
                      "CH-CompAir": wx.Bitmap("GUI/img/equipmentIcon-CH-CompAir.png"),
                      "CH-Abs": wx.Bitmap("GUI/img/equipmentIcon-CH-Abs.png"),
                      "CH-CT": wx.Bitmap("GUI/img/equipmentIcon-CH-CT.png"),
                      "CH-CTDry": wx.Bitmap("GUI/img/equipmentIcon-CH-CTDry.png"),
                      "CH-FW": wx.Bitmap("GUI/img/equipmentIcon-CH-FreshWater.png"),
                      "CHP-GT": wx.Bitmap("GUI/img/equipmentIcon-CHP-GT.png"),
                      "CHP-Engine": wx.Bitmap("GUI/img/equipmentIcon-CHP-Engine.png"),
                      "HTP-RC": wx.Bitmap("GUI/img/equipmentIcon-HTP-RC.png"),
                      "HP-Comp": wx.Bitmap("GUI/img/equipmentIcon-HP-Comp.png"),
                      "HP-MVC": wx.Bitmap("GUI/img/equipmentIcon-HP-MVC.png"),
                      "ST": wx.Bitmap("GUI/img/equipmentIcon-ST.png"),
                      "Pipe-closedH": wx.Bitmap("GUI/img/pipeIcon-ClosedH.png"),
                      "Pipe-closedC": wx.Bitmap("GUI/img/pipeIcon-ClosedC.png"),
                      "Pipe-openH": wx.Bitmap("GUI/img/pipeIcon-OpenH.png"),
                      "Pipe-openC": wx.Bitmap("GUI/img/pipeIcon-OpenC.png"),
                      "Pipe-directH": wx.Bitmap("GUI/img/pipeIcon-DirectH.png"),
                      "Pipe-directC": wx.Bitmap("GUI/img/pipeIcon-DirectC.png"),
                      "ProcH": wx.Bitmap("GUI/img/processHIcon.png"),
                      "ProcH-SMI": wx.Bitmap("GUI/img/processHIcon-SMI.png"),
                      "ProcH-SMO": wx.Bitmap("GUI/img/processHIcon-SMO.png"),
                      "ProcH-SM": wx.Bitmap("GUI/img/processHIcon-SM.png"),
                      "ProcH-IO": wx.Bitmap("GUI/img/processHIcon-IO.png"),
                      "ProcH-I": wx.Bitmap("GUI/img/processHIcon-I.png"),
                      "ProcH-BUI": wx.Bitmap("GUI/img/processHIcon-BUI.png"),
                      "ProcHC": wx.Bitmap("GUI/img/processHCIcon.png"),
                      "ProcC": wx.Bitmap("GUI/img/processCIcon.png"),
                      "ProcC-SMI": wx.Bitmap("GUI/img/processCIcon-SMI.png"),
                      "ProcC-SMO": wx.Bitmap("GUI/img/processCIcon-SMO.png"),
                      "ProcC-SM": wx.Bitmap("GUI/img/processCIcon-SM.png"),
                      "ProcC-IO": wx.Bitmap("GUI/img/processCIcon-IO.png"),
                      "ProcC-I": wx.Bitmap("GUI/img/processCIcon-I.png"),
                      "ProcC-BUI": wx.Bitmap("GUI/img/processCIcon-BUI.png"),
                      "HXSrc": wx.Bitmap("GUI/img/hxIconSrc.png"),
                      "HXSnk": wx.Bitmap("GUI/img/hxIconSnk.png"),
                      "WHEE": wx.Bitmap("GUI/img/electricityIcon.png"),
                      "WARN": wx.Bitmap("GUI/img/icon_warning.png")}

        self.ymaxDrawing = 600 # minimum window size
        
        self.lines = []
        self.connections = []
        self.flow = []
        
        self.process = self.data["Processes"]
        self.pipe = self.data["Pipes"]
        self.equipment = self.data["Equipment"]
        HXIn = self.data["HXIn"]
        HXOut = self.data["HXOut"]

        NK = len(self.process)
        NJ = len(self.equipment)
        NM = len(self.pipe)

        systemModel = Status.int.systemModel

        self.processIcon = []
        self.pipeIcon = []
        self.equipmentIcon = []

        self.hxIcon = []

        jCool = self.data["jCool"]
        yposCool = YPOSMARGIN + 100*jCool

        ypos = YPOSMARGIN
        yposMin = YPOSMARGIN
        
        for k in range(NK):
            if self.process[k].heating:
                colour = wx.RED
                UPHw = self.process[k].UPHw
                if UPHw == 0:
                    UPHw = None

                # select bitMap depending on process type
                proc = self.process[k]
                if proc.buildingId:
                    bitmap = self.bitMaps["ProcH-BUI"]
                    
                elif proc.hasMCD:
                    bitmap = self.bitMaps["ProcHC"]
                    
                elif proc.hasStartUp or proc.hasMaintenance:
                    if proc.nIn > 0 and proc.nOut > 0:
                        bitmap = self.bitMaps["ProcH"]
                    elif proc.nIn > 0:
                        bitmap = self.bitMaps["ProcH-SMI"]
                    elif proc.nOut > 0:
                        bitmap = self.bitMaps["ProcH-SMO"]
                    else:
                        bitmap = self.bitMaps["ProcH-SM"]
                        
                else:
                    if proc.nOut > 0:
                        bitmap = self.bitMaps["ProcH-IO"]
                    else:
                        bitmap = self.bitMaps["ProcH-I"]
                    
                
                self.processIcon.append(ModuleIcon(parent=self, \
                                                   pos=wx.Point(620,ypos), \
                                                   name = self.process[k].name,\
                                                   code = ("PROC", k),\
                                                   colour = colour, \
                                                   inFlow = [strRep(self.process[k].UPH),
                                                             "",
                                                             strRep(self.process[k].QHXProc, prefix=PREFIX, minVal=100)],
                                                   outFlow = [strRep(UPHw)],
                                                   bitmap = bitmap))

                self.addHXIcons(HXIn["Proc"][k], HXOut["Proc"][k], (620, ypos), colour)

                ypos += 100
            else:
                self.processIcon.append(None)

        ypos = max(ypos, yposCool)
        for k in range(NK):
            if not self.process[k].heating:
                colour = wx.BLUE
                UPHw = self.process[k].UPHw
                if UPHw == 0:
                    UPHw = None

                proc = self.process[k]
                if proc.buildingId:
                    bitmap = self.bitMaps["ProcC-BUI"]
                elif proc.hasStartUp or proc.hasMaintenance:
                    if proc.nIn > 0 and proc.nOut > 0:
                        bitmap = self.bitMaps["ProcC"]
                    elif proc.nIn > 0:
                        bitmap = self.bitMaps["ProcC-SMI"]
                    elif proc.nOut > 0:
                        bitmap = self.bitMaps["ProcC-SMO"]
                    else:
                        bitmap = self.bitMaps["ProcC-SM"]
                        
                else:
                    if proc.nOut > 0:
                        bitmap = self.bitMaps["ProcC-IO"]
                    else:
                        bitmap = self.bitMaps["ProcC-I"]
                      
                self.processIcon[k] = ModuleIcon(parent=self, \
                                                   pos=wx.Point(620,ypos), \
                                                   name = self.process[k].name,\
                                                   code = ("PROC", k), \
                                                   colour = colour, \
                                                   inFlow = [strRep(self.process[k].UPH),
                                                             "",
                                                             strRep(self.process[k].QHXProc, prefix=PREFIX, minVal=100)],
                                                   outFlow = [strRep(UPHw)],
                                                   bitmap = bitmap)

                self.addHXIcons(HXIn["Proc"][k], HXOut["Proc"][k], (620, ypos), colour, cooling=True)

                ypos += 100



        
        yposH = YPOSMARGIN
        yposC = YPOSMARGIN
        ypos = YPOSMARGIN

        occupied = []
        
        for m in range(NM):

            processFound = False
            colour = wx.BLACK #default colour for not connected pipes
            for k in range(NK):
                if (m, k) in systemModel.UPHLink or (m, k) in systemModel.UPCReqLink:
                    processFound = True

                    yposMin = self.processIcon[k].pos[1]
                    if self.process[k].heating and (m, k) in systemModel.UPHLink:
                        colour = wx.RED
                        ypos = yposMin
                        yposH = ypos+100
                    else:
                        colour = wx.BLUE
                        ypos = max(yposH,yposMin,yposCool)
                        yposC = ypos+100
                    break
                    
            if processFound:
                while ypos in occupied:
                    ypos += 100
                occupied.append(ypos)

                iconKey = "Pipe-" + self.pipe[m].pipeType + ("H" if self.pipe[m].heating else "C")
                self.pipeIcon.append(ModuleIcon(parent=self, \
                                                pos=wx.Point(420,ypos), \
                                                name = self.pipe[m].name,
                                                code = ("PIPE", m), \
                                                inFlow = [strRep(self.pipe[m].USHPipe),
                                                          "",
                                                          strRep(self.pipe[m].QHXPipe, prefix=PREFIX, minVal=100)],
                                                outFlow = [strRep(self.pipe[m].UPHProcm)],
                                                colour = colour,
                                                bitmap = self.bitMaps[iconKey]))

                self.addHXIcons(HXIn["Pipe"][m], HXOut["Pipe"][m], (420, ypos), colour, cooling=not self.pipe[m].heating)

            else:
                equipmentFound = False
                for j in range(NJ):
                    if (m, j) in systemModel.FEThLink:
                        equipmentFound = True
##                        yposMin = self.equipmentIcon[j].pos[1]
                        colour = wx.RED
                        ypos = yposMin
                        yposH = ypos+100
                        break
                        

                if equipmentFound:
                    while ypos in occupied:
                        ypos += 100
                    occupied.append(ypos)

                    iconKey = "Pipe-" + self.pipe[m].pipeType + ("H" if self.pipe[m].heating else "C")
                    self.pipeIcon.append(ModuleIcon(parent=self, \
                                                    pos=wx.Point(420,ypos), \
                                                    name = self.pipe[m].name,
                                                    code = ("PIPE", m), 
                                                    inFlow = [strRep(self.pipe[m].USHPipe),
                                                              "",
                                                              strRep(self.pipe[m].QHXPipe, prefix=PREFIX, minVal=100)],
                                                    outFlow = [strRep(self.pipe[m].UPHProcm)],
                                                    colour = colour,
                                                    bitmap = self.bitMaps[iconKey]))

                    self.addHXIcons(HXIn["Pipe"][m], HXOut["Pipe"][m], (420, ypos), colour, cooling=not self.pipe[m].heating)

                else:
                    ypos = YPOSMARGIN # find any free place
                    while ypos in occupied:
                        ypos += 100
                    occupied.append(ypos)

                    iconKey = "Pipe-" + self.pipe[m].pipeType + ("H" if self.pipe[m].heating else "C")
                    self.pipeIcon.append(ModuleIcon(parent=self, \
                                                pos=wx.Point(420,ypos), \
                                                name = self.pipe[m].name,
                                                code = ("PIPE", m),
                                                inFlow = [strRep(self.pipe[m].USHPipe)],
                                                outFlow = [strRep(self.pipe[m].UPHProcm)],
                                                colour = colour,
                                                bitmap = self.bitMaps[iconKey]))

                    self.addHXIcons(HXIn["Pipe"][m], HXOut["Pipe"][m], (420, ypos), colour, cooling=not self.pipe[m].heating)

        occupied = []
        ypos = YPOSMARGIN
        for j in range(NJ):

            yposMin = max(NM, NK, NJ)*100 + YPOSMARGIN
            for m in range(NM):
                if (j, m) in systemModel.USHLink:
                    yposMin = min(self.pipeIcon[m].pos[1], yposMin)

            if self.equipment[j].heating:
                colour = wx.RED
                if self.equipment[j].eqType in ["CHP ORC", "CHP TFC", "CHP steam turbine"]:
                    bitmap = self.bitMaps["HTP-RC"]
                elif self.equipment[j].eqType in ["CHP gas turbine"]:
                    bitmap = self.bitMaps["CHP-GT"]
                elif self.equipment[j].eqType in ["CHP engine", "CHP fuel cell"]:
                    bitmap = self.bitMaps["CHP-Engine"]
                elif self.equipment[j].eqType in ["solar thermal (flat-plate)",
                                                     "solar thermal (evacuated tubes)",
                                                     "solar thermal (parabolic trough)",
                                                     "solar thermal (fresnel)"]:
                    bitmap = self.bitMaps["ST"]
                elif self.equipment[j].eqClass=="BB" and self.equipment[j].fuel is None:
                    bitmap = self.bitMaps["BB-Heater"]                    
                elif self.equipment[j].eqType in ["steam boiler", "condensing boiler", "hot water boiler"]:
                    bitmap = self.bitMaps["BB-Boiler"]
                elif self.equipment[j].eqType in ["burner (direct heating)", "burner (indirect heating)"]:
                    bitmap = self.bitMaps["BB-Burner"]
                elif self.equipment[j].eqType in ["compression heat pump"]:
                    bitmap = self.bitMaps["HP-Comp"]             
                elif self.equipment[j].eqType in ["mechanical vapour recompression"]:
                    bitmap = self.bitMaps["HP-MVC"]             
                elif self.equipment[j].eqType in ["thermal heat pump"]:
                    bitmap = self.bitMaps["CH-Abs"]   #TODO: create an own icon for thermal heat pumps          
                else:
                    bitmap = self.bitMaps["BB"]
    ##                ypos = max(yposH,yposMin)
    ##                yposH = ypos+100
            else:
    ##                ypos = max(yposC,yposMin)
    ##                yposC = ypos+100
                colour = wx.BLUE
                if self.equipment[j].eqType in ["compression chiller (air cooled)"]:
                    bitmap = self.bitMaps["CH-CompAir"]
                elif self.equipment[j].eqType in ["thermal chiller (air cooled)", "thermal chiller (water cooled)"]:
                    bitmap = self.bitMaps["CH-Abs"]
                elif self.equipment[j].eqType in ["cooling tower (dry)"]:
                    bitmap = self.bitMaps["CH-CTDry"]
                elif self.equipment[j].eqType in ["cooling tower (wet)"]:
                    bitmap = self.bitMaps["CH-CT"]
                elif self.equipment[j].eqType in ["fresh or ground water"]:
                    bitmap = self.bitMaps["CH-FW"]             
                else:
                    bitmap = self.bitMaps["CH-Comp"]

            ypos = yposMin
            while ypos in occupied:
                ypos += 100

            occupied.append(ypos)

            try:
                FETin = self.equipment[j].FETFuel_j + self.equipment[j].FETHeat_j
            except:
                FETin = None
                
            self.equipmentIcon.append(ModuleIcon(parent=self, \
                                                 pos=wx.Point(220,ypos), \
                                                 name = self.equipment[j].name,
                                                 code = ("EQ", j), 
                                                 inFlow = [strRep(FETin),
                                                           strRep(self.equipment[j].FETel_j, minVal=100),
                                                           strRep(self.equipment[j].QHXj, prefix=PREFIX, minVal=100)],
                                                 outFlow = [strRep(self.equipment[j].USHj)],
                                                 colour = colour,
                                                 bitmap = bitmap))

            self.addHXIcons(HXIn["Eq"][j], HXOut["Eq"][j], (220, ypos+5), colour)


        self.whee = self.data["WHEE"]
        NN = len(self.whee)
        self.wheeIcon = []
        
        if len(self.equipmentIcon) > 0: ypos = max([icon.pos[1] for icon in self.equipmentIcon])

        colour = wx.RED
        bitmap = self.bitMaps["WHEE"]

        for n in range(NN):
            ypos += 100
            name = noneCheck(self.whee[n].WHEEName, "corrupt WHEE")
            self.wheeIcon.append(ModuleIcon(parent=self, \
                                                 pos=wx.Point(220,ypos), \
                                                 name = name,
                                                 code = ("WHEE", n), 
                                                 inFlow = [],
                                                 outFlow = [strRep(self.whee[n].QWHEE)],
                                                 colour = colour,
                                                 bitmap = bitmap))
            
            self.addHXIcons(HXIn["EE"][n], HXOut["EE"][n], (220, ypos+5), colour)
                   
            

        self.fuel = self.data["Fuels"]

        self.fuelIcon = []
        NI = len(self.fuel)
        for i in range(NI):
            ypos = (i)*100 + YPOSMARGIN

            fuelId = self.fuel[i].DBFuel_id
            
            try:
                fuelName = Fuel(fuelId).name
            except:
                fuelName = "unknown fuel"
                
            self.fuelIcon.append(ModuleIcon(parent=self,pos=wx.Point(20,ypos),\
                                            name = fuelName,
                                            code = ("FUEL", i),
                                            colour = wx.BLACK,
                                            outFlow = [strRep(self.fuel[i].FETFuel)],
                                            bitmap = self.bitMaps["Fuel"]))

        self.electricity = self.data["Electricity"]
        self.general = self.data["General"]
        self.electricityIcon = ModuleIcon(parent=self,pos=wx.Point(20,NI*100+YPOSMARGIN),\
                                          name = _("electricity"),
                                          code = ("EL", 0),
                                          colour = wx.BLACK,
                                          outFlow = [strRep(self.general.FETel)],
                                          bitmap = self.bitMaps["El"])


        self.icons = self.fuelIcon + \
                     [self.electricityIcon] + \
                     self.equipmentIcon + \
                     self.pipeIcon + \
                     self.processIcon + \
                     self.wheeIcon + \
                     self.hxIcon

        if Status.ANo > 0 and Status.StatusEnergy==0:
            warningIcon = ModuleIcon(parent=self,pos=wx.Point(20, 100*(NI+1)+YPOSMARGIN),\
                                          name = "\n\n\n" + _("energy flows not up to date") + ". \n" + _("run simulation"),
                                          code = ("WARN", 0),
                                          colour = wx.RED,
                                          outFlow = [],
                                          bitmap = self.bitMaps["WARN"])
            self.icons.append(warningIcon)
            
        # now build connections between icons
        nv = 0
        nconsj = [0]*NJ
        for j in range(NJ):
            nconsj[j] = 0
            for i in range(NI):
                if (i, j) in systemModel.FETFuelLink:
                    conLines = connectIcons(self.fuelIcon[i],self.equipmentIcon[j], nv = nv, nhy = nconsj[j])
                    self.lines += conLines
                    self.connections.append(conLines)
                    self.flow.append(EnergyFlow("FUEL", i, "EQ", j))
                    nv += 1
                    nconsj[j] += 1

        for j in range(NJ):
            FETel_j = self.equipment[j].FETel_j
            if FETel_j and abs(FETel_j) > EPSILON:
                conLines = connectIcons(self.electricityIcon,self.equipmentIcon[j],nv = nv, nhy = nconsj[j], colour=ORANGE, inputNo=2)
                self.lines += conLines
                self.connections.append(conLines)
                self.flow.append(EnergyFlow("EL", 0, "EQ", j))
                nv += 1
                nconsj[j] += 1
                
                    
        nv_up = 0
        nv_down = 0

        nconsj = [0]*NJ
        nconsm = [0]*NM

        heatingBlock = True
        for m in range(NM):
            nconsm[m] = 0
            for j in range(NJ):
                if (j, m) in systemModel.USHLink:
                    colour = wx.RED if self.equipment[j].heating else wx.BLUE
                        
##                        if heatingBlock:    # reset nv in order to use the full range of vertical channels again for the cooling block
##                            nv_up = 0
##                            nv_down = 0
##                        heatingBlock = False
                        

                    dy = self.pipeIcon[m].inPos(0)[1] - self.equipmentIcon[j].outPos(0)[1]
                    updown = -1*(dy > EPSILON) + 1*(dy < -EPSILON)
                    if updown > 0:
                        nv = 10 - nv_up
                        nv_up += 1

                    else:
                        nv = nv_down
                        nv_down += 1

                    for mh in range(m+1):
                        if self.pipeIcon[mh].pos[1]==self.equipmentIcon[j].pos[1]:
                            nconsMin = max(nconsj[j], nconsm[mh])
                            nconsj[j] = nconsMin
                            nconsm[mh] = nconsMin

                        
                    conLines = connectIcons(self.equipmentIcon[j],self.pipeIcon[m], nv = nv, nhx = nconsj[j], nhy = nconsm[m],colour = colour)
                    self.lines += conLines
                    self.connections.append(conLines)
                    self.flow.append(EnergyFlow("EQ", j, "PIPE", m))
                    nconsm[m] +=1
                    nconsj[j] +=1
                    
        nv = 0
        for m in range(NM):
            ncons = 0
            for k in range(NK):
                if (m, k) in systemModel.UPHLink:
                    colour = wx.RED if self.process[k].heating else wx.BLUE
                    
                    conLines = connectIcons(self.pipeIcon[m], self.processIcon[k], nv=nv, nhx=ncons, colour=colour)
                    self.lines += conLines
                    self.connections.append(conLines)
                    self.flow.append(EnergyFlow("PIPE", m, "PROC", k))
                    ncons += 1
                    nv += 1
                elif (m, k) in systemModel.UPCReqLink:
                    colour = wx.BLUE
                    conLines = connectIcons(self.pipeIcon[m],self.processIcon[k], nv=nv, nhx=ncons, nhy = 2, colour=colour)
                    self.lines += conLines
                    self.connections.append(conLines)
                    self.flow.append(EnergyFlow("PIPE", m, "PROC", k))
                    ncons +=1
                    nv += 1

            for j in range(NJ):
                if (m, j) in systemModel.FEThLink:
                    colour = wx.RED
                    conLines = connectIcons(self.pipeIcon[m],self.equipmentIcon[j], nv=nv, nhx=ncons, colour=colour)
                    self.lines += conLines
                    self.connections.append(conLines)
                    self.flow.append(EnergyFlow("PIPE", m, "EQ", j))
                    ncons+=1
                    nv+=1

        if Status.StatusCC == 0: # highlight conflicts and conflict sources if possible
            self.cc = CCScheme(self.fuelIcon, self.electricityIcon, self.equipmentIcon, self.pipeIcon, self.processIcon)
            self.cc.display()

    def addHXIcons(self, hxin, hxout, pos, colour, cooling=False):
        """
        adds HX icons if there is a HX connected to input or output of the object

        hxin, hxout: list of indexes of HX connected to the object
        pos: object position
        """
        posIn =  wx.Point(pos[0]-0,pos[1]+63)
        posOut = wx.Point(pos[0]+90,pos[1]+63)
        
        if len(hxin) > 0:
            h = hxin[0] if len(hxin)==1 else hxin
##            name = str(len(hxin))+" HX" if isinstance(h, list) else self.data["HX"][h].name
            
            self.hxIcon.append(ModuleIcon(parent=self, \
                                               pos=posOut if cooling else posIn, \
                                               iconSize=(24, 28), \
                                               name = "",\
                                               code = ("HX", h),\
                                               colour = colour, \
                                               inFlow = [""],
                                               outFlow = [""],
                                               bitmap = self.bitMaps["HXSnk"]))

        if len(hxout) > 0:
            h = hxout[0] if len(hxout)==1 else hxout
##            name = str(len(hxout))+" HX" if isinstance(h, list) else self.data["HX"][h].name
            
            self.hxIcon.append(ModuleIcon(parent=self, \
                                               pos=posIn if cooling else posOut, \
                                               iconSize=(24, 28), \
                                               name = "",\
                                               code = ("HX", h),\
                                               colour = colour, \
                                               inFlow = [""],
                                               outFlow = [""],
                                               bitmap = self.bitMaps["HXSrc"]))


    def editIcon(self, icon):
        """ this function is called from the event handler for mouse clicks
            should be substituted by a custom function

            function should return None if icon is not editable,
            otherwise any object (node) with attributes:
            - nodeType
            - scaleFactor
            - splitFraction
        """
        self.conHandler = 0
        if len(icon.code)>=2: # security control
            objectNo = icon.code[1]
            if icon.code[0]=="HX" and isinstance(icon.code[1], list):
                # create pop-up
                menu = wx.Menu()
                hxList = icon.code[1]
                items =  [Status.int.hx[h].name for h in hxList]
                self.popUpMenuIds = [wx.NewId() for item in items]
                for i, title in enumerate(items):
                    id = self.popUpMenuIds[i]
                    menu.Append( id, title )
                    wx.EVT_MENU(menu, id, self.OnPopUpSelectHX)

                # Display menu with call to PopupMenu, invoked on the source component, passing event's GetPoint. ###
                self.PopupMenu(menu, icon.pos)
                menu.Destroy() # destroy to avoid mem leak
                wx.SafeYield()
                          
                if self.actionCode >=0 and self.actionCode < len(items):
                    item = items[self.actionCode]
                    objectNo = hxList[self.actionCode]
                else:
                    return None
                
            self.main.control(icon.code[0], objectNo)
        
        return None

    def insertIcon(self, pos):
        """ this function is called from the event handler for mouse clicks
            should be substituted by a custom function
        """
        self.conHandler = 0
        if not self.editAllowed(): return
        
        # create pop-up
        menu = wx.Menu()
        for i, title in enumerate(POPUPMENU):
            id = POPUPMENUIDS[i]
            menu.Append( id, title )
            wx.EVT_MENU(menu, id, self.OnPopUpIcons)

        # Display menu with call to PopupMenu, invoked on the source component, passing event's GetPoint. ###
        self.PopupMenu(menu, pos)
        menu.Destroy() # destroy to avoid mem leak
        wx.SafeYield()

        if self.actionCode==0:
            self.main.control("FUEL", -1)
        elif self.actionCode==1:
            self.main.control("PROC", -1)
        elif self.actionCode==2:
            self.main.control("PIPE", -1)
        elif self.actionCode==3:
            self.main.control("EQ", -1)
        elif self.actionCode==4:
            logger.logMessage(_("Drag and drop a connection from source to sink object to add a heat exchanger"))
            self.conHandler = 1
        elif self.actionCode==5:
            self.main.control("WHEE", -1)

        self.display()

        return None

    def deleteIcon(self, icon):
        """ this function is called from the event handler for mouse clicks
            should be substituted by a custom function
        """
        if not self.editAllowed(): return

        objectNo = icon.code[1]
        name = icon.name
        if icon.code[0] == "EL":
            logger.showMessage(_("The electricity icon cannot be deleted"))
            return False

        elif icon.code[0] == "PROC":
            proc = Status.int.systemModel.processObject[objectNo]
            if proc.buildingId is not None:
                logger.showWarning(_("Cannot delete building heating or cooling demands directly\n") + \
                                   _("Edit the corresponding building data instead"))
                return
            
        elif icon.code[0]=="HX":
            if isinstance(icon.code[1], list):
                # create pop-up
                menu = wx.Menu()
                hxList = icon.code[1]
                items =  [Status.int.hx[h].name for h in hxList]
                self.popUpMenuIds = [wx.NewId() for item in items]
                for i, title in enumerate(items):
                    id = self.popUpMenuIds[i]
                    menu.Append( id, title )
                    wx.EVT_MENU(menu, id, self.OnPopUpSelectHX)

                # Display menu with call to PopupMenu, invoked on the source component, passing event's GetPoint. ###
                self.PopupMenu(menu, icon.pos)
                menu.Destroy() # destroy to avoid mem leak
                wx.SafeYield()
                          
                if self.actionCode >=0 and self.actionCode < len(items):
                    item = items[self.actionCode]
                    objectNo = hxList[self.actionCode]
                    name = item
                else:
                    return
            else:
                h = icon.code[1]
                name = Status.int.hx[h].name

        ret = logger.askConfirmation(_("Do You want to delete %s ?"%(name)))
        if not ret: return
            
        if len(icon.code)>=2: # security control
            Status.prj.deleteObject(icon.code[0], objectNo)   # objectNo=icon.code[1] except for HX where this code may be a list
            Globals.GUI.updateDrawings()

        self.display()
##        self.prnt.Hide()
##        self.prnt.Show()

        self.conHandler = 0
        return False    # false indicates that no default action is taken by the drawing panel

    def editAllowed(self):
        if Status.ANo <= 0 and Status.StatusCC > 0:
            logger.showWarning(_("You cannot modify the present state of the system after the data has been checked.\n") + 
                        _("If you want to change the present state you may unblock the data by ") + 
                        _("choosing to confirm the deletion ") + 
                        _("of any alternative proposals in this project the next time you switch to the present state."))
            return False
        return True
        
    def OnPopUpIcons(self, event):
        actionCode = POPUPMENUIDS.index(event.GetId())
        self.actionCode = actionCode

    def OnPopUpSelectHX(self, event):
        actionCode = self.popUpMenuIds.index(event.GetId())
        self.actionCode = actionCode

    def buildConnection(self, startIcon, startNo, endIcon, endNo):

        if not self.editAllowed():
            return False

        if self.conHandler==0:
            flow = EnergyFlow(startIcon.code[0], startIcon.code[1], endIcon.code[0], endIcon.code[1])
            
            if flow.saveToDB():
                self.display()
                self.prnt.Hide()
                self.prnt.Show()

        elif self.conHandler==1:

            Globals.GUI.control("HX", None)
            self.modQ6 = Globals.GUI.editPanel.mod

            self.modQ6.addConnection(heating=False)
            self.modQ6.setObjectType(OCODE[startIcon.code[0]], no=startIcon.code[1])
            self.modQ6.activeConn.objectNo = startIcon.code[1]
            self.modQ6.activeConn.componentNo = 0
            editor = EditConnection(Globals.GUI.editPanel)
            editor.OnCBStreamChange(None)
            if editor.ShowModal() == wx.ID_OK:
                Globals.GUI.editPanel.showConnections()
                
            self.modQ6.addConnection(heating=True)
            self.modQ6.setObjectType(OCODE[endIcon.code[0]], no=endIcon.code[1])
            self.modQ6.activeConn.objectNo = endIcon.code[1]
            self.modQ6.activeConn.componentNo = 0
            editor = EditConnection(Globals.GUI.editPanel)
            editor.OnCBStreamChange(None)
            if editor.ShowModal() == wx.ID_OK:
                Globals.GUI.editPanel.showConnections()

        self.conHandler = 0
            
        return False
        
    def deleteConnection(self, n):
        if not self.editAllowed():
            return False

        if self.flow[n].deleteFromDB():
            self.display()
            self.prnt.Hide()
            self.prnt.Show()        
            
        self.conHandler = 0
        return False

        
def strRep(val, prefix="", minVal=-INFINITE):
    return " " if (val is None or abs(val) <= minVal) else prefix+Globals.floatIO.write(val/1000, decimals=0)

       
