# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	PanelCC
#			
#------------------------------------------------------------------------------
#			
#	Short description:
#	
#	Panel for handling of consistency checking
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
import wx
from GUI.status import Status
from modules.interfaces import Interfaces
from globals import logger
from GUI.conflictFrame import conflictFrame
from GUI.GUITools import *
from modules.messageLogger import *
from auxiliary.auxiliary import toGettextAndUnicode as _U
from auxiliary.auxiliary import toUnicode as U

[wxID_PANELCC, wxID_PANELCCBASICCHECK, wxID_PANELCCBUTTONPANELBACK,
 wxID_PANELCCBUTTONPANELCANCEL, wxID_PANELCCBUTTONPANELFWD,
 wxID_PANELCCBUTTONPANELOK, wxID_PANELCCCBSETACCURACY, wxID_PANELCCCHECKLIST,
 wxID_PANELCCESTIMATEDATA, wxID_PANELCCGRID, wxID_PANELCCST1PANEL,
 wxID_PANELCCST3PANEL, wxID_PANELCCSTSETACCURACY, wxID_PANELCCSTSTATISTICS1,
 wxID_PANELCCSTSTATISTICS1VAL, wxID_PANELCCSTSTATISTICS2,
 wxID_PANELCCSTSTATISTICS2VAL, wxID_PANELCCSTSTATISTICS3,
 wxID_PANELCCSTSTATISTICS3VAL, wxID_PANELCCSTTITLEPANEL,
] = [wx.NewId() for _init_ctrls in range(20)]

# constants
MAXROWS = 50
COLNO = 5

### dummy text for new GUI in V2.0
dummyTextNewGUI = [_("Parameter Code"),
                   _("include economic data"),
                   _("minimum"),
                   _("maximum"),
                   _("calculated from user input data"),
                   _("equations"),
                   _("edit parameter"),
                   _("show accuracy"),
                   _("Name"),
                   _("Short Name"),
                   _("Estimated Parameter"),
                   _("List of estimated data"),
                   _("No. of estimated data")
                   ]


# match code of screen blocks in CC with the corresponing GUI.control parameter
GUICONTROL = {"Eq.":    "EQ",
              "Proc.":  "PROC",
              "Pipe":   "PIPE",
              "HX":     "HX",
              "WHEE":   "WHEE",
              "FET":    "FUEL",
              "FETel":  "EL",
              "ECO":    "ECO",
              "Totals": "EDIT"}


class PanelCC(wx.Panel):

##    def __init__(self, parent, main, id, pos, size, name):
    def __init__(self, parent, main):
        self.prnt = parent
        self._init_ctrls(parent)
        keys = ['CC Table', 'CC Info']
        self.keys = keys
        self.main = main

        self.mod = Status.mod.moduleCC
        self.checkOK = False

        self.ANo = Status.ANo
        labels_column = 1

        #
        # additional widgets setup
        # here, we modify some widgets attributes that cannot be changed
        # directly by Boa. This cannot be done in _init_ctrls, since that
        # method is rewritten by Boa each time.
        #
        # data cell attributes
        attr = wx.grid.GridCellAttr()
        attr.SetTextColour(GRID_LETTER_COLOR)
        attr.SetBackgroundColour(GRID_BACKGROUND_COLOR)
        attr.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.NORMAL))

        self.grid.CreateGrid(MAXROWS, COLNO)

        self.grid.EnableGridLines(True)
        self.grid.SetDefaultRowSize(20)
        self.grid.SetRowLabelSize(30)
        self.grid.SetDefaultColSize(80)
        self.grid.SetColSize(0, 100)
        self.grid.SetColSize(1, 264)
        self.grid.SetColSize(2, 120)
        self.grid.SetColSize(4, 180)
        
        self.grid.EnableEditing(False)
        self.grid.SetSelectionMode(wx.grid.Grid.wxGridSelectRows)
        self.grid.SetLabelFont(wx.Font(9, wx.ROMAN, wx.ITALIC, wx.BOLD))
        self.grid.SetColLabelValue(0, _U("Name"))
        self.grid.SetColLabelValue(1, _U("Description"))
        self.grid.SetColLabelValue(2, _U("Value"))
        self.grid.SetColLabelValue(3, _U("max.Error"))
        self.grid.SetColLabelValue(4, _U("Action to be taken"))
        #
        # copy values from dictionary to grid
        #
        for r in range(MAXROWS):
##            self.grid.SetRowAttr(r, attr)
            for c in range(COLNO):
                if c <= labels_column:
                    self.grid.SetCellAlignment(r, c, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);
                else:
                    self.grid.SetCellAlignment(r, c, wx.ALIGN_RIGHT, wx.ALIGN_CENTRE);

        self.grid.SetGridCursor(0, 0)
    
    def _init_ctrls(self, prnt):
        # generated method, don't edit
        wx.Panel.__init__(self, id=wxID_PANELCC, name='PanelCC', parent=prnt,
              pos=wx.Point(0, 0), size=wx.Size(800, 600), style=0)
        self.SetClientSize(wx.Size(800, 600))

#..............................................................................
# box 1: grid display

        self.box1 = wx.StaticBox(self, -1, _U('Cross checking of data'),
                                 pos=(10, 10), size=(780, 400))

        self.box1.SetForegroundColour(TITLE_COLOR)
        self.box1.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.st1panel = wx.StaticText(id= -1, label=_U('list of data with unsufficient accuracy'),
              name='st1panel', parent=self, pos=wx.Point(20, 30), style=0)

        self.cbSetAccuracy = wx.ComboBox(choices=[_U("quick & dirty"), _U("standard"), _U("detailed")],
              id=wxID_PANELCCCBSETACCURACY, name='cbSetAccuracy', parent=self,
              pos=wx.Point(616, 26), size=wx.Size(162, 32), style=0,
              value=_U("quick and dirty"))
        self.cbSetAccuracy.SetLabel(_U("quick and dirty"))
        self.cbSetAccuracy.Bind(wx.EVT_COMBOBOX, self.OnCbSetAccuracyCombobox,
              id=wxID_PANELCCCBSETACCURACY)

        self.stSetAccuracy = wx.StaticText(id=wxID_PANELCCSTSETACCURACY,
              label=_U('required accuracy'), name='stSetAccuracy', parent=self,
              pos=wx.Point(512, 30), size=wx.Size(87, 13), style=0)

        self.grid = wx.grid.Grid(id=wxID_PANELCCGRID, name='gridpanel',
              parent=self, pos=wx.Point(20, 60), size=wx.Size(760, 340),
              style=wx.WANTS_CHARS | wx.VSCROLL | wx.HSCROLL)
##              style=0)

        self.grid.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK,
              self.OnGridPanelGridCellLeftDclick, id=wxID_PANELCCGRID)
        self.grid.Bind(wx.grid.EVT_GRID_CELL_RIGHT_CLICK,
              self.OnGridPanelGridCellRightClick, id=wxID_PANELCCGRID)
        self.grid.Bind(wx.grid.EVT_GRID_CELL_LEFT_CLICK,
              self.OnGridPanelGridCellLeftClick)


#..............................................................................
# action buttons

        self.BasicCheck = wx.Button(id=wxID_PANELCCBASICCHECK,
              label=_U('basic check'), name='BasicCheck', parent=self,
              pos=wx.Point(20, 420), size=wx.Size(200, 24), style=0)
        self.BasicCheck.Bind(wx.EVT_BUTTON, self.OnBasicCheckButton,
              id=wxID_PANELCCBASICCHECK)

        self.estimateData = wx.Button(id=wxID_PANELCCESTIMATEDATA,
              label=_U('estimate data'), name='estimateData', parent=self,
              pos=wx.Point(240, 420), size=wx.Size(200, 24), style=0)
        self.estimateData.Bind(wx.EVT_BUTTON, self.OnEstimateDataButton,
              id=wxID_PANELCCESTIMATEDATA)

        self.checkList = wx.Button(id=wxID_PANELCCCHECKLIST, label=_U('check list'),
              name='checkList', parent=self, pos=wx.Point(460, 420),
              size=wx.Size(200, 24), style=0)
        self.checkList.Bind(wx.EVT_BUTTON, self.OnCheckListButton,
              id=wxID_PANELCCCHECKLIST)
        self.checkList.Disable()
        self.checkList.SetToolTipString(_U("The automated check list is not yet implemented, sorry."))
#..............................................................................
# box 2: display of statistics
        self.box2 = wx.StaticBox(self, -1, _U('Cross check statistics'),
                                 pos=(10, 460), size=(780, 80))

        self.box2.SetForegroundColour(TITLE_COLOR)
        self.box2.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))
        
        self.stStatistics1 = wx.StaticText(id=wxID_PANELCCSTSTATISTICS1,
              label=_U('No. of data checked'), name='stStatistics1', parent=self,
              pos=wx.Point(24, 480), size=wx.Size(98, 13), style=0)

        self.stStatistics2 = wx.StaticText(id=wxID_PANELCCSTSTATISTICS2,
              label=_U('No. of input data fixed'), name='stStatistics2',
              parent=self, pos=wx.Point(24, 500), size=wx.Size(110, 13),
              style=0)

        self.stStatistics3 = wx.StaticText(id=wxID_PANELCCSTSTATISTICS3,
              label=_U('No. of missing data'), name='stStatistics3', parent=self,
              pos=wx.Point(24, 520), size=wx.Size(93, 13), style=0)

        self.stStatistics1Val = wx.StaticText(id=wxID_PANELCCSTSTATISTICS1VAL,
              label='---', name='stStatistics1Val', parent=self,
              pos=wx.Point(176, 480), size=wx.Size(12, 13), style=0)

        self.stStatistics2Val = wx.StaticText(id=wxID_PANELCCSTSTATISTICS2VAL,
              label='---', name='stStatistics2Val', parent=self,
              pos=wx.Point(176, 500), size=wx.Size(12, 13), style=0)

        self.stStatistics3Val = wx.StaticText(id=wxID_PANELCCSTSTATISTICS3VAL,
              label='---', name='stStatistics3Val', parent=self,
              pos=wx.Point(176, 520), size=wx.Size(12, 13), style=0)
#..............................................................................
# default action buttons

        self.buttonpanelOk = wx.Button(id=wx.ID_OK, label='OK',
              name='buttonpanelOk', parent=self, pos=wx.Point(528, 544),
              size=wx.Size(75, 23), style=0)
        self.buttonpanelOk.Bind(wx.EVT_BUTTON, self.OnButtonOk,
              id=wx.ID_OK)

        self.buttonpanelCancel = wx.Button(id=wx.ID_CANCEL, label='Cancel',
              name='buttonpanelCancel', parent=self, pos=wx.Point(616, 544),
              size=wx.Size(75, 23), style=0)
        self.buttonpanelCancel.Bind(wx.EVT_BUTTON,
              self.OnButtonCancel, id=wx.ID_CANCEL)


#------------------------------------------------------------------------------		
    def display(self, no=None):
#------------------------------------------------------------------------------		
#   function activated on each entry into the panel from the tree
#------------------------------------------------------------------------------		
        self.mod.initPanel()        # prepares data for plotting

#..............................................................................
# update of alternatives table

        attr = wx.grid.GridCellAttr()
        attr.SetTextColour(GRID_LETTER_COLOR)
        attr.SetBackgroundColour(GRID_BACKGROUND_COLOR)
        attr.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.NORMAL))

        attr2 = wx.grid.GridCellAttr()
        attr2.SetTextColour(GRID_LETTER_COLOR_HIGHLIGHT)
        attr2.SetBackgroundColour(GRID_BACKGROUND_COLOR_HIGHLIGHT)
        attr2.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.BOLD))

        try:
            data = Interfaces.GData[self.keys[0]]
            rows = len(data)
            rows = min(rows,MAXROWS)
            for r in range(rows):

                highlight = data[r][COLNO]
                if highlight:
                    attr2.IncRef() # required for avoiding segmentation error; see http://www.python-it.org/forum/index.php?topic=6324.0
                    self.grid.SetRowAttr(r, attr2)
                else:
                    attr.IncRef()
                    self.grid.SetRowAttr(r, attr)
                                    
                for c in range(COLNO):
                    self.grid.SetCellValue(r, c, U(str(data[r][c])))
        except:
            logTrack("EXCEPTION OCCURED IN PANELCC")
            rows = 0

        for r in range(rows, MAXROWS):
                attr.IncRef()
                self.grid.SetRowAttr(r, attr)
                for c in range(COLNO):
                    self.grid.SetCellValue(r, c, "")

        try:
            info = Interfaces.GData[self.keys[1]]

            self.stStatistics1Val.SetLabel(U(str(info[0])))
            self.stStatistics3Val.SetLabel(U(str(info[2])))

            self.cbSetAccuracy.SetSelection(int(info[3] - 1))     
        except:
            pass
        
        self.Show()
        self.main.panelInfo.update()
       

    def emptyGrid(self, text=_U("Consistency Check")):
        for r in range(MAXROWS):
            for c in range(COLNO):
                self.grid.SetCellValue(r, c, "")
        self.grid.SetCellValue(0,1,text)
    
#------------------------------------------------------------------------------		

#==============================================================================
#   EVENT HANDLERS
#==============================================================================

#------------------------------------------------------------------------------		
    def OnBasicCheckButton(self, event):
#------------------------------------------------------------------------------		
#   Generate new alterantive proposal
#------------------------------------------------------------------------------		

        if Status.NoOfAlternatives > 0:
            title =  _U("confirm basic check")
            ret = logger.askConfirmation(_U("\nTake care: running the check will invalidate existing alternative proposals") + \
                              _U("\nDo You want to continue ?"))
            if not ret:
                return

        elif Status.StatusCC > 0:
            title = _U("confirm basic check")
            ret = logger.askConfirmation(_U("\nYour present state data are already checked") + \
                              _U("\nDo You want to continue ?"))
            if not ret:
                return

        self.emptyGrid(text = _U("basic check"))
        Status.prj.copyQuestionnaire()


        nc = self.mod.basicCheck()
        self.display()

        self.main.panelInfo.update()
        self.main.showMainMenuAlternatives()

        if nc > 0:
            self.checkOK = False
            self.main.updateDrawings()
            self.main.cf = conflictFrame(self.main)
            self.main.cf.Show()
        else:
            self.checkOK = True
            
        if event: event.Skip()
        
#------------------------------------------------------------------------------		
    def OnEstimateDataButton(self, event):
#------------------------------------------------------------------------------		
#   Generate new porposal copying from existing alternative
#------------------------------------------------------------------------------		
        if self.checkOK == False:
            showMessage(_U("First You have to run the basic check and eliminate possible conflicts in redundant data"))
            return
            
        title = _U("confirm data estimation")
        ret = logger.askConfirmation(_U("\nTake care: running data estimate will automatically estimate missing data.") + \
                          _U("\nAlternatively you can also manually add estimated data to the questionnaire and rerun basic check") + \
                          _U("\nDo You want to continue ?"))
        if ret:
            self.emptyGrid(text=_U("estimate data"))
            nc = self.mod.basicCheck(estimate=True)
            self.display()

            if nc > 0:
                self.main.updateDrawings()
                self.main.cf = conflictFrame(self.main)
                self.main.cf.Show()

        if event: event.Skip()
#------------------------------------------------------------------------------		
    def OnCheckListButton(self, event):
#------------------------------------------------------------------------------		
#   Delete alternative proposal
#------------------------------------------------------------------------------		

        pu2 = DialogOK(self, _U("confirm check list"), _U("CHECK LIST NOT YET IMPLEMENTED. DO IT YOURSELF AND BE HAPPY"))
        if pu2.ShowModal() == wx.ID_OK:
            pass
#------------------------------------------------------------------------------		

    def OnGridPanelGridCellLeftClick(self, event):
        """ selects Row and leads to the corresponding GUI entry panel
        """
	self.selectedRow = event.GetRow()
        data = Interfaces.GData[self.keys[0]]

        try:
            parInfo = data[self.selectedRow][0:7]
            objectType = parInfo[6]
            parNameInfo = parInfo[0].split('[')
##            print "parnameInfo ", parNameInfo
            parName = parNameInfo[0]
            try:
                parNo = int(parNameInfo[1].strip(']')) - 1
            except:
                parNo = None
        except:
            objectType = None
            parName = None
            parNo = None

        # open the panel corresponding to the parNo and select notebook tab for editing parName            
        if objectType in GUICONTROL.keys():
            ctrl = GUICONTROL[objectType]
            if ctrl=="ECO":
                if parName.startswith("OM"):
                    ctrl = "Q1"
                else:
                    ctrl = "ECO0"
            
            self.main.control(ctrl, parNo, goToKey=parName)

#------------------------------------------------------------------------------		
    def OnGridPanelGridCellLeftDclick(self, event):
#------------------------------------------------------------------------------		
        self.OnGridPanelGridCellLeftClick(event)
        event.Skip()

    def OnGridPanelGridCellRightClick(self, event):
        event.Skip()        

#------------------------------------------------------------------------------		
    def OnCbSetAccuracyCombobox(self, event):
#------------------------------------------------------------------------------		

        accuracy = self.cbSetAccuracy.GetSelection()
        self.mod.setPriorityLevel(accuracy + 1)
##        print "PanelCC - accuracy combobox calling display"
        self.display()

#------------------------------------------------------------------------------		
#   <<< OK Cancel >>>
#------------------------------------------------------------------------------		
    def OnButtonOk(self, event):

        if self.checkOK == True:
            title = _U("accept data")
            ret = logger.askConfirmation(_U("Press OK to confirm that the data are correct\n") + \
                              _U("(This will block the questionnaire for further modifications)"))
            if ret:
                self.mod.checkRobustness()
                Status.prj.setActiveAlternative(0, checked=True)
                self.main.panelInfo.update()
                self.main.showMainMenuAlternatives()

                Status.mod.moduleEA.update()
                self.Hide()
                self.main.control("EA4a", None)

                Status.mod.moduleECO.runPresentState()
        else:
            title = _U("revise data")
            ret = logger.askConfirmation(_U("EINSTEIN cannot accept these data. You first have to eliminate inconsistencies"))

    def OnButtonCancel(self, event):
        #warning: do you want to leave w/o saving ???
        try:
            self.prnt.Close()
        except:
            pass
##        print "Button exitModuleCancel: now I should go back to HC"



