# -*- coding: utf-8 -*- 
#==============================================================================#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	PanelEA3- GUI component for: Final energy by equipment - Yearly data
#			
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
import wx.grid
import numpy as np
from GUI.graphics import drawPiePlot
from status import *
from modules.energyStats.moduleEA3 import ModuleEA3
from displayClasses import ChoiceEntry
import modules.matPanel as Mp
from auxiliary.auxiliary import toGettextAndUnicode as _U
from auxiliary.auxiliary import toUnicode as U
from modules.interfaces import Interfaces
from GUI.GUITools import (GRID_LETTER_COLOR_HIGHLIGHT,
                                   GRID_BACKGROUND_COLOR_HIGHLIGHT)
from GUI.numCtrl import convertDoubleToString
from constants import HEATING_COOLING
from modules.messageLogger import logError

[wxID_PANELEA3, wxID_PANELEA3GRID1, wxID_PANELEA3GRID2, 
 wxID_PANELEA3PANELGRAPHFET, wxID_PANELEA3STATICTEXT1, 
 wxID_PANELEA3STATICTEXT2, wxID_PANELEA3STATICTEXT3,
 wxID_PANELEA3STATICTEXT4, wxID_PANELEA3PANELGRAPHUSH,
] = [wx.NewId() for _init_ctrls in range(9)]
#
# constants
#

ORANGE = '#FF6000'
LIGHTGREY = '#F8F8F8'
WHITE = '#FFFFFF'
DARKGREY = '#000060'
LIGHTGREEN = '#F0FFFF'

GRID_LETTER_SIZE = 8 #points
GRID_LABEL_SIZE = 9  # points
MINROWS = 10 # minmal count of rows
GRID_LETTER_COLOR = DARKGREY     # specified as hex #RRGGBB
GRID_BACKGROUND_COLOR = LIGHTGREY # idem
GRAPH_BACKGROUND_COLOR = WHITE # idem
TITLE_COLOR = ORANGE

MAXCOLS = 10


class PanelEA3(wx.Panel):
    def __init__(self, parent, main):
        self._init_ctrls(parent)
        self._keys = ['EA3_FET', 'EA3_USH'] 
        self._showHeating = True
        self.mod = ModuleEA3(self._keys, self._showHeating)
        self._labels_column = 0
        # remap drawing methods to the wx widgets.
        #
        # upper graph: FET by equipment
        (rows,cols) = np.shape(Interfaces.GData[self._keys[0]])
        ignoredrows = [rows-1]

        paramList={'labels'      : self._labels_column,          # labels column
                   'data'        : 3,                      # data column for this graph
                   'key'         : self._keys[0],                # key for Interface
                   'title'       : _U('FET by equipment'),     # title of the graph
                   'backcolor'   : GRAPH_BACKGROUND_COLOR, # graph background color
                   'ignoredrows' : ignoredrows}            # rows that should not be plotted

        dummy = Mp.MatPanel(self.panelGraphFET,
                            wx.Panel,
                            drawPiePlot,
                            paramList)
        # additional widgets setup
        # data cell attributes
        attr = wx.grid.GridCellAttr()
        attr.SetTextColour(GRID_LETTER_COLOR)
        attr.SetBackgroundColour(GRID_BACKGROUND_COLOR)
        attr.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.BOLD))
        # set upper grid
        data = Interfaces.GData[self._keys[0]]

#####Security feature against non existing GData entry
        COLNO1 = 4 #grid has usually a fixed column size, not necessary to read from GData        
        try: (rows,cols) = np.shape(data)
        except: (rows,cols) = (0,COLNO1)

        # data cell attributes
        attr = wx.grid.GridCellAttr()
        attr.SetTextColour(GRID_LETTER_COLOR)
        attr.SetBackgroundColour(GRID_BACKGROUND_COLOR)
        attr.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.NORMAL))


        attr2 = wx.grid.GridCellAttr()
        attr2.SetTextColour(GRID_LETTER_COLOR_HIGHLIGHT)
        attr2.SetBackgroundColour(GRID_BACKGROUND_COLOR_HIGHLIGHT)
        attr2.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.BOLD))
        #
        # set upper grid
        #
        self.grid1.CreateGrid(max(rows, MINROWS), COLNO1)

        self.grid1.EnableGridLines(True)
        self.grid1.SetDefaultRowSize(20)
        self.grid1.SetRowLabelSize(30)

        self.grid1.SetColSize(0,125)
        self.grid1.SetColSize(1,110)
        self.grid1.SetColSize(2,85)
        self.grid1.SetColSize(3,70)

        self.grid1.EnableEditing(False)
        self.grid1.SetLabelFont(wx.Font(9, wx.ROMAN, wx.ITALIC, wx.BOLD))
        self.grid1.SetColLabelValue(0, _U("Equipment"))
        self.grid1.SetColLabelValue(1, _U("Fuel type"))
        self.grid1.SetColLabelValue(2, _U("MWh"))
        self.grid1.SetColLabelValue(3, _U("%"))
        #
        # copy values from dictionary to grid
        #

#SD2008-06-30
        decimals = [-1,-1,0,1]   #number of decimal digits for each colum
        for r in range(rows):
            if r < rows-1:
                self.grid1.SetRowAttr(r, attr)
            else:
                self.grid1.SetRowAttr(r,attr2)  #highlight totals row
            for c in range(cols):
                try:
                    if decimals[c] >= 0: # -1 indicates text
                        self.grid1.SetCellValue(r, c, \
                            convertDoubleToString(float(data[r][c]),nDecimals = decimals[c]))
                    else:
                        self.grid1.SetCellValue(r, c, data[r][c])
                except: pass
                if c == self._labels_column:
                    self.grid1.SetCellAlignment(r, c, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);
                elif c == 1:
                    self.grid1.SetCellAlignment(r, c, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);                
                else:
                    self.grid1.SetCellAlignment(r, c, wx.ALIGN_RIGHT, wx.ALIGN_CENTRE);

        self.grid1.SetGridCursor(0, 0)
        

    def _init_ctrls(self, prnt):

        # generated method, don't edit
        wx.Panel.__init__(self, id=wxID_PANELEA3, name=u'PanelEA3', parent=prnt,
              pos=wx.Point(0, 0), size=wx.Size(800, 600))
#..............................................................................
#   box 1

        self.box1 = wx.StaticBox(self, -1, _U('Final energy consumption for thermal use (FET) by equipment'),
                                 pos = (10,10),size=(780,260))
        self.box1.SetForegroundColour(TITLE_COLOR)
        self.box1.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))


        self.grid1 = wx.grid.Grid(id=wxID_PANELEA3GRID1, name='grid1',#SD
              parent=self, pos=wx.Point(20, 40), size=wx.Size(440, 220),
              style=0)

        self.panelGraphFET = wx.Panel(id=wxID_PANELEA3PANELGRAPHFET,
              name=u'panelGraphFET', parent=self, pos=wx.Point(480, 40),
              size=wx.Size(300, 220), style=wx.TAB_TRAVERSAL|wx.SUNKEN_BORDER)
        self.panelGraphFET.SetBackgroundColour(wx.Colour(127, 127, 127))

#..............................................................................
#   box 2
        
        self.box2 = wx.StaticBox(self, -1, unicode(""),
                                 pos = (10,285),size=(780,265))
        self.box2.SetForegroundColour(TITLE_COLOR)
        self.box2.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))
        sizer1 = wx.BoxSizer(wx.VERTICAL)
        self.sizer2 = wx.BoxSizer(wx.HORIZONTAL)
        self.hcModeChooser = ChoiceEntry(self, values=[U(item) for item in HEATING_COOLING.values()],
                                         value=_U('heating'), label=_U('heating/cooling'),
                                         tip=_U('Show only heating or cooling processes.'))
        self.hcModeChooser.Bind(wx.EVT_CHOICE, self.OnModeChooser)
        sizer1.Add(self.hcModeChooser, flag=wx.ALIGN_RIGHT | wx.RIGHT, border=2)
        sizer1.Add(self.sizer2)
        sizer1.SetDimension(x=20, y=305, width=740, height=230)
        
#..............................................................................
#   default buttons
#..............................................................................
        self.btnBack = wx.Button(id=wx.ID_BACKWARD, label=u'<<<',
              name=u'btnBack', parent=self, pos=wx.Point(500, 560),
              size=wx.Size(80, 20), style=0)
        self.btnBack.Bind(wx.EVT_BUTTON, self.OnBtnBackButton,
              id=-1)

        self.btnOK = wx.Button(id=wx.ID_OK, label=_U('OK'), name=u'btnOK',
              parent=self, pos=wx.Point(600, 560), size=wx.Size(80, 20),
              style=0)
        self.btnOK.Bind(wx.EVT_BUTTON, self.OnBtnOKButton,
              id=-1)

        self.btnForward = wx.Button(id=wx.ID_FORWARD, label=u'>>>',
              name=u'btnForward', parent=self, pos=wx.Point(700, 560),
              size=wx.Size(80, 20), style=0)
        self.btnForward.Bind(wx.EVT_BUTTON, self.OnBtnForwardButton,
              id=-1)


#------------------------------------------------------------------------------		
#   Event handlers for default buttons
#------------------------------------------------------------------------------		
    def OnBtnOKButton(self, event):
        event.Skip()

    def OnBtnBackButton(self, event):
        self.Hide()
        Status.main.tree.SelectItem(Status.main.qEA2, select=True)

    def OnBtnForwardButton(self, event):
        self.Hide()
        Status.main.tree.SelectItem(Status.main.qEA4a, select=True)

    def OnModeChooser(self, event):
        if event.GetString() == _U('heating'):
            self._showHeating = True
        else:
            self._showHeating = False
        self.mod.updatePanel(self._showHeating)
        self.display()

#------------------------------------------------------------------------------	
    def display(self, data=None):
        """Update the representation in the panel."""
        (rows,cols) = np.shape(Interfaces.GData[self._keys[1]])   
        # set lower grid
        data = Interfaces.GData[self._keys[1]]
        #Security feature against non existing GData entry
        COLNO1 = 4 #grid has usually a fixed column size, not necessary to read from GData        
        try: (rows,cols) = np.shape(data)
        except: (rows,cols) = (0,COLNO1)
        # Basic setup for bottom grid, is done dynamically. 
        self.sizer2.Clear(deleteWindows=True)
        self.grid2 = wx.grid.Grid(id=wxID_PANELEA3GRID2, name='grid2',
              parent=self, size=wx.Size(440, 200), style=0)
        self.panelGraphUSH = wx.Panel(id=wxID_PANELEA3PANELGRAPHUSH,
              name=u'panelGraphUSH', parent=self, size=wx.Size(280, 210),
              style=wx.TAB_TRAVERSAL|wx.SUNKEN_BORDER)
        self.panelGraphUSH.SetBackgroundColour(wx.Colour(127, 127, 127))
        self.sizer2.Add(self.grid2)
        self.sizer2.AddSpacer(20)
        self.sizer2.Add(self.panelGraphUSH)
        self.sizer2.Layout()
        
        self.grid2.CreateGrid(max(rows, MINROWS), COLNO1)
        self.grid2.EnableGridLines(True)
        self.grid2.SetDefaultRowSize(20)
        self.grid2.SetRowLabelSize(30)
        self.grid2.SetColSize(0,125)
        self.grid2.SetColSize(1,85)
        self.grid2.SetColSize(2,70)
        self.grid2.SetColSize(3,110)
        self.grid2.EnableEditing(False)
        self.grid2.SetLabelFont(wx.Font(9, wx.ROMAN, wx.ITALIC, wx.BOLD))
        self.grid2.SetColLabelValue(0, _U("Equipment"))
        self.grid2.SetColLabelValue(1, _U("MWh"))
        self.grid2.SetColLabelValue(2, _U("%"))
        self.grid2.SetColLabelValue(3, _U(" "))
        # data cell attributes
        attr = wx.grid.GridCellAttr()
        attr.SetTextColour(GRID_LETTER_COLOR)
        attr.SetBackgroundColour(GRID_BACKGROUND_COLOR)
        attr.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.NORMAL))
        attr2 = wx.grid.GridCellAttr()
        attr2.SetTextColour(GRID_LETTER_COLOR_HIGHLIGHT)
        attr2.SetBackgroundColour(GRID_BACKGROUND_COLOR_HIGHLIGHT)
        attr2.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.BOLD))
        # copy values from dictionary to grid
        decimals = [-1,0,1,-1]   #number of decimal digits for each column
        for r in range(rows):
            if r < rows-1:
                self.grid2.SetRowAttr(r, attr)
            else:
                self.grid2.SetRowAttr(r,attr2)  #highlight totals row
            for c in range(cols):
                try:
                    if decimals[c] >= 0: # -1 indicates text
                        self.grid2.SetCellValue(r, c, \
                            convertDoubleToString(float(data[r][c]),nDecimals = decimals[c]))
                    else:
                        self.grid2.SetCellValue(r, c, data[r][c])
                except: pass
                if c == self._labels_column:
                    self.grid2.SetCellAlignment(r, c, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);
                else:
                    self.grid2.SetCellAlignment(r, c, wx.ALIGN_RIGHT, wx.ALIGN_CENTRE);
        self.grid2.SetGridCursor(0, 0)
        
        ignoredrows = [rows-1]
        if self._showHeating:
            self.box2.SetLabel(_U('Useful supply heat (USH) by equipment'))
            plottitle = _U('USH by equipment')
        else:
            self.box2.SetLabel(_U('Useful cooling supply (USC) by equipment'))
            plottitle = _U('USC by equipment')
        paramList={'labels'      : self._labels_column,     # labels column
                   'data'        : 2,                       # data column for this graph
                   'key'         : self._keys[1],           # key for Interface
                   'title'       : plottitle,               # title of the graph
                   'backcolor'   : GRAPH_BACKGROUND_COLOR,  # graph background color
                   'ignoredrows' : ignoredrows}             # rows that should not be plotted
        Mp.MatPanel(self.panelGraphUSH, wx.Panel, drawPiePlot, paramList)
        try: self.panelGraphFET.draw()
        except Exception, e:
            logError("panelEA3.display() FET.draw: " + str(e))
            pass
        try: self.panelGraphUSH.draw()
        except Exception, e:
            logError("panelEA3.display() USH.draw: " + str(e))
            pass
        
        self.Refresh()
        self.Show()
