# -*- coding: utf-8 -*-
#==============================================================================#
#    E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#    PanelEA5- GUI component for: Energy intensity - Yearly data
#            
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------        
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
import wx
from status import *
import modules.matPanel as Mp
from GUI.graphics import drawSimpleBarPlot, drawComparedBarPlot
from auxiliary.auxiliary import toGettextAndUnicode as _U, toUnicode as U
from modules.energyStats.moduleEA5 import ModuleEA5
from modules.interfaces import Interfaces
from GUI.numCtrl import convertDoubleToString

[wxID_PANELEA5, wxID_PANELEA5GRID1, wxID_PANELEA5GRID2, 
 wxID_PANELEA5PANELGRAPHEI, wxID_PANELEA5PANELGRAPHSEC, 
 wxID_PANELEA5STATICTEXT1, wxID_PANELEA5STATICTEXT2, 
] = [wx.NewId() for _init_ctrls in range(7)]

#
# constants
#

ORANGE = '#FF6000'
LIGHTGREY = '#F8F8F8'
WHITE = '#FFFFFF'
DARKGREY = '#000060'
LIGHTGREEN = '#F0FFFF'

GRID_LETTER_SIZE = 8 #points
GRID_LABEL_SIZE = 9  # points
GRID_LETTER_COLOR = DARKGREY     # specified as hex #RRGGBB
GRID_BACKGROUND_COLOR = LIGHTGREY # idem
GRAPH_BACKGROUND_COLOR = WHITE # idem
TITLE_COLOR = ORANGE


####### HS 2006-07-01 aqui simetria de nomenclatura ... 
##COLNO1 = 2#SD
##COLNO2 = 4
##MAXROWS = 10


class PanelEA5(wx.Panel):
    def __init__(self, parent, main):
        self._init_ctrls(parent)
        keys = ['EA5_EI', 'EA5_SEC']
        self.mod = ModuleEA5(keys)

#..............................................................................
# inicialización Grid 1

        labels_column = 0
        # remaps drawing methods to the wx widgets.
        #
        # upper graphic: Energy intensity
        #

        (rows,cols) = Interfaces.GData[keys[0]].shape   
        ignoredrows = [rows-1]

        paramList={'labels'      : labels_column,          # labels column
                   'data'        : 1,                      # data column for this graph
                   'key'         : keys[0],                # key for Interface
                   'title'       : _U('Energy intensity'),     # title of the graph
                   'backcolor'   : GRAPH_BACKGROUND_COLOR, # graph background color
                   'ignoredrows' : ignoredrows}                    # rows that should not be plotted

        dummy = Mp.MatPanel(self.panelGraphEI,wx.Panel,drawSimpleBarPlot,paramList)

#..............................................................................
# initialization Grid 2

        #
        # lower grid: SEC by product
        #

        rows = Interfaces.GData[keys[1]].shape[0]       
        ignoredrows = [rows-1]

        paramList={'labels'      : labels_column,          # labels column
                   'data'        : 1,                      # data column for this graph
                   'key'         : keys[1],                # key for Interface
                   'title'       : _U('SEC by product'),       # title of the graph
                   'ylabel'      : 'Energy',
                   'legend'      : [_U('Energy by fuels'), _U('Energy by electricity'), _U('Primary energy')], # legend
                   'backcolor'   : GRAPH_BACKGROUND_COLOR, # graph background color
                   'ignoredrows' : ignoredrows}            # rows that should not be plotted

        dummy = Mp.MatPanel(self.panelGraphSEC,wx.Panel,drawComparedBarPlot,paramList)
        
        #
        # additional widgets setup
        #
        #
        # data cell attributes
        attr = wx.grid.GridCellAttr()
        attr.SetTextColour(GRID_LETTER_COLOR)
        attr.SetBackgroundColour(GRID_BACKGROUND_COLOR)
        attr.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.BOLD))
        #
        # set upper grid
        #

        
#..............................................................................
# write data to grid 1

        data = Interfaces.GData[keys[0]]

#####Security feature against non existing GData entry
        COLNO1 = 3 #grid has usually a fixed column size, not necessary to read from GData        
        try: (rows,cols) = data.shape
        except: (rows,cols) = (0,COLNO1)

        self.grid1.CreateGrid(max(rows,10), COLNO1)

        self.grid1.EnableGridLines(True)
        self.grid1.SetDefaultRowSize(20)
        self.grid1.SetRowLabelSize(30)

        self.grid1.SetColSize(0,140)
        self.grid1.SetColSize(1,125)
        self.grid1.SetColSize(2,125)
        
        self.grid1.EnableEditing(False)
        self.grid1.SetLabelFont(wx.Font(9, wx.ROMAN, wx.ITALIC, wx.BOLD))
        self.grid1.SetColLabelValue(0, _U("Energy type"))
        self.grid1.SetColLabelValue(1, _U("Energy intensity") +U("\n[kWh/€]"))
        self.grid1.SetColLabelValue(2, _U(" "))
        #
        # copy values from dictionary to grid
        #


#SD2008-06-30
        decimals = [-1,2,-1]   #number of decimal digits for each colum
        for r in range(rows):
            self.grid1.SetRowAttr(r, attr)
            for c in range(cols):
                try:
                    if decimals[c] >= 0: # -1 indicates text
                        self.grid1.SetCellValue(r, c, \
                            convertDoubleToString(float(data[r][c]),nDecimals = decimals[c]))
                    else:
                        self.grid1.SetCellValue(r, c, data[r][c])
                except: pass
                if c == labels_column:
                    self.grid1.SetCellAlignment(r, c, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);
                else:
                    self.grid1.SetCellAlignment(r, c, wx.ALIGN_RIGHT, wx.ALIGN_CENTRE);


        self.grid1.SetGridCursor(0, 0)

#..............................................................................
# write data to grid 2

        data = Interfaces.GData[keys[1]]
#####Security feature against non existing GData entry

        COLNO2 = 4 #grid has usually a fixed column size, not necessary to read from GData        
        try: (rows,cols) = data.shape
        except: (rows,cols) = (0,COLNO2)

        self.grid2.CreateGrid(max(rows,20), cols)

        self.grid2.EnableGridLines(True)
        self.grid2.SetDefaultRowSize(20)
        self.grid2.SetRowLabelSize(30)
        self.grid2.SetColLabelSize(50)

        self.grid2.SetColSize(0,120)
        self.grid2.SetColSize(1,90)
        self.grid2.SetColSize(2,90)
        self.grid2.SetColSize(3,90)
        
        self.grid2.EnableEditing(False)
        self.grid2.SetLabelFont(wx.Font(9, wx.ROMAN, wx.ITALIC, wx.BOLD))
        self.grid2.SetColLabelValue(0, _U("Product"))
        self.grid2.SetColLabelValue(1, _U("Energy by\nfuels\n[kWh/pu]"))
        self.grid2.SetColLabelValue(2, _U("Energy by\nelectricity\n[kWh/pu]"))
        self.grid2.SetColLabelValue(3, _U("Primary\nenergy\n[kWh/pu]"))
        #
        # copy values from dictionary to grid
        #

#SD2008-06-30

        decimals = [-1,2,2,2]   #number of decimal digits for each colum
        for r in range(rows):
            self.grid2.SetRowAttr(r, attr)
            for c in range(cols):
                try:
                    if decimals[c] >= 0: # -1 indicates text
                        self.grid2.SetCellValue(r, c, \
                            convertDoubleToString(float(data[r][c]),nDecimals = decimals[c]))
                    else:
                        self.grid2.SetCellValue(r, c, data[r][c])
                except: pass
                if c == labels_column:
                    self.grid2.SetCellAlignment(r, c, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);
                else:
                    self.grid2.SetCellAlignment(r, c, wx.ALIGN_RIGHT, wx.ALIGN_CENTRE);


        self.grid2.SetGridCursor(0, 0)

##        self.staticText1.SetFont(wx.Font(12, wx.ROMAN, wx.NORMAL, wx.BOLD))


#HS 2006-07-01: cancelado. staticText2 ya no existe.
# es extraño que esto no te haya dado error .... ???? -> tu vas comprobando el LOGfile ????
#        self.staticText2.SetFont(wx.Font(12, wx.ROMAN, wx.NORMAL, wx.BOLD))

#------------------------------------------------------------------------------        
    def _init_ctrls(self, prnt):
#------------------------------------------------------------------------------        
#   build-up of controls
#------------------------------------------------------------------------------        
        # generated method, don't edit
        wx.Panel.__init__(self, id=wxID_PANELEA5, name=u'PanelEA5', parent=prnt,
              pos=wx.Point(0, 0), size=wx.Size(800, 600))

#..............................................................................
#   box 1

#SD2008-06-30
        self.box1 = wx.StaticBox(self, -1, _U('Energy intensity by energy type (turnover)'),
                                 pos = (10,10),size=(780,260))

        self.box1.SetForegroundColour(TITLE_COLOR)
        self.box1.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

##        self.staticText1 = wx.StaticText(id=wxID_PANELEA5STATICTEXT1,
##              label=_U('Energy intensity by energy type (turnover)'),
##              name='staticText1', parent=self, pos=wx.Point(40, 8),
##              size=wx.Size(580, 20), style=0)

        self.grid1 = wx.grid.Grid(id=wxID_PANELEA5GRID1, name='grid1',#SD
              parent=self, pos=wx.Point(20, 40), size=wx.Size(440, 220),
              style=0)

        self.panelGraphEI = wx.Panel(id=wxID_PANELEA5PANELGRAPHEI,
              name=u'panelGraphEI', parent=self, pos=wx.Point(480, 40),
              size=wx.Size(300, 220), style=wx.TAB_TRAVERSAL|wx.SUNKEN_BORDER)
        self.panelGraphEI.SetBackgroundColour(wx.Colour(127, 127, 127))

#..............................................................................
#   box 2

#SD2008-06-30
        self.box2 = wx.StaticBox(self, -1, _U('Specific energy consumption (SEC) by product.'),
                                 pos = (10,290),size=(780,260))
        
        self.box2.SetForegroundColour(TITLE_COLOR)
        self.box2.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

##        self.staticText2 = wx.StaticText(id=wxID_PANELEA5STATICTEXT2,
##              label=_U('Specific energy consumption (SEC) by product.'),
##              name='staticText2', parent=self, pos=wx.Point(40, 324),
##              size=wx.Size(580, 20), style=0)


        self.grid2 = wx.grid.Grid(id=wxID_PANELEA5GRID2, name='grid2',
              parent=self, pos=wx.Point(20, 320), size=wx.Size(440, 220),
              style=0)


        self.panelGraphSEC = wx.Panel(id=wxID_PANELEA5PANELGRAPHSEC,
              name=u'panelGraphSEC', parent=self, pos=wx.Point(480, 320),
              size=wx.Size(300, 220), style=wx.TAB_TRAVERSAL|wx.SUNKEN_BORDER)
        self.panelGraphSEC.SetBackgroundColour(wx.Colour(127, 127, 127))

#..............................................................................
#   default buttons
#..............................................................................
        self.btnBack = wx.Button(id=wx.ID_BACKWARD, label=u'<<<',
              name=u'btnBack', parent=self, pos=wx.Point(500, 560),
              size=wx.Size(80, 20), style=0)
        self.btnBack.Bind(wx.EVT_BUTTON, self.OnBtnBackButton,
              id=-1)

        self.btnOK = wx.Button(id=wx.ID_OK, label=_U('OK'), name=u'btnOK',
              parent=self, pos=wx.Point(600, 560), size=wx.Size(80, 20),
              style=0)
        self.btnOK.Bind(wx.EVT_BUTTON, self.OnBtnOKButton,
              id=-1)

        self.btnForward = wx.Button(id=wx.ID_FORWARD, label=u'>>>',
              name=u'btnForward', parent=self, pos=wx.Point(700, 560),
              size=wx.Size(80, 20), style=0)
        self.btnForward.Bind(wx.EVT_BUTTON, self.OnBtnForwardButton,
              id=-1)


#------------------------------------------------------------------------------        
#   Event handlers for default buttons
#------------------------------------------------------------------------------        
    def OnBtnOKButton(self, event):
        event.Skip()

    def OnBtnBackButton(self, event):
        self.Hide()
        Status.main.tree.SelectItem(Status.main.qEA4b, select=True)
        print "Button exitModuleBack: now I should show another window"

    def OnBtnForwardButton(self, event):
        self.Hide()
        Status.main.tree.SelectItem(Status.main.qEM1, select=True)
        print "Button exitModuleFwd: now I should show another window"


#------------------------------------------------------------------------------    
    def display(self, data=None):
#------------------------------------------------------------------------------        
#   display function. carries out all the necessary calculations before
#   showing the panel
#------------------------------------------------------------------------------    
#####Security feature against any strange thing in graphs
        try: self.panelGraphEI.draw()
        except: pass
        try: self.panelGraphSEC.draw()
        except: pass
        self.Show()


