# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	Panel Heat Recovery module
#			
#------------------------------------------------------------------------------
#			
#	Short description:
#	
#	Panel for HR design assistant
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
import wx
import numpy as np
from GUI.status import Status
from GUI.displayClasses import FieldSizes, FloatEntry, ChoiceEntry
from modules.messageLogger import logDebug, showWarning, logWarning, askConfirmation
from constants import HRMODES, TRANSHXDESIGNSTRATEGIES, findKey
import modules.energyStats.modulePA1 as modulePA1

from GUI.fonts import FontProperties

from numCtrl import convertDoubleToString
from auxiliary.auxiliary import toGettextAndUnicode as _U
from auxiliary.auxiliary import toUnicode as U
from GUITools import TITLE_COLOR, GRID_LETTER_COLOR, GRID_LETTER_SIZE, \
     GRID_BACKGROUND_COLOR, GRID_BACKGROUND_COLOR_HIGHLIGHT

[wxID_PANELHR, wxID_PANELHRBTNCALCULATE, wxID_PANELHRBTNCHANGEHX, 
 wxID_PANELHRBTNADDHXMANUALLY, wxID_PANELHRBTNSHOWSTDVALUES, 
 wxID_PANELHRBUTTONPAGEHRBACK, wxID_PANELHRBUTTONPAGEHRCANCEL, 
 wxID_PANELHRBUTTONPAGEHRFWD, wxID_PANELHRBUTTONPAGEHROK, 
 wxID_PANELHRCBCONSIDERCOND, wxID_PANELHRCBCURVEDISPLAY, wxID_PANELHRCBENERGYORPOWER, wxID_PANELHRCBEXHX, 
 wxID_PANELHRGRID, wxID_PANELHRPANEL_DRAWCURVE, wxID_PANELHRRBCALC, 
 wxID_PANELHRRBREDESIGN, wxID_PANELHRSTATICBOX1, wxID_PANELHRSTATICBOX2, 
 wxID_PANELHRSTATICBOX3, wxID_PANELHRSTATICBOX4, wxID_PANELHRSTATICTEXT1, 
 wxID_PANELHRSTATICTEXT2, wxID_PANELHRSTATICBOXDA, wxID_PANELHRBTNCONFIGURE
] = [wx.NewId() for _init_ctrls in range(25)]

# constants
MAXROWS = 50
COLNO_AEE = 12
COLNO = 6

spacing_left = 0.1
spacing_right = 0.95
spacing_bottom = 0.15
spacing_top = 0.9
spacing_w = 0.4

# constants that control the default sizes
# 1. font sizes
TYPE_SIZE_LEFT = 9
TYPE_SIZE_MIDDLE = 9
TYPE_SIZE_RIGHT = 9
TYPE_SIZE_TITLES = 10

# 2. field sizes
HEIGHT = 32
HEIGHT_RIGHT = 32

LABEL_WIDTH_LEFT = 200
LABEL_WIDTH_RIGHT = 200
LABEL_WIDTH_LEFT_TABLE = 200
LABEL_WIDTH_RIGHT_TABLE = 0
LABEL_WIDTH_ARROW = 40


DATA_ENTRY_WIDTH = 80
DATA_ENTRY_WIDTH_LEFT = 80
DATA_ENTRY_WIDTH_TABLE = 60

UNITS_WIDTH = 80
UNITS_WIDTH_TABLE = 50

# 3. vertical separation between fields
VSEP_LEFT = 2
VSEP_RIGHT = 2


ORANGE = '#FF6000'
TITLE_COLOR = ORANGE

class PanelHR(wx.Panel):

##    def __init__(self, parent, main, id, pos, size, style, name):
    def __init__(self, parent, main):
        self.main = main
        self.mod = Status.mod.moduleHR
        self.keys = self.mod.keys
        self._init_ctrls(parent)
        self.__init_custom_ctrls(parent)
        self.__do_layout()
        
        #TODO: these parameters correspond to old panels
        self.considerconden = _U('Consider condensation heat in offgas of boilers?')
        self.considerdesignassist = _U('Configure design assistant')
        self.minimalratio = _U('Minimum ratio')
        self.totheattransfered = _U('Total heat transferred')
        self.totenergydemand = _U('Total energy demand')
        self.mintempdiff = _U('minimum temperature difference in heat exchangers')
        self.nomheatrate=_U('Nominal heat transfer rate')

        self.mod.initPanel()


    def _init_ctrls(self, prnt):

        # generated method, don't edit
        wx.Panel.__init__(self, id=wxID_PANELHR, name='PanelHR', parent=prnt,
              pos=wx.Point(0, 0), size=wx.Size(808, 627), style=0)
        self.SetClientSize(wx.Size(800, 600))

        self.grid = wx.grid.Grid(id=wxID_PANELHRGRID, name='gridpageHR',
                  parent=self, size=wx.Size(760, 320),
                  style=0)
            
        self.grid.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK,
              self.OnGridGridCellLeftDclick, id=wxID_PANELHRGRID)
        self.grid.Bind(wx.grid.EVT_GRID_CELL_LEFT_CLICK,
              self.OnGridGridCellLeftClick, id=wxID_PANELHRGRID)
        self.grid.Bind(wx.grid.EVT_GRID_CELL_RIGHT_CLICK,
              self.OnGridGridCellRightClick, id=wxID_PANELHRGRID)

        # Buttons Back | Cancel | Ok | Fwd
        self.buttonOK = wx.Button(id=wxID_PANELHRBUTTONPAGEHROK,
              label=_U('ok'), name='buttonpageHROk', parent=self,
              pos=wx.Point(672, 544), size=wx.Size(120, 24), style=0)
        self.buttonOK.Bind(wx.EVT_BUTTON, self.OnButtonOK,
              id=wxID_PANELHRBUTTONPAGEHROK)

        # Grid with existing heat exchangers
        self.frameTable = wx.StaticBox(id=wxID_PANELHRSTATICBOX4,
              label=_U('Existing heat exchangers in the system'),
              name='staticBox4', parent=self, pos=wx.Point(16, 336),
              size=wx.Size(752, 200), style=0)


        # Buttons for activating design

        self.btnCalculate = wx.Button(id=wxID_PANELHRBTNCALCULATE,
              label=_U('Run design assistant'), name='btnCalculate', parent=self,
              pos=wx.Point(638, 32), size=wx.Size(160, 24), style=0)
        self.btnCalculate.Bind(wx.EVT_BUTTON, self.OnBtnCalculateButton,
              id=wxID_PANELHRBTNCALCULATE)

        self.btnAddHXManually = wx.Button(id=wxID_PANELHRBTNADDHXMANUALLY,
              label=_U('Add HX manually'), name='btnHXmanually', parent=self,
              pos=wx.Point(638, 64), size=wx.Size(160, 24), style=0)
        self.btnAddHXManually.Bind(wx.EVT_BUTTON, self.OnBtnAddHXManually,
              id=wxID_PANELHRBTNADDHXMANUALLY)
        
       # Configuration design assistant
        self.frameDA = wx.StaticBox(id=wxID_PANELHRSTATICBOXDA,
              label=_U('Configuration'), name='frameDA', parent=self,
              pos=wx.Point(632, 170), size=wx.Size(136, 60), style=0)

        self.cbExHX = wx.CheckBox(id=wxID_PANELHRCBEXHX,
              label=_U('Consider existing'), name='cbExHX', parent=self,
              pos=wx.Point(648, 112), size=wx.Size(200, 16), style=0)
        self.cbExHX.SetValue(False)
        self.cbExHX.Bind(wx.EVT_CHECKBOX, self.OnCbExHXCheckbox,
              id=wxID_PANELHRCBEXHX)

        self.frameDA.SetLabel(_U('Design assistant options:'))
        self.cbExHX.SetLabel(_U('Maintain existing heat exchangers'))

        fs = FieldSizes(wHeight=HEIGHT, wLabel=LABEL_WIDTH_LEFT,
                       wData=DATA_ENTRY_WIDTH_LEFT, wUnits=UNITS_WIDTH)

        strategyChoices = [U(entry) for entry in TRANSHXDESIGNSTRATEGIES.values()]
        self.entryStrategy = ChoiceEntry(self, values=strategyChoices,
                           label=_U("Strategy for HX Network design"),
                           tip=_U(" "))

        self.entryDT = FloatEntry(self,
                              decimals=1, minval=0, maxval=2000., value=0.,
                              unitdict='TEMPERATUREDIFF',
                              label=_U('Minimum temperature difference'),
                              tip=_U("Between the fluids in the heat exchanger"))
        
        self.entryRatioNom = FloatEntry(self,
                              decimals=0, minval=0., maxval=1.e+4, value=0.,
                              unitdict='TIME',
                              label=_U('Total heat transfered / Nominal heat transfer rate'),
                              tip=_U(" "))

        self.entryRatioNom.Hide()

        self.entryRatioDemand = FloatEntry(self,
                              decimals=0, minval=0., maxval=1.e+4, value=0.,
                              unitdict='FRACTION',
                              label=_U('Total heat transfered / Nominal heat transfer rate'),
                              tip=_U(" "))
        self.entryRatioDemand.Hide()



        self.frameInfo = wx.StaticBox(id=wxID_PANELHRSTATICBOX4,
          label=_U('Performance of HX Network'),
          name='frameInfo', parent=self, style=0)

        self.infoQHXActual = FloatEntry(self,
                              decimals=0, minval=0., maxval=1.e+10, value=0.,
                              unitdict='ENERGY',
                              label=_U('Actual heat recovery (total)'),
                              tip=_U(" "))
        self.infoQHXTarget = FloatEntry(self,
                              decimals=0, minval=0., maxval=1.e+10, value=0.,
                              unitdict='ENERGY',
                              label=_U('Pinch target for heat recovery'),
                              tip=_U(" "))
        self.infoEffHXNet = FloatEntry(self,
                              decimals=0, minval=0., maxval=1.e+4, value=0.,
                              unitdict='FRACTION',
                              label=_U('Rate of approximation to pinch target'),
                              tip=_U(" "),
                              defaultDisplayUnit='%')
        
        self.infoQHXActual.Disable()
        self.infoQHXTarget.Disable()
        self.infoEffHXNet.Disable()

    def __do_layout(self):
        flagText = wx.ALL | wx.EXPAND

        # global sizer for panel.
        sizerGlobal = wx.BoxSizer(wx.VERTICAL)
        
        sizerTable = wx.StaticBoxSizer(self.frameTable, wx.VERTICAL)
        sizerTable.Add(self.grid, 0, flagText, VSEP_RIGHT)

        # sizer for left tab
        # tab 0, Heat recovery from thermal equipment

        sizerDAButtons = wx.BoxSizer(wx.HORIZONTAL)
        sizerDAButtons.Add(self.btnCalculate, 0, flagText, VSEP_RIGHT)
        sizerDAButtons.Add(self.btnAddHXManually, 0, flagText, VSEP_RIGHT)

        sizerDA = wx.BoxSizer(wx.HORIZONTAL)
        # left part: configuration design assistant
        sizerDALeft = wx.StaticBoxSizer(self.frameDA, wx.VERTICAL)
        sizerDALeft.Add(self.cbExHX, 0, flagText, VSEP_RIGHT)
        sizerDALeft.Add(self.entryStrategy, 0, flagText, VSEP_RIGHT)
        sizerDALeft.Add(self.entryDT, 0, flagText, VSEP_RIGHT)
        sizerDALeft.Add(self.entryRatioNom, 0, flagText, VSEP_RIGHT)
        sizerDALeft.Add(self.entryRatioDemand, 0, flagText, VSEP_RIGHT)
        sizerDA.Add(sizerDALeft, 1, wx.EXPAND | wx.ALL, 2)

        # right part: Info
        sizerDARight = wx.StaticBoxSizer(self.frameInfo, wx.VERTICAL)
        sizerDARight.Add(self.infoQHXActual, 0, flagText, VSEP_RIGHT)
        sizerDARight.Add(self.infoQHXTarget, 0, flagText, VSEP_RIGHT)
        sizerDARight.Add(self.infoEffHXNet, 0, flagText, VSEP_RIGHT)
        sizerDA.Add(sizerDARight, 1, wx.EXPAND | wx.ALL, 2)
        
        sizerOKCancel = wx.BoxSizer(wx.HORIZONTAL)
        sizerOKCancel.Add(self.buttonOK, 0, wx.ALL | wx.EXPAND, 2)

        sizerGlobal.Add(sizerTable, 0, flagText, 2)
        sizerGlobal.Add(sizerDAButtons, 0, wx.ALL | wx.ALIGN_LEFT, 2)
        sizerGlobal.Add(sizerDA, 0, wx.ALL | wx.EXPAND, 2)
        sizerGlobal.Add(sizerOKCancel, 0, wx.ALL | wx.ALIGN_RIGHT, 2)
        self.SetSizer(sizerGlobal)
        self.Layout()

    def __init_custom_ctrls(self, prnt):
        # access to font properties object
        fp = FontProperties()

        # set font for titles
        # 1. save actual font parameters on the stack
        fp.pushFont()
        # 2. change size and weight
        fp.changeFont(size=TYPE_SIZE_TITLES, weight=wx.BOLD)
        
        self.frameTable.SetForegroundColour(TITLE_COLOR)
        self.frameTable.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.frameInfo.SetForegroundColour(TITLE_COLOR)
        self.frameDA.SetForegroundColour(TITLE_COLOR)

        self.frameInfo.SetFont(fp.getFont())
        self.frameDA.SetFont(fp.getFont())
        self.frameTable.SetFont(fp.getFont())

            
       # 3. recover previous font state
        fp.popFont()

        self.cbExHX.SetFont(fp.getFont())
  

            

        #----Grid-------------------------------------------------------------------------
        self.selectedRow = 0
        labels_column = 0
        
        # data cell attributes
        attr = wx.grid.GridCellAttr()
        attr.SetTextColour(GRID_LETTER_COLOR)
        attr.SetBackgroundColour(GRID_BACKGROUND_COLOR)
        attr.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.NORMAL))

        attr2 = wx.grid.GridCellAttr()
        attr2.SetTextColour(GRID_LETTER_COLOR)
        attr2.SetBackgroundColour(GRID_BACKGROUND_COLOR_HIGHLIGHT)
        attr2.SetFont(wx.Font(GRID_LETTER_SIZE, wx.SWISS, wx.NORMAL, wx.NORMAL))

        self.grid.CreateGrid(MAXROWS, COLNO)

        self.grid.SetSelectionMode(wx.grid.Grid.wxGridSelectRows)
        self.grid.EnableGridLines(True)
        self.grid.SetDefaultRowSize(16)
        self.grid.SetRowLabelSize(30)

        self.grid.SetRowLabelSize(2)
        self.grid.SetDefaultColSize(80)

        self.grid.SetColSize(0,160)
        self.grid.SetColSize(1, 120)
        self.grid.SetColSize(3,240)
        
        self.grid.EnableEditing(False)
        self.grid.SetLabelFont(wx.Font(9, wx.ROMAN, wx.ITALIC, wx.BOLD))

        self.grid.SetColLabelValue(0, _U("Name"))
        self.grid.SetColLabelValue(1, _U("Nom. Power [kW]") + u"\n" + _U("Heat rec. [MWh]"))
        self.grid.SetColLabelValue(2, _U(u"Storage\n[m³]"))
        self.grid.SetColLabelValue(3, _U("Source") + "\n" + _U("Sink"))
        self.grid.SetColLabelValue(4, _U(u"Tho [°C]") + "\n" + _U(u"Tci [°C]"))
        self.grid.SetColLabelValue(5, _U(u"Thi [°C]") + "\n" + _U(u"Tco [°C]"))
            
        #
        # copy values from dictionary to grid
        #
        cols = COLNO
            
        textColumns = [0,3,6]
        for r in range(MAXROWS):
            if r%4<2:
                attr2.IncRef() # required for avoiding segmentation error; see http://www.python-it.org/forum/index.php?topic=6324.0
                self.grid.SetRowAttr(r, attr2)
            else:
                attr.IncRef()
                self.grid.SetRowAttr(r, attr)

            for c in range(cols):
                if c in textColumns:
                    self.grid.SetCellAlignment(r, c, wx.ALIGN_LEFT, wx.ALIGN_CENTRE);
                else:
                    self.grid.SetCellAlignment(r, c, wx.ALIGN_RIGHT, wx.ALIGN_CENTRE);

        self.grid.SetGridCursor(0, 0)         
        #----------------------------------------------------------------
#        self.rbRedesign.SetValue(True)
        self.mod.redesign = True
        
        self.Show()  

    def display(self, no=None):
        """Refresh the display of all shown data.
        
        This function is activated on each entry into the panel from the tree
        
        """
        self.mod.updatePanel()        # prepares data for plotting
          
        self.fillGrid()

        info = Status.int.GData[self.keys[1]]
        if "DTmin" in info.keys():
            DTmin = info["DTmin"]
        else:
            DTmin = None

        self.entryStrategy.SetValue(U(TRANSHXDESIGNSTRATEGIES[info["Strategy"]]))
        self.entryDT.SetValue(DTmin)
        self.infoQHXActual.SetValue(info["QHX"])
        self.infoQHXTarget.SetValue(info["QHXTarget"])
        self.infoEffHXNet.SetValue(info["EffHXNet"])
        
        self.Show()
        if Status.HRTool == "estimate":
            showWarning(_U("You're using the estimative mode. The HX network is "\
                        "not simulated but estimated on a generic basis.\n"\
                        "This dialog is therefore useless."))
    
    def fillGrid(self):
        
        data = Status.int.GData[self.keys[0]]
        rows = len(data)
        cols = 6
        decimals = [-1,2,2,-1,2,2]   #number of decimal digits for each colum

        for r in range(min(rows, MAXROWS)):
            for c in range(cols):
                if decimals[c] < 0:
                    self.grid.SetCellValue(r, c, U(data[r][c]))
                else:
                    try:
                        self.grid.SetCellValue(r, c, convertDoubleToString(float(data[r][c]),nDecimals = decimals[c]))
                    except:
                        self.grid.SetCellValue(r, c, "")
                        logDebug("PanelHR (UpdateGrid): received corrupt data value [%s] at [r][c] = [%s][%s]"% \
                                 (r,c,data[r][c]))

        for r in range(rows,MAXROWS):
            for c in range(cols):
                self.grid.SetCellValue(r, c, "")
            
                    
    def roundFloat(self, n):
        try:
            r = "%.2f"%n
            return r
        except:
            return n
                        
    def setDAConfig(self):
        """ reads design assistant parameters and exports to module
        """

        DTmin = self.entryDT.GetValue()
        strategy = findKey(TRANSHXDESIGNSTRATEGIES, self.entryStrategy.GetValue(text=True))
        data = {"DTmin": DTmin,
                "Strategy": strategy}
        Status.int.GData[self.keys[1]].update(data)
        
#------------------------------------------------------------------------------		
    def OnGridGridCellLeftDclick(self, event):
#------------------------------------------------------------------------------		
        self.selectedRow = event.GetRow()
        event.Skip()

#------------------------------------------------------------------------------		
    def OnGridGridCellLeftClick(self, event):
#------------------------------------------------------------------------------		
        self.selectedRow = event.GetRow()
        event.Skip()

    def OnGridGridCellRightClick(self, event):
        self.selectedRow = event.GetRow()
        event.Skip()
        
#==============================================================================
#   <<< OK Cancel >>>
#==============================================================================

    def OnButtonOK(self, event):
        self.Hide()
        self.main.control("DA", None)

#==============================================================================
      
    def OnBtnCalculateButton(self, event):
        self.mod.HXPFlag = True
        self.setDAConfig()

        if (self.mod.redesign):
            changed = False
            exhx = self.cbExHX.GetValue()

            if exhx or len(Status.int.systemModel.hx) == 0:
                self.mod.runHRDesign(exhx)
                changed = True
            else:
                ret = askConfirmation(_U("This operation will delete all existing heat exchangers") + "\n" + _U("Do you want to continue ?"))
                if ret:
                    self.mod.runHRDesign(exhx)
                    changed = True
                    
            if changed:
                Status.mod.moduleEnergy.runSimulation()
                
        else:
            self.mod.runHRModule(exhx = self.cbExHX.GetValue())            

        self.display()        
        self.fillGrid()
        self.mod.HXPFlag = False
        if event: event.Skip()
        
    def OnBtnAddHXManually(self, event):
##        print "button add hx"
        self.Hide()
        self.main.control("HX", None)


    def OnCbExHXCheckbox(self, event):
        #self.mod.ExHX = self.cbExHX.GetValue()
        #print self.mod.ExHX
        event.Skip()

