# -*- coding: utf-8 -*-
#==============================================================================
#
#   E I N S T E I N
#
#   Expert System for an Intelligent Supply of Thermal Energy in Industry
#   (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#   PanelInfo
#
#------------------------------------------------------------------------------
#
#   INFO Panel in GUI
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
import wx

from GUI.status import Status
from auxiliary.auxiliary import toGettextAndUnicode as _U, toUnicode as U, noneCheck
from constants import INTERACTIONLEVELS, TRANSINTERACTIONLEVELS
from globals import logger

class PanelInfo(wx.StatusBar):
    def __init__(self, parent, main):
        self.main = main
        self.parent = parent
        self.main = main
        
        project = noneCheck(Status.ActiveProjectName, "---")
        alternative = noneCheck(Status.ActiveAlternativeName, "---")
        level = noneCheck(Status.UserInteractionLevel, "---")

#------------------------------------------------------------------------------
#--- UI setup
#------------------------------------------------------------------------------

        wx.StatusBar.__init__(self, id=-1, name='PanelInfo', parent=parent, style=wx.SUNKEN_BORDER)
        self.SetFieldsCount(5)
        self.SetFont(wx.Font(9, wx.ROMAN, wx.NORMAL, wx.NORMAL, False, 'Arial'))

        self.gauge = wx.Gauge(self, -1, 100, wx.Point(0, 2), wx.Size(200, 16))

        self.Bind(wx.EVT_SIZE, self.OnSize, self)
##        self.Bind(wx.EVT_CHOICE, self.OnChoiceAssistant, self.choiceAssistant)

        self.update()


#------------------------------------------------------------------------------
#--- UI actions
#------------------------------------------------------------------------------		
    def _szpos(self, index, control, labelwidth=0):
        """
        Helper functions for resizing widgets within the StatusBar-fields

        index: index of status bar field
        control: widget to be resized
        labelwidth (optional): space to be left at the left of a widget for a label
        """
        rect=self.GetFieldRect(index)
        end = rect.x + rect.width
        p = control.GetPosition()
        sz = control.GetSize()
        sz.width = rect.width-labelwidth
        npos = wx.Point(end-sz.width,p.y)
        control.SetPosition(npos)
        control.SetSize(sz)

##    def OnChoiceAssistant(self, event):
##        i = self.choiceAssistant.GetCurrentSelection()
##        self.main.logWarning(_('Design assistant changed to ')+repr(TRANSASSISTANTLIST[i]))	
##        Status.prj.setUserInteractionLevel(ASSISTANTLIST[i])
##        self.main.changeAssistantMainMenu(i)
##        if event is not None:
##            event.Skip()

    def OnSize(self, event):
        self._szpos(3, self.gauge)

#------------------------------------------------------------------------------
# public methods
#------------------------------------------------------------------------------
    def update(self):
        self.SetStatusText(U(noneCheck(Status.ActiveProjectName, "---")),0)
        self.SetStatusText(U(noneCheck(Status.ActiveAlternativeName, "---")), 1)
        interactionLevel = INTERACTIONLEVELS.index(Status.UserInteractionLevel)
        self.SetStatusText(U(TRANSINTERACTIONLEVELS[interactionLevel]), 4)
       
    def changeAssistant(self, level):
        self.SetStatusText(U(TRANSINTERACTIONLEVELS[level]))

        logger.logWarning(_('Design assistant changed to ')+repr(TRANSINTERACTIONLEVELS[level]))	
        Status.prj.setUserInteractionLevel(INTERACTIONLEVELS[level])
        self.main.changeAssistantMainMenu(level)

    def showProgress(self, msg, f):
        """
        Displays the progress of something in the status bar
        """
        self.SetStatusText(U(msg), 2)
        self.gauge.SetValue(f)
        wx.SafeYield()
        
if __name__ == '__main__':

    app = wx.PySimpleApp()
    frame = wx.Frame(parent=None, id=-1, size=wx.Size(800, 600), title="Einstein - panelInfo")
    main = Main(1)
    panel = PanelInfo(frame, main)

    frame.Show(True)
    app.MainLoop()
