# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.iee-einstein.org/" target="_blank">www.iee-einstein.org</a>)
#
#------------------------------------------------------------------------------
#
#	PanelQ2: Questionnaire on energy consumption
#
#==============================================================================
#
#   EINSTEIN Version No.: 1.0
#   Created by: 	Heiko Henning, Tom Sobota, Hans Schweiger, Stoyan Danov
#                       February 2008 - 13/10/2008
#
#   Update No. 001
#
#   Since Version 1.0 revised by:
#
#                       Hans Schweiger  06/04/2008
#               
#   06/04/2008  HS  Small bug-fix: increase of maximum value for fuel cost
#
#------------------------------------------------------------------------------
#	(C) copyleft energyXperts.BCN (E4-Experts SL), Barcelona, Spain 2008, 2009
#	http://www.energyxperts.net/
#
#	This program is free software: you can redistribute it or modify it under
#	the terms of the GNU general public license as published by the Free
#	Software Foundation (www.gnu.org).
#
#==============================================================================
import wx
from GUI import HelperClass
from GUI.status import *
from GUI.GUITools import check
import database.dbio as dbio
from GUI.displayClasses import ChoiceEntry, FloatEntry, FieldSizes
from GUI.fonts import FontProperties
from auxiliary.auxiliary import toGettextAndUnicode as _U
from auxiliary.auxiliary import toUnicode as U
from auxiliary.auxiliary import scale
from GUI.conflictFrame import conflictFrame
from GUI.displayCC import PanelCCPars
from modules.messageLogger import *
from modules.fluids import Fuel
# constants that control the default sizes
# 1. font sizes
TYPE_SIZE_LEFT = 9
TYPE_SIZE_RIGHT = 9
TYPE_SIZE_TITLES = 10

# 2. field sizes
HEIGHT_LEFT = 27
LABEL_WIDTH_LEFT = 400
DATA_ENTRY_WIDTH = 100
UNITS_WIDTH = 110

ORANGE = '#FF6000'
TITLE_COLOR = ORANGE

class PanelQ2_fuel(wx.Panel):
    def __init__(self, parent, main):
        self.prnt = parent
        self.main = main
        paramlist = HelperClass.ParameterDataHelper()
        self.PList = paramlist.ReadParameterData()
        self._init_ctrls(parent)
        self.__do_layout()
        self.selectedFuelID = None
        self.selectedFuelName = "---"
        self.fuel = None

    def _init_ctrls(self, parent):
        #--- UI setup
        wx.Panel.__init__(self, id= -1, name='PanelQ2', parent=parent,
              pos=wx.Point(0, 0), size=wx.Size(780, 580), style=0)
        self.Hide()
        
        # access to font properties object
        fp = FontProperties()

        self.page0 = wx.Panel(self, size=wx.Size(800, 400))

        self.framePage0 = wx.StaticBox(self.page0, -1, _U("Fuel consumption and cost"))
        self.framePage0.SetForegroundColour(TITLE_COLOR)
        self.framePage0.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        # set font for titles
        # 1. save actual font parameters on the stack
        fp.pushFont()
        # 2. change size and weight
        fp.changeFont(size=TYPE_SIZE_TITLES, weight=wx.BOLD)
        self.framePage0.SetFont(fp.getFont())
        # 3. recover previous font state
        fp.popFont()

        # set field sizes for the left tab.
        FieldSizes(wHeight=HEIGHT_LEFT, wLabel=LABEL_WIDTH_LEFT,
                   wData=DATA_ENTRY_WIDTH, wUnits=UNITS_WIDTH)

        # set font for labels of left tab
        fp.pushFont()
        fp.changeFont(size=TYPE_SIZE_LEFT)

        self.tc1 = ChoiceEntry(self.page0,
                               values=[],
                               label=_U("Fuel type"),
                               tip=_U(" "))
        self.Bind(wx.EVT_CHOICE, self.OnFuelChange, self.tc1.entry)
        
        
        self.tc3 = FloatEntry(self.page0,
                              decimals=2, minval=0., maxval=1.0e+12, value=0.,
                              unitdict='MASSORVOLUME',
                              label=_U("Annual consumption (fuel units)"),
                              tip=_U("If possible, provide the monthly data in separate sheet and/or the fuel bills. Specify the energy equivalent in base of LCV (lower calorific value)"))  
        
        self.tc4 = FloatEntry(self.page0,
                              decimals=2, minval=0., maxval=1.0e+12, value=0.,
                              unitdict='ENERGY',
                              label=_U("Annual consumption (LCV)"),
                              tip=_U(" "))  
        
        self.tc5 = FloatEntry(self.page0,
                              decimals=5, minval=0., maxval=999999., value=0.,
                              unitdict='ENERGYTARIFF',
                              label=_U("Fuel price(LCV)"),
                              tip=_U("Specify expenditures without VAT"))  
        
        self.tc6 = FloatEntry(self.page0,
                              decimals=2, minval=0., maxval=1.0e+12, value=0.,
                              unitdict='PRICE',
                              label=_U("Annual energy cost"),
                              tip=_U("Total cost"))  

        # fillers
        self.dummy1 = wx.StaticText(self.page0, -1, "")
        self.dummy2 = wx.StaticText(self.page0, -1, "")

        # OK/Cancel buttons
        self.buttonCancel = wx.Button(self, wx.ID_CANCEL, _U("Cancel"))
        self.buttonCancel.SetFont(fp.getFont())
        self.Bind(wx.EVT_BUTTON, self.OnButtonCancel, self.buttonCancel)

        self.buttonOK = wx.Button(self, wx.ID_OK, 'OK')
        self.buttonOK.SetDefault()
        self.buttonOK.SetFont(fp.getFont())
        self.Bind(wx.EVT_BUTTON, self.OnButtonOK, self.buttonOK)

        self.activeObject = wx.StaticText(self,id=-1,label="----- ----- -----",style = wx.ALIGN_LEFT)
        self.activeObject.SetMinSize((580, 24)) #a hack in order to achieve alignment at the left. can surely be done better.

        # Right panel controls
        fp.changeFont(size=TYPE_SIZE_RIGHT)


    def __do_layout(self):

        # right part of left tab: data entry widgets
        flagText = wx.ALIGN_CENTER_VERTICAL | wx.RIGHT | wx.LEFT

        sizerFramePage0 = wx.StaticBoxSizer(self.framePage0, wx.VERTICAL)
        sizerFramePage0.Add(self.tc1, 0, flagText, 2)
        sizerFramePage0.Add(self.tc3, 0, flagText, 2)
        sizerFramePage0.Add(self.tc4, 0, flagText, 2)
        sizerFramePage0.Add(self.tc5, 0, flagText, 2)
        sizerFramePage0.Add(self.tc6, 0, flagText, 2)

        # sizer for left tab
        sizerPage0 = wx.BoxSizer(wx.VERTICAL)
##        sizerPage0.Add(sizerFramePage0, 2, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 20)
        sizerPage0.Add(sizerFramePage0, 0,  wx.ALL | wx.EXPAND, 2)
##        self.page0.SetSizer(sizerPage0)

        # sizer for right tab
        sizerOKCancel = wx.BoxSizer(wx.HORIZONTAL)
        sizerOKCancel.Add(self.activeObject, 0, wx.ALL | wx.EXPAND, 2)
        sizerOKCancel.Add(self.buttonCancel, 0, wx.ALL | wx.EXPAND, 2)
        sizerOKCancel.Add(self.buttonOK, 0, wx.ALL | wx.EXPAND, 2)

        # global sizer for panel. Contains page0 + buttons Cancel and Ok
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_1.Add(sizerPage0, 1, wx.EXPAND, 0)
        sizer_1.Add(sizerOKCancel, 0, wx.TOP | wx.ALIGN_RIGHT, 0)
        self.SetSizer(sizer_1)
        self.Layout()

    def OnButtonOK(self, event):
        if not self.editAllowed():
            logWarning(_("Editing of data not allowed"))
            return
        self.storeFuelData()

        Status.prj.setStatus("ECO", 0) #if tariffs have been changed, TCA has to be repeated
        Status.prj.projectChanged()
        Status.int.systemModel.changed()

        self.prnt.Close()
        
    def storeFuelData(self):
        fuelName = self.tc1.GetValue(text=True)
        
        dbfuels = dbio.select("dbfuel", FuelName=fuelName)
        if len(dbfuels) > 0:
            dbfid = dbfuels[0].id
            self.selectedFuelID = dbfid
            self.selectedFuelName = fuelName
        else:
            dbfid = -1
        fuels = dbio.select("qfuel", Questionnaire_id=Status.PId, AlternativeProposalNo=Status.ANo, DBFuel_id=dbfid)

        if Status.PId <> 0 and fuelName != 'None' and fuelName.strip() != "":
            if len(fuels) == 0:
                newID = Status.prj.addFuelDummy()
                tmp = {
                    "DBFuel_id":dbfid,
                    "MFuelYear":dbio.check(self.tc3.GetValue()),
                    "FECFuel":dbio.check(self.tc4.GetValue()),
                    "FuelTariff":dbio.check(self.tc5.GetValue()),
                    "FuelCostYear":dbio.check(self.tc6.GetValue())
                    }
                
                q = dbio.get("qfuel", id=newID)
                q.set(**tmp)               
                dbio.commit()
                
##                self.fillFuelList()
##
            elif len(fuels) == 1:
                tmp = {
                    "DBFuel_id":dbfid,
                    "MFuelYear":dbio.check(self.tc3.GetValue()),
                    "FECFuel":dbio.check(self.tc4.GetValue()),
                    "FuelTariff":dbio.check(self.tc5.GetValue()),
                    "FuelCostYear":dbio.check(self.tc6.GetValue())
                    }
                
                q = dbio.get("qfuel", DBFuel_id=dbfid, Questionnaire_id=Status.PId, AlternativeProposalNo=Status.ANo)
                q.set(**tmp)               
                dbio.commit()
##                self.fillFuelList()
                          
            else:
                self.main.showError(_U("Fuel name has to be a unique value!"))
                return

            if Status.ANo <> 0: # we modified an "real" alternative proposal
                i = q.FuelNo
                conflicts = Status.mod.moduleCC.fuelCheck(i)
                Status.mod.moduleCC.initPanel()
                if conflicts > 0:
                    self.cf = conflictFrame(self)
                    self.cf.Show()

                self.display()


    def setSelectedFuel(self, i):
        """displays the fuel under self.selectedFuelID"""

        q = None
        if i >= 0:
            q = dbio.get("qfuel", \
                    Questionnaire_id=Status.PId, \
                    AlternativeProposalNo=Status.ANo, \
                    FuelNo=i+1)
            self.selectedFuelID = q.DBFuel_id
            fuel = Fuel(self.selectedFuelID)
            self.selectedFuelName = fuel.name
            
        elif self.selectedFuelID is not None:
            
            q = dbio.get("qfuel", \
                Questionnaire_id=Status.PId, \
                AlternativeProposalNo=Status.ANo, \
                DBFuel_id=self.selectedFuelID)

        if q is not None:
            self.tc1.SetValue(self.selectedFuelName)
            
            self.tc3.SetValue(q.MFuelYear)
            self.tc4.SetValue(q.FECFuel)
            self.tc5.SetValue(q.FuelTariff)
            self.tc6.SetValue(q.FuelCostYear)

            i = q.FuelNo-1
            try:
                checkFETi = Status.mod.moduleCC.ccFET[i]
            except:
                logTrack("No CCPar available for fuel %s"%(i+1))
                checkFETi = None

            if checkFETi is not None:
                ccPars = PanelCCPars({"FECFuel":(checkFETi.FECFuel, self.tc4),
                                      "MFuelYear":(checkFETi.MFuelYear, self.tc3),
                                      "FuelTariff":(checkFETi.FuelTariff, self.tc5),
                                      "FuelCostYear":(checkFETi.FuelCostYear, self.tc6)
                                     },
                                     dataGroup = ("Fuel[" + str(i+1) + "]"))
                ccPars.displayCC()

    def OnButtonCancel(self, event):
        self.clear()
        self.prnt.Close()
    
    def OnFuelChange(self, event):
        """activated if some fuel selection is changed.
            recalls the updateUnitDict-function for the combined-unit-entries"""

        fuelName = self.tc1.GetValue(text=True)
        dbfuels = dbio.select("dbfuel", FuelName=dbio.check(fuelName))
        if len(dbfuels) > 0:
            dbfid = dbfuels[0].id
            self.fuel = Fuel(dbfid)
            old_funit2 = self.tc3.funit2
            self.tc3.updateUnitDict(self.fuel.rho)
            if self.tc3.funit2 != old_funit2:
                iValue = self.tc3.GetValue() #takes over display value into internal value
                self.tc3.SetValue(iValue)
        
    #--- Public methods
    def display(self, i=None):
        self.fillChoiceOfDBFuelType()
        self.clear()
        if i is not None and i>=0:
            self.setSelectedFuel(i)
            self.activeObject.SetLabel(U(self.selectedFuelName))
        self.Show()


    def fillChoiceOfDBFuelType(self):
        fuelDict = Status.prj.getFuelDict()
        fuelNames = fuelDict.values()
        self.tc1.entry.Clear()
        for name in fuelNames:
            self.tc1.entry.Append(name)        

    def clear(self):
        self.tc3.Clear()
        self.tc4.Clear()
        self.tc5.Clear()
        self.tc6.Clear()

    def editAllowed(self):
        """Return True if the data may be modified in the current state, 
        if not inform the user and return False.
        
        """
        if Status.ANo == 0:
            showWarning(_U("It is not allowed to modify fuels in the checked state. These values are a RESULT of calculations."))
            result = False
        elif Status.ANo < 0 and Status.StatusCC > 0:
            showWarning(_U("You cannot modify the fuel consumption after the data has been checked.\n")+
                        _U("If you want to change the present state you may unblock the data by ")+
                        _U("choosing to confirm the deletion ")+ 
                        _U("of any alternative proposals in this project the next time you switch to the present state."))
            result = False
        else:   
            result = True
        
        return result

    def goTo(self, key):
        """ tries to open the page appropriate
        for a given parameter
        """
        # then open the right page in the panel
        parList = {
            "FECFuel" : 0,
            "FuelTariff" : 0,
            "FuelCostYear" : 0,
            "FETFuel" : 0,
            "FETFuelTotal" : 0,
            "FEOFuel" : 0,
        }

        if key in parList:
            return True
        else:
            return False

if __name__ == '__main__':
    import pSQL
    import MySQLdb
    class Main(object):
        def __init__(self, qid):
            self.activeQid = qid

    DBName = 'einstein'
##    Status.SQL = MySQLdb.connect(host='localhost', user='root', passwd='tom.tom', db=DBName)
##    Status.DB = pSQL.pSQL(Status.SQL, DBName)

    app = wx.PySimpleApp()
    frame = wx.Frame(parent=None, id= -1, size=wx.Size(800, 600), title="Einstein - panelQ2")
    main = Main(1)
    panel = PanelQ2(frame, main)

    frame.Show(True)
    app.MainLoop()
