# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#	PanelQ6: Heat exchangers
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
import copy
import os
import wx
from status import *
from constants import *
from modules.messageLogger import *
from auxiliary.auxiliary import toGettextAndUnicode as _U, toUnicode as U, toUTF8, noneCheck
from GUI.fonts import FontProperties
from GUI.displayClasses import FieldSizes, TextEntry, ChoiceEntry, \
    FloatEntry
from GUI.GUITools import check, fillChoice
import database.dbio as dbio
from GUI.conflictFrame import conflictFrame
from GUI.displayCC import PanelCCPars
from modules.fluids import Fluid

from constants import TRANSHXTYPES, DEBUG
from modules.guiModules.moduleQ6 import ModuleQ6
from globals import Globals

# TODO: move this to the MODULe
from modules.heatRecovery.heatExchanger import HeatExchanger
from modules.heatRecovery.hxConnection import buildHXNet, getPrimarySinkOfConn, getPrimarySourceOfConn, HXConnection

# constants that control the default sizes
# 1. font sizes
TYPE_SIZE_LEFT = 9
TYPE_SIZE_MIDDLE = 9
TYPE_SIZE_RIGHT = 9
TYPE_SIZE_TITLES = 10

# 2. field sizes
HEIGHT = 24
HEIGHT_RIGHT = 32
HEIGHT_EDITCON = 24

LABEL_WIDTH_LEFT = 280
LABEL_WIDTH_EDITCON_LEFT = 288
LABEL_WIDTH_RIGHT = 200
LABEL_WIDTH_LEFT_TABLE = 200
LABEL_WIDTH_RIGHT_TABLE = 0
LABEL_WIDTH_ARROW = 40
LABEL_WIDTH_WIDE = 550


DATA_ENTRY_WIDTH = 80
DATA_ENTRY_WIDTH_LEFT = 80
DATA_ENTRY_WIDTH_TABLE = 60
DATA_ENTRY_WIDTH_EDITCON_LEFT = 120

UNITS_WIDTH = 80
UNITS_WIDTH_TABLE = 50
UNITS_WIDTH_WIDE = 100

# 3. vertical separation between fields
VSEP_LEFT = 2
VSEP_RIGHT = 2


ORANGE = '#FF6000'
TITLE_COLOR = ORANGE

WARNING_MATCH = _("Only Streams from the same Process, Equipment, etc. can be matched.")
WARNING_DOUBLE = _("Stream already added.")


class PanelQ6(wx.Panel):
    def __init__(self, parent, main, no):
        self.main = main
        self.prnt = parent

        self.HXID = None
        self.HXNo = None

        self.hx = None
        self.HXName = None

        self._init_ctrls(parent)
        self.__do_layout()

        self.mod = ModuleQ6("Q6", no)

        self.allowListBoxEvents = True

    def _init_ctrls(self, parent):
        """UI setup"""
        import platform # this is a hack to the GUI for gtk platforms
        (dummy, arch) = platform.architecture()
        if "gtk2" in wx.PlatformInfo:
            wx.Panel.__init__(self, id= -1, name='PanelQ6', parent=parent,
                              pos=wx.Point(0, 0), size=wx.Size(780, 600), style=0)
        else:
            wx.Panel.__init__(self, id= -1, name='PanelQ6', parent=parent,
                              pos=wx.Point(0, 0), size=wx.Size(780, 580), style=0)
        self.Hide()

        # access to font properties object
        fp = FontProperties()

        self.notebook = wx.Notebook(self, -1, style=0)
        self.notebook.SetFont(fp.getFont())
        self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.OnNotebookPageChanged, self.notebook)

        self.page0 = wx.Panel(self.notebook)
        self.notebook.AddPage(self.page0, _U('Technical data'))

##        self.page1 = wx.Panel(self.notebook)
##        self.notebook.AddPage(self.page1, _U('Source and sink'))

        self.page2 = wx.Panel(self.notebook)
        self.notebook.AddPage(self.page2, _U('Economic parameters'))

##        self.notebookSinkSource = wx.Notebook(self.page1, -1, style=0)
##        self.notebookSinkSource.SetFont(fp.getFont())
##        self.pageSource = wx.Panel(self.notebookSinkSource)
##        self.notebookSinkSource.AddPage(self.pageSource, _U('Source'))
##        self.pageSink = wx.Panel(self.notebookSinkSource)
##        self.notebookSinkSource.AddPage(self.pageSink, _U('Sink'))
##        self.SinkActive = False
##        self.Sink = SinkSourcePanel(self.pageSink, self)
##        self.Source = SinkSourcePanel(self.pageSource, self)

        self.frame_exchanger_data = wx.StaticBox(self.page0, -1, _U("Heat exchanger data"))
        self.frame_exchanger_data.SetForegroundColour(TITLE_COLOR)
        self.frame_exchanger_data.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.frame_storage_data = wx.StaticBox(self.page0, -1, _U("Heat storage data"))
        self.frame_storage_data.SetForegroundColour(TITLE_COLOR)
        self.frame_storage_data.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.frame_source = wx.StaticBox(self.page0, -1, _U("Source"))
        self.frame_source.SetForegroundColour(TITLE_COLOR)
        self.frame_source.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.frame_sink = wx.StaticBox(self.page0, -1, _U("Sink"))
        self.frame_sink.SetForegroundColour(TITLE_COLOR)
        self.frame_sink.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.frame_economic_parameters = wx.StaticBox(self.page2, -1, _U("Economic parameters"))
        self.frame_economic_parameters.SetForegroundColour(TITLE_COLOR)
        self.frame_economic_parameters.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        # set font for titles
        # 1. save actual font parameters on the stack
        fp.pushFont()
        # 2. change size and weight
        fp.changeFont(size=TYPE_SIZE_TITLES, weight=wx.BOLD)
        self.frame_exchanger_data.SetFont(fp.getFont())
        self.frame_storage_data.SetFont(fp.getFont())
        self.frame_source.SetFont(fp.getFont())
        self.frame_sink.SetFont(fp.getFont())
        self.frame_economic_parameters.SetFont(fp.getFont())
        # 3. recover previous font state
        fp.popFont()

        fs = FieldSizes(wHeight=HEIGHT, wLabel=LABEL_WIDTH_LEFT,
                       wData=DATA_ENTRY_WIDTH_LEFT, wUnits=UNITS_WIDTH)

        # left tab controls
        # tab 0 - Heat recovery from thermal equipment
        fp.pushFont()
        fp.changeFont(size=TYPE_SIZE_LEFT)

        # right top static box: Heat exchanger data
        self.tc1 = TextEntry(self.page0, maxchars=255, value='',
                             label=_U("Short name of heat exchanger"),
                             tip=_U("Give a short name of the equipment"))

##        self.tc2 = ChoiceEntry(self.page0,
##                               values=[],
##                               label=_U("Heat exchanger type"),
##                               tip=_U("Specify the type of heat exchanger, e.g. shell-and-tube, plate, fin-and-tube, ..."))


        self.tc3 = FloatEntry(self.page0,
                              decimals=2, minval=0., maxval=1.e+12, value=0.,
                              unitdict='POWER',
                              label=_U("Nominal heat transfer rate"),
                              tip=_U("Heat transfer rate for the nominal working conditions"))

        self.tc4 = FloatEntry(self.page0,
                              decimals=1, minval=0, maxval=2000., value=0.,
                              unitdict='TEMPERATURE',
                              label=_U("Log. Mean Temperature Diff. (LMTD)"),
                              tip=_U("Between the fluids in the heat exchanger"))

        fs = FieldSizes(wHeight=HEIGHT, wLabel=LABEL_WIDTH_RIGHT_TABLE,
                       wData=DATA_ENTRY_WIDTH_LEFT, wUnits=UNITS_WIDTH)

        self.labelNominal = wx.StaticText(self.page0, -1, _U("Nominal"), size = (DATA_ENTRY_WIDTH_LEFT+UNITS_WIDTH+VSEP_RIGHT, 20), style = wx.ALIGN_CENTRE)
        self.labelActual = wx.StaticText(self.page0, -1, _U("Actual"), size = (DATA_ENTRY_WIDTH_LEFT+UNITS_WIDTH+VSEP_RIGHT, 20), style = wx.ALIGN_CENTRE)

        self.tc31 = FloatEntry(self.page0,
                              decimals=2, minval=0., maxval=1.e+12, value=0.,
                              unitdict='POWER',
                              label=_U("Actual heat transfer rate"),
                              tip=_U("Heat transfer rate for the actual working conditions"))
        self.tc31.Disable()

        self.tc41 = FloatEntry(self.page0,
                              decimals=1, minval=0, maxval=2000., value=0.,
                              unitdict='TEMPERATURE',
                              label=_U("Actual Log. Mean Temperature Diff. (LMTD)"),
                              tip=_U("Between the fluids in the heat exchanger"))
        self.tc41.Disable()

        fs = FieldSizes(wHeight=HEIGHT, wLabel=LABEL_WIDTH_LEFT,
                       wData=DATA_ENTRY_WIDTH_LEFT, wUnits=UNITS_WIDTH)

        self.tc5 = FloatEntry(self.page0,
                              decimals=1, minval=0., maxval=1.e+12, value=0.,
                              unitdict='ENERGY',
                              label=_U("Total heat transfered"),
                              tip=_U("Total heat transferred per year"))
        self.tc5.Disable()

        self.tcUA = FloatEntry(self.page0,
                              decimals=2, minval=-1.0, maxval=1.e+12, value=0.,
                              unitdict='HEATTRANSFERCOEF',
                              label=_U("UA value of HX"),
                              tip=_U("Heat transfer coefficent U x heat transfer area A"))

##        self.tcArea = FloatEntry(self.page0,
##                              decimals=1, minval=0., maxval=1.e+12, value=0.,
##                              unitdict='AREA',
##                              label=_U("Area"),
##                              tip=_U(""))

        self.tcMCStorage = FloatEntry(self.page0,
                              decimals=2, minval=0., maxval=1.e+12, value=0.,
                              unitdict='THERMALMASS',
                              label=_U("Thermal mass of storage"),
                              tip=_U("m*cp of heat storage associated to the heat exchanger"))

        self.tcVStorage = FloatEntry(self.page0,
                              decimals=2, minval=0., maxval=1.e+12, value=0.,
                              unitdict='VOLUME',
                              label=_U("Equivalent water storage volume"),
                              tip=_U("Size of a sensible storage with water (cp = 4.19 kJ/kgK) with equivalent thermal mass"))
        self.tcVStorage.Disable()

        #=================================== source/sink panel


        self.listBoxSource = wx.ListBox(self.page0, -1, choices=[], size=(300, 24))
        self.listBoxSink = wx.ListBox(self.page0, -1, choices=[], size=(300, 24))

        self.Bind(wx.EVT_LISTBOX, self.OnListBoxSourceClick, self.listBoxSource)
        self.Bind(wx.EVT_LISTBOX, self.OnListBoxSinkClick, self.listBoxSink)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.OnListBoxSourceDClick, self.listBoxSource)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.OnListBoxSinkDClick, self.listBoxSink)

##        self.buttonAddSource = wx.Button(self.page0, -1, label=_U("Add"), size=(100, 24))
##        self.buttonAddSource.SetToolTip(wx.ToolTip(_U("click to add Stream")))
##        self.buttonDeleteSource = wx.Button(self.page0, -1, label=_U("Delete"), size=(100, 24))
##
##        self.buttonAddSink = wx.Button(self.page0, -1, label=_U("Add"), size=(100, 24))
##        self.buttonAddSink.SetToolTip(wx.ToolTip(_U("click to add Stream")))
##        self.buttonDeleteSink = wx.Button(self.page0, -1, label=_U("Delete"), size=(100, 24))
##
##        self.Bind(wx.EVT_BUTTON, self.OnButtonAddSink, self.buttonAddSink)
##        self.Bind(wx.EVT_BUTTON, self.OnButtonDeleteSink, self.buttonDeleteSink)
##        self.Bind(wx.EVT_BUTTON, self.OnButtonAddSource, self.buttonAddSource)
##        self.Bind(wx.EVT_BUTTON, self.OnButtonDeleteSource, self.buttonDeleteSource)


##        self.labelInlet = wx.StaticText(self.page0, -1, _U("Inlet"), size = (DATA_ENTRY_WIDTH_LEFT+UNITS_WIDTH+VSEP_RIGHT, 20), style = wx.ALIGN_CENTRE)
##        self.labelOutlet = wx.StaticText(self.page0, -1, _U("Outlet"), size = (DATA_ENTRY_WIDTH_LEFT+UNITS_WIDTH+VSEP_RIGHT, 20), style = wx.ALIGN_CENTRE)


        fs = FieldSizes(wHeight=HEIGHT_RIGHT, wLabel=LABEL_WIDTH_LEFT_TABLE,
                       wData=DATA_ENTRY_WIDTH_TABLE, wUnits=UNITS_WIDTH_TABLE)

        self.entryThoNom = FloatEntry(self.page0,
                      decimals=2, minval=-1000., maxval=2000., value=0.,
                      unitdict='TEMPERATURE',
                      label=_U("Hot side temperature") + " " + _U("(nominal; out/in)"),
                      tip=_U("Temperature of fluid at hot side")+" "+_U("(nominal)"))
        self.entryThoNom.Disable()

        self.entryTciNom = FloatEntry(self.page0,
                      decimals=2, minval=-1000., maxval=2000., value=0.,
                      unitdict='TEMPERATURE',
                      label=_U("Cold side temperature") + " " + _U("(nominal; in/out)"),
                      tip=_U("Temperature of fluid at cold side")+" "+_U("(nominal)"))
        self.entryTciNom.Disable()

        self.entryThoAvg = FloatEntry(self.page0,
                      decimals=2, minval=-1000., maxval=2000., value=0.,
                      unitdict='TEMPERATURE',
                      label=_U("(average actual values)"),
                      tip=_U("Temperature of fluid at hot side")+" "+_U("(average actual value)"))
        self.entryThoAvg.Disable()

        self.entryTciAvg = FloatEntry(self.page0,
                      decimals=2, minval=-1000., maxval=2000., value=0.,
                      unitdict='TEMPERATURE',
                      label=_U("(average actual values)"),
                      tip=_U("Temperature of fluid at cold side")+" "+_U("(aveage actual value)"))
        self.entryTciAvg.Disable()

        fs = FieldSizes(wHeight=HEIGHT, wLabel=LABEL_WIDTH_ARROW,
                       wData=DATA_ENTRY_WIDTH_TABLE, wUnits=UNITS_WIDTH_TABLE)

        self.entryThiNom = FloatEntry(self.page0,
                      decimals=2, minval=-1000., maxval=2000., value=0.,
                      unitdict='TEMPERATURE',
                      label=" ",
                      tip=_U("Outlet temperature of fluid at hot side")+" "+_U("(nominal)"))
        self.entryThiNom.Disable()

        self.entryTcoNom = FloatEntry(self.page0,
                      decimals=2, minval=-1000., maxval=2000., value=0.,
                      unitdict='TEMPERATURE',
                      label=" ",
                      tip=_U("Inlet temperature of fluid at cold side")+" "+_U("(nominal)"))
        self.entryTcoNom.Disable()

        self.entryThiAvg = FloatEntry(self.page0,
                      decimals=2, minval=-1000., maxval=2000., value=0.,
                      unitdict='TEMPERATURE',
                      label=" <<< ",
                      tip=_U("Temperature of fluid at hot side")+" "+_U("(average actual value)"))
        self.entryThiAvg.Disable()

        self.entryTcoAvg = FloatEntry(self.page0,
                      decimals=2, minval=-1000., maxval=2000., value=0.,
                      unitdict='TEMPERATURE',
                      label=" >>> ",
                      tip=_U("Temperature of fluid at cold side")+" "+_U("(average actual value)"))
        self.entryTcoAvg.Disable()


        #page 2: cost parameters

        fp.changeFont(size=TYPE_SIZE_RIGHT)
        f = FieldSizes(wHeight=HEIGHT_RIGHT, wData=DATA_ENTRY_WIDTH, wLabel=LABEL_WIDTH_WIDE, wUnits=UNITS_WIDTH)

        self.entryTurnKeyPrice = FloatEntry(self.page2,
                               decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'PRICE',
                               label = _U("Turn-key price"),
                               tip = _U("Full cost including transport, installation, piping and auxiliary equipment") + \
                                       _U("Only new equipment"))

        self.entryOMFix = FloatEntry(self.page2,
                               decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'PRICE',
                               label = _U("Annual operational and maintenance fixed costs"),
                               tip = _U("Fixed costs independent of actual operation hours"))

        self.entryOMVar = FloatEntry(self.page2,
                               decimals = 5, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'ENERGYTARIFF',
                               label = _U("Annual operational and maintenance variable costs dependant on usage"),
                               tip = _U("Annual operational and maintenance variable costs dependant on usage (approximate average per unit of generated heat)"))
        #
        # buttons
        #

        self.buttonOK = wx.Button(self, wx.ID_OK, "")
        self.Bind(wx.EVT_BUTTON, self.OnButtonOK, self.buttonOK)
        self.buttonOK.SetDefault()

        self.activeObject = wx.StaticText(self,id=-1,label="----- ----- -----",style = wx.ALIGN_LEFT)
        self.activeObject.SetMinSize((580, 24)) #a hack in order to achieve alignment at the left. can surely be done better.

        self.buttonCancel = wx.Button(self, wx.ID_CANCEL, "")
        self.Bind(wx.EVT_BUTTON, self.OnButtonCancel, self.buttonCancel)

        # recover previous font parameters from the stack
        fp.popFont()


    def __do_layout(self):
        flagText = wx.ALIGN_CENTER_VERTICAL | wx.TOP

        # global sizer for panel. Contains notebook w/two tabs + buttons Cancel and Ok
        sizerGlobal = wx.BoxSizer(wx.VERTICAL)

        # sizer for left tab
        # tab 0, Heat recovery from thermal equipment
        sizerPage0 = wx.BoxSizer(wx.HORIZONTAL)

        # right part: data entries
        sizerP0Right = wx.BoxSizer(wx.VERTICAL)
        sizerP0RightTop = wx.StaticBoxSizer(self.frame_exchanger_data, wx.VERTICAL)
        sizerP0RightTop.Add(self.tc1, 0, flagText, VSEP_RIGHT)
##        sizerP0RightTop.Add(self.tc2, 0, flagText, VSEP_RIGHT)
        sizerP0RightTop.Add(self.tcUA, 0, flagText, VSEP_RIGHT)
##        sizerP0RightTop.Add(self.tcArea, 0, flagText, VSEP_RIGHT)
        sizerP0RightTop.Add(self.tc5, 0, flagText, VSEP_RIGHT)

        sizerLabels = wx.BoxSizer(wx.HORIZONTAL)
        sizerLabels.Add(wx.Size(300,40), 0, flagText, VSEP_RIGHT)
        sizerLabels.Add(self.labelNominal, 0, wx.ALIGN_CENTRE|wx.ALIGN_BOTTOM, VSEP_RIGHT)
        sizerLabels.Add(self.labelActual, 0, wx.ALIGN_CENTRE|wx.ALIGN_BOTTOM, VSEP_RIGHT)
        sizerP0RightTop.Add(sizerLabels, 0, flagText, VSEP_RIGHT)

        sizerTC3 = wx.BoxSizer(wx.HORIZONTAL)
        sizerTC3.Add(self.tc3, 0, flagText, VSEP_RIGHT)
        sizerTC3.Add(self.tc31, 0, flagText, VSEP_RIGHT)
        sizerP0RightTop.Add(sizerTC3, 0, flagText, VSEP_RIGHT)

        sizerTC4 = wx.BoxSizer(wx.HORIZONTAL)
        sizerTC4.Add(self.tc4, 0, flagText, VSEP_RIGHT)
        sizerTC4.Add(self.tc41, 0, flagText, VSEP_RIGHT)
        sizerP0RightTop.Add(sizerTC4, 0, flagText, VSEP_RIGHT)

        sizerP0RightBottom = wx.StaticBoxSizer(self.frame_storage_data, wx.VERTICAL)
        sizerP0RightBottom.Add(self.tcMCStorage, 0, flagText, VSEP_RIGHT)
        sizerP0RightBottom.Add(self.tcVStorage, 0, flagText, VSEP_RIGHT)
        sizerP0Right.Add(sizerP0RightTop, 0, wx.EXPAND, 0)
        sizerP0Right.Add(sizerP0RightBottom, 0, wx.EXPAND, 0)

        # P1H: data for source
        sizerP1H = wx.StaticBoxSizer(self.frame_source, wx.HORIZONTAL)

        sizerP1Hdata = wx.BoxSizer(wx.VERTICAL)

        sizerP1HdataRow1 = wx.BoxSizer(wx.HORIZONTAL)
        sizerP1HdataRow1.Add(self.entryThoNom, 0, flagText, VSEP_RIGHT)
        sizerP1HdataRow1.Add(self.entryThiNom, 0, flagText, VSEP_RIGHT)
        sizerP1Hdata.Add(sizerP1HdataRow1, 0, flagText, VSEP_RIGHT)

        sizerP1HdataRow2 = wx.BoxSizer(wx.HORIZONTAL)
        sizerP1HdataRow2.Add(self.entryThoAvg, 0, flagText, VSEP_RIGHT)
        sizerP1HdataRow2.Add(self.entryThiAvg, 0, flagText, VSEP_RIGHT)
        sizerP1Hdata.Add(sizerP1HdataRow2, 0, flagText, VSEP_RIGHT)

##        sizerP1HButtons = wx.BoxSizer(wx.HORIZONTAL)
##        sizerP1HButtons.Add(self.buttonAddSource, 0, flagText, VSEP_RIGHT)
##        sizerP1HButtons.Add(self.buttonDeleteSource, 0, flagText, VSEP_RIGHT)

        sizerP1HListBox = wx.BoxSizer(wx.VERTICAL)
        sizerP1HListBox.Add(self.listBoxSource, 0, wx.ALIGN_LEFT, VSEP_RIGHT)
##        sizerP1HListBox.Add(sizerP1HButtons, 0, wx.ALIGN_LEFT, VSEP_RIGHT)

        sizerP1H.Add(sizerP1HListBox, 10, wx.ALIGN_LEFT | wx.EXPAND, VSEP_RIGHT)
        sizerP1H.Add(sizerP1Hdata, 0, wx.ALIGN_RIGHT, VSEP_RIGHT)
        sizerP0Right.Add(sizerP1H, 0, wx.EXPAND, 0)

        # P1C: data for sink
        sizerP1C = wx.StaticBoxSizer(self.frame_sink, wx.HORIZONTAL)

        sizerP1Cdata = wx.BoxSizer(wx.VERTICAL)

        sizerP1CdataRow1 = wx.BoxSizer(wx.HORIZONTAL)
        sizerP1CdataRow1.Add(self.entryTciNom, 0, flagText, VSEP_RIGHT)
        sizerP1CdataRow1.Add(self.entryTcoNom, 0, flagText, VSEP_RIGHT)
        sizerP1Cdata.Add(sizerP1CdataRow1, 0, flagText, VSEP_RIGHT)

        sizerP1CdataRow2 = wx.BoxSizer(wx.HORIZONTAL)
        sizerP1CdataRow2.Add(self.entryTciAvg, 0, flagText, VSEP_RIGHT)
        sizerP1CdataRow2.Add(self.entryTcoAvg, 0, flagText, VSEP_RIGHT)
        sizerP1Cdata.Add(sizerP1CdataRow2, 0, flagText, VSEP_RIGHT)

##        sizerP1CButtons = wx.BoxSizer(wx.HORIZONTAL)
##        sizerP1CButtons.Add(self.buttonAddSink, 0, flagText, VSEP_RIGHT)
##        sizerP1CButtons.Add(self.buttonDeleteSink, 0, flagText, VSEP_RIGHT)

        sizerP1CListBox = wx.BoxSizer(wx.VERTICAL)
        sizerP1CListBox.Add(self.listBoxSink, 0, wx.ALIGN_LEFT, VSEP_RIGHT)
##        sizerP1CListBox.Add(sizerP1CButtons, 0, wx.ALIGN_LEFT, VSEP_RIGHT)

        sizerP1C.Add(sizerP1CListBox, 10, wx.ALIGN_LEFT, VSEP_RIGHT)
        sizerP1C.Add(sizerP1Cdata, 0, wx.ALIGN_RIGHT, 0)
        sizerP0Right.Add(sizerP1C, 0, wx.EXPAND, 0)

        sizerPage0.Add(sizerP0Right, 5, wx.EXPAND | wx.TOP, VSEP_RIGHT)
        self.page0.SetSizer(sizerPage0)

        sizerPage2 = wx.StaticBoxSizer(self.frame_economic_parameters,wx.VERTICAL)
        sizerPage2.Add(self.entryTurnKeyPrice, 0, flagText, VSEP_RIGHT)
        sizerPage2.Add(self.entryOMFix, 0, flagText, VSEP_RIGHT)
        sizerPage2.Add(self.entryOMVar, 0, flagText, VSEP_RIGHT)
        self.page2.SetSizer(sizerPage2)

        sizerOKCancel = wx.BoxSizer(wx.HORIZONTAL)
        sizerOKCancel.Add(self.activeObject, 0, wx.ALL | wx.EXPAND, 2)
        sizerOKCancel.Add(self.buttonCancel, 0, wx.ALL | wx.EXPAND, 2)
        sizerOKCancel.Add(self.buttonOK, 0, wx.ALL | wx.EXPAND, 2)

        sizerGlobal.Add(self.notebook, 3, wx.EXPAND, 0)
        sizerGlobal.Add(sizerOKCancel, 0, wx.TOP | wx.ALIGN_RIGHT| wx.EXPAND, 0)
        self.SetSizer(sizerGlobal)
        self.Layout()


#------------------------------------------------------------------------------
# GUI actions - 0. OK/Cancel buttons
#------------------------------------------------------------------------------
    def editAllowed(self):
        """Return True if the data may be modified in the current state,
        inform the user and return False if not.

        """
        if Status.ANo <= 0 and Status.StatusCC > 0:
            showWarning(_U("You cannot modify the heat exchangers after the data has been checked.\n") +
                        _U("If you want to change the present state you may unblock the data by ") +
                        _U("choosing to confirm the deletion ") +
                        _U("of any alternative proposals in this project the next time you switch to the present state."))
            return False
        else:
            return True

    def OnButtonCancel(self, event):
        self.mod.restoreConnections()
        self.clearHXEntryFields()
        self.mod.cancel()
        self.prnt.Close()

    def OnButtonOK(self, event):
        logTrack("PanelQ6.OnButtonOK")
        if not self.editAllowed():
            return

        hxName = self.tc1.GetValue()

        # assure that a name has been entered before continuing
        if len(hxName) == 0 or hxName is None:
            showWarning(_("You have to enter a name for the new heat exchanger before saving"))
            return

        if self.hx is None: # no heat exchanger selected
            showWarning(_("No heat exchanger selected"))
            return

        elif hxName == self.hx.name: # parameters of current hx changed
            pass

        elif hxName in self.mod.HXNames: # new name assigned that is already in the list of names
            showError("HX name has to be a unique value!")
            return

        self.hx.MCStorage = self.tcMCStorage.GetValue()


        self.hx.name = hxName
##        self.hx.type = findKey(TRANSHXTYPES, toUTF8(self.tc2.GetValue(text=True)))
##            "QdotHX":check(self.tc31.GetValue()),
        self.hx.lmtdNom = self.tc4.GetValue()
##            "HXLMTD":check(self.tc41.GetValue()),
        newUA = self.tcUA.GetValue()
        if self.hx.UA is not None and (newUA is None or abs(newUA - self.hx.UA) > 0.01) and Status.ANo > 0:   # change in UA-Value
            self.tc3.SetValue(None)
            self.tc5.SetValue(None)
            ret = askConfirmation(_("UA value changed. Do you want to reestimate the economic parameters ?"))
            if ret == wx.ID_YES:
                self.entryTurnKeyPrice.SetValue(None)
                self.entryOMFix.SetValue(None)
                self.entryOMVar.SetValue(None)
            wx.SafeYield()

        self.hx.dotQNom = self.tc3.GetValue()
        self.hx.QHX = self.tc5.GetValue()
        self.hx.turnKeyPrice = self.entryTurnKeyPrice.GetValue()
        self.hx.OMFix = self.entryOMFix.GetValue()
        self.hx.OMVar = self.entryOMVar.GetValue()
        self.hx.UA = self.tcUA.GetValue()
##        self.hx.area = self.tcArea.GetValue()

        if self.hx.UA is not None and self.hx.lmtdNom is not None and self.hx.dotQNom is not None:
            err = self.hx.dotQNom - self.hx.UA*self.hx.lmtdNom

            if abs(err) > max(1.e-2*self.hx.dotQNom, EPSILON):
                logError(_U("QdotNom (%s) is not equal to LMTDNom*UA (%s)"%(self.hx.dotQNom, self.hx.UA*self.hx.lmtdNom)))
                return

        if len(self.hx.sourcestreams) <= 0:
            showWarning(_U("No source defined for heat exchanger %s" % (self.hx.name,)))
            return

        if len(self.hx.sinkstreams) <= 0:
            showWarning(_U("No sink defined for heat exchanger %s" % (self.hx.name,)))
            return

        self.hx.writeToDB()
        for hx in self.mod.hxDS:
            hx.writeToDB() # save also downstream heat exchanger
        self.mod.hxDS = []

        Status.int.changeInHeatRecovery()
        Status.int.systemModel.build()
        h = self.hx.HXNo-1
        self.hx = Status.int.hx[h]  # reload hx after system rebuilding
        self.mod.selectHX(self.hx)

##        self.fillPage()
##        self.showHX()
##        self.showConnections()

        if Status.ANo <> 0: # we modified an "real" alternative proposal
            h = self.hx.HXNo-1
            conflicts = Status.mod.moduleCC.hxCheck(h)
            Status.mod.moduleCC.initPanel()
            if conflicts > 0:
                self.cf = conflictFrame(self)
                self.cf.Show()
            else:
                self.hx.loadFromDB(loadConnections=False)
                self.display()

        Status.prj.projectChanged()

        self.mod.new = False
        self.prnt.Close()


#------------------------------------------------------------------------------
# GUI actions - 1. Heat exchanger page
#------------------------------------------------------------------------------


    def OnListBoxSourceClick(self, event):
        if not self.allowListBoxEvents: return
##        print "OnListBoxSourceClick"
        index = self.listBoxSource.GetSelection()
        self.mod.selectSource(index)

    def OnListBoxSourceDClick(self, event):
        if not self.allowListBoxEvents: return
##        print "OnListBoxSourceDClick"
        index = self.listBoxSource.GetSelection()
        self.mod.selectSource(index)

        self.mod.editConnection(heating=False)
        editor = EditConnection(self)
        if editor.ShowModal() == wx.ID_OK:
            self.showConnections()

    def OnListBoxSinkClick(self, event):
        if not self.allowListBoxEvents: return
##        print "OnListBoxSinkClick"
        index = self.listBoxSink.GetSelection()
        self.mod.selectSink(index)

    def OnListBoxSinkDClick(self, event):
        if not self.allowListBoxEvents: return
##        print "OnListBoxSinkDClick"
        index = self.listBoxSink.GetSelection()
        self.mod.selectSink(index)

        self.mod.editConnection(heating=True)
        editor = EditConnection(self)
        if editor.ShowModal() == wx.ID_OK:
            self.showConnections()




#..............................................................
# 1b. Change of main heat exchanger parameters

    def fillPage(self):
        """ Screens the SQL tables and fills the lists of HX's and WHEE's"""
##        fillChoice(self.tc2.entry, TRANSHXTYPES.values())

        permit = self.allowListBoxEvents
        self.allowListBoxEvents = False #avoid that clear commands activate listBoxHX and listBoxWHEE event handlers


        if self.hx is None and len(Status.int.hx) > 0:
            self.hx = Status.int.hx[0]
            self.mod.selectHX(self.hx)

        if self.hx is not None:
            name = self.hx.name
            self.activeObject.SetLabel(U(name))
            Globals.GUI.editWindow.SetTitle(U(name))   # set edit window title

        self.allowListBoxEvents = permit

    def showHX(self):
        """Show the data of the currently selected heat exchanger."""

        if self.hx is not None:
            self.tc1.SetValue(self.hx.name)

##            self.tc2.SetValue(0)
##            if str(self.hx.type) in TRANSHXTYPES.keys():
##                self.tc2.SetValue(TRANSHXTYPES[str(self.hx.type)])

            self.tc3.SetValue(str(self.hx.dotQNom))
            self.tc31.SetValue(str(self.hx.dotQAvg))
            self.tc4.SetValue(str(self.hx.lmtdNom))
            self.tc41.SetValue(str(self.hx.lmtdAvg))
            self.tc5.SetValue(str(self.hx.QHX))
            self.tcUA.SetValue(str(self.hx.UA))
##            self.tcArea.SetValue(str(self.hx.area))

            self.tcMCStorage.SetValue(str(self.hx.MCStorage))

            if self.hx.MCStorage is not None:
                VStorage = self.hx.MCStorage*(3.6/4.19)
            else:
                VStorage = None
            self.tcVStorage.SetValue(str(VStorage))

            self.entryTciAvg.SetValue(self.hx.TciAvg)
            self.entryTcoAvg.SetValue(self.hx.TcoAvg)
            self.entryThiAvg.SetValue(self.hx.ThiAvg)
            self.entryThoAvg.SetValue(self.hx.ThoAvg)

            self.entryTciNom.SetValue(self.hx.TciNom)
            self.entryTcoNom.SetValue(self.hx.TcoNom)
            self.entryThiNom.SetValue(self.hx.ThiNom)
            self.entryThoNom.SetValue(self.hx.ThoNom)

            self.entryTurnKeyPrice.SetValue(self.hx.turnKeyPrice)
            self.entryOMFix.SetValue(self.hx.OMFix)
            self.entryOMVar.SetValue(self.hx.OMVar)


            h = self.hx.HXNo - 1
            try:
                checkHX = Status.mod.moduleCC.ccHX[h]
            except:
##                logWarning("No check object available for HX %s"%h)
                checkHX = None

            if checkHX is not None:
                ccPars = PanelCCPars({"QdotHXNom":(checkHX.QdotHXNom, self.tc3),
                                      "QdotHX":(checkHX.QdotHX,self.tc31),
                                      "HXLMTDNom":(checkHX.HXLMTDNom, self.tc4),
                                      "HXLMTD":(checkHX.HXLMTD, self.tc41),
                                      "QHX":(checkHX.QHX, self.tc5),
                                      "UA":(checkHX.UA, self.tcUA),
                                      "TurnKeyPrice":(checkHX.TurnKeyPrice, self.entryTurnKeyPrice),
                                      "OMFix":(checkHX.OMFix, self.entryOMFix),
                                      "OMVar":(checkHX.OMVar, self.entryOMVar)
##                                      "Area":(checkHX.Area, self.tcArea),
                                     },
                                     dataGroup = ("HX[" + str(h+1) + "]"))
                ccPars.displayCC()


    def showConnections(self):
        """ updates information of connections

        """
        self.allowListBoxEvents = False

        self.listBoxSource.Clear()
        self.listBoxSink.Clear()
        if self.hx is None:
            return

        if len(self.hx.sourcestreams)==0:
            self.listBoxSource.Append(_U("not connected"))
            
        for conn in self.hx.sourcestreams:
            dconn = conn.compactConnToUpstreamComponent() # splitters are not shown explicitely
            displayName = noneCheck(dconn.name, "---")
            if dconn.fraction < 1.0 - EPSILON:
                displayName += " [%4.3f]"%dconn.fraction

            self.listBoxSource.Append(U(displayName))

        if self.mod.selectedSourceNo is None:
            self.mod.selectSource(0)

        if self.mod.selectedSourceNo is not None:
            self.listBoxSource.SetSelection(int(self.mod.selectedSourceNo))

        if len(self.hx.sinkstreams)==0:
            self.listBoxSink.Append(_U("not connected"))

        for conn in self.hx.sinkstreams:
            dconn = conn.compactConnToUpstreamComponent()      # splitters are not shown explicitely
            displayName = noneCheck(dconn.name, "---")
            if dconn.fraction < 1.0 - EPSILON:
                displayName += " [%4.3f]"%dconn.fraction

            self.listBoxSink.Append(U(displayName))

        if self.mod.selectedSinkNo is None:
            self.mod.selectSink(0)

        if self.mod.selectedSinkNo is not None:
            self.listBoxSink.SetSelection(self.mod.selectedSinkNo)

        wx.SafeYield()
        self.allowListBoxEvents = True


    def OnNotebookPageChanged(self, event):
        pass

    def display(self, h=None):
##        if h is not None:
##            self.hx = Status.int.hx[h]
##            self.mod.selectHX(self.hx)

        self.clear()
        self.mod.updatePanel()
        self.hx = self.mod.hx
        self.fillPage()
        self.showHX()
        self.showConnections()

        self.OnNotebookPageChanged(None)

        self.Show()

    def clear(self):
        self.clearHXEntryFields()

    def clearHXEntryFields(self):
        self.tc1.SetValue('')
#        self.tc2.SetValue('')
        self.tc3.SetValue('')
        self.tc31.SetValue('')
        self.tc4.SetValue('')
        self.tc41.SetValue('')
        self.tc5.SetValue('')
        self.tcUA.SetValue('')
##        self.tcArea.SetValue('')
        self.tcMCStorage.SetValue('')
        self.tcVStorage.SetValue('')
        self.tc5.Enable(Status.ANo == -1)

        self.entryTurnKeyPrice.SetValue('')
        self.entryTurnKeyPrice.Enable(Status.ANo > 0)
        self.entryOMFix.SetValue('')
        self.entryOMVar.SetValue('')

        self.listBoxSource.Clear()
        self.listBoxSink.Clear()

    def goTo(self, key):
        """ tries to open the page appropriate
        for a given parameter

        :params no: number of item in the list to be selected (from 0 to n-1)
        :params key: shortname of variable to be displayes
        """
        # then open the right page in the panel
        parList = {
            "HXName" : 0,
            "HXType" : 0,
            "QdotHX" : 0,
            "QdotHXNom" : 0,
            "HXLMTD" : 0,
            "HXLMTDNom" : 0,
            "UA" : 0,
            "QHX" : 0,
            "HXSourceInlet" : 0,
            "HXTSourceInlet" : 0,
            "HXhSourceInlet" : 0,
            "HXTSourceOutlet" : 0,
            "HXhSourceOutlet" : 0,
            "HXSink" : 0,
            "HXTSinkInlet" : 0,
            "HXTSinkOutlet" : 0,
            "TurnKeyPrice" : 1,
            "OMFix" : 1,
            "OMVar" : 1,
        }

        if key in parList:
            panelNo = parList[key]
            self.notebook.SetSelection(panelNo)
            return True
        else:
            return False

#=====================================================================================================

class EditConnection(wx.Dialog):
    """ editor for sink or source connections
    """

    def __init__(self, parent):
        """

        conn is the active connection to be edited
        the flag heating has to be assigned, in order to know whether a source or sink is edited

        """

        wx.Dialog.__init__(self, id=-1, name='Q6Edit', parent=parent,
              pos=wx.Point(400, 400),
              size = (500, 180),
              style=wx.DEFAULT_DIALOG_STYLE, title=_U('Edit connection'))

        self.parent = parent

        self._init_ctrls(parent)
        self.__do_layout()

        self.allowListBoxEvents = False

        self.displayConn()

        wx.SafeYield() # block events sufficiently long time

        self.allowListBoxEvents = True


    def _init_ctrls(self, parent):

        self.buttonOK = wx.Button(id=wx.ID_OK, label=_U('OK'),
              name='buttonOK', parent=self, size=wx.Size(100, 24), style=0)
        self.buttonOK.Bind(wx.EVT_BUTTON, self.OnButtonOKButton,
              id=wx.ID_OK)

        self.buttonCancel = wx.Button(id=wx.ID_CANCEL,
              label=_U('Cancel'), name='buttonCancel', parent=self,
              size=wx.Size(100, 24), style=0)


        self.buttonCancel.Bind(wx.EVT_BUTTON, self.OnButtonCancelButton,
              id=wx.ID_CANCEL)


        fs = FieldSizes(wHeight=HEIGHT_EDITCON, wLabel=LABEL_WIDTH_EDITCON_LEFT,
                       wData=DATA_ENTRY_WIDTH_EDITCON_LEFT, wUnits=UNITS_WIDTH)

        # Type selector: streamType / streamSum / streamSum -> equivalent to objectType / objectNo and componentNo in HXConnection class
        UOBJECTTYPES = [U(TRANSOBJECTTYPES_HX[key]) for key in OBJECTTYPES_HX]

        self.cbObjectType = ChoiceEntry(self,
                                        values=UOBJECTTYPES,
                                        label=_U("Type of Sink/Source"),
                                        tip=_U("Specify the type of object: process, equipment, pipe/duct, heat exchanger, electrical equipment"))

        self.Bind(wx.EVT_CHOICE, self.OnCBObjectTypeChange, self.cbObjectType.entry)


        self.cbObject = ChoiceEntry(self,
                                    values=[],
                                    label=_U("Sink/Source object"),
                                    tip=_U("Select the object from the list"))

        self.Bind(wx.EVT_CHOICE, self.OnCBObjectChange, self.cbObject.entry)

        self.cbStream = ChoiceEntry(self,
                                    values=[],
                                    label=_U("Sink/Source stream"),
                                    tip=_U("Select the stream from the list"))

        self.Bind(wx.EVT_CHOICE, self.OnCBStreamChange, self.cbStream.entry)

        self.entryFraction = FloatEntry(self,
                                        decimals=3, minval=0, maxval=100, value=1.,
                                        unitdict='FRACTION',
                                        label = _U("Fraction of mass flow"),
                                        tip = _U("Indicate the fraction of the mass flow from the source / sink object"))


    def __do_layout(self):

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.cbObjectType, 10, wx.ALL|wx.EXPAND, 2)
        sizer.Add(self.cbObject, 10, wx.ALL|wx.EXPAND, 2)
        sizer.Add(self.cbStream, 10, wx.ALL|wx.EXPAND, 2)
        sizer.Add(self.entryFraction, 10, wx.ALL|wx.EXPAND, 2)

        sizerButtons = wx.BoxSizer(wx.HORIZONTAL)
        sizerButtons.Add(self.buttonOK, 10, wx.ALL| wx.ALIGN_RIGHT, 2)
        sizerButtons.Add(self.buttonCancel, 10, wx.ALL|wx.ALIGN_RIGHT, 2)

        sizer.Add(wx.Size(100, 5), 0, wx.EXPAND, 0)
        sizer.Add(sizerButtons, 10, wx.ALL| wx.ALIGN_RIGHT| wx.ALIGN_TOP, 2)
        sizer.Add(wx.Size(100, 5), 0, wx.EXPAND, 0)


        self.SetSizer(sizer)
        self.Layout()

    def displayConn(self):
        """ fills the lists of the selectors and selects the appropriate entry in
        order to display conn
        """

        conn = self.parent.mod.activeConn
        dconn = conn.compactConnToUpstreamComponent()

        oType, oName, oStream = self.parent.mod.getObjectDescriptor(conn)

##        print "EditConnection.displayConn: displaying oType %s oName %s oStream %s"%(oType, oName, oStream)

        self.cbObjectType.SetValue(OBJECTTYPES_HX.index(oType) if oType in OBJECTTYPES_HX else None)

        objectList, index = self.parent.mod.getObjectList()
        self.cbObject.SetValue(objectList)
        self.cbObject.SetValue(index)
        
        streamList, index = self.parent.mod.getStreamList()
        self.cbStream.SetValue(streamList)
        if index is None and len(streamList)==1:
            self.cbStream.SetValue(0)   # if there's only one, select this one
        else:
            self.cbStream.SetValue(index)

##        print "EditConnection.displayConn: writing fraction %s"%conn.fraction
        self.entryFraction.SetValue(dconn.fraction)


    def OnCBObjectTypeChange(self, event):
        """ Selection of object type: PROCESS - DISTRIBUTION - EQUIPMENT - WHEE
        Depending on the type selected fills the choice of streams.
        """
        if not self.allowListBoxEvents:
            return

        index = self.cbObjectType.GetValue()
        changed = self.parent.mod.setObjectType(index)

        if changed:
            objectList, dummy = self.parent.mod.getObjectList()

            self.cbObject.SetValue(objectList)

            # if there is only one object of a given type, select this one
            self.cbObject.SetValue(0 if len(objectList)==1 else None)
            self.OnCBStreamChange(None)

            self.entryFraction.SetValue(1.0)

    def OnCBObjectChange(self, event):
        """ Activate on a change of the selected object
        Builds stream list for the corrsesponding object
        """
        if not self.allowListBoxEvents:
            return

        index = self.cbObject.GetValue()
        changed = self.parent.mod.selectObject(index)

        if changed:
            streamList, index = self.parent.mod.getStreamList()
            self.cbStream.SetValue(streamList)
            self.cbStream.SetValue(0 if len(streamList) == 1 else None)
            self.OnCBStreamChange(None)

            self.entryFraction.SetValue(1.0)

    def OnCBStreamChange(self, event):

        if not self.allowListBoxEvents:
            return

        index = self.cbStream.GetValue()
        self.parent.mod.setActiveConn(index)



    def OnButtonOKButton(self, event):

        fraction = self.entryFraction.GetValue()

        if self.parent.mod.storeConnection(fraction):
            if self.IsModal(): self.EndModal(wx.ID_OK)
        else:
            showWarning("Connection not fully defined. Check your entry")

    def OnButtonCancelButton(self, event):
        self.EndModal(wx.ID_CANCEL)

