# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelQM: data entry for monitoring data
#
#==============================================================================
#
#   EINSTEIN Version No.: 2.5
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from GUI.einsteinPanel import EinsteinPanel
from GUI.dataEntry.tableEntry import TableEntry, DEFAULTMENU
from GUI.displayClasses import FloatEntry
from auxiliary.auxiliary import toGettextAndUnicode as _U
from modules.guiModules.moduleQM import ModuleQM

import wx

class PanelQM(EinsteinPanel):

    def __init__(self, parent, main):

        # generate list of notebook pages and frames
        layout = {"notebook": ["Monitoring Data"],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(0, _("Data Sets"))]}           # list of frames (pageno, title). if empty, no frame is created
        super(PanelQM, self).__init__(parent, main, layout)

        self.mod = ModuleQM()

        # add panel specific entries
        titles = [_("Parameter"), _("Object"), _("File Name"), _("Column"), _("Scale"), _("Type"), _("Min / Max")]

        self.addEntry("QMData",
                      TableEntry(self.nbPage[0], titles, self.mod.changeData, size=wx.Size(760, 400),
                                 rowSize=48, colSize=120,
                                hAlignment = [wx.ALIGN_CENTRE, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE],
                                readOnly = [False, False, False, False, False, True, True],),
                      0)

        self.entry["QMData"].setColSize(0, 150)
        self.entry["QMData"].setColSize(2, 160)
        self.entry["QMData"].setColSize(4, 100)
        self.entry["QMData"].setColSize(5, 100)
        self.entry["QMData"].setPopUpMenu(DEFAULTMENU + [_U("Load new datafile")])
        

        # conclude layout
        self.conclude_layout()


    def fillPanel(self):
        data, choices = self.mod.getTable()
        self.entry["QMData"].display(data=data, choices=choices)





