#Boa:Frame:Frame1
# -*- coding: utf-8 -*-
#==============================================================================#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	panelReport- Component for report generation
#			
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
#
# Description of the process of report generation.
#
# 1.  The master document areas subject to change are marked with 'names',
#     which should be related to the keys to the data for the panels.
#     i.e. panelEA2 uses the key 'EA2', so the name will be 'KEY_EA2' (OO does
#     not accept simply 'EA2', so I added the ' KEY_' prefix.). These names
#     are saved in the document (contents.xml) and can be read from there.
#
# 1.1   Creation of a name.
#       a. A region is selected in the sheet.
#       b. Main menu Insert -> Names ->Define (or just Ctrl-F3) will open a dialog
#          where the name is written. The area is taken from the selection. After
#          writing the new name, press Add.
#
# 2.  When the Generate Report option is activated, this module (reporter.py) starts
#     to execute. The steps are:
#
# 2.1   The contents.xml component of the master document (Master) is read and stored
#       as a DOM tree in memory.
#
# 2.2   The names are extracted, together with the ranges they point to. Only the names
#       that start with the prefix 'KEY_' are kept. The prefix is discarded and the
#       rest (that will be the same as the key to the data in Interfaces.GData) is saved
#       as the key to a dictionary entry. The data is the range information. The
#       dictionaries 'names' and 'sheetnames' manage different views of this information.
#
# 2.2   The names dictionary is saved as a XML document, as a reference and debugging
#       aid.
#
# 2.3   Now contents.xml is scanned sequentially looking for worksheets and ranges
#       that exist in the names dictionary. When a correspondence is found, the data
#       is taken from Interfaces using the name (without the prefix) as key. Then
#       the valued are stored in the tree.
#
# 3.  Finally, the DOM tree is saved as a new OO document, which is the generated report.
#
#########################################################################################

import os
import wx
import numpy as np
from GUI.status import *
from GUI.dialogOK import DialogOK
import constants as c
from dialogGauge import DialogGauge
from auxiliary.auxiliary import toGettextAndUnicode as _U
from GUI.GUITools import TITLE_COLOR
from database.dbio import check
import database.dbio as dbio
from globals import Globals

from modules.messageLogger import (logMessage, logTrack, showError,
                                            logDebug)
from modules.control import prepareDataForReport

from modules.interfaces import Interfaces

from lib.oo.oo import OODocument

#
# constants
#
PREFIX      = "KEY_"
NEWREPORT   = "New_EINSTEIN_Report.ods"
NEWDIR      = c.BASEDIR + "/reports"


wxID_BTNSELECTREPORT = wx.NewId()
wxID_PANELBUTTONFWD = wx.NewId()
wxID_PANELBUTTONBACK = wx.NewId()
    
class PanelReport(wx.Panel):
    def _init_ctrls(self, prnt):
        # generated method, don't edit
        wx.Panel.__init__(self, id=-1, name='', parent=prnt,
              pos=wx.Point(0,0), size=wx.Size(800, 600))
        self.SetClientSize(wx.Size(800, 600))

#..............................................................................
# Description of selected industry

        self.box1 = wx.StaticBox(id=-1,
              label=_U('Project Summary'),
              name='box1', parent=self, pos=wx.Point(10, 10),
              size=wx.Size(780, 520), style=0)
        self.box1.SetForegroundColour(TITLE_COLOR)
        self.box1.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.tc1 = wx.TextCtrl(id=-1, name='tc1',
              parent=self, pos=wx.Point(20, 40), size=wx.Size(760, 480),
                               style=wx.TE_MULTILINE | wx.TE_LINEWRAP, value="")

#------------------------------------------------------------------------------		
#       Button for report selection
#------------------------------------------------------------------------------		

        self.btnSelectReport = wx.Button(id=wxID_BTNSELECTREPORT,
              label=_U('Select report'), name=u'btnSelectReport', parent=self,
              pos=wx.Point(10, 560), size=wx.Size(150, 20), style=0)
        self.btnSelectReport.Bind(wx.EVT_BUTTON, self.OnBtnSelectReport,
              id=wxID_BTNSELECTREPORT)

#------------------------------------------------------------------------------		
#       Default action buttons: FWD / BACK / OK / Cancel
#------------------------------------------------------------------------------		

        self.buttonOk = wx.Button(id=wx.ID_OK, label=_U('OK'),
              name='buttonOk', parent=self, pos=wx.Point(528, 560),
              size=wx.Size(75, 20), style=0)

        self.buttonOk.Bind(wx.EVT_BUTTON,
              self.OnButtonOkButton,
              id=wx.ID_OK)

        self.buttonCancel = wx.Button(id=wx.ID_CANCEL, label=_U('Cancel'),
              name='buttonCancel', parent=self, pos=wx.Point(616,
              560), size=wx.Size(75, 20), style=0)

        self.buttonCancel.Bind(wx.EVT_BUTTON,
              self.OnButtonCancelButton,
              id=wx.ID_CANCEL)

        self.buttonFwd = wx.Button(id=wxID_PANELBUTTONFWD,
              label='>>>', name='buttonFwd', parent=self,
              pos=wx.Point(704, 560), size=wx.Size(75, 20), style=0)

        self.buttonFwd.Bind(wx.EVT_BUTTON,
              self.OnButtonFwdButton,
              id=wxID_PANELBUTTONFWD)

        self.buttonBack = wx.Button(id=wxID_PANELBUTTONBACK,
              label='<<<', name='buttonBack', parent=self,
              pos=wx.Point(440, 560), size=wx.Size(75, 20), style=0)

        self.buttonBack.Bind(wx.EVT_BUTTON,
              self.OnButtonBackButton,
              id=wxID_PANELBUTTONBACK)

    def __init__(self, parent, main):
        self.parent = parent
        self.main = main
        self._init_ctrls(parent)
        self.newreport = None # where to save the new report to

        self.display()
        
    #
    # GUI events
    #
    def OnBtnSelectReport(self, event):
        #
        # Open an existing OpenDocument workbook
        # for processing
        #
        global NEWDIR # remember the last dir on recurring uses of the dialog
        dialog = DialogOK(parent=None, title=_U("Report generation"), 
                          message=_U("You have not selected any favorite proposal, "\
                                     "is this intended?"))
        if Status.FinalAlternative <= 0 and dialog.ShowModal() != wx.ID_OK:
            self.Hide()
            self.main.tree.SelectItem(self.main.qA, select=True)

        elif hasattr(Globals, "robbie") and Globals.robbie: # operation by robbie robot

            realBaseDir = os.path.abspath(c.BASEDIR + "/../../..")       
            self.master = realBaseDir + "/reports/EINSTEIN_report_MASTER_V2.0.ods"
            self.newreport = c.APPDATA + "/EINSTEIN_Report_" + Status.ActiveProjectName + ".ods"
            self.generateReport()
            
        else:
            dialog = wx.FileDialog(parent=None,message=_U('Choose a master report file'),
                                   defaultDir=os.path.join(c.BASEDIR,  'reports'),
                                   wildcard='Open Office spreadsheets files (*.ods)|*.ods',
                                   style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST | wx.FD_PREVIEW)
            if dialog.ShowModal() == wx.ID_OK:
                self.master = dialog.GetPath()
                if self.master.strip() != "":
                    dialog = wx.FileDialog(parent=None,message=_U('Save report file as'),
                                   defaultDir=os.path.abspath(NEWDIR),
                                   defaultFile="EINSTEIN_Report_" + Status.ActiveProjectName + ".ods",
                                   wildcard='Open Office spreadsheets files (*.ods)|*.ods',
                                   style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
                    if dialog.ShowModal() == wx.ID_OK:
                        NEWDIR = dialog.GetDirectory()
                        self.newreport = os.path.abspath(dialog.GetPath())
                        self.generateReport()
                else:
                    self.master = None

            event.Skip()

#------------------------------------------------------------------------------		
    def display(self, no=None):
        """ obtains data from SQL and displays in the panel
        """		

        sproject = Status.prj.getProjectStatus()
        if sproject is not None:
            if sproject.Summary is not None:
                summary = unicode(sproject.Summary,"utf-8")
                self.tc1.SetValue(summary)

        self.Show()
        
#------------------------------------------------------------------------------		
#   Default event handlers: FWD / BACK / OK / Cancel - Buttons
#------------------------------------------------------------------------------		

    def OnButtonBackButton(self, event):
#        self.Hide
#        self.main.tree.SelectItem(self.main.qHC, select=True)
        event.Skip()

    def OnButtonFwdButton(self, event):
#        self.Hide
#        self.main.tree.SelectItem(self.main.qHP, select=True)
        event.Skip()

    def OnButtonOkButton(self, event):
        summary = self.tc1.GetValue()

        logMessage(_U("summary of project report updated"))

        sproject = Status.prj.getProjectStatus()
        if sproject is not None:
            sproject.Summary = check(summary)
            dbio.commit()
        
#        self.Hide()
#        self.main.tree.SelectItem(self.main.qHC, select=True)
        if event: event.Skip()

    def OnButtonCancelButton(self, event):
        logTrack("Button exitModuleCancel: CANCEL Option not yet foreseen")

#------------------------------------------------------------------------------		
#------------------------------------------------------------------------------		
#------------------------------------------------------------------------------		
    #
    # public instance methods
    #
    def generateReport(self):
        
        self.main.logMessage('Generating report from ' + self.master)

        try:
            prepareDataForReport()
        except:
            showError(_U("The data collection for the report has failed. "\
                         "Look into the logs in %s "\
                         "if you want more information about the error.") %
                         (os.path.join(c.APPDATA, c.APPNAME), ))
            raise
        else:
    
            dlg = DialogGauge(None,_U("Report generation"),_U("reading master document"))
            self.doc = OODocument(self.master)
            self.doc.readMaster()    
            dlg.Destroy()
            
            # replace values
            self.replaceData()

            # write and zip the output
            dlg = DialogGauge(None,_U("Report generation"),_U("storing report file"))
            self.doc.writeContentFile()

            dlg.update(30)

            self.doc.buildNewDocument(self.newreport)
            dlg.Destroy()         
            
            self.main.showInfo(_U('Report generation is finished\n'
                                 'The generated report is:') + self.newreport)

    def replaceData(self):
        #
        # Scan the sheet names list and change values on each
        # sheet on the list
        #
        dlg = DialogGauge(None,"Report generation","writing report")

        docKeys = self.doc.names.keys()
        nRanges = len(docKeys)

        for i, key in enumerate(docKeys):
            if not Interfaces.GData.has_key(key):
                # print "PanelReport: no data available for key %s"%thename
                continue

            data = Interfaces.GData[key]
            # print "panelReport: data block found. sheet:%s name:%s row %s col %s" %\
            # (sheetname,thename,nrow,ncol)
            try:
                (datarows,datacols) = np.shape(data)
            except:
                logDebug("PanelReport (replaceData): no data shape found for key %s"%thename)
                continue
 
            self.doc.modifyCellRange(key, data)

            dlg.update(100.0 * (i+1) / nRanges)
        dlg.Destroy()


