# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelCS2: hourly data
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from GUI.einsteinPanel import EinsteinPanel
from GUI.dataEntry.tableEntry import TableEntry
from GUI.displayClasses import ChoiceEntry
from lib.GUI.plot.barPlot import BarPlot

from constants import HEATING_COOLING
from GUI.status import Status
from modules.moduleCS import ModuleCS

import wx

class PanelCS2(EinsteinPanel):

    def __init__(self, parent, main):

        # generate list of notebook pages and frames
        layout = {"notebook": [],         # list of notebook-page titles. if empty or None, no notebook is created
                  "frames": [(None, _(" ")),
                             (None, _('Useful process and supply heat / cooling (UPH/C and USH/C)')),
                             (None, _('Relative comparison'))]}           # list of frames (pageno, title). if empty, no frame is created
                                                            # pageno = None for frames on the main panel

        super(PanelCS2, self).__init__(parent, main, layout, bottomRow=None)

        keys = ['CS2_Plot','CS2_Table'] 
        self.mod = ModuleCS(keys)
        self.mod.initPanel()

        self.addEntry("hcModeChooser",
                      ChoiceEntry(self, values=HEATING_COOLING.values(),
                                         value=_('heating'), label=_('heating/cooling'),
                                         tip=_('Show only heating or cooling processes.'),
                                  allowNones=False),
                      0)
        self.entry["hcModeChooser"].Bind(wx.EVT_CHOICE, self.OnModeChooser)

        # table

        # update table

        titles = [_("Alternative")]

        self._showHeating = True
        if self._showHeating:
            titles += [_("Useful process heat (UPH)"),
                       _("Savings (UPH)"),
                       _("Useful supply heat (USH)"),
                       _("Savings (USH) ")]
        else:
            titles += [_("Useful process cooling (UPC)"),
                       _("Savings (UPC)"),
                       _("Useful supply cooling (USC)"),
                       _("Savings (USC)")]

        unitSelectors = [("text", None, None)] + [("float", "ENERGY", -3), ("float", "ENERGY", -3)]*2
                         
        hAlignment = [wx.ALIGN_LEFT] + [wx.ALIGN_RIGHT]*4
        readOnly = [True]*5

        self.addEntry("Table",
                      TableEntry(self, titles, changeData = None, size=wx.Size(760, 160),
                                 rowSize=24, colSize=120, colLabelSize=32,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors),
                      1)

        self.entry["Table"].setColSize(0, 240)
        self.entry["Table"].setPopUpMenu([])
        
        # plot

        plotSize = (760, 340)        
        self.addEntry("Plot",
                      BarPlot(self, size=plotSize),
                      2)

        self.plot = self.entry["Plot"]        
        self.plot.yAxis(label=_("Relative (present state = 100)"))

        # conclude layout
        self.conclude_layout()

    def OnModeChooser(self, event):
        """Eventhandler to set whether cooling or heating processes 
        should be displayed.
        
        """
        if self.blockEventHandlers: return
        
        mode = self.entry["hcModeChooser"].GetValue()
        self.mod.chooseMode(mode)
        self.display()

