# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelCS7: Comparative Study - Financial parameters
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from GUI.einsteinPanel import EinsteinPanel
from GUI.dataEntry.tableEntry import TableEntry
from GUI.displayClasses import ChoiceEntry
from lib.GUI.plot.barPlot import BarPlot

from constants import HEATING_COOLING
from GUI.status import Status
from modules.moduleCS import ModuleCS

import wx

class PanelCS7(EinsteinPanel):

    def __init__(self, parent, main):

        # generate list of notebook pages and frames
        layout = {"notebook": [],         # list of notebook-page titles. if empty or None, no notebook is created
                  "frames": [(None, _('Economic and Financial Performance')),
                             (None, _(' '))]}           # list of frames (pageno, title). if empty, no frame is created
                                                            # pageno = None for frames on the main panel

        super(PanelCS7, self).__init__(parent, main, layout, bottomRow=None)

        keys = ['CS7_Plot','CS7_Table'] 
        self.mod = ModuleCS(keys)
        self.mod.initPanel()

        # table

        # update table

        titles = [_("Alternative"),
                  _("Modified internal rate of return (IRR)"),
                  _("Pay-back period"),
                  _("Cost benefit ratio")]

        unitSelectors = [("text", None, None), ("float", "PERCENTAGE", 3), ("float", "LONGTIME", 2), ("float", "FRACTION", 2)]
                         
        hAlignment = [wx.ALIGN_LEFT] + [wx.ALIGN_RIGHT]*3
        readOnly = [True]*4

        self.addEntry("Table",
                      TableEntry(self, titles, changeData = None, size=wx.Size(760, 160),
                                 rowSize=24, colSize=120, colLabelSize=32,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors),
                      0)

        self.entry["Table"].setColSize(0, 320)
        self.entry["Table"].setColSize(1, 160)
        self.entry["Table"].setPopUpMenu([])
        
        # plot

        plotSize = (760, 340)        
        self.addEntry("Plot",
                      BarPlot(self, size=plotSize),
                      1)

        self.plot = self.entry["Plot"]        
        self.plot.yAxis(label=_("Modified internal rate of return [%]"))

        # conclude layout
        self.conclude_layout()
