# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBAuditor: Database of fuels
#
#==============================================================================
#
#	Version 3.0 (2018-2019)
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2019
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

from GUI.panels.DB.panelDBBase import PanelDBBase
from panelDBAuditorEntry import DialogDBAuditorEntry

from constants import FUELTYPES, TRANSFUELTYPES

fuelTypeDict = dict([(ftype, i) for i, ftype in enumerate(FUELTYPES)])
fuelTypeChoice = [TRANSFUELTYPES[ftype] for ftype in FUELTYPES]

#keys for import from SQL to internal object
SQLKEYS =  {"id": (None, None),
            "Name":("name", None),
            "City":(None, None),
            "Country":(None, None),
            "Company":(None, None),
            "CompanyType":(None, None),
            "Adress":(None, None),
            "Phone":(None, None),
            "Fax":(None, None),
            "EMail":(None, None),
            "Web":(None, None)}

#keys for export from internal object to GUI
DATAKEYS =  {"id": (None, None),
            "name":(None, None),
            "City":(None, None),
            "Country":(None, None),
            "Company":(None, None),
            "CompanyType":(None, None),
            "Adress":(None, None),
            "Phone":(None, None),
            "Fax":(None, None),
            "EMail":(None, None),
            "Web":(None, None)}

class PanelDBAuditor(PanelDBBase):
    def __init__(self, parent, title, showOk = False, preselection=None, selectedId=None):
        
        # specifications for overview table
        self.colLabels = ["id", _("Name"), _("City"), _("Country"), _("Company"), _("Company type")]
        self.colKeys =   ["id", "name",     "City", "Country", "Company", "CompanyType"]
        self.colUnits =  [("int", None, None),  ("text", None, None), ("text", None, None),
                          ("text", None, None), ("text", None, None), ("text", None, None)]

        self.colSizes = [60, 160, 140, 120, 140, 144]

        # set information for the max. 3 possible filter choices
        self.filters = {"Type": ["Country", _("Country"), _("Show only auditors of country"), None],
                        "SubType": ["CompanyType", _("CompanyType"), _("Show only auditors of company type"), None],
                        "SubType2": None}

        # assign editor dialog for single entry
        super(PanelDBAuditor, self).__init__(parent, title, "Auditor", "auditor", showOk=showOk,
                                             preselection=preselection, selectedId=selectedId,
                                             dialogEntryClass=DialogDBAuditorEntry,
                                             sqlKeys=SQLKEYS,
                                             dataKeys=DATAKEYS)

        # automatically display on init
        self.display()


