# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBAuditorEntry: Editor for auditor data
#
#==============================================================================
#
#	Version 3.0, 2018-2019
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
from GUI.displayClasses import *
from constants import FLUIDTYPES, TRANSFLUIDTYPES

from GUI.einsteinPanel import EinsteinPanel
from GUI.editWindow import EditWindow

from modules.guiModules.DB.moduleDB import ModuleDBEntry

from auxiliary.auxiliary import toGettextAndUnicode as _U

class PanelDBAuditorEntry(EinsteinPanel):

    def __init__(self, prnt, main):
        layout = {"notebook": [],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(None, _("General Data")),
                             (None, _("Contact Information"))]}           # list of frames (pageno, title). if empty, no frame is created

        super(PanelDBAuditorEntry, self).__init__(prnt, main, layout, module=None, size=(780, 600), key=None)

        self.mod = ModuleDBEntry(prnt.mod, self)

        #
        # tab 1 - General Data
        #

        self.addEntry("name",
                      TextEntry(self, maxchars = 45, value = '',
                             label = _U("Name"),
                             tip = _U("Name")),
                      0)

        self.addEntry("City",
                      TextEntry(self, maxchars = 45, value = '',
                             label = _U("City"),
                             tip = _U("City")),
                      0)

        self.addEntry("Country",
                      TextEntry(self, maxchars = 45, value = '',
                             label = _U("Country"),
                             tip = _U("Country")),
                      0)

        self.addEntry("Company",
                      TextEntry(self, maxchars = 45, value = '',
                             label = _U("Company"),
                             tip = _U("Company")),
                      0)

        self.addEntry("CompanyType",
                      TextEntry(self, maxchars = 45, value = '',
                             label = _U("CompanyType"),
                             tip = _U("CompanyType")),
                      0)

        self.addEntry("Address",
                      TextEntry(self, maxchars = 100, value = '',
                             isMultiline = True, specialHeight=100,
                             label = _U("Adress"),
                             tip = _U("Adress")),
                      1)

        self.addEntry("Phone",
                      TextEntry(self, maxchars = 45, value = '',
                             label = _U("Phone"),
                             tip = _U("Phone")),
                      1)

        self.addEntry("Fax",
                      TextEntry(self, maxchars = 45, value = '',
                             label = _U("Fax"),
                             tip = _U("Fax")),
                      1)

        self.addEntry("EMail",
                      TextEntry(self, maxchars = 45, value = '',
                             label = _U("EMail"),
                             tip = _U("EMail")),
                      1)

        self.addEntry("Web",
                      TextEntry(self, maxchars = 45, value = '',
                             label = _U("Web"),
                             tip = _U("Web")),
                      1)

        # conclude layout
        self.conclude_layout()


class DialogDBAuditorEntry(EditWindow):
    def __init__(self, prnt, main):
        self.prnt = prnt
        self.mod = prnt.mod
        if self.mod.editor: return  # do not open a second dialog

        super(DialogDBAuditorEntry, self).__init__(main)

        self.panel = PanelDBAuditorEntry(self, main)
        self.ret = None

        self.panel.display()
        self.Show()

        self.mod.editor = self
        self.prnt.panel.Disable()

    def OnClose(self, event):
        self.mod.editor = None
        self.prnt.panel.display()
        self.prnt.panel.Enable()
        super(DialogDBAuditorEntry, self).OnClose(event)

class AuditorDialog(EditWindow):
    """
    Alternative container for PanelDBAuditorEntry
    For license handling from the main GUI
    """

    def __init__(self, prnt, auditorId=None):
        
        from modules.guiModules.DB.moduleDB import ModuleDB
        from panelDBAuditor import SQLKEYS, DATAKEYS
        self.table = "auditor"
        self.colKeys = ["id", "name"]
        self.mod = ModuleDB(self, self.table, self.colKeys, sqlKeys=SQLKEYS, dataKeys=DATAKEYS, preselection=[auditorId], selectedId=auditorId)
        self.mod.initPanel({})
        self.mod.updatePanel()
        self.mod.selectRow(0)
        
        super(AuditorDialog, self).__init__(prnt)
  
        self.panel = PanelDBAuditorEntry(self, prnt)

        self.panel.display()
        

class AuditorPopUp(wx.Dialog):
    def __init__(self, prnt, auditorId=None):
        wx.Dialog.__init__(self, id=-1, parent=prnt, title=_("Edit Your Auditor Information"),
                           style=wx.DEFAULT_DIALOG_STYLE,
                           size=wx.Size(400, 60))

        self.Bind(wx.EVT_CLOSE, self.onCloseWindow)

        # access to font properties object
        fp = FontProperties()
        self.SetFont(fp.getFont())
        
        self.buttonGO = wx.Button(id=-1,
                                  label=_("OK"),
                                  name="ButtonGO",
                                  parent=self, pos=wx.Point(0, 48), size=wx.Size(360, 32))
        
        self.buttonGO.Bind(wx.EVT_BUTTON, self.onButtonGO)

        sizerGlobal = wx.BoxSizer(wx.VERTICAL)
##        sizerGlobal.Add(self.text, 0, wx.ALIGN_LEFT, 10)
        sizerGlobal.Add(self.buttonGO, 0, wx.ALIGN_CENTRE, 10)
        self.SetSizer(sizerGlobal)

        self.selectedId = auditorId
        self.dlg = None

    def onButtonGO(self, event):
        self.dlg = AuditorDialog(self, self.selectedId)
        self.mod = self.dlg.mod
        self.dlg.Show()
        
    def onPseudoModal(self, ret):
        if ret: self.mod.saveToDB(None)
        self.EndModal(bool(ret))

    def onCloseWindow(self, event):
        if self.dlg: self.dlg.Close()
        self.EndModal(False)
        
