# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelQ5: PIPE data entry
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from constants import EPSILON, KELVIN
from globals import logger, Globals

from GUI.einsteinPanel import EinsteinPanel
from GUI.editWindow import EditWindow

from GUI.tools.einsteinSimpleDialog import EinsteinSimpleDialog
from GUI.tools.dialogOKCancel import DialogOKCancel

from GUI.dataEntry.tableEntry import TableEntry, DEFAULTMENU
from GUI.displayClasses import FloatEntry, TextEntry, ChoiceEntry, CheckBoxEntry

from GUI.displayClasses import FieldSizes, WallpaperedPanel, smallFloatEntryClone, smallChoiceEntryClone, mirrorTextCtrl

from GUI.GUITools import GRID_BACKGROUND_COLOR

from auxiliary.auxiliary import toGettextAndUnicode as _U, toUnicode as U

from modules.guiModules.DB.moduleDB import ModuleDB

import wx

EDIT_DATA =  [True, True, True, True, True, False, False, True]

class PanelDBBase(EinsteinPanel):


    def __init__(self, parent, title, name, table, showOk=False, preselection=None, selectedId=None, dialogEntryClass=None, sqlKeys=None, dataKeys=None):
        """
        selectMode: True - mode used for quick selection; False: normal database editing mode
        """
        main = parent
        self.editWindow = EditWindow(main)
        self.dialogEntryClass = dialogEntryClass
        self.sqlKeys = sqlKeys
        self.dataKeys = dataKeys

        self.showOk = showOk
        self.selectMode = showOk

        self.name = name
        self.table = table

        self.preselection = preselection
        
        self.blockEventHandlers = True

        # generate list of notebook pages and frames
        layout = {"notebook": [],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(None, ""),
                             (None, _("Filters"))]}           # list of frames (pageno, title). if empty, no frame is created

        super(PanelDBBase, self).__init__(self.editWindow, main, layout, title=title)

        self.mod = ModuleDB(self, self.table, self.colKeys, sqlKeys=sqlKeys, dataKeys=dataKeys, preselection=preselection, selectedId=selectedId)
        initData = self.mod.initPanel(self.filters)

        # In StaticBox "General data" within "Distribution of heat/cold"
        self.ncols = len(self.colLabels)
        self.addEntry("ObjectList",
                      TableEntry(self, self.colLabels, self.mod.changeDataObjectList, size=wx.Size(780, 460),
                                 rowSize=24, colSize=80, colLabelSize=32,
                                 hAlignment = [wx.ALIGN_CENTRE]*self.ncols,
                                 readOnly = [True]*self.ncols,
                                 unitSelectors = self.colUnits),
                      0)

        # personalize column width
        for c, w in enumerate(self.colSizes):
            self.entry["ObjectList"].setColSize(c, w)

        self.entry["ObjectList"].setPopUpMenu([_("View/Edit"), _("Add"), _("Delete")])
        self.entry["ObjectList"].setExternalEventHandler(self.onEntryObjectList)

        # entries for filters

        hasFilter = False
        if initData["Type"]:
            hasFilter = True
            self.addEntry("Type",
                          ChoiceEntry(self,
                                      values = [U(entry) for entry in initData["Type"]["values"]],
                                      label = U(initData["Type"]["label"]),
                                      tip = U(initData["Type"]["tip"]),
                                      allowNones=False,
                                      registerUserChange=False),
                          1)

            self.Bind(wx.EVT_CHOICE, self.onEntryType, self.entry["Type"].entry)  

        if initData["SubType"]:
            hasFilter = True
            self.addEntry("SubType",
                          ChoiceEntry(self,
                                      values = [U(entry) for entry in initData["SubType"]["values"]],
                                      label = U(initData["SubType"]["label"]),
                                      tip = U(initData["SubType"]["tip"]),
                                      allowNones=False,
                                      registerUserChange=False),
                          1)
            self.Bind(wx.EVT_CHOICE, self.onEntrySubType, self.entry["SubType"].entry)  

        if initData["SubType2"]:
            hasFilter = True
            self.addEntry("SubType2",
                          ChoiceEntry(self,
                                      values = [U(entry) for entry in initData["SubType2"]["values"]],
                                      label = U(initData["SubType2"]["label"]),
                                      tip = U(initData["SubType2"]["tip"]),
                                      allowNones=False,
                                      registerUserChange=False),
                          1)
            self.Bind(wx.EVT_CHOICE, self.onEntrySubType2, self.entry["SubType2"].entry)  

        if not hasFilter:
            self.frame[1].Hide()
        
        # conclude layout
        self.conclude_layout()



    def fillPanel(self):
        popBlocker = self.blockEventHandlers
        self.blockEventHandlers = True

        super(PanelDBBase, self).fillPanel()
        # display table entries
        self.entry["ObjectList"].display(data=self.data["ObjectList"]["data"],
                                         choices=self.data["ObjectList"]["choices"],
                                         selection=self.data["ObjectList"]["selection"])
##                                         toolTips = self.data["ObjectList"]["toolTips"])

        self.blockEventHandlers = popBlocker


#------------------------------------------------------------------------------
#--- UI actions
#------------------------------------------------------------------------------

    def onEntryObjectList(self, eventType, row, col, selection):
        if self.blockEventHandlers: return
        
        ncols = self.entry["ObjectList"].ncols
        nrows = len(self.entry["ObjectList"].data)
        
        if row >= 0 and row < nrows:
            if eventType!="RightClick": self.mod.selectRow(row)
            
            if eventType=="LeftDclick": # select and close
                if self.selectMode:
                    self.OnButtonOK(None)
                    self.prnt.Close()
                else:
                    dialog = self.dialogEntryClass(self, self.prnt)

            elif eventType=="RightClick": # add or delete entry
                if self.mod.editObject: dialog = self.dialogEntryClass(self, self.prnt)
                self.display()


    def onEntryType(self, event):
        if self.blockEventHandlers: return
        popBlocker = self.blockEventHandlers
        self.blockEventHandlers = True

        new = self.entry["Type"].GetValue()
        subTypeChoices = self.mod.typeChanged(new)

        if subTypeChoices is not None and ("SubType" in self.entry.keys()):
            self.entry["SubType"].SetValue(subTypeChoices)

        self.display()
        
        self.blockEventHandlers = popBlocker

    def onEntrySubType(self, event):
        if self.blockEventHandlers: return
        popBlocker = self.blockEventHandlers
        self.blockEventHandlers = True

        new = self.entry["SubType"].GetValue()
        subType2Choices = self.mod.subTypeChanged(new)

        if subType2Choices is not None and ("SubType2" in self.entry.keys()):
            self.entry["SubType2"].SetValue(subType2Choices)
        
        self.display()
        self.blockEventHandlers = popBlocker

    def onEntrySubType2(self, event):
        if self.blockEventHandlers: return
        popBlocker = self.blockEventHandlers
        self.blockEventHandlers = True

        new = self.entry["SubType2"].GetValue()
        self.mod.subType2Changed(new)
        
        self.display()
        self.blockEventHandlers = popBlocker


# temporary hacks for compatibility with old PanelDBBase - Dialog
    def ShowModal(self):
        self.display()
        self.Show()
        self.editWindow.Show()

