# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBBenchmark: Database of fuels
#
#==============================================================================
#
#	Version 3.0 (2018-2019)
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2019
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

from GUI.panels.DB.panelDBBase import PanelDBBase
from panelDBBenchmarkEntry import DialogDBBenchmarkEntry

from GUI.status import Status


#keys for import from SQL to internal object
SQLKEYS =  {"id": (None, None),
               "NACECode": (None, None),
               "UnitOp": (None, None),
               "ProductCode": (None, None),
               "Product": (None, None),
               "ProductUnit": (None, None),
               "Comments": (None, None),
               "DataRelevance": (None, None),
               "TurnoverMin": (None, None),
               "TurnoverMax": (None, None),
               "ProductionMin": (None, None),
               "ProductionMax": (None, None),
               "YearReference": (None, None),
               "Reference": (None, None),
               "Literature": (None, None),
               "E_EnergyInt_MIN_PC": (None, None),
               "E_EnergyInt_MAX_PC": (None, None),
               "E_EnergyInt_TARG_PC": (None, None),
               "E_EnergyInt_MIN_T": (None, None),
               "E_EnergyInt_MAX_T": (None, None),
               "E_EnergyInt_TARG_T": (None, None),
               "E_SEC_MIN": (None, None),
               "E_SEC_MAX": (None, None),
               "E_SEC_TARG": (None, None),
               "E_Unit": (None, None),
               "H_EnergyInt_MIN_PC": (None, None),
               "H_EnergyInt_MAX_PC": (None, None),
               "H_EnergyInt_TARG_PC": (None, None),
               "H_EnergyInt_MIN_T": (None, None),
               "H_EnergyInt_MAX_T": (None, None),
               "H_EnergyInt_TARG_T": (None, None),
               "H_SEC_MIN": (None, None),
               "H_SEC_MAX": (None, None),
               "H_SEC_TARG":  (None, None),
               "H_Unit": (None, None),
               "T_EnergyInt_MIN_PC": (None, None),
               "T_EnergyInt_MAX_PC": (None, None),
               "T_EnergyInt_TARG_PC": (None, None),
               "T_EnergyInt_MIN_T": (None, None),
               "T_EnergyInt_MAX_T": (None, None),
               "T_EnergyInt_TARG_T": (None, None),
               "T_SEC_MIN": (None, None),
               "T_SEC_MAX": (None, None),
               "T_SEC_TARG": (None, None),
               "T_Unit": (None, None)}

#keys for export from internal object to GUI
DATAKEYS =  SQLKEYS.copy()


class PanelDBBenchmark(PanelDBBase):
    def __init__(self, parent, title, showOk = False):

        ## TODO: this is not the right place here -> should be centralised in constants or somewhere
        unitOpDict = Status.prj.getUnitOpDict(code=True)
        unitOpItems = [(code, text) for code, text in unitOpDict.items()]
        sortedItems =  sorted(unitOpItems, key=lambda item: item[0])
        unitOpDict = dict((str(item[0]), item[1]) for item in sortedItems)
        unitOpList = [item[1] for item in sortedItems]
        unitOpCodes = [str(item[0]) for item in sortedItems]

        unitOpDictData = dict([(str(u), i) for i, u in enumerate(unitOpCodes)])
##        unitOpDictSQL  = dict([(str(code), i) for i, code in enumerate(unitOpCodes)])

        SQLKEYS["UnitOp"] =  (None, None)
        DATAKEYS["UnitOp"] = (None, unitOpDictData)

        # specifications for overview table
        self.colLabels = ["id", _("NACE Code"), _("Unit Operation"), _("Product Code"), _("Product"), _("Reference")]
        self.colKeys =   ["id", "NACECode", "UnitOp", "ProductCode", "Product", "Reference"]
        self.colUnits =  [("int", None, None), ("text", None, None), ("choice", unitOpList, None),
                          ("text", None, None), ("text", None, None), ("text", None, None)]

        self.colSizes = [60, 80, 180, 60, 180, 204]

        # set information for the max. 3 possible filter choices
        self.filters = {"Type": ["NACECode", _("NACE Code"), " ", None],
                        "SubType": ["ProductCode", _("Product Code"), " ", None],
                        "SubType2": ["UnitOp", _("Type of unit operation"), " ", unitOpDict]}

        # assign editor dialog for single entry
        super(PanelDBBenchmark, self).__init__(parent, title, "Fuel", "dbbenchmark", showOk=showOk,
                                           dialogEntryClass=DialogDBBenchmarkEntry,
                                           sqlKeys=SQLKEYS,
                                           dataKeys=DATAKEYS)

        # automatically display on init
        self.display()


