# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBBenchmarkEntry: Editor for fluid data
#
#==============================================================================
#
#	Version 3.0, 2018-2019
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
from GUI.displayClasses import *
from GUI.dataEntry.tableEntry import TableEntry

from constants import INFINITE

from GUI.einsteinPanel import EinsteinPanel
from GUI.editWindow import EditWindow

from modules.guiModules.DB.moduleDB import ModuleDBEntry

from auxiliary.auxiliary import toGettextAndUnicode as _U

class PanelDBBenchmarkEntry(EinsteinPanel):

    def __init__(self, prnt, main):

        layout = {"notebook": [_('Range of application'),
                               _('Data')],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(0, _("Industrial sector, unit operation and product type")),
                             (0, _("Limits of validity range")),
                             (0, _("Data source")),
                             (1, _(""))]}           # list of frames (pageno, title). if empty, no frame is created

        super(PanelDBBenchmarkEntry, self).__init__(prnt, main, layout, module=None, size=(780, 600), key=None)

        self.mod = ModuleDBEntry(prnt.mod, self, objectType="BM")
        self.initData = self.mod.initPanel()


        #
        # frame 0- Validity of benchmark - general
        #

        self.addEntry("NACECode",
                      ChoiceEntry(self.nbPage[0],
                               values = self.initData["ChoicesNACECode"],
                               label = _U("NACE code of the industrial sector"),
                               tip = _U("The benchmark can be general or only valid for a specific industrial sector")),
                      0)

        self.addEntry("UnitOp",
                      ChoiceEntry(self.nbPage[0],
                               values = self.initData["ChoicesUnitOp"],
                               label = _U("Unit operation code"),
                               tip = _U("The benchmark can be general for the industrial sector, or specific for some unit operation in the sector. \"General\" could be an unit-operation code = 0, or empty")),
                      0)

        self.addEntry("ProductCode",
                      ChoiceEntry(self.nbPage[0],
                               values = self.initData["ChoicesProductCode"],
                               label = _U("Product Code"),
                               tip = _U("Final product or intermediate product for unit operation specific benchmarks")),
                      0)

        self.addEntry("Product",
                      TextEntry(self.nbPage[0], maxchars = 45, value = '',
                             label = _U("Product short name"),
                             tip = _U("Final product or intermediate product for unit operation specific benchmarks")),
                      0)

        self.addEntry("Comments",
                      TextEntry(self.nbPage[0], value = '',
                             isMultiline = True, specialHeight=60,
                             label = _U("Comments on range of application"),
                             tip = _U("Additional comments on validity range (e.g. restrictions to certain types of machinery, etc.)")),
                      0)

        self.addEntry("DataRelevance",
                      TextEntry(self.nbPage[0], maxchars = 200, value = '',
                             isMultiline = True,
                             label = _U("Data relevance/reliability"),
                             tip = _U("Data relevance/reliability")),
                      0)

        #
        # frame 1- Limits of validity
        #
        self.addEntry("TurnoverMin",
                      FloatEntry(self.nbPage[0],
                              decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'CURRENCY',
                              label = _U("Yearly turnover (minimum)"),
                              tip = _U("Yearly turnover (minimum)")),
                      1)

        self.addEntry("TurnoverMax",
                      FloatEntry(self.nbPage[0],
                              decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'CURRENCY',
                              label = _U("Yearly turnover (maximum)"),
                              tip = _U("Yearly turnover (maximum)")),
                      1)

        self.addEntry("ProductUnit",
                      TextEntry(self.nbPage[0], maxchars = 45, value = '',
                             label = _U("Measurement unit for product"),
                             tip = _U("Measurement unit for product")),
                      1)

        self.addEntry("ProductionMin",
                      FloatEntry(self.nbPage[0],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'PU',
                               label = _U("Yearly production volume (minimum)"),
                               tip = _U("Yearly production volume (minimum)")),
                      1)

        self.addEntry("ProductionMax",
                      FloatEntry(self.nbPage[0],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'PU',
                               label = _U("Yearly production volume (maximum)"),
                               tip = _U("Yearly production volume (maximum)")),
                      1)

        #
        # frame 2 - Data source
        #

        self.addEntry("Reference",
                      TextEntry(self.nbPage[0], value = '',
                              isMultiline = True, specialHeight=60,
                              label = _U("References"),
                              tip = _U("Bibliographic reference (data source) of the benchmark")),
                      2)

        self.addEntry("Literature",
                      TextEntry(self.nbPage[0], value = '',
                              isMultiline = True, specialHeight=60,
                              label = _U("Complementary literature"),
                              tip = _U("Additional bibliography")),
                      2)

        #
        # tab 2 - Data
        #

        # Price curve as a function of system size
        titles = [_("Parameter"), _("Product Unit"), _("Unit"), "Min.", "Target", "Max."]

        self.addEntry("DataTable",
                      TableEntry(self.nbPage[1], titles, None, size=wx.Size(780, 360),
                                 rowSize=24, colSize=120, colLabelSize=32,
                                 hAlignment = [wx.ALIGN_LEFT, wx.ALIGN_CENTRE] + [wx.ALIGN_RIGHT]*4,
                                 unitSelectors = [("float", "ENERGYPERCU", 0)]*20,
                                 unitCol = 2,
                                 readOnly = [True, True, False] + [False]*4),
                      3)

##        self.entry["UPHComponents"].setRowSize(0, 48) ###TODO: increase height o title row
        self.entry["DataTable"].setColSize(0, 244)
        self.entry["DataTable"].setColSize(1, 80)
        self.entry["DataTable"].setColSize(2, 80)
        self.entry["DataTable"].setPopUpMenu([])

        self.addEntry("YearReference",
                      FloatEntry(self.nbPage[1],
                               decimals = 0, minval = 1900, maxval = 2100, value = 2010,
                               label = _U("Year (reference for economic data)"),
                               tip = _U("Reference year for the economic data")),
                      3)

        # conclude layout
        self.conclude_layout()

    def fillPanel(self):
        super(PanelDBBenchmarkEntry, self).fillPanel()

        # set special format for specific rows / cells
        self.entry["DataTable"].setAttribute("style", "italic", row=[0, 4, 8])
        self.entry["DataTable"].setAttribute("edit", False, row=[0, 4, 8])
        self.entry["DataTable"].setAttribute("edit", True, row = [9, 10, 11], col=1)
        

class DialogDBBenchmarkEntry(EditWindow):
    def __init__(self, prnt, main):
        self.prnt = prnt
        self.mod = prnt.mod
        if self.mod.editor: return  # do not open a second dialog

        super(DialogDBBenchmarkEntry, self).__init__(main)

        self.panel = PanelDBBenchmarkEntry(self, main)
        self.ret = None

        self.panel.display()
        self.Show()

        self.mod.editor = self
        self.prnt.panel.Disable()

    def OnClose(self, event):
        self.mod.editor = None
        self.prnt.panel.display()
        self.prnt.panel.Enable()
        super(DialogDBBenchmarkEntry, self).OnClose(event)
        
