# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBBoiler: Database of fuels
#
#==============================================================================
#
#	Version 3.0 (2018-2019)
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2019
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

from GUI.panels.DB.panelDBBase import PanelDBBase
from panelDBBoilerEntry import DialogDBBoilerEntry

from constants import BBTYPES, TRANSBBTYPES

boilerTypeDict = dict([(btype, i) for i, btype in enumerate(BBTYPES)])
boilerTypeChoice = [TRANSBBTYPES[btype] for btype in BBTYPES]
boilerTypeDictInverse = dict([(str(item[1]), item[0]) for item in boilerTypeDict.items()])

#keys for import from SQL to internal object
SQLKEYS =  {"id": (None, None),
               "BoilerManufacturer":(None, None),
               "BoilerModel":(None, None),
               "BoilerType":(None, None),
               "Reference":(None, None),
               "BBPnom":(None, None),
               "BBEfficiency":(None, None),
               "FuelConsum":(None, None),
               "FuelType":(None, None),
               "ElConsum":(None, None),
               "Economiser":(None, None),
               "Preheater":(None, None),
               "ExcessAirRatio":(None, None),
               "BBA1":(None, None),
               "BBA2":(None, None),
               "BBK1":(None, None),
               "BBK2":(None, None),
               "BoilerTemp":(None, None),
               "BoilerPrice":(None, None),
               "BoilerTurnKeyPrice":(None, None),
               "BoilerOandMfix":(None, None),
               "BoilerOandMvar":(None, None),
               "YearUpdate":(None, None)}

#keys for export from internal object to GUI
DATAKEYS =  {"id": (None, None),
               "BoilerManufacturer":(None, None),
               "BoilerModel":(None, None),
               "BoilerType":(None, boilerTypeDict),
               "Reference":(None, None),
               "BBPnom":(None, None),
               "BBEfficiency":(None, None),
               "FuelConsum":(None, None),
               "FuelType":(None, None),
               "ElConsum":(None, None),
               "Economiser":(None, None),
               "Preheater":(None, None),
               "ExcessAirRatio":(None, None),
               "BBA1":(None, None),
               "BBA2":(None, None),
               "BBK1":(None, None),
               "BBK2":(None, None),
               "BoilerTemp":(None, None),
               "BoilerPrice":(None, None),
               "BoilerTurnKeyPrice":(None, None),
               "BoilerOandMfix":(None, None),
               "BoilerOandMvar":(None, None),
               "YearUpdate":(None, None)}


class PanelDBBoiler(PanelDBBase):
    def __init__(self, parent, title, showOk = False):

        # specifications for overview table
        self.colLabels = ["id", _("Manufacturer"), _("Model"), _("Type"), _("Nominal heating power")]
        self.colKeys =   ["id", "BoilerManufacturer", "BoilerModel", "BoilerType", "BBPnom"]
        self.colUnits =  [("int", None, None),
                          ("text", None, None),
                          ("text", None, None),
                          ("choice", boilerTypeChoice, None),
                          ("float", "POWER", 2)]

        self.colSizes = [60, 200, 160, 160, 160]

        # set information for the max. 3 possible filter choices
        self.filters = {"Type": ["BoilerType", _("Boiler type"), _("Show only boilers of type"), TRANSBBTYPES],
                        "SubType": None,
                        "SubType2": None}

        # assign editor dialog for single entry
        super(PanelDBBoiler, self).__init__(parent, title, "BB", "dbboiler", showOk=showOk,
                                           dialogEntryClass=DialogDBBoilerEntry,
                                           sqlKeys=SQLKEYS,
                                           dataKeys=DATAKEYS)

        # automatically display on init
        self.display()


