# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBBoilerEntry: Editor for boiler data
#
#==============================================================================
#
#	Version 3.0, 2018-2019
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
from GUI.displayClasses import *
from constants import BBTYPES, TRANSBBTYPES, FUELTYPES, TRANSFUELTYPES, YESNO, TRANSYESNO, INFINITE

from GUI.einsteinPanel import EinsteinPanel
from GUI.editWindow import EditWindow

from modules.guiModules.DB.moduleDB import ModuleDBEntry

from auxiliary.auxiliary import toGettextAndUnicode as _U

class PanelDBBoilerEntry(EinsteinPanel):

    def __init__(self, prnt, main):

        layout = {"notebook": [_('Technical data'),
                               _('Off-nominal conditions'),
                               _("Economic parameters")],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(0, _("Descriptive data")),
                             (0, _('Technical Data')),
                             (2, _('Behaviour at off-nominal conditions')),
                             (0, _('Heat source / sink')),
                             (1, _('Economic Parameters'))]}           # list of frames (pageno, title). if empty, no frame is created

        super(PanelDBBoilerEntry, self).__init__(prnt, main, layout, module=None, size=(780, 600), key=None)

        self.mod = ModuleDBEntry(prnt.mod, self)

        #
        # frame 0 - Descriptive Data
        #
        self.addEntry("BoilerManufacturer",
                      TextEntry(self.nbPage[0], maxchars = 45, value = '',
                             label = _U("Boiler Manufacturer"),
                             tip = _U("Boiler Manufacturer")),
                      0)

        self.addEntry("BoilerModel",
                      TextEntry(self.nbPage[0], maxchars = 45, value = '',
                             label = _U("Boiler Model"),
                             tip = _U("Boiler Model")),
                      0)

        self.addEntry("BoilerType",
                      ChoiceEntry(self.nbPage[0],
                               values = [U(TRANSBBTYPES[btype]) for btype in BBTYPES],
                               label = _U("Boiler type"),
                               tip = _U("Boiler type")),
                      0)

        self.addEntry("Reference",
                      TextEntry(self.nbPage[0], maxchars = 200, value = '',
                             isMultiline = True, specialHeight=100,
                             label = _U("Reference"),
                             tip = _U("Source of data")),
                      0)

        #
        # frame 1 - Technical data
        #

        self.addEntry("BBPnom",
                      FloatEntry(self.nbPage[0],
                              decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'POWER',
                              label = _U("Nominal thermal power"),
                              tip = _U("Nominal thermal power")),
                      1)

        self.addEntry("BBEfficiency",
                      FloatEntry(self.nbPage[0],
                              decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'FRACTION',
                              label = _U("Nominal efficiency"),
                              tip = _U("Nominal efficiency")),
                      1)

        self.addEntry("FuelConsum",
                      FloatEntry(self.nbPage[0],
                              decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'POWER',
                              label = _U("Nominal fuel consumption (LCV)"),
                              tip = _U("Nominal fuel consumption (LCV)")),
                      1)

        self.addEntry("FuelType",
                      ChoiceEntry(self.nbPage[0],
                               values = [U(TRANSFUELTYPES[ftype]) for ftype in FUELTYPES],
                               label = _U("Fuel type"),
                               tip = _U("Fuel type")),
                      1)

        self.addEntry("ElConsum",
                      FloatEntry(self.nbPage[0],
                              decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'POWER',
                              label = _U("Nominal electrical power consumption"),
                              tip = _U("Nominal electrical power consumption")),
                      1)

        self.addEntry("Economiser",
                      ChoiceEntry(self.nbPage[0],
                                  values=[TRANSYESNO[key] for key in YESNO], value=None,
                                  label=_U("Economiser included"),
                                  tip=_U("Does the equipment include an economiser (water preheater)?"),
                                  allowNones=False),
                      1)
        self.addEntry("Preheater",
                      ChoiceEntry(self.nbPage[0],
                                  values=[TRANSYESNO[key] for key in YESNO], value=None,
                                  label=_U("Air preheater included"),
                                  tip=_U("Does the equipment include an air preheater ?"),
                                  allowNones=False),
                      1)

        self.addEntry("ExcessAirRatio",
                      FloatEntry(self.nbPage[0],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'FRACTION',
                               label = _U("Typical excess air ratio"),
                               tip = _U("Typical excess air ratio")),
                      1)

        #
        # frame 2 - off-nominal conditions
        #


        self.addEntry("BBA1",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'FRACTION',
                               label = _U("Linear dependence of the efficiency on the load"),
                               tip = _U("Linear dependence of the efficiency on the load")),
                      2)

        self.addEntry("BBA2",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'FRACTION',
                               label = _U("Quadratic dependence of the efficiency on the load"),
                               tip = _U("Quadratic dependence of the efficiency on the load")),
                      2)

        self.addEntry("BBK1",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'INVTEMP',
                               label = _U("Linear dependence of the efficiency on the temperature"),
                               tip = _U("Linear dependence of the efficiency on the temperature")),
                      2)


        self.addEntry("BBK2",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'INVTEMP2',
                               label = _U("Quadratic dependence of the efficiency on the temperature"),
                               tip = _U("Quadratic dependence of the efficiency on the temperature")),
                      2)

        #
        # tab 3 - Heat source / sink
        #

        self.addEntry("BoilerTemp",
                      FloatEntry(self.nbPage[0],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("Maximum outlet temperature"),
                               tip = _U("Maximum outlet temperature")),
                      3)

        #
        # tab 4 - Economic Parameters
        #

        self.addEntry("BoilerPrice",
                      FloatEntry(self.nbPage[2],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'PRICE',
                               label = _U("Equipment price at factory applied installer's discount"),
                               tip = _U("Equipment price at factory applied installer's discount")),
                      4)

        self.addEntry("BoilerTurnKeyPrice",
                      FloatEntry(self.nbPage[2],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'PRICE',
                               label = _U("Price of installed equipment"),
                               tip = _U("Price of installed equipment (including work, additional accessories, pumps, regulation, etc)")),
                      4)

        self.addEntry("BoilerOandMfix",
                      FloatEntry(self.nbPage[2],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'UNITPRICE',
                               label = _U("Annual operational and maintenance fixed costs"),
                               tip = _U("Annual operational and maintenance fixed costs (approximate average per kW heating)")),
                      4)

        self.addEntry("BoilerOandMvar",
                      FloatEntry(self.nbPage[2],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'ENERGYTARIFF',
                               label = _U("Annual operational and maintenance variable costs dependant on usage"),
                               tip = _U("Annual operational and maintenance variable costs dependant on usage (approximate average per MWh heating)")),
                      4)

        self.addEntry("YearUpdate",
                      FloatEntry(self.nbPage[2],
                               decimals = 0, minval = 1900, maxval = 2100, value = 2010,
                               label = _U("Year of last update of the economic data"),
                               tip = _U("Year of last update of the economic data")),
                      4)

        # conclude layout
        self.conclude_layout()

class DialogDBBoilerEntry(EditWindow):
    def __init__(self, prnt, main):
        self.prnt = prnt
        self.mod = prnt.mod
        if self.mod.editor: return  # do not open a second dialog

        super(DialogDBBoilerEntry, self).__init__(main)

        self.panel = PanelDBBoilerEntry(self, main)
        self.ret = None

        self.panel.display()
        self.Show()

        self.mod.editor = self
        self.prnt.panel.Disable()

    def OnClose(self, event):
        self.mod.editor = None
        self.prnt.panel.display()
        self.prnt.panel.Enable()
        super(DialogDBBoilerEntry, self).OnClose(event)
        
