# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBCHP: Database of fuels
#
#==============================================================================
#
#	Version 3.0 (2018-2019)
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2019
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

from GUI.panels.DB.panelDBBase import PanelDBBase
from panelDBCHPEntry import DialogDBCHPEntry

from constants import CHPTYPES, TRANSCHPTYPES

typeDict = dict([(otype, i) for i, otype in enumerate(CHPTYPES)])
typeChoice = [TRANSCHPTYPES[otype] for otype in CHPTYPES]

#keys for import from SQL to internal object
SQLKEYS =  {"id": (None, None),
            "Manufacturer": (None, None),
               "CHPequip": (None, None),
               "Type": (None, None),
##               "SubType":dbio.check(self.tc4.GetValue(text = True)),
               "Reference": (None, None),
               "CHPPt": (None, None),
               "FuelType": (None, None),
               "FuelConsum": (None, None),
               "Eta_t": (None, None),
               "CHPPe": (None, None),
               "Eta_e": (None, None),
               "FluidSupply": (None, None),
               "TSupply": (None, None),
               "FlowRateSupply": (None, None),
               "FluidSupply2": (None, None),
               "TSupply2": (None, None),
               "FlowRateSupply2": (None, None),
               "Price": (None, None),
               "InvRate": (None, None),
               "OMRateFix": (None, None),
               "OMRateVar": (None, None),
               "YearUpdate": (None, None)}

#keys for export from internal object to GUI
DATAKEYS =  {"id": (None, None),
            "Manufacturer": (None, None),
               "CHPequip": (None, None),
               "Type": (None, typeDict),
##               "SubType":dbio.check(self.tc4.GetValue(text = True)),
               "Reference": (None, None),
               "CHPPt": (None, None),
               "FuelType": (None, None),
               "FuelConsum": (None, None),
               "Eta_t": (None, None),
               "CHPPe": (None, None),
               "Eta_e": (None, None),
               "FluidSupply": (None, "FluidId"),
               "TSupply": (None, None),
               "FlowRateSupply": (None, None),
               "FluidSupply2": (None, "FluidId"),
               "TSupply2": (None, None),
               "FlowRateSupply2": (None, None),
               "Price": (None, None),
               "InvRate": (None, None),
               "OMRateFix": (None, None),
               "OMRateVar": (None, None),
               "YearUpdate": (None, None)}


class PanelDBCHP(PanelDBBase):
    def __init__(self, parent, title, showOk = False, preselection=None):

        # DBCHP_ID needs to remain as first entry although it is not shown on the GUI
        self.colKeys = "DBCHP_ID","Manufacturer", "CHPequip", "Type", "CHPPe", "CHPPt"
        self.typeDict = TRANSCHPTYPES
        
        # specifications for overview table
        self.colLabels = ["id", _("Manufacturer"), _("Model"), _("Type"), _("Nominal electrical power"), _("Nominal thermal power")]
        self.colKeys =   ["id", "Manufacturer", "CHPequip", "Type", "CHPPe", "CHPPt"]
        self.colUnits =  [("int", None, None),
                          ("text", None, None),
                          ("text", None, None),
                          ("choice", typeChoice, None),
                          ("float", "POWER", 2),
                          ("float", "POWER", 2)]

        self.colSizes = [60, 200, 120, 140, 140, 104]

        # set information for the max. 3 possible filter choices
        self.filters = {"Type": ["Type", _("Type"), _("Show only equipment of type"), TRANSCHPTYPES],
                        "SubType": None,
                        "SubType2": None}

        # assign editor dialog for single entry
        super(PanelDBCHP, self).__init__(parent, title, "Fuel", "dbchp", showOk=showOk, preselection=preselection,
                                           dialogEntryClass=DialogDBCHPEntry,
                                           sqlKeys=SQLKEYS,
                                           dataKeys=DATAKEYS)

        # automatically display on init
        self.display()


