# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBCHPEntry: Editor for fluid data
#
#==============================================================================
#
#	Version 3.0, 2018-2019
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
from GUI.displayClasses import *

from constants import CHPTYPES, TRANSCHPTYPES, FUELTYPES, TRANSFUELTYPES, INFINITE

from GUI.einsteinPanel import EinsteinPanel
from GUI.editWindow import EditWindow

from modules.guiModules.DB.moduleDB import ModuleDBEntry

from auxiliary.auxiliary import toGettextAndUnicode as _U

class PanelDBCHPEntry(EinsteinPanel):

    def __init__(self, prnt, main):

        layout = {"notebook": [_('Descriptive Data'),
                               _('Technical Data'),
                               _("Economic Parameters")],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(0, _("Descriptive Data")),
                             (1, _("Technical Data")),
                             (1, _("Electricity generation parameters")),
                             (1, _("Thermal generation parameters")),
                             (1, _("Heat sink")),
                             (1, _("Heat sink (2nd circuit)")),
                             (2, _("Economic Parameters"))]}           # list of frames (pageno, title). if empty, no frame is created

        super(PanelDBCHPEntry, self).__init__(prnt, main, layout, module=None, size=(780, 600), key=None)

        self.mod = ModuleDBEntry(prnt.mod, self)

        #
        # tab 1 - Descriptive Data
        #

        self.addEntry("Manufacturer",
                      TextEntry(self.nbPage[0], maxchars = 45, value = '',
                             label = _U("Manufacturer"),
                             tip = _U("Manufacturer")),
                      0)

        self.addEntry("CHPequip",
                      TextEntry(self.nbPage[0], maxchars = 45, value = '',
                             label = _("Model"),
                             tip = _("Model name and number")),
                      0)
               
        self.addEntry("Type",
                      ChoiceEntry(self.nbPage[0],
                               values = [TRANSCHPTYPES[ctype] for ctype in CHPTYPES],
                               label = _("Type of equipment"),
                               tip = _("Type")),
                      0)

##        self.tc4 = ChoiceEntry(self.nbPage[0],
##                               values = [],
##                               label = _U("SubType"),
##                               tip = _U("SubType"))
               
        self.addEntry("Reference",
                      TextEntry(self.nbPage[0], maxchars = 200, value = '',
                             isMultiline = True, specialHeight=100,
                             label = _U("Reference"),
                             tip = _U("Source of data")),
                      0)


        #
        # tab 2 - Technical data
        #

        self.addEntry("FuelType",
                      ChoiceEntry(self.nbPage[1],
                               values = [TRANSFUELTYPES[ftype] for ftype in FUELTYPES],
                               label = _U("Fuel type"),
                               tip = _U("Fuel type")),
                      1)
        
        self.addEntry("FuelConsum",
                      FloatEntry(self.nbPage[1],
                              decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'POWER',
                              label = _U("Nominal fuel consumption"),
                              tip = _U("Nominal fuel consumption")),
                      1)
        
        self.addEntry("CHPPt",
                      FloatEntry(self.nbPage[1],
                              decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'POWER',
                              label = _U("Nominal thermal power"),
                              tip = _U("Nominal thermal power")),
                      2)
        
        self.addEntry("Eta_t",
                      FloatEntry(self.nbPage[1],
                              decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'FRACTION',
                              label = _U("Nominal thermal conversion efficiency"),
                              tip = _U("Nominal thermal conversion efficiency")),
                      2)
        
        self.addEntry("CHPPe",
                      FloatEntry(self.nbPage[1],
                               decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'POWER',
                               label = _U("Nominal electrical power"),
                               tip = _U("Nominal electrical power")),
                      3)
        
        self.addEntry("Eta_e",
                      FloatEntry(self.nbPage[1],
                               decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'FRACTION',
                               label = _U("Electrical efficiency"),
                               tip = _U("Electrical efficiency")),
                      3)

        #
        # tab 3 - Heat source / sink
        #

        fluidChoices = [U(entry) for entry in self.mod.prnt.fluidChoices]


        self.addEntry("FluidSupply",
                      ChoiceEntry(self.nbPage[1],
                                values = fluidChoices,
                                label = _U("Heat transport medium"),
                                tip = _U("Heat transport medium")),
                      4)
               
        self.addEntry("TSupply",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("Outlet temperature at nominal conditions"),
                               tip = _U("Outlet temperature at nominal conditions")),
                      4)
               
        self.addEntry("FlowRateSupply",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'MASSFLOW',
                               label = _U("Mass flow rate of heat transport medium"),
                               tip = _U("Mass flow rate of heat transport medium")),
                      4)
               

        self.addEntry("FluidSupply2",
                      ChoiceEntry(self.nbPage[1],
                                values = fluidChoices,
                                label = _U("Heat transport medium"),
                                tip = _U("Heat transport medium")),
                      5)
               
        self.addEntry("TSupply2",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("Outlet temperature at nominal conditions"),
                               tip = _U("Outlet temperature at nominal conditions")),
                      5)
               
        self.addEntry("FlowRateSupply2",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'MASSFLOW',
                               label = _U("Mass flow rate of heat transport medium"),
                               tip = _U("Mass flow rate of heat transport medium")),
                      5)

        #
        # tab 4 - Economic Parameters
        #

        self.addEntry("Price",
                      FloatEntry(self.nbPage[2],
                               decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'PRICE',
                               label = _U("Equipment price at factory applied installer's discount"),
                               tip = _U("Equipment price at factory applied installer's discount")),
                      6)
               
        self.addEntry("InvRate",
                      FloatEntry(self.nbPage[2],
                               decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'PRICE',
                               label = _U("Turn-key price"),
                               tip = _U("Full cost including transport, installation, piping and auxiliary equipment")),
                      6)
               
        self.addEntry("OMRateFix",
                      FloatEntry(self.nbPage[2],
                               decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'PRICE',
                               label = _U("Annual operational and maintenance fixed costs"),
                               tip = _U("Fixed costs independent of actual operation hours")),
                      6)
               
        self.addEntry("OMRateVar",
                      FloatEntry(self.nbPage[2],
                               decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'ENERGYTARIFF',
                               label = _U("Annual operational and maintenance variable costs dependant on usage"),
                               tip = _U("Annual operational and maintenance variable costs dependant on usage (approximate average per unit of generated heat)")),
                      6)
               
        self.addEntry("YearUpdate",
                      FloatEntry(self.nbPage[2],
                               decimals = 0, minval = 1900, maxval = 2100, value = 2010,
                               label = _U("Year of last update of the economic data"),
                               tip = _U(" ")),
                      6)

       # conclude layout
        self.conclude_layout()


class DialogDBCHPEntry(EditWindow):
    def __init__(self, prnt, main):
        self.prnt = prnt
        self.mod = prnt.mod
        if self.mod.editor: return  # do not open a second dialog

        super(DialogDBCHPEntry, self).__init__(main)

        self.panel = PanelDBCHPEntry(self, main)
        self.ret = None

        self.panel.display()
        self.Show()

        self.mod.editor = self
        self.prnt.panel.Disable()

    def OnClose(self, event):
        self.mod.editor = None
        self.prnt.panel.display()
        self.prnt.panel.Enable()
        super(DialogDBCHPEntry, self).OnClose(event)
        
