# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBElectricityMix: Database of fuels
#
#==============================================================================
#
#	Version 3.0 (2018-2019)
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2019
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

from GUI.panels.DB.panelDBBase import PanelDBBase
from panelDBElectricityMixEntry import DialogDBElectricityMixEntry

from constants import FUELTYPES, TRANSFUELTYPES

fuelTypeDict = dict([(ftype, i) for i, ftype in enumerate(FUELTYPES)])
fuelTypeChoice = [TRANSFUELTYPES[ftype] for ftype in FUELTYPES]

#keys for import from SQL to internal object
DATAKEYS =  {"id": (None, None),
            "name": (None, None),
               "Country": (None, None),
               "Year": (None, None),
               "Type": (None, None),
               "Reference": (None, None),
               #"AuditorID":, # read only field
               "PE2ConvEl": (None, None),
               "CO2ConvEl": (None, None),
               "NoNukesConvEl": (None, None),
               "PercNaturalGas": (None, None),
               "PercCarbon": (None, None),
               "PercOil": (None, None),
               "PercRenewables": (None, None),
               "PercNukes": (None, None),
               "PercCHP": (None, None),
               "PercOther": (None, None)}

#keys for export from internal object to GUI
SQLKEYS =  DATAKEYS.copy()
SQLKEYS.update({"ShortName": ("name", None)})
SQLKEYS.pop("name")
               


class PanelDBElectricityMix(PanelDBBase):
    def __init__(self, parent, title, showOk = False):

        
        # specifications for overview table
        self.colLabels = ["id", _("Country"), _("Year"), _("Short name"), _("Type")]
        self.colKeys =   ["id", "Country", "Year", "name", "Type"]
        self.colUnits =  [("int", None, None),
                          ("text", None, None),
                          ("int", None, None),
                          ("text", None, None),
                          ("text", None, None)]

        self.colSizes = [60, 200, 160, 160, 184]

        # set information for the max. 3 possible filter choices
        self.filters = {"Type": None,
                        "SubType": None,
                        "SubType2": None}

        # assign editor dialog for single entry
        super(PanelDBElectricityMix, self).__init__(parent, title, "ElectricityMix", "dbelectricitymix", showOk=showOk,
                                           dialogEntryClass=DialogDBElectricityMixEntry,
                                           sqlKeys=SQLKEYS,
                                           dataKeys=DATAKEYS)

        # automatically display on init
        self.display()


