# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBFluid: Database of materials
#
#==============================================================================
#
#	Version 3.0 (2018-2019)
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2019
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

from GUI.panels.DB.panelDBBase import PanelDBBase
from panelDBFluidEntry import DialogDBFluidEntry

from constants import FLUIDTYPES, TRANSFLUIDTYPES

fluidTypeDict = dict([(ftype, i) for i, ftype in enumerate(FLUIDTYPES)])
fluidTypeChoice = [TRANSFLUIDTYPES[ftype] for ftype in FLUIDTYPES]

#keys for import from SQL to internal object
SQLKEYS =  {"id": (None, None),
            "FluidName": ("name", None),
            "FluidType": (None, None),
            "FluidCp": (None, None),
            "VaporCp": (None, None),
            "FluidDensity": (None, None),
            "SensibleHeat": (None, None),
            "LatentHeat": (None, None),
            "Viscosity": (None, None),
            "Conductivity": (None, None),
            "RefrigerantCode": (None, None),
            "FluidDataSource": (None, None),
            "SpecificMassFlow": (None, None),
            "THighP": (None, None),
            "TCond": (None, None),
            "FluidCpG": (None, None),
            "FluidComment": (None, None),
          }        # sqlkeys of data from SQL tables to be read; in case of different names for

#keys for export from internal object to GUI
DATAKEYS =  {"id": (None, None),
            "name": (None, None),
            "FluidType": (None, fluidTypeDict),
            "FluidCp": (None, None),
            "VaporCp": (None, None),
            "FluidDensity": (None, None),
            "SensibleHeat": (None, None),
            "LatentHeat": (None, None),
            "Viscosity": (None, None),
            "Conductivity": (None, None),
            "RefrigerantCode": (None, None),
            "FluidDataSource": (None, None),
            "SpecificMassFlow": (None, None),
            "THighP": (None, None),
            "TCond": (None, None),
            "FluidCpG": (None, None),
            "FluidComment": (None, None),
          }        # sqlkeys of data from SQL tables to be read; in case of different names for



class PanelDBFluid(PanelDBBase):
    def __init__(self, parent, title, showOk = False):
        
        # specifications for overview table
        self.colLabels = ["id", _("Name"), _("Type"), _("Specific heat"), _("Density"), _("Phase change temperature"), _("Refrigerant Code")]
        self.colKeys = ["id", "name", "FluidType", "FluidCp", "FluidDensity", "TCond", "RefrigerantCode"]
        self.colUnits =  [("int", None, None), ("text", None, None), ("choice", fluidTypeChoice, None),
                          ("float", "SPECIFICHEAT", 7),
                          ("float", "DENSITY", 2),
                          ("float", "TEMPERATURE", 2),
                          ("text", None, None)]

        self.colSizes = [60, 180, 120, 100, 100, 124, 80]

        # set information for the max. 3 possible filter choices
        self.filters = {"Type": ["FluidType", _("Fluid type"), _("Show only fluid of type"), TRANSFLUIDTYPES],
                        "SubType": None,
                        "SubType2": None}

        # assign editor dialog for single entry
        super(PanelDBFluid, self).__init__(parent, title, "Fluid", "dbfluid", showOk=showOk,
                                           dialogEntryClass=DialogDBFluidEntry,
                                           sqlKeys=SQLKEYS,
                                           dataKeys=DATAKEYS)

        # automatically display on init
        self.display()


