# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBFluidEntry: Editor for fluid data
#
#==============================================================================
#
#	Version 3.0, 2018-2019
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
from GUI.displayClasses import *
from constants import FLUIDTYPES, TRANSFLUIDTYPES, INFINITE

from GUI.einsteinPanel import EinsteinPanel
from GUI.editWindow import EditWindow

from modules.guiModules.DB.moduleDB import ModuleDBEntry

from auxiliary.auxiliary import toGettextAndUnicode as _U

class PanelDBFluidEntry(EinsteinPanel):

    def __init__(self, prnt, main):
        layout = {"notebook": [_('General data'),
                               _("Specific data for refrigerants")],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(0, _("General information on fluid")),
                             (0, _("Main physical properties")),
                             (1, _("Specific properties required only for refrigerants"))]}           # list of frames (pageno, title). if empty, no frame is created

        super(PanelDBFluidEntry, self).__init__(prnt, main, layout, module=None, size=(780, 600), key=None)

        self.mod = ModuleDBEntry(prnt.mod, self)


        self.addEntry("name",
                      TextEntry(self.nbPage[0], maxchars = 45, value = '',
                             label = _U("Name of the fluid"),
                             tip = _U("Name of the fluid")),
                      0)

        self.addEntry("FluidType",
                      ChoiceEntry(self.nbPage[0],
                                  values = [TRANSFLUIDTYPES[ftype] for ftype in FLUIDTYPES],
                             label = _U("Fluid type"),
                             tip = _U("type of fluid")),
                      0)
        self.addEntry("FluidDataSource",
                      TextEntry(self.nbPage[0], maxchars = 200, value = '',
                             isMultiline = True, specialHeight=100,
                             label = _U("Source of data"),
                             tip = _U("Source of data")),
                      0)
        self.addEntry("FluidComment",
                      TextEntry(self.nbPage[0], maxchars = 200, value = '',
                             isMultiline = True, specialHeight=100,
                             label = _U("Additional comments"),
                             tip = _U("Additional comments")),
                      0)
        self.addEntry("FluidCp",
                      FloatEntry(self.nbPage[0],
                              decimals = 7, minval = 0, maxval = INFINITE, value = 0.,
                              unitdict = 'SPECIFICHEAT',
                              label = _U("Specific heat in liquid state"),
                              tip = _U("Specific heat in liquid state")),
                      1)
        self.addEntry("TCond",
                      FloatEntry(self.nbPage[0],
                              decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'TEMPERATURE',
                              label = _U("Temperature of evaporation"),
                              tip = _U("Temperature of evaporation")),
                      1)
        self.addEntry("FluidCpG",
                      FloatEntry(self.nbPage[0],
                              decimals = 7, minval = 0, maxval = INFINITE, value = 0.,
                              unitdict = 'SPECIFICHEAT',
                              label = _U("Specific heat in gaseous state (vapour)"),
                              tip = _U("Specific heat in gaseous state (vapour)")),
                      1)
        self.addEntry("LatentHeat",
                      FloatEntry(self.nbPage[0],
                              decimals = 3, minval = 0, maxval = INFINITE, value = 0.,
                              unitdict = 'SPECIFICENTHALPY',
                              label = _U("Latent heat of evaporation"),
                              tip = _U("Latent heat of evaporation")),
                      1)
        self.addEntry("FluidDensity",
                      FloatEntry(self.nbPage[0],
                              decimals = 3, minval = 0, maxval = INFINITE, value = 0.,
                              unitdict = 'DENSITY',
                              label = _U("Density"),
                              tip = _U("Reference density for conversion of mass into nominal volume")),
                      1)

        # specific data for refrigerants
        self.addEntry("RefrigerantCode",
                      TextEntry(self.nbPage[1], maxchars = 6, value = '',
                             label = _U("Refrigerant code"),
                             tip = _U("Refrigerant code")),
                      2)


        self.addEntry("SpecificMassFlow",
                      FloatEntry(self.nbPage[1],
                               decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'CO2RATIO',
                               label = _U("Typical specific mass flow of refrigerant"),
                               tip = _U("Typical ratio of refrigerant flow to cooling power ")),
                      2)

        self.addEntry("THighP",
                      FloatEntry(self.nbPage[1],
                               decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("Typical outlet temperature of compressor"),
                               tip = _U(" ")),
                      2)

        self.addEntry("Viscosity",
                      FloatEntry(self.nbPage[1],
                               decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'CINEMATICVISCOSITY',
                               label = _U("Cinematic viscosity at typical working conditions"),
                               tip = _U(" ")),
                      2)

        self.addEntry("Conductivity",
                      FloatEntry(self.nbPage[1],
                               decimals = 4, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'CONDUCTIVITY',
                               label = _U("Thermal conductivity at typical working conditions"),
                               tip = _U(" ")),
                      2)
        self.addEntry("SensibleHeat",
                      FloatEntry(self.nbPage[1],
                               decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'SPECIFICENTHALPY',
                               label = _U("Senisble heat at typical working conditions"),
                               tip = _U(" ")),
                      2)

        # conclude layout
        self.conclude_layout()

##    def initPanel(self, no):
##        super(PanelDBFluidEntry, self).initPanel(no)
##
##        data = self.mod.getObjectData()
        


class DialogDBFluidEntry(EditWindow):
    def __init__(self, prnt, main):
        self.prnt = prnt
        self.mod = prnt.mod
        if self.mod.editor: return  # do not open a second dialog

        super(DialogDBFluidEntry, self).__init__(main)

        self.panel = PanelDBFluidEntry(self, main)
        self.ret = None

        self.panel.display()
        self.Show()

        self.mod.editor = self
        self.prnt.panel.Disable()

    def OnClose(self, event):
        self.mod.editor = None
        self.prnt.panel.display()
        self.prnt.panel.Enable()
        super(DialogDBFluidEntry, self).OnClose(event)
        
