# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBFuel: Database of fuels
#
#==============================================================================
#
#	Version 3.0 (2018-2019)
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2019
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

from GUI.panels.DB.panelDBBase import PanelDBBase
from panelDBFuelEntry import DialogDBFuelEntry

from constants import FUELTYPES, TRANSFUELTYPES

fuelTypeDict = dict([(ftype, i) for i, ftype in enumerate(FUELTYPES)])
fuelTypeChoice = [TRANSFUELTYPES[ftype] for ftype in FUELTYPES]

#keys for import from SQL to internal object
SQLKEYS =  {"id": (None, None),
            "FuelName":("name", None),
            "FuelType":(None, None),
            "DBFuelUnit":(None, None),
            "FuelDataSource":(None, None),
            "FuelComment":(None, None),
            "FuelLCV":(None, None),
            "FuelHCV":(None, None),
            "FuelDensity":(None, None),
            "OffgasHeatCapacity":(None, None),
            "OffgasDensity":(None, None),
            "Humidity":(None, None),
            "CombAir":(None, None),
            "Offgas":(None, None),
            "PEConvFuel":(None, None),
            "CO2ConvFuel":(None, None)}

#keys for export from internal object to GUI
DATAKEYS =  {"id": (None, None),
            "name":("name", None),
            "FuelType":(None, fuelTypeDict),
            "DBFuelUnit":(None, None),
            "FuelDataSource":(None, None),
            "FuelComment":(None, None),
            "FuelLCV":(None, None),
            "FuelHCV":(None, None),
            "FuelDensity":(None, None),
            "OffgasHeatCapacity":(None, None),
            "OffgasDensity":(None, None),
            "Humidity":(None, None),
            "CombAir":(None, None),
            "Offgas":(None, None),
            "PEConvFuel":(None, None),
            "CO2ConvFuel":(None, None)}


class PanelDBFuel(PanelDBBase):
    def __init__(self, parent, title, showOk = False):
        
        # specifications for overview table
        self.colLabels = ["id", _("Name"), _("Type"), _("Lower calorific value (LCV)"), _("Higher calorific value (HCV)"), _("Density")]
        self.colKeys =   ["id", "name",     "FuelType", "FuelLCV", "FuelHCV", "FuelDensity"]
        self.colUnits =  [("int", None, None), ("text", None, None), ("choice", fuelTypeChoice, None),
                          ("float", "SPECIFICENTHALPY", 4),
                          ("float", "SPECIFICENTHALPY", 4),
                          ("float", "DENSITY", 2)]

        self.colSizes = [60, 200, 120, 140, 140, 104]

        # set information for the max. 3 possible filter choices
        self.filters = {"Type": ["FuelType", _("Fuel type"), _("Show only fuel of type"), TRANSFUELTYPES],
                        "SubType": None,
                        "SubType2": None}

        # assign editor dialog for single entry
        super(PanelDBFuel, self).__init__(parent, title, "Fuel", "dbfuel", showOk=showOk,
                                           dialogEntryClass=DialogDBFuelEntry,
                                           sqlKeys=SQLKEYS,
                                           dataKeys=DATAKEYS)

        # automatically display on init
        self.display()


