# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBHeatPump: Database of fuels
#
#==============================================================================
#
#	Version 3.0 (2018-2019)
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2019
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

from GUI.panels.DB.panelDBBase import PanelDBBase
from panelDBHeatPumpEntry import DialogDBHeatPumpEntry

from constants import HPTYPES, TRANSHPTYPES, TRANSHPSUBTYPES

typeDict = dict([(otype, i) for i, otype in enumerate(HPTYPES)])
subTypes = TRANSHPSUBTYPES.keys()
subTypeDict = dict([(otype, i) for i, otype in enumerate(subTypes)])
typeChoice = [TRANSHPTYPES[otype] for otype in HPTYPES]
subTypeChoice = [TRANSHPSUBTYPES[otype] for otype in subTypes]

#keys for import from SQL to internal object
SQLKEYS =  {"id": (None, None),
                "HPManufacturer": (None, None),
               "HPModel": (None, None),
               "HPType": (None, None),
               "HPSubType": (None, None),
               "Reference": (None, None),
               "HPHeatCap": (None, None),
               "HPHeatCOP": (None, None),
               "HPCoolCap": (None, None),
               "HPCoolCOP": (None, None),
               "HPFuelConsum": (None, None),
               "FuelType": (None, None),
               "HPElectConsum": (None, None),
               "HPWorkFluid": (None, None),
               "HPCondTinC": (None, None),
               "HPAbsTinC": (None, None),
               "HPGenTinC": (None, None),
               "HPEvapTinC": (None, None),
               "HPConstExCoolCOP": (None, None),
               "HPCondTinH": (None, None),
               "HPAbsTinH": (None, None),
               "HPGenTinH": (None, None),
               "HPEvapTinH": (None, None),
               "HPConstExHeatCOP": (None, None),
               "HPExCoolCOP": (None, None),
               "HPThCoolCOP": (None, None),
               "HPExHeatCOP": (None, None),
               "HPThHeatCOP": (None, None),
               "HPSourceSink": (None, None),
               "HPLimDT": (None, None),
               "HPCondTmax": (None, None),
               "HPEvapTmin": (None, None),
               "HPAbsHeatMed": (None, None),
               "HPGenTmin": (None, None),
               "HPPrice": (None, None),
               "HPTurnKeyPrice": (None, None),
               "HPOandMfix": (None, None),
               "HPOandMvar": (None, None),
               "HPYearUpdate": (None, None)}

#keys for export from internal object to GUI
DATAKEYS =  SQLKEYS.copy()
DATAKEYS["HPType"] = (None, typeDict)
DATAKEYS["HPSubType"] = (None, subTypeDict)


class PanelDBHeatPump(PanelDBBase):
    def __init__(self, parent, title, showOk = False, preselection=None):

        self.typeDict = TRANSHPTYPES

        # specifications for overview table
        self.colLabels = ["id", _("Manufacturer"), _("Model"), _("Type"), _("SubType"), _("Nominal heating power"), _("Nominal cooling power")]
        self.colKeys =   ["id", "HPManufacturer", "HPModel", "HPType", "HPSubType", "HPHeatCap", "HPCoolCap"]
        self.colUnits =  [("int", None, None),
                          ("text", None, None),
                          ("text", None, None),
                          ("choice", typeChoice, None),
                          ("choice", subTypeChoice, None),
                          ("float", "POWER", 2),
                          ("float", "POWER", 2)]

        self.colSizes = [60, 140, 140, 100, 100, 110, 114]

        # set information for the max. 3 possible filter choices
        self.filters = {"Type": ["HPType", _("Heat pump type"), _("Show only equipment of type"), TRANSHPTYPES],
                        "SubType": None,
                        "SubType2": None}

        # assign editor dialog for single entry
        super(PanelDBHeatPump, self).__init__(parent, title, "HeatPump", "dbheatpump", showOk=showOk, preselection=preselection,
                                           dialogEntryClass=DialogDBHeatPumpEntry,
                                           sqlKeys=SQLKEYS,
                                           dataKeys=DATAKEYS)

        # automatically display on init
        self.display()


