# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBHeatPumpEntry: Editor for fluid data
#
#==============================================================================
#
#	Version 3.0, 2018-2019
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
from GUI.displayClasses import *
from constants import HPTYPES, HPSUBTYPES, TRANSHPTYPES, FUELTYPES, TRANSFUELTYPES, HPSOURCESINK, TRANSHPSOURCESINK, INFINITE

from GUI.einsteinPanel import EinsteinPanel
from GUI.editWindow import EditWindow

from modules.guiModules.DB.moduleDB import ModuleDBEntry

from auxiliary.auxiliary import toGettextAndUnicode as _U

class PanelDBHeatPumpEntry(EinsteinPanel):

    def __init__(self, prnt, main):

        layout = {"notebook": [_('Descriptive Data'),
                               _('Technical Data'),
                               _('Nominal Working Conditions'),
                               _('Economic Parameters')],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(0, _('Descriptive Data')),
                             (1, _("Heating Output")),
                             (1, _("Cooling Output")),
                             (1, _("Power Input")),
                             (2, _("Heating")),
                             (2, _("Cooling")),
                             (1, _("Temperature Limits Heat Source and Sink")),
                             (1, _("High Temperature Heat Source (Driving Heat)")),
                             (3, _('Economic Parameters'))]}           # list of frames (pageno, title). if empty, no frame is created

        super(PanelDBHeatPumpEntry, self).__init__(prnt, main, layout, module=None, size=(780, 600), key=None)

        self.mod = ModuleDBEntry(prnt.mod, self, objectType="HP")

        #
        # frame 0 - Descriptive Data
        #

        self.addEntry("HPManufacturer",
                      TextEntry(self.nbPage[0], maxchars = 45, value = '',
                             label = _U("Manufacturer"),
                             tip = _U("Manufacturer")),
                      0)

        self.addEntry("HPModel",
                      TextEntry(self.nbPage[0], maxchars = 45, value = '',
                             label = _U("Model"),
                             tip = _U("Model")),
                      0)

        self.addEntry("HPType",
                      ChoiceEntry(self.nbPage[0],
                               values = [U(TRANSHPTYPES[q]) for q in HPTYPES],
                               label = _U("Type"),
                               tip = _U("Heatpump type")),
                      0)

######                self.fillChoiceOfHPSubType(self.tc4.entry, q.HPType)
######
######            if q.HPSubType:
######                self.tc4.SetValue(str(q.HPSubType)) if q.HPSubType in self.getHPSubTypeList(q.HPType) else ''

        self.addEntry("HPSubType",
                      ChoiceEntry(self.nbPage[0],
                               values = [],
                               label = _U("Subtype"),
                               tip = _U("Heatpump sub type")),
                      0)

##        self.addEntry("HPSourceSink",
##                      ChoiceEntry(self.nbPage[0],
##                                values = [TRANSHPSOURCESINK[q] for q in HPSOURCESINK],
##                                label = _U("Heat source and sink"),
##                                tip = _U("Heat source and sink")),
##                      0)
##
        self.addEntry("HPWorkFluid",
                      TextEntry(self.nbPage[0], maxchars = 45, value = '',
                              label = _U("Refrigerant / absorbent refrigerant pair"),
                              tip = _U("Refrigerant / absorbent refrigerant pair")),
                      0)


        self.addEntry("Reference",
                      TextEntry(self.nbPage[0], maxchars = 200, value = '',
                             isMultiline = True, specialHeight=100,
                             label = _U("Reference"),
                             tip = _U("Source of data")),
                      0)

        #
        # tab 2 - Technical data
        #
        self.addEntry("HPHeatCap",
                      FloatEntry(self.nbPage[1],
                              decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'POWER',
                              label = _U("Nominal heating capacity"),
                              tip = _U("Nominal heating capacity")),
                      1)

        self.addEntry("HPHeatCOP",
                      FloatEntry(self.nbPage[1],
                              decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'FRACTION',
                              label = _U("Nominal COP for heating mode"),
                              tip = _U("Nominal COP for heating mode")),
                      1)

        self.addEntry("HPCoolCap",
                      FloatEntry(self.nbPage[1],
                              decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'POWER',
                              label = _U("Nominal cooling capacity"),
                              tip = _U("Nominal cooling capacity")),
                      2)

        self.addEntry("HPCoolCOP",
                      FloatEntry(self.nbPage[1],
                              decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'FRACTION',
                              label = _U("Nominal COP for cooling mode"),
                              tip = _U("Nominal COP for cooling mode")),
                      2)

        self.addEntry("HPFuelConsum",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'POWER',
                               label = _U("Nominal fuel consumption"),
                               tip = _U("Nominal fuel consumption")),
                      3)

        self.addEntry("FuelType",
                      ChoiceEntry(self.nbPage[1],
                                values = [U(TRANSFUELTYPES[q]) for q in FUELTYPES],
                                label = _U("Fuel type"),
                                tip = _U("Fuel type")),
                      3)

        self.addEntry("HPElectConsum",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'POWER',
                               label = _U("Nominal electrical power consumption"),
                               tip = _U("Nominal electrical power consumption")),
                      3)


##        self.addEntry("HPAbsTinH",
##                      ,
##                      4)

        self.addEntry("HPGenTinH",
                      FloatEntry(self.nbPage[2],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("inlet temperature to the generator"),
                               tip = _U("inlet temperature to the generator")),
                      4)

        self.addEntry("HPCondTinH",
                      FloatEntry(self.nbPage[2],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("inlet temperature to the condenser (and absorber)"),
                               tip = _U("inlet temperature to the condenser (and absorber)")),
                      4)

        self.addEntry("HPEvapTinH",
                      FloatEntry(self.nbPage[2],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("inlet temperature to the evaporator"),
                               tip = _U("inlet temperature to the evaporator")),
                      4)

        self.addEntry("HPConstExHeatCOP",
                      FloatEntry(self.nbPage[2],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATUREDIFF',
                               label = _U("With of temperature range for which a constant exergetic COP approximation is valid"),
                               tip = _U("Temperature range around the nominal temperatures for which the constant exergetic COP approximation is valid (e.g. +-20 K)"))
,
                      4)

        self.addEntry("HPExHeatCOP",
                      FloatEntry(self.nbPage[2],
                               isStatic = True,
                               decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'FRACTION',
                               label = _U("Exergetic efficiency"),
                               tip = _U("Calculated from the nominal and theoretical COP at the manufact. catalogue nominal conditions and applied as a constant in extrapolation for other working conditions (see next point).")),
                      4)

        self.addEntry("HPThHeatCOP",
                      FloatEntry(self.nbPage[2],
                               isStatic = True,
                               decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'FRACTION',
                               label = _U("Carnot efficiency"),
                               tip = _U("Carnot COP for heating mode at nominal conditions")),
                      4)

##        self.addEntry("HPAbsTinC",
##                      
##                      4)

        self.addEntry("HPGenTinC",
                       FloatEntry(self.nbPage[2],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("inlet temperature to the generator"),
                               tip = _U("inlet temperature to the generator")),
                      5)

        self.addEntry("HPCondTin",
                      FloatEntry(self.nbPage[2],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("inlet temperature to the condenser (and absorber)"),
                               tip = _U("inlet temperature to the condenser (and absorber)")),
                      5)

        self.addEntry("HPEvapTinC",
                      FloatEntry(self.nbPage[2],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("inlet temperature to the evaporator"),
                               tip = _U("inlet temperature to the evaporator")),
                      5)

        self.addEntry("HPConstExCoolCOP",
                      FloatEntry(self.nbPage[2],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATUREDIFF',
                               label = _U("With of temperature range for which a constant exergetic COP approximation is valid"),
                               tip = _U("Temperature range around the nominal temperatures for which the constant exergetic COP approximation is valid (e.g. +-20 K)")),
                      5)


        self.addEntry("HPExCoolCOP",
                      FloatEntry(self.nbPage[2],
                               isStatic = True,
                               decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'FRACTION',
                               label = _U("Exergetic efficiency"),
                               tip = _U("Calculated from the nominal and theoretical COP at the manufact. catalogue nominal conditions and applied as a constant in extrapolation for other working conditions (see next point).")),
                      5)

        self.addEntry("HPThCoolCOP",
                      FloatEntry(self.nbPage[2],
                               isStatic = True,
                               decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'FRACTION',
                               label = _U("Carnot efficiency"),
                               tip = _U("Carnot COP for cooling mode at nominal conditions (see next point).")),
                      5)

        #
        # tab 3 - Heat source / sink
        #

        self.addEntry("HPLimDT",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATUREDIFF',
                               label = _("Maximum acceptable temperature difference between evaporator and condenser"),
                               tip = _U("Maximum acceptable temperature difference between evaporator and condenser temperatures (inlet temperature)- working limit")),
                      6)

        self.addEntry("HPCondTmax",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("Maximum condensing (and absorption) temperature"),
                               tip = _U("Maximum condensing (and absorption) temperature (inlet temperature) - working limit")),
                      6)

        self.addEntry("HPEvapTmin",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("Minimum evaporating temperature"),
                               tip = _U("Minimum evaporating temperature (inlet temperature) - working limit")),
                      6)

######            if q.HPAbsHeatMed is not None:
######                self.tc30.SetValue(str(q.HPAbsHeatMed)) if q.HPAbsHeatMed in self.getFluidNameList() else ''

        self.addEntry("HPAbsHeatMed",
                      ChoiceEntry(self.nbPage[1],
                                values = [],
                                label = _U("Heat transport medium used for heat supply to the generator"),
                                tip = _U("Heat transport medium used for heat supply to the generator")),
                      7)

        self.addEntry("HPGenTmin",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'TEMPERATURE',
                               label = _U("Minimum required inlet temperature to the generator"),
                               tip = _U("Minimum required inlet temperature to the generator")),
                      7)

        #
        # tab 4 - Economic Parameters
        #

        self.addEntry("HPPrice",
                      FloatEntry(self.nbPage[3],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'PRICE',
                               label = _U("Equipment price at factory applied installer's discount"),
                               tip = _U("Equipment price at factory applied installer's discount")),
                      8)

        self.addEntry("HPTurnKeyPrice",
                      FloatEntry(self.nbPage[3],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'PRICE',
                               label = _U("Price of installed equipment"),
                               tip = _U("Price of installed equipment (including work, additional accessories, pumps, regulation, etc)")),
                      8)

        self.addEntry("HPOandMfix",
                      FloatEntry(self.nbPage[3],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'UNITPRICE',
                               label = _U("Annual operational and maintenance fixed costs"),
                               tip = _U("Annual operational and maintenance fixed costs (approximate average per kW heating)")),
                      8)

        self.addEntry("HPOandMvar",
                      FloatEntry(self.nbPage[3],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'ENERGYTARIFF',
                               label = _U("Annual operational and maintenance variable costs dependant on usage"),
                               tip = _U("Annual operational and maintenance variable costs dependant on usage (approximate average per MWh heating)")),
                      8)

        self.addEntry("HPYearUpdate",
                      FloatEntry(self.nbPage[3],
                               decimals = 0, minval = 1900, maxval = 2100, value = 2010,
                               label = _U("Year of last update of the economic data"),
                               tip = _U("Year of last update of the economic data")),
                      8)

        # conclude layout
        self.conclude_layout()

        self.Bind(wx.EVT_CHOICE, self.onEntryType, self.entry["HPType"].entry)
        
    def onEntryType(self, event):
        if self.blockEventHandlers: return
        
        iType = self.entry["HPType"].GetValue()
        subTypeChoices = self.mod.typeChange(iType)
        self.entry["HPSubType"].SetValue(subTypeChoices)
        self.entry["HPSubType"].SetValue(None)

        self.showAppropriateEntries(iType)

    def display(self):
        super(PanelDBHeatPumpEntry, self).display()
        self.showAppropriateEntries(self.data["HPType"])

    def showAppropriateEntries(self, HPType):
        thermalHP = (HPType==1)

        self.entry["HPGenTinC"].Show(thermalHP)
        self.entry["HPGenTinH"].Show(thermalHP)

        self.entry["HPAbsHeatMed"].Show(thermalHP)
        self.entry["HPGenTmin"].Show(thermalHP)

        self.frame[7].Show(thermalHP)
    
class DialogDBHeatPumpEntry(EditWindow):
    def __init__(self, prnt, main):
        self.prnt = prnt
        self.mod = prnt.mod
        if self.mod.editor: return  # do not open a second dialog

        super(DialogDBHeatPumpEntry, self).__init__(main)

        self.panel = PanelDBHeatPumpEntry(self, main)
        self.ret = None

        self.panel.display()
        self.Show()

        self.mod.editor = self
        self.prnt.panel.Disable()

    def OnClose(self, event):
        wx.SafeYield()              # seems to be important. otherwise crashes occur.
        self.mod.editor = None
        self.prnt.panel.display()
        self.prnt.panel.Enable()
        super(DialogDBHeatPumpEntry, self).OnClose(event)
        
