# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBSolarThermal: Database of fuels
#
#==============================================================================
#
#	Version 3.0 (2018-2019)
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2019
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

from GUI.panels.DB.panelDBBase import PanelDBBase
from panelDBSolarThermalEntry import DialogDBSolarThermalEntry

from constants import STTYPES, TRANSSTTYPES

typeDict = dict([(otype, i) for i, otype in enumerate(STTYPES)])
typeChoice = [TRANSSTTYPES[otype] for otype in STTYPES]

#keys for import from SQL to internal object
SQLKEYS =  {"id": (None, None),

                "STManufacturer": (None, None),
               "STModel": (None, None),
               "STType": (None, None),
               "STReference": (None, None),
               "STPnomColl": (None, None),
               "STc0": (None, None),
               "STc1": (None, None),
               "STc2": (None, None),
               "K50L": (None, None),
               "K50T": (None, None),
               "STMassFlowRate": (None, None),
               "STLengthGross": (None, None),
               "STHeightGross": (None, None),
               "STAreaGross": (None, None),
               "STLengthAper": (None, None),
               "STHeightAper": (None, None),
               "STAreaAper": (None, None),
               "STAreaFactor": (None, None),
               "STWeightFactor": (None, None),
               "STUnitPrice300kW": (None, None),
               "STUnitTurnKeyPrice30kW": (None, None),
               "STUnitTurnKeyPrice300kW": (None, None),
               "STUnitTurnKeyPrice3000kW": (None, None),
               "STOMUnitFix": (None, None),
               "STYearUpdate": (None, None)}


#keys for export from internal object to GUI
DATAKEYS =  SQLKEYS.copy()
DATAKEYS["STType"] = (None, typeDict)

class PanelDBSolarThermal(PanelDBBase):
    def __init__(self, parent, title, showOk = False):
        
        self.colLabels = ["id", _("Manufacturer"), _("Model"), _("Type"), _("Nominal heating power")]
        self.colKeys =   ["id", "STManufacturer", "STModel", "STType", "STPnomColl"]
        self.colUnits =  [("int", None, None),
                          ("text", None, None),
                          ("text", None, None),
                          ("choice", typeChoice, None),
                          ("float", "POWER", 2)]

        self.colSizes = [60, 200, 180, 180, 144]

        # set information for the max. 3 possible filter choices
        self.filters = {"Type": ["STType", _("Collector type"), _("Show only collectors of type"), TRANSSTTYPES],
                        "SubType": None,
                        "SubType2": None}

        # assign editor dialog for single entry
        super(PanelDBSolarThermal, self).__init__(parent, title, "Fuel", "dbsolarthermal", showOk=showOk,
                                           dialogEntryClass=DialogDBSolarThermalEntry,
                                           sqlKeys=SQLKEYS,
                                           dataKeys=DATAKEYS)

        # automatically display on init
        self.display()


