# -*- coding: utf-8 -*-
#==============================================================================
#
#       E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#       PanelDBSolarThermalEntry: Editor for fluid data
#
#==============================================================================
#
#	Version 3.0, 2018-2019
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
from GUI.displayClasses import *
from GUI.dataEntry.tableEntry import TableEntry
from constants import STTYPES, TRANSSTTYPES, INFINITE

from GUI.einsteinPanel import EinsteinPanel
from GUI.editWindow import EditWindow

from modules.guiModules.DB.moduleDB import ModuleDBEntry

from auxiliary.auxiliary import toGettextAndUnicode as _U

class PanelDBSolarThermalEntry(EinsteinPanel):

    def __init__(self, prnt, main):

        layout = {"notebook": [_('Descriptive data'),
                               _("Technical Data"),
                               _("Economic Parameters")],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(0, _("General Data")),
                             (1, _("Collector efficiency parameters")),
                             (1, _("Collector dimension and weight")),
                             (2, _("Economic Parameters"))]}           # list of frames (pageno, title). if empty, no frame is created

        super(PanelDBSolarThermalEntry, self).__init__(prnt, main, layout, module=None, size=(780, 600), key=None)

        self.mod = ModuleDBEntry(prnt.mod, self, objectType="ST")

        self.addEntry("STManufacturer",
                      TextEntry(self.nbPage[0], maxchars = 20, value = '',
                             label = _U("Solarthermal Manufacturer"),
                             tip = _U("Solarthermal Manufacturer")),
                      0)

        self.addEntry("STModel",
                      TextEntry(self.nbPage[0], maxchars = 20, value = '',
                             label = _U("Solarthermal Model"),
                             tip = _U("Solarthermal Model")),
                      0)

        self.addEntry("STType",
                      ChoiceEntry(self.nbPage[0],
                               values = [U(TRANSSTTYPES[q]) for q in STTYPES],
                               label = _U("Solarthermal Type"),
                               tip = _U("Solarthermal Type")),
                      0)

        self.addEntry("STReference",
                      TextEntry(self.nbPage[0], maxchars = 200, value = '',
                             isMultiline = True,
                             label = _U("Reference"),
                             tip = _U("Source of data")),
                      0)



        #
        # frame 1 - Technical data
        #


        self.addEntry("STPnomColl",
                      FloatEntry(self.nbPage[1],
                              isStatic = True,
                              decimals = 3, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'POWER',
                              label = _U("Nominal thermal power"),
                              tip = _U("Nominal thermal power is defined as 0.7 kW/m2 of effective (aperture) area")),
                      1)

        self.addEntry("STc0",
                      FloatEntry(self.nbPage[1],
                              decimals = 4, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'FRACTION',
                              label = _U("Optical efficiency"),
                              tip = _U("Optical efficiency") + u" " + _U("based on effective (aperture) area")),
                      1)

        self.addEntry("STc1",
                      FloatEntry(self.nbPage[1],
                              decimals = 4, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'HEATLOSSCOEFF',
                              label = _U("Linear thermal loss coefficient"),
                              tip = _U("Linear thermal loss coefficient") + u" " + _U("based on effective (aperture) area")),
                      1)

        self.addEntry("STc2",
                      FloatEntry(self.nbPage[1],
                              decimals = 6, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'HEATLOSSCOEFF2',
                              label = _U("Quadratic thermal loss coefficient"),
                              tip = _U("Quadratic thermal loss coefficient") + u" " + _U("based on effective (aperture) area")),
                      1)

        self.addEntry("K50L",
                      FloatEntry(self.nbPage[1],
                              decimals = 4, minval = -INFINITE, maxval = INFINITE, value = 0.,
                              unitdict = 'FRACTION',
                              label = _U("Incidence angle correction factor at 50° (longitudinal)"),
                              tip = _U("Ratio of efficiency at given incidence angle to efficiency at normal incidence")),
                      1)

        self.addEntry("K50T",
                      FloatEntry(self.nbPage[1],
                               decimals = 4, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'FRACTION',
                               label = _U("Incidence angle correction factor at 50° (transversal)"),
                               tip = _U("Ratio of efficiency at given incidence angle to efficiency at normal incidence")),
                      1)


        self.addEntry("STMassFlow",
                      FloatEntry(self.nbPage[1],
                               decimals = 1, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'MASSORVOLUMEFLOW',
                               label = _U("Nominal collector mass flow rate"),
                               tip = _U("Collector mass flow rate corresponding to the data of the efficiency curve")),
                      1)




        self.addEntry("STLengthGross",
                      FloatEntry(self.nbPage[1],
                               decimals = 4, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'LENGTH',
                               label = _U("Gross collector width"),
                               tip = _U("Width of collector in horizontal extension, including housing")),
                      2)


        self.addEntry("STHeightGross",
                      FloatEntry(self.nbPage[1],
                               decimals = 4, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'LENGTH',
                               label = _U("Gross collector height"),
                               tip = _U("Height of collector, including housing")),
                      2)


        self.addEntry("STAreaGross",
                      FloatEntry(self.nbPage[1],
                               decimals = 4, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'AREA',
                               label = _U("Gross collector area"),
                               tip = _U("Gross collector area, including housing")),
                      2)


        self.addEntry("STLengthAper",
                      FloatEntry(self.nbPage[1],
                               decimals = 4, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'LENGTH',
                               label = _U("Collector width (effective)"),
                               tip = _U("Effective (aperture) width of collector in horizontal extension")),
                      2)


        self.addEntry("STHeightAper",
                      FloatEntry(self.nbPage[1],
                               decimals = 4, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'LENGTH',
                               label = _U("Collector height (effective)"),
                               tip = _U("Effective (aperture) height of collector")),
                      2)


        self.addEntry("STAreaAper",
                      FloatEntry(self.nbPage[1],
                               decimals = 4, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'AREA',
                               label = _U("Effective collector area"),
                               tip = _U("Effective (aperture) area of the collector")),
                      2)


        self.addEntry("STAreaFactor",
                      FloatEntry(self.nbPage[1],
                               decimals = 4, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'FRACTION',
                               label = _U("Area utilisation ratio"),
                               tip = _U("Ratio of effective to gross collector area")),
                      2)

        self.addEntry("STWeightFactor",
                      FloatEntry(self.nbPage[1],
                               decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'MASSPERAREA',
                               label = _U("Specific collector weight"),
                               tip = _U("Collector weight per unit of effective (aperture) area")),
                      2)


        #
        # tab 3 - Economic Parameters
        #

        self.addEntry("STUnitPrice300kW",
                      FloatEntry(self.nbPage[2],
                               decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'UNITPRICE',
                               label = _U("Collector cost 300kW"),
                               tip = _U("Collector cost for a system size of 300 kW")),
                      3)


        # Price curve as a function of system size
        titles = [_("System Size"), _("Unit"), " 30kW", " 300kW", " 3000kW"]

        self.addEntry("TurnKeyPriceTable",
                      TableEntry(self.nbPage[2], titles, None, size=wx.Size(780, 160),
                                 rowSize=24, colSize=120, colLabelSize=32,
                                 hAlignment = [wx.ALIGN_LEFT, wx.ALIGN_CENTRE, wx.ALIGN_RIGHT, wx.ALIGN_RIGHT, wx.ALIGN_RIGHT],
                                 unitSelectors = [("float", "UNITPRICE", 0)],
                                 unitCol = 1,
                                 readOnly = [True, False, False, False, False]),
                      3)

##        self.entry["UPHComponents"].setRowSize(0, 48) ###TODO: increase height o title row
        self.entry["TurnKeyPriceTable"].setColSize(0, 292)
        self.entry["TurnKeyPriceTable"].setColSize(1, 80)
        self.entry["TurnKeyPriceTable"].setPopUpMenu([])
##        self.entry["QWXTable"].setExternalEventHandler(self.onTableClickQWX)


        self.addEntry("STOMUnitFix",
                      FloatEntry(self.nbPage[2],
                               decimals = 2, minval = -INFINITE, maxval = INFINITE, value = 0.,
                               unitdict = 'UNITPRICE',
                               label = _U("O&M Costs"),
                               tip = _U("Yearly O&M costs per unit of installed power")),
                      3)

        self.addEntry("STYearUpdate",
                      FloatEntry(self.nbPage[2],
                               decimals = 0, minval = 1900, maxval = 2100, value = 2010,
                               label = _U("Year of last update of the economic data"),
                               tip = _U("Year of last update of the economic data")),
                      3)

         # conclude layout
        self.conclude_layout()

##    def initPanel(self, no):
##        super(PanelDBSolarThermalEntry, self).initPanel(no)
##
##        data = self.mod.getObjectData()
        


class DialogDBSolarThermalEntry(EditWindow):
    def __init__(self, prnt, main):
        self.prnt = prnt
        self.mod = prnt.mod
        if self.mod.editor: return  # do not open a second dialog

        super(DialogDBSolarThermalEntry, self).__init__(main)

        self.panel = PanelDBSolarThermalEntry(self, main)
        self.ret = None

        self.mod.activeObject.name = self.mod.activeObject.STManufacturer + ": " + self.mod.activeObject.STModel
        self.panel.display()
        self.Show()

        self.mod.editor = self
        self.prnt.panel.Disable()

    def OnClose(self, event):
        self.mod.editor = None
        self.prnt.panel.display()
        self.prnt.panel.Enable()
        super(DialogDBSolarThermalEntry, self).OnClose(event)
        
