# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelECO: hourly data
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from GUI.einsteinPanel import EinsteinPanel
from GUI.dataEntry.tableEntry import TableEntry
from lib.GUI.plot.XYPlot import XYPlot
from modules.guiModules.ECO.moduleECOPanel import ModuleECOPanel
from GUI.status import Status

import wx

class PanelECO(EinsteinPanel):

    def __init__(self, parent, main):

        # generate list of notebook pages and frames
        layout = {"notebook": None,          # list of notebook-page titles. if empty or None, no notebook is created
                  "frames": [(None, _("Results")),
                             (None, _(" "))]}           # list of frames (pageno, title). if empty, no frame is created
                                                            # pageno = None for frames on the main panel
        super(PanelECO, self).__init__(parent, main, layout)

        self.mod = ModuleECOPanel(self, "ECO")


        # Results

        titles = [_("Parameter"), _("Unit"), _("Value")]
        hAlignment = [wx.ALIGN_LEFT, wx.ALIGN_CENTRE, wx.ALIGN_RIGHT]
        readOnly = [True, False, True]

        self.addEntry("Results",
                      TableEntry(self, titles, changeData = self.mod.changeData, size=wx.Size(792, 160),
                                 rowSize=24, colSize=120, colLabelSize=32,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = [("float", "POWER", 2)]*20,
                                 unitCol = 1),
                      0)

        self.entry["Results"].setColSize(0, 540)
        self.entry["Results"].setPopUpMenu([])
##        self.entry["Investment"].setExternalEventHandler(self.onTableClick)

        # Plot

        plotSize = (780, 360)        
        self.addEntry("Plot",
                      XYPlot(self, size=plotSize),
                      1)

        self.plot = self.entry["Plot"]        
        self.plot.xAxis(label=_("Year"), xmin=0)
        self.plot.yAxis(label=_("Cash Flow (in prices of year 0)/ NPV"))

        # conclude layout
        self.conclude_layout()


##    def fillPanel(self):        
##        super(PanelECO, self).fillPanel()

