# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.iee-einstein.org/" target="_blank">www.iee-einstein.org</a>)
#
#------------------------------------------------------------------------------
#
#	PanelQ9: Economic data
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
import wx
from GUI.status import *
from database.dbio import check
import database.dbio as dbio
from GUI.displayClasses import FloatEntry, FieldSizes
from GUI.fonts import FontProperties
from auxiliary.auxiliary import toGettextAndUnicode as _U
from modules.economy.dataECO import DataECO

from globals import logger

# constants that control the default sizes
# 1. font sizes
TYPE_SIZE_NORMAL        =   9
TYPE_SIZE_LEFT          =   TYPE_SIZE_NORMAL
TYPE_SIZE_MIDDLE        =   TYPE_SIZE_NORMAL
TYPE_SIZE_RIGHT         =   TYPE_SIZE_NORMAL
TYPE_SIZE_TITLES        =  10

# 2. field sizes
HEIGHT                  =  32
HEIGHT_RIGHT            =  32

LABEL_WIDTH_LEFT_TOP    = 300
LABEL_WIDTH_LEFT_MIDDLE =  20
LABEL_WIDTH_RIGHT       = 200

DATA_ENTRY_WIDTH_RIGHT  = 100
DATA_ENTRY_WIDTH_LEFT   = 100

UNITS_WIDTH             =  90

# 3. vertical separation between fields
VSEP_LEFT               =   2
VSEP_RIGHT              =   2

ORANGE = '#FF6000'
TITLE_COLOR = ORANGE

cu = Status.ProjectCurrency

class PanelECO0(wx.Panel):
    def __init__(self, parent, main):
        self.main = main
        self._init_ctrls(parent)
        self.__do_layout()
        self.prnt = parent


    def _init_ctrls(self, parent):

#------------------------------------------------------------------------------
#--- UI setup
#------------------------------------------------------------------------------

        wx.Panel.__init__(self, id=-1, name='PanelQ9', parent=parent,
              pos=wx.Point(0, 0), size=wx.Size(780, 580), style=0)
        self.Hide()

        # access to font properties object
        fp = FontProperties()

        self.notebook = wx.Notebook(self, -1, style=0)
        self.notebook.SetFont(fp.getFont())

        self.page0 = wx.Panel(self.notebook)
        self.notebook.AddPage(self.page0, _U('Economic Parameters'))

        self.page1 = wx.Panel(self.notebook)
        self.notebook.AddPage(self.page1, _U('Operation and maintenance'))
        #
        # frames
        #
##        self.frame_management = wx.StaticBox(self.page0, -1, _U("Management of energetic services"))
##        self.frame_management.SetForegroundColour(TITLE_COLOR)
##        self.frame_management.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.frame_parameters = wx.StaticBox(self.page0, -1, _U("Inflation rates and time frame of economic analysis"))
        self.frame_parameters.SetForegroundColour(TITLE_COLOR)
        self.frame_parameters.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.frame_external = wx.StaticBox(self.page0, -1, _U("External financing and interest rates"))
        self.frame_external.SetForegroundColour(TITLE_COLOR)
        self.frame_external.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.frame_costs = wx.StaticBox(self.page1, -1, _U("Yearly operation and maintenance costs"))
        self.frame_costs.SetForegroundColour(TITLE_COLOR)
        self.frame_costs.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))
        #
        # labels
        #

        self.labelSpc0= wx.StaticText(self.page1, -1,'')
        self.label_14 = wx.StaticText(self.page1, -1, _U("Total costs\n[%s]"%cu))
        self.label_15 = wx.StaticText(self.page1, -1, _U("Utilities and\noperating materials\n[%s]"%cu))
        self.label_16 = wx.StaticText(self.page1, -1, _U("Labour costs\n[%s]"%cu))
        self.label_17 = wx.StaticText(self.page1, -1, _U("External costs\n[%s]"%cu))
        self.label_18 = wx.StaticText(self.page1, -1, _U("Regulatory compliance,\ninsurance and\nfuture liability costs\n[%s]"%cu))

        # set font for frames
        # 1. save actual font parameters on the stack
        fp.pushFont()
        # 2. change size and weight
        fp.changeFont(size=TYPE_SIZE_TITLES, weight=wx.BOLD)
        self.frame_costs.SetFont(fp.getFont())
        self.frame_parameters.SetFont(fp.getFont())
        self.frame_external.SetFont(fp.getFont())
##        self.frame_management.SetFont(fp.getFont())
        # set font for labels
        fp.changeFont(size=8, weight = wx.NORMAL)
        self.label_14.SetFont(fp.getFont())
        self.label_15.SetFont(fp.getFont())
        self.label_16.SetFont(fp.getFont())
        self.label_17.SetFont(fp.getFont())
        self.label_18.SetFont(fp.getFont())
        # restore font
        fp.popFont()
        #
        # left tab controls
        # tab 0 - parameters and management
        #
        fs = FieldSizes(wHeight=HEIGHT,wLabel=LABEL_WIDTH_LEFT_TOP,
                       wData=DATA_ENTRY_WIDTH_LEFT,wUnits=UNITS_WIDTH)

        # tab 0 top side. parameters
        self.tc1 = FloatEntry(self.page0, decimals=4, minval=0., maxval=1.e+4, value=0.,
                              unitdict='GROWTHRATE',
                              label=_U("General inflation rate"),
                              tip=_U("Specify the rate of prices variation estimated for the useful life of the installations (e.g. in the next 15-20 years)"),
                              defaultDisplayUnit="%/a")
        self.tc2 = FloatEntry(self.page0, decimals=4, minval=0., maxval=1.e+4, value=0.,
                              unitdict='GROWTHRATE',
                              label=_U("Rate of increment of energy prices"),
                              tip=_U("Specify the rate of prices variation estimated for the useful life of the installations (e.g. in the next 15-20 years)"),
                              defaultDisplayUnit="%/a")
        self.tc3 = FloatEntry(self.page0, decimals=4, minval=0., maxval=1.e+4, value=0.,
                              unitdict='GROWTHRATE',
                              label=_U("Nominal rate of interest for financing of installations"),
                              tip=_U("This rate is used as discount rate for computing the net present value (NPV) of cash flows and the benefit cost ratio"),
                              defaultDisplayUnit="%/a")
        self.tc4 = FloatEntry(self.page0, decimals=4, minval=0., maxval=1.e+4, value=0.,
                              unitdict='FRACTION',
                              label=_U("Percentage of external financing for installations"),
                              tip=_U("Percentage of the external financing for the inversions"),
                              defaultDisplayUnit="%")
        self.tc4b = FloatEntry(self.page0, decimals=4, minval=0., maxval=1.e+4, value=0.,
                              unitdict='GROWTHRATE',
                              label=_U("Company specific discount rate"),
                              tip=_U("Nomial rate of interest for re-investment of intermediate positive cash flows. Used in the calculation of MIRR"),
                              defaultDisplayUnit="%/a")
        self.tc5 = FloatEntry(self.page0, decimals=1, minval=0., maxval=1.e+4, value=0.,
                              unitdict='LONGTIME',
                              label=_U("Time for economic amortization of installations"),
                              tip=_U("Amortization time"))

        fs = FieldSizes(wLabel=LABEL_WIDTH_LEFT_MIDDLE)

        # tab 0 bottom side. energy management
##        self.checkBox6 = wx.CheckBox(self.page0, -1, _U("An energy management system is already implemented"))
##        self.checkBox6.SetValue(False)
##        self.checkBox6.SetFont(fp.getFont())
##        self.checkBox7 = wx.CheckBox(self.page0, -1, _U("The energy management is externalized"))
##        self.checkBox7.SetValue(False)
##        self.checkBox7.SetFont(fp.getFont())


        #
        # right tab controls
        # tab 1 - operation and maintenance costs
        #
        fs = FieldSizes(wHeight=HEIGHT,wLabel=LABEL_WIDTH_RIGHT,
                       wData=DATA_ENTRY_WIDTH_RIGHT,wUnits=UNITS_WIDTH)
        

        self.label_9 = wx.StaticText(self.page1, -1, _U("General maintenance"))
        self.tc10_1 = wx.TextCtrl(self.page1,-1, '')
        self.tc10_2 = wx.TextCtrl(self.page1,-1, '')
        self.tc10_3 = wx.TextCtrl(self.page1,-1, '')
        self.tc10_4 = wx.TextCtrl(self.page1,-1, '')
        self.tc10_5 = wx.TextCtrl(self.page1,-1, '')
        
        self.label_10 = wx.StaticText(self.page1, -1, _U("Buildings"))
        self.tc11_1 = wx.TextCtrl(self.page1,-1, '')
        self.tc11_2 = wx.TextCtrl(self.page1,-1, '')
        self.tc11_3 = wx.TextCtrl(self.page1,-1, '')
        self.tc11_4 = wx.TextCtrl(self.page1,-1, '')
        self.tc11_5 = wx.TextCtrl(self.page1,-1, '')
        
        self.label_11 = wx.StaticText(self.page1, -1, _U("Machines and equipment for processes"))
        self.tc12_1 = wx.TextCtrl(self.page1,-1, '')
        self.tc12_2 = wx.TextCtrl(self.page1,-1, '')
        self.tc12_3 = wx.TextCtrl(self.page1,-1, '')
        self.tc12_4 = wx.TextCtrl(self.page1,-1, '')
        self.tc12_5 = wx.TextCtrl(self.page1,-1, '')
        
        self.label_12 = wx.StaticText(self.page1, -1, _U("Generation and distribution of heat and cold"))
        self.tc13_1 = wx.TextCtrl(self.page1,-1, '')
        self.tc13_2 = wx.TextCtrl(self.page1,-1, '')
        self.tc13_3 = wx.TextCtrl(self.page1,-1, '')
        self.tc13_4 = wx.TextCtrl(self.page1,-1, '')
        self.tc13_5 = wx.TextCtrl(self.page1,-1, '')
        
        self.label_13 = wx.StaticText(self.page1, -1, _U("Total"))
        self.tc14_1 = wx.TextCtrl(self.page1,-1, '')
        self.tc14_2 = wx.TextCtrl(self.page1,-1, '')
        self.tc14_3 = wx.TextCtrl(self.page1,-1, '')
        self.tc14_4 = wx.TextCtrl(self.page1,-1, '')
        self.tc14_5 = wx.TextCtrl(self.page1,-1, '')

        # restore font
        fp.pushFont()
        #
        fp.changeFont(size=8,weight=wx.NORMAL)
        self.label_9.SetFont(fp.getFont())
        self.label_10.SetFont(fp.getFont())
        self.label_11.SetFont(fp.getFont())
        self.label_12.SetFont(fp.getFont())
        self.label_13.SetFont(fp.getFont())

        # restore font
        fp.popFont()
        #
        #
        # buttons
        #
        self.buttonOK = wx.Button(self,wx.ID_OK,"OK")
        self.Bind(wx.EVT_BUTTON, self.OnButtonOK, self.buttonOK)
        self.buttonOK.SetDefault()

        self.buttonCancel = wx.Button(self,wx.ID_CANCEL,"Cancel")
        self.Bind(wx.EVT_BUTTON, self.OnButtonCancel, self.buttonCancel)

        # recover previous font parameters from the stack
        fp.popFont()

    def __do_layout(self):
        flagText = wx.ALIGN_CENTER_VERTICAL|wx.TOP
        flagLabel = wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT

        # global sizer for panel. Contains notebook w/two tabs + buttons Cancel and Ok
        sizerGlobal = wx.BoxSizer(wx.VERTICAL)
        
        # sizer for left tab
        # tab 0, parameters and management
        # top part: parameters
        
        sizerP0PartTop = wx.StaticBoxSizer(self.frame_parameters, wx.VERTICAL)
        sizerP0PartTop.Add(self.tc1, 0, flagText, VSEP_LEFT)
        sizerP0PartTop.Add(self.tc2, 0, flagText, VSEP_LEFT)
        sizerP0PartTop.Add(self.tc5, 0, flagText, VSEP_LEFT)

        sizerP0PartBot = wx.StaticBoxSizer(self.frame_external, wx.VERTICAL)
        sizerP0PartBot.Add(self.tc3, 0, flagText, VSEP_LEFT)
        sizerP0PartBot.Add(self.tc4, 0, flagText, VSEP_LEFT)
        sizerP0PartBot.Add(self.tc4b, 0, flagText, VSEP_LEFT)

        sizerPage0= wx.BoxSizer(wx.VERTICAL)
        sizerPage0.Add(sizerP0PartTop,1,wx.EXPAND|wx.TOP,10)
        sizerPage0.Add(sizerP0PartBot,1,wx.EXPAND|wx.TOP,10)
        self.page0.SetSizer(sizerPage0)
        #
        # sizer for right tab
        # tab 1, operation and maintenance costs
        sizerPage1 = wx.StaticBoxSizer(self.frame_costs, wx.VERTICAL)
        sizerP1 = wx.FlexGridSizer(6, 6, 1, 2) #r,c,vsep,hsep
        sizerP1.Add(self.labelSpc0,0,0,0)
        sizerP1.Add(self.label_14,0,flagText,0)
        sizerP1.Add(self.label_15,0,flagText,0)
        sizerP1.Add(self.label_16,0,flagText,0)
        sizerP1.Add(self.label_17,0,flagText,0)
        sizerP1.Add(self.label_18,0,flagText,0)
        sizerP1.Add(self.label_9,0,flagLabel,0)
        sizerP1.Add(self.tc10_1,0,0,0)
        sizerP1.Add(self.tc10_2,0,0,0)
        sizerP1.Add(self.tc10_3,0,0,0)
        sizerP1.Add(self.tc10_4,0,0,0)
        sizerP1.Add(self.tc10_5,0,0,0)
        sizerP1.Add(self.label_10,0,flagLabel,0)
        sizerP1.Add(self.tc11_1,0,0,0)
        sizerP1.Add(self.tc11_2,0,0,0)
        sizerP1.Add(self.tc11_3,0,0,0)
        sizerP1.Add(self.tc11_4,0,0,0)
        sizerP1.Add(self.tc11_5,0,0,0)
        sizerP1.Add(self.label_11,0,flagLabel,0)
        sizerP1.Add(self.tc12_1,0,0,0)
        sizerP1.Add(self.tc12_2,0,0,0)
        sizerP1.Add(self.tc12_3,0,0,0)
        sizerP1.Add(self.tc12_4,0,0,0)
        sizerP1.Add(self.tc12_5,0,0,0)
        sizerP1.Add(self.label_12,0,flagLabel,0)
        sizerP1.Add(self.tc13_1,0,0,0)
        sizerP1.Add(self.tc13_2,0,0,0)
        sizerP1.Add(self.tc13_3,0,0,0)
        sizerP1.Add(self.tc13_4,0,0,0)
        sizerP1.Add(self.tc13_5,0,0,0)
        sizerP1.Add(self.label_13,0,flagLabel,0)
        sizerP1.Add(self.tc14_1,0,0,0)
        sizerP1.Add(self.tc14_2,0,0,0)
        sizerP1.Add(self.tc14_3,0,0,0)
        sizerP1.Add(self.tc14_4,0,0,0)
        sizerP1.Add(self.tc14_5,0,0,0)
        sizerPage1.Add(sizerP1,2,wx.EXPAND|wx.TOP,10)

        self.page1.SetSizer(sizerPage1)
        sizerOKCancel = wx.BoxSizer(wx.HORIZONTAL)
        sizerOKCancel.Add(self.buttonCancel, 0, wx.ALL|wx.EXPAND, 2)
        sizerOKCancel.Add(self.buttonOK, 0, wx.ALL|wx.EXPAND, 2)

        sizerGlobal.Add(self.notebook, 3, wx.EXPAND, 0)
        sizerGlobal.Add(sizerOKCancel, 0, wx.TOP|wx.ALIGN_RIGHT, 0)
        self.SetSizer(sizerGlobal)
        self.Layout()

#------------------------------------------------------------------------------
#--- UI actions
#------------------------------------------------------------------------------
    def editAllowed(self):
        """Return True if the data may be modified in the current state, 
        inform the user and return False if not.
        
        """
        return True
##        if Status.StatusCC > 0:
##            showWarning(_U("You cannot modify the economic parameters here after the data has been checked.\n")+
##                        _U("You may change some of them in the Cost Assessment dialog.\n")+
##                        _U("If you want to change the present state you may unblock the data by ")+
##                        _U("choosing to confirm the deletion ")+ 
##                        _U("of any alternative proposals in this project the next time you switch to the present state."))
##            return False
##        else:   
##            return True

    def OnButtonCancel(self, event):
        self.prnt.Close()
        
    def OnButtonOK(self, event):
        if not self.editAllowed():
            return

        ret = logger.askConfirmation(_("Changes will apply to all alternatives. Do You want to continue ?"))
        if not ret:
            return
                                     
        self.updateOM()
        
        if Status.PId != 0:
            questionnaire = Status.prj.getQuestionnaire()
            if questionnaire:                
##                tmp = {"PercentExtFinancing":dbio.check(self.tc4.GetValue()),
##                       "OMGenTot":dbio.check(self.tc10_1.GetValue()),
##                       "OMGenUtilities":dbio.check(self.tc10_2.GetValue()),
##                       "OMGenLabour":dbio.check(self.tc10_3.GetValue()),
##                       "OMGenExternal":dbio.check(self.tc10_4.GetValue()),
##                       "OMGenRegulatory":dbio.check(self.tc10_5.GetValue()),
##                       "OMBuildTot":dbio.check(self.tc11_1.GetValue()),
##                       "OMBuildUtilities":dbio.check(self.tc11_2.GetValue()),
##                       "OMBuildLabour":dbio.check(self.tc11_3.GetValue()),
##                       "OMBuildExternal":dbio.check(self.tc11_4.GetValue()),
##                       "OMBuildRegulatory":dbio.check(self.tc11_5.GetValue()),
##                       "OMMachEquipTot":dbio.check(self.tc12_1.GetValue()),
##                       "OMMachEquipUtilities":dbio.check(self.tc12_2.GetValue()),
##                       "OMMachEquipLabour":dbio.check(self.tc12_3.GetValue()),
##                       "OMMachEquipExternal":dbio.check(self.tc12_4.GetValue()),
##                       "OMMachEquipRegulatory":dbio.check(self.tc12_5.GetValue()),
##                       "OMHCGenDistTot":dbio.check(self.tc13_1.GetValue()),
##                       "OMHCGenDistUtilities":dbio.check(self.tc13_2.GetValue()),
##                       "OMHCGenDistLabour":dbio.check(self.tc13_3.GetValue()),
##                       "OMHCGenDistExternal":dbio.check(self.tc13_4.GetValue()),
##                       "OMHCGenDistRegulatory":dbio.check(self.tc13_5.GetValue()),
##                       "OMTotalTot":dbio.check(self.tc14_1.GetValue()),
##                       "OMTotalUtilities":dbio.check(self.tc14_2.GetValue()),
##                       "OMTotalLabour":dbio.check(self.tc14_3.GetValue()),
##                       "OMTotalExternal":dbio.check(self.tc14_4.GetValue()),
##                       "OMTotalRegulatory":dbio.check(self.tc14_5.GetValue())}

                financeExt = dbio.check(self.tc4.GetValue())
                tmp = {"PercentExtFinancing": financeExt}
                
                if Status.ANo > 0: questionnaire.set(**tmp)

                tmp = {"InflationRate":dbio.check(self.tc1.GetValue()),
                       "FuelPriceRate":dbio.check(self.tc2.GetValue()),
                       "InterestExtFinancing":dbio.check(self.tc3.GetValue()),
                       "CompSpecificDiscountRate":dbio.check(self.tc4b.GetValue()),
                       "AmortisationTime":dbio.check(self.tc5.GetValue())}

                for ANo in range(Status.NoOfAlternatives+1):
                    generalData = Status.prj.getGeneralData(ANo = ANo)
                    if generalData: generalData.set(**tmp)

                    Status.prj.setStatus("ECO", 0, ANo=ANo)

                    if ANo==0:
                        self.exportGeneralData(tmp, financeExt)
                
                dbio.commit()
##                Status.prj.projectChanged()

            else:
                logger.logTrack("PanelQ9.OnButtonOK: Couldn't save to database, due "\
                         "to missing data set in database.")

            self.prnt.Close()


    def exportGeneralData(self, data, financeExt):
        """
        writes general data into all TCAGeneralData tables
        """
        inflationRate = data["InflationRate"]
        energyRate    = data["FuelPriceRate"]       
        interestRate  = data["InterestExtFinancing"]
        discountRate  = data["CompSpecificDiscountRate"]
        amortisation  = data["AmortisationTime"]

        inflationRate = None if inflationRate is None else 100.0*inflationRate
        energyRate = None if energyRate is None else 100.0*energyRate
        interestRate = None if interestRate is None else 100.0*interestRate
        discountRate = None if discountRate is None else 100.0*discountRate
        
        for ANo in range(Status.NoOfAlternatives + 1):
            TCAGeneralData = dbio.get("tcageneraldata", ProjectID=Status.PId, AlternativeProposalNo=ANo)
            if TCAGeneralData is not None:
                TCAGeneralData.set(**{"AmotisationTime": dbio.check(amortisation),
                                 "InflationRate": dbio.check(inflationRate),
                                 "NominalInterestRate": dbio.check(interestRate),
                                 "CompSpecificDiscountRate": dbio.check(discountRate),
                                 "FulePriceRate": dbio.check(energyRate)})   #TODO typing error not yet corrected in SQL

#------------------------------------------------------------------------------
#--- Public methods
#------------------------------------------------------------------------------

    def display(self, data=None):
        self.clear()
        self.fillPage()
        self.updateOM(doWarnings = True)
        self.page1.Hide()
        self.Show()

    def clear(self):
##        self.checkBox6.SetValue(False)
##        self.checkBox7.SetValue(False)
        self.tc1.entry.Clear()
        self.tc2.entry.Clear()
        self.tc3.entry.Clear()
        self.tc4.entry.Clear()
        self.tc4b.entry.Clear()
        self.tc5.entry.Clear()
        self.tc10_1.Clear()
        self.tc10_2.Clear()
        self.tc10_3.Clear()
        self.tc10_4.Clear()
        self.tc10_5.Clear()
        self.tc11_1.Clear()
        self.tc11_2.Clear()
        self.tc11_3.Clear()
        self.tc11_4.Clear()
        self.tc11_5.Clear()
        self.tc12_1.Clear()
        self.tc12_2.Clear()
        self.tc12_3.Clear()
        self.tc12_4.Clear()
        self.tc12_5.Clear()
        self.tc13_1.Clear()
        self.tc13_2.Clear()
        self.tc13_3.Clear()
        self.tc13_4.Clear()
        self.tc13_5.Clear()
        self.tc14_1.Clear()
        self.tc14_2.Clear()
        self.tc14_3.Clear()
        self.tc14_4.Clear()
        self.tc14_5.Clear()

    def fillPage(self):
        """Put the data for the current alternative into the appropriate
        entry fields.
        
        """
        if Status.PId == 0:
            return
        
        q = Status.prj.getGeneralData()
        self.tc1.SetValue(str(q.InflationRate))
        self.tc2.SetValue(str(q.FuelPriceRate))
        self.tc3.SetValue(str(q.InterestExtFinancing))
        self.tc4.SetValue(str(q.PercentExtFinancing))
        self.tc4b.SetValue(str(q.CompSpecificDiscountRate))
        self.tc5.SetValue(str(q.AmortisationTime))
        
        q = Status.prj.getQuestionnaire()
        self.tc4.SetValue(str(q.PercentExtFinancing))
        self.tc10_1.SetValue(str(q.OMGenTot))
        self.tc10_2.SetValue(str(q.OMGenUtilities))
        self.tc10_3.SetValue(str(q.OMGenLabour))
        self.tc10_4.SetValue(str(q.OMGenExternal))
        self.tc10_5.SetValue(str(q.OMGenRegulatory))
        self.tc11_1.SetValue(str(q.OMBuildTot))
        self.tc11_2.SetValue(str(q.OMBuildUtilities))
        self.tc11_3.SetValue(str(q.OMBuildLabour))
        self.tc11_4.SetValue(str(q.OMBuildExternal))
        self.tc11_5.SetValue(str(q.OMBuildRegulatory))
        self.tc12_1.SetValue(str(q.OMMachEquipTot))
        self.tc12_2.SetValue(str(q.OMMachEquipUtilities))
        self.tc12_3.SetValue(str(q.OMMachEquipLabour))
        self.tc12_4.SetValue(str(q.OMMachEquipExternal))
        self.tc12_5.SetValue(str(q.OMMachEquipRegulatory))
        self.tc13_1.SetValue(str(q.OMHCGenDistTot))
        self.tc13_2.SetValue(str(q.OMHCGenDistUtilities))
        self.tc13_3.SetValue(str(q.OMHCGenDistLabour))
        self.tc13_4.SetValue(str(q.OMHCGenDistExternal))
        self.tc13_5.SetValue(str(q.OMHCGenDistRegulatory))
        self.tc14_1.SetValue(str(q.OMTotalTot))
        self.tc14_2.SetValue(str(q.OMTotalUtilities))
        self.tc14_3.SetValue(str(q.OMTotalLabour))
        self.tc14_4.SetValue(str(q.OMTotalExternal))
        self.tc14_5.SetValue(str(q.OMTotalRegulatory))
        
##        if q.EnergyManagExisting is None:
##            self.checkBox6.SetValue(False)
##        else:
##            self.checkBox6.SetValue(bool(q.EnergyManagExisting))
##        
##        if q.EnergyManagExternal is None:
##            self.checkBox7.SetValue(False)
##        else:
##            self.checkBox7.SetValue(bool(q.EnergyManagExternal))

    def __lockFields(self,fields):
        for field in fields:
            field.Enabled = False
    
    def __getValueSecure(self,obj):
    # reads value from a textfield and returns it as float 
    # if value is not a float 0.0 is returned
        str = obj.GetValue()        
        try: return float(str)
        except: return 0.0    

    def __checkFields(self,fields,override_if_not_match_total = False,doWarnings = False):
    # fields = [<name>,<totalfield>,[<listofsubfields>]]
    # override_if_not_match_total : if TRUE the value from the totalfield will be
    #                               overridden by the calculated sum     
        for category in fields:
            name        = category[0]  
            totalField  = category[1]
            valueFields = category[2]            
            sum = 0.0
            for valueField in valueFields:
                sum += self.__getValueSecure(valueField)
            totalValue = self.__getValueSecure(totalField)
            if (totalValue != sum)and(doWarnings):
                logger.logWarning(_U("Yearly OM: total from database does not match sum from values for: "+name))
            if (override_if_not_match_total):
                totalField.SetValue(str(sum))

    def updateOM(self,doWarnings = False):
        
        fields = [self.tc10_1,self.tc11_1,self.tc12_1,self.tc13_1,self.tc14_1,self.tc14_2,self.tc14_3,self.tc14_4,self.tc14_5]
        self.__lockFields(fields)
        
        fields = []
        #               NAME                                          TOTAL FIELD  SUBFIELDS
        fields.append(["General maintenance"                         ,self.tc10_1,[self.tc10_2,self.tc10_3,self.tc10_4,self.tc10_5]])
        fields.append(["Buildings"                                   ,self.tc11_1,[self.tc11_2,self.tc11_3,self.tc11_4,self.tc11_5]])
        fields.append(["Machines and equipment for processes"        ,self.tc12_1,[self.tc12_2,self.tc12_3,self.tc12_4,self.tc12_5]])
        fields.append(["Generation and distribution of heat and cold",self.tc13_1,[self.tc13_2,self.tc13_3,self.tc13_4,self.tc13_5]])      
        #check subtotals ; Total costs
        self.__checkFields(fields,override_if_not_match_total = True,doWarnings = doWarnings)
  
        fields = []
        #               NAME                              TOTAL FIELD  SUBFIELDS
        fields.append(["Total Costs"                      ,self.tc14_1,[self.tc10_1,self.tc11_1,self.tc12_1,self.tc13_1]])
        fields.append(["Utilities and operating materials",self.tc14_2,[self.tc10_2,self.tc11_2,self.tc12_2,self.tc13_2]])
        fields.append(["Labour costs"                     ,self.tc14_3,[self.tc10_3,self.tc11_3,self.tc12_3,self.tc13_3]])
        fields.append(["External costs"                   ,self.tc14_4,[self.tc10_4,self.tc11_4,self.tc12_4,self.tc13_4]])
        fields.append(["Regulatory"                       ,self.tc14_5,[self.tc10_5,self.tc11_5,self.tc12_5,self.tc13_5]])
        #check totals ; Total of Total costs
        self.__checkFields(fields,override_if_not_match_total = True,doWarnings = doWarnings)    

      

    def goTo(self, key):
        return True
