# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelECO1: hourly data
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from GUI.einsteinPanel import EinsteinPanel
from GUI.dataEntry.tableEntry import TableEntry
from lib.GUI.plot.XYPlot import XYPlot
from modules.guiModules.ECO.moduleECOPanel import ModuleECOPanel
from constants import OMCATEGORIES, TRANSOMCATEGORIES
from GUI.status import Status

import wx

class PanelECO1(EinsteinPanel):

    def __init__(self, parent, main):

        # generate list of notebook pages and frames
        layout = {"notebook": [_("Investment"),
                               _("Energy Cost"),
                               _("O&&M Cost"),
                               _("Non-Recurring Costs"),
                               _("Contingencies")],          # list of notebook-page titles. if empty or None, no notebook is created
                  "frames": [(0, _(" ")),
                             (1, _(" ")),
                             (2, _(" ")),
                             (3, _(" ")),
                             (4, _(" "))]}           # list of frames (pageno, title). if empty, no frame is created
                                                            # pageno = None for frames on the main panel
        super(PanelECO1, self).__init__(parent, main, layout)

        self.mod = ModuleECOPanel(self, "ECO1")


        # Investment
        titles = [_("Description"), _("Investment"), _("Funding Rate"), _("Funding - Additional Fixed Amount"), _("Own Cost")]
        unitSelectors = [("text", None, None), ("float", "CURRENCY", 0), ("float", "FRACTION", 3), ("float", "CURRENCY", 0), ("float", "CURRENCY", 0)]
        hAlignment = [wx.ALIGN_LEFT] + [wx.ALIGN_RIGHT]*4
        readOnly = [False, False, False, False, True]

        self.addEntry("Investment",
                      TableEntry(self.nbPage[0], titles, changeData = self.mod.changeData, size=wx.Size(780, 540),
                                 rowSize=24, colSize=120, colLabelSize=48,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors,
                                 totalRow = True),
                      0)

        self.entry["Investment"].setColSize(0, 300)
        self.entry["Investment"].setPopUpMenu([_('add'), _('delete')])
##        self.entry["Investment"].setExternalEventHandler(self.onTableClick)

        # EnergyCost
        titles = [_("Description"), _("Yearly Energy Consumption"), _("Energy Tariff"), _("Yearly Energy Cost")]
        unitSelectors = [("text", None, None), ("float", "ENERGY", 0), ("float", "ENERGYTARIFF", 4), ("float", "CURRENCY", 0)]
        hAlignment = [wx.ALIGN_LEFT] + [wx.ALIGN_RIGHT]*3
        readOnly = [False, False, False, True]

        self.addEntry("EnergyCost",
                      TableEntry(self.nbPage[1], titles, changeData = self.mod.changeData, size=wx.Size(780, 540),
                                 rowSize=24, colSize=120, colLabelSize=48,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors,
                                 totalRow = True),
                      1)

        self.entry["EnergyCost"].setColSize(0, 300)
        self.entry["EnergyCost"].setPopUpMenu([_('add'), _('delete')])

        # OM
        titles = [_("Description"), _("Category"), _("Yearly O&M Cost")]
        categories = [TRANSOMCATEGORIES[cat] for cat in OMCATEGORIES]
        unitSelectors = [("text", None, None), ("choice", categories, None), ("float", "CURRENCY", 0)]
        hAlignment = [wx.ALIGN_LEFT]*2 + [wx.ALIGN_RIGHT]
        readOnly = [False, False, False]

        self.addEntry("OM",
                      TableEntry(self.nbPage[2], titles, changeData = self.mod.changeData, size=wx.Size(780, 540),
                                 rowSize=24, colSize=120, colLabelSize=48,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors,
                                 totalRow = True),
                      2)

        self.entry["OM"].setColSize(0, 300)
        self.entry["OM"].setPopUpMenu([_('add'), _('delete')])

        # Non-Recurring Costs
        titles = [_("Description"), _("Cost"), _("Year"), _("Type")]
        unitSelectors = [("text", None, None), ("float", "CURRENCY", 0), ("float", "LONGTIME", 0), ("text", None, None)]
        hAlignment = [wx.ALIGN_LEFT] + [wx.ALIGN_RIGHT]*2 + [wx.ALIGN_LEFT]
        readOnly = [False, False, False, False]

        self.addEntry("NonRecurring",
                      TableEntry(self.nbPage[3], titles, changeData = self.mod.changeData, size=wx.Size(780, 540),
                                 rowSize=24, colSize=120, colLabelSize=48,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors,
                                 totalRow = True),
                      3)

        self.entry["NonRecurring"].setColSize(0, 300)
        self.entry["NonRecurring"].setPopUpMenu([_('add'), _('delete')])

        # Contingencies
        titles = [_("Description"), _("Cost"), _("Year")]
        unitSelectors = [("text", None, None), ("float", "CURRENCY", 0), ("float", "LONGTIME", 0)]
        hAlignment = [wx.ALIGN_LEFT] + [wx.ALIGN_RIGHT]*2
        readOnly = [False, False, False]

        self.addEntry("Contingencies",
                      TableEntry(self.nbPage[4], titles, changeData = self.mod.changeData, size=wx.Size(780, 540),
                                 rowSize=24, colSize=120, colLabelSize=48,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors,
                                 totalRow = True),
                      4)

        self.entry["Contingencies"].setColSize(0, 300)
        self.entry["Contingencies"].setPopUpMenu([_('add'), _('delete')])


       # conclude layout
        self.conclude_layout()

