# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelEA4c: UPH/USH as a function of time
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from GUI.einsteinPanel import EinsteinPanel
from GUI.dataEntry.tableEntry import TableEntry
from lib.GUI.plot.XYPlot import XYPlot
from lib.GUI.widgets.rangeSlider import RangeSlider
from modules.energyStats.moduleEA4 import ModuleEA4

from GUI.status import Status

import wx

class PanelEA4c(EinsteinPanel):

    def __init__(self, parent, main):

        # generate list of notebook pages and frames
        layout = {"notebook": [_("plot")+" - "+_("heating"),
                               _("data")+" - "+_("heating"),
                               _("plot")+" - "+_("cooling"),
                               _("data")+" - "+_("cooling")],         # list of notebook-page titles. if empty or None, no notebook is created
                  "frames": [(0, _(" ")),
                             (1, _(" ")),
                             (2, _(" ")),
                             (3, _(" "))]}           # list of frames (pageno, title). if empty, no frame is created
                                                            # pageno = None for frames on the main panel
        super(PanelEA4c, self).__init__(parent, main, layout, bottomRow=None)

##            self.box1.SetLabel(_U('Useful supply heat demand (USH) - peak and base load'))
##            self.box2.SetLabel(_U('Cumulative heat demand (USH)'))
##            self.box1.SetLabel(_U('Useful supply cooling demand (USC) - peak and base load'))
##            self.box2.SetLabel(_U('Cumulative cooling demand (USC)'))

        keys = ['EA4c_Table','EA4c_Plot']
        self.mod = ModuleEA4(keys)

        # add panel specific entries

        plotSize = (760, 540)        
        self.addEntry("PlotH",
                      XYPlot(self.nbPage[0], size=plotSize),
                      0)

        self.plotH = self.entry["PlotH"]        
        self.plotH.xAxis(label=_('Annual operating hours [h]'), xmin=0)
        self.plotH.yAxis(label=_('Heat supply load [kW]'), ymin=0)


        self.addEntry("PlotC",
                      XYPlot(self.nbPage[2], size=plotSize),
                      2)

        self.plotC = self.entry["PlotC"]        
        self.plotC.xAxis(label=_('Annual operating hours [h]'), xmin=0)
        self.plotC.yAxis(label=_('Cooling supply load [kW]'), ymin=0)

##        self.addEntry("Slider",
##                      RangeSlider(self, size=(760, 24),
##                                  minValue=0, maxValue=365-7),
##                      0)
        # conclude layout

        # Data tables

        titles = [_("Temperature Levels"),
                  _("Base Load\n(power)"),
                  _("Base Load\n(energy)"),
                  _("Medium Load\n(power)"),
                  _("Medium Load\n(energy)"),
                  _("Peak Load\n(power)"),
                  _("Peak Load\n(energy)")]

        unitSelectors = [("text", None, None)] + [("float", "POWER", 2), ("float", "ENERGY", -3)]*3
        hAlignment = [wx.ALIGN_LEFT] + [wx.ALIGN_RIGHT]*6
        readOnly = [True]*7

        self.addEntry("DataH",
                      TableEntry(self.nbPage[1], titles, size=plotSize,
                                 rowSize=24, colSize=80, colLabelSize=48,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors,
                                 totalRow = True),
                      1)

        self.entry["DataH"].setColSize(0, 280)
        self.entry["DataH"].setPopUpMenu([])
##        self.entry["Investment"].setExternalEventHandler(self.onTableClick)

        self.addEntry("DataC",
                      TableEntry(self.nbPage[3], titles, size=plotSize,
                                 rowSize=24, colSize=80, colLabelSize=48,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors,
                                 totalRow = True),
                      3)

        self.entry["DataC"].setColSize(0, 280)
        self.entry["DataC"].setPopUpMenu([])
##        self.entry["Investment"].setExternalEventHandler(self.onTableClick)

        self.conclude_layout()
