# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelEH2: hourly data
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from GUI.einsteinPanel import EinsteinPanel
##from GUI.dataEntry.tableEntry import TableEntry
from lib.GUI.plot.XYPlot import XYPlot
from lib.GUI.widgets.rangeSlider import RangeSlider
from modules.guiModules.Energy.moduleEH import ModuleEH

import wx

class PanelEH2(EinsteinPanel):

    def __init__(self, parent, main):

        # generate list of notebook pages and frames
        layout = {"notebook": [],         # list of notebook-page titles. if empty or None, no notebook is created
                  "frames": [(None, _(" "))]}           # list of frames (pageno, title). if empty, no frame is created
                                                            # pageno = None for frames on the main panel
        super(PanelEH2, self).__init__(parent, main, layout, bottomRow=None)

        self.mod = ModuleEH("EH2")

        # add panel specific entries

        plotSize = (760, 440)        
        self.addEntry("Plot",
                      XYPlot(self, size=plotSize),
                      0)

        self.plot = self.entry["Plot"]        
        self.plot.xAxis(label=_("hour of the year [h]"), xmin=0, xmax=2000)
        self.plot.yAxis(label=_("heating / cooling power USH/USC [kW]"), ymin=0)

        self.addEntry("Slider",
                      RangeSlider(self, size=(760, 24),
                                  minValue=0, maxValue=365-7),
                      0)
        # conclude layout
        self.conclude_layout()


    def fillPanel(self):        
        super(PanelEH2, self).fillPanel()

        self.xData = self.data["Plot"].get("xData")
        self.yData = self.data["Plot"].get("yData")

        self.onSlider(0)

    def onSlider(self, val):
        h0 = val*24
        h1 = h0 + 168
        self.plot.xAxis(label=_("hour of the year [h]"), xmin=h0, xmax=h1)
        self.plot.setXTicks(range(h0, h1+24, 24))
        self.plot.setAxvLines(range(h0+24, h1, 24))
        self.plot.setMinorXTicks(range(h0+6, h1, 6))
        self.plot.draw(self.xData, self.yData)
