#Boa:FramePanel:PanelEnergy
# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	PanelEnergy- GUI component for: Energetic performance simulation
#			
#==============================================================================
#
#	Version No.: 0.04
#	Created by: 	    Hans Schweiger
#       Revised by:         Tom Sobota 31/03/2008
#                           Tom Sobota 31/03/2008
#                           Hans Schweiger  03/04/2008
#                           Hans Schweiger  16/04/2008
#                           Hans Schweiger  29/04/2008
#                           Stoyan Danov            18/06/2008
#                           Hans Schweiger  26/06/2008
#                           Hans Schweiger  09/07/2008
#                           Stoyan Danov    13/10/2008
#
#       Changes to previous version:
#       31/03/08:           mod. to use numpy based graphics arg passing
#       03/04/08:           moduleEnergy passed from Interfaces
#       16/04/08:   HS      main as argument in __init__
#       29/04/08:   HS      method display added
#       18/06/2008 SD: change to translatable text _(...)
#       26/06/2008: HS  figure of instantaneous supply added
#       09/07/2008: HS  restructuring of the panel
#                       - elimination of pie-plot (already in E-stats)
#       13/10/2008: SD  change _() to _U()
#       15/02/2010 MW: fixed visualization
#	
#------------------------------------------------------------------------------		
#	(C) copyleft energyXperts.BCN (E4-Experts SL), Barcelona, Spain 2008
#	www.energyxperts.net / info@energyxperts.net
#
#	This program is free software: you can redistribute it or modify it under
#	the terms of the GNU general public license as published by the Free
#	Software Foundation (www.gnu.org).
#
#============================================================================== 
from GUI.einsteinPanel import EinsteinPanel
from GUI.dataEntry.tableEntry import TableEntry
from GUI.displayClasses import ChoiceEntry
from lib.GUI.plot.XYPlot import XYPlot
from constants import HEATING_COOLING
from globals import Globals
from GUI.status import Status
from lib.text.textConversion import toGettextAndUnicode as _U

import wx

class PanelEnergy(EinsteinPanel):

    def __init__(self, parent, main):

        # generate list of notebook pages and frames
        layout = {"notebook": [],         # list of notebook-page titles. if empty or None, no notebook is created
                  "frames": [(None, _(" ")),
                             (None, _('global energy performance data')),
                             (None, _('weekly heat supply by equipment'))]}           # list of frames (pageno, title). if empty, no frame is created
                                                            # pageno = None for frames on the main panel
        super(PanelEnergy, self).__init__(parent, main, layout, bottomRow=None)

        self._showHeating = True # default to heating parameters
        self.mod = Status.mod.moduleEnergy

        self.addEntry("buttonRunSimulation",
                      wx.Button(id=-1,
                                  label=_U('run simulation'), name='buttonRunSimulation', parent=self,
                                  size=wx.Size(760, 24), style=0),
                      0)
        self.entry["buttonRunSimulation"].Bind(wx.EVT_BUTTON, self.OnButtonRunSimulation)
        
        # Table
        titles = [_("Primary energy (PET)"),
                  _("Primary energy - Fuels (LCV)"),
                  _("Primary energy - Electricity"),
                  _("Useful supply heat"),
                  _("Useful process heat"),
                  _("Useful supply cooling"),
                  _("Useful process cooling")]

        unitSelectors = [("float", "ENERGY", 0)]*7
                         
        hAlignment = [wx.ALIGN_RIGHT]*7
        readOnly = [True]*7

        self.addEntry("Table",
                      TableEntry(self, titles, changeData = None, size=wx.Size(760, 104),
                                 rowSize=24, colSize=107, colLabelSize=42,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors),
                      1)

        self.entry["Table"].setPopUpMenu([])

        # Table
        plotSize = (760, 360)        
        self.addEntry("Plot",
                      XYPlot(self, size=plotSize),
                      2)

        self.plot = self.entry["Plot"]        
        self.plot.xAxis(label=_("week of the year"), xmin=0, xmax=52)
        self.plot.yAxis(label=_("heating / cooling power USH/USC [kW]"), ymin=0)

        # conclude layout
        self.conclude_layout()


    # Event handlers
    def OnButtonRunSimulation(self, event):
        self.mod.calculate()
        self.display()

    def display(self, no=None):
        """
        no: unused but kept as optional input for compatibility with generic panel
        """
        
        super(PanelEnergy, self).display()
        Globals.GUI.updateDrawings()
    
