# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelPA1: pinch analysis - stream table
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from constants import KELVIN, HEATING_COOLING, TRANSYESNO, YESNO, TRANSPROCTYPES, TRANSSCHEDULETYPES, SCHEDULETYPES
from globals import logger, Globals

from GUI.einsteinPanel import EinsteinPanel
from GUI.tools.einsteinSimpleDialog import EinsteinSimpleDialog
from GUI.tools.dialogOKCancel import DialogOKCancel

from GUI.dataEntry.tableEntry import TableEntry, DEFAULTMENU
from GUI.displayClasses import FloatEntry, TextEntry, ChoiceEntry, CheckBoxEntry
from GUI.schedules.scheduleEditor import ScheduleEditor
from GUI.GUITools import GRID_BACKGROUND_COLOR

from auxiliary.auxiliary import toGettextAndUnicode as _U, toUnicode as U
from modules.energyStats.modulePA1 import ModulePA1

import wx


class PanelPA1(EinsteinPanel):

    def __init__(self, parent, main, no, size=(780, 600)):
        self.blockEventHandlers = True

        # generate list of notebook pages and frames
        layout = {"notebook": [_("Hot streams"),
                               _("Cold streams")],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(0, _(" ")),
                             (1, _(" "))]}           # list of frames (pageno, title). if empty, no frame is created

        super(PanelPA1, self).__init__(parent, main, layout, size=size)

        self.mod = ModulePA1(self)
##        initData = self.mod.initPanel(no)

        titles = [_("Name"), _("Select"), _("Yearly energy"),
                  _("Avg. Power"), _("Real operating hours"), _("Start temperature"), _("End temperature"),
                  _("Avg. heat transport coefficient"), _("Medium"), _("Avg. flow rate")]

        unitSelectors = [("text", None, None), ("text", None, None), ("float", "ENERGY", 0),
                         ("float", "POWER", 2), ("float", "TIME", 2), ("float", "TEMPERATURE", 2), ("float", "TEMPERATURE", 2),
                         ("float", "HEATTRANSFERCOEF", 2), ("text", None, None), ("float", "MASSORVOLUMEFLOW", 0)]
        hAlignment = [wx.ALIGN_LEFT] + [wx.ALIGN_RIGHT]*(len(titles)-1)

        readOnly = [True]*len(titles)
        readOnly[1] = False

        # Table for hot streams

        self.addEntry("HSTable",
                      TableEntry(self.nbPage[0], titles, None, size=wx.Size(980, 400),
                                 rowSize=24, colSize=90, colLabelSize=48,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors),
                      0)

        self.entry["HSTable"].setColSize(0, 206)
        self.entry["HSTable"].setColSize(1, 50)
        self.entry["HSTable"].setExternalEventHandler(self.onTableClickHS)

        # Table for cols streams

        self.addEntry("CSTable",
                      TableEntry(self.nbPage[1], titles, None, size=wx.Size(980, 400),
                                 rowSize=24, colSize=90, colLabelSize=48,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors),
                      1)

        self.entry["CSTable"].setColSize(0, 206)
        self.entry["CSTable"].setColSize(1, 50)
        self.entry["CSTable"].setExternalEventHandler(self.onTableClickCS)

        # conclude layout
        self.conclude_layout()


    def fillPanel(self):
        self.blockEventHandlers = True
        super(PanelPA1, self).fillPanel()

        # display table entries
        self.entry["HSTable"].display(data=self.data["HSTable"]["data"], choices=self.data["HSTable"]["choices"])
##        self.onTableClickUPX("RightClick", None, None, None)       # apply special formats for maintenance and start-up rows
        
        self.entry["CSTable"].display(data=self.data["CSTable"]["data"], choices=self.data["CSTable"]["choices"])

        self.setSecondaryUnits()    # have to be set AFTER call to display of the tables

        self.setFrameTitle(_("Hot and cold streams"))

        self.blockEventHandlers = False

    def setSecondaryUnits(self):
        """
        Sets the unit conversion factor for the secondary unit for MASSORVOLUME entries
        """
        popBlocker = self.blockEventHandlers
        self.blockEventHandlers = True

        for label in ["HSTable", "CSTable"]:
            funit2Data = self.data[label]["unit2"]     
            for s, funit2 in enumerate(funit2Data):
                self.entry[label].setAttribute("unit2", funit2, row=s, col=2)

        self.blockEventHandlers = popBlocker

    def onTableClickHS(self, eventType, row, col, selection):
        if self.blockEventHandlers: return
        if row>=0 and col==1:
            self.blockEventHandlers = True
            err, data, choices = self.mod.changeDataHSTable(row, -2, None)
            self.entry["HSTable"].display(data=data, choices=choices)
            self.main.updateChildFrames(["PA2"])
            self.blockEventHandlers = False
    def onTableClickCS(self, eventType, row, col, selection):
        if self.blockEventHandlers: return
        if row >= 0 and col==1:
            self.blockEventHandlers = True
            err, data, choices = self.mod.changeDataCSTable(row, -2, None)
            self.entry["CSTable"].display(data=data, choices=choices)
            self.main.updateChildFrames(["PA2"])
            self.blockEventHandlers = False
