# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelPA2: pinch analysis - curves HCC/CCC/GCC
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from globals import logger, Globals
from GUI.einsteinPanel import EinsteinPanel

from GUI.dataEntry.tableEntry import TableEntry, DEFAULTMENU
from GUI.displayClasses import FloatEntry, TextEntry, ChoiceEntry, CheckBoxEntry
from GUI.GUITools import GRID_BACKGROUND_COLOR

from lib.GUI.plot.XYPlot import XYPlot
from lib.text.textConversion import toGettextAndUnicode as _U, toUnicode as U

from modules.energyStats.modulePA2 import ModulePA2

import wx


class PanelPA2(EinsteinPanel):

    def __init__(self, parent, main, size=(780, 600)):
        self.blockEventHandlers = True

        self.keys = ['HR Table','HR Curves', "PA2"]
        self.key = "PA2"    
        self.prnt = parent
        self.prnt.panel = self
        self.blockOptions = False

        # generate list of notebook pages and frames
        layout = {"notebook": [_("Hot/Cold CC"),
                               _("Grand CC"),
                               _("Summary")],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(0, _("Hot and Cold Composite Curve")),
                             (1, _("Grand Composite Curve")),
                             (2, _("Results"))]}           # list of frames (pageno, title). if empty, no frame is created

        super(PanelPA2, self).__init__(parent, main, layout, size=size, bottomRow="control")

        self.mod = ModulePA2(self)
##        initData = self.mod.initPanel(no)

        # CC and GCC plots
        plotSize = (760, 540)        
        self.addEntry("CC",
                      XYPlot(self.nbPage[0], size=plotSize),
                      0)

        self.entry["CC"].xAxis(label=_('Energy [MWh]'))
        self.entry["CC"].yAxis(label=_('Temperature [°C]'))

        self.addEntry("GCC",
                      XYPlot(self.nbPage[1], size=plotSize),
                      1)

        self.entry["GCC"].xAxis(label=_('Energy [MWh]'))
        self.entry["GCC"].yAxis(label=_('Temperature') + " (" + _('hot side') + ") [°C]")

        #Table for Options
        self.addEntry("DTmin",
                      FloatEntry(self,
                            decimals=1,minval=-1000., maxval=2000., value=0,
                            unitdict='TEMPERATUREDIFF',
                            label=_U("Minimum temperature difference"),
                            tip=_U("Minimum temperature difference on heat exchangers in the network")),
                      -1)
        self.entry["DTmin"].entry.Bind(wx.EVT_KILL_FOCUS, self.onOptionsChanged)

##        self.addEntry("EnergyOrPower",
##                      ChoiceEntry(self.nbPage[2],
##                                  values=[_('Energy'), _('Power')],
##                                  label=_("Display energy or average power"),
##                                  tip=_(" "),
##                                  allowNones=False),
##                      2)
##        self.Bind(wx.EVT_CHOICE, self.onOptionsChanged, self.entry["EnergyOrPower"].entry)


        # Table for results
        titles = [_("Parameter"), _("Unit"), _("Value")]

        self.addEntry("Results",
                      TableEntry(self.nbPage[2], titles, None, size=wx.Size(784, 440),
                                 rowSize=24, colSize=120, colLabelSize=32,
                                 hAlignment = [wx.ALIGN_LEFT, wx.ALIGN_CENTRE, wx.ALIGN_RIGHT],
                                 readOnly = [True, False, True],
                                 unitSelectors = [("float", "ENERGY", 2)]*3,
                                 unitCol = 1),
                      2)

        self.entry["Results"].setColSize(0, 420)
        self.entry["Results"].setPopUpMenu([])

        # conclude layout
        self.conclude_layout()


    def fillPanel(self):
        self.blockEventHandlers = True
        super(PanelPA2, self).fillPanel()

        self.setFrameTitle(_("Pinch Analysis"))

        self.blockEventHandlers = False


    def onOptionsChanged(self, event):
        print "TEST TEST onOptionsChanged"
        import sys
        sys.stdout.flush()
        if not self.blockOptions:
            DTmin = self.entry["DTmin"].GetValue()
            self.mod.setOptions(DTmin)

        self.blockOptions = True
        self.Hide()
        self.display()
        self.blockOptions = False

    # Substituting default function getUserChange of EinsteinPanel in order to avoid
    # messages if DTmin or anything else is changed by user interaction
    def getUserChange(self):
        return False
