# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelPA1: pinch analysis - stream table
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from globals import logger, Globals
from constants import APPDATA

from GUI.einsteinPanel import EinsteinPanel
from modules.guiModules.moduleQ2_el import ModuleQ2_el

from GUI.panels.DB.panelDBElectricityMix import PanelDBElectricityMix
from GUI.conflictFrame import conflictFrame

from GUI.dataEntry.tableEntry import TableEntry, DEFAULTMENU
from GUI.displayClasses import FloatEntry, TextEntry, ChoiceEntry, CheckBoxEntry
from lib.GUI.tools.filePicker import pickFile
from GUI.tools.popUp import catchPopUp

from GUI.status import Status

from lib.text.textConversion import toGettextAndUnicode as _U

import wx


class PanelQ2_el(EinsteinPanel):

    def __init__(self, parent, main, no, size=(780, 600)):
        self.blockEventHandlers = True

        # generate list of notebook pages and frames
        layout = {"notebook": [],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(None, _("tariff and consumption")),
                             (None, _("electricity mix")),
                             (None, _("consumption by type of use"))]}           # list of frames (pageno, title). if empty, no frame is created

        super(PanelQ2_el, self).__init__(parent, main, layout, size=size)

        self.mod = ModuleQ2_el(self)

        # TC Table
        titles = [_(' '), _('Yearly consumption'), _('Contracted power'), _('Tariff type'), _('Tariff installed power (monthly)'), _('Tariff on consumption'), _('Yearly electricity cost')]
        unitSelectors = [("text", None, None), ("float", "ENERGY", 0), ("float", "POWER", 2), ("text", None, None), ("float", "UNITPRICE", 2), ("float", "ENERGYTARIFF", 4), ("float", "CURRENCY", 0)]

        hAlignment = [wx.ALIGN_RIGHT]*len(titles)
        readOnly = [True] + [False]*(len(titles)-1)

        self.addEntry("TC", # TC: Tariff and Consumption
                      TableEntry(self, titles, changeData = self.mod.changeDataTC, size=wx.Size(780, 280),
                                 rowSize=24, colSize=100, colLabelSize=48,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors),
                      0)

        self.entry["TC"].setColSize(0, 140)
        self.entry["TC"].setColSize(4, 120)
        self.entry["TC"].setPopUpMenu([]) # deactivate popUp

        self.addEntry("emix",
                      TextEntry(self, maxchars=255, value='',
                                wLabel = 220,
                                wData = 540,
                                label=_("Electricity mix"),
                                tip=_("File containing hourly data (see user manual)"),
                                eventHandlerLeftDown=self.changeEMix),
                      1)

        # CByUse Table: consumption by type of use
        titlesCByUse = [_('Thermal uses'), _('Yearly consumption'), _('Non-thermal uses'), _('Yearly consumption')]
        unitSelectorsCByUse = [("text", None, None), ("float", "ENERGY", 0), ("text", None, None), ("float", "ENERGY", 0)]

        hAlignmentCByUse = [wx.ALIGN_LEFT, wx.ALIGN_RIGHT, wx.ALIGN_LEFT, wx.ALIGN_RIGHT]
        readOnlyCByUse = [True, False, True, False]

        self.addEntry("CByUse", # 
                      TableEntry(self, titlesCByUse, changeData = self.mod.changeDataCByUse, size=wx.Size(780, 160),
                                 rowSize=24, colSize=160, colLabelSize=48,
                                 hAlignment = hAlignmentCByUse,
                                 readOnly = readOnlyCByUse,
                                 unitSelectors = unitSelectorsCByUse),
                      2)

        self.entry["CByUse"].setColSize(0, 220)
        self.entry["CByUse"].setColSize(2, 220)
        self.entry["CByUse"].setPopUpMenu([]) # deactivate popUp

        # conclude layout
        self.conclude_layout()

##    def display(self, no):
##        super(PanelQ2_el, self).display(no)
##
##        self.main.panelInfo.update()
##        self.main.showMainMenuAlternatives()        
##


    def fillPanel(self):
        self.blockEventHandlers = True
        super(PanelQ2_el, self).fillPanel()

        #TODO: r/w permits should be set in module !!!
        self.entry["TC"].setAttribute("edit", False, row=[3])
        self.entry["TC"].setAttribute("edit", False, row=[1], col=[2, 3, 4])
        self.entry["TC"].setAttribute("edit", False, row=[2], col=[2])

        if Status.ANo >= 0:
            self.entry["TC"].setAttribute("edit", False, col=[1, 6])
            self.entry["CByUse"].Disable()

        self.entry["TC"].setAttribute("edit", (self.mod.obj.DataFile is None), col=[3, 5])
            

        self.displayCC()        
        self.blockEventHandlers = False

    def changeEMix(self, entry, pos):
        menu = [_("select from data base"), _("open file"), _("clear")]
        index = catchPopUp(entry, menu, pos)
        if index==0: # select from DB
            self.dialog = PanelDBElectricityMix(self, 
                               _("Choose the used electricity mix."),
                               showOk = True)
            self.dialog.ShowModal()
            return "*.*"

        elif index==1: # select data file
            fileName = pickFile()
            self.mod.setDataFile(fileName)

        elif index==2:
            fileName = "---"
            self.mod.setDataFile(None)
        else:
            return "---"

        self.entry["emix"].SetValue(fileName)

        return fileName

    def onPseudoModal(self, ret):
        if ret:
            self.mod.setEMix(self.dialog.selectedId)            
            self.display()

    def displayCC(self):
        pass
    
    def goTo(self, key):
        """ tries to open the page appropriate
        for a given parameter
        """
        # then open the right page in the panel
        parList = {
            "PowerContrTot" : 1,
            "PowerContrStd" : 1,
            "PowerContrPeak" : 1,
            "PowerContrVall" : 1,
            "ElectricityTotYear" : 1,
            "ElectricityPeakYear" : 1,
            "ElectricityStandYear" : 1,
            "ElectricityValleyYear" : 1,
            "ElectricityGen" : 1,
            "ElectricitySales" : 1,
            "ElectricityRef" : 1,
            "ElectricityAC" : 1,
            "ElectricityThOther" :  1,
            "ElectricityMotors" :  1,
            "ElectricityChem" :  1,
            "ElectricityLight" :  1,
            "ElTariffClassTot" :  1,
            "ElTariffClassStd" :  1,
            "ElTariffClassPeak" :  1,
            "ElTariffClassTotVall" :  1,
            "ElTariffClassCHP" :  1,
            "ElTariffPowTot" :  1,
            "ElTariffPowStd" :  1,
            "ElTariffPowPeak" :  1,
            "ElTariffPowVall" :  1,
            "ElTariffPowCHP" :  1,
            "ElTariffCTot" :  1,
            "ElTariffCStd" :  1,
            "ElTariffCPeak" :  1,
            "ElTariffCVall" :  1,
            "ETariffCHP" :  1,
            "ElCostYearTot" :  1,
            "ElCostYearStd" :  1,
            "ElCostYearPeak" :  1,
            "ElCostYearVall" :  1,
            "ElSalesYearCHP" :  1,
            "FECel" : 1,
            "FETel" : 1,
            "FEOel" : 1,
        }

        if key in parList:
            return True
        else:
            return False

