# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelPA1: pinch analysis - stream table
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from constants import KELVIN, HEATING_COOLING, TRANSYESNO, YESNO, TRANSPROCTYPES, TRANSSCHEDULETYPES, SCHEDULETYPES
from globals import logger, Globals

from GUI.einsteinPanel import EinsteinPanel
from modules.guiModules.moduleA import ModuleA

from GUI.dataEntry.tableEntry import TableEntry, DEFAULTMENU
from GUI.displayClasses import FloatEntry, TextEntry, ChoiceEntry, CheckBoxEntry
from lib.GUI.plot.XYPlot import XYPlot

import wx


class PanelA(EinsteinPanel):

    def __init__(self, parent, main, no, size=(780, 600)):
        self.blockEventHandlers = True

        # generate list of notebook pages and frames
        layout = {"notebook": [_("Table"),
                               _("Graphics")],         # list of notebook-page titles. if empty, no notebook is created
                  "frames": [(0, _(" ")),
                             (1, _("comparison of primary energy consumption and effective annual cost"))]}           # list of frames (pageno, title). if empty, no frame is created

        super(PanelA, self).__init__(parent, main, layout, size=size, bottomRow=None)

        self.mod = ModuleA(self)
##        initData = self.mod.initPanel(no)

        # Table
        titles = [_("No."), _("Name"), _("Description"), _("State"), _("Primary energy consumption"), _("Total annual energy cost")]
        unitSelectors = [("text", None, None), ("text", None, None), ("text", None, None), ("text", None, None), ("float", "ENERGY", 0),
                         ("float", "CURRENCY", 0)]
        hAlignment = [wx.ALIGN_RIGHT, wx.ALIGN_LEFT, wx.ALIGN_LEFT, wx.ALIGN_CENTRE, wx.ALIGN_RIGHT, wx.ALIGN_RIGHT]
        readOnly = [True, False, False, True, True, True]

        self.addEntry("Table",
                      TableEntry(self.nbPage[0], titles, changeData = self.mod.changeData, size=wx.Size(780, 540),
                                 rowSize=40, colSize=100, colLabelSize=48,
                                 hAlignment = hAlignment,
                                 readOnly = readOnly,
                                 unitSelectors = unitSelectors),
                      0)

        self.entry["Table"].setColSize(0, 32)
        self.entry["Table"].setColSize(1, 240)
        self.entry["Table"].setColSize(2, 240)
        self.entry["Table"].setColSize(3, 60)
        self.entry["Table"].setPopUpMenu([_('generate new proposal'),
                                          _('copy proposal'),
                                          _('select proposal'),
                                          _('delete proposal')])
        self.entry["Table"].setExternalEventHandler(self.onTableClick)

        # Table for cols streams

        plotSize = (760, 540)        
        self.addEntry("Plot",
                      XYPlot(self.nbPage[1], size=plotSize),
                      1)

        self.plot = self.entry["Plot"]        
        self.plot.xAxis(label=_('Total Energy System Cost')  + (' [k€]'))
        self.plot.yAxis(label=_('Primary Energy Consumption [MWh]'))

        # conclude layout
        self.conclude_layout()

    def display(self, no):
        super(PanelA, self).display(no)

        self.main.panelInfo.update()
        self.main.showMainMenuAlternatives()        

    def fillPanel(self):
        super(PanelA, self).fillPanel()
        self.entry["Table"].setAttribute("edit", False, row=[0, 1])

    def onTableClick(self, eventType, row, col, selection):
        if self.blockEventHandlers: return

        if eventType=="LeftClick":
            ret = self.mod.setSelection(row)
            if ret: self.display(None)
            
        elif eventType=="LeftDClick":
            ret = self.mod.setSelection(row)
            if Status.StatusCC == 0: # if the state is no longer checked go back 
                self.main.tree.SelectItem(self.main.qCC, select=True)
                self.Destroy()
    
