# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	PanelPRJ: project selection from database
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from globals import Globals
from GUI.einsteinPanel import EinsteinPanel
from GUI.displayClasses import FloatEntry, TextEntry
from modules.weatherdata import WeatherData
from auxiliary.auxiliary import toGettextAndUnicode as _U
from lib.GUI.tools.filePicker import pickFile
from GUI.tools.popUp import catchPopUp
from constants import APPDATA
from GUI.status import Status
import database.dbio as dbio


import wx

class PanelMET(EinsteinPanel):

    def __init__(self, parent, main):

        # generate list of notebook pages and frames
        layout = {"notebook": [],         # list of notebook-page titles. if empty or None, no notebook is created
                  "frames": [(None, _("Yearly data")),
                             (None, _("Data file (optional)"))]}           # list of frames (pageno, title). if empty, no frame is created
                                                            # pageno = None for frames on the main panel
        super(PanelMET, self).__init__(parent, main, layout)

##        self.mod = ModulePRJ(self)

        # add panel specific entries
##        self.WeatherDataFilePickerCtrl = wx.xrc.XRCCTRL(self.page5, 'WeatherDataFilePickerCtrl')
##        self.Bind(wx.EVT_FILEPICKER_CHANGED, self.OnWeatherDataFileChanged, self.WeatherDataFilePickerCtrl)
##        self.NoWeatherDataFileButton = wx.xrc.XRCCTRL(self.page5, 'NoWeatherDataFileButton')
##        self.Bind(wx.EVT_BUTTON, self.OnNoWeatherDataFileButton, self.NoWeatherDataFileButton)

        self.addEntry("Latitude",
                      FloatEntry(self,
                              decimals=2, minval=-90.0, maxval=90.0, value=0.,
                              unitdict='ANGLE',
                              label=_U("Geographic latitude"),
                              tip=_U("Latitude in degrees with decimals, e.g. 41,50 degrees instead of 41° 30\' 00\'\'\n") + \
                                 _U("(+) northern and (-) southern hemisphere")),
                      frameNo=0)

        self.addEntry("I",
                      FloatEntry(self,
                        decimals=1, minval=0., maxval=12000.,
                        unitdict='ENERGYFLOW',label=_U("Solar radiation on horizontal"),\
                        tip=_U("Yearly total solar radiation on horizontal")),       
                      frameNo=0)

        self.addEntry("Idn",
                      FloatEntry(self,
                        decimals=1, minval=0., maxval=12000.,
                        unitdict='ENERGYFLOW',label=_U("Direct normal solar radiation"),\
                        tip=_U("Yearly total direct normal solar radiation")),
                      frameNo=0)

        self.addEntry("TAmb",
                      FloatEntry(self,
                                 decimals=1, minval=0., maxval=300.,
                                 unitdict='TEMPERATURE',label=_U("Average ambient temperature"),
                                 tip=_U("Yearly average ambient temperature")),
                      frameNo=0)

        self.addEntry("WDataFile",
                      TextEntry(self,
                                label=_U("Weather data file"),
                                tip=_U("File containing hourly weather data in TMY2 - format"),
                                eventHandlerLeftDown=self.changeDataFile),
                      frameNo=1)



        # conclude layout
        self.conclude_layout()

    def changeDataFile(self, entry, pos):
        """ Event handler for pop-up of TextCtrl (WDAtaFile)
        """
        menu = [_("open file"), "clear"]
        index = catchPopUp(entry, menu, pos)

        if index==0:
            fileName = pickFile(rootDir=APPDATA+"/weather")
            self.wDataFile = fileName
        elif index==1:
            fileName = "---"
            self.wDataFile = None

        Status.WeatherData.buildData(self.wDataFile)
        self.fillPanel()

        return fileName

    def fillPanel(self):
        self.activeObject.SetLabel(" ")
        Globals.GUI.editWindow.SetTitle(_U("Weather Data"))

        if Status.WeatherData is None:
            Status.WeatherData = WeatherData(Status.PId)

        # update parameters (for the case that different values have been imported from datafile)
        self.entry["Latitude"].SetValue(Status.WeatherData.latitude)
        self.entry["I"].SetValue(Status.WeatherData.I)
        self.entry["TAmb"].SetValue(Status.WeatherData.TAmbAvg)
        self.entry["Idn"].SetValue(Status.WeatherData.Idn)

        self.wDataFile = Status.WeatherData.dataFilePath

        if self.wDataFile is not None:
                self.entry["WDataFile"].SetValue(self.wDataFile)
        else:
            self.entry["WDataFile"].SetValue("---")


    def storeWeatherData(self):

        Status.WeatherData.latitude = self.entry["Latitude"].GetValue()
        Status.WeatherData.I = self.entry["I"].GetValue()
        Status.WeatherData.Idn = self.entry["Idn"].GetValue()
        Status.WeatherData.TAmbAvg = self.entry["TAmb"].GetValue()
        Status.WeatherData.buildData(self.wDataFile)                                        
        Status.WeatherData.saveToDB()
        Status.prj.projectChanged()
        
    def OnButtonOK(self, event):
        self.storeWeatherData()
        super(PanelMET, self).OnButtonOK(event)
        
                


    def clearPanel():
        for key in ["Latitude", "I", "Idn", "TAmb"]:
            self.entry[key].Clear()




