# -*- coding: utf-8 -*-
#==============================================================================
#
#    E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.iee-einstein.org/" target="_blank">www.iee-einstein.org</a>)
#
#------------------------------------------------------------------------------
#
#    profileSelector: Selects profiles for definition of schedules
#
#==============================================================================
#
#    Version No.: 0.01
#    Created by:         Hans Schweiger 07/07/2011
#
#------------------------------------------------------------------------------
#    (C) copyleft energyXperts.BCN (E4-Experts SL), Barcelona, Spain 2011-2012
#    http://www.energyxperts.net/
#
#    This program is free software: you can redistribute it or modify it under
#    the terms of the GNU general public license as published by the Free
#    Software Foundation (www.gnu.org).
#
#==============================================================================
import wx
from GUI.status import *
from auxiliary.auxiliary import toUnicode as U, toGettextAndUnicode as _U
from GUI.dataEntry.tableEntry import TableEntry
from GUI.displayClasses import TextEntry
from constants import MONTHSTARTDAY
from GUI.tools.dialogOKCancel import DialogOKCancel
from modules.guiModules.moduleProfileSelector import ModuleProfileSelector
from GUI.schedules.scheduleViewer import ScheduleView

from GUI.fonts import FontProperties
from GUI.displayClasses import FieldSizes

# constants that control the default sizes
# 1. font sizes
TYPE_SIZE = 9
TYPE_SIZE_TITLES = 10

# 2. field sizes
HEIGHT = 24
LABEL_WIDTH = 200
DATA_ENTRY_WIDTH = 180
UNITS_WIDTH = 100

# 3. vertical separation between fields
VSEP_LEFT = 2
VSEP_RIGHT = 2


class ProfileEditor(DialogOKCancel):
    """
    Edits the active profile in the backend module (self.prnt.mod)
    """
    def __init__(self, prnt):
        super(ProfileEditor, self).__init__(prnt, title=_U('Profile Editor'), size=wx.Size(500,540))
        self.prnt = prnt

##        self._init_ctrls(prnt)
        self._custom_ctrls(prnt)        
        self.__custom_layout()

        self.name = None
        self.intervalTable.setExternalEventHandler(self.OnIntervalTableClick)

        self.display()
        self.Show()

    def _custom_ctrls(self, prnt):

        fs = FieldSizes(wHeight=HEIGHT, wLabel=LABEL_WIDTH,
                       wData=DATA_ENTRY_WIDTH, wUnits=UNITS_WIDTH)

        # left tab controls
        # tab 0 - Heat recovery from thermal equipment
        fp = FontProperties()
        fp.pushFont()
        fp.changeFont(size=TYPE_SIZE)

        self.entryName = TextEntry(self, maxchars=255, value='',
                                   label=_U("Profile name"))
        
        summaryTitles = ["ProfileXY", "", ""]
        self.summaryTable = TableEntry(self, summaryTitles, None,
                                readOnly = [True, True, True],
                                colSize=90,
                                colLabelSize=0,
                                size=wx.Size(484, 92))
        self.summaryTable.setColSize(0, 300)
        self.summaryTable.setAttribute("edit", [False]*len(summaryTitles))

        intervalTitles = [_U("start"), _U("stop"), _U("scale")]
        self.intervalTable = TableEntry(self, intervalTitles, self.prnt.mod.changeDataIntervalTable,
                                readOnly = [False, False, False],
                                colSize=100,
                                colLabelSize=24,
                                size=wx.Size(310, 220))
##        self.intervalTable.setPopUpMenu([_U("edit")])
##        self.intervalTable.setColSize(0, 300)
        self.intervalTable.setAttribute("edit", [True]*len(intervalTitles))

        weekDayTitles = [_U("weekday"), _U("on/off")]
        self.weekDayTable = TableEntry(self, weekDayTitles, self.prnt.mod.changeDataWeekDayTable,
                                readOnly = [True, False],
                                colSize=80,
                                colLabelSize=24,
                                size=wx.Size(170, 220))
        self.weekDayTable.setAttribute("edit", [False, True])

        self.profileView = ScheduleView(self, size=wx.Size(484, 150), view="day")
            
            
    def __custom_layout(self):
        flagText = wx.ALIGN_CENTER_VERTICAL|wx.TOP

        # global sizer for panel.

        sizerCenter = wx.BoxSizer(wx.HORIZONTAL)
        sizerCenter.Add(self.intervalTable, 0, wx.ALIGN_CENTRE, 8)
        sizerCenter.AddSpacer(4)
        sizerCenter.Add(self.weekDayTable, 0, wx.ALIGN_CENTRE, 8)
        
        sizerGlobal = wx.BoxSizer(wx.VERTICAL)
        sizerCenter.AddSpacer(2)
        sizerGlobal.Add(self.entryName, 0, wx.ALIGN_CENTRE, 8)
        sizerCenter.AddSpacer(4)
        sizerGlobal.Add(self.summaryTable, 0, wx.ALIGN_CENTRE, 8)
        sizerGlobal.AddSpacer(4)
        sizerGlobal.Add(sizerCenter, 0, wx.ALIGN_CENTRE, 8)
        sizerGlobal.AddSpacer(4)
        sizerGlobal.Add(self.profileView,  0, wx.ALIGN_CENTRE, 8)
        self._do_layout(sizerGlobal)
 
    def display(self):

        data = self.prnt.mod.getActiveProfileData()
        selection = None


        if data:
            if self.name is None:
                self.name = data["name"]
                self.entryName.SetValue(self.name)
                
            self.intervalTable.display(data["interval"], selection)
            self.summaryTable.display(data["summary"], selection)
            self.weekDayTable.display(data["weekDay"], selection, choices=[[None, [_("on"), _("off")]]]*len(data["weekDay"]))

            SVf, SVt = self.prnt.mod.getScheduleView()
            xmin, xmax = self.prnt.mod.defaultPlotRange
            self.profileView.display(data=([SVt], [SVf]), xmin=xmin, xmax=xmax)

        self.Show()

    def OnIntervalTableClick(self, eventType, r, c, selection):
        if eventType in ["SelectRange"]:
            self.prnt.selection = selection
        elif eventType == "LeftDclick":
            self.OnButtonOK(None)
            

    def OnButtonOK(self, event):
        self.name = self.entryName.GetValue()
        if self.prnt.mod.saveProfile(self.name):
            super(ProfileEditor, self).OnButtonOK(event)
