# -*- coding: utf-8 -*-
#==============================================================================
#
#    E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.iee-einstein.org/" target="_blank">www.iee-einstein.org</a>)
#
#------------------------------------------------------------------------------
#
#    profileSelector: Selects profiles for definition of schedules
#
#==============================================================================
#
#    Version No.: 0.01
#    Created by:         Hans Schweiger 07/07/2011
#
#------------------------------------------------------------------------------
#    (C) copyleft energyXperts.BCN (E4-Experts SL), Barcelona, Spain 2011-2012
#    http://www.energyxperts.net/
#
#    This program is free software: you can redistribute it or modify it under
#    the terms of the GNU general public license as published by the Free
#    Software Foundation (www.gnu.org).
#
#==============================================================================
import wx
from GUI.status import *
from auxiliary.auxiliary import toUnicode as U, toGettextAndUnicode as _U
from GUI.dataEntry.tableEntry import TableEntry
from constants import MONTHSTARTDAY, PLATFORM
from globals import logger
from GUI.tools.dialogOKCancel import DialogOKCancel
from globals import logger
from profileEditor import ProfileEditor
from modules.guiModules.moduleProfileSelector import ModuleProfileSelector

WINDOWSIZE = wx.Size(820, 600) if PLATFORM=="WIN" else wx.Size(800,600)
TABLESIZE = wx.Size(804, 550) if PLATFORM=="WIN" else wx.Size(784, 550)

class ProfileSelector(DialogOKCancel):
    def __init__(self, prnt, mode="select"):
        super(ProfileSelector, self).__init__(prnt, title=_U('Profile data base'), size=WINDOWSIZE, custom_layout=True)
        self.parent = prnt
        self.mode = mode
        self._custom_ctrls(prnt)
        
        self.__custom_layout()

        self.mod = ModuleProfileSelector(self)

        self.table.setExternalEventHandler(self.OnTableClick)

        self.Show()

    def _custom_ctrls(self, prnt):
        titles = [_U("profile"), _U("days per\nweek/period"), _U("operation\nhours\nper day"), _U("avg. ratio"), _U("min./max.\nratio"), _U("period\nduration")]
        self.table = TableEntry(self, titles, self.changeData,
                                readOnly = [True]*len(titles),
                                hAlignment = [wx.ALIGN_LEFT] + [wx.ALIGN_CENTRE]*(len(titles)-1),
                                colSize=94,
                                colLabelSize=60,
                                size=TABLESIZE)
        self.table.setPopUpMenu([_U("edit"), _U("add"), _("copy"), _U("delete")])
        self.table.setColSize(0, 300)
        self.table.setAttribute("edit", [False]*len(titles))
            
    def __custom_layout(self):
        flagText = wx.ALIGN_CENTER_VERTICAL|wx.TOP

        # global sizer for panel.
        sizerGlobal = wx.BoxSizer(wx.VERTICAL)
        sizerGlobal.Add(self.table, 0, wx.ALIGN_CENTRE, 8)
        self._do_layout(sizerGlobal)
 
    def display(self, data=None, selection=None):

        if data:
            self.table.display(data, selection)
        else: # call from main menu -> data and selection = None !!!
            data = self.mod.getProfileTable()
            self.table.display(data)

        self.Refresh()

    def changeData(self, index, mode, selection=None, data=None):
        if mode==0: # edit
            if index is not None and index >= 0: # edit a single profile
                self.mod.editProfile(index)
                self.dialog = ProfileEditor(self)
                ret = self.dialog.ShowModal()
                self.display()
                
        elif mode==1: # add
            self.mod.newProfile()
            self.dialog = ProfileEditor(self)
            ret = self.dialog.ShowModal()
            if ret:
                self.mod.addProfile()
                self.display()
##            else:
##                self.mod.deleteProfile(newIndex)
            
        elif mode==2: # copy
            newIndex = self.mod.newProfile(index=index)
            self.dialog = ProfileEditor(self)
            ret = self.dialog.ShowModal()
            if ret:
                self.mod.addProfile()
                self.display()
##            else:
##                self.mod.deleteProfile(newIndex)

        elif mode==3: # delete
            self.mod.deleteProfile(index)
            self.display()

        return 0, None

    def OnTableClick(self, eventType, r, c, selection):
        if eventType in ["SelectRange"]:
            self.parent.selection = selection
        elif eventType == "LeftDclick":
            if self.mode=="select":
                self.OnButtonOK(None)           # select-mode: select the current profile and close
            else:
                self.changeData(r, 0, None)     # edit-mode:   edit the current profile
            

