# -*- coding: utf-8 -*-
#==============================================================================
#
#    E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.iee-einstein.org/" target="_blank">www.iee-einstein.org</a>)
#
#------------------------------------------------------------------------------
#
#    scheduleViewer: Visualise time schedules
#
#==============================================================================
#
#    Version No.: 0.01
#    Created by:         Hans Schweiger 07/07/2011
#
#------------------------------------------------------------------------------
#    (C) copyleft energyXperts.BCN (E4-Experts SL), Barcelona, Spain 2011-2012
#    http://www.energyxperts.net/
#
#    This program is free software: you can redistribute it or modify it under
#    the terms of the GNU general public license as published by the Free
#    Software Foundation (www.gnu.org).
#
#==============================================================================
import wx
from GUI.status import *
from auxiliary.auxiliary import toUnicode as U, toGettextAndUnicode as _U
from GUI.plot.XYPlot import XYPlot
from constants import MONTHSTARTDAY
import numpy as np

class ScheduleView(wx.Panel):
    def __init__(self, prnt, size=wx.Size(800, 600), view="year"):
        self.size = size
        self._init_ctrls(prnt)
        self.__do_layout()
        self.view = view

    def _init_ctrls(self, prnt):
        # generated method, don't edit
        wx.Panel.__init__(self, id=wx.ID_ANY, name='', parent=prnt,
              pos=wx.Point(200, 50), size=self.size,
              style=wx.DEFAULT_FRAME_STYLE | wx.FRAME_FLOAT_ON_PARENT)
        self.SetClientSize(self.size)

        self.plot = XYPlot(self, size=self.size)
        self.plot.xAxis(label=_("hour of the year [h]"), xmin=0, xmax=2000)
        self.plot.yAxis(label=_("scale factor"), ymin=0)

            
    def __do_layout(self):
        # global sizer for panel.
        sizerGlobal = wx.BoxSizer(wx.VERTICAL)
        sizerGlobal.Add(self.plot, 0, wx.ALIGN_LEFT, 0)
        self.SetSizer(sizerGlobal)

    def display(self, data=None, xmin=None, xmax=None):
        if data:
            xData, yData = data
            self.plot.xData(xData)
            self.plot.yData(yData, labels=[""])

        if self.view=="year":
            self.plot.xAxis(label=_("hour of the year [h]"), xmin=xmin, xmax=xmax)
            self.plot.setXTicks(range(24, 8760, 24))
            self.plot.setAxvLines(range(168, 8760, 168))
            self.plot.setMinorXTicks(range(6, 8760, 6))
        elif self.view=="day":
            self.plot.xAxis(label=_("hour [h]"), xmin=xmin, xmax=xmax)
            self.plot.setXTicks(range(0, 25, 1))
            self.plot.setAxvLines(range(0, 28, 4))
            self.plot.setMinorXTicks(np.linspace(0.5, 23.5, 47))

        self.plot.draw()
        self.Show()
    

class PeriodView(wx.Panel):
    def __init__(self, prnt, size=wx.Size(800, 600)):
        self.size = size
        self._init_ctrls(prnt)
        self.__do_layout()


    def _init_ctrls(self, prnt):
        # generated method, don't edit
        wx.Panel.__init__(self, id=wx.ID_ANY, name='', parent=prnt,
              pos=wx.Point(200, 50), size=self.size,
              style=wx.DEFAULT_FRAME_STYLE | wx.FRAME_FLOAT_ON_PARENT)
        self.SetClientSize(self.size)

        self.plot = XYPlot(self, size=self.size)
        self.plot.xAxis(label=_("day of the year"), xmin=0, xmax=365)
        self.plot.yAxis(label=_("scale factor"), ymin=0)

        self.plot.setAxvLines(MONTHSTARTDAY[1:-1])
        self.plot.setXTicks(MONTHSTARTDAY)
        self.plot.setMinorXTicks(range(7, 365, 7))
            
    def __do_layout(self):
        # global sizer for panel.
        sizerGlobal = wx.BoxSizer(wx.VERTICAL)
        sizerGlobal.Add(self.plot, 0, wx.ALIGN_LEFT, 0)
        self.SetSizer(sizerGlobal)

    def display(self, data=None):
        if data:
            xData, yData = data
            self.plot.xData(xData)
            self.plot.yData(yData, labels=[""])

        self.plot.draw()
        self.Show()


#TODO: PeriodView and ScheduleView could be easily unified into one customisable class




