# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	STATUS
#			
#------------------------------------------------------------------------------
#			
#	Class defining the processing status in the module
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
import constants as c

class Status(object):
    PId = None # Active project identity [==> SQL: ActiveProject]
    ANo = None # Active alternative under processing [==> SQL: ActiveAlternative]
    ActiveAlternativeName = ""
    ActiveProjectName = ""
    FinalAlternativeName = "" # selected proposal
    SetUpId = 0 # Active configuration
    DB = None
    SQL = None
    prj = None # Access to controller of the project data

    # Statistical data of project [==> SQL Table STATUS]
    ProjectCreationDate = None
    ProjectFinalisationDate = None
    LastModificationDate = None
    AuditorId = None
    ConfidentialityLevel = "Public"
    ProjectCurrency = "EUR"

    # Tool interface configuration [==> SQL Table STATUS]
    UserInteractionLevel = "interactive" # Selected level of user interaction
    LanguageTool = "en"         # Selected language for tool interface
    Units = "SI-kWh"                 # System of units (default: SI-kWh)
    Country = "Spain"                # Configuration for country specific aids ... NOT USED AT PRESENT
    UserType = "Expert"              # type of user: expert, student, end-user ... ???
    Debug = c.DEBUG
    
    # Status of processing [==> SQL Table STATUS]
    NoOfAlternatives = 0 
    LanguageReport = "english"       # Selected language for report
    Report_NJMax = 10  # Maximum number of equipment
    StatusECO = None            # If == 0 then the equipment has changed since the last
                                # TCA run, and is therefore stale
                                # If == 1 then the current TCA data that depends on
                                # simulation results or other equipment dependent data
                                # is up to date
                                # If == 2 then the user has been warned over changes
                                # since the last access to TCA data

    StatusQ = None
    StatusCC = None
    
    # Tool configuration
    TimeStep = 1.0
    
    TemperatureInterval = c.TemperatureInterval
    MaximumTemperature = c.MaximumTemperature
  
    TemperatureIntervalC = c.TemperatureIntervalC
    MaximumTemperatureC = c.MaximumTemperatureC
    MinimumTemperatureC = c.MinimumTemperatureC
  
    NT = c.NT
    Nt = c.Nt
    SIMULATED_YEAR = Nt*TimeStep
    EXTRAPOLATE_TO_YEAR = 8760.0 / SIMULATED_YEAR

    HRTool = c.HRMODES[2]
    SimTool = "Plus" # NP: numpy-based V2.2 tool, Plus: cython-stepByStep simulation tool
  
    #------------------------------------------------------------------------------
    # Weather data
    WeatherData = None

    #------------------------------------------------------------------------------
    # Fluid and fuel properties
    DefaultAir = None

    # Instance variables
    def __init__(self, name):
        self.myname = name


if __name__ == "__main__":
    # for testing purposes only
    # should be invoked: python status.py
    #
    # example of class variables
    s1 = Status("first instance")  # one instance of Status is created 
    s2 = Status("second instance") # another instance is created
    # now class variable is changed. This change is propagated to
    # all instances of the class
    Status.Country = "Mongolia"
    # let's verify:
    print 's1.myname=%s s1.Country=%s' % (s1.myname, s1.Country) 
    print 's2.myname=%s s2.Country=%s' % (s2.myname, s2.Country) 
