# -*- coding: utf-8 -*-
#==============================================================================
#
#   E I N S T E I N
#
#   Expert System for an Intelligent Supply of Thermal Energy in Industry
#   (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#   EinsteinSimpleDialog
#
#------------------------------------------------------------------------------
#
#   Simplified version of EinsteinPanel in form of a dialog
#   (- no notebooks)
#
#==============================================================================
#
#   Version 3.0, 2018
#
#------------------------------------------------------------------------------
#
#   (C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#   http://www.energyxperts.net
#
#   all rights reserved
#
#==============================================================================
import wx
from auxiliary.auxiliary import toGettextAndUnicode as _U, toUnicode as U, toUTF8
DEFAULTBUTTONSIZE = wx.Size(80, 24)
from GUI.GUITools import TITLE_COLOR, TITLE_FONTSIZE
from GUI.fonts import FontProperties

[wxID_DIALOG, wxID_DIALOGBUTTONCANCEL, wxID_DIALOGBUTTONOK, 
 wxID_DIALOGSTDIALOG, 
] = [wx.NewId() for _init_ctrls in range(4)]

class EinsteinSimpleDialog(wx.Dialog):

    def __init__(self, prnt, layout, title="EinsteinSimpleDialog", mod=None,
                 size=wx.Size(400, 200), buttonSize=DEFAULTBUTTONSIZE):
        self.mod = mod
        self.prnt = prnt
        self.size = size
        self.buttonSize = buttonSize
        
        self.entry = {}

        wx.Dialog.__init__(self, id=-1, name='EinsteinSimpleDialog', parent=prnt, size=self.size,
              style=wx.DEFAULT_DIALOG_STYLE, title=U(title))
##        self.SetClientSize(self.size)
        
        self._init_ctrls(prnt, layout)
        self.__do_layout()
        
##        self.tcShortName.SetValue(U("papperlapapp"))
##        self.tcDescription.SetValue(U("quatschkopf"))


    def _init_ctrls(self, prnt, layout):

        # access to font properties object
        fp = FontProperties()

        self.frame = []
        self.pageOfFrame = []
        
        if "frames" in layout.keys():
            # set font for titles
            # 1. save actual font parameters on the stack
            fp.pushFont()
            # 2. change size and weight
            fp.changeFont(size=TITLE_FONTSIZE, weight=wx.BOLD)

            for frame in layout["frames"]:
                page, title = frame

                self.frame.append(wx.StaticBox(self, -1, U(title)))
                self.frame[-1].SetForegroundColour(TITLE_COLOR)
                self.frame[-1].SetFont(fp.getFont())
            
            # 3. recover previous font state
            fp.popFont()

##        self.stDialog1 = wx.StaticText(id=wxID_DIALOGPSTDIALOG1,
##              label=_U('shortname:'), name='stDialog1', parent=self,
##              pos=wx.Point(24, 16), size=wx.Size(56, 13), style=0)
##        self.stDialog1.Center(wx.HORIZONTAL)
##

        self.buttonOK = wx.Button(id=wxID_DIALOGBUTTONOK, label=_U('OK'),
              name='buttonOK', parent=self,
              size=self.buttonSize, style=0)
        self.buttonOK.Bind(wx.EVT_BUTTON, self.OnButtonOK,
              id=wxID_DIALOGBUTTONOK)

        self.buttonCancel = wx.Button(id=wxID_DIALOGBUTTONCANCEL,
              label=_U('Cancel'), name='buttonCancel', parent=self,
              size=self.buttonSize, style=0)
        self.buttonCancel.Bind(wx.EVT_BUTTON, self.OnButtonCancel,
              id=wxID_DIALOGBUTTONCANCEL)

##        self.stDescription = wx.StaticText(id=wxID_DIALOGPSTDESCRIPTION,
##              label=_U('description:'), name='stDescription', parent=self,
##              pos=wx.Point(24, 40), size=wx.Size(57, 13), style=0)
##        self.stDescription.Center(wx.HORIZONTAL)
##
##        self.tcShortName = wx.TextCtrl(id=wxID_DIALOGPTCSHORTNAME,
##              name='tcShortName', parent=self, pos=wx.Point(96, 16),
##              size=wx.Size(280, 21), style=0, value=_U('New alternative'))
##        self.tcShortName.SetMaxLength(45)
##        
##        self.tcDescription = wx.TextCtrl(id=wxID_DIALOGPTCDESCRIPTION,
##              name='tcDescription', parent=self, pos=wx.Point(96, 40),
##              size=wx.Size(280, 64), style=wx.TE_MULTILINE,
##              value=_U('Describe briefly the new alternative'))
##        self.tcDescription.SetMaxLength(200)

    def __do_layout(self):
        """
        First part of layout. Carries out the layout of default entries
        """

        # global sizer for panel. Contains notebook w/two tabs + buttons Cancel and Ok
        self.sizerGlobal = wx.BoxSizer(wx.VERTICAL)

        # sizer for each frame
        self.sizerFrame = []
        for frame in self.frame:
            self.sizerFrame.append(wx.StaticBoxSizer(frame, wx.VERTICAL))

    def conclude_layout(self):
        """
        Second part of the layout - to be called after a possible personalised layout function
        """
        pass

        for i, sizerFrame in enumerate(self.sizerFrame):
            self.sizerGlobal.Add(sizerFrame, 0, wx.ALIGN_TOP | wx.EXPAND, 0)
            
        sizerOKCancel = wx.BoxSizer(wx.HORIZONTAL)
##        sizerOKCancel.Add(self.activeObject, 0, wx.ALL | wx.EXPAND, 2)
        sizerOKCancel.Add(self.buttonCancel, 0, wx.ALL | wx.EXPAND, 2)
        sizerOKCancel.Add(self.buttonOK, 0, wx.ALL | wx.EXPAND, 2)

        self.sizerGlobal.Add(sizerOKCancel, 0, wx.ALIGN_BOTTOM | wx.ALIGN_RIGHT, 0)
        self.SetSizer(self.sizerGlobal)

        self.Layout()


    def OnButtonOK(self, event):
##        self.mod.shortName = toUTF8(self.tcShortName.GetValue())
##        self.mod.description = toUTF8(self.tcDescription.GetValue())
        if self.mod: self.mod.fetchData(self.readPanel())
        if self.IsModal(): self.EndModal(wx.ID_OK)

    def OnButtonCancel(self, event):
        if self.IsModal(): self.EndModal(wx.ID_CANCEL)

    def addEntry(self, label, entry, frameNo=None, eventHandlers = None):
        """
        adds any entry to the panel

        label: identifier of the entry
        frameNo: number of frame to which the entry belongs
        """
        self.entry.update({label: entry})
        
        if frameNo is not None:
            self.sizerFrame[frameNo].Add(entry, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP, 0)

    def fillPanel(self, data):
        """
        helper function for display
        
        writes data to the corresponding panel entries
        this function should be personalised for the specific panel entries where necessary
        """
        if data is None: return

        for label in self.entry.keys():
            if label in data.keys():
                self.entry[label].SetValue(data[label])

    def readPanel(self):
        """
        inverse of fill panel
        reads out all the entries on the display
        """
        newData = {}
        for label in self.entry.keys():
            newData[label] = self.entry[label].GetValue()
        return newData
