from auxiliary import noneCheck, findKey
import database.dbio as dbio

class DataTransfer():
    """ Helper class for data tranfer between two objects / dictionaries

    origin: some source object of data
    dest:   dictionary where data are copied to (write) or read from (read)

    key:    key in origin
    destKey: key in destination
    
    dictionaries: dynamic dictionaries

    transferDict: dictionary containing {key: (destKey, trans)}
    """
    def __init__(self, dictionaries, transferDict):
        self.dictionaries = dictionaries
        self.transferDict = transferDict
        self.keys = transferDict.keys()

    def write(self, origin):
        """
        copies data from origin to dest
        """

        dest = {}
        for key in self.keys:
            val = getattr(origin, key)
            destKey = noneCheck(self.transferDict[key][0], key)
            trans = self.transferDict[key][1]
            trans = self.dictionaries[trans] if isinstance(trans, str) else trans
            if trans is None:
                dest[destKey] = val
            else:
                dest[destKey] = trans.get(val)

        return dest

    def read(self, dest, origin):
        """
        inverse movement reads back data from dest to origin
        """
        for key in self.keys:
            destKey = noneCheck(self.transferDict[key][0], key)


            # check if a value is available on dest (construction necessary as dest can be a dictionary or a sqlObject-table)
            try:
                val = dest[destKey]
            except:
                try:
                    val = getattr(dest, destKey)
                except:
                    continue

            trans = self.transferDict[key][1]
            trans = self.dictionaries[trans] if isinstance(trans, str) else trans
            if trans is None:
                setattr(origin, key, dbio.check(val))
            else:
                setattr(origin, key, dbio.check(findKey(trans, val)))
