# -*- coding: utf-8 -*-
#============================================================================== 				
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	CONSTANTS
#			
#------------------------------------------------------------------------------
#			
#	Short description:
#	
#	Constants for general use in the tool
#
#==============================================================================
#
#	Version 3.0plus, October 2019
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
import gettext
import sys
from os import path
import ctypes
from auxiliary.auxiliary import toGettextAndUnicode as _U, toUTF8
gettext.install("einstein", "GUI/locale", unicode=False)
language = gettext.translation("einstein", "locale", languages=['%s' % ('en',)], fallback=True)
language.install()

#------------------------------------------------------------------------------		
VERSION         = "3.0.04" # caution this is a string, e.g. 1.2.9 is larger than 1.2.10. Major releases should be numbered 1.0, 1.1, 2.0 ... etc. without sub-numbering
                                # for beta versions: "." is > "-", therefore V3.0.00 > V3.0-beta.01 ...
RELEASE_VERSION = "3.0"         # major release version -> determines subfolder for release in APPDATA folder. Versions with same release version should have the same default data base
RELEASE_DATE    = "January 2021"
SUBVERSION      = "plus"    # <empty> or plus
#------------------------------------------------------------------------------		
DEBUG = "OFF"   #Set to:
DEBUGMODES = ["OFF","BASIC","MAIN","ALL"]
                #"OFF" (default): no debug
                #"BASIC": basic debugging
                #"MAIN": ccheck plots the show-alls in each block
                #"ALL": highest level of debugging,
                #"CALC": only ccheck debug in CALC Functions
                #"ADJUST": only ccheck debug in ADJUST Functions
DEBUGCCUNTIL = 0 # number of iterations for debugging of Cons.check. (<=0: debugging off)
SNAPSHOTS = [] # time steps for which a DEBUG snapshot is plot

EXPERIMENTAL_FEATURES = True    # flag to shut down features that are not 
                                # completely implemented or otherwise not
                                # for public usage

BASEDIR = path.abspath("")
APPNAME = "einstein"
if sys.platform == 'darwin':
    from AppKit import NSSearchPathForDirectoriesInDomains #@UnresolvedImport
    # http://developer.apple.com/DOCUMENTATION/Cocoa/Reference/Foundation/Miscellaneous/Foundation_Functions/Reference/reference.html#//apple_ref/c/func/NSSearchPathForDirectoriesInDomains
    # NSApplicationSupportDirectory = 14
    # NSUserDomainMask = 1
    # True for expanding the tilde into a fully qualified path
    APPDATA = path.join(NSSearchPathForDirectoriesInDomains(14, 1, True)[0], APPNAME)
    PLATFORM = "MACOS"
elif sys.platform == 'win32':
    n = ctypes.windll.kernel32.GetEnvironmentVariableW(u"APPDATA", None, 0) #@UndefinedVariable
    buf = ctypes.create_unicode_buffer(u'\0'*n)
    ctypes.windll.kernel32.GetEnvironmentVariableW(u"APPDATA", buf, n) #@UndefinedVariable
    APPDATA = path.join(buf.value, APPNAME, APPNAME + RELEASE_VERSION + SUBVERSION)
    PLATFORM = "WIN"
else:
    APPDATA = path.expanduser(path.join("~", "." + APPNAME))
    PLATFORM = "LINUX"
    
#------------------------------------------------------------------------------		
DBSERVER = "SQLITE" # SQLITE or MYSQL
#------------------------------------------------------------------------------		
# constants of general use in EINSTEIN
#------------------------------------------------------------------------------		

INFINITE = 1.e99
EPSILON = 1.e-9
KELVIN = 273.15
MAXTEMP = 2000      # maximum allowed temperature in analysis 
MINTEMP = -KELVIN   # minimum allowed temperature for cooling streams in analysis
TENV = 18.0         # default temperature assumed for (internal) environment
NDAYS = 365
YEAR = 8760.0
DAY = 24.0
WEEK = DAY*7
MONTHSTARTDAY = [0,31,59,90,120,151,181,212,243,273,304,334,365]
    #note: time 0:00h of the first day of each month = MONTHSTARTDAY*DAY
MONTHS = [_('January'),
          _('February'),
          _('March'),
          _('April'),
          _('May'),
          _('June'),
          _('July'),
          _('August'),
          _('September'),
          _('October'),
          _('November'),
          _('December')]

TRANSMONTHS = {'January':_('January'),
          'February':_('February'),
          'March':_('March'),
          'April':_('April'),
          'May':_('May'),
          'June':_('June'),
          'July':_('July'),
          'August':_('August'),
          'September':_('September'),
          'October':_('October'),
          'November':_('November'),
          'December':_('December')}

WEEKDAYS = [_('Monday'),
            _('Tuesday'),
            _('Wednesday'),
            _('Thursday'),
            _('Friday'),
            _('Saturday'),
            _('Sunday')]

TemperatureInterval = 5.0
MaximumTemperature = 400.0

TemperatureIntervalC = 2.5
MaximumTemperatureC = 140.0
MinimumTemperatureC = -60.0

NT = int(MaximumTemperature / TemperatureInterval + 0.1)
# check that the heating and cooling scale have the same length
assert(NT == int((MaximumTemperatureC-MinimumTemperatureC)/TemperatureIntervalC + 0.1))
Nt = 8760


#------------------------------------------------------------------------------		
#default values for status variables

EINSTEIN_OK = 1
EINSTEIN_NOTOK = 0

INTERACTIONLEVELS = ['interactive','semi-automatic','automatic']
TRANSINTERACTIONLEVELS = [_('interactive'),_('semi-automatic'),_('automatic')]
LANGUAGES = ['bg', 'cat', 'cz', 'de', 'en', 'es', 'fi', 'fr', 'it', 'pl', 'si', 'sk']

#------------------------------------------------------------------------------		
#default entries for parameters

#equiptype is the full description of the type containing information on the equipement-class (BB,HP, ...)
# and on the equipment sub-class (STEAM boiler, CONDENSING boiler, etc.)

EQUIPTYPE = ["compression heat pump",
             "mechanical vapour recompression",
             "thermal heat pump",
             "steam boiler",
             "condensing boiler",
             "hot water boiler",
             "burner (direct heating)",
             "burner (indirect heating)",
             "compression chiller (air cooled)",
             "compression chiller (water cooled)",
             "thermal chiller (air cooled)",
             "thermal chiller (water cooled)",
             "cooling tower (dry)",
             "cooling tower (wet)",
             "fresh or ground water",
             "solar thermal (flat-plate)",
             "solar thermal (evacuated tubes)",
             "solar thermal (parabolic trough)",
             "solar thermal (fresnel)",
             "CHP engine",
             "CHP steam turbine",
             "CHP gas turbine",
             "CHP fuel cell",
             "CHP ORC",
             "CHP TFC"]


#translatable dictionary
TRANSEQUIPTYPE = {"compression heat pump":_("compression heat pump"),
             "mechanical vapour recompression":_("mechanical vapour recompression"),
             "thermal heat pump":_("thermal heat pump"),
             "steam boiler":_("steam boiler"),
             "condensing boiler":_("condensing boiler"),
             "hot water boiler":_("hot water boiler"),
             "burner (direct heating)":_("burner (direct heating)"),
             "burner (indirect heating)":_("burner (indirect heating)"),
             "compression chiller (air cooled)":_("compression chiller (air cooled)"),
             "compression chiller (water cooled)":_("compression chiller (water cooled)"),
             "thermal chiller (air cooled)":_("thermal chiller (air cooled)"),
             "thermal chiller (water cooled)":_("thermal chiller (water cooled)"),
             "cooling tower (dry)":_("cooling tower (dry)"),
             "cooling tower (wet)":_("cooling tower (wet)"),
             "fresh or ground water":_("fresh or ground water"),
             "solar thermal (flat-plate)":_("solar thermal (flat-plate)"),
             "solar thermal (evacuated tubes)":_("solar thermal (evacuated tubes)"),
             "solar thermal (parabolic trough)":_("solar thermal (parabolic trough)"),
             "solar thermal (fresnel)":_("solar thermal (fresnel)"),
             "CHP engine":_("CHP engine"),
             "CHP steam turbine":_("CHP steam turbine"),
             "CHP gas turbine":_("CHP gas turbine"),
             "CHP fuel cell":_("CHP fuel cell"),
             "CHP ORC":_("Heat to Power (Rankine cycle)"),
             "CHP TFC":_("Heat to Power (TFC)")}

EQCLASSES = ["BB", "CH", "CHP", "ST", "HP"]
TRANSEQCLASSES = {"BB": _("Boilers, Burners and Heaters"),
                  "CH": _("Cooling equipment"),
                  "CHP": _("CHP and Heat-to-power"),
                  "ST": _("Solar Thermal"),
                  "HP": _("Heat Pumps")}

# BBTYPES, STTYPES etc. are the short classifications of the equipment sub-class (usually used within the specific
# equipment modules

BBTYPES = ["steam boiler",
           "condensing boiler",
           "hot water boiler",
           "burner (direct heating)",
           "burner (indirect heating)"]

TRANSBBTYPES = {"steam boiler":_("steam boiler"),
                "condensing boiler":_("condensing boiler"),
                "hot water boiler":_("hot water boiler"),
                "burner (direct heating)":_("burner (direct heating)"),
                "burner (indirect heating)":_("burner (indirect heating)")}

CHTYPES = [ "compression chiller (air cooled)",
            "compression chiller (water cooled)",
            "thermal chiller (air cooled)",
            "thermal chiller (water cooled)",
            "cooling tower (dry)",
            "cooling tower (wet)",
            "fresh or ground water"]

TRANSCHTYPES = { "compression chiller (air cooled)":_("compression chiller (air cooled)"),
                 "compression chiller (water cooled)":_("compression chiller (water cooled)"),
                 "thermal chiller (air cooled)":_("thermal chiller (air cooled)"),
                 "thermal chiller (water cooled)":_("thermal chiller (water cooled)"),
                 "cooling tower (dry)":_("cooling tower (dry)"),
                 "cooling tower (wet)":_("cooling tower (wet)"),
                 "fresh or ground water":_("fresh or ground water")}

####dummy text for translation package
NEWTRANSCHPTYPES = [_("jet pump"),
                    _("desiccant evaporative cooling")]

HPTYPES = ["compression",
           "mechanical vapour recompression",
           "thermal"]

TRANSHPTYPES = {"compression":_("compression"),
                "mechanical vapour recompression":_("mechanical vapour recompression"),
                "thermal":_("thermal")}


HPSUBTYPES = {
              'compression' : ['water-water', 'water-air', 'air-water', 'air-air'],
              'mechanical vapour recompression' : ['direct', 'via HX'],
              'thermal'     : ['water-water', 'water-air', 'air-water', 'air-air'],
              }

TRANSHPSUBTYPES = {'water-water':_('water-water'),
                   'water-air':_('water-air'),
                   'air-water':_('air-water'),
                   'air-air':_('air-air'),
                   'direct':_('direct'),
                   'via HX':_('via HX')}


STTYPES = [ "Flat-plate collector",
            "Evacuated tube collector",
            "Parabolic trough collector",
            "Fresnel collector"]

TRANSSTTYPES = {"Flat-plate collector":     _("Flat-plate collector"),
                "Evacuated tube collector": _("Evacuated tube collector"),
                "Parabolic trough collector":  _("Parabolic trough collector"),
                "Fresnel collector":  _("Fresnel collector")
                }

HXTYPES = [ "plate HX (liquid-liquid)",
            "plate HX (air-air)",
            "shell and tube HX (liquid-liquid)",
            "finned tubes (liquid-air)"]

global TRANSHXTYPES
TRANSHXTYPES = {"plate HX (liquid-liquid)":     _("plate HX (liquid-liquid)"),
                "plate HX (air-air)":           _("plate HX (air-air)"),
                "shell and tube HX (liquid-liquid)":_("shell&tube HX (liquid-liquid)"),
                "finned tubes (liquid-air)":    _("finned tubes (liquid-air)")
                }

CHPTYPES = [ "CHP engine",
             "CHP steam turbine",
             "CHP gas turbine",
             "CHP fuel cell",
             "CHP ORC",
             "CHP TFC"]

NEWCHPTYPES = [_("engine"),_("steam turbine"),_("gas turbine"),_("fuel cell"),_("stirling engine")]

CHPSUBTYPES = {"CHP engine"        : ["subtype1", "subtype2"],
               "CHP steam turbine" : ["subtype1", "subtype2"],
               "CHP gas turbine"   : ["subtype1", "subtype2"],
               "CHP fuel cell"     : ["subtype1", "subtype2"],
               "CHP ORC"     : ["subtype1", "subtype2"]}

TRANSCHPTYPES = {
             "CHP engine":_("CHP engine"),
             "CHP steam turbine":_("CHP steam turbine"),
             "CHP gas turbine":_("CHP gas turbine"),
             "CHP fuel cell":_("CHP fuel cell"),
             "CHP ORC":_("Heat to Power (Rankine cycle)"),
             "CHP TFC":_("Heat to Power (TFC)")}

HPIndex = 0
BBIndex = len(HPTYPES)
CHIndex = BBIndex + len(BBTYPES)
STIndex = CHIndex + len(CHTYPES)
CHPIndex = STIndex + len(STTYPES)

# EQUIPMENT associates equipment class and sub-class
EQUIPMENT = {"HP":                 # equipment class
             [(EQUIPTYPE[HPIndex+i],TRANSEQUIPTYPE[EQUIPTYPE[HPIndex+i]], # type of equipment, translated
               HPTYPES[i], TRANSHPTYPES[HPTYPES[i]]) # type of process, translated
               for i in range(len(HPTYPES))], 
              
             "BB": [(EQUIPTYPE[BBIndex+i],TRANSEQUIPTYPE[EQUIPTYPE[BBIndex+i]],
                     BBTYPES[i],TRANSBBTYPES[BBTYPES[i]]) 
                     for i in range(len(BBTYPES))],
                    
             "CH": [(EQUIPTYPE[CHIndex+i],TRANSEQUIPTYPE[EQUIPTYPE[CHIndex+i]],
                     CHTYPES[i],TRANSCHTYPES[CHTYPES[i]]) 
                     for i in range(len(CHTYPES))],

             "ST": [(EQUIPTYPE[STIndex+i],TRANSEQUIPTYPE[EQUIPTYPE[STIndex+i]],
                     STTYPES[i],TRANSSTTYPES[STTYPES[i]])
                     for i in range(len(STTYPES))],
                    
             "CHP": [(EQUIPTYPE[CHPIndex+i],TRANSEQUIPTYPE[EQUIPTYPE[CHPIndex+i]],
                      CHPTYPES[i],TRANSCHPTYPES[CHPTYPES[i]])
                      for i in range(len(CHPTYPES))]
             }

#....................................................................
PIPETYPES = ["closed",
             "open",
             "direct"]

TRANSPIPETYPES = {"closed":_("closed circuit"),
                  "open": _("open circuit"),
                  "direct": _("built-in heater/cooler")}
#....................................................................

PROCTYPES = ["continuous",
             "batch"]
#translatable dictionary
TRANSPROCTYPES = {"continuous":_("continuous"),
                  "batch":_("batch")}

SCHEDULETYPES = ["simple", "detailed", "fit to data"]
TRANSSCHEDULETYPES = {"simple":_("simple"),
                      "detailed":_("detailed"),
                      "fit to data": _("fit to data")}

YESNO = ["no", "yes"]
#translatable dictionary
TRANSYESNO = {"no":_("no"), "yes":_("yes")}

STORAGETYPES = ["sensible",
                "latent"]
#translatable dictionary
TRANSSTORAGETYPES = {"Sensible":_("sensible"),
                     "Latent":_("latent")}

WHEEEQTYPES = ["electric motor",
               "compressor","other"]
#translatable dictionary
TRANSWHEEEQTYPES = {"ElectricMotor":_("electric motor"),
                    "Compressor":_("compressor"),
                    "Other":_("other")}

WHEEWASTEHEATTYPES = ["cooling water",
                      "intercooler",
                      "cooling air"]
#translatable dictionary
TRANSWHEEWASTEHEATTYPES = {"CoolingWater":_("cooling water"),
                      "Intercooler":_("intercooler"),
                      "CoolingAir":_("cooling air")}

OBJECTTYPES = ['Process', 'Distribution Line', 'Equipment', 'WHEE']
TRANSOBJECTTYPES = {'Process':_('Process'),
                    'Distribution Line':_('Distribution Line'),
                    'Equipment':_('Equipment'),
                    'WHEE':_('WHEE')}

OBJECTTYPES_HX = OBJECTTYPES + ['HX']
TRANSOBJECTTYPES_HX = {'Process':_('Process'),
                    'Distribution Line':_('Distribution Line'),
                    'Equipment':_('Equipment'),
                    'WHEE':_('Electrical equipment'),
                       'HX':_('Heat Exchanger')}

AMBIENTSOURCE = ["ambient air",
               "ground heat exchanger"]
#translatable dictionary
TRANSAMBIENTSOURCE = {"ambient air":_("ambient air"),
                    "ground heat exchanger":_("ground heat exchanger")}

AMBIENTSINK = ["cooling tower (wet)",
               "cooling tower (dry)",
               "direct heat rejection"]

#not yet implemented, but left here for translation of text
FUTUREAMBIENTSINK = ["ground heat exchanger"]

#translatable dictionary
TRANSAMBIENTSINK = {"cooling tower (wet)":_("cooling tower (wet)"),
                    "cooling tower (dry)":_("cooling tower (dry)"),
                    "direct heat rejection":_("direct heat rejection")}

TRANSFUTUREAMBIENTSINK = {"ground heat exchanger":_("ground heat exchanger")}


#translatable dictionary
ORIENTATIONS = {"S":_("S"),
                "SE":_("SE"),
                "SW":_("SW"),
                "E":_("E"),
                "W":_("W"),
                "NE":_("NE"),
                "NW":_("NW"),
                "N":_("N")}

AZIMUTH = { "S":    0.0,
            "SE":   -45.0,
            "SW":   45.0,
            "E":    -90.0,
            "W":    90.0,
            "NE":   -135.0,
            "NW":   135.0,
            "N":    180.0}

SHADINGTYPES = {"No":_("No"), 
                "Yes,partially shaded":_("Yes,partially shaded"),
                "Yes,fully shaded":_("Yes,fully shaded")}

ROOFTYPES = {"Corrugated metal roof":_("Corrugated metal roof"), 
                "Composite sandwich panels":_("Composite sandwich panels"),
                "Concrete roof":_("Concrete roof"),
                "Tiled roof":_("Tiled roof"),
                "Other":_("Other")}

TRANSPRODUCTCODES = {"ZZ999":_("ZZ999: auxiliary services"),
                "ZZ000":_("ZZ000: other products"),
                "DA010":_("DA010: milk products"),
                "DA020":_("DA020: fruits/vegetables/herbs"),
                "DA030":_("DA030: sugar"),
                "DA040":_("DA040: beer"),
                "DA050":_("DA050: fats/oils"),
                "DA060":_("DA060: chocolate/cacao/coffee"),
                "DA070":_("DA070: starch/potatoes/grain mill products"),
                "DA080":_("DA080: wine/beverage"),
                "DA090":_("DA090: meat"),
                "DA100":_("DA100: fish"),
                "DA110":_("DA110: aroma")}

PRODUCTCODES = sorted(TRANSPRODUCTCODES.keys())



HEATING_COOLING = {}

BUILDINGPROCESSES = {"heating_temperature": 20,
                     "cooling_temperature": 10,
                     "hotwater_final_temperature": 50,
                     "hotwater_inlet_temperature": 10}

TRANS_AIR_HANDLING_UNIT_STATE = {}
AIR_HANDLING_UNIT_STATE = ["no AHU", "only air inlet", "in- and outlet"]
    
FUELTYPES = ['NG', 'LPG', 'HFO', 'BUT', 'PRO', 'GO', 'BIOS', 'BIOG', 'BIOF', 'HNET', 'CNET']
TRANSFUELTYPES = {'NG' : _('Natural gas'),
             'LPG': _('Liquid petroleum gas'),
             'HFO': _('Heavy fuel oil'),
             'BUT': _('Butane'),
             'PRO': _('Propane'),
             'GO' : _('Gas oil'),
             'BIOS': _('Solid Biomass'),
             'BIOG': _('Biogas'),
             'BIOF': _('Biofuels'),
             'HNET': _('Heating network'),
             'CNET': _('Cooling network')}

FLUIDTYPES = ['WATER', 'STEAM', 'ICE', 'DRYAIR', 'MOISTAIR', 'LIQUID', 'GAS', 'SOLID']

TRANSFLUIDTYPES = {'WATER' : _('Water'),
             'STEAM': _('Steam'),
             'ICE': _('Ice'),
             'DRYAIR': _('Dry air'),
             'MOISTAIR': _('Moist air'),
             'LIQUID': _('General liquid'),
             'GAS' : _('General gas'),
             'SOLID' : _('General solid')}
              
HPSOURCESINK = ['air-air', 'air-water', 'water-air', 'water-water']

TRANSHPSOURCESINK = {'air-air'    : _('air-air'),
                'air-water'  : _('air-water'),
                'water-air'  : _('water-air'),
                'water-water': _('water-water')}

HRMODES = ("calculate heat exchanger", "estimate","actualHXNet_V2")
TRANSHRMODES = {"calculate heat exchanger": _("Actual HX Network calculation (V1 - deprecated)"),
                "estimate": _("Estimative calculation"),
                "actualHXNet_V2": _("Actual HX Network calculation")}

HXDESIGNSTRATEGIES = ["FLOWRATE", "EXERGY", "MINSIZE"]
TRANSHXDESIGNSTRATEGIES = {"FLOWRATE":  _("Match flow rates"),
                   "EXERGY":    _("Exergy optimisation"),
                   "MINSIZE":    _("Minimum HX area")}

PROCMAINPARS = ["UPH"]
PROCINPARS = ["UPHProc_k", "UPHc", "mInFlowYear", "PTInFlow"]
PROCOUTPARS = ["UPHw", "mOutFlowYear", "PTOutFlow"]
PROCESSPARS = PROCMAINPARS + PROCINPARS + PROCOUTPARS

PARCODES = {"FECFuel_i": _("Final energy consumption (FEC) by fuel"),
            "FECel": _("Electricity consumption (FEC)"),
            "FETFuel_i": _("Final energy consumption (FET) by fuel"),
            "FETel": _("Electricity consumption (FET)"),
            "FETj": _("Final energy consumption (FET) by equipment"),
            "USHj": _("Useful supply heat supplied (USH) by equipment"),
            "USHm": _("Useful supply heat (USH) supplied to pipe"),
            "USHPipe": _("Useful supply heat (USH) distributed by pipe"),
            "UPHProc_m": _("Useful process heat (UPH_proc) at pipe"),
            "UPHProc_k": _("Useful process heat (UPH_proc) w/o heat rec."),
            "UPH": _("Useful process heat (UPH) incl. heat rec."),
            "UPHc": _("Useful process heat (UPH) - inflow"),
            "UPHw": _("Useful process heat (UPH) - outflow"),
            "Product": _("Quantity of product"),
            "RawMat": _("Quantity of raw material or other input"),
            "TDrive": _("Equipment driving temperature (inlet)"),
            "TReject": _("Equipment heat rejection temperature (inlet)"),
            "Control": _("Equipment master control"),         
            "PTInFlow": _("Process inlet temperature"),
            "PTOutFlow": _("Process outlet temperature"),
            "mInFlowYear": _("Process inlet flow rate"),
            "mOutFlowYear": _("Process outlet flow rate")}

PROCMAINPARS = ["UPH"]
PROCINPARS = ["UPHProc_k", "UPHc", "mInFlowYear", "PTInFlow"]
PROCOUTPARS = ["UPHw", "mOutFlowYear", "PTOutFlow"]
PROCESSPARS = PROCMAINPARS + PROCINPARS + PROCOUTPARS
PIPEPARS = ["USHm", "USHPipe", "UPHProc_m"]
EQPARS = ["FETj", "USHj", "TDrive", "TReject", "Control"]
FUELPARS = ["FECFuel_i", "FETFuel_i"]
ELPARS = ["FECel", "FETel"]
FECPARS = FUELPARS + ELPARS
MFPARS = ["Product", "RawMat"] 
               
DATATYPES = [_("hourly data"), _("data for periods")]

OMCATEGORIES = ["utilities", "opmaterials", "labour", "extcost", "compliance", "insurance", "liability", "water"]
TRANSOMCATEGORIES = {"utilities":    _("Utilities"),
                     "opmaterials":  _("Operating materials"),
                     "labour":       _("Labour"),
                     "extcost":      _("External costs"),
                     "compliance":   _("Regulatory compliance costs"),
                     "insurance":    _("Insurance cost"),
                     "liability":    _("Future liability"),
                     "water":        _("Water cost")}

TRANSCONTINGENCIES = {0: _("tax disadvantages (other than energy tax)"),
                      1: _("obligatory provisions"),
                      2: _("costs for remediation activities"),
                      3: _("cost for compliance with legislation"),
                      4: _("negative impacts on market share"),
                      5: _("deterioration of company image"),
                      6: _("affection by CO2-emission trading directive")}
CONTINGENCIES = range(len(TRANSCONTINGENCIES.keys()))

TRANSNONRECURRING = { 0: _("repair works for energy equipment"),
                      1: _("exchange of collectors"),
                      2: _("irregular maintainance costs"),
                      3: _("permits"),
                      4: _("fines/penalties"),
                      5: _("legal costs"),
                      6: _("property/natural resource damage"),
                      7: _("remediation costs"),
                      8: _("clean up costs"),
                      9: _("revenue for marketable permits"),
                     10: _("other revenues")}
NONRECURRING = range(len(TRANSNONRECURRING.keys()))

# TODO: submenu from old PanelECO1 -> check if still required
##        #choice-----------------------------------------------------------------------
##        self.cbName.Append(_U("H&C Storage"))
##        self.cbName.Append(_U("CHP"))
##        self.cbName.Append(_U("Solar thermal"))
##        self.cbName.Append(_U("Heat pump"))
##        self.cbName.Append(_U("Biomass"))
##        self.cbName.Append(_U("Chillers"))
##        self.cbName.Append(_U("Boilers"))
##        self.cbName.Append(_U("HX network"))
##        self.cbName.Append(_U("H&C distribution"))
 


#==============================================================================
#   auxiliary functions for lookup in default tables and dictionaries
#==============================================================================
#------------------------------------------------------------------------------
def getEquipmentItem(item, equipeType, fuzzy=False):
    """Return the class of the equipment as a function of the equipment type."""
    for cls in EQUIPMENT.keys():
        theList = EQUIPMENT[cls]
        for elem in theList:
            if elem[item] == equipeType:
                return (cls,elem[1],elem[2],elem[3])
    if fuzzy is True:   # a second try, only matches the first part of the
                        # equipment name
        try:
            equipeType = equipeType.split("(")[0].strip().lower()
            for cls in EQUIPMENT.keys():
                theList = EQUIPMENT[cls]
                for elem in theList:
                    if equipeType in elem[item].lower():
                        print ("einstein.modules.constants.getEquipmentItem: fuzzy match: '%s' against '%s'" % 
                               (equipeType, elem[item]))
                        return (cls,elem[1],elem[2],elem[3])
        except AttributeError:
            pass
    return "cannot find element %s)" % equipeType


def getEquipmentClass(equipeType, fuzzy=False):
    """Return the class of the untranslated equipment as a function of 
    the equipment type.
    
    :param equipeType: The untranslated equipment type string 
    :param fuzzy: If True also match the equipment type string up to a (
        against parts of the keyword. Good for generic equipment like 
        "Boiler (specify subtype)"
         
    """
    return getEquipmentItem(0,equipeType, fuzzy)[0]

def getEquipmentClassTrans(equipeType):
#------------------------------------------------------------------------------
#       returns the class of the translated equipment as a function of the equipment type
#------------------------------------------------------------------------------
    return getEquipmentItem(1,equipeType)[0]
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def getEquipmentSubClass(equipeType):
#------------------------------------------------------------------------------
#       returns the untranslated subclass of the untranslated equipment as a function of the equipment type
#------------------------------------------------------------------------------
    return getEquipmentItem(0,equipeType)[2]

#------------------------------------------------------------------------------
def getEquipmentType(equipeClass,equipeSubClass):
#------------------------------------------------------------------------------
#       returns the type of the equipment as a function of the untranslated equipment class and subclass
#------------------------------------------------------------------------------
    theList = EQUIPMENT[equipeClass]
    for eq in theList:
        if eq[2] == equipeSubClass:
            return eq[0]

    return "type undefined (equipeSubClass = %s)"%equipeSubClass
#------------------------------------------------------------------------------
def findKey(dictionary, value, default=None):
    """Return the key to the given value in the given dict, None if it
    does not exist.
    
    :param dictionary: dict to look in 
    :param value: the value to look for
    :param default: what to return if the value is not found

    :returns: The first key found that points to value. Or the default
    value if no key is found.
    
    """

    if isinstance(value, unicode):
        thisKey = toUTF8(value)
    else:
        thisKey = value
        
    for key in dictionary.keys():
        if dictionary[key] == thisKey:
            return key
        
    for dictKey in dictionary.keys():
        if isinstance(dictKey, unicode):
            logTrack("Deprecation-ERROR: function constants.findKey should no longer be used with unicode dictionaries (key = %r)"%dictKey)

    return default
            
#------------------------------------------------------------------------------
def setDebugMode(value):
#------------------------------------------------------------------------------
#   sets the DEBUG variable
#------------------------------------------------------------------------------
    global DEBUG
    if value in DEBUGMODES:
        DEBUG = value
#------------------------------------------------------------------------------    

def tempBinFromTemp(temp, heating=True):
    """Return the index of the temperature bin containing the given
    temperature.
    Gives the lower limit: T[result] <= temp !!
    
    Defaults to the heating scale Interface.T for heating=False it calculates the
    bin for the cooling scale Interface.TC 
    
    """
    EPS = 1.e-6
    result = None
    if heating:
        result = int((temp/TemperatureInterval) + EPS)
        result = min(result, NT + 1)
        result = max(result, 0)
    else:
        result = int(((temp - MinimumTemperatureC)/TemperatureIntervalC) + EPS)
        result = min(result, NT + 1)
        result = max(result, 0)
    return result

def translateConstants():
    """Update the translation of constants that contain translations.
    This function has to be called after each language change.
     
    """
    global TRANSMONTHS
    TRANSMONTHS.update({'January':_('January'),
              'February':_('February'),
              'March':_('March'),
              'April':_('April'),
              'May':_('May'),
              'June':_('June'),
              'July':_('July'),
              'August':_('August'),
              'September':_('September'),
              'October':_('October'),
              'November':_('November'),
              'December':_('December')})

    global TRANSOBJECTTYPES
    TRANSOBJECTTYPES.update({'Process':_('Process'),
                    'Distribution Line':_('Distribution Line'),
                    'Equipment':_('Equipment'),
                    'WHEE':_('WHEE')})

    global TRANSPIPETYPES
    TRANSPIPETYPES.update({"closed":_("closed circuit"),
                  "open": _("open circuit"),
                  "direct": _("built-in heater/cooler")})

    global TRANSPROCTYPES
    TRANSPROCTYPES.update({"continuous":_("continuous"),
                  "batch":_("batch")})

    global TRANSSCHEDULETYPES
    TRANSSCHEDULETYPES = {"simple":_("simple"),
                      "detailed":_("detailed"),
                      "fit to data": _("fit to data")}
    global HEATING_COOLING
    HEATING_COOLING.update({"heating": _("heating"), "cooling": _("cooling")}) 

    global TRANS_AIR_HANDLING_UNIT_STATE
    TRANS_AIR_HANDLING_UNIT_STATE.update({"no AHU": _("no AHU"), 
                                    "only air inlet": _("only air inlet"),
                                    "in- and outlet": _("in- and outlet")})
    global TRANSYESNO
    TRANSYESNO.update({"yes":_("yes"),"no":_("no")})

    
    global TRANSHXTYPES
    TRANSHXTYPES.update({"plate HX (liquid-liquid)":     _("plate HX (liquid-liquid)"),
                    "plate HX (air-air)":           _("plate HX (air-air)"),
                    "shell and tube HX (liquid-liquid)":_("shell&tube HX (liquid-liquid)"),
                    "finned tubes (liquid-air)":    _("finned tubes (liquid-air)")
                    })
    
    global TRANSWHEETYPES
    TRANSWHEEEQTYPES.update({"ElectricMotor":_("electric motor"),
                        "Compressor":_("compressor"),
                        "Other":_("other")})

    global TRANSWHEEWASTEHEATTYPES
    TRANSWHEEWASTEHEATTYPES.update({"CoolingWater":_("cooling water"),
                          "Intercooler":_("intercooler"),
                          "CoolingAir":_("cooling air")})

    global SHADINGTYPES
    SHADINGTYPES.update({"No":_("No"), 
                    "Yes,partially shaded":_("Yes,partially shaded"),
                    "Yes,fully shaded":_("Yes,fully shaded")})

    global ROOFTYPES
    ROOFTYPES.update({"Corrugated metal roof":_("Corrugated metal roof"), 
                    "Composite sandwich panels":_("Composite sandwich panels"),
                    "Concrete roof":_("Concrete roof"),
                    "Tiled roof":_("Tiled roof"),
                    "Other":_("Other")})

    global TRANSAMBIENTSINK
    TRANSAMBIENTSINK.update({"cooling tower (wet)":_("cooling tower (wet)"),
                        "cooling tower (dry)":_("cooling tower (dry)"),
                        "direct heat rejection":_("direct heat rejection")})

    global TRANSEQUIPTYPE
    TRANSEQUIPTYPE.update({"compression heat pump":_("compression heat pump"),
             "mechanical vapour recompression":_("mechanical vapour recompression"),
             "thermal heat pump":_("thermal heat pump"),
             "steam boiler":_("steam boiler"),
             "condensing boiler":_("condensing boiler"),
             "hot water boiler":_("hot water boiler"),
             "burner (direct heating)":_("burner (direct heating)"),
             "burner (indirect heating)":_("burner (indirect heating)"),
             "compression chiller (air cooled)":_("compression chiller (air cooled)"),
             "compression chiller (water cooled)":_("compression chiller (water cooled)"),
             "thermal chiller (air cooled)":_("thermal chiller (air cooled)"),
             "thermal chiller (water cooled)":_("thermal chiller (water cooled)"),
             "cooling tower (dry)":_("cooling tower (dry)"),
             "cooling tower (wet)":_("cooling tower (wet)"),
             "fresh or ground water":_("fresh or ground water"),
             "solar thermal (flat-plate)":_("solar thermal (flat-plate)"),
             "solar thermal (evacuated tubes)":_("solar thermal (evacuated tubes)"),
             "solar thermal (parabolic trough)":_("solar thermal (parabolic trough)"),
             "solar thermal (fresnel)":_("solar thermal (fresnel)"),
             "CHP engine":_("CHP engine"),
             "CHP steam turbine":_("CHP steam turbine"),
             "CHP gas turbine":_("CHP gas turbine"),
             "CHP fuel cell":_("CHP fuel cell")})

    global TRANSEQCLASSES
    TRANSEQCLASSES.update({"BB": _("Boilers, Burners and Heaters"),
                      "CH": _("Cooling equipment"),
                      "CHP": _("CHP and Heat-to-power"),
                      "ST": _("Solar Thermal"),
                      "HP": _("Heat Pumps")})

    global TRANSCHPTYPES
    TRANSCHPTYPES.update({
                 "CHP engine":_("CHP engine"),
                 "CHP steam turbine":_("CHP steam turbine"),
                 "CHP gas turbine":_("CHP gas turbine"),
                 "CHP fuel cell":_("CHP fuel cell")})

    global TRANSHPTYPES
    TRANSHPTYPES.update({"compression":_("compression"),
                    "mechanical vapour recompression":_("mechanical vapour recompression"),
                    "thermal":_("thermal")})

    global TRANSHPSUBTYPES
    TRANSHPSUBTYPES.update({'water-water':_('water-water'),
                   'water-air':_('water-air'),
                   'air-water':_('air-water'),
                   'air-air':_('air-air'),
                   'direct':_('direct'),
                   'via HX':_('via HX')})

    global TRANSHPSOURCESINK
    TRANSHPSOURCESINK.update({'air-air'    : _('air-air'),
                    'air-water'  : _('air-water'),
                    'water-air'  : _('water-air'),
                    'water-water': _('water-water')})

    global TRANSBBTYPES
    TRANSBBTYPES.update({"steam boiler":_("steam boiler"),
                    "condensing boiler":_("condensing boiler"),
                    "hot water boiler":_("hot water boiler"),
                    "burner (direct heating)":_("burner (direct heating)"),
                    "burner (indirect heating)":_("burner (indirect heating)")})

    global TRANSCHTYPES
    TRANSCHTYPES.update({ "compression chiller (air cooled)":_("compression chiller (air cooled)"),
                     "compression chiller (water cooled)":_("compression chiller (water cooled)"),
                     "thermal chiller (air cooled)":_("thermal chiller (air cooled)"),
                     "thermal chiller (water cooled)":_("thermal chiller (water cooled)"),
                     "cooling tower (dry)":_("cooling tower (dry)"),
                     "cooling tower (wet)":_("cooling tower (wet)"),
                     "fresh or ground water":_("fresh or ground water")})
    global TRANSSTTYPES
    TRANSSTTYPES.update({"Flat-plate collector":     _("Flat-plate collector"),
                    "Evacuated tube collector": _("Evacuated tube collector"),
                    "Parabolic trough collector":  _("Parabolic trough collector"),
                    "Fresnel collector":  _("Fresnel collector")
                    })


    HPIndex = 0
    BBIndex = len(HPTYPES)
    CHIndex = BBIndex + len(BBTYPES)
    STIndex = CHIndex + len(CHTYPES)
    CHPIndex = STIndex + len(STTYPES)

    global EQUIPMENT
    # EQUIPMENT associates equipment class and sub-class
    EQUIPMENT.update({"HP":                 # equipment class
             [(EQUIPTYPE[HPIndex+i],TRANSEQUIPTYPE[EQUIPTYPE[HPIndex+i]], # type of equipment, translated
               HPTYPES[i], TRANSHPTYPES[HPTYPES[i]]) # type of process, translated
               for i in range(len(HPTYPES))], 
              
             "BB": [(EQUIPTYPE[BBIndex+i],TRANSEQUIPTYPE[EQUIPTYPE[BBIndex+i]],
                     BBTYPES[i],TRANSBBTYPES[BBTYPES[i]]) 
                     for i in range(len(BBTYPES))],
                    
             "CH": [(EQUIPTYPE[CHIndex+i],TRANSEQUIPTYPE[EQUIPTYPE[CHIndex+i]],
                     CHTYPES[i],TRANSCHTYPES[CHTYPES[i]]) 
                     for i in range(len(CHTYPES))],

             "ST": [(EQUIPTYPE[STIndex+i],TRANSEQUIPTYPE[EQUIPTYPE[STIndex+i]],
                     STTYPES[i],TRANSSTTYPES[STTYPES[i]])
                     for i in range(len(STTYPES))],
                    
             "CHP": [(EQUIPTYPE[CHPIndex+i],TRANSEQUIPTYPE[EQUIPTYPE[CHPIndex+i]],
                      CHPTYPES[i],TRANSCHPTYPES[CHPTYPES[i]])
                      for i in range(len(CHPTYPES))]
             })

    global TRANSFUELTYPES
    TRANSFUELTYPES.update({'NG' : _('Natural gas'),
                         'LPG': _('Liquid petroleum gas'),
                         'HFO': _('Heavy fuel oil'),
                         'BUT': _('Butane'),
                         'PRO': _('Propane'),
                         'GO' : _('Gas oil'),
                         'BIOS': _('Solid Biomass'),
                         'BIOG': _('Biogas'),
                         'BIOF': _('Biofuels'),
                         'HNET': _('Heating network'),
                         'CNET': _('Cooling network')})

    global TRANSFLUIDTYPES
    TRANSFLUIDTYPES.update({'WATER' : _('Water'),
             'STEAM': _('Steam'),
             'ICE': _('Ice'),
             'DRYAIR': _('Dry air'),
             'MOISTAIR': _('Moist air'),
             'LIQUID': _('General liquid'),
             'GAS' : _('General gas'),
             'SOLID' : _('General solid')})

    global TRANSHRMODES
    TRANSHRMODES.update({"calculate heat exchanger": _("Actual HX Network calculation (V1 - deprecated)"),
                "estimate": _("Estimative calculation"),
                "actualHXNet_V2": _("Actual HX Network calculation")})

    global TRANSHXDESIGNSTRATEGIES
    TRANSHXDESIGNSTRATEGIES.update({"FLOWRATE":  _("Match flow rates"),
                                   "EXERGY":    _("Exergy optimisation")})

    global PARCODES
    PARCODES.update({"FECFuel_i": _("Final energy consumption (FEC) by fuel"),
            "FECel": _("Electricity consumption (FEC)"),
            "FETFuel_i": _("Final energy consumption (FET) by fuel"),
            "FETel": _("Electricity consumption (FET)"),
            "FETj": _("Final energy consumption (FET) by equipment"),
            "USHj": _("Useful supply heat supplied (USH) by equipment"),
            "USHm": _("Useful supply heat (USH) supplied to pipe"),
            "USHPipe": _("Useful supply heat (USH) distributed by pipe"),
            "UPHProc_m": _("Useful process heat (UPH_proc) at pipe"),
            "UPHProc_k": _("Useful process heat (UPH_proc) w/o heat rec."),
            "UPH": _("Useful process heat (UPH) incl. heat rec."),
            "UPHc": _("Useful process heat (UPH) - inflow"),
            "UPHw": _("Useful process heat (UPH) - outflow"),
            "Product": _("Quantity of product"),
            "RawMat": _("Quantity of raw material or other input"),
            "TDrive": _("Equipment driving temperature (inlet)"),
            "TReject": _("Equipment heat rejection temperature (inlet)"),
            "Control": _("Equipment master control"),         
            "PTInFlow": _("Process inlet temperature"),
            "PTOutFlow": _("Process outlet temperature"),
            "mInFlowYear": _("Process inlet flow rate"),
            "mOutFlowYear": _("Process outlet flow rate")})


    global TRANSPRODUCTCODES
    TRANSPRODUCTCODES.update({"ZZ999":_("ZZ999: auxiliary services"),
                    "ZZ000":_("ZZ000: other products"),
                    "DA010":_("DA010: milk products"),
                    "DA020":_("DA020: fruits/vegetables/herbs"),
                    "DA030":_("DA030: sugar"),
                    "DA040":_("DA040: beer"),
                    "DA050":_("DA050: fats/oils"),
                    "DA060":_("DA060: chocolate/cacao/coffee"),
                    "DA070":_("DA070: starch/potatoes/grain mill products"),
                    "DA080":_("DA080: wine/beverage"),
                    "DA090":_("DA090: meat"),
                    "DA100":_("DA100: fish"),
                    "DA110":_("DA110: aroma")})

    global TRANSOMCATEGORIES
    TRANSOMCATEGORIES.update({"utilities":    _("Utilities"),
                         "opmaterials":  _("Operating materials"),
                         "labour":       _("Labour"),
                         "extcost":      _("External costs"),
                         "compliance":   _("Regulatory compliance costs"),
                         "insurance":    _("Insurance cost"),
                         "liability":    _("Future liability"),
                         "water":        _("Water cost")})

    global TRANSCONTINGENCIES
    TRANSCONTINGENCIES.update({0: _("tax disadvantages (other than energy tax)"),
                          1: _("obligatory provisions"),
                          2: _("costs for remediation activities"),
                          3: _("cost for compliance with legislation"),
                          4: _("negative impacts on market share"),
                          5: _("deterioration of company image"),
                          6: _("affection by CO2-emission trading directive")})

    global TRANSNONRECURRING
    TRANSNONRECURRING.update({ 0: _("repair works for energy equipment"),
                          1: _("exchange of collectors"),
                          2: _("irregular maintainance costs"),
                          3: _("permits"),
                          4: _("fines/penalties"),
                          5: _("legal costs"),
                          6: _("property/natural resource damage"),
                          7: _("remediation costs"),
                          8: _("clean up costs"),
                          9: _("revenue for marketable permits"),
                         10: _("other revenues")})

#------------------------------------------------------------------------------
class updateConstants():
#------------------------------------------------------------------------------
#   gets the updated values for translatable constants
#   this function has to be called after each language change !!!
#
#   OLD CONSTANTS TRANS_XY... should be substituted by attribute of this class
#------------------------------------------------------------------------------

    def __init__(self):
        NEWTRANSHXTYPES = {"plate HX (liquid-liquid)":     _U("plate HX (liquid-liquid)"),
                        "plate HX (air-air)":           _U("plate HX (air-air)"),
                        "shell and tube HX (liquid-liquid)":_U("shell&tube HX (liquid-liquid)"),
                        "finned tubes (liquid-air)":    _U("finned tubes (liquid-air)")
                        }

        self.HXTYPES = NEWTRANSHXTYPES

if __name__ == '__main__':
    print 'Type    = '+getEquipmentType("BB","direct heating")
