# -*- coding: utf-8 -*-
#==============================================================================
#
#   E I N S T E I N
#
#   Expert System for an Intelligent Supply of Thermal Energy in Industry
#   (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#   LICENSE
#
#------------------------------------------------------------------------------
#
#   License class for management of user access
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
import logging
import time
import os
import sys
import codecs
import shutil
from ConfigParser import ConfigParser
from constants import APPDATA
from modules.messageLogger import *
from modules.checkLicense import checkAuditor, getHardWareId
from modules.xmlIO import openfilecreate
from auxiliary.auxiliary import toUTF8

global KeyFile
KeyFile = os.path.join(APPDATA, 'license.key')

class License():

    def __init__(self):
        """ check license key """

        self.readKeyFile()

    def setDemoPars(self):
        """ sets the License info to the
            parameters for demo installation
        """

        self.config.add_section("USER")
        self.config.set("USER", "user", "demo")
        self.config.set("USER", "institute", "demo version")
        self.config.set("USER", "city", "-")
        self.config.set("USER", "country", "-")
        self.config.set("USER", "address", "---")
        self.config.set("USER", "VATNo", "---")

        self.config.add_section("KEY")
        self.config.set("KEY", "no", "-")
        self.config.set("KEY", "key", "0000-0000-0000-0000")
        self.config.set("KEY", "id", "-")

    def check(self, auditorId=None):
        if auditorId is not None:
            self.check = checkAuditor(auditorId)
        else:
            self.check = 0
##        except:
##            logError("Error reading key file")

    def readKeyFile(self, keyFile=None):
        """ reads the configuration data from the file "license.key"
        """
        global KeyFile
        if keyFile is not None:
            KeyFile = keyFile
            
        self.config = ConfigParser()
        # Check for existing key file, if none is found
        # copy the default config file in the place.
        if os.path.isfile(KeyFile):
            self.config.read(KeyFile)
        else:
            try:    #try-except is necessary, as license.py is used also independently of einsteinMain
                logWarning(_("No license key file found"))
            except:
                pass
            self.setDemoPars()
##            with open(KeyFile, 'wb') as kf:
##                self.config.write(kf)
            
        self.user = self.getOption("USER", "user")
        self.institute = self.getOption("USER", "institute")
        self.city = self.getOption("USER", "city")
        self.country = self.getOption("USER", "country")
        self.address = self.getOption("USER", "address")
        self.VATNo = self.getOption("USER", "VATNO")
        
##        print "License.readKeyFile:\nuser %s\ninstitute %s (%s, %s)\naddress %s\nVATNO %s"%(self.user,
##                                                                               self.institute,
##                                                                               self.city,
##                                                                               self.country,
##                                                                               self.address,
##                                                                               self.VATNo)
        self.no = self.getOption("KEY", "no")
        key = self.getOption("KEY", "key")
        self.key = key.replace("-", "")
        self.id = self.getOption("KEY", "id")

        self.info = "%s (%s, %s)"%(self.institute, self.city, self.country)

    def getOption(self, section, option):
        """ auxiliary function, combining config.get
            with a check of existence of the option
            otherwise returns None
        """
        
        try:
            return str(self.config.get(section, option))
        except:
            return None

    def request(self):
        hardwareId = getHardWareId()
        self.config.set("KEY", "id", hardwareId)


        import database.dbio as dbio
        from auxiliary.auxiliary import noneCheck
        #TODO -> function in project.py for getting auditor info 
        stool = dbio.get("stool")
        if stool is not None:
            auditorId = stool.Auditor_ID

            auditor = dbio.get("auditor", id=auditorId)

            city = noneCheck(auditor.City, "-")
            country = noneCheck(auditor.Country, "-")
            institute = noneCheck(auditor.Company, "-")
            user = noneCheck(auditor.Name, "-")

            if any([(x == "-" or x == "" or x.startswith("?")) for x in (user, institute, city, country)]):
                return False

            self.config.set("USER", "user", user)
            self.config.set("USER", "institute", institute)
            self.config.set("USER", "city", city)
            self.config.set("USER", "country", country)
            
        else:
            logError("License.request: Error reading auditor information")
            return False


        requestFile = openfilecreate(text='Output file for request of a license key',filetype='TXT files (*.txt)|*.txt')
        if requestFile is not None:
            with open(requestFile, 'wb') as rf:
                self.config.write(rf)
            logMessage(_("Request for license key created as %s"%requestFile))

        return True
                
    
