######################################################
#
#   dataHub
#
#   Data hub for management of monitoring data and EMS
#
######################################################

if __name__=="__main__":
    import sys
    import os
    sys.path.append(os.path.abspath(".."))
    sys.path.append(os.path.abspath("../.."))

from sqlobject import connectionForURI, SQLObject, MixedCaseStyle, IntCol, FloatCol, StringCol, DateTimeCol
from sqlobject.dbconnection import ConnectionHub
from datetime import datetime
import dbio
from lib.fileIO.config import Config
from constants import APPDATA
from modules.messageLogger import logError

chub = ConnectionHub()

class DataHub():

    def __init__(self):
        self.conn = None
        self.sensorIdsStr = None
        self.lastRead = datetime(2000, 1, 1)

        self.defaultPlant = 1

    def connect(self, lastRead = None):
        self.conn = connectionForURI(self.connStr)
        chub.threadConnection = self.conn
        if lastRead is not None:
            self.lastRead = lastRead
        else:
            self.lastRead = datetime(2000, 1, 1)

        # test connection: Plant table has to be within data base
        try:
            plant = dbio.get(Plant)
            return 0
        except:
            logError("DataHub.connect: Error reading plant table in data base")
            return 1


    def getSensors(self, plantId=None):
        """ retrieves the list of sensorIds converted into a query string """


        checkSensorDict = False
        if plantId is None:
            plantId = self.defaultPlant
            checkSensorDict = True

        sensors = Sensor.selectBy(plantId=plantId)
        sensorIds = [sensor.id for sensor in sensors]
        self.sensorIdsStr = dbio.listToQueryString(sensorIds)

        if checkSensorDict:
            #TODO: check that the numbers identified in the configuration file are in coincidence with
            # the sensors available for this plant
            pass

        return sensorIds
        
    def read(self, readUntil=None):
        """ reads new data from data hub """

        if self.sensorIdsStr is None:
            return []
        
        if readUntil is None:
            query = "sensorId IN (%s) AND dateTime > '%s' ORDER BY dateTime"%(self.sensorIdsStr, self.lastRead)
        else:
            query = "sensorId IN (%s) AND dateTime > '%s' AND dateTime <= '%s' ORDER BY dateTime"%(self.sensorIdsStr, self.lastRead, readUntil)

        new = dbio.selectByQuery(Data, query)

        newData = [(elem.sensorId, elem.dateTime, elem.value) for elem in new]

        if len(newData) > 0:
            self.lastRead = newData[-1][1]

        return newData


    def write(self, dataList):
        """ writes data from a data list containing tuples (sensorId, datetime, value)
        """

        for s, t, f in dataList:
            dbio.insertRow(Data, sensorId=int(s), dateTime=t, value=float(f))

    def checkSource(self, source):
        
##        try:
        if True:
            configFile = APPDATA + "/" + source
            self.config = Config(configFile) # get configuration parameters before set-up

            host = self.config.get("connection", "host")
            user = self.config.get("connection", "user")
            pwd = self.config.get("connection", "pwd")
            db = self.config.get("connection", "db")
            plant = self.config.get("sensors", "plant")
            labels = self.config.get("sensors", "labels")
            sensors = self.config.get("sensors", "sensors")
            self.sensors = sensors

            self.connStr = 'mysql://%s:%s@%s/%s?charset=utf8'%(user, pwd, host, db)

            self.defaultPlant = int(plant)
            self.sensorDict = {}
            for s, sensor in enumerate(sensors):
                self.sensorDict.update({labels[s]: sensors[s]})
            return 0
##        except:
##            return 1
                                 
                 
            
hub = DataHub()
chub = ConnectionHub()
conn = hub.conn

class Data(SQLObject):
    _connection = chub
    class sqlmeta:
        style = MixedCaseStyle()
        table = "data"

    sensorId = IntCol(default=None)
    dateTime = DateTimeCol(default=None)
    value = FloatCol(default=None)
        

class Plant(SQLObject):
    _connection = chub
    class sqlmeta:
        style = MixedCaseStyle()
        table = "plants"

    description = StringCol(default=None)
    lastUpdate = DateTimeCol(default=None)

class Sensor(SQLObject):
    _connection = chub
    class sqlmeta:
        style = MixedCaseStyle()
        table = "sensors"

    label = StringCol(default=None)
    plantId = IntCol(default=None)
    unit = StringCol(default=None)
    frecuency = FloatCol(default=None)
    description = StringCol(default=None)
        

if __name__=="__main__":

    hub.connStr = "someString"
    hub.connect()
    hub.getSensors(1)
    data = hub.read()
    print "Number of data read: %s"%len(data)
    print "Time of most recent data entry: %s"%hub.lastRead
    if len(data) > 0:
        for entry in data:
            print entry
        
        
