#!/usr/bin/env python
# -*- coding: utf-8 -*-

#==============================================================================
#
#   einstein
#
#------------------------------------------------------------------------------
#
#   einstein main
#
#==============================================================================
#
#   (C) copyleft energyXperts.BCN (E4-Experts SL), Barcelona, Spain 2008,2009
#   www.energyxperts.net / info@energyxperts.net
#
#   This program is free software: you can redistribute it or modify it under
#   the terms of the GNU general public license v3 as published by the Free
#   Software Foundation (www.gnu.org).  #
#==============================================================================
import sys
import os

LIBPATH = os.path.abspath("..")
sys.path.append(LIBPATH)

from constants import VERSION, APPDATA, APPNAME
from globals import Globals, configFile, defaultConfigFile, logger
from GUI.status import Status
import database.dbio as dbio

from lib.fileIO.config import Config
from lib.dateTime.dateTimeTools import stringToDate3, dateToDateTime

#TODO: many Status info should be shifted to Globals

from modules.project import Project
from modules.interfaces import Interfaces
from modules.modulesClass import Modules

from view.units import UNITSYSTEM



from modules.energy.realTime import RealTimeHandler
from control.license import License

from GUI.einsteinGUI import EinsteinApp


def setUnitSystem():
    stool = dbio.get("stool")
    selectedUnits = stool.UnitsTool

    if selectedUnits in UNITSYSTEM.keys():
        Status.Units = selectedUnits
    else:
        Status.Units = "SI-kWh"
        stool.UnitsTool = Status.Units

    from view.units import UnitConversion
    Globals.unitConversion = UnitConversion(Globals.currency)


def setUp():
    glb = Globals()

    # initialize Einstein helpers
    Globals.conf = Config(configFile, defaultConfigFile)
    Globals.tracker.logMessage("\n==================================\n" + \
                             ("Starting EINSTEIN Version %s" % VERSION) + \
                             "\n==================================\n")



    # attempt to connect to database
    # if not possible, invoke a database setup dialog, so the user can
    # attempt to change some parameters and fix the problem.
    DBHost = Globals.conf.get('DB', 'DBHost')
    DBUser = Globals.conf.get('DB', 'DBUser')
    DBPass = Globals.conf.get('DB', 'DBPass')
    DBName = Globals.conf.get('DB', 'DBName')
    
    Globals.DBCon = dbio.DBConnector(DBHost, DBUser, DBPass, DBName)
    Globals.SQL = Globals.DBCon.connect()
    Globals.DBName = DBName
    Globals.DBHost = DBHost if DBHost != "" else "localhost"

    # initialize language
    glb.setLanguage()

    # initialize unit system
    setUnitSystem()

    # upgrade to current version if necessary
    from modules.upgrade import upgradeEINSTEIN
    try:
        upgradeEINSTEIN()
    except Exception, e:
        import traceback 
        dummy, dummy, tb = sys.exc_info()
        while tb:
            lineNo = traceback.tb_lineno(tb)
            tb = tb.tb_next
        logger.showError(_("The database update has failed with the error: %s "\
                     "in line %s. "
                     "Look into the logs in %s and %s "\
                     "if you need more information about the error.") %
                     (e, lineNo, os.path.join(APPDATA, APPNAME),  
                      os.path.abspath('../../..')))
        raise



    # Set-up tool modules and load last opened project

    logger.logTrack("Loading EINSTEIN kernel modules")
    Status.main = None

    Status.mod = None
    Status.int = Interfaces()
    Status.prj = Project()
    Status.mod = Modules()
    logger.logTrack("Loading EINSTEIN kernel modules OK\n")

    # read in license information
    Status.License = License()
    Status.License.check(Status.AuditorId)

    
    
#   Application start
def main():    
    # read in command line arguments
    import argparse

    setUp()

    parser = argparse.ArgumentParser(description='EINSTEIN')
    parser.add_argument('projectName', nargs='?', default=None, help='name of the project to be simulated (default: last project opened)')
    parser.add_argument('alternativeProposalNo', nargs='?', default=1, help='Number of the alternative to be simulated (default: 1 / last alternative opened)')
    parser.add_argument('--real-time', nargs='?', default=None, help='carries out real time simulation from data')
    parser.add_argument('--rtmode', nargs='?', default=None, const=None, help='set to real-time-emulation or ex-post if desired')
    parser.add_argument('--start', nargs='?', default=None, const=None, help='simulation from date dd-mm-yyyy')
    parser.add_argument('--stop', nargs='?', default=None, const=None, help='simulation to date dd-mm-yyyy')
    parser.add_argument('--forecast', nargs='?', default="(0, 0)", const="(0, 0)", help='total time and time step of forecast')
    parser.add_argument('--opt', nargs='?', default=None, const=None, help='parameter to be optimised')

    args = parser.parse_args()

    Globals.realTime = args.real_time
    Globals.fromTo = [args.start, args.stop]
    Globals.rtMode = args.rtmode if args.rtmode in ["real-time", "real-time-emulation", "ex-post"] else "real-time-emulation"
    Globals.projectFile = args.projectName
    Globals.forecast = eval(args.forecast) if args.forecast is not None else None
    Globals.opt = args.opt


    # Open GUI (if not in real-time mode)
    if Globals.realTime is not None:
        logger.logTrack('@Real-time mode %s' % (Globals.realTime))
        app = None
##        if not Status.License.check: return
            
    else:
        # open GUI
        app = EinsteinApp()
        Globals.GUI = app.frame
        Globals.tracker.linkToGUI(Globals.GUI)
        Globals.GUI.updateDrawings()
        
        ret = Globals.GUI.handleLicense()

    logger.logMessage(_("EINSTEIN started. ") + _("[Auditor: %s. Database: %s@%s]"%(Status.prj.getAuditorName(Status.AuditorId),
                                                                             Globals.DBName,
                                                                             Globals.DBHost)))

    # Start GUI main loop or execute command line options
    if Globals.realTime:

        rt = RealTimeHandler(Globals.realTime, Globals.rtMode, forecast=Globals.forecast, opt=Globals.opt)
        startTime = None if Globals.fromTo[0] is None else dateToDateTime(stringToDate3(Globals.fromTo[0]))
        stopTime = None if Globals.fromTo[1] is None else dateToDateTime(stringToDate3(Globals.fromTo[1]))
        rt.start(startTime, stopTime)

    else:
        if Globals.SQL is None:
            e = Globals.DBCon.error
            if app: app.frame.showWarning('Cannot connect to database. ' + \
                                   'Error is:\n\n%s\n\nPlease verify your database parameters.' % str(e))

            dlgdb = DlgDatabase(parent=None, id= -1)
            rsp = dlgdb.ShowModal()
            if rsp == wx.ID_OK:
                # read ini file again
                Globals.conf = HelperClass.ConfigHelper()
            # user has canceled
            sys.exit(1)

    

        app.MainLoop()



if __name__ == '__main__':
    
    PROFILE = False
    if PROFILE is True:
        try:
            import cProfile
            import pstats
            cProfile.run("main()", "runstats.txt")
            s = pstats.Stats("runstats.txt")
            s.sort_stats("cumulative")
            s.print_stats(100)
        except ImportError:
            print >> sys.stderr, "Profiling module could not be imported, starting normal." 
            main()
    else:
        main()
    
