# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	GeneralDataObject: objects for general data entry
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from constants import EPSILON, PIPETYPES

from globals import logger
from GUI.status import Status

import database.dbio as dbio
from auxiliary.auxiliary import noneCheck
from auxiliary.dataTransfer import DataTransfer

from model.objects.genericObject import GenericObject

# keys for parameters which are 1:1 passed to data
# struture: sqlLabel: (attr.-name [None if identical to sqlLabel], dictionary for conversion)


# TODO:
# currently this key is used for
# a) list those parameters to be written to SQL
# b) list those parameters to be written to GUI
# subset of (a) and (b) may be different, with conversions in between, which may not be only 1:1 conversions

yesNoDict = {"no": False, "yes": True}

SQLKEYS_QUESTIONNAIRE = {"Name": (None, None),
            "City": (None, None),
            "DescripIndustry": (None, None),
            "Branch": (None, None),
            "SubBranch": (None, None),
            "Contact": (None, None),
            "Role": (None, None),
            "Address": (None, None),
            "Phone": (None, None),
            "Fax": (None, None),
            "Email": (None, None),
            "NEmployees": (None, None),
            "Turnover": (None, None),
            "ProdCost": (None, None),
            "BaseYear": (None, None),
            "Growth": (None, None),
            "Independent": (None, None),
            "OMThermal": (None, None),
            "OMElectrical": (None, None),
            "PercentFuelTotcost": (None, None),
            "PercentElTotcost": (None, None),
            "HPerDayInd": (None, None),
            "NShifts": (None, None),
            "NDaysInd": (None, None),
            "NoProdStart_1": (None, None),
            "NoProdStop_1": (None, None),
            "NoProdStart_2": (None, None),
            "NoProdStop_2": (None, None),
            "NoProdStart_3": (None, None),
            "NoProdStop_3": (None, None)}        # sqlkeys of data from SQL tables to be read; in case of different names for


MANDATORYDATA = []
OPTIONALDATA = []
                                 
class QuestionnaireObject(GenericObject):
    """
    Base object representing a process in the system model
    """
    def __init__(self, prnt, id=None, no=None, sqlData=None):
        """
        :param inStreams, outStreams: streamObjects for inflowing and outflowing streams
        """

        super(QuestionnaireObject, self).__init__(prnt, id, no, sqlData,
                 sqlKeys=SQLKEYS_QUESTIONNAIRE, mandatoryData=MANDATORYDATA, optionalData=OPTIONALDATA,
                 sqlTableName="questionnaire", sqlObjectList=None,
                 keyPId=None, colObjectNo=None)

SQLKEYS_PROJECTSTATUS = {"ProjectName": (None, None),
                   "CreationDate": (None, None),
                   "CreatedBy": (None, None),
                   "LastModified": (None, None),
                   "ModifiedBy": (None, None)}        # sqlkeys of data from SQL tables to be read; in case of different names for

class ProjectStatusObject(GenericObject):
    """
    Base object representing a process in the system model
    """
    def __init__(self, prnt, id=None, no=None, sqlData=None):
        """
        :param inStreams, outStreams: streamObjects for inflowing and outflowing streams
        """

        super(ProjectStatusObject, self).__init__(prnt, id, no, sqlData,
                 sqlKeys=SQLKEYS_PROJECTSTATUS, mandatoryData=MANDATORYDATA, optionalData=OPTIONALDATA,
                 sqlTableName="sproject", sqlObjectList=None,
                 keyPId=None, colObjectNo=None)

SQLKEYS_PRODUCT = {"Product": (None, None),
                   "ProductCode": (None, None),
                   "QProdYear": (None, None),
                   "ProdUnit": (None, None),
                   "TurnoverProd": (None, None),
                   "ElProd": (None, None),
                   "FuelProd": (None, None)}        # sqlkeys of data from SQL tables to be read; in case of different names for


class ProductObject(GenericObject):
    """
    Base object representing a process in the system model
    """
    def __init__(self, prnt, id=None, no=None, sqlData=None):
        """
        :param inStreams, outStreams: streamObjects for inflowing and outflowing streams
        """

        super(ProductObject, self).__init__(prnt, id, no, sqlData,
                 sqlKeys=SQLKEYS_PRODUCT, mandatoryData=MANDATORYDATA, optionalData=OPTIONALDATA,
                 sqlTableName="qproduct", sqlObjectList=prnt.productList,
                 keyPId="Questionnaire_id", colObjectNo=None)

