# -*- coding: utf-8 -*-
#============================================================================== 				
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	CALENDAR
#			
#------------------------------------------------------------------------------
#			
#	Short description:
#	
#	Auxiliary functions for the definition of calendars
#       (processing of holiday periods)
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 

import numpy as np
from datetime import date, datetime, timedelta
from GUI.status import Status
from modules.messageLogger import *

PERIODREFERENCE = date(2007, 1, 1) # year 2007 starts and ends on Monday

class Calendar():
    """
    Class contains a vector (np-array) of 365 entries with 0es for holidays.
    Further attributes:

    days: total number of days
    weekDays: number of days between Monday and Friday
    holidayPeriods: list of tuples (startDate, stopDate) with  startDate, stopDate = datetime
    """

    def __init__(self, holidayPeriods = []):
        
        self.data = np.array([1]*365)
        for i, period in enumerate(holidayPeriods):
            try:
                startYear = period[0].year
                startDay = (period[0] - date(startYear, 1, 1)).days
            except:
                continue

            try:
                endYear = period[1].year
                endDay = (period[1] - date(startYear, 1, 1)).days
            except:
                continue

            if startDay >= 0 and startDay < 365 and endDay >=0 and endDay < 365:
                if startDay <= endDay:
                    for i in range(startDay, endDay+1):
                        self.data[startDay:endDay+1] = 0
                else:
                    self.data[startDay:-1] = 0
                    self.data[0:endDay] = 0                    
            else:
                continue

        self.countWeekDays()

    def countWeekDays(self):
        self.days = np.sum(self.data)
        self.weekEndDays = 0
        for i in range(5,365, 7):
            self.weekEndDays += np.sum(self.data[i:i+2])
        self.weekDays = self.days - self.weekEndDays
        

def overlapCalendars(cal1, cal2):
    cal = Calendar()
    cal.data = cal1.data * cal2.data

    cal.countWeekDays()
    return cal
    
def getGeneralCalendar():
    generalHolidays = Status.prj.getHolidays(datetime=True)
    return Calendar(generalHolidays)
    
    
    

