# -*- coding: utf-8 -*-
#==============================================================================                 
#
#	 E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#    CCheck (Consistency Check)
#            
#------------------------------------------------------------------------------
#            
#    Short description:
#    
#    Functions for consistency checking of data
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------        
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================                 
from math import fabs, log, exp
from constants import *                
from modules.messageLogger import *
from GUI.GUITools import *     #needed for function check
from ccTracker import estimateTracker, screen, conflict, cycle
from ccPar import CCPar
import numpy as np

global DEBUG
EPSILON = 1.e-10     # required accuracy for function "isequal"
INFINITE = 1.e99    # numerical value assigned to "infinite"
MINIMUM_VALUE = 1.e-10
DEFAULT_SQERR = 1.e-6 # default value for sqerr assigned to questionnaire values (until V2.1 default had been set to 1e-4)
NUMERIC_ERR = 1.e-10 # accuracy of numeric calculations
CONFLICT_ERR = 1.e-6
MAX_SQERR = 0.09     # critical square error for screening
CONFIDENCE = 4.0    # maximum relation between statistical error and abs.min/max
CHECKMODE = "BEST"  # MEAN or BEST. MEAN recommened as BEST doesn't give consistent results
MAXTRACKENTRIES = 10 # maximum number of variables to be tracked

def setCheckMode(mode):
    global CHECKMODE
    CHECKMODE = mode
    
TEST = False
TESTCASE = 3


#------------------------------------------------------------------------------
def ccheck1(y0, y1):
#------------------------------------------------------------------------------
#   Carries out consistency checking of the actual value with one external
#   calculated input.
#------------------------------------------------------------------------------

    cycle.checkBalance([y0.val, y1.val],name=y0.name, parType = y0.parType)
    
    if DEBUG in ["ALL", "CHECK"]:
        print "CCheck1_(before...)________________________________"
        y0.show()
        y1.show()
        print "________________________________Mode = ", CHECKMODE

    y = CCPar("CCheck1", parType=y0.parType)
    
    if (y0.val is None and y1.val is None): #Case 1: Nothing to do
        y.update(y0)
        y.valMin = max(y0.valMin, y1.valMin)
        y.valMax = min(y0.valMax, y1.valMax)
        
    elif y1.val is None: #Case 2: Pnom1 = ok, Pnom2 = None
        y.update(y0)
        y.valMin = max(y0.valMin, y1.valMin)
        y.valMax = min(y0.valMax, y1.valMax)
                
    elif y0.val is None: #case 3: Pnom1 = None, Pnom2 = ok
        y.update(y1)
        y.valMin = max(y0.valMin, y1.valMin)
        y.valMax = min(y0.valMax, y1.valMax)
               
    else: #case 4 Pnom1 = ok, Pnom2 = ok
        if CHECKMODE == "BEST":
            y = bestOf(y0, y1)
        else:
            y = meanValueOf(y0, y1)
                     
#..............................................................................
# Updating values
   
    if DEBUG in ["ALL", "CHECK"]:
        print "CCheck1_(... before constrain ...)_________________"
        y.show()
        print "___________________________________________________"

    y.constrain()
    
    y0.update(y)
    y1.update(y)

    if DEBUG in ["ALL", "CHECK"]:
        print "CCheck1_(... and after adjustment)_________________"
        y0.show()
        y1.show()
        print "___________________________________________________"


###------------------------------------------------------------------------------
##def ccheck2(y0, y1, y2):
###------------------------------------------------------------------------------
###   Carries out consistency checking of the actual value with two externally
###   calculated inputs.
###------------------------------------------------------------------------------
###..............................................................................
##
##    bal = cycle.checkBalance([y0.val, y1.val, y2.val],name=y0.name, parType = y0.parType)
##    
##    if DEBUG in ["ALL", "CHECK"]:
##        print "CCheck1_(before...)________________________________"
##        y0.show()
##        y1.show()
##        y2.show()
##        print "________________________________Mode = ", CHECKMODE
##
##    row = CCRow("ccheck2-row", 3)
##    y = CCPar("ccheck2-y", parType=y0.parType)
##    
##    row[0] = y0
##    row[1] = y1
##    row[2] = y2
##                
##    if CHECKMODE == "BEST":
##        y = bestOfRow(row)
##    else:
##        y = meanOfRow(row)
##
##    y0.update(y)
##    y1.update(y)
##    y2.update(y)
##
##    if DEBUG in ["ALL", "CHECK"]:
##        print "CCheck2_(... and after adjustment)_________________"
##        y0.show()
##        y1.show()
##        y2.show()
##        print "___________________________________________________"
##    
###------------------------------------------------------------------------------
###------------------------------------------------------------------------------
##def ccheck2old(y0, y1, y2):
###------------------------------------------------------------------------------
###   Carries out consistency checking of the actual value with two externally
###   calculated inputs.
###------------------------------------------------------------------------------
###..............................................................................
##
##    if DEBUG in ["ALL", "CHECK"]:
##        print "CCheck2_(before...)________________________________"
##        y0.show()
##        y1.show()
##        y2.show()
##        print "_____________________________________Mode = ", CHECKMODE
##
##
##    ccheckMsg = []
##
##    y = CCPar("CCheck2")
##
##    if y1.val == None: 
##        if y2.val == None:
##            if y0.val == None:
###..............................................................................
###Case 1: Nothing to do
##                
###                print "Case 1: all parameters are unknown. Calculation not possible"
###                ccheckMsg.append("Case 1: all parameters are unknown. Calculation not possible")
##                y.val = None
##                y.sqerr = INFINITE
##                y.valMin = max(y0.valMin, y1.valMin, y2.valMin)
##                y.valMax = min(y0.valMax, y1.valMax, y2.valMax)
##
##            else:               
###..............................................................................
###Case 2: USH3 known, USH1 unknown,USH2 unknown,
##                y.update(y0)
##                y.valMin = max(y0.valMin, y1.valMin, y2.valMin)
##                y.valMax = min(y0.valMax, y1.valMax, y2.valMax)
###                print "Case 2: USH3 known. USH1 and USH2 adjusted"
##             
##        else:
##            if y0.val == None:
###..............................................................................
###Case 3: USH3 unknown, USH1 unknown,USH2 known
##
##                y.update(y2)
##                y.valMin = max(y0.valMin, y1.valMin, y2.valMin)
##                y.valMax = min(y0.valMax, y1.valMax, y2.valMax)
###                print "Case 3: USH2 known. USH1 and USH3 adjusted"
##                
##            else:
###..............................................................................
###Case 4: USH3 known, USH1 unknown,USH2 known
##                if isequal(y2.val, y0.val):
###                    print "Case4: USH1 unknown. USH2 and USH3 redundant and consistent"
##                    pass
##                else:
###                    print "Case4: USH1 unknown. USH2 and USH3 redundant and not consistent"
##                    pass
##
##                if CHECKMODE == "BEST":
##                    y = bestOf(y0, y2)
##                else:
##                    y = meanValueOf(y0, y2)
##                y.valMin = max(y.valMin, y1.valMin)
##                y.valMax = min(y.valMax, y1.valMax)
##                    
##    else:
##        if y2.val == None:
##            if y0.val == None:
###..............................................................................
###Case 5: USH3 unknown, USH1 known,USH2 unknown,
##                y.update(y1)
##                y.valMin = max(y0.valMin, y1.valMin, y2.valMin)
##                y.valMax = min(y0.valMax, y1.valMax, y2.valMax)
###                print "Case 5: USH1 known. USH2 and USH3 adjusted"
##                
##            else:
###..............................................................................
###Case 6: USH3 known, USH1 known,USH2 unknown,
##                if isequal(y1.val, y0.val):
###                    print "Case6: USH2 unknown. USH1 and USH3 redundant and consistent"
##                    pass
##                else:
###                    print "Case6: USH2 unknown. USH1 and USH3 redundant and not consistent"
##                    pass
##
##                if CHECKMODE == "BEST":
##                    y = bestOf(y0, y1)
##                else:
##                    y = meanValueOf(y0, y1)
##                y.valMin = max(y.valMin, y2.valMin)
##                y.valMax = min(y.valMax, y2.valMax)
##                    
##        else:
##            if y0.val == None:
###..............................................................................
###Case 7: USH3 unknown, USH1 known,USH2 known,
##
###                if isequal(y1.val, y2.val):
###                    print "Case7: USH3 unknown. USH2 and USH1 redundant and consistent"
###                else:
###                    print "Case7: USH3 unknown. USH2 and USH1 redundant and not consistent"
##
##                if CHECKMODE == "BEST":
##                    y = bestOf(y1, y2)
##                else:
##                    y = meanValueOf(y1, y2)
##                y.valMin = max(y.valMin, y0.valMin)
##                y.valMax = min(y.valMax, y0.valMax)
##                                        
##            else:
##    
###..............................................................................
### Case 8:USH3 known, USH1 known,USH2 known,
##
##                    if CHECKMODE == "BEST":
##                        y = bestOf3(y0, y1, y2)
##                    else:
##                        y = meanValueOf3(y0, y1, y2)
##                                                                            
###..............................................................................
### Updating values
##
##    y.constrain()
##    
##    y0.update(y)
##    y1.update(y)
##    y2.update(y)
##
##    if DEBUG in ["ALL", "CHECK"]:
##        print "CCheck2_(... and after adjustment)_________________"
##        y0.show()
##        y1.show()
##        y2.show()
##        print "___________________________________________________"
##
##
###------------------------------------------------------------------------------
###------------------------------------------------------------------------------
##def ccheck3(y0, y1, y2, y3):
###------------------------------------------------------------------------------
###   Carries out consistency checking of the actual value with three externally
###   calculated inputs.
###------------------------------------------------------------------------------
###..............................................................................
##
##    cycle.checkBalance([y0.val, y1.val, y2.val, y3.val],name=y0.name, parType = y0.parType)
##    
##    row = CCRow("ccheck3-row", 4)
##    y = CCPar("ccheck3-y")
##    
##    row[0] = y0
##    row[1] = y1
##    row[2] = y2
##    row[3] = y3
##                
##    if CHECKMODE == "BEST":
##        y = bestOfRow(row)
##    else:
##        y = meanOfRow(row)
##
##    y0.update(y)
##    y1.update(y)
##    y2.update(y)
##    y3.update(y)
##
##    
###------------------------------------------------------------------------------
###------------------------------------------------------------------------------
##def ccheck4(y0, y1, y2, y3, y4):
###------------------------------------------------------------------------------
###   Carries out consistency checking of the actual value with three externally
###   calculated inputs.
###------------------------------------------------------------------------------
###..............................................................................
##
##    cycle.checkBalance([y0.val, y1.val, y2.val, y3.val, y4.val],name=y0.name, parType = y0.parType)
##
##    row = CCRow("ccheck4-row", 5)
##    y = CCPar("ccheck4-y", parType=y0.parType)
##    
##    row[0] = y0
##    row[1] = y1
##    row[2] = y2
##    row[3] = y3
##    row[4] = y4
##                
##    if CHECKMODE == "BEST":
##        y = bestOfRow(row)
##    else:
##        y = meanOfRow(row)
##
##    y0.update(y)
##    y1.update(y)
##    y2.update(y)
##    y3.update(y)
##    y4.update(y)
##
##    
###------------------------------------------------------------------------------
###------------------------------------------------------------------------------
##def ccheck5(y0, y1, y2, y3, y4, y5):
###------------------------------------------------------------------------------
###   Carries out consistency checking of the actual value with three externally
###   calculated inputs.
###------------------------------------------------------------------------------
###..............................................................................
##
##    cycle.checkBalance([y0.val, y1.val, y2.val, y3.val, y4.val, y5.val],name=y0.name, parType = y0.parType)
##
##    row = CCRow("ccheck5-row", 6)
##    y = CCPar("ccheck5-y", parType=y0.parType)
##    
##    row[0] = y0
##    row[1] = y1
##    row[2] = y2
##    row[3] = y3
##    row[4] = y4
##    row[5] = y5
##                
##    if CHECKMODE == "BEST":
##        y = bestOfRow(row)
##    else:
##        y = meanOfRow(row)
##
##    y0.update(y)
##    y1.update(y)
##    y2.update(y)
##    y3.update(y)
##    y4.update(y)
##    y5.update(y)
##
##    
###------------------------------------------------------------------------------
#------------------------------------------------------------------------------
def checkIfConflict(y1, y2):
#------------------------------------------------------------------------------
#   Checks if there is a conflict between values and error ranges of y1 and y2
#------------------------------------------------------------------------------
#..............................................................................

    hasConflict = False
    if y1.val is not None and y2.val is not None:
        dif = abs(y1.val - y2.val)
        maxDif = max(abs(y1.val), abs(y2.val)) * \
                 max(pow(y1.sqerr + y2.sqerr, 0.5) * CONFIDENCE, NUMERIC_ERR)
        
        if dif - maxDif > NUMERIC_ERR:
            print "======================================================"
            print "WARNING !!!! "
            print "CCheckFunctions (checkIfConflict): contradiction found"
            print "-> stochastic error !!!"
            y1.show()
            y2.show()
            print "dif = ", dif, " maxDif = ", maxDif
            print "======================================================"
            conflict.screen(y1, y2, dif, maxDif)
            hasConflict = True

    difLimits = max(y2.valMin - y1.valMax, y1.valMin - y2.valMax)
    if difLimits > CONFLICT_ERR * max(abs(y1.valMin),abs(y1.valMax),abs(y2.valMin),abs(y2.valMax)) and difLimits > NUMERIC_ERR:
        print "======================================================"
        print "WARNING !!!! "
        print "CCheckFunctions (checkIfConflict): contradiction found"
        print "-> absolute limits error !!!"
        y1.show()
        y2.show()
        print "difLimits = ", difLimits
        print "======================================================"
        conflict.screen(y1, y2, difLimits, 0.0)
        hasConflict = True
        
    return hasConflict
        

    
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
def trackBestOf(y1, y2):
#------------------------------------------------------------------------------
#   takes the track of the best
#------------------------------------------------------------------------------

    if (y1.sqerr < y2.sqerr):
            return y1.track
    else:
            return y2.track

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
def trackBestOf3(y1, y2, y3):
#------------------------------------------------------------------------------
#   takes the track of the best
#------------------------------------------------------------------------------

    if (y1.sqerr < y2.sqerr and y1.sqerr < y3.sqerr):
            return y1.track
    elif (y2.sqerr < y3.sqerr):
            return y2.track
    else:
            return y3.track

#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
def trackBestOfRow(row, m):
#------------------------------------------------------------------------------
#   takes the track of the best
#------------------------------------------------------------------------------

    best = INFINITE
    track = []
    for i in range(m):
        if row[i].sqerr <= best:
            best = row[i].sqerr
            track = row[i].track
    return track            

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
def bestOf(y1, y2, constrain = True):
#------------------------------------------------------------------------------
#   Selects the best of three values (the one with lowest errors)
#   Does not carry out None-check !!!!
#------------------------------------------------------------------------------

#    print "BestOf: starting"
#    y1.show()
#    y2.show()

    if checkIfConflict(y1, y2):
        
        best = y1 # do not adjust if there is a conflict but use the first one

    else:

        y1.calcDev()
        y2.calcDev()

        prefer_y1 = (y2.val is None or (y1.userDefined and not y2.userDefined)) # preference for one of both independent of sqerr-comparison
        prefer_y2 = (y1.val is None or (y2.userDefined and not y1.userDefined))
        if not (prefer_y1 or prefer_y2):
            prefer_y1 = (y1.sqdev < y2.sqdev)
            
        best = y1 if prefer_y1 else y2

# adding too much tracks gave problems ... -> back to the roots
# just following the track of the best
#    best.track = []
#    best.track.extend(y1.track)
#    best.track.extend(y2.track)
            
        best.valMin = max(y1.valMin, y2.valMin)
        best.valMax = min(y1.valMax, y2.valMax)

        if constrain:
            best.constrain()

    return best   

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
def bestOf3(y1, y2, y3):
#------------------------------------------------------------------------------
#   Selects the best of three values (the one with lowest errors)
#   Does not carry out None-check !!!!
#------------------------------------------------------------------------------

    best = bestOf(y1, y2)
    best = bestOf(best, y3)
    
    return best   

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
def bestOfRow(row):
#------------------------------------------------------------------------------
#   Selects the best of three values (the one with lowest errors)
#   Does not carry out None-check !!!!
#------------------------------------------------------------------------------

##TODO: commented out V2.2.development 17.04.12 -> conflict checking already done in indiviual best-of-s
##    for i in range(len(row)):
##        if row[i].val is not None:
##            for j in range(i + 1, len(row)):
##                if row[j].val is not None:
##                    checkIfConflict(row[i], row[j])

    if len(row) > 0:
        best = CCPar(row[0].name, parType=row[0].parType)
        best.update(row[0])
    
    for i in range(1, len(row)):
        best = bestOf(best, row[i], constrain=False)

    best.constrain()
    
    return best   

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
def meanValueOf(y1, y2):
#------------------------------------------------------------------------------
#   Selects the best of three values (the one with lowest errors)
#   Does not carry out None-check !!!!
#------------------------------------------------------------------------------

    mean = CCPar("meanValueOf", parType=y1.parType)
    mean.update(bestOf(y1, y2))

#    mean.track = trackBestOf(y1,y2)
# adding too much tracks gave problems ... -> back to the roots
# just following the track of the best
    mean.track = trackBestOf(y1, y2) #assure that the best comes first
    mean.track.extend(y1.track)
    mean.track.extend(y2.track)
    mean.cleanUp()

#    sumSqErr = pow(y1.sqerr,2) + pow(y2.sqerr,2)
    if y1.userDefined and not y2.userDefined:
        mean.val = y1.val
    elif not y1.userDefined and y2.userDefined:
        mean.val = y2.val
    else:
        sumSqDev = y1.calcDev() + y2.calcDev()
        
        dif = abs(y1.val - y2.val)
        maxDif = pow(sumSqDev, 0.5) * CONFIDENCE
        
        if dif - maxDif > NUMERIC_ERR:
            print "======================================================"
            print "WARNING !!!! "
            print "CCheckFunctions (meanValueOf): contradiction found"
            y1.show()
            y2.show()
            print "======================================================"

        if y1.sqdev > 10.0 * y2.sqdev or y2.sqdev > 10.0 * y1.sqdev:
            pass
        
        elif sumSqDev > 0:
            mean.val = (y1.sqdev * y2.val + y2.sqdev * y1.val) / sumSqDev
            
        else:
            mean.val = 0.5 * (y1.val + y2.val)
        
    shift = min(abs(mean.val - y1.val), abs(mean.val - y2.val))
    y1.calcDev()
    y2.calcDev()
    mean.dev = min(y1.dev, y2.dev) + shift
    mean.sqdev = pow(mean.dev, 2.0)
    
    mean.valMin = max(y1.valMin, y2.valMin)
    mean.valMax = min(y1.valMax, y2.valMax)
    mean.calcErr()

    mean.constrain()
        
    return mean   

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
def meanValueOf3(y1, y2, y3):
#------------------------------------------------------------------------------
#   Selects the best of three values (the one with lowest errors)
#   Does not carry out None-check !!!!
#------------------------------------------------------------------------------

    checkIfConflict(y1, y2)
    checkIfConflict(y1, y3)
    checkIfConflict(y2, y3)

    mean = CCPar("meanValueOf3", parType=y1.parType, unitType = y1.unitType)
    mean.track = trackBestOf3(y1, y2, y3) #assure that best comes first
    mean.track.extend(y1.track)
    mean.track.extend(y2.track)
    mean.track.extend(y3.track)
    mean.cleanUp()

    sumSqDev = y1.calcDev() + y2.calcDev() + y3.calcDev()
    
    if (y1.sqdev == 0):
        mean = y1
        if (y2.sqdev == 0) and not isequal(y1.val, y2.val):
            mean = meanValueOf(mean, y1)

            if DEBUG == "ALL":
                print "======================================================"
                print "WARNING !!!! "
                print "CCheckFunctions (meanValueOf3): contradiction found", y1.name, y1.val, y2.name, y2.val
                y1.show()
                y2.show()
                print "======================================================"

        if (y3.sqdev == 0) and not isequal(y1.val, y3.val):
            mean = meanValueOf(mean, y3)

            if DEBUG == "ALL":
                print "======================================================"
                print "WARNING !!!! "
                print "CCheckFunctions (meanValueOf3): contradiction found", y1.name, y1.val, y3.name, y3.val
                y1.show()
                y2.show()
                print "======================================================"

    elif (y2.sqdev == 0 and y1.sqdev != 0):
        mean = y2
        if (y3.sqerr == 0) and not isequal(y2.val, y3.val):
            mean = meanValueOf(mean, y3)

            if DEBUG == "ALL":           
                print "======================================================"
                print "WARNING !!!! "
                print "CCheckFunctions (meanValueOf3): contradiction found", y2.name, y2.val, y3.name, y3.val
                y1.show()
                y2.show()
                print "======================================================"

    elif (y3.sqdev == 0 and y2.sqdev != 0 and y1.sqdev != 0):
        mean = y3
    else:
        userDefined = y1.userDefined or y2.userDefined or y3.userDefined

        p1 = 1.0 / y1.sqdev
        p2 = 1.0 / y2.sqdev
        p3 = 1.0 / y3.sqdev

        userDefined = p1.userDefined or p2.userDefined or p3.userDefined

        if userDefined:
            if not y1.userDefined: p1 = 0
            if not y1.userDefined: p2 = 0
            if not y1.userDefined: p3 = 0

        mean.val = (y1.val * p1 + y2.val * p2 + y3.val * p3) / (p1 + p2 + p3)

    shift = min(abs(mean.val - y1.val), abs(mean.val - y2.val), abs(mean.val - y3.val))
                
    y1.calcDev()
    y2.calcDev()
    y3.calcDev()
                
    mean.dev = min(y1.dev, y2.dev, y3.dev) + shift
                
    mean.sqdev = pow(mean.dev, 2.0)

    mean.valMin = max(y1.valMin, y2.valMin, y3.valMin)
    mean.valMax = min(y1.valMax, y2.valMax, y3.valMax)
    mean.calcErr()
            
    mean.constrain()

    return mean   

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
def meanOfRow(row):
    """
    Calculates the mean value of a row of CCPar values

    """

    m = len(row)
    for i in range(m):
        if row[i].val is not None:
            for j in range(i + 1, m):
                if row[j].val is not None:
                    checkIfConflict(row[i], row[j])

    mean = CCPar("meanOfRow", parType=row[0].parType, unitType = row[0].unitType)
    mean.track = trackBestOfRow(row, m)  #assure that best comes first
    for i in range(m):
        mean.track.extend(row[i].track)
    mean.cleanUp()

    if DEBUG in ["ALL", "ADJUST"]:
        print "meanOfRow__________________________________________"
        for i in range(m):
            row[i].show()  
        print "___________________________________________________"

    
    minDev = INFINITE
    userDefined = False
    for i in range(m):
        minDev = min(minDev, row[i].calcDev())
        userDefined = userDefined or row[i].userDefined
        

    if minDev == 0:
        minWeight = INFINITE
    else:
        minWeight = 0.1 / minDev
    
    sumVal = 0
    sumWeight = 0

    for i in range(m):
        if userDefined and not row[i].userDefined:  # in case of user defined data take only mean of user defined ones
            pass
            
        elif not (row[i].val == None):
            if row[i].sqdev == 0:
                weight = INFINITE
            else:
                weight = 1. / row[i].sqdev

            if weight < minWeight and not row[i].userDefined:
                weight = 0            # skip very bad estimates

                            
            sumWeight += weight
            sumVal += row[i].val * weight

        mean.valMin = max(mean.valMin, row[i].valMin)
        mean.valMax = min(mean.valMax, row[i].valMax)
    
    if sumWeight > 0:
        mean.val = sumVal / sumWeight            
        mean.userDefined = userDefined

        mean.sqdev = INFINITE
        for i in range(m):
            if row[i].val is not None:
                if not userDefined or row[i].userDefined:  # in case of user defined data take only mean of user defined ones
                    shift = mean.val - row[i].val
                    newDev = pow(row[i].sqdev, 0.5) + abs(shift)
                    mean.sqdev = min(mean.sqdev, pow(newDev, 2.0))

        mean.calcErr()

    if DEBUG in ["ALL", "ADJUST"]:
        print "TEST TEST Mean before constrain"
        mean.show()
        print "___________________________________________________"

    mean.constrain()
    
    if DEBUG in ["ALL", "ADJUST"]:
        mean.show()
        print "___________________________________________________"
            
    return mean              
