# -*- coding: utf-8 -*-
#============================================================================== 				
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	CheckBUI
#			
#------------------------------------------------------------------------------
#			
#	Short description:
#	
#	Consistency check of buildings
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 				
from GUI.status import *
from ccPar import CCPar
from constants import DEBUG
from ccheckFunctions import *
from GUI.GUITools import check
from modules.fluids import Fluid
import database.dbio as dbio

##ParList = ["BuildConstructSurface",
##           "BuildUsefulSurface",
##           "BuildEnergyDemand",
##           "BuildMaxHP",
##           "BuildMaxCP",
##           "BuildAnnualHeating",
##           "BuildAnnualAirCond",
##           "BuildTHeating",
##           "IndoorTHeating",
##           "BuildTAirCond",
##           "IndoorTAirCond",
##           "AirFlowRateC",
##           "AirFlowRateH",
##           "TInflowRecC",
##           "TInflowRecH",
##           "BuildDailyDHW",
##           "BuildHoursOccup",
##           "BuildDaysInUse"]


class CheckBUI():
    """Carry out consistency checking for heat exchanger h"""
    def __init__(self, b, checkInPlace = False):
        """Setup the check with the data of the WHEE with WHEENo == n + 1""" 

        self.checkInPlace = checkInPlace
        self.index = b
        self.dataGroup = "BUI"

        # assign a variable to all intermediate/calculated values needed
        
        self.importData(b)

        if DEBUG in ["ALL", "BASIC"]:
            self.showAll()

    def importData(self, b):
        """Import data from SQL tables (from AlternativeProposalNo = -1)
        for the WHEE with WHEENo == n + 1
        
        """

        if self.checkInPlace:
            ANo = Status.ANo
        else:
            ANo = -1

        self.BuildingNo = b + 1
        
        # assign empty CCPar to all questionnaire parameters
        self.BuildConstructSurface = CCPar("BuildConstructSurface")
        self.BuildUsefulSurface = CCPar("BuildUsefulSurface")
        self.BuildEnergyDemand = CCPar("BuildEnergyDemand")

        self.BuildMaxHP = CCPar("BuildMaxHP")
        self.BuildMaxCP = CCPar("BuildMaxCP")

        self.BuildAnnualHeating = CCPar("BuildAnnualHeating")
        self.BuildAnnualAirCond = CCPar("BuildAnnualAirCond")

        self.BuildTHeating = CCPar("BuildTHeating")
        self.IndoorTHeating = CCPar("IndoorTHeating")

        self.BuildTAirCond = CCPar("BuildTAirCond")
        self.IndoorTAirCond = CCPar("IndoorTAirCond")

##        self.AirHandlingUnit = IntCol(length=45, default=None)
        self.AirFlowRateC = CCPar("AirFlowRateC")
        self.AirFlowRateH = CCPar("AirFlowRateH")

        self.TInflowRecC = CCPar("TInflowRecC")
        self.TInflowRecH = CCPar("TInflowRecH")

        self.BuildDailyDHW = CCPar("BuildDailyDHW")
        self.BuildHoursOccup = CCPar("BuildHoursOccup")
        self.BuildDaysInUse = CCPar("BuildDaysInUse")
##        BuildHolidaysPeriodStart_1 = CCPar()
##        BuildHolidaysPeriodStart_2 = DateCol(default=None)
##        BuildHolidaysPeriodStart_3 = DateCol(default=None)
##        BuildHolidaysPeriodStop_1 = DateCol(default=None)
##        BuildHolidaysPeriodStop_2 = DateCol(default=None)
##        BuildHolidaysPeriodStop_3 = DateCol(default=None)
##        BuildHeatingPeriodStart = DateCol(default=None)
##        BuildHeatingPeriodStop = DateCol(default=None)
##        BuildAirCondPeriodStart = DateCol(default=None)
##        BuildAirCondPeriodStop = DateCol(default=None)



        
        self.allParameters = (self.BuildConstructSurface,
                              self.BuildUsefulSurface,
                              self.BuildEnergyDemand,
                              self.BuildMaxHP,
                              self.BuildMaxCP,
                              self.BuildAnnualHeating,
                              self.BuildAnnualAirCond,
                              self.BuildTHeating,
                              self.IndoorTHeating,
                              self.BuildTAirCond,
                              self.IndoorTAirCond,
                              self.AirFlowRateC,
                              self.AirFlowRateH,
                              self.TInflowRecC,
                              self.TInflowRecH,
                              self.BuildDailyDHW,
                              self.BuildHoursOccup,
                              self.BuildDaysInUse)

        # reading data from table "qbuildings"
        building = Status.prj.getBuilding(PId=Status.PId, ANo=ANo, no=self.BuildingNo)
        if building is not None:

            for par in self.allParameters:
                parName = par.name
                par.setValue(getattr(building, parName))

    def exportData(self):
        """Store corrected data in SQL (under AlternativeProposalNo = 0)"""
        if self.checkInPlace:
            ANo = max(Status.ANo, 0)
        else:
            ANo = 0
            
        # reading data from table "qbuildings"
        building = Status.prj.getBuilding(PId=Status.PId, ANo=ANo, no=self.BuildingNo)
        if building is not None:

            tmp = {}
            for par in self.allParameters:
                parName = par.name
                tmp.update({parName: dbio.check(par.val)})
                 
            building.set(**tmp)
            
            dbio.commit()
                
    def showAll(self):
        """Print all values for debugging"""
        print "====================="
        for param in self.allParameters:
            param.show()
        print "====================="
        
    def screen(self):  
        """Screen all variables in the block"""
        for param in self.allParameters:
            param.screen()

    def check(self):
        """main function, carry out the check of the block"""
        pass
##        if DEBUG in ["ALL"]:
##            print "-------------------------------------------------"
##            print " Checking heat exchangers "
##            print "-------------------------------------------------"
##
##        for n in range(1):
##
##            cycle.initCheckBalance()
##
##            # Step 1: Call all calculation routines in a given sequence
##            if DEBUG in ["ALL"]:
##                print "-------------------------------------------------"
##                print "Cycle %s" % n
##                print "-------------------------------------------------"
##                print "Step 1: calculating from left to right (CALC)"
##
##            self.HPerYear.push(calcProd("HPerYearWHEE", self.NDaysWHEE, self.HPerDayWHEE))
##            self.QWHEEYear.push(calcProd("QWHEEYear", self.QWHEE, self.HPerYear))
##            self.HPerDayWHEE.push(calcProd("HPerDayWHEE", self.NBatchWHEE, self.HBatchWHEE))
##
##            if self.fluid is not None:
##                self.dotQWHEEMax.push(calcFlow_stack("dotQWHEEMax", 
##                                                            self.fluid.rho*self.fluid.cp, 
##                                                            self.WHEEFlow,
##                                                            self.WHEETOutlet, 
##                                                            self.TEnv,
##                                                            self.DTWHEEMax,
##                                                     unitType = "POWER"))
##                self.QWHEE.push(calcFlow_stack("QWHEE", 
##                                                            self.fluid.rho*self.fluid.cp, 
##                                                            self.WHEEFlow,
##                                                            self.WHEETOutlet, 
##                                                            self.WHEETReturn,
##                                                            self.DTWHEE,
##                                                     unitType = "POWER"))
##                
##                self.diffQWHEE.push(calcDiff("diffQWHEE",self.dotQWHEEMax, self.QWHEE))
##
##            pass     #here calcSum(self.QWHEE,sel.QWHEERec,self.QWHEEAmb)
##
##            # Step 2: Cross check the variables
##            if DEBUG in ["ALL"]:
##                self.showAll()
##                print "Step 2: cross checking"
##
##            self.ccheckAll()
##            
##            # Step 3: Adjust the variables (inverse of calculation routines)
##            if DEBUG in ["ALL"]:
##                self.showAll()
##                print "Step 3: calculating from right to left (ADJUST)"
##            
##            adjustProd_stack(self.HPerYear, self.NDaysWHEE, self.HPerDayWHEE)
##            adjustProd_stack(self.QWHEEYear, self.QWHEE, self.HPerYear)
##            adjustProd_stack(self.HPerDayWHEE, self.NBatchWHEE, self.HBatchWHEE)
##            
##            if self.fluid is not None:
##                adjustFlow_stack(self.dotQWHEEMax, self.fluid.rho*self.fluid.cp, 
##                                                            self.WHEEFlow,
##                                                            self.WHEETOutlet, 
##                                                            self.TEnv,
##                                                            self.DTWHEEMax)
##                adjustFlow_stack(self.QWHEE, self.fluid.rho*self.fluid.cp, 
##                                                            self.WHEEFlow,
##                                                            self.WHEETOutlet, 
##                                                            self.WHEETReturn,
##                                                            self.DTWHEE)
##                adjustDiff_stack(self.diffQWHEE, self.dotQWHEEMax, self.QWHEE)
##
##            # Step 4: Cross check again the variables
##            if DEBUG in ["ALL"]:
##                self.showAll()
##                print "Step 4: second cross checking"
##
##            self.ccheckAll()
##
##            if DEBUG in ["ALL"]:
##                self.showAll()
##
##        if DEBUG in ["ALL", "BASIC"]:
##            self.showAll()
##
##        cycle.checkTotalBalance()
##        return cycle.getMeanBalance()
##        
##    def ccheckAll(self):    
##        """Adjust the approximations of each WHEE parameter, and check
##        each each one for internal for consistency.
##        
##        """
##        for param in self.allParameters:
##            param.check()


if __name__ == "__main__":
    pass

