# -*- coding: utf-8 -*-
#============================================================================== 				
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	CheckHX
#			
#------------------------------------------------------------------------------
#			
#	Short description:
#	
#	Consistency check of heat exchanger data
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 				
from GUI.status import *
from constants import *
from ccheckFunctions import *
from ccPar import CCPar
import database.dbio as dbio

EPSILON = 1.e-3     # required accuracy for function "isequal"
INFINITE = 1.e99    # numerical value assigned to "infinite"


class CheckECO():
    #   Carry out consistency checking for economic parameters
    def __init__(self):
        # assign a variable to all intermediate/calculated values needed
        self.dataGroup = "ECO"
        self.importData()

        if DEBUG in ["ALL", "BASIC"]:
            self.showAll()

    def importData(self):  
        #   imports data from SQL tables (from AlternativeProposalNo = -1)
        ANo = -1
        
        # assign empty CCPar to all questionnaire parameters
        self.OMThermal = CCPar("OMThermal", priority=1)
        self.OMElectrical = CCPar("OMElectrical")

        self.OMTotalTot = CCPar("OMTotalTot")
        self.OMHCGenDistTot = CCPar("OMHCGenDistTot")
        
        self.inflationRate = CCPar("InflationRate")
        self.fuelPriceRate = CCPar("FuelPriceRate")
        self.interestExtFinancing = CCPar("InterestExtFinancing")
        self.compSpecificDiscountRate = CCPar("CompSpecificDiscountRate")
        self.amortisationTime = CCPar("AmortisationTime")
        
        # reading data from the database
        if ANo == -1:       
##            qq = Status.DB.cgeneraldata.Questionnaire_id[Status.PId].AlternativeProposalNo[ANo]
##            if len(qq) > 0:
##                q = qq[0]
            q = Status.prj.getGeneralData(PId=Status.PId, ANo=ANo)
            if q is not None:
                
                self.inflationRate.setValue(q.InflationRate, err=0, default=0.02)
                self.fuelPriceRate.setValue(q.FuelPriceRate, err=0, default=0.03)
                self.interestExtFinancing.setValue(q.InterestExtFinancing, err=0, default=0.06)
                self.compSpecificDiscountRate.setValue(q.CompSpecificDiscountRate, err=0, default=0.03)
                self.amortisationTime.setValue(q.AmortisationTime, err=0, default=20.0)

##            qq = Status.DB.questionnaire.Questionnaire_ID[Status.PId]
##            if len(qq) > 0:
##                q = qq[0]
            q = Status.prj.getQuestionnaire()
            if q is not None:

                self.OMTotalTot.setValue(q.OMTotalTot)
                self.OMHCGenDistTot.setValue(q.OMHCGenDistTot)                

                self.OMThermal.setValue(q.OMThermal)                
                self.OMElectrical.setValue(q.OMElectrical)

                if self.OMElectrical.val is None:
                    self.OMElectrical.setValue(0.0)
                
    def exportData(self):  
        # stores corrected data in SQL (under AlternativeProposalNo = 0)
        ANo = 0
        
        # writing data into SQL table 
        if ANo == 0:
            q = Status.prj.getGeneralData(PId=Status.PId, ANo=ANo)
            if q is not None:               
                q.set(OMThermal = dbio.check(self.OMThermal.val),
                      OMElectrical = dbio.check(self.OMElectrical.val),
                      InflationRate = dbio.check(self.inflationRate.val),
                      FuelPriceRate = dbio.check(self.fuelPriceRate.val),
                      InterestExtFinancing = dbio.check(self.interestExtFinancing.val),
                      CompSpecificDiscountRate = dbio.check(self.compSpecificDiscountRate.val),
                      AmortisationTime = dbio.check(self.amortisationTime.val))
                
                dbio.commit()

    def showAll(self):
        # plotting of values for debugging
        print "====================="
        self.OMThermal.show()
        self.OMElectrical.show()
        print "====================="

    def screen(self):  
        #   screens all variables in the block
        self.OMThermal.screen()
        self.OMElectrical.screen()

    def check(self):
        #   main function carrying out the check of the block
        if DEBUG in ["ALL"]:
            print "-------------------------------------------------"
            print " Checking ECONOMIC parameters "
            print "-------------------------------------------------"

        for n in range(1):

            cycle.initCheckBalance()

            if DEBUG in ["ALL"]:
                print "-------------------------------------------------"
                print "Cycle %s" % n
                print "-------------------------------------------------"
        
                # Step 1: Call all calculation routines in a given sequence
                print "Step 1: calculating from left to right (CALC)"

            pass #for the moment nothing to be calculates

            # Step 2: Cross check the variables
            if DEBUG in ["ALL"]:
                self.showAll()
                print "Step 2: cross checking"

            self.OMThermal.check()
        
            # Step 3: Adjust the variables (inverse of calculation routines)
            if DEBUG in ["ALL"]:
                self.showAll()
                print "Step 3: calculating from right to left (ADJUST)"
            
            pass    #for the moment nothing to adjust

            
            # Step 4: Cross check again the variables
            if DEBUG in ["ALL"]:
                self.showAll()
                print "Step 4: second cross checking"

#            ccheck1(self.QHX,self.QWH)  #second ccheck is superfluous, as none of the three data is adjusted

            if DEBUG in ["ALL"]:
                self.showAll()

        if DEBUG in ["ALL", "BASIC"]:
            self.showAll()

        cycle.checkTotalBalance()
        return cycle.getMeanBalance()

    def estimate(self):  
        """Estimate some of the data that is not sufficiently precise
        should be a subset of the data that is within screen
        (not necessarily ALL data has to be estimated)
        
        """
        # if nothing is specified, suppose that electricity sales and generation are 0
        # could be improved later on looking whether there's some CHP in the system or
        # not ...
        OMmin = self.OMThermal.valMin
        if OMmin > 0:
            self.OMThermal.setEstimate(OMmin)
        else:
            self.OMThermal.setEstimate(0.0, limits=(0.0, 0.0))

# limits: optional and fix absolute minimum and maximum values
# sqerr: optional input that fixes the (stochastic) relative square error
