# -*- coding: cp1252 -*-
#============================================================================== 				
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	CCheck (Consistency Check)
#			
#------------------------------------------------------------------------------
#			
#	Short description:
#	
#	Functions for consistency checking of data
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 				

EPSILON = 1.e-3     # required accuracy for function "isequal"
INFINITE = 1.e99    # numerical value assigned to "infinite"

from math import *
from ccheckFunctions import *
from ccPar import *
from modules.fluids import *

#libraries necessary for SQL access:
from GUI.status import *
import database.dbio as dbio

#------------------------------------------------------------------------------
class CheckFETfuel():
#------------------------------------------------------------------------------
#   Carries out consistency checking for fuel i
#------------------------------------------------------------------------------


    def __init__(self,i, checkInPlace = False):     #function that is called at the beginning when object is created

#        print "CheckFETfuel: __INIT__ running"

        self.checkInPlace = checkInPlace
        self.index = i-1 # Exception for fuels: i = 1 ... NFuel, whereas generic index from 0 to NFuel - 1 for symmetry with other CCheck-objects
        self.dataGroup = "Fuel"

# assign a variable to all intermediate/calculated values needed

        self.FETFuel = CCPar("FETFuel",priority=2, unitType = "ENERGY")
        
        self.importData(i)

        if DEBUG in ["ALL","BASIC"]:
            self.showAllFETfuel()
#------------------------------------------------------------------------------
    def importData(self,i):  
#------------------------------------------------------------------------------
#   imports data from SQL tables (from AlternativeProposalNo = -1)
#   i = 1 ... NFuels: number of fuel
#------------------------------------------------------------------------------

        self.FuelNo = i
#        print "CheckFETfuel: importing data"
        if self.checkInPlace:
            ANo = Status.ANo
        else:
            ANo = -1
        
#..............................................................................
# assign empty CCPar to all questionnaire parameters

        
        self.MFuelYear = CCPar("MFuelYear", unitType = "MASS") 
        self.FECFuel = CCPar("FECFuel", priority = 1, unitType = "ENERGY")
        self.FEOFuel = CCPar("FEOFuel", unitType = "ENERGY") #FEOfuel assigned only here=0: missed in the questionnaire but present in the list of the parameters

        self.FuelTariff = CCPar("FuelTariff", priority = 1, unitType = "ENERGYTARIFF")
        self.FuelCostYear = CCPar("FuelCostYear",priority = 1, unitType = "PRICE")

#..............................................................................
# reading data from table "qfuel"
#        print "CheckFETFuel: trying to read database"
        try:
##            qfuelTable = Status.DB.qfuel.Questionnaire_id[Status.PId].AlternativeProposalNo[ANo].FuelNo[i]
###            print "CheckFETFuel: number of fuels = ",len(qfuelTable)
##            
##            if len(qfuelTable) > 0:
##                qfuel = qfuelTable[0]

            qfuel = Status.prj.getFuel(PId=Status.PId, ANo=ANo, no = i)
            if qfuel is not None:

                fuel_number = qfuel.DBFuel_id   #IMPORT from the fuelDB
                fet_fuel = Fuel(fuel_number)
                self.FuelLCV = fet_fuel.LCV
                #self.FuelLCV = 10 # kWh/unit # IMPORT Constant from the FuelDB

                self.MFuelYear.setValue(qfuel.MFuelYear,err=0.0) #if specified, take as exact
                self.FECFuel.setValue(qfuel.FECFuel)
                
                self.FEOFuel.setValue(0.0,err=0)
                logDebug("checkFuel (importData): FEO = 0.0 assumed")

                self.FuelTariff.setValue(qfuel.FuelTariff)
                self.FuelCostYear.setValue(qfuel.FuelCostYear)
            else:
                self.FuelLCV = 0.0
                self.MFuelYear.setValue(qfuel.MFuelYear,err=0.0)
        except:
            self.FuelLCV = 0.0
            print "CheckFETfuel(importData): error reading data from qfuel"
            pass
                               

#------------------------------------------------------------------------------
    def exportData(self):  
#------------------------------------------------------------------------------
#   stores corrected data in SQL (under AlternativeProposalNo = 0)
#------------------------------------------------------------------------------

        if self.checkInPlace:
            ANo = max(Status.ANo, 0) # if ANo == -1, write to ANo = 0 !!!
        else:
            ANo = 0
        
#..............................................................................
# writing data into table " qfuel"
        if ANo == 0:
##            qfuelTable = Status.DB.qfuel.Questionnaire_id[Status.PId].AlternativeProposalNo[ANo].FuelNo[self.FuelNo]
##            if len(qfuelTable) > 0:
##                qfuel = qfuelTable[0]

            qfuel = Status.prj.getFuel(PId=Status.PId, ANo=ANo, no = self.FuelNo)
            if qfuel is not None:

                qfuel.set(MFuelYear = dbio.check(self.MFuelYear.val),
                          FEOFuel = dbio.check(self.FEOFuel.val),
                          FECFuel = dbio.check(self.FECFuel.val),
                          FETFuel = dbio.check(self.FETFuel.val),
                          FuelTariff = dbio.check(self.FuelTariff.val),
                          FuelCostYear = dbio.check(self.FuelCostYear.val))
                
                dbio.commit()
                
    def showAllFETfuel(self):
        
        print "====================="
      
        self.MFuelYear.show()
        self.FECFuel.show()
        self.FEOFuel.show()
        self.FETFuel.show()
        self.FuelTariff.show()
        self.FuelCostYear.show()
                
        print "====================="
#-----------------------------------------------------------------------------
    def screen(self):  
#------------------------------------------------------------------------------
#   screens all variables in the block
#------------------------------------------------------------------------------

#        print "CheckFETFuel (screen): screening"
        self.MFuelYear.screen()
        self.FECFuel.screen()
        self.FEOFuel.screen()
        self.FETFuel.screen()
        self.FuelCostYear.screen()
        self.FuelTariff.screen()

#------------------------------------------------------------------------------
    def check(self):     #function that is called at the beginning when object is created
#------------------------------------------------------------------------------
#   main function carrying out the check of the block
#------------------------------------------------------------------------------

#        print "CheckFETfuel: starting check with"
#        self.FECFuel.show()
#        self.MFuelYear.show()
        
        if DEBUG in ["ALL"]:
            print "-------------------------------------------------"
            print " FETfuel checking"
            print "-------------------------------------------------"

        
        for n in range(1):
            
            cycle.initCheckBalance()

            if DEBUG in ["ALL"]:
                print "-------------------------------------------------"
                print "Ciclo %s"%n
                print "-------------------------------------------------"
        
# Step 1: Call all calculation routines in a given sequence
            if DEBUG in ["ALL"]:
                print "Step 1: calculating from left to right (CALC)"
            
            self.FECFuel.push(calcK("FECFuel",self.FuelLCV,self.MFuelYear, unitType = "ENERGY"))
            self.FETFuel.push(calcDiff("FETFuel",self.FECFuel,self.FEOFuel))
            self.FuelCostYear.push(calcProd("FuelCostYear",self.FuelTariff,self.FECFuel, unitType = "PRICE"))
            
            if DEBUG in ["ALL"]:          
                self.showAllFETfuel()

# Step 2: Cross check the variables

            if DEBUG in ["ALL"]:
                print "Step 2: cross checking"

            self.checkAll()
                                    
            if DEBUG in ["ALL"]:
                self.showAllFETfuel()
                
# Step 3: Adjust the variables (inverse of calculation routines)

            if DEBUG in ["ALL"]:
                print "Step 3: calculating from right to left (ADJUST)"

            adjustProd_stack(self.FuelCostYear,self.FuelTariff,self.FECFuel)
            adjustDiff_stack(self.FETFuel,self.FECFuel,self.FEOFuel)
            adjustcalcK_stack(self.FECFuel,self.FuelLCV,self.MFuelYear)
                        
            if DEBUG in ["ALL"]:
                self.showAllFETfuel()
                

# Step 4: Second cross check the variables: added in order to provisionally solve the calcDiff/adjustDiff problem

            if DEBUG in ["ALL"]:
                print "Step 4: cross checking"

            self.checkAll()
                                    
            if DEBUG in ["ALL"]:
                self.showAllFETfuel()
                
        
    #aadido este ltimo show all. sino no se ven los ultimos dos resultados ...
        if DEBUG in ["ALL","BASIC"]:
            self.showAllFETfuel()

        cycle.checkTotalBalance()
        return cycle.getMeanBalance()


    def checkAll(self):
        self.FETFuel.check()
        self.FECFuel.check()
        self.FEOFuel.check()
        self.FuelTariff.check()
        self.FuelCostYear.check()
        self.MFuelYear.check()

#------------------------------------------------------------------------------
    def estimate(self):  
#------------------------------------------------------------------------------
#   estimates some of the data that are not sufficiently precise
#   should be a subset of the data that are within screen
#   (not necessarily ALL data have to be estimated)
#------------------------------------------------------------------------------

        self.FuelTariff.setEstimate(0.04,limits = (0.02,0.06))

#==============================================================================
if __name__ == "__main__":
    
# direct connecting to SQL database w/o GUI. for testing only
    stat = Status("testCheckProc")
##    Status.SQL = MySQLdb.connect(user="root", db="einstein")
##    Status.DB = pSQL.pSQL(Status.SQL, "einstein")
    Status.PId = 41
    Status.ANo = -1
#..............................................................................
    
    screen = CCScreen()
    
    ccFETfuel = CheckFETfuel(1)       # creates an instance of class CCheck
    ccFETfuel.check()
    ccFETfuel.exportData(1)

    ccFETfuel.screen()
    screen.show()
    
#==============================================================================

