# -*- coding: utf-8 -*-
#==============================================================================#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	ModuleCC (consistency check module)
#			
#------------------------------------------------------------------------------
#			
#	
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
import numpy as np
from parameterList import *

from auxiliary.auxiliary import *
from GUI.status import *
from globals import Globals

from modules.interfaces import *
import modules.matPanel as mP
from modules.messageLogger import *
from GUI.dialogGauge import DialogGauge

from ccheckFunctions import *
from ccTracker import estimateTracker, screen, conflict, cycle
from ccPar import CCPar, CCRow
from ccCheckers import setCheckMode

from checkMatrix import *
from checkCon import *
from checkProc import *
from checkEq import *
from checkFETfuel import *
from checkFETel import *
from checkPipe import *
from checkHX import *
from checkWHEE import *
from checkTotals import CheckTotals
from checkECO import *
from checkBUI import CheckBUI
from checkSchedules import CheckSchedules
from checkMassFlow import CheckMassFlow
from auxiliary.auxiliary import toGettextAndUnicode as _U

global DEBUG

DATAGROUPS = {"checkFETel": "Electricity",  # associates object Type of ccObject with data group name
              "checkFETfuel": "Fuel",
              "checkEq": "Equipment",
              "checkPipe": "Pipe",
              "checkProc": "Process",
              "checkHX": "HX",
              "checkWHEE": "WHEE"}

class ModuleCC(object):
#------------------------------------------------------------------------------
    def __init__(self):
#------------------------------------------------------------------------------
#   basic initialisation at the start-up of the tool
#------------------------------------------------------------------------------

        self.keys = ["CC Table", "CC Info"] # the key to the data is sent by the panel

#..............................................................................
# creates an empty space for the different check-blocks

        self.ccFET = []
        self.NFET = 0
        
        self.ccEq = []
        self.NEquipe = 0
        
        self.ccProc = []
        self.NProc = 0

        self.ccPipe = []
        self.NPipeDuct = 0

        self.ccHX = []
        self.NHX = 0

        self.ccWHEE = []
        self.NWHEE = 0

        self.screen_priority = 2

        self.parameterList = ParameterList().list
        
#------------------------------------------------------------------------------
    def reset(self):
        """ function carried out, when a change in an alternative is carried out """
        
        if Status.ANo > 0:
            self.__init__()
            
#------------------------------------------------------------------------------
    def initPanel(self):
#------------------------------------------------------------------------------
#   is called at entering the CC panel
#------------------------------------------------------------------------------

        self.updatePanel()
    
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
    def updatePanel(self):
#------------------------------------------------------------------------------
#   function that updates the information on the CC panel on the GUI
#------------------------------------------------------------------------------

#..............................................................................
# export conflict data to panel
        
        conflictReport = []

        conflictPairs = []
        n = len(conflict.conflictList)

##        print "ModuleCC (updatePanel): number of conflicts = ", n
##        
        for entry in conflict.conflictList:
            
##            print entry
##            
            pair = str(entry[2]) + "/" + str(entry[6])
                                         
            if pair not in conflictPairs:
                conflictPairs.append(pair)

                listedVarName = str(entry[2])
                nameParts = listedVarName.split('[')
                if len(nameParts) > 0:
                    varName = nameParts[0].strip()
                    if varName in self.parameterList.keys():
                        description = self.parameterList[varName].encode("utf-8")
                    else:
                        description = ""
                else:
                    description = ""

##                row0 = [str(entry[2]) + " [<>" + str(entry[6]) + "]", entry[10], description]
                mainparname = str(entry[2]).split('[')[0]  #update V2.2 -> 13.04.2012
                if entry[10] in ["UPH[-]", "USH[-]", "FET[-]"]:
                    splitters = entry[2].split('[')
                    if len(splitters) > 3:
                        mainparname = str(splitters[0]+"["+splitters[1] + "[" + splitters[2])
                    elif len(splitters) == 3:
                        mainparname = str(splitters[0]+"["+splitters[1])
                else:
                    splitters = entry[2].split('[')
                    if len(splitters) >= 3:
                        attribute = '['+ splitters[1]
                        description = description + " " + attribute
                        
                                          
                row0 = [mainparname, entry[10], description]

                origin1 = ""
                for parname in entry[3]:
                    origin1 = origin1 + str(parname) + "; "

                if entry[4] is not None:
                    dval = defaultDisplayValue(entry[4], entry[11])
##                    row1 = ["%12.2f" % entry[4],
                    row1 = [dval,
                            "+/- " + "%5.2f" % (entry[5] * 100.0) + "%",
                            origin1]
                else:
                    row1 = ["---",
                            "+/- " + "%5.2f" % (entry[5] * 100.0) + "%",
                            origin1]
                    

                origin2 = ""
                for parname in entry[7]:
                    origin2 = origin2 + str(parname) + "; "

                if entry[8] is not None:
                    dval = defaultDisplayValue(entry[8], entry[11])
##                    row2 = ["%12.2f" % entry[8],
                    row2 = [dval,
                            "+/- " + "%5.2f" % (entry[9] * 100.0) + "%",
                            origin2]
                else:
                    row2 = ["---",
                            "+/- " + "%5.2f" % (entry[9] * 100.0) + "%",
                            origin2]
                    
                       
                conflictReport.append(noneFilter(row0))
                conflictReport.append(noneFilter(row1))
                conflictReport.append(noneFilter(row2))
                                         
        data = np.array(conflictReport)
        Status.int.setGraphicsData("CC Conflict", data)  #sends the data to the GUI

#..............................................................................
# export screening data

        if n == 0:  #no conflicts 
            logDebug("ModuleCC (updatePanel): screening with priority %s" % self.screen_priority)
            
            CCList = []
            nMissingVarsOfPriority = 0
            
            for entry in screen.screenList:

                if entry[4] <= self.screen_priority:

                    if entry[4] == 1:
                        action = _("Calculations w/o this are nonsense !!!")
                    elif entry[4] == 2:
                        action = _("Value required for detail analysis !!!")
                    else:
                        action = _("not strictly necessary")

                    varName = str(entry[0])
                    parts = varName.split('[')
                    if len(parts) > 0:
                        varKey = parts[0].strip()                        
                    else:
                        varKey = "---" #just for security ...
                        
                    if varKey in self.parameterList.keys():
                        description = self.parameterList[varKey].encode("utf-8")
                    else:
                        description = ""
    #                print "ModuleCC (updatePanel): "
    #                print self.parameterList
    #                print MONTHS

                    if entry[1] is not None:
                        val = defaultDisplayValue(entry[1], entry[5])
##                        val = '%9.2f' % entry[1]
                        if entry[2] >= 1.0:
                            err = '>100.00%'
                        else:
                            err = '%5.2f' % (100.0 * pow(entry[2], 0.5)) + "%"
                    else:
                        val = '---'
                        err = '---'

                    highlight = (entry[4] == 1)
                    try:
                        objectGroupParts = entry[6].split('[')
                        objectGroup = objectGroupParts[0]
                    except:
                        objectGroup = entry[6]
                        
                    row = [entry[0] + "[" + entry[3] + "]",
                           description,
                           val,
                           err,
                           action,
                           highlight,
                           objectGroup]
                    
                    CCList.append(noneFilter(row))
                    nMissingVarsOfPriority += 1

            if nMissingVarsOfPriority == 0:
                if screen.nScreened == 0:
                    CCList.append(["", "", "", "", "", False, ""])
                else:
                    CCList.append(["", _("CONGRATULATION: data set is sufficiently complete!"), "", "", "", True, ""])

        else:  # in case of conflicts conflicts 
            nMissingVarsOfPriority = "---"
            CCList = [["", _("ERROR: Your data set is not consistent."), "", "", "", False, ""]]
            
##        data = np.array(CCList)
        Status.int.setGraphicsData(self.keys[0], CCList)  #sends the data to the GUI

        nScreenedVars = screen.nScreened
        CCList2 = [nScreenedVars, "---", nMissingVarsOfPriority, self.screen_priority]

        Status.int.setGraphicsData(self.keys[1], CCList2)  #sends the data to the GUI


#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
    def setPriorityLevel(self, level):
#------------------------------------------------------------------------------
#   is called at entering the CC panel
#------------------------------------------------------------------------------
        logDebug("ModuleCC: setting priority level to %s" % level)
        self.screen_priority = level
        self.updatePanel()
    
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
    def getQuestionnaireData(self):
#------------------------------------------------------------------------------
#   function that gets the data from the questionnaire
#------------------------------------------------------------------------------

#..............................................................................
# creates an empty space for the different check-blocks

        self.ccFET = []
        self.ccEq = []
        self.ccPipe = []
        self.ccHX = []
        self.ccWHEE = []
        self.ccBUI = []

        logDebug("ModuleCC (getQuestionnaireData): running in new general mode (import connections from SQL)")

        Status.int.systemModel.build() # build-up connections

#..............................................................................
# import data on fuel and electricity consumption (FET)

        self.NFET = Status.NFET
        NI = self.NFET - 1
        
        self.FETFuel_i = CCRow("FETi", NI, unitType = "ENERGY")
        
        self.FECi = CCRow("FETi", NI, unitType = "ENERGY")
        self.ElGenera = CCPar("ElGenera", unitType = "ENERGY")

        self.ccFETel = CheckFETel()
        
        for i in range(NI):      
            self.ccFET.append(CheckFETfuel(i + 1))

#..............................................................................
# import data on existing equipment 

        NJ = Status.int.systemModel.NJ
        self._getEquipeData()
#..............................................................................
# import data on existing processes

        # calculate process schedules
##        if Status.dataPeriods:
##            print "ModuleCC.getQuestionnaireData: calculating process schedules"
        Status.schedules.calculatePlantSchedule()
        Status.schedules.calculateProcessSchedules()

        self._getProcData(checkInPlace=False)
    
#..............................................................................
# import data on existing pipeducts

        self.NPipeDuct = Status.NPipeDuct
        NM = self.NPipeDuct
        self.UPHProcm = CCRow("UPHProcm", NM, unitType = "ENERGY")     #note: this is a ROW of USH values, and not identical with the USHj variable within checkEq !!!
        
        self.USHm = CCRow("USHm", NM, unitType = "ENERGY") 

        self.QHXPipe = CCRow("QHXPipe", NM, unitType = "ENERGY")        # incoming waste heat from heat recovery
        self.QWHPipe = CCRow("QWHPipe", NM, unitType = "ENERGY")        # outgoing waste heat to be recovered

        for m in range(NM):
            self.ccPipe.append(CheckPipe(m))  # añade un objeto checkProc con todas las variables necesarias a la listac

#..............................................................................
# import data on existing heat exchangers

        self.NHX = Status.NHX
        NH = self.NHX
        self.QWH = CCRow("QWH", NH, unitType = "ENERGY")     
        self.QHX = CCRow("QHX", NH, unitType = "ENERGY")
        self.OMh = CCRow("OMh", NH, unitType = "PRICE")

        for h in range(NH):
            self.ccHX.append(CheckHX(h))  # añade un objeto checkHX con todas las variables necesarias a la listac

            self.OMh[h] = self.ccHX[h].OpCostTot

#..............................................................................
# import data on existing WHEEs

        self.NWHEE = Status.NWHEE
        NN = self.NWHEE
        self.QWHEE = CCRow("QWHEE", NN, unitType = "ENERGY")     

        for n in range(NN):
            self.ccWHEE.append(CheckWHEE(n))  # añade un objeto checkWHEE con todas las variables necesarias a la listac

#..............................................................................
# import data on link of fuels and equipment

        self.FETFuelMatrix = CheckMatrix("FETFuel", self.FETFuel_i, self.FETFuel_j, Status.FETFuelLink, unitType = "ENERGY")

        self.FETel_cMatrix = CheckMatrix("FETel_c", [self.FETel_c], self.FETel_c_j, Status.FETelLink, unitType = "ENERGY")

        self.ElGenMatrix = CheckMatrix("ElGen", [self.ElGen], self.ElGen_j, Status.FETelLink, unitType = "ENERGY")

#..............................................................................
# import data on link of equipment and pipes

        self.USHMatrix = CheckMatrix("USH", self.USHj, self.USHm, Status.int.USHLink, unitType = "ENERGY")
        
#..............................................................................
# import data on link of  pipes and processes
        NK = self.NThProc
        self.UPHCLink = np.empty((2*NK+NJ, NM))
        for k in range(NK):
            for m in range(NM):
                self.UPHCLink[k][m] = 1 if (m, k) in Status.int.systemModel.UPHLink else 0
                self.UPHCLink[k+NK][m] = 1 if (m, k) in Status.int.systemModel.UPCReqLink else 0
        for j in range(NJ):
            for m in range(NM):
                self.UPHCLink[2*NK+j][m] = 1 if (m, j) in Status.int.systemModel.FEThLink else 0
            
        logTrack("ModuleCC: UPHCLink %s" % (self.UPHCLink, ))
        
        self.UPHCMatrix = CheckMatrix("UPHProc", self.UPHProcm, self.UPHCProck, self.UPHCLink, unitType = "ENERGY")

#..............................................................................
# import data on link of heat exchangers inlet / outlet with equipments, pipes and processes

        self.QWHEqCon = CheckCon("QWHEq-Con", self.QWH, self.QWHEq, Status.QWHEqLink, ambient=True, unitType = "ENERGY")
        self.QWHPipeCon = CheckCon("QWHPipe-Con", self.QWH, self.QWHPipe, Status.QWHPipeLink, ambient=True, unitType = "ENERGY")
        self.QWHProcCon = CheckCon("QWHProc-Con", self.QWH, self.QWHProcUsed, Status.QWHProcLink, unitType = "ENERGY")
        self.QWHEECon = CheckCon("QWHEE-Con", self.QWH, self.QWHEE, Status.QWHEELink, ambient=True, unitType = "ENERGY")

        self.QHXEqCon = CheckCon("QHXEq-Con", self.QHX, self.QHXEq, Status.QHXEqLink, unitType = "ENERGY")
        self.QHXPipeCon = CheckCon("QHXPipe-Con", self.QHX, self.QHXPipe, Status.QHXPipeLink, unitType = "ENERGY")
        self.QHXProcCon = CheckCon("QHXProc-Con", self.QHX, self.QHXProc, Status.QHXProcLink, unitType = "ENERGY")

#..............................................................................
# import data on existing totals

        self.UPHk = CCRow("UPHk", NK, unitType = "ENERGY")
        self.UPHwReqk = CCRow("UPHwReqk",NK, unitType = "ENERGY")
        self.totals = CheckTotals(self.FECi, self.FETFuel_i, self.USHj, self.UPHk, self.UPHwReqk,self.WaterConsum_j, self.OMh, self.OMj)

        self.ccECO = CheckECO()

#..............................................................................
# import building data
        for b, bui in enumerate(Status.int.systemModel.bui):
            self.ccBUI.append(CheckBUI(b))

#..............................................................................
# mass flow checking
        self.ccMassFlow = CheckMassFlow()

#..............................................................................
# initialise schedule checker

        self.sc = CheckSchedules(self)

    def _getProcData(self, checkInPlace=False):
        """import data on existing processes
        
        :param checkInPlace: If false take the data from ANo -1 otherwise from
        the current alternative proposal.
        
        """
        self.ccProc= []
        self.NThProc = Status.NThProc
        NK = self.NThProc
        self.NEquipe = Status.NEquipe
        NJ = self.NEquipe

        self.UPHCProck = CCRow("UPHCProck", 2*NK+NJ, unitType = "ENERGY")     #first NK values: UPHProc, second NK values: UPCProc
        
        self.QHXProc = CCRow("QHXProc", NK, unitType = "ENERGY")      # incoming waste heat from heat recovery
        self.QWHProcUsed = CCRow("QWHProcUsed", NK, unitType = "ENERGY")    # outgoing waste heat to be recovered

        for k in xrange(NK):
            proc = Status.int.systemModel.processObject[k]
            self.ccProc.append(CheckProc(k, proc=proc, checkInPlace=checkInPlace))  # añade un objeto checkProc con todas las variables necesarias a la listac

    def _getEquipeData(self, checkInPlace=False):
        """import data on existing equipment
        
        :param checkInPlace: If false take the data from ANo -1 otherwise from
        the current alternative proposal.
        
        """
        self.NEquipe = Status.NEquipe
        NJ = self.NEquipe
        self.USHj = CCRow("USHj", NJ, unitType = "ENERGY")     #note: this is a ROW of USH values, and not identical with the USHj variable within checkEq !!!

        self.FETj = CCRow("FETj", NJ, unitType = "ENERGY")     #creates the space for intermediate storge towards matrix
        self.FETFuel_j = CCRow("FETFuel_j", NJ, unitType = "ENERGY")     #creates the space for intermediate storge towards matrix

        self.FETel_c_j = CCRow("FETel_c_j", NJ, unitType = "ENERGY")     #creates the space for intermediate storge towards matrix
        self.FETel_c = CCPar("FETel_c", unitType = "ENERGY")     #creates the space for intermediate storge towards matrix
        self.ElGen_j = CCRow("ElGen_j", NJ, unitType = "ENERGY")
        self.ElGen = CCPar("ElGen", unitType = "ENERGY")     #creates the space for intermediate storge towards matrix
        self.WaterConsum_j = CCRow("WaterConsum_j",NJ)

        self.QHXEq = CCRow("QHXEq", NJ, unitType = "ENERGY")      # incoming waste heat from heat recovery
        self.QWHEq = CCRow("QWHEq", NJ, unitType = "ENERGY")    # outgoing waste heat to be recovered

        self.ElGen_j = CCRow("ElGen_j", NJ, unitType = "ENERGY")

        self.OMj = CCRow("OMj", NJ, unitType = "PRICE")
        
        for j in range(NJ):
            self.ccEq.append(CheckEq(j, checkInPlace=checkInPlace))     # añade un objeto checkEq con todas las variables necesarias a la lista

            self.OMj[j] = self.ccEq[j].OpCostTot


#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
    def basicCheck(self, matrixCheck=True, continueCheck=False, estimate=False):
        """Run the first basic Check
        (basic = still without estimation procedures for missing data)
        
        :param matrixCheck:
        :param continueCheck: if True continue a check after data estimate, 
            without reloading the data from questionnaire ...
        :param estimate:
        
        """
#------------------------------------------------------------------------------
#   runs the first basic Check
#   (basic = still without estimation procedures for missing data)
#   continueCheck: continues a check after data estimate, without reloading
#   data from questionnaire ...
#------------------------------------------------------------------------------

        global CHECKMODE

        logTrack("ModuleCC (basicCheck): estimate = %s;continue = %s" % \
                 (estimate, continueCheck))
#        print "ModuleCC (basicCheck): estimate = %s;continue = %s"%\
#                 (estimate,continueCheck)
        
        if DEBUG in ["ALL", "BASIC"]:
            logDebug("====================================================")
            logDebug("ModuleCC: getting Test data")
            logDebug("====================================================")
        
        if continueCheck == False and estimate == False:
            self.getQuestionnaireData()
            estimateTracker.reset()

        dlg = DialogGauge(Status.main, _("consistency check"), _("calculating"), plot = True)
        gaugeData = [[_U("number of iterations")],
                     [_U("average error") + U(" (log10)")],
                     [_U("maximum error") + U(" (log10)")]]

#..............................................................................
# reset the counters of conflicts before starting

        conflict.reset()
        
#..............................................................................
        
        NCycles = 200        #maximum number of cycles
        NBestCycles = 10     #run the first cycles in CHECKMODE = "BEST", the
                            #remaining ones in "MEAN"

        balanceCtrl = 0.1   #initial setting: lagged value of (rel.) imbalance
        balanceMaxCtrl = 1.0
        diffBCctr = 0       #counts the number of consecutive cycles in which the balance does not improve
            
        if DEBUG != "OFF":
            if estimate:
                print "CCC.ESTIMATE"
            else:
                print "CCC.BASIC"

            if DEBUGCCUNTIL > 0:
                NCycles = min(NCycles, DEBUGCCUNTIL)
                NCycles = max(NCycles, NBestCycles+1)
                        
        for ncycle in range(NCycles):

            if ncycle < NBestCycles:
                setCheckMode("BEST")
            else:
                setCheckMode("MEAN")

            cycle.initTotalBalance()

            if estimate == True:
                self.dataEstimate()

            if DEBUG in ["ALL", "BASIC", "MAIN"]:
                print "===================================================="
                print "ModuleCC: starting cycle no. %s" % (ncycle + 1)
                print "===================================================="

#..............................................................................
# Step 1: do an independent checking of all the blocks as initialisation
#         (saves computing time for the start-up of the matrix-algorithm)

#..............................................................................
# check of fuel and electricity consumption (FEC)

            if DEBUG in ["ALL", "BASIC"]:
                print "===================================================="
                print "ModuleCC: checking fuels (FET)"
                print "===================================================="

            NI = self.NFET - 1
            
            if DEBUG in ["ALL"]:
                print "checking electricity consumption"

            conflict.setDataGroup("Electricity", "-")
            
            self.ccFETel.check()
            
            self.FETel_c.update(self.ccFETel.FETel_c)

            self.ElGenera.update(self.ccFETel.ElectricityGen)

            for i in range(NI):       #then check all the Nfuels = NI-1 fuels

                if DEBUG in ["ALL"]:
                    print "checking fuel no. %s" % (i + 1)

                conflict.setDataGroup("Fuel", i+1)
                
                self.ccFET[i].check()
                self.FETFuel_i[i].update(self.ccFET[i].FETFuel)

                self.FECi[i].update(self.ccFET[i].FECFuel)
            
#..............................................................................
# check of equipment

            if DEBUG in ["ALL", "BASIC"]:
                print "===================================================="
                print "ModuleCC: checking equipment (USH)"
                print "===================================================="

            NJ = self.NEquipe
            NK = self.NThProc

            for j in range(NJ):
                if DEBUG in ["ALL"]:
                    print "checking equipment no. %s" % j
                conflict.setDataGroup("Equipment", j + 1)

                self.ccEq[j].check()               # ejecuta la funcion check para equipo j

                self.USHj[j].update(self.ccEq[j].USHj)      #obtain results 

                self.FETj[j].update(self.ccEq[j].FETj)
                
                self.FETFuel_j[j].update(self.ccEq[j].FETFuel_j)
                
                self.FETel_c_j[j].update(self.ccEq[j].FETel_c_j)
                    
                self.ElGen_j[j].update(self.ccEq[j].ElGen_j)
                self.WaterConsum_j[j].update(self.ccEq[j].WaterConsum)
                
                self.QHXEq[j].update(self.ccEq[j].QHXEq)
                self.QWHEq[j].update(self.ccEq[j].QWHEq)

                self.UPHCProck[2*NK+j].update(self.ccEq[j].FETHeat_j)

#..............................................................................
# check of pipes

            if DEBUG in ["ALL", "MAIN", "BASIC"]:
                print "===================================================="
                print "ModuleCC: checking Pipes (Pipe)"
                print "===================================================="

            NM = self.NPipeDuct

            for m in range(NM):

                conflict.setDataGroup("Pipe/Duct", m + 1)

                if DEBUG in ["ALL"]:
                    print "checking pipe no. %s" % m
                self.ccPipe[m].check()               # ejecuta la funcion check para pipe m

                self.USHm[m].update(self.ccPipe[m].USHm)      #obtain results 
                self.UPHProcm[m].update(self.ccPipe[m].UPHProcm)
                self.QHXPipe[m].update(self.ccPipe[m].QHXPipe) 
                self.QWHPipe[m].update(self.ccPipe[m].QWHPipe) # if necessary change in QWHPipeRec

#..............................................................................
# check of thermal processes

            if DEBUG in ["ALL", "MAIN", "BASIC"]:
                print "===================================================="
                print "ModuleCC: checking processes (UPH)"
                print "===================================================="

            NK = self.NThProc

            for k in range(self.NThProc):
                if DEBUG in ["ALL"]:
                    print "checking process no. %s" % k
                    
                conflict.setDataGroup("Process", k + 1)

                self.ccProc[k].check()             # ejecuta la funcion check para proceso k

                self.UPHCProck[k].update(self.ccProc[k].UPHProc)

                self.UPHCProck[k+NK].update(self.ccProc[k].UPCProc)
                self.UPHk[k].update(self.ccProc[k].UPH)      #obtain results
                self.UPHwReqk[k].update(self.ccProc[k].UPHwReqOfProcess)
                
                self.QHXProc[k].update(self.ccProc[k].QHXProc)
                self.QWHProcUsed[k].update(self.ccProc[k].QWHProcUsed)

            # check mass flows between processes
            self.ccMassFlow.check()

#..............................................................................
# check of heat exchangers

            if DEBUG in ["ALL", "MAIN", "BASIC"]:
                print "===================================================="
                print "ModuleCC: checking heat exchangers (QHX)"
                print "===================================================="

            NH = self.NHX
            
            for h in range(self.NHX):
                if DEBUG in ["ALL"]:
                    print ("checking HX no. %s" % h)
                    
                conflict.setDataGroup("HX", h + 1)

                self.ccHX[h].check()             # ejecuta la funcion check para HX no. h

                self.QHX[h].update(self.ccHX[h].QHX)      #obtain results 
                self.QWH[h].update(self.ccHX[h].QWH)      #obtain results 

#..............................................................................
# check of whees

            if DEBUG in ["ALL", "MAIN", "BASIC"]:
                print "===================================================="
                print "ModuleCC: checking WHEEs (QWHEE)"
                print "===================================================="

            NN = self.NWHEE

            for n in range(self.NWHEE):
                if DEBUG in ["ALL"]:
                    print "checking WHEE no. %s" % n
                conflict.setDataGroup("WHEE", n + 1)

                self.ccWHEE[n].check()             # ejecuta la funcion check para proceso k

                self.QWHEE[n].update(self.ccWHEE[n].QWHEEYear)      #obtain results 

#..............................................................................
# check of totals

            if DEBUG in ["ALL", "MAIN", "BASIC"]:
                print "===================================================="
                print "ModuleCC: checking Totals"
                print "===================================================="

            conflict.setDataGroup("Totals", 0)

            self.totals.check()             # ejecuta la funcion check para proceso k

            conflict.setDataGroup("ECO", 0)

            self.ccECO.OMThermal.push(self.totals.OMTotal)
            
            self.ccECO.check()
                        
            self.totals.OMTotal.update(self.ccECO.OMThermal)
#..............................................................................
# Before going to matrix check, assure that there are no data conflicts

            if conflict.nConflicts > 0: break

#..............................................................................
# Step 2: now check the link of all the blocks via the matrix checking function

            if matrixCheck == True:
#..............................................................................
# link of fuels and equipment

                if DEBUG in ["ALL", "MAIN", "BASIC"]:
                    print "===================================================="
                    print "ModuleCC: checking FET matrix"
                    print "===================================================="

                conflict.setDataGroup("FET", "-")

#                self.FETMatrix.check()
                self.FETFuelMatrix.check()

                self.FETel_cMatrix.check()
                    
                self.ElGenMatrix.check()

                self.ccFETel.FETel_c.update(self.FETel_c)
                
                self.ccFETel.ElectricityGen.update(self.ElGen)
                for i in range(NI):
                    self.ccFET[i].FETFuel.update(self.FETFuel_i[i])

                for j in range(NJ):
#                    self.ccEq[j].FETj.update(self.FETj[j])
                    self.ccEq[j].FETFuel_j.update(self.FETFuel_j[j])
                    self.ccEq[j].FETel_c_j.update(self.FETel_c_j[j])
                    self.ccEq[j].ElGen_j.update(self.ElGen_j[j])

#..............................................................................
# If any matrix conflict appears, break immediately.

##                if conflict.nConflicts > 0: break
        
#..............................................................................
# link of equipment and pipes

                if DEBUG in ["ALL", "MAIN", "BASIC"]:
                    print "===================================================="
                    print "ModuleCC: checking USH matrix"
                    print "===================================================="

                conflict.setDataGroup("USH", "-")

                self.USHMatrix.check()

                for j in range(NJ):
                    self.ccEq[j].USHj.update(self.USHj[j])

                for m in range(NM):
                    self.ccPipe[m].USHm.update(self.USHm[m])

#..............................................................................
# If any matrix conflict appears, break immediately.

##                if conflict.nConflicts > 0: break
        
#..............................................................................
# link of pipes and processes

                if DEBUG in ["ALL", "MAIN", "BASIC"]:
                    print "===================================================="
                    print "ModuleCC: checking UPH matrix"
                    print "===================================================="

                conflict.setDataGroup("UPHproc", "-")

                self.UPHCMatrix.check()

                for m in range(NM):
                    self.ccPipe[m].UPHProcm.update(self.UPHProcm[m])

                for k in range(NK):
                    self.ccProc[k].UPHProc.update(self.UPHCProck[k])                        
                    self.ccProc[k].UPCProc.update(self.UPHCProck[k+NK])

                for j in range(NJ):
                    self.ccEq[j].FETHeat_j.update(self.UPHCProck[2*NK+j])

#..............................................................................
# If any matrix conflict appears, break immediately.

##                if conflict.nConflicts > 0: break
        
#..............................................................................
# Check operating hours of pipes

                for m in range(NM):
                    self.ccPipe[m].constrainOperatingHours()

#..............................................................................
# link of heat exchanger with the rest

                if DEBUG in ["ALL", "MAIN", "BASIC"]:
                    print "===================================================="
                    print "ModuleCC: checking Heat exchanger connections"
                    print "===================================================="

                conflict.setDataGroup("QWHEq", "-")
                self.QWHEqCon.check()

                conflict.setDataGroup("QWHPipe", "-")
                self.QWHPipeCon.check()

                conflict.setDataGroup("QWHProc", "-")
                self.QWHProcCon.check()

                conflict.setDataGroup("QWHEE", "-")
                self.QWHEECon.check()

                conflict.setDataGroup("QHXEq", "-")
                self.QHXEqCon.check()

                conflict.setDataGroup("QHXPipe", "-")
                self.QHXPipeCon.check()

                conflict.setDataGroup("QHXProc", "-")
                self.QHXProcCon.check()

                for j in range(NJ):
                  
                    self.ccEq[j].QHXEq.update(self.QHXEq[j])
                    self.ccEq[j].QWHEq.update(self.QWHEq[j])

                for m in range(NM):
                    self.ccPipe[m].QHXPipe.update(self.QHXPipe[m])
                    self.ccPipe[m].QWHPipe.update(self.QWHPipe[m]) #to be changed in QWHPipeRec if necessary

                for k in range(NK):
                    
                    self.ccProc[k].QHXProc.update(self.QHXProc[k])
                    self.ccProc[k].QWHProcUsed.update(self.QWHProcUsed[k])

                for n in range(NN):
                    self.ccWHEE[n].QWHEEYear.update(self.QWHEE[n])

                for h in range(NH):
                    self.ccHX[h].QWH.update(self.QWH[h])
                    self.ccHX[h].QHX.update(self.QHX[h])
                    HPerYearEstimates = [self.ccProc[k - 1].HPerYearProc for
                                         k in self.ccHX[h].attachedProcessNumbers]

                    self.ccHX[h].updateHPerYearEstimate(HPerYearEstimates)
                    
#..............................................................................
# If any matrix conflict appears, break immediately.

#### TODO - temporarily deactivated for testing -> improvement of CCcheck output
##                if conflict.nConflicts > 0: break
        
#..............................................................................
# Checking of schedules

                if ncycle >= NBestCycles: self.sc.check() # do time intensive schedule checking only after the BEST-mode cycles
                
#..............................................................................
# END OF CYCLE: check convergence

            balanceCtrlOld = balanceCtrl
            balanceCtrl = balanceCtrl + 0.5 * (cycle.getMeanTotalBalance() - balanceCtrl)

            balanceMaxCtrlOld = balanceMaxCtrl
            balanceMaxCtrl = balanceMaxCtrl + 0.5 * (cycle.getMaxTotalBalance() - balanceMaxCtrl)
            
            if balanceCtrl > 1.e-3:
                balanceImprovement = balanceCtrlOld - balanceCtrl + balanceMaxCtrlOld - balanceMaxCtrl
            else: # if average error has already reached convergence, check only further improvement of maximum error
                balanceImprovement = (balanceMaxCtrlOld - balanceMaxCtrl)
            
            if balanceImprovement < 1.e-6:
                diffBCctr += 1
            else:
                diffBCctr = 0
                
            if DEBUG in ["ALL", "MAIN", "BASIC"]:
                print "GLOBAL CYCLE CONVERGENCE: %s [lagged: %s]| %s " % \
                      (cycle.getMeanTotalBalance(), balanceCtrl, \
                       cycle.getMaxTotalBalance())

            convergenceReached = (balanceCtrl <= 1.e-3 and balanceMaxCtrl <= 1.e-2)

            message = _("Remaining balance error %6.3f percent (max: %6.3f)") % (balanceCtrl * 100, balanceMaxCtrl*100) + "\n" + \
                      _("Improvement of balance = %5.3e"%balanceImprovement)
                      
            if ncycle % 10 == 0 or convergenceReached:
                logTrack(_("Consistency check: ") + message)

            if convergenceReached and ncycle > NBestCycles + 2: # at least 3 cycles in mode "MEAN" before deciding on convergence break
                logTrack("ModuleCC (basic check): convergence reached at %s cycles" % (ncycle + 1))
                break

            if diffBCctr >= 25:
                logTrack("ModuleCC (basic check): no further improvement after %s cycles" % (50))
                logWarning("Consistency check calculations did not converge. Data may not be fully balanced")
                logWarning("Consistency check: remaining balance error %6.2f percent (max: %6.2f)" % (balanceCtrl * 100, balanceMaxCtrl*100))
                logWarning("Maximum error in variable %s"%cycle.totalBlackSheep)
                break

            if ncycle == NCycles - 1 and not convergenceReached:
                logWarning("Consistency check calculations did not converge. Data may not be fully balanced")
                logWarning("Consistency check: remaining balance error %6.2f percent (max: %6.2f)" % (balanceCtrl * 100, balanceMaxCtrl*100))
                logWarning("Maximum error in variable %s"%cycle.totalBlackSheep)
            
            gaugeData[0].append(ncycle)
            gaugeData[1].append(log10(max(balanceCtrl, NUMERIC_ERR)))
            gaugeData[2].append(log10(max(balanceMaxCtrl, NUMERIC_ERR)))
                
            dlg.update(100.0 * (ncycle+1) / NCycles, message=message, data=gaugeData)
##            if dlg.checkActivation():
##                if dlg.ShowModal() == wx.ID_CANCEL:
##                    return(-1)
##                else:
##                    break


        # set schedule globals from schedule checking and check processes again
        estimateTracker.setDataGroup("Schedules", 0)
        if self.sc.calculateScheduleGlobals():
            for k in self.sc.autoFit:
                if DEBUG in ["ALL"]:
                    print "checking process no. %s" % k
                    
                conflict.setDataGroup("Process", k + 1)
                for n in range(3):
                    self.ccProc[k].check()             # ejecuta la funcion check para proceso k
        
        #..............................................................................
        # At the end of the checking, screen the modules

        self.selectMainProcesses()

        screen.reset()

        screen.setDataGroup("FETel", 0)
        self.ccFETel.screen()
        
        for i in range(NI):       #then check all the Nfuels = NI-1 fuels
            screen.setDataGroup("FET", i + 1)
            self.ccFET[i].screen()
        for j in range(NJ):       
            screen.setDataGroup("Eq.", j + 1)
            self.ccEq[j].screen()
        for k in range(NK):       
            screen.setDataGroup("Proc.", k + 1)
            self.ccProc[k].screen()
        for m in range(NM):      
            screen.setDataGroup("Pipe", m + 1)
            self.ccPipe[m].screen()
        for h in range(NH):       
            screen.setDataGroup("HX", h + 1)
            self.ccHX[h].screen()       
        for n in range(NN):       
            screen.setDataGroup("WHEE", n + 1)
            self.ccWHEE[n].screen()

        screen.setDataGroup("Totals", 0)
        self.totals.screen()

        screen.setDataGroup("ECO", 0)
        self.ccECO.screen()

        self.sc.screen()
            
        if DEBUG in ["ALL", "MAIN", "BASIC"]:
            screen.show()
            conflict.show()

#..............................................................................
# And finally export all the data

        logTrack("ModuleCC (basicCheck): exporting results to SQL")
#        print NI,NJ,NK,NM,NH,NN
        
        self.ccFETel.exportData()
        for i in range(NI):       #then check all the Nfuels = NI-1 fuels
            self.ccFET[i].exportData()
        for j in range(NJ):       
            self.ccEq[j].exportData()
        for k in range(NK):       
            self.ccProc[k].exportData()
        for m in range(NM):
#            print "ModuleCC (basicCheck): exporting pipe %s data"%(m+1)
            self.ccPipe[m].exportData()
        for h in range(NH):       
            self.ccHX[h].exportData()
        for n in range(NN):       
            self.ccWHEE[n].exportData()

        self.totals.exportData()

        self.ccMassFlow.exportData()

        self.ccECO.exportData()

        self.sc.exportData()

        for bui in self.ccBUI:
            bui.exportData()

        try:
            dlg.Destroy() # 16.11.2012 - TODO: try - except became necessary on some computers when running auto-pilot. maybe related with recent changes in dialogGauge ?
        except:
            pass

        return conflict.nConflicts

    def procCheck(self, proc):
        """Run the consistency check for the process data only.
        
        The check is made "inPlace" not only as part of the step from
        alternative Proposal -1 to 0 (present state, orginal to present state,
        checked), but also for an alternative Proposal later. Therefore, in the second case the changed
        values are written into the same alternative, the current active one.
            
        """

        ccLocal = CheckProc(None, proc=proc, checkInPlace = True)     # añade un objeto checkEq con todas las variables necesarias a la lista

        return self.localCheck(ccLocal)

    def fuelCheck(self, i):
        """Run the consistency check for the fuel data only.
        
        The check is made "inPlace" not only as part of the step from
        alternative Proposal -1 to 0 (present state, orginal to present state,
        checked), but also for an alternative Proposal later. Therefore, in the second case the changed
        values are written into the same alternative, the current active one.
            
        """

        ccLocal = CheckFETfuel(i, checkInPlace = True)     # añade un objeto checkEq con todas las variables necesarias a la lista

        return self.localCheck(ccLocal)
        
    def elCheck(self):
        """Run the consistency check for the electricity data only.
        
        The check is made "inPlace" not only as part of the step from
        alternative Proposal -1 to 0 (present state, orginal to present state,
        checked), but also for an alternative Proposal later. Therefore, in the second case the changed
        values are written into the same alternative, the current active one.
            
        """

        ccLocal = CheckFETel(checkInPlace = True)     # añade un objeto checkEq con todas las variables necesarias a la lista

        return self.localCheck(ccLocal)
        
    def eqCheck(self, eq):
        """Run the consistency check for the process' data only.
        
        The check is made "inPlace" not only as part of the step from
        alternative Proposal -1 to 0 (present state, orginal to present state,
        checked), but also for an alternative Proposal later. Therefore, in the second case the changed
        values are written into the same alternative, the current active one.
            
        """

        ccLocal = CheckEq(None, eq=eq, checkInPlace = True)     # añade un objeto checkEq con todas las variables necesarias a la lista
        return self.localCheck(ccLocal)

    def pipeCheck(self, pipe):
        """Run the consistency check for the electricity data only.
        
        The check is made "inPlace" not only as part of the step from
        alternative Proposal -1 to 0 (present state, orginal to present state,
        checked), but also for an alternative Proposal later. Therefore, in the second case the changed
        values are written into the same alternative, the current active one.
            
        """

        ccLocal = CheckPipe(None, pipe=pipe, checkInPlace = True)     # añade un objeto checkEq con todas las variables necesarias a la lista
        return self.localCheck(ccLocal)
        
    def hxCheck(self, h):
        """Run the consistency check for the electricity data only.
        
        The check is made "inPlace" not only as part of the step from
        alternative Proposal -1 to 0 (present state, orginal to present state,
        checked), but also for an alternative Proposal later. Therefore, in the second case the changed
        values are written into the same alternative, the current active one.
            
        """

        ccLocal = CheckHX(h, checkInPlace = True)     # añade un objeto checkEq con todas las variables necesarias a la lista
        return self.localCheck(ccLocal)
        
    def wheeCheck(self, n):
        """Run the consistency check for the electricity data only.
        
        The check is made "inPlace" not only as part of the step from
        alternative Proposal -1 to 0 (present state, orginal to present state,
        checked), but also for an alternative Proposal later. Therefore, in the second case the changed
        values are written into the same alternative, the current active one.
            
        """

        ccLocal = CheckWHEE(n, checkInPlace = True)     # añade un objeto checkEq con todas las variables necesarias a la lista
        return self.localCheck(ccLocal)
        
    def localCheck(self, ccObject, showProgress=True):
        """Run the consistency check for local data only.
        
        The check is made "inPlace" not only as part of the step from
        alternative Proposal -1 to 0 (present state, orginal to present state,
        checked), but also for an alternative Proposal later. Therefore, in the second case the changed
        values are written into the same alternative, the current active one.

        :params index: index of the ccObject from 0 to N-1
            
        """
        self.ccLocal = ccObject
        
        if showProgress: dlg = DialogGauge(Status.main, _("consistency check"), _("calculating"))
        logTrack("ModuleCC (localCheck):")
        
        ccObject.checkInPlace = True    # avoid general checking functions
        index = ccObject.index
        try:
            dataGroup = ccObject.dataGroup
        except:
            dataGroup = "something" #TODO -> can be eliminated once all ccObjects have attribute "dataGroup"

        estimateTracker.reset()

        conflict.reset() # reset the counters of conflicts before starting
        
        NCycles = 50 # maximum number of cycles
        NBestCycles = 10    # run the first cycles in CHECKMODE = "BEST", the
                            # remaining ones in "MEAN"

        balanceCtrl = 1.0   # initial setting: lagged value of (rel.) imbalance
        balanceMaxCtrl = 1.0
        diffBCctr = 0       # counts the number of consecutive cycles in which 
                            # the balance does not improve
                            
        for ncycle in xrange(NCycles):
            if ncycle < NBestCycles:
                setCheckMode("BEST")
            else:
                setCheckMode("MEAN")

            cycle.initTotalBalance()

            if DEBUG in ["ALL", "BASIC", "MAIN"]:
                print "===================================================="
                print "ModuleCC.localCheck: starting cycle no. %s" % (ncycle + 1)
                print "===================================================="

            # check of thermal processes
            if DEBUG in ["ALL"]:
                print "checking something no. %s" % index
                
            conflict.setDataGroup(dataGroup, index + 1)
            estimateTracker.setDataGroup(dataGroup, index + 1)

            ccObject.check() # run the check for each process
            
            # end of iteration: check convergence
            balanceCtrlOld = balanceCtrl
            balanceCtrl = balanceCtrl + 0.5 * (cycle.getMeanTotalBalance() - balanceCtrl)
            balanceImprovement = balanceCtrlOld - balanceCtrl
            
            if balanceImprovement < 1.e-7:
                diffBCctr += 1
            else:
                diffBCctr = 0
                
            if DEBUG in ["ALL", "MAIN", "BASIC"]:
                print "GLOBAL CYCLE CONVERGENCE: %s [lagged: %s]| %s " % \
                      (cycle.getMeanTotalBalance(), balanceCtrl, \
                       cycle.getMaxTotalBalance())

            message = "remaining balance error %6.2f percent" % (balanceCtrl * 100)
            if (ncycle % 10) == 0:
                logTrack(_("Consistency check: ") + "\n" + message)

            if balanceCtrl <= 1.e-3 and ncycle > NBestCycles:
                logTrack("ModuleCC.localCheck: convergence reached at %s cycles" % (ncycle + 1))
                break

            if diffBCctr >= 25:
                logTrack("ModuleCC.localCheck: no further improvement after %s cycles" % (50))
                break

            if ncycle == NCycles - 1 and balanceCtrl > 5.e-3:
                logTrack("ModuleCC.localCheck: Consistency check calculations did not converge. Data may not be fully balanced")

            if showProgress: dlg.update(100.0 * (ncycle + 1) / NCycles, message)

            if conflict.nConflicts > 0:
                break

        # At the end of the checking, screen the modules
        screen.reset()

        #TODO: -> substitute Loc by the corresponding identifier
        screen.setDataGroup("Obj.", index + 1)
        ccObject.screen()
                    
        if DEBUG in ["ALL", "MAIN", "BASIC"]:
            screen.show()
            conflict.show()
        
        if conflict.nConflicts == 0:
            logTrack("ModuleCC.localCheck: exporting results to SQL")
            ccObject.exportData()
        else:
            logTrack("ModuleCC.localCheck: consistency error didn't save adjusted values")

        if showProgress: dlg.Destroy()
        return conflict.nConflicts

#------------------------------------------------------------------------------
    def dataEstimate(self):
#------------------------------------------------------------------------------
#   calls the functions "estimate()" of all the checkers ...
#------------------------------------------------------------------------------

        estimateTracker.setDataGroup("FETel", 0)
        self.ccFETel.estimate()

        NI = self.NFET - 1
        for i in range(NI):       #then check all the Nfuels = NI-1 fuels
            estimateTracker.setDataGroup("FET", i+1)
            self.ccFET[i].estimate()
        
        NJ = self.NEquipe
        for j in range(NJ):       
            estimateTracker.setDataGroup("Eq.", j+1)
            self.ccEq[j].estimate()
        
        NK = self.NThProc
        for k in range(NK):       
            estimateTracker.setDataGroup("Proc", k+1)
            self.ccProc[k].estimate()

        NM = self.NPipeDuct
        for m in range(NM):      
            estimateTracker.setDataGroup("Pipe", m+1)
            self.ccPipe[m].estimate()

        NH = self.NHX
        for h in range(NH):       
            estimateTracker.setDataGroup("HX", h+1)
            self.ccHX[h].estimate()

        NN = self.NWHEE
        for n in range(NN):       
            estimateTracker.setDataGroup("WHEE", n+1)
#            self.ccWHEE[n].estimate()
            pass

        estimateTracker.setDataGroup("ECO", 0)
        self.ccECO.estimate()

#------------------------------------------------------------------------------
    def selectMainProcesses(self, checkInPlace=False):
#------------------------------------------------------------------------------
#   decides which of the processes are the most relevant ones
#------------------------------------------------------------------------------

        if checkInPlace is True:
            directSum = sum([item.UPH.val for item in self.ccProc 
                             if item.UPH.val is not None])
            UPHTotal = CCPar("UPHTotal", "S")
            UPHTotal.setValue(directSum)
        else:
            UPHTotal = self.totals.UPH
        if UPHTotal.val is None:
            return

        NK = self.NThProc
        
        UPH = CCRow("UPHk", NK)

        UPH_Ok = 0
        UPH_Ok_min = 0
        UPH_Ok_max = 0
        
        UPH_None = 0

        procList = []
        for k in range(NK):
            UPH[k] = self.ccProc[k].UPH

            if UPH[k].sqerr < MAX_SQERR:
                UPH_Ok += UPH[k].val
                UPH_Ok_min += UPH[k].valMin
                UPH_Ok_max += UPH[k].valMax
                procList.append((UPH[k].valMax, UPH[k].val, k))
            else:
                if UPH[k].val is not None:
                    UPH_None += UPH[k].val
                    procList.append((UPH[k].valMax, UPH[k].val, k))
                else:
                    UPH_None += UPH[k].valMax
                    procList.append((UPH[k].valMax, UPH[k].valMax, k))
            
#        if UPHTotal.val > 0:
#            f_Ok = UPH_Ok / UPHTotal.val
#            f_Ok_min = UPH_Ok_min / UPHTotal.val
#        else:
#            f_Ok = 1.0
#            f_Ok_min = 1.0
#        print "ModuleCC (selectMainProcesses): fraction of processes with desired accuracy on\n total heat demand = %s (%s)"%(f_Ok,f_Ok_min)

#..............................................................................
# sorts the processes by UPH
# while SUM(UPH_estimated) < 30% of estimated Total (sum of the smallest) or
#       SUM(UPH_maximum)   < 40% of estimated Total , those processes are
# considered as secondary processes.

        UPH_Sum = 0
        UPHMax_Sum = 0
        mainProcess = False
        
        procList.sort()
        for proc in procList:
            
            UPHMax_Sum += proc[0]
            UPH_Sum += proc[1]
            k = proc[2]
#            print "ModuleCC (selectMainProcesses): k %s UPH_Sum %s UPHMax_Sum %s:"%(k,UPH_Sum,UPHMax_Sum)
            if (UPH_Sum > 0.3 * UPHTotal.val) or \
               (UPHMax_Sum > 0.4 * UPHTotal.val):
                mainProcess = True

            self.ccProc[k].definePriority(mainProcess)
            

    def checkRobustness(self):
        """ called when OK button is pressed after consistency checking
            this function shall assure that a sufficient number of parameters is
            defined in order to avoid crashes in future processing
        """

        for eq in self.ccEq:
            eq.estimateEconomicParameters()
            eq.storeEconomicParameters()
        
#------------------------------------------------------------------------------

def defaultDisplayValue(internalValue, unitType, asString = True):
    """ same as display value, but uses default units """

    if internalValue is None:
        if asString:
            return "---"
        else:
            return internalValue

    unit = Globals.unitConversion.defaultUnit(unitType)
    if unit is None:
        unit = "-"
        unitType = "FRACTION"

    val = Globals.unitConversion.displayValue(internalValue, unit, unitType)

    if asString:
        return "%12.3f %s"%(val,unit)
    else:
        return val

#==============================================================================
#==============================================================================
#==============================================================================
if __name__ == "__main__":

# direct connecting to SQL database w/o GUI. for testing only
    stat = Status("testModuleCC")
    Status.SQL = MySQLdb.connect(user="root", db="einstein")
    Status.DB = pSQL.pSQL(Status.SQL, "einstein")
    Status.PId = 2
    Status.ANo = 0

#..............................................................................
    
    CC = ModuleCC()       # creates an instance of class CCheck
    CC.basicCheck()
