# -*- coding: utf-8 -*-
#==============================================================================
#
#   E I N S T E I N
#
#   Expert System for an Intelligent Supply of Thermal Energy in Industry
#   (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#   ParameterList
#
#------------------------------------------------------------------------------
#
#   List of parameters for text translation and output of CCHECK
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
from auxiliary.auxiliary import toGettextAndUnicode as _U


class ParameterList():
    def __init__(self):
        self.list = {
            "Code" : _U("Description"),
            "ProjectCreationDate" : _U("Date of creation of the project in EINSTEIN"),
            "ProjectFinalisationDate" : _U("Date of finalisation of the project"),
            "LastModificationDate" : _U("Date of last modification"),
            "Auditor_ID" : _U("Id of auditor who carryied out the audit"),
            "OriginalProjectID" : _U("Original ProjectID (maybe different of actual ProjectID for imported projects)"),
            "Campaign" : _U("Description of framework in which the audit was carried out (audit campaign or project)"),
            "ConfidentialityLevel" : _U("Level of confidentiality"),
            "ToolVersion" : _U("Version of EINSTEIN software tool"),
            "Name" : _U("name of the company"),
            "City" : _U("city"),
            "Country" : _U("country"),
            "DescripIndustry" : _U("description of the industry"),
            "Branch" : _U("branch"),
            "SubBranch" : _U("sub-branch"),
            "NACE" : _U("NACE code branch"),
            "SubNACE" : _U("NACE code sub-branch"),
            "DBNaceCode_id" : _U("ID of NACE Code in data base"),
            "Contact" : _U("name of contact person"),
            "Role" : _U("position of contact person in the company"),
            "Address" : _U("address"),
            "Phone" : _U("Telephone  No"),
            "Fax" : _U("Fax No"),
            "Email" : _U("E-mail"),
            "NEmployees" : _U("number of employees"),
            "Turnover" : _U("annual turnover"),
            "ProdCost" : _U("annual production cost"),
            "BaseYear" : _U("base year for economic data"),
            "Growth" : _U("growth rate of the production volume foreseen for the next 5 years"),
            "Independent" : _U("Is the company independent ?"),
            "OMThermal" : _U("yearly O&M heat & cold"),
            "OMElectrical" : _U("yearly O&M electrical"),
            "HPerDayInd" : _U("total hours of operation per working day"),
            "NShifts" : _U("number of shifts"),
            "NDaysInd" : _U("days of production / operation per year"),
            "NoProdStart" : _U("principal period of holidays or stops for maintenance - start date"),
            "NoProdStop" : _U("principal period of holidays or stops for maintenance - end date"),
            "Product" : _U("type of product"),
            "ProductCode" : _U("product's code"),
            "QProdYear" : _U("quantity of product(s) per year"),
            "ProdUnit" : _U("measurement unit for product quantity"),
            "TurnoverProd" : _U("annual turnover per product"),
            "NProducts" : _U("Number of products"),
            "HoursInd" : _U("Total yearly working hours in industry"),
            "Process" : _U("process short name"),
            "DBUnitOperation_id" : _U("0"),
            "UnitOperation" : _U("unit operation type"),
            "ProcType" : _U("process type"),
            "ProcMed" : _U("product or process medium (water, oil, air, ...)"),
            "PT" : _U("typical (final) temperature of the  process medium during operation"),
            "PTInFlow" : _U("inlet temperature of the process medium (before heat recovery)"),
            "PTInMax" : _U("maximum allowed temperature for external preheating"),
            "PTStartUp" : _U("start-up temperature of process medium (after breaks)"),
            "VInFlowDay" : _U("daily inflow of process medium"),
            "mInFlowYear" : _U("yearly inflow of process medium"),
            "VInFlowCycle" : _U("inflow of process medium per cycle") + _U(" (nominal)"),
            "VInFlowCycleEff" : _U("inflow of process medium per cycle") + _U(" (real)"),
            "VOutFlowCycle" : _U("outflow of process medium per cycle") + _U(" (nominal)"),
            "VOutFlowCycleEff" : _U("outflow of process medium per cycle") + _U(" (real)"),
            "VolProcMed" : _U("volume of the process medium within the equipment or storage"),
            "ThermalMass" : _U("thermal mass of the process"),
            "QOpProc" : _U("Power requirement of the process in operation"), #deprecated
            "dotQOpProc" : _U("Power requirement of the process in operation"),
            "HPerDayProc" : _U("hours of process operation per day"),
            "NBatch" : _U("number of cycles per day"),
            "NCyclesPerYear" : _U("number of cycles per year"),
            "HBatch" : _U("duration of 1 batch"),
            "NDaysProc" : _U("days of process operation per year"),
            "ProcMedOut" : _U("medium of outgoing waste heat flows"),
            "PTOutFlow" : _U("temperature of outgoing (waste) heat flows after heat recovery"),
            "PTOutFlowRec" : _U("temperature of outgoing (waste) heat flows"),
            "HOutFlow" : _U("specific enthalpy of outgoing (waste) heat flows"),
            "PTFinal" : _U("final  temperature of outgoing (waste) heat flows"),
            "PTFinalRequired" : _U("required temperature for mandatory cool-down of outgoing (waste) heat flows"),
            "VOutFlow" : _U("daily outflow of process medium"),
            "HeatRecOK" : _U("can heat be recovered from the outflowing medium?"),
            "HeatRecExist" : _U("Exists heat from heat  recovery for the process?"),
            "SourceWasteHeat" : _U("source of waste heat"),
            "PTInFlowRec" : _U("inlet temperature of the process medium  (after heat recovery)"),
            "EnclChartHCSupply" : _U("Enclosing Flowchart of HC supply system"),
            "SupplyMed" : _U("Medium supplying heat or cold to the process (water, steam, air)"),
            "PipeDuctProc" : _U("heat or cold supply to the process from distribution line / branch No."),
            "TSupply" : _U("temperature of the incoming medium supplying heat or cold to the process/heat exchanger"),
            "SupplyMedFlow" : _U("flow rate of the heat supply medium (close to process)"),
            "UPH" : _U("Total yearly process heat consumption"),
            "UPHTotal" : _U("Total yearly process heat consumption")+_U(" (All processes)"),
            "UPHProc" : _U("External heat supply to the process"),
            "UPHProck" : _U("External heat supply to the process"),
            "UPHProcTotal" : _U("External heat supply to the process")+_U(" (All processes)"),
            "UPHc" : _U("UPH - circulation / renovation"),
            "UPHcdot" : _U("power requirement for heating or cooling of inflow"),
            "DTUPHcNet" : _U("temperature lift of inflowing medium") + _U("after interhal heat rec.)"),
            "UPHcOfProcess" : _U("UPH - circulation / renovation") + _U(" (total for process)"),
            "UPHm" : _U("UPH - maintenance"),
            "UPHs" : _U("UPH - preheating at start-up"),
            "DTUPHs" : _U("Temperature difference for start-up of process"),
            "UPHw" : _U("available waste heat"),
            "UPHwOfProcess" : _U("available waste heat") + _U(" (total for process)"),
            "QWHProc" : _U("available waste heat from processes"),
            "UPHp" : _U("preheating from waste heat"),
            "UPHmin" : _U("minimum required heat demand with heat recovery"),
            "ProcHC" : _U("Heating (1) or Cooling (-1)"),
            "UPCtot" : _U("total process cooling demand"),
            "UPCnet" : _U("total process cooling demand after heat recovery"),
            "PMU" : _U("measurement unit for process medium (pmu)"),
            "MProcMed" : _U("yearly quantity of medium processed"),
            "PE_SEC" : _U("Specific energy consumption (primary energy)"),
            "EL_SEC" : _U("Specific energy consumption (electricity)"),
            "UPH_SEC" : _U("Specific energy consumption (UPH)"),
            "TEnvProc" : _U("Temperature of the process environment for calculation of heat losses under operation"),
            "UAProc" : _U("thermal heat loss coefficient of the process"),
            "dotQLossProc" : _U("Power requirement for compensating thermal losses"),
            "QEvapProc" : _U("Power requirement for evaporation or endothermal chemical reactions (negative for condensation or exothermal reactions)"), #deprecated
            "dotQEvapProc" : _U("Power requirement for evaporation or endothermal chemical reactions (negative for condensation or exothermal reactions)"),
            "NThProc" : _U("Number of thermal processes"),
            "QdotProc_c" : _U("circulation / renovation"),
            "QdotProc_m" : _U("maintenance"),
            "QdotProc_s" : _U("preheating at start-up"),
            "QdotProc_w" : _U("waste heat recovery"),
            "dotUPHc" : _U("circulation / renovation"),
            "dotUPHm" : _U("maintenance"),
            "dotUPHs" : _U("preheating at start-up"),
            "dotUPHw" : _U("waste heat recovery"),
            "HperYearProc" : _U("Yearly operating hours of process=HPerDayProc*NDaysProc"),
            "QHXProcInt" : _U("Heat recovered internally in each process via internal heat excanges (e.g. Pasteurisation)"),
            "UPHcGross" : _U("UPHc Gross (before internal heat recovery: only circulation!)"),
            "Fuel" : _U("Fuels used"),
            "FuelUnit" : _U("unit"),
            "MFuelYear" : _U("annual consumption"),
            "FECFuel" : _U("total own fuel consumption (LCV)"),
            "FuelTariff" : _U("fuel price"),
            "FuelCostYear" : _U("annual energy cost"),
            "PowerContrTot" : _U("contracted power"),
            "PowerContrStd" : _U("Contracted power standard"),
            "PowerContrPeak" : _U("Contracted power peak"),
            "PowerContrVall" : _U("Contracted power valley"),
            "ElectricityTotYear" : _U("annual consumption"),
            "ElectricityPeakYear" : _U("Annual consumption peak"),
            "ElectricityStandYear" : _U("Annual consumption standard"),
            "ElectricityValleyYear" : _U("Annual consumption valley"),
            "ElectricityGen" : _U("self-generation (co-generation)"),
            "ElectricitySales" : _U("sales to grid (co-generation)"),
            "text for Quest." : _U("Peak"),
            "text for Quest." : _U("Standard"),
            "text for Quest." : _U("Valley"),
            "text for Quest." : _U("TOTAL"),
            "text for Quest." : _U("Electric consumption according type of use"),
            "text for Quest." : _U("Electricity for thermal uses"),
            "text for Quest." : _U("Electricity for non-thermal uses"),
            "text for Quest." : _U("Refrigeration"),
            "ElectricityRef" : _U("Electricity for refrigeration"),
            "text for Quest." : _U("Air Conditioning"),
            "ElectricityAC" : _U("Electricity for air conditioning"),
            "text for Quest." : _U("Other uses"),
            "ElectricityThOther" : _U("Electricity for other thermal uses"),
            "text for Quest." : _U("Motors and machines"),
            "ElectricityMotors" : _U("Electricity for motors"),
            "text for Quest." : _U("Electro-chemics"),
            "ElectricityChem" : _U("Electricity for electro-chemical processes"),
            "text for Quest." : _U("Lightning"),
            "ElectricityLight" : _U("Electricity for lightning"),
            "ElTariffClassTot" : _U("Tariff type / class (total)"),
            "ElTariffClassStd" : _U("Tariff type / class (standard)"),
            "ElTariffClassPeak" : _U("Tariff type / class (peak)"),
            "ElTariffClassTotVall" : _U("Tariff type / class (valley)"),
            "ElTariffClassCHP" : _U("Tariff type / class (CHP)"),
            "ElTariffPowTot" : _U("Tariff on installed power (total)"),
            "ElTariffPowStd" : _U("Tariff on installed power (standard)"),
            "ElTariffPowPeak" : _U("Tariff on installed power (peak)"),
            "ElTariffPowVall" : _U("Tariff on installed power (valley)"),
            "ElTariffPowCHP" : _U("Tariff on installed power (CHP)"),
            "ElTariffCTot" : _U("Tariff on consumption (total)"),
            "ElTariffCStd" : _U("Tariff on consumption (standard)"),
            "ElTariffCPeak" : _U("Tariff on consumption (peak)"),
            "ElTariffCVall" : _U("Tariff on consumption (valley)"),
            "ETariffCHP" : _U("Tariff sales of CHP (without feedin-tariff)"),
            "ElCostYearTot" : _U("Annual electricity cost total"),
            "ElCostYearStd" : _U("Annual electricity cost standard"),
            "ElCostYearPeak" : _U("Annual electricity cost peak"),
            "ElCostYearVall" : _U("Annual electricity cost valley"),
            "ElSalesYearCHP" : _U("Annual electricity sales CHP"),
            "PercentElTotcost" : _U("Percentage of electricity cost on overall production cost"),
            "PercentFuelTotcost" : _U("Percentage of fuel cost on overall production cost"),
            "EnclMonthlyFuelBills" : _U("Enclosing monthly fuel bills?"),
            "EnclMonthlyElBills" : _U("Enclosing monthly electricity bills?"),
            "ElProd" : _U("Electricity consumption per product"),
            "FuelProd" : _U("Fuel consumption per product"),
            "FEC" : _U("Final energy consumption total (for the analysed industry)"),
            "FET" : _U("FET total (for the analysed industry)"),
            "FETTotal" : _U("FET total (for the analysed industry)"),
            "FEO" : _U("FEO total (for the analysed industry)"),
            "----FECFuel in questionnnaire---" : _U("0"),
            "FETi" : _U("FET by type of energy source"),
            "FETFuel" : _U("total own fuel consumption (LCV)"),
            "FETFuelTotal" : _U("total own fuel consumption (LCV)"),
            "FETFuel_j" : _U("total own fuel consumption (LCV)")+_U(" (equipment)"),
            "FEOFuel" : _U("total own fuel consumption (LCV)"),
            "FECel" : _U("Total electricity consumed by the factory"),
            "FETel" : _U("Electricity for thermal use (total industry)"),
            "FETel_j" : _U("Electricity for thermal use (total industry)")+_U(" (equipment)"),
            "FEOel" : _U("Electricity for non thermal use (total industry)"),
            "PEC" : _U("Primary energy consumptioin total (for the analysed industry)"),
            "PET" : _U("PET total (for the analysed industry)"),
            "PEO" : _U("PEO total (for the analysed industry)"),
            "PECFuel" : _U("Total Primary Energy per all fuels used in industry"),
            "PETFuel" : _U("Total Primary Energy per all fuels thermal use"),
            "PECel" : _U("Total Primary Energy per all electricity used in industry"),
            "PETel" : _U("Total Primary Energy per all electricity thermal use"),
            "FECel_EXT" : _U("Total own electricity consumption extrapolated to growth of production volume"),
            "FECFuel_EXT" : _U("Total own fuel consumption extrapolated to growth of production volume"),
            "PEC_EXT" : _U("Total primary energy consumption extrapolated to growth of production volume"),
            "FUEL_INT" : _U("Intensity of fuel consumption (LCV) on turnover"),
            "FUEL_SEC" : _U("Specific energy consumption (fuel)"),
            "EL_INT" : _U("Intensity of electricity consumption on turnover"),
            "EL_SEC" : _U("Specific energy consumption (electricity)"),
            "PE_INT" : _U("Primary energy intensity (turnover)"),
            "PE_SEC" : _U("Specific energy consumption (primary energy)"),
            "FuelCode" : _U("Code"),
            "NFuels" : _U("Number of fuels"),
            "ELECTRICITY_Type" : _U("Electricity by type of use"),
            "PEConvFuelMix" : _U("Conversion to primary energy of fuel mix"),
            "CO2ConvFuelMix" : _U("Specific CO2 emission of fuel mix"),
            "PEConvFuelCHP" : _U("Conversion to primary energy of fuel mix"),
            "CO2ConvFuelCHP" : _U("Specific CO2 emission of fuel mix"),
            "FuelTariffMix" : _U("Mean tariff of the fuel mix"),
            "FuelTariffCHP" : _U("Fuel tariff for CHP"),
            "Equipment" : _U("Short name of equipment"),
            "Manufact" : _U("Manufacturer"),
            "YearManufact" : _U("Year of  manufacturing or/and installation?"),
            "Model" : _U("Model"),
            "EquipType" : _U("Type of equipment"),
            "NumEquipUnits" : _U("Number of units of the same type"),
            "FuelType" : _U("Fuel type"),
            "HCGPnom" : _U("Nominal power (heat or cold, output)"),
            "FuelConsum" : _U("Fuel consumption (power)"),
            "FuelConsumNom" : _U("Fuel consumption - power") + _U(" (nominal)"),
            "FuelConsumFL" : _U("Fuel consumption - power") + _U(" (at full load)"),
            "UnitsFuelConsum" : _U("Units (fuel consumption)"),
            "ElectriConsum" : _U("Electrical power input"),
            "ElectriConsumNom" : _U("Electrical power input") + _U(" (nominal)"),
            "ElectriConsumFL" : _U("Electrical power input") + _U(" (at full load)"),
            "HCGTEfficiency" : _U("Mean overall thermal conversion efficiency"),
            "HCGTEfficiencyNom" : _U("Mean overall thermal conversion efficiency") + _U(" (nominal)"),
            "HCGTEfficiencyFL" : _U("Mean overall thermal conversion efficiency") + _U(" (at full load)"),
            "TExhaustGas" : _U("Temperature of exhaust gas at standard operation conditions"),
            "ExcessAirRatio" : _U("Excess air ratio"),
            "PartLoad" : _U("Mean utilisation factor (full capacity = 100%)"),
            "HPerDayEq" : _U("Hours of operation per day"),
            "NDaysEq" : _U("Days of operation per year"),
            "USHtotQ" : _U("Useful Supply Heat from questionnaire"),
            "PipeDuctEquip" : _U("Heat or cold supplyed to the distribution line / branch (piping or duct) no."),
            "CHP only" : _U("0"),
            "ElectriProduction" : _U("Electricity production"),
            "HCGEEfficiency" : _U("Electrical conversion efficiency"),
            "HCGEEfficiencyNom" : _U("Electrical conversion efficiency") + _U(" (nominal)"),
            "HCGEEfficiencyFL" : _U("Electrical conversion efficiency") + _U(" (at full load)"),
            "Heat pumps only" : _U("Heat pumps only:"),
            "HeatSourceLT" : _U("Low temperature heat source"),
            "THeatSourceLT" : _U("Temperature of heat source"),
            "Thermal heat pumps" : _U("Heat pumps only:"),
            "ThermalConsum" : _U("Thermal power input  (thermal heat pumps only)"),
            "THeatSourceHT" : _U("Driving temperature (thermal heat pumps only)"),
            "HeatSourceHT" : _U("Origin of high temperature heat source (thermal heat pumps only)"),
            "Cold generation only" : _U("0"),
            "Refrigerant" : _U("Refrigerant"),
            "ThermalConsum" : _U("Thermal power input"),
            "THeatSourceHT" : _U("Driving temperature"),
            "HeatSourceHT" : _U(" Origin of high temperature heat source"),
            "DestinationWasteHeat" : _U("Destination of waste heat"),
            "TemperatureReCooling" : _U("Temperature of re-cooling"),
            "CoolTowerType" : _U("Only for cooling: type of cooling tower: dry/wet ?"),
            "NEquip" : _U("Number of supply equipments"),
            "USH" : _U("Useful Supply Heat"),
            "USHTotal" : _U("Useful Supply Heat")+_U(" (Total)"),
            "USHj" : _U("Useful Supply Heat by equipment"),
            "USHConv" : _U("Useful Supply Heat added by conversion equipment"),
            "FET" : _U("Final Energy consumption for Thermal use for industry (total)"),
            "FETj" : _U("Final energy consumed by the equipment"),
            "QHXEqTotal" : _U("Useful Supply Heat for industry (total)"),
            "QHXEq" : _U("Waste heat input (equipment)"),
            "QWHEq" : _U("Waste heat output (exhaust gas, waste heat fr. heat pumps, ...)"),
            "HGEffAvg" : _U("Average conversion efficiency from final energy to useful supply heat"),
            "FlowExhaustGas" : _U("Exhaust gas flow rate"),
            "QExhaustGas" : _U("Energy content in exhaust gas"),
            "HPerYearEq" : _U("Yearly operating hours of equipment = HperDayEq*NDaysEq"),
            "HPerYearEqNom" : _U("Yearly operating hours of equipment (nominal) = HPerYearEq*PartLoad"),
            "EqNo" : _U("Consecutive numbering of equipments in the system"),
            "CascadeIndex" : _U("Index of equipment in heat supply cascade"),
            "OandMfix" : _U("O&M costs (fix)") + ". " + _U("Equipment"),
            "OandMvar" : _U("O&M costs (variable)") + ". " + _U("Equipment"),
            "HXName" : _U("Short name of heat exchanger"),
            "HXType" : _U("Heat exchanger type"),
            "QdotHX" : _U("Heat transfer rate"),
            "QdotHXNom" : _U("Heat transfer rate") + _U(" (nominal)"),
            "HXLMTD" : _U("Log. Mean Temperature Diff. (LMTD)"),
            "HXLMTDNom" : _U("Log. Mean Temperature Diff. (LMTD)") + _U(" (nominal)"),
            "UA" : _U("Heat transfer coefficient"),
            "QHX" : _U("Total heat transfered"),
            "HXSourceInlet" : _U("Heat source (process [+outflow no.], equipment, ...)"),
            "HXTSourceInlet" : _U("Inlet temperature (source)"),
            "HXhSourceInlet" : _U("Inlet specific enthalpy (source)"),
            "HXTSourceOutlet" : _U("Outlet temperature (source)"),
            "HXhSourceOutlet" : _U("Outlet specific enthalpy (source)"),
            "HXSink" : _U("Heat sink (process, pipe/duct)"),
            "HXTSinkInlet" : _U("Inlet temperature (sink)"),
            "HXTSinkOutlet" : _U("Outlet temperature (sink)"),
            "OMFix" : _U("O&M costs (fix)") + ". " + _U("HX"),
            "OMVar" : _U("O&M costs (variable)") + ". " + _U("HX"),
            "WHEEName" : _U("Short name of electrical equipment"),
            "WHEEEqType" : _U("Equipment type"),
            "WHEEWasteHeatType" : _U("Waste heat type"),
            "QWHEE" : _U("Available waste heat"),
            "WHEEMedium" : _U("Medium"),
            "WHEEFlow" : _U("Flow rate"),
            "WHEETOutlet" : _U("waste heat temperature"),
            "WHEEPresentUse" : _U("present use of waste heat"),
            "'----" : _U("Schedule"),
            "HPerDayWHEE" : _U("hours of  operation per day"),
            "NBatchWHEE" : _U("number of batches per day"),
            "HBatchWHEE" : _U("duration of 1 batch"),
            "NDaysWHEE" : _U("days of process operation per year"),
            "PipeDuct" : _U("name of the branch / distribution system"),
            "HeatSource(s)" : _U("heat or cold supply comes from (HX or equipment) No.:"),
            "HeatDistMedium" : _U("heat or cold distribution medium"),
            "DistribCircFlow" : _U("nominal production or circulation rate (specify units)"),
            "UnitsDistribCircFlow" : _U("Units (of nominal production or circulation rate)"),
            "ToutDistrib" : _U("outlet temperature (to distribution)"),
            "TreturnDistrib" : _U("return temperature (from distribution)"),
            "PercentRecirc" : _U("Percentage of recirculation"),
            "Tfeedup" : _U("feed-up in open circuit"),
            "PressDistMedium" : _U("pressure of heat or cold distribution medium"),
            "TotLengthDistPipe" : _U("total length of distribution piping or ducts (one way)"),
            "UAPipe" : _U("total coefficient of heat losses for piping or ducts"),
            "DDistPipe" : _U("mean pipe diameter"),
            "DeltaDistPipe" : _U("insulation thickness"),
            "NumStorageUnits" : _U("number of the storage units"),
            "VtotStorage" : _U("volume of the storage"),
            "TypeStorage" : _U("type of heat storage"),
            "PmaxStorage" : _U("pressure of heat storage medium"),
            "TmaxStorage" : _U("maximum temperature of the storage"),
            "EnclChartHCDistribSys" : _U("Enclosing flowchart of the heat supply and distribution system?"),
            "NPipeDuct" : _U("Number of pipes or ducts"),
            "PipeDuctNo" : _U("name of the branch / distribution system"),
            "HDEffAvg" : _U("Average distribution efficiency"),
            "QHXPipe" : _U("Waste heat input to the pipe"),
            "QWHPipe" : _U("Waste heat output from circuit (E.g. Not directrly returned condensate)"),
            "USHm" : _U("Useful heat supply from equipment to pipe"),
            "USHPipe" : _U("Useful heat supply from equipment to pipe") + _U(" (incl. waste heat)"),
            "UPHProcm" : _U("Useful heat supply from pipe to process"),
            "HPerYearPipe" : _U("Annual hours of operation of the pipe (relevant time for calculation of thermal losses)"),
            "NBuild" : _U("0"),
            "BuildName" : _U("Building short name"),
            "BuildConstructSurface" : _U("Constructed surface"),
            "BuildUsefulSurface" : _U("Useful surface"),
            "BuildUsage" : _U("Use of the building"),
            "BuildEnergyDemand" : _U("Global data on energy demand"),
            "BuildMaxHP" : _U("Maximum heating power"),
            "BuildMaxCP" : _U("Maximum cooling power"),
            "BuildAnnualHeating" : _U("Annual heating demand"),
            "BuildAnnualAirCond" : _U("Annual demand of air conditioning"),
            "BuildDailyDHW" : _U("Daily consumption of  DHW"),
            "BuildHoursOccup" : _U("Hours of occupation"),
            "BuildDaysInUse" : _U("Days of use per year"),
            "BuildHolidaysPeriod" : _U("Holidays period"),
            "BuildHeatingPeriod" : _U("Heating period"),
            "BuildAirCondPeriod" : _U("Air conditioning period"),
            "EnclBuildDraw" : _U("Enclosing building drawing or sketch?"),
            "REMotivation" : _U("Main motivation for renewable energy use"),
            "REInterest" : _U("Are you interested in the use of renweable energy? (solar thermal/biomass)"),
            "REReason" : _U("If affirmative, please specify the principal reasons:"),
            "REReason1" : _U(" possibility of saving fuel cost"),
            "REReason2" : _U(" contribution to a more ecologic energy supply"),
            "text for Quest., value of REReason" : _U("using solar energy helps for a better marketing of your products"),
            "text for Quest., value of REReason" : _U("others:"),
            "SurfAreaRoof" : _U("Available roof area"),
            "SurfAreaGround" : _U("Available ground area"),
            "InclinationRoof" : _U("Positioning of the roof area"),
            "InclinationGround" : _U("Positioning of the ground area"),
            "OrientationRoof" : _U("Orientation of roof (to south)"),
            "OrientationGround" : _U("Orientation of ground (to south)"),
            "ShadingRoof" : _U("shading of roof"),
            "ShadingGround" : _U("shading of ground"),
            "DistanceToRoof" : _U("Distance between roof and process"),
            "DistanceToGround" : _U("Distance between ground and process"),
            "RoofType" : _U("type of roof"),
            "RoofStaticLoadCap" : _U("static load capacity of the roof(s)"),
            "EnclBuildGroundSketch" : _U("Enclosing of drawing or scheme of building?"),
            "BiomassFromProc" : _U("Type of biomass available from processes"),
            "PeriodBiomassProcStart" : _U("Period of year the biomass is available - start"),
            "PeriodBiomassProcStop" : _U("Period of year the biomass is available - stop"),
            "NDaysBiomassProc" : _U("Number of days biomass is produced"),
            "QBiomassProcDay" : _U("Daily quantity of biomass"),
            "SpaceBiomassProc" : _U("Space availability to stock biomass?"),
            "LCVBiomassProc" : _U("LCV biomass"),
            "HumidBiomassProc" : _U("Humidity"),
            "BiomassFromRegion" : _U("Type of biomass available"),
            "PriceBiomassRegion" : _U("Unit price of biomass"),
            "PeriodBiomassRegionStart" : _U("Period of year the biomass is available - start"),
            "PeriodBiomassRegionStop" : _U("Period of year the biomass is available - stop"),
            "NDaysBiomassRegion" : _U("Number of days the biomass is produced"),
            "InflationRate" : _U("General inflation rate"),
            "FuelPriceRate" : _U("Rate of increment of energy prices"),
            "InterestExtFinancing" : _U("Nominal rate of interest for external financing of installations"),
            "PercentExtFinancing" : _U("Percentage of external financing for installations"),
            "AmortisationTime" : _U("Time for economic amortisation of installations"),
            "PublicFundType" : _U("Type (credit, subvention)"),
            "OMGenTot" : _U("General maintenance - total costs"),
            "OMGenOP" : _U("General maintenance - own personnel"),
            "OMGenEP" : _U("General maintenance - external personnel"),
            "OMGenFung" : _U("General maintenance - fungible assets"),
            "OMBuildTot" : _U("Buildings - total costs"),
            "OMBuildOP" : _U("Buildings - own personnel"),
            "OMBuildEP" : _U("Buildings - external personnel"),
            "OMBiuildFung" : _U("Buildings - fungible assets"),
            "OMMachEquipTot" : _U("Machines and equipment - total costs"),
            "OMMachEquipOP" : _U("Machines and equipment - own personnel"),
            "OMMachEquipEP" : _U("Machines and equipment  - external personnel"),
            "OMMachEquipFung" : _U("Machines and equipment  - fungible assets"),
            "OMHCGenDistTot" : _U("Generation and distribution of heat and cold - total costs"),
            "OMHCGenDistOP" : _U("Generation and distribution of heat and cold - own personnel"),
            "OMHCGenDistEP" : _U("Generation and distribution of heat and cold  - external personnel"),
            "OMHCGenDistFung" : _U("Generation and Distribution of heat and cold  - fungible assets"),
            "OMTotalTot" : _U("Total - total costs"),
            "OMTotalOP" : _U("Total - own personnel"),
            "OMTotalEP" : _U("Total - external personnel"),
            "OMTotalFung" : _U("Total - fungible assets"),
            "OMjTotal" : _U("Total O&M costs for equipment"),
            "OMhTotal" : _U("Total O&M costs for heat exchangers"),
            "OMTotal" : _U("Total O&M costs"),
            "EnergyManagExisting" : _U("Is there any energy menagement system implemented?"),
            "EnergyManagExternal" : _U("Is the energy menagent externalized?"),
            "General energetic and environmental conversion parameters" : _U("0"),
            "PEConvEl" : _U("Conversion to primary energy of electricity"),
            "CO2ConvEl" : _U("Specific CO2 emission of electricity"),
            "HRInvRate" : _U("Specific investment cost for heat recovery system"),
            "HROMRate" : _U("Specific annual O&M cost for heat recovery system"),
            "FeedInCHP" : _U("Feedin Tariff for CHP Electricity"),
            "SolarInvRateL" : _U("Specific turn-key installation cost large systems ( > 1 MW)"),
            "SolarInvRateS" : _U("Specific turn-key installation cost small systems (< 10 kW)"),
            "SolarOMRate" : _U("specific yearly O&M costs"),
            "SolarFunding" : _U("Percentage of public funding"),
            "QuSolar" : _U("Specifc solar useful heat production"),
            "Economic parameters" : _U("0"),
            "Inflation" : _U("General inflation rate"),
            "InflationFuels" : _U("Foreseen rise of fuel prices"),
            "InterestRate" : _U("Interest rate for external financing"),
            "ExternalFinance" : _U("Percentage of external financing"),
            "Amortisation" : _U("Economic depreciation period"),
            "Annuity" : _U("Annuity of investment"),
            "EEFunding" : _U("Public funding for energy saving measures"),
            "PE_INT_BML" : _U("benchmark (lower limit)"),
            "PE_INT_BMH" : _U("benchmark (upper limit)"),
            "PE_INT_TAR" : _U("target"),
            "PE_INT_REF" : _U("reference code selected benchmark and target"),
            "EL_INT_BML" : _U("benchmark (lower limit)"),
            "EL_INT_BMH" : _U("benchmark (upper limit)"),
            "EL_INT_TAR" : _U("target"),
            "EL_INT_REF" : _U("reference code selected benchmark and target"),
            "FUEL_INT_BML" : _U("benchmark (lower limit)"),
            "FUEL_INT_BMH" : _U("benchmark (upper limit)"),
            "FUEL_INT_TAR" : _U("target"),
            "FUEL_INT_REF" : _U("reference code selected benchmark and target"),
            "PE_SEC_BML" : _U("benchmark (lower limit)"),
            "PE_SEC_BMH" : _U("benchmark (upper limit)"),
            "PE_SEC_TAR" : _U("target"),
            "PE_SEC_REF" : _U("reference code selected benchmark and target"),
            "EL_SEC_BML" : _U("benchmark (lower limit)"),
            "EL_SEC_BMH" : _U("benchmark (upper limit)"),
            "EL_SEC_TAR" : _U("target"),
            "EL_SEC_REF" : _U("reference code selected benchmark and target"),
            "FUEL_SEC_BML" : _U("benchmark (lower limit)"),
            "FUEL_SEC_BMH" : _U("benchmark (upper limit)"),
            "FUEL_SEC_TAR" : _U("target"),
            "FUEL_SEC_REF" : _U("reference code selected benchmark and target"),
            "PE_UOS_BML" : _U("benchmark (lower limit)"),
            "PE_UOS_BMH" : _U("benchmark (upper limit)"),
            "PE_UOS_TAR" : _U("target"),
            "PE_UOS_REF" : _U("reference code selected benchmark and target"),
            "EL_UOS_BML" : _U("benchmark (lower limit)"),
            "EL_UOS_BMH" : _U("benchmark (upper limit)"),
            "EL_UOS_TAR" : _U("target"),
            "EL_UOS_REF" : _U("reference code selected benchmark and target"),
            "UPHS_BML" : _U("benchmark (lower limit)"),
            "UPHS_BMH" : _U("benchmark (upper limit)"),
            "UPHS_TAR" : _U("target"),
            "UPHS_REF" : _U("reference code selected benchmark and target"),
            "ENE_INT_CHK" : _U("evaluation of energy intensity"),
            "ENE_INT_COM" : _U("comment on evaluation of energy intensity"),
            "ENE_SEC_CHK" : _U("evaluation of energy intensity"),
            "ENE_SEC_COM" : _U("comment on evaluation of energy intensity"),
            "ENE_UOS_CHK" : _U("evaluation of energy intensity"),
            "ENE_UOS_COM" : _U("comment on evaluation of energy intensity"),
            "QWHEq-Con": _U("available waste heat") + _U(" (equipment)"),
            "QWHPipe-Con": _U("available waste heat") + _U(" (pipes and ducts)"),
            "QWHProc-Con": _U("available waste heat") + _U(" (processes)"), 
            "QWHEE-Con": _U("available waste heat") + _U(" (electrical equipment)"), 
            "QHXEq-Con": _U("recovered waste heat") + _U(" (to equipment)"), 
            "QHXPipe-Con": _U("recovered waste heat") + _U(" (to pipes or duct)"), 
            "QHXProc-Con": _U("recovered waste heat") + _U(" (to process)"),
            "MRec": _U("recovered waste heat"),
            "MAmb": _U("waste heat dissipated to ambient"),
            "dummy":"dummy"
        }
