# -*- coding: utf-8 -*-
#============================================================================== 				
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#	CONTROL
#			
#------------------------------------------------------------------------------
#			
#	Short description:
#	
#	Auxiliary functions for the control of execution of the tool
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
from globals import Globals
from GUI.status import Status
from modules.messageLogger import *
from GUI.conflictFrame import conflictFrame
from GUI.dialogGauge import DialogGauge
from modules.energyStats.moduleEA1 import ModuleEA1
from modules.energyStats.moduleEA2 import ModuleEA2
from modules.energyStats.moduleEA3 import ModuleEA3
from modules.energyStats.moduleEA4 import ModuleEA4
from modules.energyStats.moduleEA5 import ModuleEA5
from modules.moduleCS import ModuleCS
from modules.guiModules.moduleA import ModuleA
from auxiliary.auxiliary import toGettextAndUnicode as _U
import numpy as np
import wx 
from modules.energyStats.moduleEM1 import ModuleEM1
from modules.energyStats.moduleEM2 import ModuleEM2

from GUI.panels.DB.panelDBCHP import PanelDBCHP
from GUI.panels.DB.panelDBHeatPump import PanelDBHeatPump

from constants import APPDATA
from lib.fileIO.fileIO import CSVTables

REPORTKEYS = ["A_REPORT",
              "BM1_REPORT", "BM1_REPORT_F",
              "BM2_01_REPORT", "BM2_01_REPORT_F",
              "BM2_02_REPORT", "BM2_02_REPORT_F",
              "BM2_03_REPORT", "BM2_03_REPORT_F",
              "BM3_01_REPORT", "BM3_01_REPORT_F",
              "BM3_02_REPORT", "BM3_02_REPORT_F",
              "BM3_03_REPORT", "BM3_03_REPORT_F",
              "BM3_04_REPORT", "BM3_04_REPORT_F",
              "BM3_05_REPORT", "BM3_05_REPORT_F",
              "CS1_REPORT",
              "CS2_REPORT", "CS2_REPORT_C",
              "CS3_REPORT",
              "CS4_REPORT",
              "CS5_REPORT",
              "CS6_REPORT",
              "CS7_REPORT", "CS7_REPORT_PAR", "CS71_REPORT", "CS72_REPORT",
              "EA1_REPORT", "EA1_REPORT_F",
              "EA2_REPORT", "EA2_REPORT_F",
              "EA3_FET_REPORT", "EA3_FET_REPORT_F",
              "EA3_USH_REPORT", "EA3_USH_REPORT_F",
              "EA3_USH_REPORT_C", "EA3_USH_REPORT_F_C",
              "EA4a_REPORT", "EA4a_REPORT_F",
              "EA4a_REPORT_C", "EA4a_REPORT_F_C",
              "EA4a_REPORT_ANNEX",
              "EA4b_REPORT", "EA4b_REPORT_F",
              "EA4b_REPORT_C", "EA4b_REPORT_F_C",
              "EA4b_PLOT_REPORT", "EA4b_PLOT_REPORT_F",
              "EA4b_PLOT_REPORT_C", "EA4b_PLOT_REPORT_F_C",
              "EA4c_REPORT", "EA4c_REPORT_F",
              "EA4c_REPORT_C", "EA4c_REPORT_F_C",
              "EA4c_PLOT_REPORT", "EA4c_PLOT_REPORT_F",
              "EA4c_PLOT_REPORT_C", "EA4c_PLOT_REPORT_F_C",
              "EM1_REPORT", "EM1_REPORT_F",
              "EM1_REPORT_C", "EM1_REPORT_F_C",
              "EM2_REPORT", "EM2_REPORT_F",
              "EM2_REPORT_C", "EM2_REPORT_F_C",
              "EQ01", "EQ01ECO",
              "EQ02", "EQ02ECO",
              "EQ03", "EQ03ECO",
              "EQ04", "EQ04ECO",
              "EQ05", "EQ05ECO",
              "EQ06", "EQ06ECO",
              "EQ07", "EQ07ECO",
              "EQ08", "EQ08ECO",
              "EQ09", "EQ09ECO",
              "EQ10", "EQ10ECO",
              "HC01_REPORT",
              "HC02_REPORT",
              "HC03_REPORT",
              "HC04_REPORT",
              "HC05_REPORT",
              "HC_REPORT_F",
              "HX00",
              "HX01", "HX01_PLOT_REPORT",
              "HX02", "HX02_PLOT_REPORT",
              "HX03", "HX03_PLOT_REPORT",
              "HX04", "HX04_PLOT_REPORT",
              "HX05", "HX05_PLOT_REPORT",
              "HX_F", "HX_SECOND_F",
              "PROD01", "PROD02", "PROD03",
              "SUMMARY", "SUMMARY_TABLE",
              "TCA", "TCA_CRITERIA", "TCA_RATING", "TCA_UNVIABLE", "TCAPARS",
              "TITLE"]

#------------------------------------------------------------------------------		
def prepareDataForReport():
#------------------------------------------------------------------------------		
#   calls the functions necessary for writing the report
#------------------------------------------------------------------------------		

    dlg = DialogGauge(Status.main,"Report generation","preparing data")
    
#Title page
    (projectData, generalData) = Status.prj.getProjectData()
    reportTitle = unicode(projectData.Name, "utf-8")
    if projectData.City is not None: reportTitle += u", " + unicode(projectData.City, "utf-8")
    if projectData.Country is not None: reportTitle += u", " + unicode(projectData.Country, "utf-8")
    Status.int.setGraphicsData("TITLE", np.array([[reportTitle]]))
    logTrack("Control (prepareDataForReport): Title = %s" % (reportTitle.encode("utf-8"), ))
    logTrack("Type: %s" % (type(reportTitle), ))
    logTrack(Status.int.GData["TITLE"])
    
    Status.prj.setActiveAlternative(0)  #select present state
    
    Status.mod.moduleEA.update()

    dlg.update(10)

#Cap. 2.1 - 2.3
    modEA1 = ModuleEA1(['EA1'])
    modEA1.initModule()

    modEA2 = ModuleEA2(['EA2'])
    modEA2.initPanel()

    modEA3 = ModuleEA3(['EA3_FET','EA3_USH'], showHeating=True)
    modEA3.initModule()
    modEA3 = ModuleEA3(['EA3_FET','EA3_USH'], showHeating=False)
    modEA3.initModule()
    
    dlg.update(15)

    modEA4 = ModuleEA4(['EA4a_Table','EA4a_Plot'])
    modEA4.updatePanel(heating=True)   
    modEA4.updatePanel(heating=False)
    modEA4 = ModuleEA4(['EA4b_Table','EA4b_Plot'])
    modEA4.updatePanel(heating=True)   
    modEA4.updatePanel(heating=False)
    modEA4 = ModuleEA4(['EA4c_Table','EA4c_Plot'])
    modEA4.updatePanel(heating=True)   
    modEA4.updatePanel(heating=False)
    
    modEA5 = ModuleEA5(['EA5_EI','EA5_SEC'])
    modEA5.initModule()

    modEM1 = ModuleEM1(['EM1_Table','EM1_Plot'], showHeating=True) 
    modEM1.initModule()
    modEM1 = ModuleEM1(['EM1_Table','EM1_Plot'], showHeating=False) 
    modEM1.initModule()
    
    Status.mod.moduleHR.updateReportData(includePinch=True)
    dlg.update(20)


#Cap. 2.4
    Status.mod.moduleBM.initPanel(['BM1'])

    dlg.update(30)

    products = Status.mod.moduleBM.products
    if len(products) > 0:
        Status.mod.moduleBM.product = products[0]
    Status.mod.moduleBM.initPanel(['BM2'])

    dlg.update(40)

    processes = Status.mod.moduleBM.processes
    if len(processes) > 0:
        Status.mod.moduleBM.process = processes[0]
    Status.mod.moduleBM.initPanel(['BM3'])

    dlg.update(50)

#Cap. 3
    moduleA = ModuleA(None)
    moduleA.updatePanel()

    for ANo in range(1,(Status.NoOfAlternatives+1)):
        Status.prj.setActiveAlternative(ANo)

        Status.mod.moduleEA.update()    #calculate for update of CS plots

        Status.mod.moduleHR.updateReportData(includePinch=False) #for all alternatives create only yearly HX results
        
        Status.mod.moduleHC.updatePanel()
        
        dlg.update(50 + 20.0 * ANo / Status.NoOfAlternatives)
    dlg.update(70)
    
#Cap. 4
    modCS = ModuleCS(['CS1_Plot'])
    modCS.updatePanel()
    
    modCS = ModuleCS(['CS2_Plot'])
    modCS.updatePanel(heating=True)
    modCS.updatePanel(heating=False)
    
    modCS = ModuleCS(['CS3_Plot'])
    modCS.updatePanel()
    
    modCS = ModuleCS(['CS4_Plot'])
    modCS.updatePanel()

    # assure update of economics module. TODO: should be done in a more coherent way instead of here in between
    try:
        Status.mod.moduleECO.runModule()
    except:
        print "Error in TCA Module"
        logTrack("prepareDataForReport: error in TCA module")
    
    modCS = ModuleCS(['CS5_Plot'])
    modCS.updatePanel()
    
    modCS = ModuleCS(['CS6_Plot'])
    modCS.updatePanel()
    
    modCS = ModuleCS(['CS7_Plot'])
    modCS.updatePanel()

    dlg.update(80)


#Cap. 5 and 6
    
    if Status.FinalAlternative is not None:
        Status.prj.setActiveAlternative(Status.FinalAlternative)
    else:
        Status.prj.setActiveAlternative(0)

    Status.mod.moduleEA.update()
    if Status.ANo > 0:
        Status.mod.moduleEnergy.runSimulation() # necessary for obtaining converged results in EA and EM panels
                                                # TODO: check here - converged simulation should have been done already
                                                # in chapter 4 (comparative study)


#Chap. 5


#Chap. 6.1 - 6.3

    Status.mod.moduleHR.updateReportData(includePinch=True) #for selected alternatives create detailed HX results

    modEA1 = ModuleEA1(['EA1'])
    modEA1.initModule()

    modEA2 = ModuleEA2(['EA2'])
    modEA2.initPanel()

    modEA3 = ModuleEA3(['EA3_FET','EA3_USH'], showHeating=True)
    modEA3.initModule()
    modEA3 = ModuleEA3(['EA3_FET','EA3_USH'], showHeating=False)
    modEA3.initModule()
    
    modEA4 = ModuleEA4(['EA4a_Table','EA4a_Plot'])
    modEA4.updatePanel(heating=True)   
    modEA4 = ModuleEA4(['EA4b_Table','EA4b_Plot'])
    modEA4.updatePanel(heating=True)   
    modEA4 = ModuleEA4(['EA4c_Table','EA4c_Plot'])
    modEA4.updatePanel(heating=True)   

    modEA4 = ModuleEA4(['EA4a_Table','EA4a_Plot'])
    modEA4.updatePanel(heating=False)   
    modEA4 = ModuleEA4(['EA4b_Table','EA4b_Plot'])
    modEA4.updatePanel(heating=False)   
    modEA4 = ModuleEA4(['EA4c_Table','EA4c_Plot'])
    modEA4.updatePanel(heating=False)

    modEA5 = ModuleEA5(['EA5_EI','EA5_SEC'])
    modEA5.initModule()

    modEM1 = ModuleEM1(['EM1_Table','EM1_Plot'], showHeating=True) 
    modEM1.updatePanel(showHeating=True)
    modEM1.updatePanel(showHeating=False)

    modEM2 = ModuleEM2(['EM2_Table','EM2_Plot'], showHeating=True) 
    modEM2.updatePanel(showHeating=True)
    modEM2.updatePanel(showHeating=False)
    
    dlg.update(90)

#Cap. 6.4
    Status.mod.moduleBM.initPanel(['BM1'])

    dlg.update(93)

    products = Status.mod.moduleBM.products
    if len(products) > 0:
        Status.mod.moduleBM.product = products[0]
    Status.mod.moduleBM.initPanel(['BM2'])

    dlg.update(96)

    processes = Status.mod.moduleBM.processes
    if len(processes) > 0:
        Status.mod.moduleBM.process = processes[0]
    Status.mod.moduleBM.initPanel(['BM3'])

    dlg.update(99)

#Summary
    
    modCS = ModuleCS(['Summary'])
    modCS.updatePanel()


    dlg.update(100)
    dlg.Destroy()

    # write report data to the report_logger (currently used for testing purposes only)
    if hasattr(Globals, "robbie") and Globals.robbie:
        reportLoggerFile = APPDATA + "/report_data.csv"
        reportLogger = CSVTables(reportLoggerFile)

        dataKeys = Status.int.GData.keys()
        keys = [key for key in REPORTKEYS if key in dataKeys]

        array = [Status.int.GData[key] for key in keys]
        colTitles = None
        rowTitles = [["%03d"%i for i in xrange(1, len(a)+1)] for a in array]
        
        reportLogger.write(keys, colTitles, rowTitles, array)
    
#------------------------------------------------------------------------------		
def autoRun(parent,subset=[]):
#------------------------------------------------------------------------------		
#   calls the functions necessary for writing the report
#------------------------------------------------------------------------------		

    if subset == []: subset = ["HR","ST","CHP","HP","BB","MIX"]
    
    logTrack("Control (autoRun): starting with subset of alternatives %s"%subset)

#..............................................................................
# If there's no data, there's nothing to do ...

    if Status.StatusQ == 0:
        logError("Control (autoRun): sorry, but SOME data have to be filled in before starting")

#..............................................................................
# Consistency check ...

    if Status.StatusCC == 0:
        logMessage("Control (autoRun): starting consistency check")

        showMessage("Control (autoRun): first we start consistency check")

        Status.prj.copyQuestionnaire()
        nc = Status.mod.moduleCC.basicCheck()

        if nc > 0:
            checkOK = False
            if Status.UserInteractionLevel in ["semi-automatic","interactive"]:
                Status.mod.moduleCC.updatePanel()
                cf = conflictFrame(parent)
                cf.Show()
            else:
                showMessage("Control (autoRun): data conflicts detected.\n"+\
                            "Don't know yet what to do in this case, as I'm in automatic mode")
            Globals.GUI.control("CC", None)
            return #stop autopilot if there are conflicts

        nc = Status.mod.moduleCC.basicCheck(estimate=True)

        if nc > 0:
            checkOK = False
            if Status.UserInteractionLevel in ["semi-automatic","interactive"]:
                Status.mod.moduleCC.updatePanel()
                cf = conflictFrame(parent)
                cf.Show()
            else:
                showMessage("Control (autoRun): data conflicts detected.\n"+\
                            "Don't know yet what to do in this case, as I'm in automatic mode")
            Globals.GUI.control("CC", None)
            return #stop autopilot if there are conflicts
                   
        else:
            checkOK = True
            Globals.GUI.control("CC", None)

            showMessage("Control (autoRun): congratulations. data are consistent.\n"+\
                        "now let's continue calculating some energy balances")
            
            Status.prj.setActiveAlternative(0,checked = True)
            Status.mod.moduleEA.update()
            Globals.GUI.control("EA4a", None)

    else:
        logTrack("Control (autoRun): project already checked")

    Status.mod.moduleEA.update()
    Globals.GUI.control("EA4b", None)


    if Status.NoOfAlternatives >= 5:
            showMessage(_("You already have a lot of alternative proposals in your study"+\
                        "Delete some of them and then call me again ..."))
            return

#..............................................................................
# Benchmarking

    pass #for the future

#..............................................................................
# Now let's create some alternative proposals


    if "HR" in subset:
    #..............................................................................
    # Alternative proposal 1: Heat recovery only

        ret = askConfirmation("First let's test the remaining heat recovery potential (Alternative 1)\n"+\
                              "The result will be used as base for system optimisation")
        
        if ret == wx.ID_NO:
            ret = askConfirmation(_U("Do You want to interrupt the auto-design?"))
            if ret == wx.ID_YES:
                return

        shortName = "Heat recovery (HR)"
        description = "EINSTEIN default design of heat recovery system"
        basedOn = 0
            
        Status.prj.createNewAlternative(basedOn,shortName,description)

        if Status.HRTool != "estimate":
            Status.mod.moduleHR.runHRDesign()
        else:
            if Status.SimTool=="NP": Status.mod.moduleHR.runHRModule()

        # Finally check the boiler dimensioning for the remaining heat demand
        #    Status.mod.moduleBB.designAssistant()

        Status.mod.moduleEA.update()
        # First a dummy panel change to ensure a redraw of the second.
        Globals.GUI.control("EA4a", None) 
        Globals.GUI.control("EA4b", None)

        baseANo = Status.ANo

    else:
        baseANo = 0

    if "ST" in subset:

        #..............................................................................
        # Alternative proposal 2: Heat recovery + solar system

        ret = askConfirmation("Now let's try to install a solar system (Alternative 2)\n")

        if ret == wx.ID_NO:
            ret = askConfirmation(_U("Do You want to interrupt the auto-design?"))
            if ret == wx.ID_YES:
                return
        
        shortName = "Solar thermal"
        description = "EINSTEIN default design of a solar thermal system"
        basedOn = baseANo
            
        Status.prj.createNewAlternative(basedOn,shortName,description)

        Status.mod.moduleEnergy.updatePanel()
        Status.mod.moduleEnergy.runSimulation()

        Status.mod.moduleST.initPanel()
        Status.mod.moduleST.updatePanel()
        Status.int.GData.update({'ST Config':[50.,"any",300.0]})
        Status.mod.moduleST.setUserDefinedPars()
        Status.mod.moduleST.designAssistant1()
        
        Status.mod.moduleST.updatePanel()

        Status.mod.moduleBB.initPanel() #preparation sequence of HP Module
        Status.mod.moduleBB.updatePanel()

        Status.mod.moduleBB.designAssistant()
        Status.mod.moduleBB.updatePanel()

        Status.mod.moduleEA.update()
        Globals.GUI.control("ST", None)

    if "CHP" in subset:
        #..............................................................................
        # Alternative proposal 3: CHP

        shortName = "CHP"
        description = "EINSTEIN default design of a CHP system"

        ret = askConfirmation(_U("Now let's try to install a CHP system (Alternative %s)?\n") % shortName)
        
        if ret == wx.ID_NO:
            ret = askConfirmation(_U("Do You want to interrupt the auto-design?"))
            if ret == wx.ID_YES:
                return

        basedOn = baseANo
            
        Status.prj.createNewAlternative(basedOn,shortName,description)

        #updating heat recovery and corresponding demand calculations
        Status.mod.moduleEnergy.updatePanel()
        Status.mod.moduleEnergy.runSimulation()

        Status.mod.moduleCHP.initPanel() #preparation sequence of HP Module
        Status.mod.moduleCHP.updatePanel()

        (mode,CHPList) = Status.mod.moduleCHP.designAssistant1()

        if (mode == "MANUAL"):
            dbe = PanelDBCHP(parent,
                                   _U('Select CHP equipment from preselected list'), # title for the dialogs
                                   True,
                                   preselection = CHPList)      # database table can be edited in DBEditFrame?
            if dbe.ShowModal() == wx.ID_OK:
                CHPId = dbe.theId
            else:
                CHPId = -1

        elif (mode == "AUTOMATIC"):
            CHPId = CHPList[0]    #in automatic mode just take first in the list

        elif (mode == "CANCEL"):
            CHPId = -1 #make designAssistant2 to understand that
        else:
            CHPId = -1
            logTrack("Control (DesignAssistant CHP): erroneous panel mode: %s"%mode)
        
        Status.mod.moduleCHP.designAssistant2(CHPId)
        Status.mod.moduleCHP.updatePanel()

        # Finally check the boiler dimensioning for the remaining heat demand

        Status.mod.moduleBB.initPanel() #preparation sequence of HP Module
        Status.mod.moduleBB.updatePanel()

        Status.mod.moduleBB.designAssistant()
        Status.mod.moduleBB.updatePanel()

        Status.mod.moduleEA.update()
        Globals.GUI.control("CHP", None)

    if "HP" in subset:
        #..............................................................................
        # Alternative proposal 4: Heat recovery + heat pump

        shortName = "Heat pump"
        description = "EINSTEIN default design of a heat pump based system"
        
        ret = askConfirmation("Now let's try to install a heat pump (Alternative %s)\n"%shortName)
        
        if ret == wx.ID_NO:
            ret = askConfirmation(_U("Do You want to interrupt the auto-design?"))
            if ret == wx.ID_YES:
                return

        basedOn = baseANo
            
        Status.prj.createNewAlternative(basedOn,shortName,description)

        #updating heat recovery and corresponding demand calculations
        Status.mod.moduleEnergy.updatePanel()
        Status.mod.moduleEnergy.runSimulation()

        Status.mod.moduleHP.initPanel() #preparation sequence of HP Module
        Status.mod.moduleHP.updatePanel()

        (mode,HPList) = Status.mod.moduleHP.designAssistant1()

        if (mode == "MANUAL"):
            dbe = PanelDBHeatPump(parent,
                            _U('Select heat pump from preselected list'), # title for the dialogs
			    True,
                            preselection = HPList)      # database table can be edited in DBEditFrame?
            if dbe.ShowModal() == wx.ID_OK:
                HPId = dbe.theId
            else:
                HPId = -1

        elif (mode == "AUTOMATIC"):
            HPId = HPList[0]    #in automatic mode just take first in the list

        elif (mode == "CANCEL"):
            HPId = -1 #make designAssistant2 to understand that
        else:
            logDebug("Control (DesignAssistant-Button): erroneous panel mode: %s"%mode)
        
        Status.mod.moduleHP.designAssistant2(HPId)
        Status.mod.moduleHP.updatePanel()

        # Finally check the boiler dimensioning for the remaining heat demand

        Status.mod.moduleBB.initPanel() #preparation sequence of HP Module
        Status.mod.moduleBB.updatePanel()

        Status.mod.moduleBB.designAssistant()
        Status.mod.moduleBB.updatePanel()

        Status.mod.moduleEA.update()
        Globals.GUI.control("HP", None)

        HPANo = Status.ANo  # if HP design included in subset, use as starting point for Super-Mix
    else:
        HPAno = baseANo

    if "BB" in subset:
        #..............................................................................
        # Alternative proposal 5: New boiler cascade

        shortName = "Boiler cascade"
        description = "EINSTEIN default design of a new boiler cascade"

        ret = askConfirmation("Now let's try to install a new boiler cascade (Alternative %s)\n"%shortName)
        
        if ret == wx.ID_NO:
            ret = askConfirmation(_U("Do You want to interrupt the auto-design?"))
            if ret == wx.ID_YES:
                return

        shortName = "Boiler cascade"
        description = "EINSTEIN default design of a new boiler cascade"
        basedOn = baseANo
            
        Status.prj.createNewAlternative(basedOn,shortName,description)

        #updating heat recovery and corresponding demand calculations
        Status.mod.moduleEnergy.updatePanel()
        Status.mod.moduleEnergy.runSimulation()

        Status.mod.moduleBB.initPanel() #preparation sequence of HP Module
        Status.mod.moduleBB.updatePanel()

        Status.mod.moduleBB.designAssistant()
        Status.mod.moduleBB.updatePanel()

        # Finally check the boiler dimensioning for the remaining heat demand

        #    Status.mod.moduleBB.designAssistant()

        Status.mod.moduleEA.update()
        Globals.GUI.control("BB", None)

    if "MIX" in subset and False: # deactivate the supermix for now.
        #..............................................................................
        # Alternative proposal 6: And now lets mix up everything

        shortName = "EINSTEIN Super Mix"
        description = "EINSTEIN default design - combination of all technologies"

        ret = askConfirmation("Now let's try to combine everything (Alternative %s)\n"%shortName)
        
        if ret == wx.ID_NO:
            ret = askConfirmation(_U("Do You want to interrupt the auto-design?"))
            if ret == wx.ID_YES:
                return

        basedOn = HPANo
            
        Status.prj.createNewAlternative(basedOn,shortName,description)

        #updating heat recovery and corresponding demand calculations
        Status.mod.moduleEnergy.updatePanel()
        Status.mod.moduleEnergy.runSimulation()

        Status.mod.moduleCHP.initPanel() #preparation sequence of HP Module
        Status.mod.moduleCHP.updatePanel()

        (mode,CHPList) = Status.mod.moduleCHP.designAssistant1()
        if len(CHPList) > 0:  
            CHPId = CHPList[0]    #in automatic mode just take first in the list
            logMessage(_("Control (autoRun): %s possible CHP models found")%len(CHPList))
        else:
            CHPId = -1
            logMessage(_("Control (autoRun): no CHP application possible"))
        Status.mod.moduleCHP.designAssistant2(CHPId)
        Status.mod.moduleCHP.updatePanel()

        Status.mod.moduleST.initPanel()
        Status.mod.moduleST.updatePanel()
        Status.int.GData.update({'ST Config':[50.,"any",300.0]})
        Status.mod.moduleST.setUserDefinedPars()
        Status.mod.moduleST.designAssistant1()
        
        Status.mod.moduleST.updatePanel()

        Status.mod.moduleBB.initPanel() #preparation sequence of HP Module
        Status.mod.moduleBB.updatePanel()

        Status.mod.moduleBB.designAssistant()
        Status.mod.moduleBB.updatePanel()

        Status.mod.moduleEA.update()
        #    Status.main.tree.SelectItem(Status.main.qEA3, select=True)

#..............................................................................
# End of the journey

    Globals.GUI.control("CS1", None)
    
    showMessage("We arrived at the end of the journey\n"+\
                "Have a look on the results ...")
    

#============================================================================== 
