# -*- coding: utf-8 -*-
#==============================================================================
#
#   E I N S T E I N
#
#   Expert System for an Intelligent Supply of Thermal Energy in Industry
#   (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#   DataECO
#           
#------------------------------------------------------------------------------
#           
#   Manages data storage for the ECO-Module (Economic analysis)
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
from GUI.status import Status
from constants import EPSILON, getEquipmentClass
from GUI.GUITools import check
import math
from collections import defaultdict
from modules.messageLogger import logWarning
from auxiliary.auxiliary import toGettextAndUnicode as _U, toUnicode as U, toUTF8, noneCheck
import database.dbio as dbio
from modules.fluids import Fuel
from lib.auxiliary.float import toFloat
import numpy as np

class DataECO(object):
    """ Class contains all information of economic data for a given alternative
    """
        
    def __init__(self, pid, ano):
        self.pid = pid
        self.ano = ano
        self.tcaid = None   #will be set after generaldata is loaded
        self.ResultPresent = False

        self.investment = None
        self.energycosts = None
        self.totalopcost = None
    
    
    def __repr__(self):
        string = "DataECO(%s.%s)\n"%(self.pid, self.ano)
        string += "Investment: %s Energy %s OM %s\n"%(self.investment, self.energycosts, self.totalopcost)
        return string
    
##            return str((self.pid, self.ano, self.tcaid, self.ResultPresent, self.contingencies, self.CSDR, self.DEP, self.detailedopcost, self.display,
##                        self.energycost, self.energycosts, self.Inflation, self.investment, self.aaname, self.NIR, self.nonrecurring,
##                        self.revenue, self.revenues, self.TimeFrame, self.totalopcost))
##        except:
##            return str((self.pid, self.ano, self.tcaid, self.ResultPresent))
#--------------------------------------------------------------------------------------------
# Public Methodes
#--------------------------------------------------------------------------------------------                                 
    def loadTCAData(self, forceUpdate=False):
        """Load the TCA data from the database. If none exists create
        default values.
        
        :param forceUpdate: If True then throw away the old entries whoose
        default values depend on simulation results, or equipment data, and
        calculate new default values.
        
        """
        #General Data
        if (self.__getTCAGeneralData() == False):          
            self.__setTCAGeneralDataDefault()            
            self.__storeTCAGeneralDataEntry()
            if (self.__getTCAGeneralData() == False):
                print "TCA FALSE"
        #Investments
        if forceUpdate is True:
            self.__deleteTCAInvestmentDataEntry()
        if (self.__getTCAInvestmentData() == False):
            self.__setTCAInvestmentDataDefault()
            self.__storeTCAInvestmentDataEntry()
            self.__getTCAInvestmentData()
        #EnergyDemand
        if forceUpdate is True:
            self.__deleteTCAEnergyDataEntry()
        if (self.__getTCAEnergyData() == False):
            self.__setTCAEnergyDataDefault()
            self.__storeTCAEnergyDataEntry()
            self.__getTCAEnergyData()
        #Non reocuring costs
        if (self.__getTCANonData() == False):
            self.__setTCANonDataDefault()
            self.__storeTCANonDataEntry()
            self.__getTCANonData()
        #Contingencies
        if (self.__getTCAContingenciesData() == False):
            self.__setTCAContingenciesDataDefault()
            self.__storeTCAContingenciesDataEntry()
            self.__getTCAContingenciesData()
        #Detailed Operating Cost
        if forceUpdate is True:
            self.__deleteTCADetailedOpCostDataEntry()
        if (self.__getTCADetailedOpCostData() == False):
            self.__setTCADetailedOpCostDataDefault()
            self.__storeTCADetailedOpCostDataEntry()
            self.__getTCADetailedOpCostData()
        #Detailed Revenues  
        if (self.__getTCARevenueData() == False):
            self.__setTCARevenueDataDefault()
            self.__storeTCARevenueDataEntry()
            self.__getTCARevenueData()

    def storeTCAData(self):
        """Store the data in the database.
        
        The data of the current alternative proposal plus
        the general economic data for all is set to the current values.
          
        """
        self.__storeTCAGeneralDataEntry()
        
        self.__deleteTCAInvestmentDataEntry()  
        self.__deleteTCAEnergyDataEntry()
        self.__deleteTCANonDataEntry()
        self.__deleteTCAContingenciesDataEntry()
        self.__deleteTCADetailedOpCostDataEntry()
        self.__deleteTCARevenueDataEntry()
        
        self.__storeTCAInvestmentDataEntry()  
        self.__storeTCAEnergyDataEntry()
        self.__storeTCANonDataEntry()
        self.__storeTCAContingenciesDataEntry()
        self.__storeTCADetailedOpCostDataEntry() 
        self.__storeTCARevenueDataEntry()     
    
    def resetTCAData(self):
##        print "Reset TCA Data %s %s" % (self.pid, self.ano)     
        if (self.tcaid == None):            
            if (self.__getTCAGeneralData() == False):
                print "get failed"  
        if (self.tcaid != None):                   
            self.__deleteTCAInvestmentDataEntry()  
            self.__deleteTCAEnergyDataEntry()
            self.__deleteTCANonDataEntry()
            self.__deleteTCAContingenciesDataEntry()
            self.__deleteTCADetailedOpCostDataEntry()
            self.__deleteTCARevenueDataEntry()
            self.__deleteTCAGeneralDataEntry() 
        
                       
          
    def setResult(self, name, npv, mirr, bcr, annuity, paybackperiod, additionalcost, additionalcostpersavePE, display):
##        try:
        self.name = name
        self.npv = npv[:]
        self.mirr = mirr[:]
        self.bcr = bcr[:]
        self.annuity = annuity
        self.display = display
        self.TIC = self.cashflow.TotalInvestmentCapital
        self.EIC = self.cashflow.EffectiveInvestmentCapital  
        self.PP = paybackperiod 
        self.totalfunding = self.cashflow.TotalFundings
        self.energycost = self.getTotalEnergyCost()
        self.totalenergydemand = self.getTotalEnergyDemand()
        self.additionalcost = additionalcost
        self.additionalcostpersavePE = additionalcostpersavePE
        
        #TODO - calc this
        self.totalopcost = self.__gettotalopcost()
        
        self.energySystemCost = self.energycost + self.annuity + self.totalopcost
        self.ResultPresent = True

        costItems = self.getElectricityCostItems()                

        #update tariffs as result
        self.ElTariffCAvg = costItems["Purchase"][0]
        if costItems["Sales"][1] > 0: # update tariffs only if necessary
            self.ElTariffCHP = costItems["Sales"][0]
        else:
            self.ElTariffCHP = None

        if costItems["FeedIn"][1] > 0: # update tariffs only if necessary
            self.ElTariffFeedIn = costItems["FeedIn"][0]
        else:
            self.ElTariffFeedIn = None
        
        self.ElCostYearTot = costItems["Purchase"][1]
        self.ElSalesYearCHP = costItems["Sales"][1]
        self.ElSalesYearFeedIn = costItems["FeedIn"][1]
            
##        except Exception, inst:
##            print "TCAData.setResult:"
##            print type(inst)
##            print inst.args
##            print inst
    
    def setResultInvalid(self, name, display):
        self.name = name
        self.display = display
        self.ResultPresent = False
        
    def storePresentStateInformationToCGeneral(self):
        #Store function for the current process
        self.energycost = self.getTotalEnergyCost()
        try:
            self.energySystemCost = self.energycost + self.totalopcost  # investment/annuity in present state is 0
        except:
            self.energySystemCost = None
        generalData = Status.prj.getGeneralData(PId=self.pid, ANo=self.ano)
##        table = Status.DB.cgeneraldata.Questionnaire_id[self.pid].AlternativeProposalNo[self.ano]
        if generalData is not None:
            generalData.set(**{"EnergyCost":self.energycost,
                               "EnergySystemCost": self.energySystemCost,
                               "OMThermal":self.totalopcost})
            dbio.commit()        
        
    def storeResultToCGeneralData(self):

        #store function for proposals
        if self.ano <= 0:
            logWarning("TCA.storeResultToCGeneralData: cannot overwrite data in present state")
            return
        
        if (self.ResultPresent):
            mirr = self.mirr[-1]  #mirr at the final year 
            bcr = self.bcr[-1]
            if np.isinf(bcr) or np.isnan(bcr): bcr = None
            generalData = Status.prj.getGeneralData(PId=self.pid, ANo=self.ano)
            if generalData is not None:
                generalData.set(**{"TotalInvCost": toFloat(self.TIC),
                                 "OwnInvCost": toFloat(self.EIC),
                                 "Subsidies": toFloat(self.totalfunding),
                                 "RevenueReplaceEquipment": toFloat(self.revenue),
                                 "IRR": toFloat(mirr),
                                 "PayBack": toFloat(self.PP),
                                 "NPV": toFloat(self.npv[-1]),
                                 "BCR": toFloat(bcr),
                                 "EnergyCost": toFloat(self.energycost),
                                 "OMThermal": toFloat(self.totalopcost),
                                 "Amortization": toFloat(self.annuity),
                                 "EnergySystemCost": toFloat(self.energySystemCost),
                                 "AddCost": toFloat(self.additionalcost),
                                 "AddCostperSavedPE": toFloat(self.additionalcostpersavePE)})
                dbio.commit()
            
            #if electricity tariffs have changed, retransfer information back to table qelectricity
##            edata = Status.prj.getElectricity(Status.PId, self.ano)

            edata = Status.int.systemModel.elObject[0]
            edata.ElTariffCTot = self.ElTariffCAvg
            #TODO: this gives confusion for variable electricity tariff from data file
            
            if self.ElTariffCHP is not None: edata.ETariffCHP = self.ElTariffCHP
            if self.ElTariffFeedIn is not None: edata.ElTariffFeedIn = self.ElTariffFeedIn

            edata.ElCostYearTot = self.ElCostYearTot
            edata.ElSalesYearCHP = self.ElSalesYearCHP
            edata.ElSalesYearFeedIn = self.ElSalesYearFeedIn

            edata.saveToDB()

            #TODO: update of fuel tariff information missing !!!
            #TODO: correct treatment of inflation rate for energy tariffs missing !!!

            
            fuels = Status.prj.getFuels(Status.PId, self.ano)
            fuelDict = Status.prj.getFuelDict()
            for costItem in self.energycosts:
                for fuel in fuels:
                    fuelID = fuel.DBFuel_id
                    if fuelID in fuelDict.keys():
                        fuelName = fuelDict[fuelID]
                    else:
                        fuelName = None
                    
                    if toUTF8(costItem[0]) == fuelName:
                        fuel.FuelTariff = costItem[2]
                        fuel.FuelCostYear = costItem[1] * costItem[2]
                        break
                    
                        
            dbio.commit()
                    
        
#--------------------------------------------------------------------------------------------
# General Data
#--------------------------------------------------------------------------------------------     
    def __setTCAGeneralDataDefault(self):
        self.tcaid = None
        self.revenue = 0.0 # will be read in later
        if self.ano == 0: # get the defaults from cgeneraldata
            generalData = Status.prj.getGeneralData(PId=self.pid, ANo=self.ano)
            questionnaire = Status.prj.getQuestionnaire(PId=self.pid)
            if (generalData is not None):
                default = generalData
                #print "result form questionnaire"
                self.Inflation = default.InflationRate
                if self.Inflation is None:
                    self.Inflation = 0.02
                self.NIR = default.InterestExtFinancing
                if self.NIR is None:
                    self.NIR = 0.06
                self.CSDR = default.CompSpecificDiscountRate
                if self.CSDR is None:
                    self.CSDR = 0.03
                self.DEP = default.FuelPriceRate
                if self.DEP is None:
                    self.DEP = 0.03
                self.TimeFrame = default.AmortisationTime

                if self.TimeFrame is None:
                    self.TimeFrame = 20.0
                self.totalopcost = questionnaire.OMTotalTot
                if self.totalopcost is None:
                    self.totalopcost = 0.0
            # Fix up inconsistencies between questionnaire and tca
            self.Inflation = self.Inflation * 100
            self.NIR = self.NIR * 100
            self.CSDR = self.CSDR * 100
            self.DEP = self.DEP * 100           
        else: # an alternative proposal gets its defaults from the ANo 0 data
            try:
                data = DataECO(self.pid, 0)
                data.loadTCAData()
                self.Inflation = data.Inflation
                self.NIR = data.NIR
                self.CSDR = data.CSDR
                self.DEP = data.DEP
                self.TimeFrame = data.TimeFrame
                
                self.totalopcost = data.totalopcost
            except:
                logWarning("Erroneous economic data in alternative 0 (present state). Charging default values")
                self.Inflation = 2.0
                self.NIR = 6.0
                self.CSDR = 3.0
                self.DEP = 3.0
                self.TimeFrame = 20.0
                self.totalopcost = 0.0
        
    def __getTCAGeneralData(self):
        TCAGeneralData = dbio.get("tcageneraldata", ProjectID=self.pid, AlternativeProposalNo=self.ano)
        if (TCAGeneralData is None):
            return False
        else:
            self.tcaid = TCAGeneralData.id
            self.Inflation = TCAGeneralData.InflationRate
            self.NIR = TCAGeneralData.NominalInterestRate
            self.CSDR = TCAGeneralData.CompSpecificDiscountRate
            self.DEP = TCAGeneralData.FulePriceRate
            #TODO: correct orthographic errors and (optionally) use same naming in python and sql parameters for reducing confusion
            self.TimeFrame = TCAGeneralData.AmotisationTime

            self.totalopcost = TCAGeneralData.TotalOperatingCost
            self.revenue = TCAGeneralData.TotalRevenue
            # self.PETFuels = result[8]
            # self.PETel = result[9]
            return True
        
    def __storeTCAGeneralDataEntry(self):
        """Store the current TCAGeneral data in the database.
        
        Keep the amortisation time, InflationRate, 
        NominalInterestRate, CompSpecificDiscountRate and 
        FulePriceRate  in sync over all alternatives.
            
        """
        if (self.tcaid == None): # create new data set            
##            Status.DB.tcageneraldata.insert({"ProjectID": self.pid,
##                                             "AlternativeProposalNo": self.ano,
##                                             })
            dbio.insertRow("tcageneraldata", **{"ProjectID": self.pid,
                                             "AlternativeProposalNo": self.ano,
                                             })
            dbio.commit()        

        # update old data sets
        # update for all proposals
        for i in range(Status.NoOfAlternatives + 1):
            TCAGeneralData = dbio.get("tcageneraldata", ProjectID=self.pid, AlternativeProposalNo=i)
##            table = Status.DB.tcageneraldata.ProjectID[self.pid].AlternativeProposalNo[i]
            if TCAGeneralData is not None:
                TCAGeneralData.set(**{"AmotisationTime": dbio.check(self.TimeFrame),
                                 "InflationRate": dbio.check(self.Inflation),
                                 "NominalInterestRate": dbio.check(self.NIR),
                                 "CompSpecificDiscountRate": dbio.check(self.CSDR),
                                 "FulePriceRate": dbio.check(self.DEP)})
                dbio.commit()

        # Set for current alternative only - will be replaced after actual 
        # calculations via e.g. storeDetailedOpCost.
##        table = Status.DB.tcageneraldata.ProjectID[self.pid].AlternativeProposalNo[self.ano]
        TCAGeneralData = dbio.get("tcageneraldata", ProjectID=self.pid, AlternativeProposalNo=self.ano)
        if TCAGeneralData is not None:
            TCAGeneralData.set(**{"TotalOperatingCost": self.totalopcost,
                             "TotalRevenue": self.revenue})
            dbio.commit()        
        
    def __deleteTCAGeneralDataEntry(self):
        dbio.delete("tcageneraldata", ProjectID=self.pid, AlternativeProposalNo=self.ano)
        
#--------------------------------------------------------------------------------------------
# Investment
#--------------------------------------------------------------------------------------------  

    def __setTCAInvestmentDataDefault(self):
        #print "Set Default Data (Investment)"
        # Structure [Descritpion, Investment, Fundig%, FundingFix]
        #try:
        self.investment = []
        if (self.ano > 0):
            self.__setDefaultEquipmentInvestment()
            self.__setDefaultHXInvestment()
            self.__setDefaultStorageInvestment()
    
    def __setDefaultHXInvestment(self):
        total_hx_investment = 0.0
##        query = """SELECT TurnKeyPrice, OMFix, OMVar FROM qheatexchanger WHERE `ProjectID`=%s AND `AlternativeProposalNo`=%s"""
##        query = query % (self.pid, self.ano)
##        results = Status.DB.sql_query(query)
        hxes = dbio.select("qheatexchanger", ProjectID=self.pid, AlternativeProposalNo=self.ano)
        if len(hxes) > 0:
##            if (type(results[0]) != type(())):
##                results = [ results ]
##            
            for hx in hxes:
                Investment = hx.TurnKeyPrice
                if Investment is None:
                    Investment = 0
                total_hx_investment += Investment 
        self.investment.append(["HX Network", total_hx_investment, 30, 0])  
        
    def __setDefaultEquipmentInvestment(self):
        keys = []
        dict = {}

        equipment = dbio.select("qgenerationhc", Questionnaire_id=self.pid, AlternativeProposalNo=self.ano)
        for eq in equipment:
            EquipType = eq.EquipType
            Investment = eq.TurnKeyPrice
            InvestmentBasedOnPrice = eq.Price   
                               
            if (EquipType is not None):              
                if not(EquipType in keys):
                    keys.append(EquipType)
                    dict[EquipType] = 0
                if Investment == None:
                    if InvestmentBasedOnPrice != None:
                        Investment = InvestmentBasedOnPrice * 1.2
                        logWarning(_U("The Investment costs had to be estimated for %s "\
                                      "as there was neither a turn key price nor a "\
                                      "calculated price given.") % (eq.Equipment, ))
                    else:
                        Investment = 0
                dict[EquipType] += Investment 
                
        for key in keys:
            self.investment.append([key, dict[key], 30, 0])     
#        self.investment.append(["CHP gas turbine",0,30,0])                                      
        #except:        
        #    self.investment = []
        
    def __setDefaultStorageInvestment(self):
##        query = """SELECT `NumStorageUnits`,`VUnitStorage` FROM `qdistributionhc` WHERE `Questionnaire_id`=%s AND `AlternativeProposalNo` = %s"""
##        query = query % (self.pid, self.ano)
##        results = Status.DB.sql_query(query)  
##        if len(results) > 0:
##            if (type(results[0]) != type(())):
##                results = [ results ]
        pipes = dbio.select("qdistributionhc", Questionnaire_id=self.pid, AlternativeProposalNo=self.ano)
        #XXX Constants should come from database! 
        #fixed cepci bad.... 
        CEPCI1976 = 192.1
        CEPCI2008 = 539.7
        EUR_USD_ratio = 1.55
        m3_ft3_ratio = 0.0283
        ratio = (CEPCI2008 / CEPCI1976) / EUR_USD_ratio
        total_storage = 0.0
        for pipe in pipes:
            NumStorageUnits = pipe.NumStorageUnits
            VUnitStorage = pipe.VUnitStorage
            if not((NumStorageUnits is None) or (VUnitStorage is None) or (NumStorageUnits == 0) or (VUnitStorage == 0)):                     
                V = VUnitStorage / m3_ft3_ratio 
                Storage_cost = (352 * pow(V, 0.515)) * ratio
                Storage_cost *= NumStorageUnits
                total_storage += Storage_cost
        self.investment.append(["H&C Storage", total_storage, 30, 0])
                
    def __storeTCAInvestmentDataEntry(self):
        for investment in self.investment:
            tmp = {"TcaID":self.tcaid,
                   "ProjectID": self.pid,
                   "AlternativeProposalNo": self.ano,
                   "Description":check(investment[0]),
                   "Investment":investment[1],
                   "FundingPerc":investment[2],
                   "FundingFix":investment[3]}
            
            dbio.insertRow("tcainvestments", **tmp)
            dbio.commit()
                
    def __getTCAInvestmentData(self):
        keys = []
        dict = {}

        investments = dbio.select("tcainvestments", TcaID = self.tcaid)
        newinvestments = dbio.select("qgenerationhc", Questionnaire_id=self.pid, AlternativeProposalNo=self.ano)
        
        self.investment = []
        inv = []
        if (len(investments) == 0):        
            return False        
        else:
            for investment in investments:
                if investment.Investment > 0:
                    self.investment.append([unicode(investment.Description, "utf-8"), investment.Investment, investment.FundingPerc, investment.FundingFix])
                    inv.append(investment.Description)
            
            
            if len(newinvestments) > 0:
                
                for eq in newinvestments:
                    EquipType = eq.EquipType
                    Investment = eq.TurnKeyPrice
                    InvestmentBasedOnPrice = eq.Price   
                                       
                    if (EquipType is not None):              
                        if not(EquipType in keys):
                            keys.append(EquipType)
                            dict[EquipType] = 0
                        if Investment == None:
                            if InvestmentBasedOnPrice != None:
                                Investment = InvestmentBasedOnPrice * 1.2
                            else:
                                Investment = 0
                        dict[EquipType] += Investment 
            invl = []
            for elem in self.investment:
                invl.append(elem[0])

            for key in keys:
                if key not in invl:
                    if dict[key] > 0:
                        self.investment.append([key, dict[key], 30, 0]) 

            return True                                                   

    def __deleteTCAInvestmentDataEntry(self):
        dbio.delete("tcainvestments", TcaID = self.tcaid)

#--------------------------------------------------------------------------------------------
# Energy
#-------------------------------------------------------------------------------------------- 
    def __setTCAEnergyDataDefault(self):
        self.energycosts = []
        self.__setElectricityDefault()
        self.__setOtherFuelsDefault()
    
    def __setElectricityDefault(self):
        el = Status.int.systemModel.elObject[0]

## TODO: find a consistent procedure whether to do this here or in cEEB.
##        if self.ano > 0:
##            el.distributeSelfGen()
##            el.calculateAverageTariffs()

        # build effective tariff from term on consumption and power term
        # TODO: should be handled / presented separately in PanelECO in the future
    
        Tariff = el.ElTariffCTot
        if (el.ElTariffPowTot is not None) and (el.PowerContrTot is not None) and (el.ElectricityTotYear > EPSILON):
            Tariff += (el.ElTariffPowTot * el.PowerContrTot * 12) / el.ElectricityTotYear                      

        self.energycosts.append(["Electricity", el.ElectricityTotYear, Tariff, self.DEP])

        if el.ElGenera > EPSILON:
            self.energycosts.append(["Feedin-Tariff", -el.ElGenera, el.ElTariffFeedIn, self.DEP])
            self.energycosts.append(["Electricity (sales)", -el.ElSales, el.ElTariffCHP, self.DEP])
    
    def __setOtherFuelsDefault(self):
        qfuels = dbio.select("qfuel", Questionnaire_id=self.pid, AlternativeProposalNo=self.ano)
        for qfuel in qfuels:
            fuelId = qfuel.DBFuel_id
            fuel = Fuel(fuelId)
            Name = fuel.name
            FECFuel = qfuel.FETFuel
            if (FECFuel is None):
                FECFuel = 0.0
            FuelTariff = qfuel.FuelTariff
            if (FuelTariff is None):
                FuelTariff = 0.0
            self.energycosts.append([Name, FECFuel, FuelTariff, self.DEP])                                                         
                
    def __storeTCAEnergyDataEntry(self):
        for energycost in self.energycosts:
            tmp = {"TcaID": self.tcaid,
                   "ProjectID": self.pid,
                   "AlternativeProposalNo": self.ano,
                   "Description": check(energycost[0]),
                   "EnergyDemand": energycost[1],
                   "EnergyPrice": energycost[2],
                   "DevelopmentOfEnergyPrice": energycost[3]}

##            Status.DB.tcaenergy.insert(tmp)
            dbio.insertRow("tcaenergy", **tmp)
            dbio.commit()
            
    
    def getTotalEnergyCost(self):
        totalenergycost = 0.0
        for energycost in self.energycosts:
            if not any([c is None for c in energycost[1:3]]): # security feature. had become necessary in TC1.01 (V2.4)
                totalenergycost += energycost[1] * energycost[2]

        return totalenergycost

    def getElectricityCostItems(self):
        """ retreives the electricity cost items from TCA tables"""
        
        ElSales = 0
        ElPurchase = 0
        ElGenera = 0
        ElTariffCAvg = 0
        ElTariffFeedIn = 0
        ElTariffCHP = 0
        
        for energycost in self.energycosts:
            if energycost[0] == "Electricity":
                ElPurchase = energycost[1]
                ElTariffCAvg = energycost[2]
            elif energycost[0] == "Feedin-Tariff":
                ElGenera = energycost[1]
                ElTariffFeedIn = - energycost[2]
            elif energycost[0] == "Electricity (sales)":
                ElSales = energycost[1]
                ElTariffCHP = - energycost[2]

        costItems = {"Purchase": (ElTariffCAvg, ElTariffCAvg*noneCheck(ElPurchase, 0.0)),
                     "Sales":   (ElTariffCHP, ElTariffCHP*noneCheck(ElSales, 0.0)),
                     "FeedIn":  (ElTariffFeedIn, ElTariffFeedIn*noneCheck(ElGenera, 0.0))}

        return costItems

    def getTotalEnergyDemand(self):
        totalenergydemand = 0.0
        for energycost in self.energycosts:
            if energycost[1] is not None: totalenergydemand += energycost[1]
        return totalenergydemand
  
    def getTotalPrimaryEnergy(self):
        generalData = Status.prj.getGeneralData(self.ano)
        if generalData is not None:
            return generalData.PEC
        else:
            return None
  
    def __getTCAEnergyData(self):
        tcaEnergyData = dbio.select("tcaenergy", TcaID=self.tcaid)
        self.energycosts = []
        if (len(tcaEnergyData) == 0):        
            return False        
        else:
            for entry in tcaEnergyData:
                self.energycosts.append([unicode(entry.Description, "utf-8"), entry.EnergyDemand, entry.EnergyPrice, entry.DevelopmentOfEnergyPrice])            
            return True                                                   

    def __deleteTCAEnergyDataEntry(self):
##        query = """DELETE FROM tcaenergy
##                   WHERE TcaID = %s"""
##        query = query % (self.tcaid)
##        Status.DB.sql_query(query)
        dbio.delete("tcaenergy", TcaID=self.tcaid)

#--------------------------------------------------------------------------------------------
# Non recurring costs
#-------------------------------------------------------------------------------------------- 
    def __setTCANonDataDefault(self):
        self.nonrecurring = []
                
    def __storeTCANonDataEntry(self):
        for cost in self.nonrecurring:
            tmp = {"TcaID": self.tcaid,
                   "ProjectID": self.pid,
                   "AlternativeProposalNo": self.ano,
                   "Description": check(cost[0]),
                   "Value": cost[1],
                   "Year": cost[2],
                   "Type": cost[3]}
##            Status.DB.tcanonrecurring.insert(tmp)
##            Status.SQL.commit()
            dbio.insertRow("tcanonreoccuringcosts", **tmp)
            dbio.commit()
    
    def __getTCANonData(self):
##        query = """SELECT Description, Value, Year, Type
##                   FROM tcanonreoccuringcosts
##                   WHERE TcaID = %s"""
##        query = query % (self.tcaid)
##        results = Status.DB.sql_query(query)
        entries = dbio.select("tcanonreoccuringcosts", TcaID=self.tcaid)
        self.nonrecurring = []
        if (len(entries) == 0):        
            return False        
        else:
##            if (type(results[0]) != type(())):
##                results = [ results ]         #needed to have same behaviour if only one result
            for entry in entries:
                self.nonrecurring.append([unicode(entry.Description, "utf-8"), entry.Value, entry.Year, entry.Type])            
            return True                                                   

    def __deleteTCANonDataEntry(self):
##        query = """DELETE FROM tcanonreoccuringcosts
##                   WHERE TcaID = %s"""
##        query = query % (self.tcaid)
##        Status.DB.sql_query(query)
        dbio.delete("tcanonreoccuringcosts", TcaID=self.tcaid)

#--------------------------------------------------------------------------------------------
# Contingencies
#--------------------------------------------------------------------------------------------

    def __setTCAContingenciesDataDefault(self):
        self.contingencies = []
                
    def __storeTCAContingenciesDataEntry(self):
        for contingency in self.contingencies:
            tmp = {"TcaID": self.tcaid,
                   "ProjectID": self.pid,
                   "AlternativeProposalNo": self.ano,
                   "Description": check(contingency[0]),
                   "Value": contingency[1],
                   "TimeFrame": contingency[2]}
##            Status.DB.tcacontingencies.insert(tmp)
##            Status.SQL.commit()
            dbio.insertRow("tcacontingencies", **tmp)
            dbio.commit()
    
    def __getTCAContingenciesData(self):
##    query = """SELECT Description, Value, TimeFrame
##               FROM tcacontingencies
##               WHERE TcaID = %s"""
##    query = query % (self.tcaid)
##    results = Status.DB.sql_query(query)
        entries = dbio.select("tcacontingencies", TcaID=self.tcaid)
        self.contingencies = []
        if (len(entries) == 0):           
            return False        
        else:
##            if (type(results[0]) != type(())):
##                results = [ results ]         #needed to have same behaviour if only one result
            for entry in entries:                
                self.contingencies.append([U(entry.Description), entry.Value, entry.TimeFrame])            
            return True
                                                                

    def __deleteTCAContingenciesDataEntry(self):
##        query = """DELETE FROM tcacontingencies
##                   WHERE TcaID = %s"""
##        query = query % (self.tcaid)
##        Status.DB.sql_query(query)
        dbio.delete("tcacontingencies", TcaID=self.tcaid)
          
#--------------------------------------------------------------------------------------------
# Detailed operating cost
#--------------------------------------------------------------------------------------------

    def __setTCADetailedOpCostDataDefault(self):
        if self.ano == 0:
            self.__setDefaultOPCostCurrent()
        else:
            self.__setDefaultOPCostProposal()
        
            cost = 0
            for category in self.detailedopcost:
                for entry in category:
                    cost += entry[1]               
            self.totalopcost = cost
     
    def __setDefaultOPCostCurrent(self):
        """ gets the OM costs for the present state
        """
        
        self.detailedopcost = [[], [], [], [], [], [], [], []]
        generalData = Status.prj.getGeneralData(self.ano)
        if generalData is not None:
            self.totalopcost = generalData.OMThermal

        if self.totalopcost is None: self.totalopcost = 0.0
    
    def __setDefaultOPCostProposal(self):
        self.detailedopcost = [[], [], [], [], [], [], [], []] 
        self.totalopcost = 0.0
        self.__setDefaultEquipOpcost()
        self.__setDefaultHXOpcost()
    
    def __setDefaultHXOpcost(self):
        total_hx_opcost = 0.0
##        query = """SELECT TurnKeyPrice, OMFix, OMVar FROM qheatexchanger WHERE `ProjectID`=%s AND `AlternativeProposalNo`=%s"""
##        query = query % (self.pid, self.ano)
##        results = Status.DB.sql_query(query)
        hxes = dbio.select("qheatexchanger", ProjectID=self.pid, AlternativeProposalNo=self.ano)
        if len(hxes) > 0:
##            if (type(results[0]) != type(())):
##                results = [ results ]
            
            for hx in hxes:                
                OMVar = hx.OMVar
                OMFix = hx.OMFix
                if OMVar == None:
                    OMVar = 0
                if OMFix == None:
                    OMFix = 0
                total_hx_opcost += (OMFix + OMVar)        
        self.detailedopcost[0].append(["HX network", total_hx_opcost])               
        self.totalopcost += total_hx_opcost    
        
        
    def __setDefaultEquipOpcost(self):
        #                   page1                page2             page7
        # Structure [[[Desc,Value],[Desc,Value]],[...],[],[],[],[],[]]
        costitems = defaultdict(int)
##        equipments = (Status.DB.qgenerationhc.Questionnaire_id[self.pid].
##                      AlternativeProposalNo[self.ano])
        equipment = dbio.select("qgenerationhc", Questionnaire_id = self.pid, AlternativeProposalNo=self.ano)
        if len(equipment) > 0:
            for eq in equipment:
                EquipType = eq.EquipType
                OMVar = eq.OandMvar
                OMFix = eq.OandMfix
                QHXEq = eq.QHXEq
                USHj = eq.USHj
                                   
                if EquipType is not None:
                    eqClass = getEquipmentClass(EquipType)
                    try:
                        if eqClass == "BB": 
                            equipCosts = (OMVar * (USHj - QHXEq)) + OMFix
                        else:
                            equipCosts = OMVar*USHj + OMFix
                            
                    except TypeError: # one of the parameters is missing
                        equipCosts = 0
                    costitems[EquipType] += equipCosts
                
        for key, value in costitems.items():
            self.detailedopcost[0].append([key, value])
               
        self.totalopcost += sum(costitems.values())                                       

    def __gettotalopcost(self):
        if (self.ano <= 0): #dont calculate opcost from detailed-opcost for current process
            return self.totalopcost
        
        cost = 0
        for category in self.detailedopcost:
            for entry in category:
                cost += entry[1]
                
        return cost
                
    def __storeTCADetailedOpCostDataEntry(self):
        for i in range(0, 8): #7 categorys in detailed opcost
            for opcost in self.detailedopcost[i]:
                tmp = {"TcaID": self.tcaid,
                       "ProjectID": self.pid,
                       "AlternativeProposalNo": self.ano,
                       "Description": check(opcost[0]),
                       "Value": opcost[1],
                       "Category": i}
##                Status.DB.tcadetailedopcost.insert(tmp)
##                Status.SQL.commit()
                dbio.insertRow("tcadetailedopcost", **tmp)
                dbio.commit()
        
##        query = """UPDATE tcageneraldata 
##                SET TotalOperatingCost = %s
##                WHERE IDTca=%s"""
##        query = query % (self.totalopcost, self.tcaid)
##        Status.DB.sql_query(query)
        entry = dbio.get("tcageneraldata", id=self.tcaid)
        entry.TotalOperatingCost = self.totalopcost
        dbio.commit()

    def __getTCADetailedOpCostData(self):
        self.detailedopcost = [[], [], [], [], [], [], [], []] 
        for i in range(0, 8):
##            query = """SELECT Description, Value
##                       FROM tcadetailedopcost
##                       WHERE TcaID = %s AND Category = %s"""
##            query = query % (self.tcaid, i)
##            results = Status.DB.sql_query(query)
            entries = dbio.select("tcadetailedopcost", TcaID=self.tcaid, Category=i)
            if (len(entries) > 0):                           
##                if (type(results[0]) != type(())):
##                    results = [ results ]         #needed to have same behaviour if only one result
                for entry in entries:                
                    self.detailedopcost[i].append([U(entry.Description), entry.Value])           
        if all(item == [] for item in self.detailedopcost):
            return False                      
        return True

    def __deleteTCADetailedOpCostDataEntry(self):
##        query = """DELETE FROM tcadetailedopcost
##                   WHERE TcaID = %s"""
##        query = query % (self.tcaid)
##        Status.DB.sql_query(query)
        dbio.delete("tcadetailedopcost", TcaID=self.tcaid)

    def __deleteTCAOPCostDataEntry(self):
        """Delete the OPCosts from the database that are results of the
        simulation, opposed to the other categories that are pure
        user input.
        
        """
##        query = """DELETE FROM tcadetailedopcost
##                   WHERE TcaID = %s AND Category = 0"""
##        query = query % (self.tcaid)
##        Status.DB.sql_query(query)
        dbio.delete("tcadetailedopcost", TcaID=self.tcaid, Category=0)
        
#--------------------------------------------------------------------------------------------
# Revenue
#--------------------------------------------------------------------------------------------

    def __setTCARevenueDataDefault(self):
        self.revenues = [] 
                
    def __storeTCARevenueDataEntry(self):
        for revenue in self.revenues:
            tmp = {"TcaID": self.tcaid,
                   "ProjectID": self.pid,
                   "AlternativeProposalNo": self.ano,
                   "Description": revenue[0],
                   "InitialInvestment": revenue[1],
                   "DeprecationPeriod": revenue[2],
                   "RemainingPeriod": revenue[3]}
##            Status.DB.tcadetailedrevenue.insert(tmp)
##            Status.SQL.commit()
            dbio.insertRow("tcadetailedrevenue", **tmp)
            dbio.commit()
    
    def __getTCARevenueData(self):
##        query = """SELECT Description, InitialInvestment, DeprecationPeriod, RemainingPeriod
##                   FROM tcadetailedrevenue
##                   WHERE TcaID = %s"""
##        query = query % (self.tcaid)
##        results = Status.DB.sql_query(query)
        entries = dbio.select("tcadetailedrevenue", TcaID=self.tcaid)
        self.revenues = []
        if (len(entries) == 0):        
            return False        
        else:
##            if (type(results[0]) != type(())):
##                results = [ results ]         #needed to have same behaviour if only one result
            for entry in entries:
                self.revenues.append([unicode(entry.Description, "utf-8"), entry.InitialInvestment, entry.DeprecationPeriod, entry.RemainingPeriod])            
            return True          
                                                                

    def __deleteTCARevenueDataEntry(self):
##        query = """DELETE FROM tcadetailedrevenue
##                   WHERE TcaID = %s"""
##        query = query % (self.tcaid)
##        Status.DB.sql_query(query)
        dbio.delete("tcadetailedrevenue", TcaID=self.tcaid)
         
