# -*- coding: utf-8 -*-
#==============================================================================#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	ModuleEA - module that calls the energyStatistics functions without
#       displaying the panels (used for report generation)
#			
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
import wx
import numpy as np
from GUI.GUITools import check
from moduleEA1 import ModuleEA1
from moduleEA2 import ModuleEA2
from moduleEA3 import ModuleEA3
from moduleEA4 import ModuleEA4
from moduleEA5 import ModuleEA5
from GUI.status import *
from constants import getEquipmentClass
from modules.messageLogger import (logMessage, logTrack, logWarning,
                                            logDebug)
from auxiliary.auxiliary import toGettextAndUnicode as _U

class ModuleEA(object):
    def __init__(self):
        pass

    def update(self):
        """Update all the energyStatistics needed for the 
        energyStatisticsDisplay - called from PanelCC
        
        """
        logTrack("ModuleEA (update): StatusEnergy %s" % Status.StatusEnergy)
        if Status.StatusCC > 0 and Status.processData.outOfDate:
            Status.processData.createYearlyDemand()
            
        if Status.StatusEnergy == 0:
            if Status.ANo > 0:
                logMessage(_("EINSTEIN running system simulation for updating energy balances"))
                logMessage(_("This may take a while. please be patient ..."))
                wx.SafeYield()
                Status.mod.moduleEnergy.runSimulation()
            logMessage(_("EINSTEIN now updating annual energy balances"))
            self.calculateEquipmentEnergyBalances()

    def updateForReport(self):
        """Update all the energyStatistics needed for the report."""
        self.calculateAnnualResults()
        self.procHeatTemp()

    def procHeatTemp(self):
        """Report field PROCHEATTEMP: process heat demand by temperature 
        levels.
        
        """
        tableReport = [["T", "UPH", "UPHnet", "USH"]]
        for iT in range(Status.NT + 1):
            if iT < 81:
                temp = Status.TemperatureInterval * iT
                tableReport.append([temp,
                                    Status.int.QD_T[iT], # cols E-K 
                                    Status.int.QA_T[iT], # cols L-M are merged
                                    Status.int.QA_T[iT]]) # col P-Q
            elif iT == 81:
                logWarning("WARNING: present standard report is limited to 80 temperature intervals !!!")

        for iT in range(Status.NT + 1, 81):
            tableReport.append([" ",
                                " ",
                                " ",
                                " "])                            
        
        Status.int.setGraphicsData("PROCHEATTEMP", np.array(tableReport))

    def calculateAnnualResults(self):
        """Calculate the full statistics based on the results from the 
        consistency check.
        
        """
        ModuleEA1()
        ModuleEA2()
        ModuleEA3()
        ModuleEA4()
        ModuleEA5()

    def calculateEquipmentEnergyBalances(self):
        """Calculate the annual fuel balances and energy transferred
        by pipes from the results of system simulations and store 
        them within the database.

        Function has been shifted to ModuleEnergy.
        
        """
        logTrack("ModuleEA (calculateEquipmentEnergyBalances). Deprecated !")
        #TODO: eliminate this function once all calls have been redirected
        Status.mod.moduleEnergy.calculateEquipmentEnergyBalances()
