# -*- coding: utf-8 -*-
#==============================================================================#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	ModuleEA1- Final energy by fuels- Yearly data
#			
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
from sys import *
from math import *
from numpy import *
import wx

from auxiliary.auxiliary import *
from GUI.status import *
from modules.interfaces import *
from auxiliary.auxiliary import toGettextAndUnicode as _U

class ModuleEA1(object):

    def __init__(self, keys):
        self.keys = keys
        self.initModule()

#------------------------------------------------------------------------------
    def initModule(self):
#------------------------------------------------------------------------------

        PId = Status.PId
        ANo = Status.ANo

        Status.mod.moduleEA.update()    #checks if data in SQL are uptodate
                                        #and otherwise carries out necessary
                                        #calculations

#..............................................................................
#Check: Protection for missing data(Pid and ANo)in cgeneraldata
        
        projectData = Status.int.systemModel.projectData
        generalData = Status.int.systemModel.generalData

        if generalData is None:
        
            PEC = [0.0, 0.0]
            PECTotal = 0.0
            PECPercentage = [0.0,0.0]

            PET = [0.0, 0.0]
            PETTotal = 0.0
            PETPercentage = [0.0,0.0]

        else:
            PEC = [generalData.PECFuels, generalData.PECel]

            for i in range(len(PEC)):
                if PEC[i] is not None:
                    PEC[i] /= 1000.0        #convert to MWh
                else:
                    PEC[i] = 0.0

            PECTotal = PEC[0] + PEC[1]
            
            if PECTotal > 0.0 and PECTotal is not None: #SD: zeroDivision and None check
                PECPercentage = [PEC[0]*100.0/PECTotal, PEC[1]*100.0/PECTotal]
            else:
                PECPercentage = [0.0,0.0]

            PET = [generalData.PETFuels, generalData.PETel]
            
            for i in range(len(PET)):
                if PET[i] is not None:
                    PET[i] /= 1000.0        #convert to MWh
                else:
                    PET[i] = 0.0

            PETTotal = PET[0]+PET[1]
            
            if PETTotal > 0.0 and PETTotal is not None: #SD: zeroDivision and None check
                PETPercentage = [PET[0]*100.0/PETTotal, PET[1]*100.0/PETTotal]
            else:
                PETPercentage = [0.0,0.0]

#..............................................................
        #finish the table columns, add total, percentage
        Labels = [_U('Total fuels'),_U('Total electricity'),_U('Total (fuels + electricity)')]
        PEC.append(PECTotal)
        PET.append(PETTotal)

        suma = 0
        for i in PECPercentage:
            suma += i
        PECPercentage.append(suma)

        suma = 0
        for i in PETPercentage:
            suma += i
        PETPercentage.append(suma)
      
#.................................................................

        TableColumnList = [Labels,PEC,PECPercentage,PET,PETPercentage]

        matrix = transpose(TableColumnList)
        data = array(matrix)
        
        Status.int.setGraphicsData(self.keys[0], data)

        if Status.ANo == 0:
            Status.int.setGraphicsData("EA1_REPORT", data)
        elif Status.ANo == Status.FinalAlternative:
            Status.int.setGraphicsData("EA1_REPORT_F", data)

#------------------------------------------------------------------------------

#==============================================================================
