# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (http://www.iee-einstein.org/)
#
#------------------------------------------------------------------------------
#
#	ModuleEA2- Final energy (FEC/FET) - Yearly data
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

from sys import *
from math import *
from numpy import *

from auxiliary.auxiliary import *
from GUI.status import *
from modules.interfaces import *
from modules.fluids import *
from modules.messageLogger import *
from auxiliary.auxiliary import toGettextAndUnicode as _U

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
class ModuleEA2(object):
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
    def __init__(self, keys):
#------------------------------------------------------------------------------
#   instance of module is created at the beginning when GUI is built-up
#   only basic initialisation
#------------------------------------------------------------------------------
        self.keys = keys # keys for accessing the data in Interfaces
        logTrack("ModuleEA2 (init): starting")

    def initPanel(self):
#------------------------------------------------------------------------------
#   initialisation of the Panel before displaying.
#   activated when EA2 is selected on the tree
#------------------------------------------------------------------------------

        Status.mod.moduleEA.update()
        
#..............................................................................
#   get access to the info in SQL

        PId = Status.PId
        ANo = Status.ANo

        fuels = Status.prj.getQFuels()
        (projectData,generalData) = Status.prj.getProjectData()
        
        self.NFuels = len(fuels)

#..............................................................................
# Final energy consumption by fuels (data for panel EA2)

        TotalFEC = 0.0
        TotalFETi = 0.0
        FEC = []
        FETi = []
        fuelNames = []
            
        for fuel in fuels:  #sum all FECFuel/FETFuel for fuels used #SD 
            fuelID = fuel.DBFuel_id #SD

            f = Fuel(fuelID)   
            fuelName = f.name
            
            fuelNames.append(fuelName)
            
            try: dFEC = fuel.FECFuel/1000.0
            except: dFEC = 0.0
            
            TotalFEC += dFEC 
            FEC.append(dFEC)
            
            try: dFET = fuel.FETFuel/1000.0
            except: dFET = 0.0
            
            TotalFETi += dFET
            FETi.append(dFET)

        try: FECel = generalData.FECel / 1000.0
        except: FECel = 0.0
        
        try: FETel = generalData.FETel / 1000.0
        except: FETel = 0.0
        
        if FECel is not None:
            TotalFEC += FECel  #add FECel to TotalFEC
            FEC.append(FECel) #SD: check if OK here
        else:
            TotalFEC += 0.0
            FEC.append(0.0) #SD: check if OK here

        if FETel is not None:
            TotalFETi += FETel
            FETi.append(FETel) #SD: check if OK here
        else:
            TotalFETi += 0.0
            FETi.append(0.0) #SD: check if OK here
            
        fuelNames.append(_U("Electricity"))

        FECPercentage = []
        FETPercentage = []

        for i in range (self.NFuels + 1):
            if TotalFEC > 0.0: #SD avoid division by zero
                FECPercentage.append(FEC[i]*100.0/TotalFEC)
            else:
                FECPercentage.append(0.0)
                
            if TotalFETi > 0.0: #SD avoid division by zero
                FETPercentage.append(FETi[i]*100.0/TotalFETi)
            else:
                FETPercentage.append(0.0)

#.........................................................        
        #finish the table columns, add total, sum percentage
        fuelNames.append(_U('Total'))
        FEC.append(TotalFEC)
        FETi.append(TotalFETi)

        suma = 0
        for i in FECPercentage:
            suma += i
        FECPercentage.append(suma)

        suma = 0
        for i in FETPercentage:
            suma += i
        FETPercentage.append(suma)

#.........................................................

        TableColumnList = [fuelNames,FEC,FECPercentage,FETi,FETPercentage]
        matrix = transpose(TableColumnList)

        data = array(matrix)

        Status.int.setGraphicsData(self.keys[0], data)

        reportMatrix = []
        for i in range(len(matrix)-1):
            if i < 10:
                reportMatrix.append(matrix[i])
        for i in range(len(matrix)-1,10):
            reportMatrix.append([" "," "," "," "," "])
        reportMatrix.append(matrix[len(matrix)-1])

        reportData = array(reportMatrix)
        if Status.ANo == 0:
            Status.int.setGraphicsData("EA2_REPORT", reportData)
        elif Status.ANo == Status.FinalAlternative:
            Status.int.setGraphicsData("EA2_REPORT_F", reportData)
        
#------------------------------------------------------------------------------

#==============================================================================
