# -*- coding: utf-8 -*-
#==============================================================================#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	ModuleEA5- Energy intensity- Yearly data
#			
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
from GUI.status import *
import database.dbio as dbio
import numpy as np
from auxiliary.auxiliary import noneFilter, transpose
from auxiliary.auxiliary import toGettextAndUnicode as _U
from modules.messageLogger import *

class ModuleEA5(object):

    def __init__(self, keys):
        logTrack("ModuleEA5(__init__)")
        self.keys = keys

        self.initModule()

    def initModule(self):
        """module initialization"""
        Status.mod.moduleEA.update()
        
        self.cgeneraldata = Status.prj.getGeneralData()
        self.questionnaire = Status.prj.getQuestionnaire()
        self.qproduct = Status.prj.getProducts()

#Part 1: Energy Intensity

        TotalFEC = self.cgeneraldata.FEC   #FEC in [MWh]
        ElectFEC = self.cgeneraldata.FECel
        FuelsFEC = TotalFEC - ElectFEC
        PE_FEC = self.cgeneraldata.PEC #substitute later the conv. coef. from SetUp #pending put conv. coef. in status.py
            
        Turnover = self.questionnaire.Turnover #in [million euros]

        if (Turnover > 0) and not (Turnover==None):    
            self.cgeneraldata.FUEL_INT = (FuelsFEC)/(Turnover) #converted to [kWh/euro]
            self.cgeneraldata.EL_INT = (ElectFEC)/(Turnover) #converted to [kWh/euro]
            self.cgeneraldata.PE_INT = (PE_FEC)/(Turnover) #converted to [kWh/euro]
        else:
            self.cgeneraldata.FUEL_INT = None
            self.cgeneraldata.EL_INT = None
            self.cgeneraldata.PE_INT = None

        dbio.commit()

        EI_values = []
        EI_values.append(self.cgeneraldata.FUEL_INT)
        EI_values.append(self.cgeneraldata.EL_INT)
        EI_values.append(self.cgeneraldata.PE_INT)

        EI_labels = [_U('Fuels'), _U('Electricity'), _U('Total primary energy')]

#.........................................................................
#Part 2: SEC by product
        for row in self.qproduct:

            #calculate SEC for each product
            if (row.QProdYear > 0) and row.QProdYear is not None:    
                if row.FuelProd is not None: row.FUEL_SEC = (row.FuelProd) / row.QProdYear #converted to [kWh/pu]
                elif row.FUEL_SEC is None: row.FUEL_SEC = 0
                
                if row.ElProd is not None: row.EL_SEC = (row.ElProd) / row.QProdYear #converted to [kWh/pu]
                elif row.EL_SEC is None: row.EL_SEC = 0.
                
                if row.FuelProd is not None and row.ElProd is not None: row.PE_SEC = (1.1 * row.FuelProd + 3.0 * row.ElProd) / row.QProdYear #converted to [kWh/pu], fixed energy conv. coef. ->change this later
                elif row.PE_SEC is None: row.PE_SEC = 0.

                dbio.commit()

        SEC_values = []
        for row in self.qproduct:#SD
            SECPerProduct = [unicode(row.Product, "utf-8")]
            SECPerProduct.append(row.FUEL_SEC)
            SECPerProduct.append(row.EL_SEC)
            SECPerProduct.append(row.PE_SEC)

            SEC_values.append(noneFilter(SECPerProduct))

#...............................................................................
#Part 3: SEC by unit operation
#XXX Still missing...

#...............................................................................

        #
        # upper grid: Energy intensity by type
        #
        TableColumnList1 = [EI_labels, noneFilter(EI_values)]

        matrix1 = transpose(TableColumnList1)
        data1 = np.array(matrix1)
                          
        Status.int.setGraphicsData(self.keys[0], data1)

        #
        # lower grid: Energy consumption by product
        #
        data2 = np.array(SEC_values)
                          
        Status.int.setGraphicsData(self.keys[1], data2)
