# -*- coding: utf-8 -*-
#==============================================================================#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	ModuleEditCon - Calculations for connection editor
#			
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
from GUI.status import Status
from globals import logger

class ModuleEditCon(object):

    def __init__(self, key="EDITCON"):
        self.key=key

    def updatePanel(self):

        if not Status.int.systemModel or (Status.PId is None):
            return None
            
        Status.int.systemModel.build()
        sys = Status.int.systemModel
        
        data = {}
        data["Fuels"] = Status.prj.getFuels()
        data["Electricity"] = Status.prj.getElectricity()
        data["Equipment"] = sys.eqObject
        data["Pipes"] = sys.pipeObject
        data["Processes"] = sys.processObject
        data["WHEE"] = sys.whee
        projectData,generalData = Status.prj.getProjectData()
        data["General"] = generalData

        njH = sum(1*(eq.heating != False) for eq in sys.eqObject)
        nmH = sum(1*(pipe.heating!=False) for pipe in sys.pipeObject)
        nkH = sum(1*(proc.heating!=False) for proc in sys.processObject)
        nH = max(njH, nmH, nkH)
        data["jCool"] = nH
        

        HXIn = {}
        HXOut = {}
        nmax = [sys.NK, sys.NM, sys.NJ, sys.NN]
        for ikey, key in enumerate(["Proc", "Pipe", "Eq", "EE"]):
            n = nmax[ikey]
            
            HXIn.update({key: [[] for i in range(n)]})
            HXOut.update({key: [[] for i in range(n)]})
            QWHLink = getattr(sys, "QWH" + key + "Link")
            QHXLink = getattr(sys, "QHX" + key + "Link") if key!="EE" else []
            for link in QHXLink:
                HXIn[key][link[1]].append(link[0])

            for link in QWHLink:
                HXOut[key][link[0]].append(link[1])

        data["HXIn"] = HXIn
        data["HXOut"] = HXOut
        data["HX"] = Status.int.hx
        Status.int.GData[self.key] = data

        return data

