# -*- coding: utf-8 -*-
#==============================================================================
#
#   E I N S T E I N
#
#   Expert System for an Intelligent Supply of Thermal Energy in Industry
#   (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#   ModulePA1
#
#------------------------------------------------------------------------------
#
#   Backend module for PA1 (pinch analysis)
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================
from GUI.status import *
from constants import INFINITE, EPSILON
from globals import Globals, logger

def sortStreams():
    """
    classifies streams into source and sink
    and sorts by temperatures
    
    """
    # update energy data and fetch from systemModel
##    Status.int.systemModel.update()
    allStreams = Status.int.systemModel.streams


    # classify into hot and cold streams
    sinkStreams = [stream for stream in allStreams if stream.heating]
    sourceStreams = [stream for stream in allStreams if not stream.heating]

    # sort by temperature
    sinkStreams   =  sorted(sinkStreams, key=lambda stream: stream.T0)
    sourceStreams =  sorted(sourceStreams, key=lambda stream: stream.T1, reverse=True)
        
    return sourceStreams, sinkStreams

class ModulePA1(object):
    def __init__(self, keys):
        self.keys = keys

    def initPanel(self):
        pass
        
    def updatePanel(self):
        """Update the cached data according to showHeating"""
        Status.int.systemModel.update()
        if Status.SimTool=="NP" and Status.StatusEnergy==0 and Status.ANo==0:
            logger.logWarning(_("Warning: Pinch analysis in present state does not include waste heat from equipment. No simulation results available"))
        self.sourceStreams, self.sinkStreams = sortStreams()

        self.sourceStreams = [stream for stream in self.sourceStreams if stream.Q > EPSILON]
        self.sinkStreams   = [stream for stream in self.sinkStreams   if stream.Q > EPSILON]

        sourceTable = []
        sinkTable = []

        sourceUnit2 = []
        sinkUnit2 = []
        
        for streamType, streams in enumerate([self.sourceStreams, self.sinkStreams]):

##            streams = [stream for stream in streams if stream.Q > EPSILON]

            for s, stream in enumerate(streams):

                if streamType == 0:
                    startTemp = stream.T1
                    endTemp = stream.T0
                else:
                    startTemp = stream.T0
                    endTemp = stream.T1

                try:
                    diffT = stream.T1 - stream.T0
                except:
                    diffT = 0.0
                    
                if diffT > 0:
                    avgHeatTransferCoeff = stream.dotQavg/diffT if stream.dotQavg is not None else None
                else:
                    avgHeatTransferCoeff = INFINITE

##                if stream.heating:
##                    streamType = _("sink")
##                else:
##                    streamType = _("source")

                try:
                    fluid = stream.hf.fluid
                    fluidName = fluid.name
                except:
                    fluidName = "-"

                try:
                    fluid = stream.hf.fluid
                    if stream.hours==0:
                        mdotAvg=0.0
                    else:
                        mdotAvg = stream.mcp / (fluid.cp*stream.hours) if stream.mcp is not None else None
                except:
                    mdotAvg = None

                try:
                    name = stream.origin.objectName + ":" + stream.origin.streamName
                except:
                    name = stream.name

                row = [name,
##                       _("sink") if streamType else _("source"),
                       "x" if stream.pinchMask else "-",
                       stream.Q, # yearly energy
                       stream.dotQavg, # average power
                       stream.hours,
                       startTemp,
                       endTemp,
                       avgHeatTransferCoeff,
                       fluidName,
                       mdotAvg]

                try:
                    funit2 = stream.fluid.rho
                except:
                    funit2 = 1.0

                if streamType == 0:
                    sourceTable.append(row)
                    sourceUnit2.append(funit2)
                else:
                    sinkTable.append(row)
                    sinkUnit2.append(funit2)

##        choices = [None, ["x", "-"], None, None, None,
##                   None, None,       None, None, None]
        choices = None

        data = {"HSTable": {"data": sourceTable,
##                            "choices": [choices]*len(sourceTable),
                            "choices": None, 
                            "unit2": sourceUnit2},
                "CSTable": {"data": sinkTable,
##                            "choices": [choices]*len(sinkTable),
                            "choices": None, 
                            "unit2": sinkUnit2}}

        return data

    def changeDataHSTable(self, index, mode, data=None):
        if mode==-1 and len(data)>=2 and data[1] in ["-", "x"]:
            self.sourceStreams[index].pinchMask = (data[1]=="x")
        elif mode==-2:
            self.sourceStreams[index].pinchMask = not self.sourceStreams[index].pinchMask

        panelData = self.updatePanel()
        return 0, panelData["HSTable"]["data"], panelData["HSTable"]["choices"]
    
    def changeDataCSTable(self, index, mode, data=None):
        if mode==-1 and len(data)>=2 and data[1] in ["-", "x"]:
            self.sinkStreams[index].pinchMask = (data[1]=="x")

        elif mode==-2:
            self.sinkStreams[index].pinchMask = not self.sinkStreams[index].pinchMask
        panelData = self.updatePanel()
        return 0, panelData["CSTable"]["data"], panelData["CSTable"]["choices"]
    

            

            
        
                
