# -*- coding: utf-8 -*-
#==============================================================================#
#   E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#   modulePA2
#           
#------------------------------------------------------------------------------
#
#
#           
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------     
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
from GUI.status import *
from modules.messageLogger import logTrack, logDebug, logWarning
from modules.heatRecovery.pinchAnalysis import PinchAnalysis

SHOWENERGY = True           #Controls wether energy or power is shown in pinch graphics

TIPS = {"TPinchH":  _("Pinch temperature at hot side of HX"),
        "TPinchC":  _("Pinch temperature at cold side of HX"),
        "QXTarget": _("Possible heat recovery in an ideal HX network with given DTmin"),
        "QCTarget": _("Minimum external heating requirement for an ideal HX network with given DTmin"),
        "QHTarget": _("Minimum external cooling / dissipation requirement for an ideal HX network with given DTmin")}
                
class ModulePA2(object):
    def __init__(self, keys):
        self.keys = keys
        self.initModule()

    def initModule(self):
        self.DTmin = Status.mod.moduleHR.DTmin

    def updatePanel(self):
        """Update the cached data according to showHeating"""

        
        pa = PinchAnalysis()
        pa.calculate(self.DTmin)
        pa.exportData()


        pinchData = Status.int.GData["PA2"]
        pinchInfo = pinchData[3]

        self.TPinchC = pinchInfo["TPinchC"]
        self.TPinchH = pinchInfo["TPinchH"]
        self.QXTarget = pinchInfo["QHXmax"]
        self.QCTarget = pinchInfo["QextC"]
        self.QHTarget = pinchInfo["QextH"]
        

        data = {}
        data["DTmin"] = pinchInfo["DTmin"]
        data["EnergyOrPower"] = 0
        data["Results"] = self.getTableData_Results()



        cccPlot = pinchData[0]
        hccPlot = pinchData[1]
        gccPlot = pinchData[2]
                    
                                    
        data["CC"] = {"xData": [hccPlot.X, cccPlot.X],
                      "yData": [hccPlot.Y, cccPlot.Y],
                      "labels": ["HCC", "CCC"],
##                      "lineTypes": lineTypesEff,
                      "colours": ["r", "b"]}
##                      "lineWidths": lineWidthsEff, "markerSizes": markerSizesEff}
        data["GCC"] = {"xData": [gccPlot.X],
                      "yData": [gccPlot.Y],
                      "labels": ["GCC"],
##                      "lineTypes": lineTypesEff,
                      "colours": ["g"]}
##                      "lineWidths": lineWidthsEff, "markerSizes": markerSizesEff}

            
        return data

    def setOptions(self, DTmin):
        if DTmin is not None and DTmin > 0:
            self.DTmin = DTmin
            Status.mod.moduleHR.DTmin = self.DTmin

    def getTableData_Results(self):
        # collect data for results table
        resultsData = {}
        table = []
        choices = []

        # formatting information for table
        subTitles = []
        editable = []
##        funit2Info = []
        pointers = {}
        parCodes = []
        
        # Pinch temperatures
        subTitles.append(len(table))
        pointers.update({"TPinch": len(table)})

        table.append([_("Pinch temperatures"), ("text", None, None), None])
        choices.append([None]*3)
        editable.append([False])
        parCodes.append([None])
        
        table.append([_("Pinch temperature (hot streams)"), ("float", "TEMPERATURE", 2), self.TPinchH])
        choices.append([None]*3)
        editable.append([False])
        parCodes.append(["TPinchH"])
            
        table.append([_("Pinch temperature (cold streams)"), ("float", "TEMPERATURE", 2), self.TPinchC])
        choices.append([None]*3)
        editable.append([False])
        parCodes.append(["TPinchC"])
            

        # Energy
        subTitles.append(len(table))
        pointers.update({"Energy": len(table)})

        table.append([_("Energy targets"), ("text", None, None), None])
        choices.append([None]*3)
        editable.append([False])
        parCodes.append([None])
        
        table.append([_("Maximum possible heat exchange"), ("float", "ENERGY", -1), self.QXTarget])
        choices.append([None]*3)
        editable.append([False])
        parCodes.append(["QXTarget"])
            
        table.append([_("Minimum required external heat"), ("float", "ENERGY", -1), self.QHTarget])
        choices.append([None]*3)
        editable.append([False])
        parCodes.append(["QHTarget"])
            
        table.append([_("Minimum required heat dissipation"), ("float", "ENERGY", -1), self.QCTarget])
        choices.append([None]*3)
        editable.append([False])
        parCodes.append(["QCTarget"])

        toolTips = []
        for row in parCodes:
            toolTips.append([None, None]+[TIPS[par] if par in TIPS.keys() else None for par in row])
            
        return {"data": table,
                           "choices": choices,
                           "pointers": pointers,
                           "parCodes": parCodes,
                           "toolTips": toolTips}


        
if __name__ == "__main__":
    print "Hello World";
