#==============================================================================#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	ModuleECOPanel - Hourly energy data
#			
#==============================================================================
#
#------------------------------------------------------------------------------		
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	www.energyxperts.net / info@energyxperts.net
#
#	All rights reserved
#
#============================================================================== 
import numpy as np
from constants import EPSILON, OMCATEGORIES
from globals import logger
from GUI.status import Status

from modules.schedules.scheduleTools import BasicSchedule

OBJECTKEYS = ["investment", "energycosts", "opCost", "nonrecurring", "contingencies"]
COLNO = [4, 4, 3, 4, 3] # number of columns in the object rows

##_U('If you think there would be revenue from sale of replaced equipments, please indicate the possible value')
##_U('Revenue from sale of replaced equipments')
##_U('The values of investment are suggested by the program.(The default funding for equipment is 30%) \nIf this do not apply to your process, please indicate your own values')
##_U('<enter custom description or choose from list>')
##_U('Total investment and fundings of the new equipments')
##_U('The values come from the audit questionnaire\nIf you would like to make any changes, please edit them')
##_U('The value is the total of the operating and maintenance costs given in the questionaire. If you would like \nto change this value, please go back to the questionnaire module'),
##_U("The values come from the audit questionnaire and Einstein database.\nIf you would like to make any change on them, please edit the values."))
##_U("If you would like to calculate the operating costs in details, please choose \"Detailed operating cost calculation\""))
##_U('Do you expect future costs for perpetuating use of the current energy source in the process?')
##_U('time frame (in X years from now, X=0 for current year)')
##_U('year =  indicate when presumably occuring in years after investment')

class ModuleECOPanel(object):
    def __init__(self, panel, key):
        self.panel = panel
        self.key = key
        self.eco = Status.mod.moduleECO
        Status.mod.moduleEA.update()

        self.deleted = []

        self.selection = None

    def initPanel(self):
        if Status.StatusECO == 1:
            logger.showWarning(_U("The equipment has changed since your last work on TCA "\
                           "data. Your data for contingencies and non "\
                           "reocurring costs may need modification."))
            Status.prj.setStatus("StatusECO", 2)

        return None

    def updatePanel(self, showHeating=True):
        """Update the cached data according to showHeating"""

##        print "ModuleECOPanel: calling moduleECO.updatePanel --------------------"
        self.eco.updatePanel()
##        print "ModuleECOPanel: calling moduleECO.updatePanel. DONE =============="

        if self.key == "ECO1":
            self.eco.calculateTotalOpCostFromDetailedOpcost()
            self.getOpCost()
            
            tableInvestment, selection = self.getTable("investment")
            tableEnergyCost, selection = self.getTable("energycosts")
            tableOM, selection = self.getTable("opCost")
            tableContingencies, selection = self.getTable("contingencies")
            tableNonRecurring, selection = self.getTable("nonrecurring")

    ##        choicesOM = [[None, OMCATEGORIES, None] for row in tableOM]

            data = {"Investment":       {"data": tableInvestment},
                    "EnergyCost":       {"data": tableEnergyCost},
                    "OM":               {"data": tableOM},
                    "Contingencies":    {"data": tableContingencies},
                    "NonRecurring":     {"data": tableNonRecurring}}

        elif self.key=="ECO":
            self.reference = self.eco.runPresentState()
            if Status.ANo > 0:
                self.current = self.eco.run(Status.ANo)
            else:
                self.current = self.reference

            table, plot = self.getResults()
            data = {"Results": {"data": table},
                    "Plot": plot}

##        print data

        return data
                    
    def changeData(self, index, mode, data=None):
        """
        this fucntion is called from the TableEntry event handler
        """

        if self.key=="ECO": # the table in the ECO panel is read only
            return 1, None, None
                    
        objectKey = OBJECTKEYS[self.panel.activePage]
        obj = getattr(self.eco.data, objectKey)

        if objectKey=="investment" and Status.ANo==0:
            logger.showWarning("There are no investments for the present state")
            return 1, None, None
        
        err = False
        nDataSets = len(obj)

        if (index >= nDataSets or index < -1) and mode != 0: # invalid index
            return 1, None, None

        if mode == 0:     # add
            if objectKey in OBJECTKEYS:
                self.selection = [nDataSets]

                ocode = OBJECTKEYS.index(objectKey)
                colNo = COLNO[ocode]
                newRow = [None]*colNo
                if objectKey=="investment":
                    newRow[2] = 0.0
                    newRow[3] = 0.0
                obj.append([None]*colNo)
        
        elif mode == 1:   # delete
            self.deleted.append(obj[index])
            obj.pop(index)

            self.selection = None
        
        elif mode == -1:  # data changed
            newData = self.readLine(data, objectKey)
            if newData is None:
                logger.showWarning("Erroneous data. please revise your entry")
            else:
                obj[index] = newData
                self.selection = [index]

            if objectKey=="opCost": self.eco.calculateTotalOpCostFromDetailedOpcost()

        table, selection = self.getTable(objectKey)
        choices = None

        return 0, table, choices, selection

    def getTable(self, objectKey):
        """
        Gets the project table to be displayed
        """

        obj = getattr(self.eco.data, objectKey)
        
##        print "ModuleECOPanel.getTable: obj = %s"%obj
        table = []

        total1 = 0.0
        total2 = 0.0 
        for row in obj:
            if objectKey=="investment":
##                print "investment row = %s"%row
                name, investment, fundingperc, fundingfix = row
                fundingRatio = None if fundingperc is None else fundingperc/100.0
                if investment is not None: total1 += investment
                try:
                    own = investment * (1 - fundingperc/100.) - fundingfix
                    total2 += own
                except:
                    own = None
                    
                table.append([name, investment, fundingRatio, fundingfix, own])
                
            elif objectKey=="energycosts":
##                print "energycosts row = %s"%row
                name, consumption, tariff, rate = row
                if consumption is not None: total1 += consumption
                try:
                    cost = tariff * consumption
                    total2 += cost
                except:
                    cost = None
                table.append([name, consumption, tariff, cost])
                
            elif objectKey=="opCost":
##                print "row = %s"%row
                name, cost, icat = row
                if cost is not None: total1 += cost
                table.append([name, icat, cost])

            elif objectKey=="nonrecurring":
##                print "row = %s"%row
                name, cost, year, ctype = row
                if cost is not None: total1 += cost
                table.append([name, cost, year, ctype])

            elif objectKey=="contingencies":
##                print "row = %s"%row
                name, cost, year = row
                if cost is not None: total1 += cost
                table.append([name, cost, year])

        if objectKey=="investment":
            table.append([_("Total"), total1, None, None, total2])
            
        elif objectKey=="energycosts":
            table.append([_("Total"), total1, None, total2])
            
        elif objectKey=="opCost":
##            print "OpCostTable before: %s"%table
            table.append([_("Total"), None, total1])
##            print "OpCostTable after: %s"%table

        elif objectKey=="nonrecurring":
            table.append([_("Total"), total1, None, None])

        elif objectKey=="contingencies":
            table.append([_("Total"), total1, None])

        return table, self.selection

    def getResults(self):

        titles = [_("Total investment"), _("Own investment"), _("Benefit cost ratio"), _("Payback period (years)"), _("MIRR at final year")]
        unitSelectors = [("float", "CURRENCY", 0), ("float", "CURRENCY", 0), ("float", "FRACTION", 2), ("float", "LONGTIME", 2), ("float", "PERCENTAGE", 3)]

        tableData = [self.current.TIC, self.current.EIC, self.current.bcr[-1], self.current.PP, self.current.mirr[-1]]        

        # PLOT (TabPage2)

        xData = [range(len(self.current.npv))]
        yData = [np.array(self.current.cashflow.CF()) - np.array(self.reference.cashflow.CF()), self.current.npv]
        labels = [_("Net cash flow (savings)"), _("NPV")]
        plot = {"xData": xData, "yData": yData, "labels": labels}
  
        table = []
        for r, title in enumerate(titles):      
            table.append([titles[r], unitSelectors[r], tableData[r]])

        return table, plot

    def readLine(self, line, objectKey):
        """ read in a line of data and converts into the format as required in the dataECO-object
        """
        if objectKey=="investment":
            name, investment, fundingRatio, fundingfix, owncost = tuple(line)
            
            fundingperc = None if fundingRatio is None else 100.0 * fundingRatio
##            if (investment<0) or (fundingperc<0) or (fundingperc>100) or (fundingfix<0) or fundingfix > investment:
##                return None
##            
##            else:
            return [name, investment, fundingperc, fundingfix]

        elif objectKey=="energycosts":
            return line[:4]
            
        elif objectKey=="opCost":
            return  [line[0], line[2], line[1]]

        elif objectKey=="nonrecurring":
            return line

        elif objectKey=="contingencies":
            return line

        
    def saveToDB(self, data=None):
##        print "ModuleECOPanel.saveToDB. data = %s"%data
        if self.key=="ECO1":
            self.setOpCost()
            
        self.eco.storeData()
        return True

# helper functions for translating the opcost entries into sub-categories used in the old ECO module

    def getOpCost(self):
        self.eco.data.opCost = []
        for i, item in enumerate(self.eco.data.detailedopcost):
            self.eco.data.opCost += [elem + [i] for elem in item]
    
    def setOpCost(self):
        for i, category in enumerate(OMCATEGORIES):
            self.eco.data.detailedopcost[i] = [item[:-1] for item in self.eco.data.opCost if item[-1]==i]

