#==============================================================================#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	ModuleEH - Hourly energy data
#			
#==============================================================================
#
#------------------------------------------------------------------------------		
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	www.energyxperts.net / info@energyxperts.net
#
#	All rights reserved
#
#============================================================================== 
import numpy as np
from GUI.status import Status
from constants import EPSILON

from modules.schedules.scheduleTools import BasicSchedule

class ModuleEH(object):
    def __init__(self, keys, showHeating=True):
        self.keys = keys
        self._showHeating = showHeating
            

    def initPanel(self):
        return None

    def updatePanel(self, showHeating=True):
        """Update the cached data according to showHeating"""
        self._showHeating = showHeating

        # In this grid the nr. of cols is variable, so we generate the
        # column headings dynamically here
        systemModel = Status.int.systemModel
        systemModel.build()
        systemModel.update()  # assure that the system model is up to date

        xData = [np.linspace(1, 8760, 8760)]

        if self.keys=="EH1":

            yData = []

            if Status.SimTool == "Plus":  
                UPX_t = np.zeros(Status.Nt)
                UPXwReq_t = np.zeros(Status.Nt)

                for k, proc in enumerate(systemModel.processObject):
                    Status.mod.moduleEnergy.sim.systemModel.process[k].getDemand(UPX_t, UPXwReq_t)
                    yData.append(UPX_t.copy())

            labels = [proc.name for proc in systemModel.processObject]

        else:
            equipment = systemModel.eqObject

            labels = [eq.name for eq in equipment]
            yData = [Status.int.USHj_t[j] for j, eq in enumerate(equipment)]
            nonZero = [np.sum(row) > EPSILON for row in yData]

            yData = [row for i, row in enumerate(yData) if nonZero[i]]
            labels = [elem for i, elem in enumerate(labels) if nonZero[i]]

            el = systemModel.elObject[0]
            if hasattr(el, "ElGen_t") and np.sum(el.ElGen_t) > EPSILON:
                yData.append(el.ElGen_t)
                labels.append(_("power generation"))
        

        # convert into rectangles
        # TODO: could be integrated as a standard option into xy-plot
        yS = [rect(row, xData[0])  for row in yData]
        if len(yData) > 0:
            xS = [rect(yData[0], xData[0], axis="x")]
            data = {"Plot": {"xData": xS, "yData": yS, "labels": labels}}
        else:
            data = None

        return data
                    

def rect(f, t, axis="y"):
    bs = BasicSchedule(f=f, t=t, step=True, tmin=0)
    bsp = bs.plot()
    print "Rect: f = %s"%bsp.f[:8]
    if axis == "y":
        return np.array(bsp.f)
    else:
        return np.array(bsp.t)

