#==============================================================================#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	ModuleEditCon - Calculations for connection editor
#			
#==============================================================================
#
#	Version No.: 0.01
#	
#------------------------------------------------------------------------------		
#	(C) copyleft energyXperts.BCN (E4-Experts SL), Barcelona, Spain 2011
#	www.energyxperts.net / info@energyxperts.net
#
#	This program is free software: you can redistribute it or modify it under
#	the terms of the GNU general public license as published by the Free
#	Software Foundation (www.gnu.org).
#
#============================================================================== 
from GUI.status import Status
from modules.ccheck.checkMassFlow import MFSystem, Flow, Node


class ModuleEditMF(object):

    def __init__(self, key="EDITMF"):
        self.key=key

    def updatePanel(self):
        if not Status.int.systemModel:
            return None

        Status.int.systemModel.build()
        sys = Status.int.systemModel

        self.sysMF = MFSystem()
        
        return self.exportData()

    def buildConnection(self, startNode, startNo, endNode, endNo):
        self.sysMF.flow.append(Flow(startNode, startNo, endNode, endNo))
        self.sysMF.flow[-1].exportToNode()

        self.sysMF.saveToDB()

        self.exportData()

    def deleteConnection(self, n):
        self.sysMF.flow[n].exportToNode(delete=True)
        self.sysMF.deletedFlows.append(self.sysMF.flow.pop(n))
        self.sysMF.saveToDB()

        self.exportData()

    def changeConnection(self, n, mode):
        ret = self.sysMF.flow[n].setMode(mode)
        if ret:
            self.sysMF.saveToDB()
            self.exportData()
        return ret

    def insertNode(self):
        intNodes = len([node for node in self.sysMF.node if node.objectType in ["RAW", "INT"]])
        self.sysMF.node.append(Node(None, objectType="INT", name="Node %s"%(intNodes+1), objectId=None, autoLoad=False))
        self.exportData()
        return self.sysMF.node[-1]

    def deleteNode(self, node):
        # first delete all connections to this node
        for q in node.qin + node.qout:
            if q is not None:
                n = self.sysMF.flow.index(q)
                self.deleteConnection(n)

        # then delete the node itself
        index = self.sysMF.node.index(node)
        self.sysMF.deletedNodes.append(self.sysMF.node.pop(index))
        
        self.sysMF.saveToDB()
        self.exportData()

    def editNode(self):
        pass

    def saveNode(self, node, nodeData):

        if node.objectType in ["RAW", "INT"]:
            node.nodeType = nodeData.nodeType
            node.name = nodeData.name

            # update objectType corresponding to nodeType
            if node.nodeType == "SRC":
                node.objectType="RAW"
            else:
                node.objectType="INT"

            node.scaleFactor = nodeData.scaleFactor
            node.splitFraction = nodeData.splitFraction

            nInputs = len(nodeData.scaleFactor)
            nOutputs = len(nodeData.splitFraction)

            nInCons = 0
            for i, qin in enumerate(node.qin):
                if qin is not None:
                    nInCons = i
                    
            nOutCons = 0
            for i, qout in enumerate(node.qout):
                if qout is not None:
                    nOutCons = i
                    
            if nInputs >= nInCons: # maintain or extend number of inputs
                node.scaleFactor = nodeData.scaleFactor
                node.qin = node.qin[:nInCons] + [None]*(nInputs-nInCons)
                node.qinVal = node.qinVal[:nInCons] + [None]*(nInputs-nInCons)
            else: # do not eliminate connected inputs but assign scale factor 0
                node.scaleFactor[:nInputs] = nodeData.scaleFactor
                node.scaleFactor[nInputs:] = [0]*(nInCons-nInputs)
                
            if nOutputs >= nOutCons: # maintain or extend number of inputs
                node.splitFraction = nodeData.splitFraction
                node.qout = node.qout[:nOutCons] + [None]*(nOutputs-nOutCons)
                node.qoutVal = node.qoutVal[:nOutCons] + [None]*(nOutputs-nOutCons)
            else: # do not eliminate connected outputs but assign split fraction 0
                node.splitFraction[:nOutputs] = nodeData.splitFraction
                node.scaleFactor[nOutputs:] = [0]*(nOutCons-nOutputs)
                
            node.updateNodeData()

            self.sysMF.saveToDB()

            self.exportData()
            
    def exportData(self):
        data = {"Nodes": self.sysMF.node,
                "Flows": self.sysMF.flow}
        Status.int.GData[self.key] = data
        return data

