# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
#==============================================================================
#
#      E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#      ModulePeriodEditor
#            
#------------------------------------------------------------------------------
#            
#      Data table for data entry
#
#==============================================================================
#       (C) energyXperts.NET (E4-Experts GmbH, Berlin, Germany)
#============================================================================== 

from datetime import datetime

#==============================================================================
class ModulePeriodEditor():
#==============================================================================
#==============================================================================

    def __init__(self):
        """
        :titles     list of titles for each column (-> defines the number of columns)
        """
        self.periods = []

    def changeData(self, index, mode):
        """
        modifies the period table and calculates
        default data when adding / deleting a new line
        """

        nPeriods = len(self.periods)
        print "change data index %s [%s] mode %s"%(index, nPeriods, mode)

        if index > nPeriods or index < 0:
            return
        po = self.periods[index] if index < nPeriods else None


        if mode == 0:     # insert above
            self.periods.insert(index, PeriodData())
            nPeriods += 1
            if index+1 < nPeriods:
                self.periods[index].profiles = self.periods[index+1].profiles

            # assign default values
                self.periods[index].stop = self.periods[index+1].start
            if index > 0:
                self.periods[index].start = self.periods[index-1].stop

        elif mode == 1:   # insert below
            if nPeriods == 0:
                index = -1
                
            self.periods.insert(index+1, PeriodData())
            nPeriods += 1
            if index >= 0:
                self.periods[index+1].profiles = self.periods[index].profiles

            # assign default values
                self.periods[index+1].start = self.periods[index].stop
            if index + 2 < len(self.periods):
                self.periods[index+1].stop = self.periods[index+2].start
            
        elif mode == 2:   # period split
            if po is None: return
            pnew = po.copy()
            pnew.stop = po.start
            self.periods.insert(index, pnew)

            self.periods[index+1].start = pnew.stop

        elif mode == 3:
            self.periods.pop(index)

        return 0, self.getTable()


    def getTable(self):
        """ transforms the information contained in the periods into a table of strings for
            representation on the GUI
        """
        table = []
        for p in self.periods:
            startString = dateToString(p.start)
            stopString = dateToString(p.stop)
            scaleString = "%6.4f"%p.scale
            profileString = ";".join([profile.name for profile in p.profiles])

            table.append([startString, stopString, scaleString, profileString])
        print table
        return table

    def readLine(self, s, index=None):
        start = stringToDate(s[0])
        stop = stringToDate(s[1])
        scale = float(s[2])
        profileNames = profileString.split(";")

        #### TODO Checks
        #### start >= 1/1 and stop <= 31/12; scale >=0; profileNames in list and not empty

        return [start, stop, scale, profileNames]


def dateToString(date):
    return date.strftime("%d-%m")
        

class PeriodData():

    def __init__(self):
        self.start = datetime(2007, 1, 1)
        self.stop = datetime(2007, 12, 31)
        self.scale = 1.0
        self.profiles = ["myProfile"]

    def copy(self):
        new = PeriodData()
        new.start = self.start
        new.stop = self.stop
        new.scale = self.scale
        new.profiles = self.profiles[:]
        return new
