# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (<a href="http://www.einstein-energy.net/" target="_blank">www.einstein-energy.net</a>)
#
#------------------------------------------------------------------------------
#
#	ModuleQ1: general data entry
#
#==============================================================================
#
#       EINSTEIN Version No.: 3.0
#
#------------------------------------------------------------------------------
#	(C) energyXperts.NET (E4-Experts GmbH), Berlin, Germany
#	http://www.energyxperts.net/
#
#==============================================================================

from constants import PRODUCTCODES, TRANSPRODUCTCODES
from globals import logger, Globals
from GUI.status import Status
from GUI.conflictFrame import conflictFrame

import database.dbio as dbio
from auxiliary.auxiliary import noneCheck, findKey
from auxiliary.dataTransfer import DataTransfer
from lib.auxiliary.float import toFloat

from model.objects.generalDataObject import QuestionnaireObject, ProjectStatusObject, ProductObject

# keys for parameters which are 1:1 passed to data
# structure: key_in_procObject: (key_in_Panel (or None, if identical), dictionary)

DATAKEYS_QUESTIONNAIRE = {"Name": (None, None),
            "City": (None, None),
            "DescripIndustry": (None, None),
            "Branch": (None, "NACE"),
            "SubBranch": (None, "NACESub"),
            "Contact": (None, None),
            "Role": (None, None),
            "Address": (None, None),
            "Phone": (None, None),
            "Fax": (None, None),
            "Email": (None, None),
            "NEmployees": (None, None),
            "Turnover": (None, None),
            "ProdCost": (None, None),
            "BaseYear": (None, None),
            "Growth": (None, None),
            "Independent": (None, None),
            "OMThermal": (None, None),
            "OMElectrical": (None, None),
            "PercentFuelTotcost": (None, None),
            "PercentElTotcost": (None, None),
            "HPerDayInd": (None, None),
            "NShifts": (None, None),
            "NDaysInd": (None, None),
            "NoProdStart_1": (None, None),
            "NoProdStop_1": (None, None),
            "NoProdStart_2": (None, None),
            "NoProdStop_2": (None, None),
            "NoProdStart_3": (None, None),
            "NoProdStop_3": (None, None)}



DATAKEYS_PROJECTSTATUS = {"ProjectName": (None, None),
                   "CreationDate": (None, None),
                   "CreatedBy": (None, "Auditor"),
                   "LastModified": (None, None),
                   "ModifiedBy": (None, "Auditor")}


###TRANSPRODUCTCODES
productDict = dict([(code, i) for i, code in enumerate(PRODUCTCODES)])
DATAKEYS_PRODUCT={"Product": (None, None),
                  "ProductCode": (None, productDict),
                    "QProdYear": (None, None),
                    "ProdUnit": (None, None),
                    "TurnoverProd": (None, None),
                    "ElProd": (None, None),
                    "FuelProd": (None, None)}


####### TODO: FOR THE MOMENT HERE. TO BE PLACED IN ITS OWN FILE ... ###
class ModuleQ1(object):
    """

    Attributes:

    
    """
    def __init__(self, panel):
        self.panel = panel


####        self.Entry.SetValue(projectStatus.ProjectName)
####
####        self.Entry.Enable()
####        self.creationDateEntry.SetValue(projectStatus.CreationDate)
####        self.creationDateEntry.Disable()
####
####        self.Entry.Enable()
####        self.createdByEntry.SetValue()
####        self.createdByEntry.Disable()
####
####        self.lastModifiedEntry.Enable()        
####        self.lastModifiedEntry.SetValue(projectStatus.LastModified)
####        self.lastModifiedEntry.Disable()
####
####        self.modifiedByEntry.Enable()
####        self.modifiedByEntry.SetValue(Status.prj.getAuditorName(projectStatus.ModifiedBy))
####        self.modifiedByEntry.Disable()

        
    def initPanel(self):
        """  functions that have to be carried out only once
        when the panel is created
        """
        q = Status.prj.getQuestionnaire()        
        self.questionnaireSQL = [q]     # Generic object needs a container for saveToDB function !!! 
        self.questionnaire = QuestionnaireObject(self, id=q.id, no=0, sqlData=q)

        projectStatus = Status.prj.getProjectStatus()
        self.projectStatusSQL = [projectStatus]
        self.projectStatus = ProjectStatusObject(self, id=projectStatus.id, no=0, sqlData=projectStatus)

        self.productList = Status.prj.getProducts()
        self.productObject = []
        for no, p in enumerate(self.productList):
            self.productObject.append(ProductObject(self, id=p.id, no=no, sqlData=p))
        self.deletedProducts = []

        # dictionary for dynamic translations
        self.dynamicTrans = {}

        NACEList, NACESubList, NACEText, NACESubText = self.fillChoiceOfNaceCode()
        self.NACEText = NACEText
        self.dynamicTrans["NACE"] = dict([(code, i) for i, code in enumerate(NACEText)])
        self.dynamicTrans["NACESub"] = dict([(code, i) for i, code in enumerate(NACESubText)])
        self.dynamicTrans["Auditor"] = dict([(a.id, a.Name) for a in dbio.select("auditor")])

        data = {"NACEChoices": NACEList,
                "NACESubChoices": NACESubList}
        return data
    
    def updatePanel(self):
        """Functions to be carried out each time when information
            to be displayed on the panel has changed
        """
        data = self.getDisplayData()       # fetches data to be directly displayed in entries


        # export parameters for PanelCC
####        dataGroup = ("Pipe[" + str(self.pipe.no+1) + "]")
####        data["cc"] = (self.ccPipe, dataGroup)

        return data

####    def setPipeType(self, index):
####        """
####        index: index within PIPETYPES
####        """
####        self.pipeTypePrevious = self.pipe.pipeType
####        
####        if index >= 0 and index < len(PIPETYPES):
####            self.pipe.setPipeType(PIPETYPES[index])

####    def changeFluidId(self, index):
####        fluidId = findKey(self.dynamicTrans["Fluid"], index)
####        self.pipe.fluidId = fluidId
####        self.pipe.updateFluid()
####        funit2 = self.pipe.fluid.rho if self.pipe.fluid is not None else 1.0
####        return funit2

    def readPanel(self, data={}):

        # collects here all the parameters from the panel (exclunding that in tables) which can be stored directly or via translation dictionary
        # note: parameters displayed in tables have already been updated in proc directly when changed

        for key in DATAKEYS_QUESTIONNAIRE.keys():
            if key in data.keys():
                val = data[key]

                trans = DATAKEYS_QUESTIONNAIRE[key][1]
                trans = self.dynamicTrans[trans] if isinstance(trans, str) else trans
                if trans is None:
                    setattr(self.questionnaire, key, dbio.check(val))
                else:
                    setattr(self.questionnaire, key, dbio.check(findKey(trans, val)))

        #TODO: reactivate INTEntry in panel as a cleaner solution for this problem
        nEmployees = None if self.questionnaire.NEmployees is None else int(self.questionnaire.NEmployees)
        self.questionnaire.NEmployees = nEmployees
        baseYear = None if self.questionnaire.BaseYear is None else int(self.questionnaire.BaseYear)
        self.questionnaire.BaseYear = baseYear


        for key in DATAKEYS_PROJECTSTATUS.keys():
            if key in data.keys():
                val = data[key]

                trans = DATAKEYS_PROJECTSTATUS[key][1]
                trans = self.dynamicTrans[trans] if isinstance(trans, str) else trans
                if trans is None:
                    setattr(self.projectStatus, key, dbio.check(val))
                else:
                    setattr(self.projectStatus, key, dbio.check(findKey(trans, val)))
        
    def saveToDB(self, data={}):
        """ saves the data received from the panel to the data base
        """
        if not Status.prj.editAllowed() or Status.ANo >=0:
            self.errMsgSave = _("Editing of this alternative is not allowed")
            return False

        self.readPanel(data)
        newProjectName = self.projectStatus.ProjectName
        
        if len(newProjectName) <= 0:
            self.errMsgSave = _("Please specify a valid project name")
            return False
                
        if len(newProjectName) > 45:
            newProjectName = newProjectName[:45]
            logger.logWarning(_("Project name is too long. Truncated to 45 characters !"))
            self.projectStatus.ProjectName = newProjectName

        if newProjectName != Status.ActiveProjectName: # project name changed
            existingProjects = Status.prj.getProjectsWithName(newProjectName)

            if len(existingProjects) > 0:
                self.errMsgSave = _("Project name has to be a unique value!")
                return False
            else:
                Status.ActiveProjectName = newProjectName
                Globals.GUI.panelInfo.update()

######        # read schedule parameters from panel        
######        conflicts = Status.mod.moduleCC.pipeCheck(self.generalData)
######        self.generalData.cc = Status.mod.moduleCC.ccLocal
######        Status.mod.moduleCC.initPanel()
######        if conflicts > 0:
######            cf = conflictFrame(Globals.GUI.editPanel)
######            cf.Show()
######            return False

        # auto complete some mandatory data
        if Status.ANo >= 0 and not (self.questionnaire.checkCompleteness() and self.projectStatus.checkCompleteness()):
            self.errMsgSave = _("Some mandatory data are missing")
            return False

        self.questionnaire.saveToDB()
        self.projectStatus.saveToDB()
        for product in self.productObject:
            product.saveToDB()

        for product in self.deletedProducts:
            product.deleteFromDB()
        self.deletedProducts = []

        Status.prj.projectChanged()
        Status.int.systemModel.build()

        self.errMsgSave = None
        self.new = False
        return True


    def getDisplayData(self):
        """
        translates the data in self.questionnaire into a data dicitionary as
        required by the panel
        """
        
        # read in table questionnaire
        q = self.questionnaire
        
        data = {}
        for dataKey in DATAKEYS_QUESTIONNAIRE:
            val = getattr(q, dataKey)
            key = noneCheck(DATAKEYS_QUESTIONNAIRE[dataKey][0], dataKey)
            trans = DATAKEYS_QUESTIONNAIRE[dataKey][1]
            trans = self.dynamicTrans[trans] if isinstance(trans, str) else trans
            if trans is None:
                data[key] = val
            else:
                data[key] = trans.get(val)

        # read in table sproject
        q = self.projectStatus
        
        for dataKey in DATAKEYS_PROJECTSTATUS:
            val = getattr(q, dataKey)
            key = noneCheck(DATAKEYS_PROJECTSTATUS[dataKey][0], dataKey)
            trans = DATAKEYS_PROJECTSTATUS[dataKey][1]
            trans = self.dynamicTrans[trans] if isinstance(trans, str) else trans
            if trans is None:
                data[key] = val
            else:
                data[key] = trans.get(val)      

        # read in productData
        productTable = self.getTable()
        data.update({"ProductTable": {"data": productTable}})        

        return data
    
    def onEntry(self, code, val):
        if code=="Branch":
            if val is None:
                return []
            else:
                branch = self.NACEText[val]
                self.questionnaire.Branch = branch
                naceList, naceSubList, naceText, naceSubText = self.fillChoiceOfNaceCode(mainCode=False)
                self.dynamicTrans["NACESub"] = dict([(code, i) for i, code in enumerate(naceSubText)])
                return naceSubList            


    def fillChoiceOfNaceCode(self, mainCode=True):
        naceDict, naceSubDict = Status.prj.getNACEDict(self.questionnaire.Branch)

        naceCodes = naceDict.keys()
        naceCodes.sort()
        naceText = [naceDict[code] for code in naceCodes]
        naceList = ["%s|%s"%(code, naceDict[code]) for code in naceCodes]

        naceSubCodes = naceSubDict.keys()
        naceSubCodes.sort()
        naceSubText = [naceSubDict[code] for code in naceSubCodes]
        naceSubList = ["%s|%s"%(code, naceSubDict[code]) for code in naceSubCodes]   

        return naceList, naceSubList, naceText, naceSubText

    def changeDataProductTable(self, index, mode, data=None):
        """
        modifies the period table and calculates
        default data when adding / deleting a new line
        or modifying data in the table

        this fucntion is called from the TableEntry event handler
        """

        err = False
        nDataSets = len(self.productObject)

        if Status.ANo >= 0: logger.logWarning(_("Products cannot be modified in checked state"))
        
        if index is None:
            if mode != 0:
                return 1, None, None
        else:
            if index > nDataSets or index < -1: # invalid index
                return 1, None, None
            elif index <= 0 and mode==1:
                return 1, None, None

        if mode == 0:     # add
            self.productObject.append(ProductObject(self, no=len(self.productObject)))
            
        elif mode == 1:   # delete
            no = index
            deleted = self.productObject.pop(index)
            self.deletedProducts.append(deleted)

        elif mode == -1:  # data changed
            no = index
            self.readLine(data, self.productObject[index])

        data = self.getTable()
        choices = None
        selection = None

        return 0, data, choices, selection

    def getTable(self):
        """
        Gets the project table to be displayed
        """
        productTable = [[p.Product,
                         PRODUCTCODES.index(p.ProductCode) if p.ProductCode in PRODUCTCODES else None,
                         p.QProdYear, p.ProdUnit, p.TurnoverProd, p.ElProd, p.FuelProd] for p in self.productObject]
        return productTable

    def readLine(self, data, p):
        """ reads a line of raw data from the table in the GUI
        and translates it into internal code
        (inverse of getTable)
        """

        p.Product, codeIndex, p.QProdYear, p.ProdUnit, p.TurnoverProd, p.ElProd, p.FuelProd = data
        p.ProductCode = None if codeIndex is None else PRODUCTCODES[codeIndex]

