# -*- coding: utf-8 -*-
#==============================================================================#
#   E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#   moduleA
#           
#------------------------------------------------------------------------------
#
#   Background module for data entry panel A (alternative proposal table)
#           
#==============================================================================
#
#   EINSTEIN Version No.: 3.0
#   Created by:     Hans Schweiger
#
#------------------------------------------------------------------------------     
#   (C) copyleft energyXperts.BCN (E4-Experts SL), Barcelona, Spain 2014
#   www.energyxperts.net / info@energyxperts.net
#
#   This program is free software: you can redistribute it or modify it under
#   the terms of the GNU general public license as published by the Free
#   Software Foundation (www.gnu.org).
#
#============================================================================== 
from GUI.status import Status

from globals import logger, Globals
from auxiliary.auxiliary import noneCheck
import numpy as np

from GUI.conflictFrame import conflictFrame

rowTitlesTC = [_("TOTAL"), _('Self-generation'), _('Sales to grid'), _(" "), _('Peak'), _('Standard'), _('Valley')]
parCodesTC = [[None,  "ElectricityTotYear",   "PowerContrTot", "ElTariffClassTot", "ElTariffPowTot", "ElTariffCTot", "ElCostYearTot"],
            [None,  "ElGenera",   None, None, None, "ElTariffFeedIn", "ElSalesYearFeedIn"],
            [None,  "ElSales",   None, "ElTariffClassCHP", "ElTariffPowCHP", "ElTariffCHP", "ElSalesYearCHP"],
            [None]*7,
            [None,  "ElectricityPeakYear",   "PowerContrPeak", "ElTariffClassPeak", "ElTariffPowPeak", "ElTariffCPeak", "ElCostYearPeak"],
            [None,  "ElectricityStandYear",   "PowerContrStd", "ElTariffClassStd", "ElTariffPowStd", "ElTariffCStd", "ElCostYearStd"],
            [None,  "ElectricityValleyYear",   "PowerContrVall", "ElTariffClassTotVall", "ElTariffPowVall", "ElTariffCVall", "ElCostYearVall"]]

parCodesCByUse = [[None, "ElectricityRef", None, "ElectricityMotors"],
                  [None, "ElectricityAC", None, "ElectricityChem"],
                  [None, "ElectricityThOther", None, "ElectricityLight"]]

rowTitlesCByUse = [_('Refrigeration'), _('Air Conditioning'), _('Other uses'), _('Motor and machines'), _('Electro-chemics'), _('Lighting')]


class ModuleQ2_el(object):
    """

    Attributes:

    
    """
    def __init__(self, panel):
        self.keys = []
        self.panel = panel
        
        self.initPanel()

        
    def initPanel(self):
        """  functions that have to be carried out only once
        when the panel is created
        """
        self.obj = Status.int.systemModel.elObject[0]

        self.updatePanel()
    
#------------------------------------------------------------------------------
    def updatePanel(self):
#------------------------------------------------------------------------------
#       Here all the information should be prepared so that it can be plotted on the panel
#------------------------------------------------------------------------------

        self.emixId = self.obj.dbelectricitymix_id
        self.emix = self.obj.name

        self.dataFile = self.obj.DataFile
        
        dataTC, dataCByUse = self.getTables()

        data = {"TC": {"data": dataTC},
                "emix": self.dataFile if self.dataFile else self.emix,
                "CByUse": {"data": dataCByUse}}

        return data


    def getTables(self):
        """
        Gets the tables to be displayed
        """                          

        nrows = 7
        ncols = 7
        tableTC = []
        for i in range(nrows):
            tableTC.append([rowTitlesTC[i]])
            for j in range(1, ncols):
                par = parCodesTC[i][j]
                val = None if (par is None or self.obj is None) else getattr(self.obj, par)
                val = None if val in ["None", "none"] else val # backward compatibility where some Nones were stored as text
                tableTC[i].append(val)
                
        tableTC[1][3] = _("feed-in tariff")

        nrows = 3
        ncols = 4
        tableCByUse = []
        for i in range(nrows):
            tableCByUse.append([rowTitlesTC[i]])
            for j in range(1, ncols):
                par = parCodesCByUse[i][j]
                tableCByUse[i].append(None if (par is None or self.obj is None) else getattr(self.obj, par))

        for i in range(3):
            tableCByUse[i][0] = rowTitlesCByUse[i]
            tableCByUse[i][2] = rowTitlesCByUse[3+i]

        return tableTC, tableCByUse

    def changeDataTC(self, index, mode, data=None):
        """
        modifies the period table and calculates
        default data when adding / deleting a new line
        or modifying data in the table

        this fucntion is called from the TableEntry event handler
        """

        err = False
        if index < 0 or index > 6: return
        if mode == -1:   # change of data in a dataSet
            row = data
            for j, parCode in enumerate(parCodesTC[index]):
                if parCode is not None:
                    setattr(self.obj, parCode, row[j])
                                
        tableData, dummy = self.getTables()
        choices = None
        
        return 0, tableData, choices

    def changeDataCByUse(self, index, mode, data=None):
        """
        modifies the period table and calculates
        default data when adding / deleting a new line
        or modifying data in the table

        this fucntion is called from the TableEntry event handler
        """

        err = False
        if index < 0 or index > 2: return
        if mode == -1:   # change of data in a dataSet
            row = data
            for j, parCode in enumerate(parCodesCByUse[index]):
                if parCode is not None:
                    setattr(self.obj, parCode, row[j])
                    
            
        dummy, tableData = self.getTables()
        choices = None
        
        return 0, tableData, choices

    def setDataFile(self, fileName):
        self.obj.DataFile = fileName

        if fileName is not None:
            self.obj.dbelectricitymix_id = None

        self.obj.setName()

    def setEMix(self, emixId):
        self.obj.dbelectricitymix_id = emixId
        self.obj.DataFile = None
        self.obj.setName()
        
    def saveToDB(self, data=None):
        if Status.ANo < 0:

            conflicts = Status.mod.moduleCC.elCheck()
            Status.mod.moduleCC.initPanel()
            if conflicts > 0:
                self.cf = conflictFrame(Globals.GUI.editPanel)
                self.cf.Show()
                return False

            else:
                self.obj.saveToDB()
                Status.prj.setStatus("StatusECO", 0)
                return True

        else:
##            if Status.ANo >= 0 and not proc.checkCompleteness():
##                return False
            logger.showWarning(_("It is not allowed to modify electricity consumption in the checked state. These values are a RESULT of calculations."))
