# -*- coding: utf-8 -*-
#==============================================================================#
#   E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#   moduleScheduleViewer
#           
#------------------------------------------------------------------------------
#
#   Background module for the display of time schedules
#           
#==============================================================================
#
#   EINSTEIN Version No.: 2.3
#   Created by:     Hans Schweiger
#
#   Update No. 001
#
#                   
#------------------------------------------------------------------------------     
#   (C) copyleft energyXperts.BCN (E4-Experts SL), Barcelona, Spain 2014
#   www.energyxperts.net / info@energyxperts.net
#
#   This program is free software: you can redistribute it or modify it under
#   the terms of the GNU general public license as published by the Free
#   Software Foundation (www.gnu.org).
#
#============================================================================== 
from GUI.status import Status
from modules.messageLogger import logTrack, logDebug, logWarning, showWarning
from constants import YEAR, NDAYS

import numpy as np
from datetime import date
from modules.schedules.schedules import periodReference
from modules.schedules.scheduleTools import BasicSchedule

from math import ceil, floor



def periodView(periods, holidays=None, holidayScale=None):
    """ generates the data for the visualisation of the period scales

    :param periods:  list of PeriodProfile - objects
    :param holidays: sorted list of holiday intervals (in hours !!!)
    
    """

    s = BasicSchedule()
    h = BasicSchedule()

    f_previous = 0
    t_previous = 0

    plotPeriods = []
    for period in periods:
        start = period.start
        stop = period.stop
        startDay = (date(periodReference.year, start.month, start.day) - periodReference).days
        stopDay  = (date(periodReference.year,  stop.month,  stop.day) - periodReference).days + 1
        scale = period.scale/100
        plotPeriods.append((startDay, stopDay, scale))

    s = BasicSchedule()
    s.getPeriods(plotPeriods, tmax=365.0)

    if holidays: # add holidays if defined
        hPeriods = [(holidayPeriod[0], holidayPeriod[1], holidayScale) for holidayPeriod in holidays]
        h = BasicSchedule()
        h.getPeriods(hPeriods, tmax=365.0, default=1.0)
                        
        # multiply s(t)*h(t)
        s.multiply(h)

    view = s.plot()

    return view.f, view.t


def scheduleView(intervals, start, stop):
    """ Generates a plot of a schedule from start to stop (in hours)
    """
##    start = int(start)
##    hstop = YEAR - int(YEAR - stop)

    startDay = int(floor(start/24.))
    stopDay =  int(ceil(stop/24.))

    plotPeriods = []
    for day in range(startDay, stopDay):
        dayIntervals = intervals[day]
        t0 = day*24.0
        for interval in dayIntervals:
            istart, istop, iscale = interval
            tstart = max(start, t0 + timeToFloat(istart))
            tstop  = min(stop, t0 + timeToFloat(istop))
            scale  = iscale

            if tstop > tstart: plotPeriods.append((tstart, tstop, scale))

    s = BasicSchedule()
    s.getPeriods(plotPeriods, tmin=start, tmax=stop)
    view = s.plot()

    return view.f, view.t


# TODO: this function is copied from PeriodSchedule. -> create a separate module for datetime-handling
def timeToFloat(time): 
    """Return the float represenation of the given datetime.time object.
    
    e.g. datetime.time(3,30) becomes 3.5
    """
    return time.hour + time.minute / 60.0


    


        

    
