# -*- coding: utf-8 -*-
#==============================================================================#
#   E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#   moduleXY
#           
#------------------------------------------------------------------------------
#
#   Background module for data entry panel XY (template)
#           
#==============================================================================
#
#   EINSTEIN Version No.: 3.0
#   Created by:     Hans Schweiger
#
#   Update No. 001
#
#                   
#------------------------------------------------------------------------------     
#   (C) copyleft energyXperts.BCN (E4-Experts SL), Barcelona, Spain 2014
#   www.energyxperts.net / info@energyxperts.net
#
#   This program is free software: you can redistribute it or modify it under
#   the terms of the GNU general public license as published by the Free
#   Software Foundation (www.gnu.org).
#
#============================================================================== 
from GUI.status import Status
from globals import logger, Globals

class ModulePRJ(object):
    """

    Attributes:

    
    """
    def __init__(self, panel):
        self.panel = panel          # gives access to the GUI panel

        self.initPanel()


###CUSTOM: inisialise here state variables, e.g.:
        self.activeObject = None    # number (or instance ???) of active object currently being edited
        self.backgroundTable = []   # background table to be displayed
        self.selection = []         # current selection in table
        
    def initPanel(self):
        """  functions that have to be carried out only once
        when the panel is created
        """
###CUSTOM: here custom functions when initialising the panel
        pass

###CUSTOM: in most cases updatePanel may be necessary to be called at init
        self.updatePanel()
    
    def updatePanel(self):
        """Functions to be carried out each time when information
            to be displayed on the panel has changed
        """
        pass
        # create system model -> contains list of heat exchangers
        # to do: shift to module !!!
        logger.logTrack("ModulePRJ.updatePanel")

###CUSTOM: generate here all the info to be displayed
        ### e.g. on tables: data and selection
        self.getTable()
        projectIds = [row[0] for row in self.backgroundTable]
        if Status.PId in projectIds:
            self.pId = Status.PId
            self.selection = [projectIds.index(Status.PId)]
        else:
            self.selection = []

    def changeData(self, index, mode, data=None):
        """
        eventHandler for actions within the table XXX
        
        modifies the background table and calculates
        default data when adding / deleting a new line
        or modifying data in the table

        this function is called from the TableEntry event handler
        """

        err = False
        nDataSets = len(self.backgroundTable)

        if index > nDataSets or index < -1:
            return 1, None, None
        elif index==-1 and mode not in [1, 4]:
            return 1, None, None

###CUSTOM: take care - mode numbers are depending on PopUpMenu defined

        if mode == 0:     # copy
            self.activeProject = index
            logger.logTrack("PanelPRJ (copy): copying %s"%projectName)

###CUSTOM: here action in the model when copying some item

          #e.g.:
            original = self.backgroundTable[index]
            
            pu1 =  DialogPRJ(self.gui, self)

###CUSTOM: call personalised dialogs on GUI (here example from modulePRJ)            
            import wx
            if pu1.ShowModal() == wx.ID_OK:
                logger.logTrack("PanelP - OK %r %r"%(self.shortName,self.description))

                Status.prj.createNewProject(-1, self.shortName, self.description, originalName=projectName)
                Globals.GUI.panelInfo.update()
                Globals.GUI.showMainMenuAlternatives()
                Globals.GUI.updateDrawings()
            
        elif mode in [1]:   # delete

### (...)
            
        self.activeObject = index-1 if index > 0 else (0 if len(self.projectData) > 0 else None)
            

        data, selection = self.getTable()
        choices = []
        return 0, data, choices     # default return values for changeData function of TableEntry

    def getTable(self):
        """
        Gets the data from the backend and transforms them into a
        data set for the project table to be displayed
        """
        table = []

        sproject = dbio.select("sproject")

        self.projectData = []
        for s in sproject:
            q = dbio.get("questionnaire", id=s.ProjectID)

            if q is not None:
                self.projectData.append([s.ProjectID, s.ProjectName, q.DescripIndustry, q.Branch])
            else:
                print ("Corrupt DB entry for ProjectID %s: no corresponding entry in table questionnaire."%s.ProjectID)
                
        self.projectData = sorted(self.projectData, key=lambda row: row[1]) 

        table = [[str(noneCheck(elem, "")) for elem in row[1:]] for row in self.projectData]

###CUSTOM: selection to be returned ? (PRJ) In some cases (where ???) also choices have been updated by getTable            
        return table, self.selection

    def setSelection(self, selection):
        """
        Sets the current selection by specifying a list of indexes in the backgroundTable
        """
        print "ModulePRJ: setSelection to %s"%selection
        self.selection = selection
        
    def readLine(self, data):
        """ reads a line of raw data from the table in the GUI
        and translates it into internal code
        (inverse of getTable)
        """
        pass


    def saveToDB(self):
        """
        Default event handler called from OK button in the EinsteinPanel
        """

###CUSTOM: here active object data should be saved, or any other action corresponding to the OK-button
        
        if len(self.selection)==1:
            r = self.selection[0]
            self.openProject(r)
            return True
        else:
            return False
### ??? return values True/False -> close panel or not ???


###CUSTOM: specific helper functions, e.g.:
    def openProject(self, r):
        """ opens the project corresponding to row r in the table
        """
        pId = self.projectData[r][0]
        Status.prj.setActiveProject(pId)
        Globals.GUI.updateDrawings()
        



                        
        

if __name__ == "__main__":
    print "Hello World";
