# -*- coding: utf-8 -*-
#==============================================================================#
#   E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.einstein-energy.net)
#
#------------------------------------------------------------------------------
#
#   modulePA2
#           
#------------------------------------------------------------------------------
#
#
#           
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------     
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
from GUI.status import *
from modules.heatFlow_Tit import CompositeCurve, pinchCC
from modules.messageLogger import *
if Status.SimTool == "NP":
    from modules.buildCC import buildCC

SHOWENERGY = True           #Controls wether energy or power is shown in pinch graphics


class PinchAnalysis():
    def __init__(self):
        Status.int.systemModel.update()
        self.streamGlb = Status.int.systemModel.streams


        #TODO: this should be done later on already before running system simulation

        self.ccc_o = CompositeCurve()
        self.hcc_o = CompositeCurve()

        if self.streamGlb is not None:
            self.pinchStreams = [streamGlb for streamGlb in self.streamGlb if streamGlb.pinchMask]
            for streamGlb in self.pinchStreams:

                if Status.SimTool == "Plus":
                    stream = streamGlb.hf

                    if stream.heating:
                        self.ccc_o.add(stream.buildCC())
                    else:
                        self.hcc_o.add(stream.buildCC())
                else:
                    if streamGlb.Q > 0:
                        if streamGlb.heating:
                            self.ccc_o.add(buildCC(streamGlb))
                        else:
                            self.hcc_o.add(buildCC(streamGlb))

        else:
            self.streamGlb = []
     
    def calculate(self, DTmin):
        """Update the cached data according to showHeating"""

        self.DTmin = DTmin

        self.hcc = self.hcc_o.copy()
        self.ccc = self.ccc_o.copy()

        self.gcc, self.TPinch, self.shift = pinchCC(self.hcc, self.ccc, self.DTmin)

        self.TPinchC = self.TPinch - self.DTmin
        self.TPinchH = self.TPinch 
        if len(self.gcc.Q) > 0:
            if len(self.ccc.Q) > 0: # check just for security. probably not necessary (-> if no ccc, then no gcc ?)
                self.qhxMax = self.ccc.Q[-1] - self.gcc.Q[-1]
            else:
                self.qhxMax = - self.gcc.Q[-1]
                
            self.qextH = self.gcc.Q[-1]
            self.qextC = self.gcc.Q[0]
        else:
            self.qhxMax = 0
            self.qextH = 0
            self.qextC = 0

        self.ccc.shiftQ(-self.shift)

    def exportData(self):
        pinchInfo = {"TPinchC": self.TPinchC,
                     "TPinchH": self.TPinchH,
                     "QHXmax": self.qhxMax,
                     "QextH": self.qextH,
                     "QextC": self.qextC,
                     "DTmin": self.DTmin}

        data = [Curve(self.ccc), Curve(self.hcc), Curve(self.gcc), pinchInfo]

        Status.mod.moduleHR.qhxTarget = self.qhxMax

##        print "PinchAnalysis: data = ", data
        
        Status.int.setGraphicsData("PA2", data)

class Curve():
    def __init__(self, cc, name=""):
        self.X = [q/1000.0 for q in cc.Q]
        self.Y = cc.T

    def __repr__(self):
        result = "CURVE"
        for s, x in enumerate(self.X):
            result += "\n%10.2f: %10.2f"%(x, self.Y[s])
        return result

