# -*- coding: utf-8 -*-
#==============================================================================
#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	MESSAGE LOGGER
#			
#------------------------------------------------------------------------------
#			
#	Auxiliary module for using the main-fucntions
#       logMessage, logError and logWarning also from other modules
#
#       logMessage, logWarnings, logError: display on the GUI interface
#       logTrack: tracks messages in data file for further analysis
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#==============================================================================

DEBUG_TO = "OFF"    #SCREEN: prints debug information on the screen
                    #LOGFILE: writes debug information to the log-File
                    #OFF: no debug information written

#------------------------------------------------------------------------------		
def setDebugMode(mode):
#------------------------------------------------------------------------------		
#   function that can be called from somewhere for setting the mode of debugging
#------------------------------------------------------------------------------		

    global DEBUG_TO

    if mode in ["SCREEN","LOGFILE","OFF"]:
        DEBUG_TO = mode
    else:
        DEBUG_TO = "OFF"
        logWarning("messageLogger (setDebugMode): error in debug mode [%s]"%mode)
#------------------------------------------------------------------------------		
#------------------------------------------------------------------------------		
def logTrack(message):
#------------------------------------------------------------------------------		
#   keeps track of what the user does within the EINSTEIN log-file
#   this information can be very useful for debuggig
#   e.g. each user interaction on the GUI should be logged
#------------------------------------------------------------------------------		
    from globals import Globals
    Globals.tracker.logTrack(message)

    if DEBUG_TO == "SCREEN":
        print(message)

#------------------------------------------------------------------------------		
def logMessage(message):
#------------------------------------------------------------------------------		
#   writes a message to the EINSTEIN GUI and parallely to the log-file
#------------------------------------------------------------------------------		
    from globals import Globals
    Globals.tracker.logMessage(message)
##    if Globals.GUI: Globals.GUI.logMessage(message)

#------------------------------------------------------------------------------		
def logError(message):
#------------------------------------------------------------------------------		
#   writes a message to the EINSTEIN GUI and parallely to the log-file
#   highlights as ERROR
#------------------------------------------------------------------------------		
    from globals import Globals
    Globals.tracker.logError(message)
##    if Globals.GUI: Globals.GUI.logError(message)

#------------------------------------------------------------------------------		
def logWarning(message):
#------------------------------------------------------------------------------		
#   writes a message to the EINSTEIN GUI and parallely to the log-file
#   highlights as WARNING
#------------------------------------------------------------------------------		
    from globals import Globals
    Globals.tracker.logWarning(message)
##    if Globals.GUI: Globals.GUI.logWarning(message)

#------------------------------------------------------------------------------		
def logDebug(message):
#------------------------------------------------------------------------------		
#   writes a message to the log-file or to the screen (depending on DEBUG mode)
#   doesn't do anything if DEBUG = "OFF"
#------------------------------------------------------------------------------		
    if DEBUG_TO == "SCREEN":
        print message
    elif DEBUG_TO == "LOGFILE":
        logTrack("DEBUG: "+message)
    
#------------------------------------------------------------------------------		
def showError(text):
#------------------------------------------------------------------------------		
#   like logError, but opens a pop-up for confirmation
#------------------------------------------------------------------------------		
    from globals import Globals
    Globals.tracker.showError(text)
##    if Globals.GUI:
##        Globals.GUI.showError(text)

#------------------------------------------------------------------------------		
def showWarning(text):
#------------------------------------------------------------------------------		
#   like logWarning, but opens a pop-up for confirmation
#------------------------------------------------------------------------------		
    from globals import Globals
    Globals.tracker.showWarning(text)
##    if Globals.GUI:
##        Globals.GUI.showWarning(text)

#------------------------------------------------------------------------------		
def showMessage(text):
#------------------------------------------------------------------------------		
#   like logMessage, but opens a pop-up for confirmation
#------------------------------------------------------------------------------		
    from globals import Globals
    Globals.tracker.showMessage(text)
##    if Globals.GUI:
##        Globals.GUI.showInfo(text)
        
#------------------------------------------------------------------------------		
def askConfirmation(text, alwaysAsk=False):
#------------------------------------------------------------------------------		
#   like showMessage, but returns the answer of the user (wx.YES_NO)
#------------------------------------------------------------------------------		
    from globals import Globals
    if Globals.GUI:
        return Globals.GUI.askConfirmation(text, alwaysAsk)
    else:
        return True
    
#------------------------------------------------------------------------------		

