# -*- coding: utf-8 -*-
#==============================================================================#
#	E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#	ModuleCS - Comparative study
#
#       Preparation of comparative tables for GUI and report
#			
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------		
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 
import numpy as np
import copy
from auxiliary.auxiliary import noneFilter, transpose
from GUI.status import *
from modules.messageLogger import logWarning, logTrack, logDebug,\
    showWarning
from auxiliary.auxiliary import toGettextAndUnicode as _U
from lib.auxiliary.auxiliary import noneCheck
import database.dbio as dbio

class ModuleCS(object):
    def __init__(self, keys):
        self.keys = keys
        self.heating = True        

    def initPanel(self):
        """
        #   data that have to be provided at the init of panel creation
        """
        self.heating = True
    
    def updatePanel(self, heating=None):
        """
        send information to the GUI necessary for updating the panel
        """

        if heating is not None: self.heating = heating
        
        #..............................................................................
        # first collect the information for all the alternatives in the project
        
        generalData = sorted(dbio.select("cgeneraldata", Questionnaire_id = Status.PId), key=lambda s: s.AlternativeProposalNo)
        salternatives = Status.prj.getAlternatives()
        aNames = [a.ShortName for a in salternatives[1:]]
        energyOK = np.array([1 if a.StatusEnergy else None for a in salternatives[1:]], dtype=float)

        qelectricity = dbio.select("qelectricity", Questionnaire_id = Status.PId)
        questionnaire = Status.prj.getQuestionnaire()
        sproject = Status.prj.getProjectStatus()

        if self.keys[0] in ["CS4_Plot", "CS5_Plot", "CS6_Plot", "CS7_Plot"] and any([(a.StatusECO==0) for a in salternatives[1:]]):
            Status.mod.moduleECO.runPresentState()

        # check simulation results for all alternatives
        for ANo, alt in enumerate(salternatives[1:]):

            logTrack("ModuleCS (CS1): StatusEnergy(%s) = %s"%(ANo,alt.StatusEnergy))
            if alt.StatusEnergy == 0:
                Status.prj.setActiveAlternative(ANo)
                Status.mod.moduleEA.update()

            # update economic parameters where necessary
            if self.keys[0] in ["CS4_Plot", "CS5_Plot", "CS6_Plot", "CS7_Plot"] and alt.StatusECO==0:
                if ANo != Status.ANo: Status.prj.setActiveAlternative(ANo)
                Status.mod.moduleECO.run(ANo)

        PEC = []
        PECTable = []
        PECPlot = []
        
        #Panel CS1: Primary energy
        if self.keys[0] == "CS1_Plot":

            PEC = np.array([g.PEC for g in generalData[1:]], dtype=float)

            PEC = PEC*energyOK
            dPEC, rPEC, PEC0 = getRelativeSaving(PEC)


            PEC_Table = list(PEC)
            
            dPEC_Table = list(dPEC)
            dPEC_Table[0] = None
            
            rPEC_Table = list(rPEC)
            rPEC_Table[0] = None

            CS1Table = transpose([aNames, PEC, dPEC_Table, rPEC_Table])

            # TODO: check if this has some impact
            ######            if Status.ANo <> presentAlternative:
            ######                Status.prj.setActiveAlternative(presentAlternative)

            #..............................................................................
            # then send everything to the GUI

            Status.int.setGraphicsData("CS1_Table", CS1Table)

##            matrix2 = np.transpose(PECPlot)
##            data2 = np.array(PECPlot)
##
##            Status.int.setGraphicsData("CS1_Plot", data2)

#            print 'Table: data1\n',data1
#            print 'Plot: data2\n',data2

            PECReport = copy.deepcopy(CS1Table)

            # convert fractions to percentage for report master
            for i in range(1,len(PECReport)):
                try:
                    PECReport[i][3] *= 100.0
                except:
                    pass
            # fill with empty lines in order to match field size in master report                
            for i in range(len(CS1Table),11):
                PECReport.append([" "," "," "," "])
                
            dataReport1 = np.array(PECReport)                  
            Status.int.setGraphicsData("CS1_REPORT", dataReport1)


            panelData = {"Table": {"data": CS1Table},
                         "Plot": {"xData": aNames,
                                  "yData": [PEC/1000.0],
                                  "labels": ["PEC"]}}

            return panelData


        #------------------------------------------------------------------------------
        # Panel CS2: Process & supply heat
        elif self.keys[0] == "CS2_Plot":

            if self.heating:
                UPX = np.array([g.UPH for g in generalData[1:]], dtype=float)
                USX = np.array([g.USH for g in generalData[1:]], dtype=float)
            else:
                UPX = np.array([g.UPC for g in generalData[1:]], dtype=float)
                USX = np.array([g.USC for g in generalData[1:]], dtype=float)
                
            dUPX, rUPX, UPX0 = getRelativeSaving(UPX)
            dUSX, rUSX, USX0 = getRelativeSaving(USX)

            UPX_Table = list(UPX)
            dUPX_Table = list(dUPX)
            
            USX_Table = list(USX)
            dUSX_Table = list(dUSX)

            dUPX_Table[0] = None
            dUSX_Table[0] = None

            CS2Table = transpose([aNames, UPX, dUPX, USX, dUSX])

            UPX_Table = list(UPX/1000.0)
            dUPX_Table = list(dUPX/1000.0)
            
            USX_Table = list(USX/1000.0)
            dUSX_Table = list(dUSX/1000.0)

            dUPX_Table[0] = "---"
            dUSX_Table[0] = "---"


            CS2Report = transpose([aNames, UPX_Table, dUPX_Table, USX_Table, dUSX_Table])
            Status.int.setGraphicsData("CS2_Table", np.array(CS2Report))

            for i in range(len(CS2Table),11):
                CS2Report.append([" "," "," "," "," "])

            if self.heating:
                key = "CS2_REPORT"
            else:
                key = "CS2_REPORT_C"
            Status.int.setGraphicsData(key, np.array(CS2Report))


            titles = [_("Alternative")]

            if self.heating:
                titles += [_("Useful process heat (UPH)"),
                           _("Savings (UPH)"),
                           _("Useful supply heat (USH)"),
                           _("Savings (USH) ")]
            else:
                titles += [_("Useful process cooling (UPC)"),
                           _("Savings (UPC)"),
                           _("Useful supply cooling (USC)"),
                           _("Savings (USC)")]

            panelData = {"Table": {"data": CS2Table, "titles": titles},
                         "Plot": {"xData": aNames,
                                  "yData": [100.0*(1-rUPX), 100.0*(1.0-rUSX)],
                                  "labels": ["UPH", "USH"] if self.heating else ["UPC", "USC"]},
                         "hcModeChooser": self.heating}

            return panelData
            
        #------------------------------------------------------------------------------
        # Panel CS3: Ambiental impact
        elif self.keys[0] == "CS3_Plot":

            CO2El = np.array([g.ProdCO2el for g in generalData[1:]], dtype=float)
            CO2Fuel = np.array([g.ProdCO2Fuels for g in generalData[1:]], dtype=float)
            CO2Total = CO2El + CO2Fuel

            NucWaste = np.array([g.ProdNoNukesEl for g in generalData[1:]], dtype=float) * 1.e-6 # ProdNoNukes is stored in mg !!!
            WaterConsum  = np.array([g.WaterConsum for g in generalData[1:]], dtype=float)
            
            dCO2Total, rCO2Total, CO2Total0 = getRelativeSaving(CO2Total)
            dNucWaste, rNucWaste, NucWaste0 = getRelativeSaving(NucWaste)
            dWaterConsum, rWaterConsum, WaterConsum0 = getRelativeSaving(WaterConsum)
            
            CO2Total_Table = list(CO2Total)         
            NucWaste_Table = list(NucWaste)
            WaterConsum_Table = list(WaterConsum)
            
            CS3Table = transpose([aNames, CO2Total_Table, NucWaste_Table, WaterConsum_Table])
               
#..............................................................................
# then send everything to the GUI

            data1 = np.array(CS3Table)
                              
            Status.int.setGraphicsData("CS3_Table", data1)

            CS3Report = transpose([aNames, list(CO2Total/1000.0), NucWaste_Table, WaterConsum_Table])
            for i in range(len(CS3Table),11):
                CS3Report.append([" "," "," "," "])
            dataReport3 = np.array(CS3Report)                  
            Status.int.setGraphicsData("CS3_REPORT", dataReport3)

            panelData = {"Table": {"data": CS3Table},
                         "Plot": {"xData": aNames,
                                  "yData": [100.0*(1.0-rCO2Total), 100.0*(1.0-rNucWaste), 100.0*(1.0-rWaterConsum)],
                                  "labels": ["CO2", "Nuclear. Waste", "Water consumption"]}}

            return panelData

        #------------------------------------------------------------------------------
        # Panel CS4: Investment cost
        elif self.keys[0] == "CS4_Plot":

            TotalCost = np.array([g.TotalInvCost for g in generalData[1:]])
            OwnCost   = np.array([g.OwnInvCost for g in generalData[1:]])
            Subsidies = np.array([g.Subsidies for g in generalData[1:]])

            dTotalCost, rTotalCost, TotalCost0 = getRelativeSaving(TotalCost)
            dOwnCost, rOwnCost, OwnCost0 = getRelativeSaving(OwnCost)
            dSubsidies, rSubsidies, Subsidies0 = getRelativeSaving(Subsidies)
            
            TotalCost_Table = list(TotalCost)         
            OwnCost_Table = list(OwnCost)
            Subsidies_Table = list(Subsidies)
            
            CS4Table = transpose([aNames, TotalCost_Table, OwnCost_Table, Subsidies_Table])

        #..............................................................................
        # then send everything to the GUI

            Status.int.setGraphicsData("CS4_Table", CS4Table)

            CS4Report = copy.deepcopy(CS4Table)
            for i in range(len(CS4Table),11):
                CS4Report.append([" "," "," "," "])

            dataReport4 = np.array(CS4Report)                  
            Status.int.setGraphicsData("CS4_REPORT", dataReport4)

            panelData = {"Table": {"data": CS4Table[1:]},
                         "Plot": {"xData": aNames[1:],
                                  "yData": [OwnCost[1:], Subsidies[1:]],
                                  "labels": ["Own Cost", "Subsidies"]}}

            return panelData


        #------------------------------------------------------------------------------
        # Panel CS5: Annual cost
        elif self.keys[0] == "CS5_Plot":

            # copy OMThermal from questionnaire into generalData for present state (TODO: should be done elsewhere)
            generalData[1].OMThermal = questionnaire.OMThermal

            Amortization = np.array([g.Amortization for g in generalData[1:]])
            Amortization[0] = 0.0

            ElCostYearTot     = np.array([q.ElCostYearTot for q in qelectricity[1:]])
            ElSalesYearCHP    = np.array([q.ElSalesYearCHP for q in qelectricity[1:]])
            ElSalesYearFeedIn = np.array([q.ElSalesYearFeedIn for q in qelectricity[1:]])

            try:
                ElCost = ElCostYearTot - ElSalesYearCHP - ElSalesYearFeedIn
            except:
                ElCost = np.zeros(len(generalData[1:]), dtype=float)

            FuelCost = np.zeros(len(generalData[1:]), dtype=float)
            for ANo in range(len(generalData[1:])):
                fuels = Status.prj.getFuels(PId=Status.PId, ANo=ANo)
                for fuel in fuels:
                    FuelCost[ANo] += noneCheck(fuel.FuelCostYear, np.nan)
            
            EnergyCost = ElCost + FuelCost

            OMCost = np.array([g.OMThermal for g in generalData[1:]])
            OMCost[0] = noneCheck(questionnaire.OMThermal, 0.0) # security feature if present state O&M is not defined

            TotalCost = Amortization + EnergyCost + OMCost

            dAmortization, rAmortization, Amortization0 = getRelativeSaving(Amortization)
            dEnergyCost, rEnergyCost, EnergyCost0 = getRelativeSaving(EnergyCost)
            dOMCost, rOMCost, OMCost0 = getRelativeSaving(OMCost)

            Amortization_Table = list(Amortization)
            Amortization_Table[0] = None
            
            EnergyCost_Table = list(EnergyCost)
            OMCost_Table = list(OMCost)
            TotalCost_Table = list(TotalCost)
            
            CS5Table = transpose([aNames, Amortization_Table, EnergyCost_Table, OMCost_Table, TotalCost_Table])

            CS5Report = transpose([aNames, Amortization_Table, EnergyCost_Table, OMCost_Table])
            Status.int.setGraphicsData("CS5_Table", CS5Report)

            for i in range(len(CS5Report), 11):
                CS5Report.append([" "," "," "," "])
                             
            dataReport5 = np.array(CS5Report)                  
            Status.int.setGraphicsData("CS5_REPORT", dataReport5)

            # export economic parameters used in the analysis for the report
            i = generalData[1].InterestExtFinancing      
            N = int(generalData[1].AmortisationTime) 
            
            sum = 0
            for n in range(1,N+1):
                sum += 1/pow(1.0+i,n)

            if sum > 0:
                annuity = 1./sum
            else:
                annuity = 1.0
                
            ecopars = [i*100.0,
                       N,
                       annuity*100.0]

            Status.int.setGraphicsData("CS5_REPORT_PARS", np.array(ecopars))

            # export data for GUI
            panelData = {"Table": {"data": CS5Table},
                         "Plot": {"xData": aNames,
                                  "yData": [Amortization, EnergyCost, OMCost],
                                  "labels": ["Amortisation", "Energy Cost", "O&M Cost"]}}

            return panelData

        #------------------------------------------------------------------------------
        # Panel CS6: Annual cost
        elif self.keys[0] == "CS6_Plot":

            PEC = np.array([g.PEC for g in generalData[1:]])
            dPEC, rPEC, PEC0 = getRelativeSaving(PEC)
                             
            EnergySystemCost = np.array([g.EnergySystemCost for g in generalData[1:]])
            dEnergySystemCost, rEnergySystemCost, EnergySystemCost0 = getRelativeSaving(EnergySystemCost)

            AddCost = dEnergySystemCost
            np.seterr(divide="ignore")
            AddCostSavedPEC = AddCost / dPEC
            np.seterr(divide="warn")
            
            EnergySystemCost_Table = list(EnergySystemCost)         
            AddCost_Table = list(AddCost)
            AddCost_Table[0] = None
            
            AddCostSavedPEC_Table = list(AddCostSavedPEC)
            AddCostSavedPEC_Table[0] = None
            
            CS6Table = transpose([aNames, EnergySystemCost_Table, AddCost_Table, AddCostSavedPEC_Table])
            Status.int.setGraphicsData("CS6_Table", CS6Table)

            CS6Report = copy.deepcopy(CS6Table)
            for i in range(len(CS6Table),11):
                CS6Report.append([" "," "," "," "])
            dataReport6 = np.array(CS6Report)                  
            Status.int.setGraphicsData("CS6_REPORT", dataReport6)

            panelData = {"Table": {"data": CS6Table},
                         "Plot": {"xData": aNames[1:],
                                  "yData": [1000.0*AddCostSavedPEC[1:]],
                                  "labels": [" "]}}

            return panelData


        #------------------------------------------------------------------------------
        # Panel CS7: Annual cost
        elif self.keys[0] == "CS7_Plot":

            IRR = np.array([g.IRR for g in generalData[1:]], dtype=float)
            BCR = np.array([g.BCR for g in generalData[1:]], dtype=float)
            PayBack = np.array([g.PayBack for g in generalData[1:]], dtype=float)

            dIRR, rIRR, IRR0 = getRelativeSaving(IRR)
            dBCR, rBCR, BCR0 = getRelativeSaving(BCR)
            dPayBack, rPayBack, PayBack0 = getRelativeSaving(PayBack)
            
            IRR_Table = list(IRR)
            IRR_Table[0] = None

            BCR_Table = [None if np.isnan(val) else val for val in BCR]
            BCR_Table[0] = None

            PayBack_Table = list(PayBack)
            PayBack_Table[0] = None
            
            CS7Table = transpose([aNames, IRR_Table, PayBack_Table, BCR_Table])

            Status.int.setGraphicsData("CS7_Table", CS7Table)

            # add data for own investment and NPV for report

            ownInvestment = np.array([g.OwnInvCost for g in generalData[1:]], dtype=float)
            NPV = np.array([g.NPV for g in generalData[1:]], dtype=float)
                             
            CS7Report = transpose([aNames[1:], IRR_Table[1:], PayBack_Table[1:], BCR_Table[1:], ownInvestment[1:], NPV[1:]])
            for i in range(len(CS7Table),10):
                CS7Report.append([" "]*6)
                             
            Status.int.setGraphicsData("CS7_REPORT", CS7Report)

            data = dbio.get("tcageneraldata", ProjectID=Status.PId, AlternativeProposalNo=0)
            N = noneCheck(data.AmotisationTime, 15)
            Status.int.setGraphicsData("CS7_REPORT_PARS", np.array([[N, u"  "]]))

            panelData = {"Table": {"data": CS7Table[1:]},
                         "Plot": {"xData": aNames[1:],
                                  "yData": [IRR[1:]*100],
                                  "labels": [_("IRR")]}}

            return panelData

        #------------------------------------------------------------------------------
        # Summary for report
        elif self.keys[0] == "Summary":
            if sproject is not None:
                summary = sproject.Summary
            else:
                summary = _(" ")

            Status.int.setGraphicsData("SUMMARY",np.array([[summary]]))

# build summary table

            if Status.FinalAlternative is not None:
                ANoList = [0,int(Status.FinalAlternative),None]
            else:
                ANoList = [0,-1,None]
            for ANo in range(1,len(generalData)-1):
                if ANo <> Status.FinalAlternative:
                    ANoList.append(ANo)
            
            summaryTable = []
                             
            for ANo in ANoList:

                if ANo is None:
                    row = [_("Other possible alternatives studied:"),
                           " "," "," "]
                elif ANo == -1:
                    row = [_("No alternative selected"),
                           " "," "," "]
                else:
                    row = []
                    i = ANo + 1

                    if salternatives[i].ShortName is None:
                        row.append('---')
                    else:
                        row.append(salternatives[i].ShortName)

                    if generalData[i].TotalInvCost is None: #protection against None
                        row.append('---')
                    else:
                        row.append(generalData[i].TotalInvCost)

                    if generalData[i].EnergyCost is None: #protection against None
                        row.append('---')
                    else:
                        row.append(generalData[i].EnergySystemCost)
            
                    if generalData[i].PEC is None: #protection against None
                        row.append('---')
                    else:
                        row.append(generalData[i].PEC/1000.0)

                summaryTable.append(row)

            Status.int.setGraphicsData("SUMMARY_TABLE", np.array(summaryTable))

    def chooseMode(self, imode):
        if imode==0:
            self.heating = False
        else:
            self.heating = True

#==============================================================================            

def getRelativeSaving(x_in):
    """
    helper function for comparative study
    
    calculates the difference dx = x[i]-x[0] and the relative difference rdx=dx/x[0]
    for an input vector x (np.array)

    automatically checks for None or NaN's

    """
    x = np.array(x_in, dtype=float)
    
    if x is None or len(x)==0:
        return None, None
    
    xref = x[0]
    dx = xref - x

    
    if np.isnan(xref) or xref==0:
        xref = np.nanmax(np.abs(x))

    if np.isnan(xref) or xref==0:
        xref = 1.0
        
    rdx = 1.0 - x / xref
    return dx, rdx, xref

    

    

    
