# -*- coding: cp1252 -*-
#==============================================================================#
#   E I N S T E I N
#
#       Expert System for an Intelligent Supply of Thermal Energy in Industry
#       (www.iee-einstein.org)
#
#------------------------------------------------------------------------------
#
#   ModuleHC (Heat and Cold Supply)
#           
#------------------------------------------------------------------------------
#           
#   Module for design of HC Supply cascade
#
#==============================================================================
#
#	Version 2.3plus, 30.06.2015
#
#------------------------------------------------------------------------------     
#
#	(C) copyright energyXperts.NET (E4-Experts GmbH), Berlin, Germany 2008-2015
#	http://www.energyxperts.net
#
#	all rights reserved
#
#============================================================================== 

from sys import *
from math import *
from numpy import *
import copy

from GUI.status import *
from modules.interfaces import *
from constants import *
from auxiliary.auxiliary import toGettextAndUnicode as _U
from auxiliary.auxiliary import toUnicode as U, noneFilter
from modules.messageLogger import logTrack
from modules.fluids import Fluid, Fuel

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
class ModuleHC(object):
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------

    HCList = []
    
#------------------------------------------------------------------------------
    def __init__(self, keys):
#------------------------------------------------------------------------------
        self.keys = keys # the key to the data is sent by the panel
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
    def initPanel(self):
#------------------------------------------------------------------------------
#       screens existing equipment, whether there are already heat pumps
#------------------------------------------------------------------------------

        pass    

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
    def updatePanel(self):
#------------------------------------------------------------------------------
#       Here all the information should be prepared so that it can be plotted on the panel
#------------------------------------------------------------------------------

        equipments = Status.prj.getEquipments(cascade=True)

        dataList = []
        dataListReport = []

        j = 0
        USHjTotal = 0
        USCjTotal = 0
        nIsHeatingEquipment = []        
        for equipe in equipments:
            j += 1
#..............................................................................
# getting pipe names of equipes

            IDString = equipe.PipeDuctEquip
            pipeDict = Status.prj.getPipeDict()
            if IDString is not None:
                pipeIDsSQL = IDString.split(';')
            else:
                pipeIDsSQL = []

            pipes = []
            for i in pipeIDsSQL:
                try:
                    pipeID = int(i)
                    if pipeID in pipeDict.keys():
                            pipes.append(U(pipeDict[pipeID]))
                except:
                    pass
            pipes = "o==" + "==o\no==".join(pipes) + "==o" 

#..............................................................................
# getting pipe names of equipes

            dataList.append(noneFilter([equipe.EqNo,
                                        U(equipe.Equipment),
                                        equipe.EquipType,
                                        equipe.HCGPnom,
                                        pipes]))

            if j <= Status.Report_NJMax:
                eqObject = Status.int.systemModel.eqObject[j-1]
                nIsHeatingEquipment.append(eqObject.heating)
                try:
                    USHj = equipe.USHj/1000.
                    if nIsHeatingEquipment[-1]:
                        USHjTotal += USHj
                    else:
                        USCjTotal += USHj
                except:
                    USHj = " "
                dataListReport.append(noneFilter([U(equipe.Equipment),
                                                  equipe.EquipType,
                                                  pipes,
                                                  equipe.HCGPnom,
                                                  USHj,
                                                  0.0]))

                if Status.ANo == Status.FinalAlternative:
                    if equipe.Model is not None:
                        model = U(equipe.Model)
                    else:
                        model = " "

                    if equipe.Manufact is not None:
                        manufact = U(equipe.Manufact)
                    else:
                        manufact = " "

                    if equipe.Refrigerant is not None:
                        refrigerant = U(Fluid(equipe.Refrigerant).name)
                    else:
                        refrigerant = " "
                        
                    if equipe.DBFuel_id is None:
                        fuel = u"-"
                    else:
                        fuel = Fuel(equipe.DBFuel_id).name
                    equipmentDataReport = [[equipe.EquipType,
                                            manufact,
                                            model,
                                            equipe.HCGPnom,
                                            fuel,
                                            equipe.FuelConsum,
                                            equipe.ElectriConsum,
                                            equipe.ElectriProduction,
                                            equipe.HCGEEfficiency,
                                            refrigerant]]

                    try:
                        OandMvar = 1000.*equipe.OandMvar   # Convert into CU/MWh as used in the report
                    except:
                        OandMvar = " "
                    equipmentEconomicDataReport = [[equipe.TurnKeyPrice,
                                                    equipe.OandMfix,
                                                    OandMvar]]

                    equipeData = array(transpose(noneFilter(equipmentDataReport,substitute=" ")))
                    equipeEconomicData = array(transpose(noneFilter(equipmentEconomicDataReport,substitute=" ")))
##                    print "ModuleHC (updatePanel): writing data for Key %s"%("EQ%02dECO"%(j))
##                    print equipeEconomicData
                    Status.int.setGraphicsData("EQ%02d"%(j),copy.deepcopy(equipeData))
                    Status.int.setGraphicsData("EQ%02dECO"%(j),copy.deepcopy(equipeEconomicData))
                
        data = array(dataList)

        Status.int.setGraphicsData(self.keys[0], data)

#Now create table for Report
        
        if Status.ANo > 0:
            SumUSH = 0.0
            SumPnom = 0.0
            for row in dataListReport:
                try:
                    SumUSH += row[4]
                except:
                    pass
                try:
                    SumPnom += row[3]
                except:
                    pass

            for n, row in enumerate(dataListReport):
                try:
                    if nIsHeatingEquipment[n]:
                        row[5] = 100.0 * row[4] / SumUSH
                    else:
                        row[5] = 100.0 * row[4] / SumUSH
                except:
                    row[5] = 0.0

            NJ = len(dataListReport)
            for j in range(NJ,Status.Report_NJMax):
                dataListReport.append([" "," "," "," "," "," "])
            totalPercentage = 100.
            dataListReport.append([_U("Total"), " ", " ", SumPnom, SumUSH,
                                   totalPercentage])
            dataReport = array(dataListReport)
        
            Status.int.setGraphicsData("HC%02d_REPORT"%Status.ANo, dataReport)

            if Status.ANo == Status.FinalAlternative:
                Status.int.setGraphicsData("HC_REPORT_F", dataReport)
                
        

#------------------------------------------------------------------------------
    def cascadeMoveUp(self,actualCascadeIndex):
#------------------------------------------------------------------------------
#        moves equipment up in the cascade sequence
#------------------------------------------------------------------------------

        self.equipment = Status.prj.getEquipment(cascade=True)
        self.NEquipe = len(self.equipment)

        if actualCascadeIndex < 1 or actualCascadeIndex > self.NEquipe:
            logWarning("moduleHC: cascadeMoveUp: there is no equipe with CascadeIndex = %s"%actualCascadeIndex)
            return 0

        elif actualCascadeIndex == 1:
            logWarning("moduleHC: cascadeMoveUp: equipe cannot be moved up, actualCascadeIndex = %s"%actualCascadeIndex)
            return 1            

        else:
            j = actualCascadeIndex - 1
            try:
                actualEquip = self.equipment[j]
                upEquip = self.equipment[j-1]

                actualEquip.CascadeIndex = actualCascadeIndex - 1
                upEquip.CascadeIndex = actualCascadeIndex          
                dbio.commit()
            except:
                logError("ModuleHC (cascadeMoveUp): severe error - couldn't find equipe")
            Status.int.changeInCascade(actualCascadeIndex-1)
                
##            Status.int.getEquipmentCascade()

            return actualEquip.CascadeIndex


#------------------------------------------------------------------------------
    def cascadeMoveDown(self,actualCascadeIndex):
#------------------------------------------------------------------------------
#        moves equipment down in the cascade sequence
#------------------------------------------------------------------------------

        self.equipment = Status.prj.getEquipment(cascade=True)
        self.NEquipe = len(self.equipment)

        if actualCascadeIndex < 1 or actualCascadeIndex > self.NEquipe:
            logWarning("moduleHC: cascadeMoveDown: there is no equipe with CascadeIndex = %s"%actualCascadeIndex)
            return 0

        elif actualCascadeIndex == self.NEquipe:
            logWarning("moduleHC: cascadeMoveDown: equipe cannot be moved down, actualCascadeIndex = %s"%actualCascadeIndex)
            return 1            

        else:
            j = actualCascadeIndex - 1
            try:
                actualEquip = self.equipment[j]
                downEquip = self.equipment[j+1]

                actualEquip.CascadeIndex = actualCascadeIndex + 1
                downEquip.CascadeIndex = actualCascadeIndex          
                dbio.commit()
            except:
                logError("ModuleHC (cascadeMoveDown): severe error - couldn't find equipe")

##            Status.int.getEquipmentCascade()
            Status.int.changeInCascade(actualCascadeIndex)

            return actualEquip.CascadeIndex

#------------------------------------------------------------------------------
    def cascadeMoveToTop(self,actualCascadeIndex):
#------------------------------------------------------------------------------
#   moves equipment to top in cascade sequence
#------------------------------------------------------------------------------

        newIndex = 1
        for i in range(actualCascadeIndex,1,-1):
            newIndex = self.cascadeMoveUp(i)

        return newIndex

#------------------------------------------------------------------------------
    def cascadeMoveToBottom(self,actualCascadeIndex):
#------------------------------------------------------------------------------
#   moves equipment to bottom in cascade sequence
#------------------------------------------------------------------------------

        newIndex = Status.int.NEquipe
        for i in range(actualCascadeIndex,Status.int.NEquipe):
            newIndex = self.cascadeMoveDown(i)

        return newIndex

#------------------------------------------------------------------------------
    def cascadeMoveTo(self,actualIndex,finalIndex):
#------------------------------------------------------------------------------
#   moves equipment to a userdefined finalIndex position in cascade sequence
#------------------------------------------------------------------------------

        if (finalIndex > actualIndex and finalIndex <= Status.int.NEquipe):
            for i in range (actualIndex,finalIndex):
                newIndex = self.cascadeMoveDown(i)

        elif (finalIndex < actualIndex and finalIndex > 0):
            for i in range (actualIndex,finalIndex,-1):
                newIndex = self.cascadeMoveUp(i)

        else: newIndex = actualIndex

        return newIndex

#------------------------------------------------------------------------------

if __name__ == "__main__":
    pass    
