import wx

from lib.text.textConversion import toUnicode as U, toGettextAndUnicode as _U
from lib.auxiliary.auxiliary import noneCheck
import matPanel as Mp
import numpy as np

import matplotlib.font_manager as font
from matplotlib.ticker import FuncFormatter, FixedLocator
##from einstein.gui.tools.GUITools import *
# EINSTEIN colours
WHITE = '#FFFFFF'
LIGHTGREY = '#F6F6F6'
MIDDLEGREY = '#C0C0C0'
DARKGREY = '#000060'
BLACK = '#000000'

GRAPH_BACKGROUND_COLOR = WHITE

ORANGECASCADE = ['#FF6000','#EE5000','#DD4000','#CC2000','#BB1000',\
                 LIGHTGREY,MIDDLEGREY,DARKGREY,BLACK]
LINETYPES = ['-','--',':','.','-.',]

# some constants
COLORTABLE = ['#0000FF', '#00FF00', '#FF0000', '#00FFFF', '#FF00FF',
              '#FFFF00', '#000000', '#FFFFFF']
# title properties dictionary
TITLE_FONT_DICT = {'fontname'  : ('Roman', 'Times New Roman'),
                   'fontweight': 'bold',
                   'color'     : '#808080',
                   'fontsize'  : 12}
import textwrap

# fix parameters for plot
axeslabel_fontsize = 8
axesticks_fontsize = 8
legend_fontsize = 8
spacing_left = 0.12
spacing_right = 0.98
spacing_bottom = 0.16
spacing_top = 0.98

###CHECK
######        # some constants for the simple bar plot
######        #
######        spacing_left = 0.2
######        spacing_right = 0.9
######        spacing_bottom = 0.1
######        spacing_top = 0.9



class BarPlot(wx.Panel):

    def __init__(self, parent, style=wx.SUNKEN_BORDER, size=wx.Size(800, 600), stacked=False):

        self.stacked = stacked
        
        super(wx.Panel, self).__init__(parent, style=style, size=size)
        
        # remap drawing methods to the wx widgets.
        paramList = {'backcolor'   : GRAPH_BACKGROUND_COLOR} # graph background color

        Mp.MatPanel(self, wx.Panel)
        
        self.subplot = None

        # default values for axis-limits
        self.xmin = None
        self.xmax = None
        self.ymin = None
        self.ymax = None

        self.axvLines = []
        self.xTicks = None
        self.minorXTicks = None

        self.grid = True

        self.lineTypes = None
        self.lineWidths = None
        self.colours = None

        self.markerSize = 8

        self.options = {"legend": True}

        self.xlabel = None
        self.ylabel = None

    def xAxis(self, label="x", xmin=None, xmax=None):
        """ sets Parameters of x-Axis
        """
        self.xlabel = label
        self.xmin = xmin
        self.xmax = xmax
    
    def yAxis(self, label="y", ymin=None, ymax=None):
        """ sets Parameters of y-Axis
        """
        self.ylabel = label
        self.ymin = ymin
        self.ymax = ymax

    def setDataLabels(self, labels, lineTypes=None, colours=None, lineWidths=None, markerSizes=None):
        n = len(labels)
        
        self.dataLabel = labels if labels else ["Row %s"%(i+1) for i in range(n)]
        self.lineTypes = lineTypes if lineTypes else [LINETYPES[j%len(LINETYPES)] for j in range(n)]
        self.colours = colours if colours else [ORANGECASCADE[j%len(ORANGECASCADE)] for j in range(n)]
        self.lineWidths = lineWidths if lineWidths else [2.0]*n
        self.markerSizes = markerSizes

    def setAxvLines(self, xvals):
        """ sets vertical lines within the plot """
        self.axvLines = xvals

    def setXTicks(self, xvals):
        """ sets vertical lines within the plot """
        self.xTicks = xvals

    def setMinorXTicks(self, xvals):
        """ sets vertical lines within the plot """
        self.minorXTicks = xvals


####    def drawComparedBarPlot(self):
####        #
####        # the following params are mandatory
####        #
####        try:
####            index_data = int(self.params['data'])      # index to the column of data
####        except KeyError:
####            print "drawComparedBarPlot: missing index to data column."
####            return
####
####        try:
####            key = self.params['key']                   # key for Interfaces.GData
####        except KeyError:
####            print "drawComparedBarPlot: missing key to data."
####            return
####
####        # the following params are optative
####
####        try:
####            index_labels = int(self.params['labels'])  # index to the column of labels
####        except KeyError:
####            index_labels = 0
####
####        try:
####            title = self.params['title']               # title of the graphic
####        except KeyError:
####            title = ''
####
####        try:
####            ylabel = self.params['ylabel']             # label on the y axis
####        except KeyError:
####            ylabel = ''
####
####        try:
####            legend = self.params['legend']             # legend list
####        except KeyError:
####            legend = []
####
####        try:
####            backcolor = self.params['backcolor']       # background color of the graph
####        except KeyError:
####            backcolor = '#FFFFFF'
####
####        try:
####            tickfontsize = self.params['tickfontsize'] # fontsize of the tick mark labels
####        except KeyError:
####            tickfontsize = 8
####
####
####
####
####        legendlabels = legend
####        legendptr = []
####
####        if hasattr(self, 'subplot'):
####            del self.subplot
####
####
######################################################
####
####        major_formatter = FuncFormatter(format_int_wrapper)
####        self.subplot.axes.xaxis.set_major_formatter(major_formatter)
####        self.subplot.axes.yaxis.set_major_formatter(major_formatter)
####        self.figure.subplots_adjust(left=spacing_left, right=spacing_right, bottom=spacing_bottom, top=spacing_top)
####        self.figure.set_facecolor(backcolor)
####
####
####        if not hasattr(self, 'subplot'):
####            self.subplot = self.figure.add_subplot(1, 1, 1)
####
####
####
####        # This should take care of l10n matters with decimal separators but breaks
####        # text labels. 
####        #major_formatter = FuncFormatter(format_int_wrapper)
####        #self.subplot.axes.xaxis.set_major_formatter(major_formatter)
####        #self.subplot.axes.yaxis.set_major_formatter(major_formatter)


    def draw(self, xData, yData):
        """ Plotting method
        will be dynamically added as method of class XYPlot by Mp.matPanel
        """
        cols = len(xData)
        rows = len(yData)
        if rows==0 or cols==0: return
        
        gap = 0.2
        groupWidth = 1.0 - gap
        
        width = groupWidth if self.stacked else groupWidth/rows                     # the width of the bars
        vx = np.linspace(0.5, 1.0*(cols-0.5), cols) # the x locations for the groups
        
        self.figure.clf()
        self.subplot = self.figure.add_subplot(1, 1, 1)
        self.figure.subplots_adjust(left=spacing_left, right=spacing_right, bottom=spacing_bottom, top=spacing_top)

        legendptr = []
        bottom = np.zeros(cols, dtype=float) if self.stacked else None
        for j, row in enumerate(yData):
            if self.stacked:
                barPos = vx
            else:
                barPos = vx - 0.5*groupWidth + (j+0.5)*width
            p = self.subplot.bar(barPos, row, width, color=COLORTABLE[j % 8], bottom=bottom)
            if self.stacked: bottom = row + bottom
            legendptr.append(p[0])
            
######        # set limits of x- and y-axes
######        if any([val is None for val in [self.xmin, self.xmax]]):
######            xmin, xmax = self.getAutoLimits(xData)
######            self.xmin = noneCheck(self.xmin, xmin)
######            self.xmax = noneCheck(self.xmax, xmax)
######
######        if any([val is None for val in [self.ymin, self.ymax]]):
######            ymin, ymax = self.getAutoLimits(yData)
######            self.ymin = noneCheck(self.ymin, ymin)
######            self.ymax = noneCheck(self.ymax, ymax)
######
######        self.subplot.axes.set_xlim([self.xmin, self.xmax])
######        self.subplot.axes.set_ylim([self.ymin, self.ymax])
######
######        if self.xTicks:
######            ticks = [tick for tick in self.xTicks if tick >= self.xmin and tick <= self.xmax]
######            self.subplot.xaxis.set_ticks(ticks)
######
######        if self.minorXTicks:
######            ticks = [tick for tick in self.minorXTicks if tick >= self.xmin and tick <= self.xmax]
######            minor_locator = FixedLocator(ticks)
######            self.subplot.xaxis.set_minor_locator(minor_locator)
######
        if self.grid:
##            if self.minorXTicks:
##                self.subplot.grid(which='minor')
            self.subplot.grid()



######        self.subplot.set_title(title, TITLE_FONT_DICT)

        # set labels of axes
    ##    major_formatter = FuncFormatter(format_int_wrapper)
    ##    self.subplot.axes.xaxis.set_major_formatter(major_formatter)
    ##    self.subplot.axes.yaxis.set_major_formatter(major_formatter)
        
        fp = font.FontProperties(size=axeslabel_fontsize)

##        self.subplot.axes.set_xlabel(xData, fontproperties=fp)

        if self.ylabel:        
            padding = 10 if len(self.ylabel) > 40 else None
            self.subplot.axes.set_ylabel(U(textwrap.fill(self.ylabel, 40, break_long_words=False)),
                                         fontproperties=fp,
                                         horizontalalignment='center',
                                         labelpad=padding)
##
##        for label in self.subplot.axes.get_yticklabels():
##    #        label.set_color(self.params['ytickscolor'])
##            label.set_fontsize(axesticks_fontsize)
##    #        label.set_rotation(self.params['yticksangle'])
##
##        # properties of labels on the x axis
##        for label in self.subplot.axes.get_xticklabels():
##    #        label.set_color(self.params['xtickscolor'])
##            label.set_fontsize(axesticks_fontsize)
##    #        label.set_rotation(self.params['xticksangle'])


        # create xticks
        self.subplot.axes.set_xticks(vx)
##        print "xticks at %s"%(vx) 
        xlabels = [U(textwrap.fill(label, 12, break_long_words=False)) 
                   for label in xData]
        xlabels = self.subplot.axes.set_xticklabels(xlabels)
        for xlabel in xlabels:
            xlabel.set_size(axesticks_fontsize)

        # legend

##        if self.options["legend"]:
        if self.dataLabel is not None and len(self.dataLabel) > 0:
            # draw legend and set legend parameters
            self.subplot.legend(legendptr, self.dataLabel, loc='best')
##        self.subplot.legend(loc=0)   #4: left lower; 0: best
            lg = self.subplot.get_legend()
            ltext = lg.get_texts()             # all the text.Text instance in the legend
            for txt in ltext:
                txt.set_fontsize(legend_fontsize)  # the legend text fontsize
            # legend line thickness
            llines = lg.get_lines()             # all the lines.Line2D instance in the legend
            for lli in llines:
                lli.set_linewidth(1.5)          # the legend linewidth
            # color of the legend frame
            # this only works when the frame is painted (see below draw_frame)
            frame = lg.get_frame()             # the patch.Rectangle instance surrounding the legend
            frame.set_facecolor('#F0F0F0')      # set the frame face color to light gray
            # should the legend frame be painted
            lg.draw_frame(False)
##        except:
##            # no legend
##            pass



        lines = [line for line in self.axvLines if line >= self.xmin and line <= self.xmax]
        for xval in lines:
            self.subplot.axvline(x=xval)

        self.canvas.draw()
        self.canvas.flush_events()

    def SetValue(self, dataDict, emitEvent=True):
        """
        For call from EinsteinPanel - symmetry with other displayClasses
        """
        self.setDataLabels(dataDict.get("labels"))

####        ,
####                           lineTypes= dataDict.get("lineTypes"),
####                           colours = dataDict.get("colours"),
####                           lineWidths = dataDict.get("lineWidths"),
####                           markerSizes = dataDict.get("markerSizes"))
                           
        self.draw(dataDict.get("xData"), dataDict.get("yData"))


######        try:
######            data = Interfaces.GData[key].transpose()
######            (rows, cols) = np.shape(data)
######            xticklabels = data[0]
######
######        except:
######            print "drawComparedBarPlot: values %s missing or bad." % (key,)
######            print "Interfaces.GData['%s'] contains:\n%s\n" % (key, repr(Interfaces.GData[key]))
######            return


    def getAutoLimits(self, data):
        """
        helper function for setting axis limits to round values around the data
        """
        if data is None or len(data) == 0: return 0, 0 # security feature

        fmin = np.min([np.min(row) for row in data]) 
        fmax = np.max([np.max(row) for row in data])

        df = fmax - fmin
        fabs = max(abs(fmax), abs(fmin))

        minDist = max(0.02*df, 0.001*fabs, 1.e-3)

        return fmin - minDist, fmax + minDist


##########################
############    from CS2 panel
####
####        if self._showHeating:
####            plotlegend = [_U('USH [% of present state]'),_U('UPH [% of present state]')]
####        else:
####            plotlegend = [_U('USC [% of present state]'),_U('UPC [% of present state]')]
####        paramList={'labels'      : 0,                            # labels column
####                   'data'        : 2,                            # data column for this graph
####                   'key'         : "CS2_Plot",                      # key for Interface
####                   'title'       :_U('Relative comparison of process & supply heat'), # title of the graph
####                   'ylabel'      :_U('[%]'),                   # y axis label
####                   'legend'      :plotlegend,                   # y axis label
####                   'backcolor'   :GRAPH_BACKGROUND_COLOR,        # graph background color
####                   'tickfontsize': 8,                            # tick label fontsize
####                   'ignoredrows' :[]}                        # rows that should not be plotted
####        
####        Mp.MatPanel(self.panelGraphMPHD, wx.Panel, drawComparedBarPlot, 
####                    paramList)
####
####        
        
        
