# -*- coding: utf-8 -*-
from math import floor, ceil
import bisect
import numpy as np

def interpolateTable(x, xlist, ylist):
    """Return the interpolated table values ylist(xlist) for values of 
    x not in xlist. It only works for montonous curves 
    (both ascending and descending)

    if x is out of the bounds, the maximum or minimum value of ylist will be returned
    
    """
    n = len(xlist)
    
    if xlist[0] < xlist[-1]: # ascending list
        i = bisect.bisect_left(xlist, x, hi=n - 1) - 1

        if x < xlist[0] or i < 0:
            y = ylist[0]
            
        elif x > xlist[-1] or i>n-2:
            y = ylist[n - 1]

        else:
            y = (ylist[i]*(xlist[i + 1] - x) + ylist[i + 1]*(x - xlist[i])) / (xlist[i + 1] - xlist[i])
            
            
    else: # descending list
        i = n - bisect.bisect_left(xlist[::-1], x, hi=n - 1) - 1

        if x > xlist[0] or i < 0:
            y = ylist[0]
            
        elif x < xlist[-1] or i>n-2:
            y = ylist[n - 1]
            
        else:
            y = (ylist[i]*(xlist[i + 1] - x) + ylist[i + 1]*(x - xlist[i])) / (xlist[i + 1] - xlist[i])
            
    return y

