from datetime import datetime, date, timedelta, time
from calendar import isleap
REFERENCEYEAR = 2007             # year starting with a Monday
REFERENCEDATE = date(REFERENCEYEAR, 1, 1)
FIRSTOFYEAR = date(REFERENCEYEAR, 1, 1)
LASTOFYEAR = date(REFERENCEYEAR, 12, 31)
MONTHSTARTDAY = [0,31,59,90,120,151,181,212,243,273,304,334,365]

MONTHLYPERIODS = [(date(REFERENCEYEAR, m+1, 1),
                   date(REFERENCEYEAR, m+1, MONTHSTARTDAY[m+1]-MONTHSTARTDAY[m])) for m in xrange(12)]                  # returns tuples of the first and last day of the month in date-format


DEFAULT_FORMAT_DATETIME = "%Y-%m-%d %H:%M"
MIDNIGHT = time(23, 59, 59)  # this is a hack, because 24:00 cannot be represented in datetime.time
MIDNIGHT_AS_ZERO = time(0, 0, 0)

#--------------------------------------------------------------------------
# Datetime - TIMEDELTA / TIME - tools. Conversion to hours
#--------------------------------------------------------------------------

def timeDeltaToHours(td):
##    return float(td.days*24. + td.hours + td.minutes/60. + td.seconds/3600.0)
    return float(td.days*24. + td.seconds/3600.0)

def hoursToTimeDelta(dt):
    return timedelta(hours=int(dt), minutes=int((dt-int(dt))*60 + 0.5))


def timeToHour(time): 
    """Return the float represenation of the given datetime.time object.
    
    e.g. datetime.time(3,30) becomes 3.5
    """
    if time==MIDNIGHT:
        return 24.0
    else:
        return time.hour + time.minute / 60.0
  
#--------------------------------------------------------------------------
# Conversion from to day-of-the-year (DOY) or hour of the year (HOY)
#--------------------------------------------------------------------------

##def dateToDOY(dd, reference=REFERENCEDATE):
##    td = dd - reference
##    return td.days
##
##def datetimeToHOY(dt, reference=REFERENCEDATE):
##    return timeDeltaToHours(dt-reference)
##    
##
def dateToDOY(dd, noleap=True):
    """ returns the day of the year from 0 to 364

    noleap=True: matches 28.02 and 29.02 for leap years
    """
    
    try:
        doy = dd.timetuple().tm_yday - 1
    except:
        return None
    
    if noleap and isleap(dd.year) and doy > 60: # combine 29.02 and 01.03 for leap years
        doy -= 1
    return doy

def DOYToDate(doy):
    """
    doy = day of the year from 0 to 364
    """
    return date.fromordinal(FIRSTOFYEAR.toordinal() + doy)


def dateTimeToHOY(dt, noleap=True):
    """ returns the day of the year from 0 to 364

    noleap=True: matches 28.02 and 29.02 for leap years
    """
    doy = dateToDOY(dt, noleap)
    hod = timeToHour(dt.time())
    return doy*24.0 + hod

def dateTimeToH(dt):
    """
    like dateTimeToHoy within the reference year, but allows for multiple years
    """
    day = dt.toordinal() - FIRSTOFYEAR.toordinal()
    hod = timeToHour(dt.time())
    return day*24.0 + hod

def dateToHOY(dd, startHour=True, noleap=True):
    """ returns the day of the year from 0 to 364

    noleap=True: matches 28.02 and 29.02 for leap years
    """
    doy = dateToDOY(dd, noleap)
    if doy is None:
        return None
    
    if not startHour: # shift one day if the stopHour is required 
        doy += 1
    
    return doy*24

#--------------------------------------------------------------------------
# Conversion from date / datetime to string (dd-mm)
#--------------------------------------------------------------------------

def dateToString(date):
    try:
        return date.strftime("%d-%m")
    except:
        return "-----"

def dateTimeToString(datetime):
    try:
        return datetime.strftime(DEFAULT_FORMAT_DATETIME)
    except:
        return "-----"

def dateToString3(date):
    try:
        return date.strftime("%d-%m-%Y")
    except:
        return "----------"

def timeToString(time):
    if time==MIDNIGHT:
        return "24:00"
    else:
        try:
            return time.strftime("%H:%M")
        except:
            return "----------"
    

def stringToDate(string):
    data = string.split("-")
    try:
        result = date(REFERENCEYEAR, int(data[1]), int(data[0]))
    except:
        result = None
    return result

def stringToDate3(string):
    try:
        return datetime.strptime(string, "%d-%m-%Y").date()
    except:
        return None

def stringToDateTime(string):
    try:
        return datetime.strptime(string, DEFAULT_FORMAT_DATETIME)
    except:
        return None

def stringToTime(string):
    data = string.split(":")
    try:
        if int(data[0])==24 and int(data[1])==0:
            return MIDNIGHT
        else:
            return time(int(data[0]), int(data[1]))
    except:
        return None
    

def dateToDateTime(date):
    return None if date is None else datetime(date.year, date.month, date.day)





























if __name__=="__main__":
    print dateToDOY(date(2007,1,1)), \
          dateToDOY(date(2007, 1, 31)), \
          dateToDOY(date(2007, 12, 31))         # 0, 30, 364

    print datetimeToHOY(date(2007,2,1))         # 744

