#!/usr/bin/env python
# -*- coding: utf-8 -*-
#==============================================================================
#
#   CONFIG PARSER
#
#==============================================================================
from ConfigParser import ConfigParser, MissingSectionHeaderError
from ast import literal_eval
import time

class Config():
    def __init__(self, filePath, default=None):
        self.config = ConfigParser()
        self.filePath = filePath
        self.err = 0
        try:
            self.file = open(self.filePath, "r")
        except:
            if default:
                self.file = open(default, "r")
            else:
                self.file = None

        if self.file:
            try:
                self.config.readfp(self.file)
            except MissingSectionHeaderError:
                self.err = 1
                self.file.close()
                return
            
            self.file.close()
            self.write()    # in case of default file: create a copy in filePath

        else:
            self.file = None

            

    def get(self, section, option):
        """
        Gets the value of some option
        (expressions are interpreted by ast.literal_eval)
        Returns None if the option does not exist
        """
        if self.config.has_option(section, option):
            str = self.config.get(section, option)
            try:
                return literal_eval(str)
            except:
                return str
        else:
            return None

    def set(self, section, option, value):
        """
        Sets the value of some option
        Creates a new section / option if not existing
        """
        
        if section not in self.config.sections():
            self.config.add_section(section)

        self.config.set(section, option, str(value))
            
        
    def write(self):
        tStr = time.strftime("%Y-%m-%d %H:%M:%S")
        self.set("time", "saved at", tStr)
            
        self.file = open(self.filePath, "w")
        self.config.write(self.file)
        self.file.close()

    def __repr__(self):

        res = ""
        sections = self.config.sections()

        for section in sections:
            res += "Section %s:"%section
            options = self.config.items(section)
            res += "%s\n"%options

        return res

if __name__=="__main__":
    state = Config("state.txt")

    print "State after init: %s"%state
    
    state.set("drive", "time", "2017-09-27 16:11:04")
    state.set("drive","pos", (288, -213000))

    state.set("site", "lat", 50)
    state.set("site", "long", 0)
    state.set("site", "TZ", 1)

    state.set("orientation", "axis", (15, 0))
    state.set("orientation", "tower", (30, 45))

    print "state after setting sections: %s"%state

    print "pos: %s"%repr(state.get("drive", "pos"))
            

    state.write()



# Test 1: Config file exists

    state = Config("state.txt")
    print "Sections in config file: %s"%state.config.sections()
