from scipy.linalg import eigvalsh, eigh
import numpy as np

def shift(xo, dxi, i):
    """
    helper function
    check, maybe there's some standard in numpy to do this
    """
    x = xo.copy()
    x[i] += dxi
    return x

def hessian(f, xo, diag):
    """
    calculates the hessian matrix at xo
    (h_ij = d2f/dxi_dxj)

    dxi/dxj are normalised by diag: cdx = dxi*diag
    """
    n = len(xo)
    h = np.zeros((n, n), dtype=float)

    cdx = 0.001
    dx = cdx/diag

    fo = f(xo)

    for i in range(n):
        x_E = shift(xo,  dx[i], i)
        x_W = shift(xo, -dx[i], i)

##        print "xE: %s xW: %s"%(x_E, x_W)
##
        for j in range(i):
            x_NE = shift(x_E,  dx[j], j)
            x_NW = shift(x_W,  dx[j], j)
            x_SE = shift(x_E, -dx[j], j)
            x_SW = shift(x_W, -dx[j], j)

##            print "x_NE %s x_NW %s x_SE %s x_SW %s"%(x_NE, x_NW, x_SE, x_SW)
##
            f_NE = f(x_NE)
            f_NW = f(x_NW)
            f_SE = f(x_SE)
            f_SW = f(x_SW)

##            print "f_NE %s f_NW %s f_SE %s f_SW %s"%(f_NE, f_NW, f_SE, f_SW)

            grad_iN = (f_NE - f_NW) / (2*cdx)
            grad_iS = (f_SE - f_SW) / (2*cdx)

##            print "grad_iN %s grad_iS %s"%(grad_iN, grad_iS)

            grad_jE = (f_NE - f_SE) / (2*cdx)
            grad_jW = (f_NW - f_SW) / (2*cdx)

            h[i, j] = (grad_iN - grad_iS) / (2*cdx)
            h[j, i] = (grad_jE - grad_jW) / (2*cdx)

        f_E = f(x_E)
        f_W = f(x_W)

        grad_iE = (f_E - fo) / cdx
        grad_iW = (fo - f_W) / cdx

        h[i][i] = (grad_iE - grad_iW) / cdx

    return h

def test_hessian(h):
    """
    Calculates the minimum eigen value of the hessian
    """
    # check if there is any row without sensitivity at all (hij = 0 for al j)

    hmax = np.max(np.abs(h), axis=0)
    remove = [i for i, val in enumerate(hmax) if val < 1.e-15]
    hmod = np.delete(h, remove, axis=0)
    hmod = np.delete(hmod, remove, axis=1)

    sqdev = None
    
    try:
##        eigen = eigvalsh(hmod)
        eigen, eigV = eigh(hmod)
        if np.min(eigen) > 0:
            sqdev = 0.0*eigen
            for i, val in enumerate(eigen):
                v = np.abs(eigV[i])
                sqdev += v/val
    except:
        return None, None

    return np.min(eigen), sqdev


FTYPE = 5
def f(x):
    if FTYPE==1:
        a = 1.0
        b = 3.0
        return pow(a*x[0], 2) + pow(b*x[1], 2)

    elif FTYPE==2:
        a = 1.0
        b = 3.0
        return 0.5*pow(a*(x[0]+x[1]), 2) + 0.5*pow(b*(x[0]-x[1]), 2)

    elif FTYPE==3:
        a = 1.0
        return pow(a*x[0], 2)

    elif FTYPE==4:
        a = 1.0
        return 0.5*pow(a*(x[0]+x[1]), 2)

    elif FTYPE==5:
        a = 1.0
        return 0.5*pow(a*(x[0]-x[1]), 2)

    elif FTYPE==6:  # saddle point
        a = 1.0
        b = 3.0
        return 0.5*pow(a*(x[0]+x[1]), 2) - 0.5*pow(b*(x[0]-x[1]), 2)


##    a = 1
##    b = 2
##    c = 0.2
##    return a*pow(c*x[0] + (1-c)*x[1], 2) + b*pow((c-1)*x[0] + c*x[1], 2)

if __name__=="__main__":

    for FTYPE in range(1, 7):
        print "FTYPE: %s"%FTYPE
        xo = np.array([0.0, 0.0], dtype=float)
        dx = np.array([0.1, 0.1], dtype=float)
        hess = hessian(f, xo, dx)
        print "Hessian Matrix: %s"%hess

        print "Eigenvalues: %s"%eigvalsh(hess)
        print "-"

        print eigh(hess)


#### NOTE for convergence test


