import numpy as np
from lib.auxiliary.interpolate import interpolateTable

def crossCorrelation(x, y, n):
    """ calculates the covariance
        two data series x and xref of length n
    """

    cc = []
    
    shifts = range(n+1) + range(-1, -n-1, -1)

    for shift in shifts:
        if shift == 0:
            xp = xo
            xrefp = xrefo
        elif shift < 0:
            xp = xo[:shift]
            xref = xrefo[-shift:]
        else:
            xp = xo[shift:]
            xrefp = xrefo[:-shift]
            
        cc.append(np.correlate(x,xref))
        
    return cc

def autoCorrelation_t(x, t, tstep=None, dtmax=None):
    """ calculates the auto-correlation of data
    in non-regular time intervals

    x: vector of data
    t: corresponding time
    tstep: intervals of time for autocorrelation
    dtmax: maximum time shift for autocorrelation

    Results:
    NT: period of cycle in number of intervals
    ac: 
    """
    t0 = t[0]
    t1 = t[-1]
    n = len(t)

    if tstep is None: tstep = (t1-t0)/(n-1)
    if dtmax is None: dtmax = 0.5*(t1-t0)

    imax = int(dtmax/tstep)

    ac = []
    for i in xrange(0, imax+1):
        dt = tstep*i
        t0xp = t0 + dt
        t1xpref = t1 - dt

##        tp = np.array([val for val in t if val >= t0xp])
        ntp = np.searchsorted(t, t0xp)
##        ntp = len(tp)
        xp = x[ntp:]
        tp = t[ntp:]

        xpref = np.interp(tp-dt, t, x)
##        xpref = np.array([interpolateTable(tau-dt, t, x) for tau in tp])

        ac.append([dt, np.correlate(xp, xpref)])

    return ac
    
    

def checkAutoCorrelation(ac):
    """
    Looks for the first maximum within the autocorrelation

    Results:
    NT: period of cycle in number of intervals
    """
    n = len(ac)
    dac = ac[1:] - ac[:-1]
    minima = np.logical_and(np.less_equal(dac[:-1], 0), np.greater_equal(dac[1:], 0))
    maxima = np.logical_and(np.greater_equal(dac[:-1], 0), np.less_equal(dac[1:], 0))

    if np.nonzero(minima)[0].size > 0:
        imin = np.nonzero(minima)[0][0]
        Tmin = (imin+1)
    else:
        Tmin = None

    if np.nonzero(maxima)[0].size > 0:
        imax = np.nonzero(maxima)[0][0]
        Tmax = (imax+1)
    else:
        Tmax = None

    if Tmax:
        NT = Tmax
    elif Tmin:
        NT = 2*Tmin
    else:
        NT = None
        
    
    return NT



