
#----------------------------------------------------------------------
# Installer for Einstein Thermal Energy Audit Program
Name EINSTEIN
#----------------------------------------------------------------------

# General Symbol Definitions
!define REGKEY "SOFTWARE\$(^Name)"

# Installer attributes
OutFile ..\..\..\EINSTEIN_3.0plus_windows.exe
InstallDir $PROGRAMFILES\einstein\einstein3.0plus
CRCCheck on
XPStyle on
ShowInstDetails hide
InstallDirRegKey HKLM "${REGKEY}" Path
ShowUninstDetails hide
SetCompressor /SOLID lzma

#----------------------------------------------------------------------


# Definition of the MUI Installer Options
# Customizes Look and Feel
!define MUI_FINISHPAGE_NOAUTOCLOSE
!define MUI_FINISHPAGE_RUN "$INSTDIR\EINSTEIN.bat"
!define MUI_UNFINISHPAGE_NOAUTOCLOSE
!define MUI_WELCOMEFINISHPAGE_BITMAP "res\installtitle.bmp"
!define MUI_COMPONENTSPAGE_SMALLDESC

!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_BITMAP "res\einstein_logo.bmp"

#----------------------------------------------------------------------
# Include Files and Librarys

# Look and Feel of the Installer
!include MUI2.nsh
!include InstallOptions.nsh
!include nsDialogs.nsh

# Logic and various
!include Sections.nsh
!include LogicLib.nsh

#----------------------------------------------------------------------

# Installerpages with the correct order and custom pages
# Default pages are part of Modern User Interface 2 (MUI 2) or nsDialogs
!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_LICENSE "res\EINSTEIN-Plus-License.rtf"
!insertmacro MUI_PAGE_COMPONENTS
### Page custom nsAccountsetting nsAccountsettingLeave
### Page custom nsInstallPath nsPathLeave
!insertmacro MUI_PAGE_DIRECTORY
!insertmacro MUI_PAGE_INSTFILES
!insertmacro MUI_PAGE_FINISH
!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES

# Installer languages
!insertmacro MUI_LANGUAGE English

Var main.dir.path

Var sqlite.dbname
Var sqlite.einstein.exists
Var sqlite.selected

Var Windows.Installer.MajorVersion
Var Windows.Installer.MinorVersion



### !macro CheckMySqlReg _RESULT _RESULT2

#----------------------------------------------------------------------

### !macro CheckforEinsteinDB _RESULT _UNAME _PW

#----------------------------------------------------------------------
!macro CheckMSIInstallerVersion
	GetDllVersion "$SYSDIR\msi.dll" $R0 $R1
	IntOp $R2 $R0 >> 16
	IntOp $R2 $R2 & 0x0000FFFF # major version
	IntOp $R3 $R0 & 0x0000FFFF # minor version
	StrCpy $Windows.Installer.MajorVersion $R2
	StrCpy $Windows.Installer.MinorVersion $R3
!macroend

#----------------------------------------------------------------------
Function .onInit
	InitPluginsDir
	StrCpy $main.dir.path $PROGRAMFILES 3
	StrCpy $sqlite.dbname "einstein"
	StrCpy $sqlite.einstein.exists "0"
	StrCpy $sqlite.selected "1"
	!insertmacro CheckMSIInstallerVersion
	${If} $Windows.Installer.MajorVersion < 3
		MessageBox MB_OK "Your Version of Windows Installer is too old. Please upgrade to Version 3.1 or newer!"
		Abort
	${ElseIf} $Windows.Installer.MajorVersion >= 4
	${ElseIf} $Windows.Installer.MinorVersion < 1
		MessageBox MB_OK "Your Version of Windows Installer is too old. Please upgrade to Version 3.1 or newer!"
		Abort
	${EndIf}
FunctionEnd

#----------------------------------------------------------------------
Function WriteToFile
 Exch $0 ;file to write to
 Exch
 Exch $1 ;text to write
 
  FileOpen $0 $0 a #open file
   FileSeek $0 0 END #go to end
   FileWrite $0 $1 #write to file
  FileClose $0
 
 Pop $1
 Pop $0
FunctionEnd

#----------------------------------------------------------------------
#Begin of Installer Sections

Section -Main SEC0000
    SetOutPath $INSTDIR
    SetOverwrite on
	
	# Remove files from EINSTEIN pre 1.2 installation (non-installer)
	IfFileExists "$INSTDIR\einstein\*.*" found
	Goto notfound
	found:
	RMDir /r /REBOOTOK "$INSTDIR\einstein"
	notfound:
	Delete "$INSTDIR\EINSTEIN.bat"
	Delete "$INSTDIR\ReadMe.txt"
	Delete "$INSTDIR\checkInstallation.bat"
	Delete "$INSTDIR\install_einsteinDB.bat"
	Delete "$INSTDIR\update_einsteinDB.bat"
	
	File /r ..\..\*
	
    WriteRegStr HKLM "${REGKEY}\Components" Main 1

SectionEnd

#----------------------------------------------------------------------

Section "EINSTEIN" einsteinsection
	SectionIn RO
	
	# try patch the visual c++ libraries (may be necessary on WinXP)
	SetOutPath "$INSTDIR\Prerequisites\"
	File "..\Prerequisites\vcredist_x86.exe"
    ExecWait '"$INSTDIR\Prerequisites\vcredist_x86" /q'
	
	SetOutPath "$INSTDIR\Prerequisites\"
	File "..\Prerequisites\writeconfig.py"
	Push '"$INSTDIR\Python26Einstein\python.exe" "$INSTDIR\Prerequisites\writeconfig.py" "$INSTDIR"'
	Push "$INSTDIR\updateconf.bat"
	Call WriteToFile
	ExecWait "$INSTDIR\updateconf.bat"
	Delete "$INSTDIR\updateconf.bat"

	Push 'IF NOT EXIST "%appdata%\einstein" mkdir "%appdata%\einstein" $\r$\n \
		 IF NOT EXIST "%appdata%\einstein\einstein3.0plus" mkdir "%appdata%\einstein\einstein3.0plus" $\r$\n \
	     IF NOT EXIST "%appdata%\einstein\einstein3.0plus\einstein.db" copy "$INSTDIR\sql\einstein.db" "%appdata%\einstein\einstein3.0plus\einstein.db"'
	Push "$INSTDIR\copyDB.bat"
	Call WriteToFile
	ExecWait "$INSTDIR\copyDB.bat"
 	Delete "$INSTDIR\copyDB.bat"

SectionEnd

#----------------------------------------------------------------------

Section -finish

    RmDir /r "$INSTDIR\Prerequisites"
    RmDir /r "$INSTDIR\Install"
    Delete "$INSTDIR\EINSTEIN.bat"
    Push 'IF NOT EXIST "%appdata%\einstein" mkdir "%appdata%\einstein" $\n \
         cd  /D "$INSTDIR\src\einstein" $\n \
         ..\..\Python26Einstein\python.exe -O einstein.py > "%appdata%\einstein\einsteinMSG.txt" 2> "%appdata%\einstein\einsteinLOG.txt" $\n \
         cd ..\..\..'
    Push "$INSTDIR\EINSTEIN.bat"
    Call WriteToFile

    createShortCut "$DESKTOP\EINSTEIN.lnk" "$INSTDIR\EINSTEIN.bat" "" "$INSTDIR\src\einstein\GUI\img\einstein.ico" 0
    Call CreateStartMenu
    
SectionEnd
#----------------------------------------------------------------------

Function CreateStartMenu
  CreateDirectory "$SMPROGRAMS\EINSTEIN"
  CreateShortCut "$SMPROGRAMS\EINSTEIN\Uninstall.lnk" "$INSTDIR\uninstall.exe" "" "$INSTDIR\uninstall.exe" 0
  CreateShortCut "$SMPROGRAMS\EINSTEIN\EINSTEIN.lnk" "$INSTDIR\EINSTEIN.bat" "" "$INSTDIR\GUI\img\einstein.ico" 0
FunctionEnd
#----------------------------------------------------------------------

Section -post SEC0001
    WriteRegStr HKLM "${REGKEY}" Path $INSTDIR
    SetOutPath $INSTDIR
    WriteUninstaller $INSTDIR\uninstall.exe
    WriteRegStr HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" DisplayName "$(^Name)"
    WriteRegStr HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" DisplayIcon $INSTDIR\uninstall.exe
    WriteRegStr HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" UninstallString $INSTDIR\uninstall.exe
    WriteRegDWORD HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" NoModify 1
    WriteRegDWORD HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" NoRepair 1
SectionEnd

#----------------------------------------------------------------------
# Macro for selecting uninstaller sections
!macro SELECT_UNSECTION SECTION_NAME UNSECTION_ID
    Push $R0
    ReadRegStr $R0 HKLM "${REGKEY}\Components" "${SECTION_NAME}"
    StrCmp $R0 1 0 next${UNSECTION_ID}
    !insertmacro SelectSection "${UNSECTION_ID}"
    Goto done${UNSECTION_ID}
	next${UNSECTION_ID}:
    !insertmacro UnselectSection "${UNSECTION_ID}"
	done${UNSECTION_ID}:
    Pop $R0
!macroend


#----------------------------------------------------------------------

# Uninstaller sections
Section /o -un.Main UNSEC0000

	# white list of files to delete
	# generate by: git ls-tree -r --name-only <tree-ish>
	# remove GUI/einstein.ini
	Delete /REBOOTOK "$INSTDIR\.gitignore"
	Delete /REBOOTOK "$INSTDIR\Install/Prerequisites/writeconfig.py"
	Delete /REBOOTOK "$INSTDIR\Install/Prerequisites/writeupdate.py"
	Delete /REBOOTOK "$INSTDIR\Install/src/Readme.txt"
	Delete /REBOOTOK "$INSTDIR\Install/src/changelog.txt"
	Delete /REBOOTOK "$INSTDIR\Install/src/res/EINSTEIN-Plus-License.rtf"
	Delete /REBOOTOK "$INSTDIR\Install/src/res/Thumbs.db"
	Delete /REBOOTOK "$INSTDIR\Install/src/res/einstein_logo.bmp"
	Delete /REBOOTOK "$INSTDIR\Install/src/res/installtitle.bmp"
	Delete /REBOOTOK "$INSTDIR\Install/src/src.nsi"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/_bsddb.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/_ctypes.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/_ctypes_test.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/_elementtree.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/_hashlib.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/_msi.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/_multiprocessing.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/_socket.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/_sqlite3.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/_ssl.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/_testcapi.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/_tkinter.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/bz2.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/py.ico"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/pyc.ico"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/pyexpat.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/select.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/sqlite3.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/tcl85.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/tclpip85.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/tk85.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/unicodedata.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/DLLs/winsound.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Doc/python266.chm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/LICENSE.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/BaseHTTPServer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/Bastion.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/CGIHTTPServer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ConfigParser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/Cookie.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/DocXMLRPCServer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/HTMLParser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/MimeWriter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/Queue.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/SimpleHTTPServer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/SimpleXMLRPCServer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/SocketServer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/StringIO.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/UserDict.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/UserList.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/UserString.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/_LWPCookieJar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/_MozillaCookieJar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/__future__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/__phello__.foo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/_abcoll.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/_strptime.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/_threading_local.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/abc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/aifc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/anydbm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/argparse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ast.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/asynchat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/asyncore.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/atexit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/audiodev.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/base64.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bdb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/binhex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bisect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/db.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/dbobj.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/dbrecio.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/dbshelve.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/dbtables.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/dbutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_all.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_associate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_basics.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_compare.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_compat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_cursor_pget_bug.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_dbobj.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_dbshelve.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_dbtables.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_distributed_transactions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_early_close.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_get_none.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_join.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_lock.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_misc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_pickle.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_queue.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_recno.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_replication.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_sequence.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/bsddb/test/test_thread.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/cProfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/calendar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/cgi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/cgitb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/chunk.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/cmd.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/code.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/codecs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/codeop.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/collections.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/colorsys.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/commands.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/compileall.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/compiler/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/compiler/ast.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/compiler/consts.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/compiler/future.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/compiler/misc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/compiler/pyassem.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/compiler/pycodegen.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/compiler/symbols.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/compiler/syntax.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/compiler/transformer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/compiler/visitor.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/contextlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/cookielib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/copy.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/copy_reg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/csv.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/_endian.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/macholib/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/macholib/dyld.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/macholib/dylib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/macholib/framework.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/runtests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_anon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_array_in_pointer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_arrays.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_as_parameter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_bitfields.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_buffers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_byteswap.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_callbacks.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_cast.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_cfuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_checkretval.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_delattr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_errcheck.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_errno.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_find.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_frombuffer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_funcptr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_functions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_incomplete.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_init.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_integers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_internals.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_keeprefs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_libc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_loading.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_macholib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_memfunctions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_numbers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_objects.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_parameters.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_pep3118.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_pickling.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_pointers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_prototypes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_python_api.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_random_things.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_refcounts.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_repr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_returnfuncptrs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_simplesubclasses.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_sizes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_slicing.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_stringptr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_strings.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_struct_fields.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_structures.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_unaligned_structures.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_unicode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_values.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_varsize_struct.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/test/test_win32.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ctypes/wintypes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/curses/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/curses/ascii.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/curses/has_key.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/curses/panel.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/curses/textpad.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/curses/wrapper.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/dbhash.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/decimal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/difflib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/dircache.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/dis.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/archive_util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/bcppcompiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/ccompiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/cmd.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/bdist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/bdist_dumb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/bdist_msi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/bdist_rpm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/bdist_wininst.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/build.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/build_clib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/build_ext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/build_py.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/build_scripts.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/clean.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/config.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/install.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/install_data.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/install_egg_info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/install_headers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/install_lib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/install_scripts.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/register.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/sdist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/upload.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/wininst-6.0.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/wininst-7.1.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/wininst-8.0.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/wininst-9.0-amd64.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/command/wininst-9.0.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/config.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/core.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/cygwinccompiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/debug.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/dep_util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/dir_util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/dist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/emxccompiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/errors.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/extension.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/fancy_getopt.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/file_util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/filelist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/log.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/msvc9compiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/msvccompiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/mwerkscompiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/spawn.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/sysconfig.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/setuptools_build_ext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/setuptools_extension.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/support.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_bdist_wininst.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_build_ext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_build_py.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_build_scripts.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_config.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_core.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_dist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_filelist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_install.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_install_lib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_install_scripts.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_msvc9compiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_register.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_sdist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_sysconfig.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_unixccompiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_upload.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/tests/test_versionpredicate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/text_file.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/unixccompiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/version.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/distutils/versionpredicate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/doctest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/dumbdbm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/dummy_thread.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/dummy_threading.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/_parseaddr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/base64mime.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/charset.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/encoders.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/errors.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/feedparser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/generator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/header.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/iterators.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/message.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/mime/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/mime/application.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/mime/audio.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/mime/base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/mime/image.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/mime/message.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/mime/multipart.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/mime/nonmultipart.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/mime/text.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/parser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/quoprimime.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/PyBanner048.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/audiotest.au"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_01.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_02.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_03.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_04.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_05.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_06.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_07.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_08.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_09.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_10.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_11.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_12.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_12a.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_13.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_14.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_15.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_16.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_17.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_18.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_19.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_20.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_21.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_22.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_23.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_24.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_25.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_26.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_27.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_28.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_29.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_30.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_31.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_32.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_33.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_34.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_35.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_36.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_37.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_38.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_39.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_40.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_41.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_42.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_43.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_44.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_45.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/data/msg_46.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/test_email.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/test_email_codecs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/test_email_codecs_renamed.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/test_email_renamed.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/test/test_email_torture.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/email/utils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/aliases.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/ascii.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/base64_codec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/big5.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/big5hkscs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/bz2_codec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/charmap.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp037.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp1006.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp1026.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp1140.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp1250.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp1251.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp1252.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp1253.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp1254.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp1255.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp1256.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp1257.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp1258.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp424.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp437.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp500.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp737.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp775.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp850.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp852.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp855.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp856.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp857.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp860.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp861.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp862.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp863.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp864.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp865.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp866.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp869.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp874.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp875.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp932.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp949.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/cp950.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/euc_jis_2004.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/euc_jisx0213.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/euc_jp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/euc_kr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/gb18030.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/gb2312.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/gbk.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/hex_codec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/hp_roman8.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/hz.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/idna.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso2022_jp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso2022_jp_1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso2022_jp_2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso2022_jp_2004.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso2022_jp_3.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso2022_jp_ext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso2022_kr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_10.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_11.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_13.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_14.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_15.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_16.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_3.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_4.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_5.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_6.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_7.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_8.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/iso8859_9.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/johab.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/koi8_r.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/koi8_u.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/latin_1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/mac_arabic.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/mac_centeuro.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/mac_croatian.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/mac_cyrillic.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/mac_farsi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/mac_greek.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/mac_iceland.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/mac_latin2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/mac_roman.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/mac_romanian.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/mac_turkish.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/mbcs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/palmos.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/ptcp154.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/punycode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/quopri_codec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/raw_unicode_escape.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/rot_13.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/shift_jis.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/shift_jis_2004.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/shift_jisx0213.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/string_escape.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/tis_620.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/undefined.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/unicode_escape.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/unicode_internal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/utf_16.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/utf_16_be.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/utf_16_le.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/utf_32.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/utf_32_be.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/utf_32_le.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/utf_7.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/utf_8.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/utf_8_sig.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/uu_codec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/encodings/zlib_codec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/filecmp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/fileinput.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/fnmatch.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/formatter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/fpformat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/fractions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ftplib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/functools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/genericpath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/getopt.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/getpass.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/gettext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/glob.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/gzip.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/hashlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/heapq.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/hmac.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/hotshot/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/hotshot/log.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/hotshot/stats.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/hotshot/stones.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/htmlentitydefs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/htmllib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/httplib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/AutoComplete.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/AutoCompleteWindow.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/AutoExpand.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/Bindings.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/CREDITS.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/CallTipWindow.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/CallTips.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/ClassBrowser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/CodeContext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/ColorDelegator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/Debugger.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/Delegator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/EditorWindow.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/FileList.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/FormatParagraph.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/GrepDialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/HISTORY.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/HyperParser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/IOBinding.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/Icons/folder.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/Icons/idle.icns"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/Icons/minusnode.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/Icons/openfolder.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/Icons/plusnode.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/Icons/python.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/Icons/tk.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/IdleHistory.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/MultiCall.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/MultiStatusBar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/NEWS.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/ObjectBrowser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/OutputWindow.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/ParenMatch.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/PathBrowser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/Percolator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/PyParse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/PyShell.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/RemoteDebugger.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/RemoteObjectBrowser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/ReplaceDialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/ScriptBinding.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/ScrolledList.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/SearchDialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/SearchDialogBase.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/SearchEngine.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/StackViewer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/TODO.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/ToolTip.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/TreeWidget.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/UndoDelegator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/WidgetRedirector.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/WindowList.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/ZoomHeight.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/aboutDialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/config-extensions.def"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/config-highlight.def"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/config-keys.def"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/config-main.def"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/configDialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/configHandler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/configHelpSourceEdit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/configSectionNameDialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/dynOptionMenuWidget.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/extend.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/help.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/idle.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/idle.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/idle.pyw"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/idlever.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/keybindingDialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/macosxSupport.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/rpc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/run.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/tabbedpages.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/testcode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/idlelib/textView.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ihooks.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/imaplib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/imghdr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/imputil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/inspect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/io.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/decoder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/encoder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/scanner.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_decode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_default.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_dump.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_encode_basestring_ascii.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_fail.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_float.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_indent.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_pass1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_pass2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_pass3.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_recursion.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_scanstring.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_separators.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_speedups.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tests/test_unicode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/json/tool.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/keyword.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/Canvas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/Dialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/FileDialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/FixTk.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/ScrolledText.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/SimpleDialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/Tix.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/Tkconstants.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/Tkdnd.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/Tkinter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/tkColorChooser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/tkCommonDialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/tkFileDialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/tkFont.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/tkMessageBox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/tkSimpleDialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib-tk/turtle.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/Grammar.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/PatternGrammar.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixer_base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixer_util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_apply.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_basestring.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_buffer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_callable.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_dict.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_except.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_exec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_execfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_exitfunc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_filter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_funcattrs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_future.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_getcwdu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_has_key.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_idioms.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_import.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_imports.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_imports2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_input.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_intern.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_isinstance.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_itertools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_itertools_imports.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_long.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_map.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_metaclass.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_methodattrs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_ne.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_next.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_nonzero.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_numliterals.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_operator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_paren.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_print.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_raise.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_raw_input.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_reduce.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_renames.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_repr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_set_literal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_standarderror.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_sys_exc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_throw.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_tuple_params.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_types.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_unicode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_urllib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_ws_comma.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_xrange.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_xreadlines.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/fixes/fix_zip.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/main.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/patcomp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/pgen2/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/pgen2/conv.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/pgen2/driver.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/pgen2/grammar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/pgen2/literals.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/pgen2/parse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/pgen2/pgen.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/pgen2/token.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/pgen2/tokenize.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/pygram.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/pytree.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/refactor.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/bom.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/crlf.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/different_encoding.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/fixers/bad_order.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/fixers/myfixes/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/fixers/myfixes/fix_explicit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/fixers/myfixes/fix_first.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/fixers/myfixes/fix_last.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/fixers/myfixes/fix_parrot.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/fixers/myfixes/fix_preorder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/fixers/no_fixer_cls.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/fixers/parrot_example.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/infinite_recursion.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/py2_test_grammar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/data/py3_test_grammar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/pytree_idempotency.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/support.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/test_all_fixers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/test_fixers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/test_main.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/test_parser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/test_pytree.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/test_refactor.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/lib2to3/tests/test_util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/linecache.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/locale.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/logging/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/logging/config.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/logging/handlers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/macpath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/macurl2path.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/mailbox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/mailcap.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/markupbase.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/md5.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/mhlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/mimetools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/mimetypes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/mimify.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/modulefinder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/msilib/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/msilib/schema.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/msilib/sequence.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/msilib/text.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multifile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/connection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/dummy/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/dummy/connection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/forking.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/heap.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/managers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/pool.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/process.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/queues.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/reduction.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/sharedctypes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/synchronize.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/multiprocessing/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/mutex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/netrc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/new.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/nntplib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ntpath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/nturl2path.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/numbers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/opcode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/optparse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/os.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/os2emxpath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/pdb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/pickle.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/pickletools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/pipes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/pkgutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/platform.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/plistlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/popen2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/poplib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/posixfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/posixpath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/pprint.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/profile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/pstats.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/pty.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/py_compile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/pyclbr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/pydoc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/pydoc_topics.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/quopri.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/random.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/re.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/repr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/rexec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/rfc822.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/rlcompleter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/robotparser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/runpy.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sched.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sets.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sgmllib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sha.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/shelve.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/shlex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/shutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQL_python-1.2.2-py2.6.egg-info/PKG-INFO"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQL_python-1.2.2-py2.6.egg-info/SOURCES.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQL_python-1.2.2-py2.6.egg-info/dependency_links.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQL_python-1.2.2-py2.6.egg-info/top_level.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/connections.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/constants/CLIENT.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/constants/CR.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/constants/ER.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/constants/FIELD_TYPE.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/constants/FLAG.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/constants/REFRESH.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/constants/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/converters.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/cursors.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/release.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/MySQLdb/times.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/PyWin32.chm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/_mysql.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/_mysql_exceptions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/adodbapi/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/adodbapi/ado_consts.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/adodbapi/adodbapi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/adodbapi/license.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/adodbapi/readme.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/adodbapi/tests/adodbapitest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/adodbapi/tests/adodbapitestconfig.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/adodbapi/tests/db_print.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/adodbapi/tests/dbapi20.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/adodbapi/tests/testADOdbapi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/adodbapi/tests/testADOdbapiConfig.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/adodbapi/tests/test_adodbapi_dbapi20.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/dateutil/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/dateutil/easter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/dateutil/parser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/dateutil/relativedelta.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/dateutil/rrule.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/dateutil/tz.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/dateutil/tzwin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/dateutil/zoneinfo/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/dateutil/zoneinfo/zoneinfo-2010g.tar.gz"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/api.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/compound.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/context.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/declarative.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/doctest_xml_compare.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/fieldstorage.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/foreach.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/htmlfill.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/htmlfill_schemabuilder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/htmlgen.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/htmlrename.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/interfaces.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/javascript/ordering.js"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/national.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/rewritingparser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/schema.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/util/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/util/doctest24.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/util/threadinglocal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/validators.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/formencode/variabledecode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/PyISAPI_loader.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/doc/isapi.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/install.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/isapicon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/samples/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/samples/advanced.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/samples/redirector.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/samples/redirector_asynch.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/samples/redirector_with_filter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/samples/test.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/simple.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/_socket.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/_sre.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/_ssl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/_win32sysloader.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/_winreg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/datetime.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/unicodedata.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/win32api.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/win32event.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/win32file.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/win32security.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/win32trace.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/win32ui.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/build/bdist.win32/winexe/temp/zlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/extension_simple.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/test/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/isapi/threaded_extension.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib-1.0.1-py2.6.egg-info"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/_cm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/_cntr.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/_delaunay.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/_image.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/_mathtext_data.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/_path.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/_png.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/_pylab_helpers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/_tri.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/_windowing.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/afm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/artist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/axes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/axis.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backend_bases.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/Matplotlib.nib/classes.nib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/Matplotlib.nib/info.nib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/Matplotlib.nib/keyedobjects.nib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/_backend_agg.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/_backend_gdk.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/_gtkagg.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/_tkagg.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_agg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_cairo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_cocoaagg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_emf.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_fltkagg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_gdk.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_gtk.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_gtkagg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_gtkcairo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_macosx.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_mixed.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_pdf.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_ps.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_qt.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_qt4.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_qt4agg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_qtagg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_svg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_template.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_tkagg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_wx.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/backend_wxagg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/qt4_editor/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/qt4_editor/figureoptions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/qt4_editor/formlayout.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/tkagg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/backends/windowing.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/bezier.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/blocking_input.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/cbook.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/cm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/collections.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/colorbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/colors.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/contour.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/dates.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/delaunay/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/delaunay/interpolate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/delaunay/testfuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/delaunay/triangulate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/docstring.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/dviread.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/figure.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/finance.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/font_manager.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/fontconfig_pattern.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/ft2font.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/gridspec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/hatch.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/image.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/legend.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/lines.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mathtext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mlab.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/cmex10.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/cmmi10.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/cmr10.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/cmsy10.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/cmtt10.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pagd8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pagdo8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pagk8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pagko8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pbkd8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pbkdi8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pbkl8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pbkli8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pcrb8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pcrbo8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pcrr8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pcrro8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/phvb8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/phvb8an.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/phvbo8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/phvbo8an.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/phvl8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/phvlo8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/phvr8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/phvr8an.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/phvro8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/phvro8an.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pncb8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pncbi8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pncr8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pncri8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pplb8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pplbi8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pplr8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pplri8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/psyr.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/ptmb8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/ptmbi8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/ptmr8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/ptmri8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/putb8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/putbi8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/putr8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/putri8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pzcmi8a.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/afm/pzdr.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Courier-Bold.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Courier-BoldOblique.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Courier-Oblique.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Courier.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Helvetica-Bold.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Helvetica-BoldOblique.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Helvetica-Oblique.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Helvetica.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Symbol.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Times-Bold.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Times-BoldItalic.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Times-Italic.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/Times-Roman.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/ZapfDingbats.afm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/pdfcorefonts/readme.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/COPYRIGHT.TXT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/LICENSE_STIX"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/README.TXT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/RELEASENOTES.TXT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXGeneral.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXGeneralBol.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXGeneralBolIta.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXGeneralItalic.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXNonUni.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXNonUniBol.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXNonUniBolIta.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXNonUniIta.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXSizFiveSymReg.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXSizFourSymBol.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXSizFourSymReg.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXSizOneSymBol.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXSizOneSymReg.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXSizThreeSymBol.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXSizThreeSymReg.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXSizTwoSymBol.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/STIXSizTwoSymReg.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/Vera.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/VeraBI.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/VeraBd.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/VeraIt.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/VeraMoBI.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/VeraMoBd.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/VeraMoIt.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/VeraMono.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/VeraSe.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/VeraSeBd.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/cmb10.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/cmex10.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/cmmi10.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/cmr10.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/cmss10.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/cmsy10.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/fonts/ttf/cmtt10.ttf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/back.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/back.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/back.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/back.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/filesave.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/filesave.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/filesave.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/filesave.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/forward.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/forward.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/forward.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/forward.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/hand.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/hand.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/hand.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/hand.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/home.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/home.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/home.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/home.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/matplotlib.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/matplotlib.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/move.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/move.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/move.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/move.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/qt4_editor_options.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/qt4_editor_options.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_close.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_close.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_down.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_down.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_left.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_left.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_refresh.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_refresh.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_right.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_right.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_save_as.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_save_as.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_up.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_up.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_zoom-in.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_zoom-in.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_zoom-out.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/stock_zoom-out.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/subplots.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/subplots.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/subplots.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/zoom_to_rect.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/zoom_to_rect.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/zoom_to_rect.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/images/zoom_to_rect.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/lineprops.glade"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/matplotlib.conf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl-data/matplotlibrc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/mpl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/numerix/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/numerix/_sp_imports.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/numerix/fft/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/numerix/linear_algebra/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/numerix/ma/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/numerix/mlab/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/numerix/random_array/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/nxutils.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/offsetbox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/patches.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/path.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/patheffects.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/projections/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/projections/geo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/projections/polar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/pylab.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/pyparsing.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/pyplot.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/quiver.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/rcsetup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/scale.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/sphinxext/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/sphinxext/ipython_console_highlighting.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/sphinxext/ipython_directive.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/sphinxext/mathmpl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/sphinxext/only_directives.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/sphinxext/plot_directive.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/spines.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/table.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/testing/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/testing/compare.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/testing/decorators.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/testing/jpl_units/Duration.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/testing/jpl_units/Epoch.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/testing/jpl_units/EpochConverter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/testing/jpl_units/StrConverter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/testing/jpl_units/UnitDbl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/testing/jpl_units/UnitDblConverter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/testing/jpl_units/UnitDblFormatter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/testing/jpl_units/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/testing/noseclasses.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/axhspan_epoch.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/axhspan_epoch.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/axhspan_epoch.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/axvspan_epoch.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/axvspan_epoch.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/axvspan_epoch.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/const_xy.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/const_xy.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/const_xy.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/fill_between_interpolate.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/fill_between_interpolate.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/fill_between_interpolate.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/fill_units.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/fill_units.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/fill_units.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_001.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_001.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_001.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_002.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_002.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_002.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_003.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_003.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_003.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_004.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_004.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_004.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_005.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_005.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/formatter_ticker_005.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/hexbin_extent.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/hexbin_extent.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/hexbin_extent.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/imshow.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/imshow.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/imshow.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/imshow_clip.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/imshow_clip.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/imshow_clip.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/nonfinite_limits.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/nonfinite_limits.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/nonfinite_limits.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/offset_points.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/offset_points.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/offset_points.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/pcolormesh.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/pcolormesh.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/pcolormesh.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_axes.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_axes.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_axes.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_coords.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_coords.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_coords.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_rmin.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_rmin.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_rmin.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_units.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_units.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_units.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_wrap_180.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_wrap_180.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_wrap_180.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_wrap_360.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_wrap_360.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polar_wrap_360.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polycollection_joinstyle.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polycollection_joinstyle.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/polycollection_joinstyle.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/shaped_data.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/shaped_data.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/shaped_data.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/single_date.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/single_date.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/single_date.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/single_point.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/single_point.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/single_point.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/symlog.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/symlog.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_axes/symlog.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/DateFormatter_fractionalSeconds.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/DateFormatter_fractionalSeconds.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/DateFormatter_fractionalSeconds.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/RRuleLocator_bounds.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/RRuleLocator_bounds.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/RRuleLocator_bounds.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_axhline.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_axhline.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_axhline.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_axhspan.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_axhspan.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_axhspan.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_axvline.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_axvline.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_axvline.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_axvspan.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_axvspan.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_axvspan.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_empty.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_empty.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_dates/date_empty.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_image/figimage-0.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_image/figimage-1.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_image/image_clip.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_image/image_clip.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_image/image_clip.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_image/image_interps.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_image/image_interps.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_image/image_interps.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_image/imshow.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_image/imshow.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_image/imshow.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_mathtext/mathtext.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_mathtext/mathtext.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_mathtext/mathtext.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_mathtext/mathtext_stix.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_mathtext/mathtext_stix.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_mathtext/mathtext_stix.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_mathtext/mathtext_stixsans.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_mathtext/mathtext_stixsans.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_mathtext/mathtext_stixsans.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/clipper_edge.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/clipper_edge.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/clipper_edge.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/clipping.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/clipping.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/clipping.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/clipping_diamond.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/clipping_diamond.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/clipping_diamond.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/fft_peaks.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/fft_peaks.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/fft_peaks.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/hatch_simplify.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/hatch_simplify.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/hatch_simplify.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/overflow.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/overflow.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/overflow.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/para_equal_perp.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/para_equal_perp.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/para_equal_perp.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/simplify_curve.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/simplify_curve.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_simplification/simplify_curve.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_spines/spines_axes_positions.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_spines/spines_axes_positions.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_spines/spines_axes_positions.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_text/font_styles.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_text/font_styles.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_text/font_styles.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_text/multiline.pdf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_text/multiline.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/baseline_images/test_text/multiline.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_agg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_axes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_backend_svg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_basic.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_cbook.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_dates.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_image.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_mathtext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_mlab.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_simplification.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_spines.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_text.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tests/test_transforms.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/texmanager.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/text.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/textpath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/ticker.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tight_bbox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/transforms.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tri/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tri/triangulation.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tri/tricontour.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tri/tripcolor.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/tri/triplot.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/ttconv.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/type1font.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/units.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/matplotlib/widgets.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/anchored_artists.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/angle_helper.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/axes_divider.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/axes_grid.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/axes_rgb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/axes_size.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/axis_artist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/axisline_style.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/axislines.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/clip_path.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/colorbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/floating_axes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/grid_finder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/grid_helper_curvelinear.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/inset_locator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid/parasite_axes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid1/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid1/anchored_artists.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid1/axes_divider.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid1/axes_grid.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid1/axes_rgb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid1/axes_size.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid1/colorbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid1/inset_locator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid1/mpl_axes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axes_grid1/parasite_axes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axisartist/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axisartist/angle_helper.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axisartist/axis_artist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axisartist/axisline_style.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axisartist/axislines.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axisartist/clip_path.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axisartist/floating_axes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axisartist/grid_finder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/axisartist/grid_helper_curvelinear.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/exceltools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/gtktools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/mplot3d/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/mplot3d/art3d.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/mplot3d/axes3d.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/mplot3d/axis3d.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/mpl_toolkits/mplot3d/proj3d.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy-1.5.1-py2.6.egg-info"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/COMPATIBILITY"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/DEV_README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/INSTALL.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/LICENSE.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/THANKS.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/__config__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/_import_tools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/add_newdocs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/compat/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/compat/_inspect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/compat/py3k.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/compat/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/compat/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/_internal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/_mx_datetime_parser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/_sort.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/arrayprint.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/defchararray.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/fromnumeric.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/function_base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/generate_numpy_api.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/getlimits.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/__multiarray_api.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/__ufunc_api.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/_neighborhood_iterator_imp.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/_numpyconfig.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/arrayobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/arrayscalars.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/multiarray_api.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/ndarrayobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/ndarraytypes.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/noprefix.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/npy_3kcompat.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/npy_common.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/npy_cpu.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/npy_endian.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/npy_interrupt.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/npy_math.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/npy_os.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/numpyconfig.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/old_defines.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/oldnumeric.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/ufunc_api.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/ufuncobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/include/numpy/utils.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/lib/libnpymath.a"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/lib/npy-pkg-config/mlib.ini"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/lib/npy-pkg-config/npymath.ini"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/machar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/memmap.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/multiarray.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/multiarray_tests.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/numeric.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/numerictypes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/records.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/scalarmath.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/scons_support.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/setup_common.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/shape_base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/data/astype_copy.pkl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/data/recarray_from_file.fits"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_arrayprint.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_blasdot.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_defchararray.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_dtype.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_errstate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_function_base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_getlimits.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_machar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_memmap.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_multiarray.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_numeric.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_numerictypes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_print.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_records.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_regression.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_scalarmath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_shape_base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_ufunc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_umath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_umath_complex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/tests/test_unicode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/umath.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/core/umath_tests.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ctypeslib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/__config__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/__version__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/ccompiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/autodist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/bdist_rpm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/build.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/build_clib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/build_ext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/build_py.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/build_scripts.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/build_src.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/config.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/config_compiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/develop.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/egg_info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/install.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/install_clib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/install_data.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/install_headers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/scons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/command/sdist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/compat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/conv_template.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/core.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/cpuinfo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/environment.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/exec_command.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/extension.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/absoft.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/compaq.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/g95.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/gnu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/hpux.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/ibm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/intel.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/lahey.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/mips.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/nag.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/none.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/pg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/sun.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/fcompiler/vast.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/from_template.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/intelccompiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/interactive.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/lib2def.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/line_endings.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/log.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/mingw/gfortran_vs2003_hack.c"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/mingw32ccompiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/misc_util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/npy_pkg_config.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/numpy_distribution.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/system_info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/f2py_ext/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/f2py_ext/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/f2py_ext/src/fib1.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/f2py_ext/src/fib2.pyf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/f2py_ext/tests/test_fib2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/f2py_f90_ext/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/f2py_f90_ext/include/body.f90"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/f2py_f90_ext/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/f2py_f90_ext/src/foo_free.f90"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/f2py_f90_ext/tests/test_foo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/gen_ext/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/gen_ext/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/gen_ext/tests/test_fib3.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/pyrex_ext/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/pyrex_ext/primes.pyx"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/pyrex_ext/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/pyrex_ext/tests/test_primes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/swig_ext/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/swig_ext/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/swig_ext/src/example.c"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/swig_ext/src/example.i"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/swig_ext/src/zoo.cc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/swig_ext/src/zoo.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/swig_ext/src/zoo.i"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/swig_ext/tests/test_example.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/swig_ext/tests/test_example2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/test_fcompiler_gnu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/test_misc_util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/tests/test_npy_pkg_config.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/distutils/unixccompiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/basics.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/broadcasting.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/byteswapping.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/constants.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/creation.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/glossary.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/howtofind.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/indexing.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/internals.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/io.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/jargon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/methods_vs_functions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/misc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/performance.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/structured_arrays.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/subclassing.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/doc/ufuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/dual.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/__version__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/auxfuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/capi_maps.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/cb_rules.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/cfuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/common_rules.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/crackfortran.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/diagnose.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/FAQ.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/HISTORY.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/OLDNEWS.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/TESTING.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/THANKS.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/default.css"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/docutils.conf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/hello.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/pyforttest.pyf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/pytest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/simple.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/simple_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/allocarr.f90"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/allocarr_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/array.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/array_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/calculate.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/calculate_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/callback.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/callback2.pyf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/callback_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/common.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/common_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/compile_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/default.css"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/docutils.conf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/extcallback.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/extcallback_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/fib1.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/fib1.pyf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/fib2.pyf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/fib3.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/ftype.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/ftype_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/index.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/moddata.f90"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/moddata_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/run_main_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/scalar.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/scalar_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/setup_example.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/spam.pyf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/spam_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/string.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/string_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/var.pyf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/docs/usersguide/var_session.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/f2py.1"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/f2py2e.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/f2py_testing.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/f90mod_rules.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/func2subr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/rules.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/src/fortranobject.c"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/src/fortranobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/src/array_from_pyobj/wrapmodule.c"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/src/mixed/foo.f"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/src/mixed/foo_fixed.f90"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/src/mixed/foo_free.f90"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/test_array_from_pyobj.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/test_callback.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/test_mixed.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/test_return_character.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/test_return_complex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/test_return_integer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/test_return_logical.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/test_return_real.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/tests/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/f2py/use_rules.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/fft/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/fft/fftpack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/fft/fftpack_lite.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/fft/helper.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/fft/info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/fft/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/fft/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/fft/tests/test_fftpack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/fft/tests/test_helper.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/_compiled_base.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/_datasource.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/_iotools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/arraysetops.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/arrayterator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/benchmarks/bench_arraysetops.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/financial.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/format.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/function_base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/index_tricks.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/npyio.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/polynomial.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/recfunctions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/scimath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/shape_base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/stride_tricks.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test__datasource.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test__iotools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_arraysetops.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_arrayterator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_financial.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_format.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_function_base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_index_tricks.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_io.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_polynomial.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_recfunctions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_regression.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_shape_base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_stride_tricks.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_twodim_base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_type_check.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_ufunclike.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/tests/test_utils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/twodim_base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/type_check.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/ufunclike.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/user_array.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/lib/utils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/linalg/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/linalg/info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/linalg/lapack_lite.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/linalg/linalg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/linalg/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/linalg/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/linalg/tests/test_build.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/linalg/tests/test_linalg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/linalg/tests/test_regression.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/bench.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/core.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/extras.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/mrecords.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/tests/test_core.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/tests/test_extras.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/tests/test_mrecords.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/tests/test_old_ma.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/tests/test_regression.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/tests/test_subclassing.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/testutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/timer_comparison.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/ma/version.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/matlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/matrixlib/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/matrixlib/defmatrix.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/matrixlib/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/matrixlib/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/matrixlib/tests/test_defmatrix.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/matrixlib/tests/test_multiarray.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/matrixlib/tests/test_numeric.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/matrixlib/tests/test_regression.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/_capi.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/alter_code1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/alter_code2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/compat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/convolve.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/fft.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/functions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/image.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/include/numpy/arraybase.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/include/numpy/cfunc.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/include/numpy/ieeespecial.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/include/numpy/libnumarray.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/include/numpy/numcomplex.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/include/numpy/nummacro.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/linear_algebra.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/ma.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/matrix.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/mlab.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/nd_image.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/numerictypes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/random_array.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/session.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/ufuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/numarray/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/alter_code1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/alter_code2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/array_printer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/arrayfns.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/compat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/fft.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/fix_default_axis.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/functions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/linear_algebra.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/ma.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/matrix.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/misc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/mlab.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/precision.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/random_array.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/rng.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/rng_stats.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/tests/test_oldnumeric.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/tests/test_regression.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/typeconv.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/ufuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/oldnumeric/user_array.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/polynomial/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/polynomial/chebyshev.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/polynomial/polynomial.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/polynomial/polytemplate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/polynomial/polyutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/polynomial/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/polynomial/tests/test_chebyshev.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/polynomial/tests/test_polynomial.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/polynomial/tests/test_polyutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/random/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/random/info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/random/mtrand.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/random/randomkit.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/random/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/random/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/random/tests/test_random.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/site.cfg.example"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/testing/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/testing/decorators.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/testing/noseclasses.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/testing/nosetester.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/testing/nulltester.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/testing/numpytest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/testing/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/testing/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/testing/tests/test_decorators.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/testing/tests/test_utils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/testing/utils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/tests/test_ctypeslib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/tests/test_matlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/tools/py3tool.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/numpy/version.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pylab.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythoncom.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/Microsoft.VC90.MFC.manifest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/Pythonwin.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/dde.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/license.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/mfc90.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/mfc90u.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/mfcm90.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/mfcm90u.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/app/basictimerapp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/app/customprint.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/app/demoutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/app/dlgappdemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/app/dojobapp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/app/helloapp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/cmdserver.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/createwin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/demoutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/dibdemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/dlgtest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/dyndlg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/fontdemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/guidemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/hiertest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/menutest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/objdoc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/ocx/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/ocx/demoutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/ocx/flash.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/ocx/msoffice.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/ocx/ocxserialtest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/ocx/ocxtest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/ocx/webbrowser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/openGLDemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/progressbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/sliderdemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/splittst.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/threadedgui.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/Demos/toolbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/IDLE.cfg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/debugger/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/debugger/configui.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/debugger/dbgcon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/debugger/dbgpyapp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/debugger/debugger.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/debugger/fail.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/default.cfg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/dialogs/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/dialogs/ideoptions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/dialogs/list.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/dialogs/login.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/dialogs/status.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/docking/DockingBar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/docking/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/app.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/bitmap.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/cmdline.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/dbgcommands.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/dlgappcore.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/editor/ModuleBrowser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/editor/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/editor/color/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/editor/color/coloreditor.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/editor/configui.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/editor/document.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/editor/editor.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/editor/frame.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/editor/template.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/editor/vss.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/help.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/interact.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/intpyapp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/intpydde.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/mdi_pychecker.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/scriptutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/sgrepmdi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/startup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/stdin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/toolmenu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/window.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/framework/winout.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/idle/AutoExpand.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/idle/AutoIndent.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/idle/CallTips.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/idle/FormatParagraph.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/idle/IdleHistory.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/idle/PyParse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/idle/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/mfc/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/mfc/activex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/mfc/afxres.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/mfc/dialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/mfc/docview.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/mfc/object.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/mfc/thread.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/mfc/window.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/scintilla/IDLEenvironment.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/scintilla/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/scintilla/bindings.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/scintilla/config.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/scintilla/configui.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/scintilla/control.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/scintilla/document.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/scintilla/find.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/scintilla/formatter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/scintilla/keycodes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/scintilla/scintillacon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/scintilla/view.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/tools/TraceCollector.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/tools/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/tools/browseProjects.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/tools/browser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/tools/hierlist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/tools/regedit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/pywin/tools/regpy.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/scintilla.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/win32ui.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pythonwin/win32uiole.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/reference.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/tzfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/tzinfo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Abidjan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Accra"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Addis_Ababa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Algiers"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Asmara"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Asmera"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Bamako"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Bangui"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Banjul"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Bissau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Blantyre"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Brazzaville"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Bujumbura"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Cairo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Casablanca"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Ceuta"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Conakry"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Dakar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Dar_es_Salaam"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Djibouti"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Douala"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/El_Aaiun"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Freetown"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Gaborone"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Harare"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Johannesburg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Kampala"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Khartoum"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Kigali"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Kinshasa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Lagos"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Libreville"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Lome"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Luanda"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Lubumbashi"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Lusaka"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Malabo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Maputo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Maseru"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Mbabane"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Mogadishu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Monrovia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Nairobi"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Ndjamena"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Niamey"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Nouakchott"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Ouagadougou"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Porto-Novo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Sao_Tome"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Timbuktu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Tripoli"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Tunis"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Africa/Windhoek"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Adak"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Anchorage"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Anguilla"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Antigua"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Araguaina"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/Buenos_Aires"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/Catamarca"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/ComodRivadavia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/Cordoba"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/Jujuy"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/La_Rioja"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/Mendoza"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/Rio_Gallegos"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/Salta"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/San_Juan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/San_Luis"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/Tucuman"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Argentina/Ushuaia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Aruba"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Asuncion"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Atikokan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Atka"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Bahia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Barbados"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Belem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Belize"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Blanc-Sablon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Boa_Vista"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Bogota"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Boise"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Buenos_Aires"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Cambridge_Bay"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Campo_Grande"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Cancun"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Caracas"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Catamarca"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Cayenne"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Cayman"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Chicago"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Chihuahua"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Coral_Harbour"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Cordoba"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Costa_Rica"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Cuiaba"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Curacao"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Danmarkshavn"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Dawson"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Dawson_Creek"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Denver"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Detroit"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Dominica"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Edmonton"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Eirunepe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/El_Salvador"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Ensenada"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Fort_Wayne"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Fortaleza"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Glace_Bay"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Godthab"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Goose_Bay"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Grand_Turk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Grenada"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Guadeloupe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Guatemala"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Guayaquil"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Guyana"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Halifax"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Havana"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Hermosillo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Indiana/Indianapolis"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Indiana/Knox"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Indiana/Marengo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Indiana/Petersburg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Indiana/Tell_City"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Indiana/Vevay"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Indiana/Vincennes"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Indiana/Winamac"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Indianapolis"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Inuvik"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Iqaluit"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Jamaica"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Jujuy"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Juneau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Kentucky/Louisville"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Kentucky/Monticello"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Knox_IN"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/La_Paz"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Lima"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Los_Angeles"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Louisville"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Maceio"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Managua"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Manaus"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Marigot"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Martinique"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Matamoros"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Mazatlan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Mendoza"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Menominee"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Merida"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Mexico_City"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Miquelon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Moncton"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Monterrey"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Montevideo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Montreal"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Montserrat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Nassau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/New_York"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Nipigon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Nome"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Noronha"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/North_Dakota/Center"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/North_Dakota/New_Salem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Ojinaga"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Panama"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Pangnirtung"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Paramaribo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Phoenix"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Port-au-Prince"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Port_of_Spain"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Porto_Acre"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Porto_Velho"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Puerto_Rico"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Rainy_River"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Rankin_Inlet"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Recife"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Regina"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Resolute"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Rio_Branco"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Rosario"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Santa_Isabel"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Santarem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Santiago"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Santo_Domingo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Sao_Paulo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Scoresbysund"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Shiprock"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/St_Barthelemy"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/St_Johns"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/St_Kitts"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/St_Lucia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/St_Thomas"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/St_Vincent"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Swift_Current"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Tegucigalpa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Thule"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Thunder_Bay"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Tijuana"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Toronto"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Tortola"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Vancouver"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Virgin"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Whitehorse"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Winnipeg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Yakutat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/America/Yellowknife"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Antarctica/Casey"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Antarctica/Davis"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Antarctica/DumontDUrville"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Antarctica/Macquarie"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Antarctica/Mawson"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Antarctica/McMurdo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Antarctica/Palmer"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Antarctica/Rothera"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Antarctica/South_Pole"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Antarctica/Syowa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Antarctica/Vostok"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Arctic/Longyearbyen"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Aden"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Almaty"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Amman"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Anadyr"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Aqtau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Aqtobe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Ashgabat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Ashkhabad"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Baghdad"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Bahrain"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Baku"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Bangkok"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Beirut"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Bishkek"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Brunei"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Calcutta"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Choibalsan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Chongqing"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Chungking"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Colombo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Dacca"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Damascus"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Dhaka"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Dili"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Dubai"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Dushanbe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Gaza"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Harbin"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Ho_Chi_Minh"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Hong_Kong"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Hovd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Irkutsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Istanbul"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Jakarta"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Jayapura"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Jerusalem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Kabul"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Kamchatka"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Karachi"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Kashgar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Kathmandu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Katmandu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Kolkata"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Krasnoyarsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Kuala_Lumpur"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Kuching"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Kuwait"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Macao"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Macau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Magadan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Makassar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Manila"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Muscat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Nicosia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Novokuznetsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Novosibirsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Omsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Oral"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Phnom_Penh"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Pontianak"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Pyongyang"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Qatar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Qyzylorda"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Rangoon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Riyadh"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Riyadh87"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Riyadh88"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Riyadh89"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Saigon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Sakhalin"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Samarkand"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Seoul"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Shanghai"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Singapore"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Taipei"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Tashkent"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Tbilisi"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Tehran"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Tel_Aviv"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Thimbu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Thimphu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Tokyo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Ujung_Pandang"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Ulaanbaatar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Ulan_Bator"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Urumqi"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Vientiane"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Vladivostok"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Yakutsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Yekaterinburg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Asia/Yerevan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Atlantic/Azores"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Atlantic/Bermuda"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Atlantic/Canary"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Atlantic/Cape_Verde"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Atlantic/Faeroe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Atlantic/Faroe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Atlantic/Jan_Mayen"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Atlantic/Madeira"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Atlantic/Reykjavik"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Atlantic/South_Georgia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Atlantic/St_Helena"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Atlantic/Stanley"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/ACT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Adelaide"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Brisbane"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Broken_Hill"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Canberra"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Currie"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Darwin"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Eucla"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Hobart"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/LHI"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Lindeman"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Lord_Howe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Melbourne"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/NSW"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/North"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Perth"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Queensland"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/South"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Sydney"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Tasmania"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Victoria"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/West"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Australia/Yancowinna"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Brazil/Acre"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Brazil/DeNoronha"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Brazil/East"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Brazil/West"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/CET"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/CST6CDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Canada/Atlantic"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Canada/Central"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Canada/East-Saskatchewan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Canada/Eastern"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Canada/Mountain"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Canada/Newfoundland"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Canada/Pacific"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Canada/Saskatchewan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Canada/Yukon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Chile/Continental"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Chile/EasterIsland"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Cuba"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/EET"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/EST"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/EST5EDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Egypt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Eire"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+0"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+1"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+10"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+11"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+12"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+2"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+3"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+4"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+5"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+6"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+7"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+8"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT+9"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-0"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-1"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-10"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-11"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-12"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-13"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-14"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-2"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-3"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-4"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-5"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-6"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-7"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-8"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT-9"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/GMT0"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/Greenwich"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/UCT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/UTC"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/Universal"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Etc/Zulu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Amsterdam"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Andorra"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Athens"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Belfast"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Belgrade"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Berlin"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Bratislava"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Brussels"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Bucharest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Budapest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Chisinau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Copenhagen"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Dublin"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Gibraltar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Guernsey"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Helsinki"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Isle_of_Man"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Istanbul"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Jersey"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Kaliningrad"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Kiev"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Lisbon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Ljubljana"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/London"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Luxembourg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Madrid"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Malta"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Mariehamn"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Minsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Monaco"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Moscow"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Nicosia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Oslo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Paris"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Podgorica"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Prague"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Riga"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Rome"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Samara"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/San_Marino"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Sarajevo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Simferopol"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Skopje"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Sofia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Stockholm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Tallinn"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Tirane"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Tiraspol"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Uzhgorod"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Vaduz"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Vatican"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Vienna"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Vilnius"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Volgograd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Warsaw"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Zagreb"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Zaporozhye"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Europe/Zurich"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Factory"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/GB"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/GB-Eire"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/GMT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/GMT+0"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/GMT-0"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/GMT0"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Greenwich"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/HST"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Hongkong"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Iceland"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Indian/Antananarivo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Indian/Chagos"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Indian/Christmas"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Indian/Cocos"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Indian/Comoro"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Indian/Kerguelen"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Indian/Mahe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Indian/Maldives"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Indian/Mauritius"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Indian/Mayotte"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Indian/Reunion"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Iran"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Israel"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Jamaica"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Japan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Kwajalein"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Libya"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/MET"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/MST"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/MST7MDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Mexico/BajaNorte"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Mexico/BajaSur"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Mexico/General"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Mideast/Riyadh87"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Mideast/Riyadh88"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Mideast/Riyadh89"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/NZ"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/NZ-CHAT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Navajo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/PRC"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/PST8PDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Apia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Auckland"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Chatham"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Easter"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Efate"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Enderbury"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Fakaofo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Fiji"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Funafuti"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Galapagos"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Gambier"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Guadalcanal"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Guam"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Honolulu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Johnston"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Kiritimati"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Kosrae"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Kwajalein"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Majuro"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Marquesas"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Midway"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Nauru"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Niue"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Norfolk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Noumea"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Pago_Pago"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Palau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Pitcairn"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Ponape"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Port_Moresby"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Rarotonga"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Saipan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Samoa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Tahiti"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Tarawa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Tongatapu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Truk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Wake"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Wallis"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Pacific/Yap"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Poland"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Portugal"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/ROC"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/ROK"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Singapore"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Turkey"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/UCT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/Alaska"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/Aleutian"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/Arizona"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/Central"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/East-Indiana"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/Eastern"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/Hawaii"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/Indiana-Starke"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/Michigan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/Mountain"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/Pacific"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/Pacific-New"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/US/Samoa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/UTC"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Universal"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/W-SU"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/WET"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/Zulu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/iso3166.tab"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/localtime"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/posixrules"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pytz/zoneinfo/zone.tab"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pywin32-215-py2.6.egg-info"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pywin32.pth"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pywin32.version.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pywin32_system32/pythoncom26.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/pywin32_system32/pywintypes26.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/BENTO_BUILD.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/HACKING.rst.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/INSTALL.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/LATEST.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/LICENSE.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/THANKS.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/TOCHANGE.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/__config__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/_build_utils/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/_build_utils/_fortran.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/constants/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/constants/codata.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/constants/constants.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/constants/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/constants/tests/test_codata.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/constants/tests/test_constants.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/_util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/blas/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/blas/cblas.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/blas/fblas.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/blas/scons_support.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/blas/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/blas/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/blas/tests/test_blas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/blas/tests/test_fblas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/decorator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/lapack/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/lapack/calc_lwork.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/lapack/clapack.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/lapack/flapack.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/lapack/scons_support.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/lapack/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/lapack/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/lapack/tests/common.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/lapack/tests/test_esv.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/lapack/tests/test_gesv.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/lapack/tests/test_lapack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/lib/six.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/_cblas.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/_clapack.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/_decomp_qz.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/_fblas.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/_flapack.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/_flinalg.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/_solvers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/_testutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/basic.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/benchmarks/bench_basic.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/benchmarks/bench_decom.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/blas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/calc_lwork.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/cblas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/clapack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/decomp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/decomp_cholesky.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/decomp_lu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/decomp_qr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/decomp_schur.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/decomp_svd.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/fblas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/flapack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/flinalg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/lapack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/linalg_version.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/matfuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/misc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/special_matrices.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/tests/test_basic.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/tests/test_blas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/tests/test_build.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/tests/test_decomp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/tests/test_decomp_cholesky.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/tests/test_fblas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/tests/test_lapack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/tests/test_matfuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/tests/test_solvers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/linalg/tests/test_special_matrices.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/ascent.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/common.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/doccer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/face.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/lena.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/pilutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/tests/data/icon.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/tests/data/icon_mono.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/tests/data/icon_mono_flat.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/tests/test_common.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/tests/test_doccer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/misc/tests/test_pilutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/_basinhopping.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/_cobyla.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/_lbfgsb.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/_minimize.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/_minpack.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/_nnls.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/_root.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/_slsqp.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/_tstutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/_zeros.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/anneal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/benchmarks/bench_zeros.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/cobyla.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/lbfgsb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/linesearch.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/minpack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/minpack2.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/moduleTNC.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/nnls.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/nonlin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/optimize.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/slsqp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tests/test__basinhopping.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tests/test__root.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tests/test_anneal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tests/test_cobyla.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tests/test_linesearch.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tests/test_minpack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tests/test_nnls.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tests/test_nonlin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tests/test_optimize.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tests/test_regression.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tests/test_slsqp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tests/test_zeros.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/tnc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/optimize/zeros.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/benchmarks/bench_sparse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/bsr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/compressed.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/construct.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/coo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/_components.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/_laplacian.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/_min_spanning_tree.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/_shortest_path.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/_tools.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/_traversal.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/_validation.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/tests/test_connected_components.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/tests/test_conversions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/tests/test_graph_components.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/tests/test_graph_laplacian.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/tests/test_shortest_path.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/tests/test_spanning_tree.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csgraph/tests/test_traversal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/csr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/data.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/dia.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/dok.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/extract.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/lil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/dsolve/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/dsolve/_superlu.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/dsolve/linsolve.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/dsolve/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/dsolve/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/dsolve/tests/test_linsolve.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/dsolve/umfpack/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/dsolve/umfpack/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/dsolve/umfpack/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/dsolve/umfpack/tests/test_umfpack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/dsolve/umfpack/tests/try_umfpack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/dsolve/umfpack/umfpack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/arpack/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/arpack/_arpack.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/arpack/arpack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/arpack/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/arpack/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/arpack/tests/test_arpack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/lobpcg/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/lobpcg/lobpcg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/lobpcg/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/lobpcg/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/lobpcg/tests/benchmark.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/lobpcg/tests/large_scale.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/lobpcg/tests/test_lobpcg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/eigen/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/interface.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/_iterative.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/iterative.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/lgmres.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/lsmr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/lsqr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/minres.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/tests/demo_lgmres.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/tests/test_iterative.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/tests/test_lgmres.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/tests/test_lsmr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/tests/test_lsqr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/tests/test_utils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/isolve/utils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/matfuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/tests/test_interface.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/linalg/tests/test_matfuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/_bsr.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/_coo.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/_csc.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/_csgraph.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/_csr.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/_dia.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/bsr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/coo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/csc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/csgraph.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/csr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/dia.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sparsetools/setupscons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/spfuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/sputils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/tests/test_base.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/tests/test_construct.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/tests/test_csc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/tests/test_csr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/tests/test_extract.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/tests/test_spfuncs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/sparse/tests/test_sputils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/scipy/version.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/boundattributes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/cache.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/classregistry.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/col.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/conftest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/constraints.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/converters.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/dbconnection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/dberrors.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/declarative.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/events.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/firebird/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/firebird/firebirdconnection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/include/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/include/hashcol.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/include/pydispatch/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/include/pydispatch/dispatcher.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/include/pydispatch/errors.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/include/pydispatch/robust.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/include/pydispatch/robustapply.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/include/pydispatch/saferef.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/index.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/inheritance/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/inheritance/iteration.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/joins.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/main.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/manager/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/manager/command.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/maxdb/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/maxdb/maxdbconnection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/mssql/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/mssql/mssqlconnection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/mysql/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/mysql/mysqlconnection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/postgres/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/postgres/pgconnection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/sqlbuilder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/sqlite/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/sqlite/sqliteconnection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/sresults.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/styles.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/sybase/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/sybase/sybaseconnection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/util/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/util/csvexport.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/util/csvimport.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/util/moduleloader.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/util/threadinglocal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/versioning/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/views.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/sqlobject/wsgi_middleware.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/BackupRead_BackupWrite.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/BackupSeek_streamheaders.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/CopyFileEx.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/CreateFileTransacted_MiniVersion.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/FileSecurityTest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/GetSaveFileName.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/NetValidatePasswordPolicy.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/OpenEncryptedFileRaw.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/RegCreateKeyTransacted.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/RegRestoreKey.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/SystemParametersInfo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/c_extension/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/cerapi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/dde/ddeclient.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/dde/ddeserver.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/desktopmanager.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/eventLogDemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/getfilever.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/images/frowny.bmp"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/images/smiley.bmp"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/mmapfile_demo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/pipes/cat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/pipes/runproc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/print_desktop.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/rastest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/GetTokenInformation.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/account_rights.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/explicit_entries.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/get_policy_info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/list_rights.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/localized_names.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/lsaregevent.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/lsastore.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/query_information.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/regsave_sa.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/regsecurity.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/sa_inherit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/security_enums.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/set_file_audit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/set_file_owner.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/set_policy_info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/setkernelobjectsecurity.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/setnamedsecurityinfo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/setsecurityinfo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/setuserobjectsecurity.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/sspi/fetch_url.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/sspi/simple_auth.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/sspi/socket_server.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/security/sspi/validate_password.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/service/nativePipeTestService.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/service/pipeTestService.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/service/pipeTestServiceClient.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/service/serviceEvents.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/timer_demo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32clipboardDemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32clipboard_bitmapdemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32comport_demo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32console_demo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32cred_demo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32fileDemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32gui_demo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32gui_devicenotify.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32gui_dialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32gui_menu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32gui_taskbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32netdemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32rcparser_demo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32servicedemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32ts_logoff_disconnected.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32wnet/testwnet.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/win32wnet/winnetwk.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/Demos/winprocess.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/_win32sysloader.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/_winxptheme.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/include/PyWinTypes.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/afxres.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/commctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/dbi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/mmsystem.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/netbios.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/ntsecuritycon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/pywin32_testutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/pywintypes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/rasutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/regcheck.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/regutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/sspi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/sspicon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32con.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32cryptcon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32evtlogutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32gui_struct.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32inetcon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32netcon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32pdhquery.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32pdhutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32rcparser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32serviceutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32timezone.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32traceutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/win32verstamp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/winerror.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/winioctlcon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/winnt.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/winperf.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/lib/winxptheme.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/libs/pywintypes.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/license.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/mmapfile.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/odbc.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/perfmon.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/perfmondata.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/pythoncom26.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/pythonservice.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/pywintypes26.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/scripts/ControlService.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/scripts/VersionStamp/BrandProject.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/scripts/VersionStamp/bulkstamp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/scripts/VersionStamp/vssutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/scripts/backupEventLog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/scripts/ce/pysynch.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/scripts/killProcName.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/scripts/rasutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/scripts/regsetup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/scripts/setup_d.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/servicemanager.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/handles.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_clipboard.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_exceptions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_odbc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_pywintypes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_security.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_sspi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32api.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32crypt.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32event.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32file.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32gui.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32guistruct.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32inet.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32net.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32pipe.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32rcparser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32timezone.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32trace.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/test_win32wnet.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/testall.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/win32rcparser/python.bmp"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/win32rcparser/python.ico"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/win32rcparser/test.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/test/win32rcparser/test.rc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/timer.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win2kras.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32api.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32clipboard.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32console.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32cred.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32crypt.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32event.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32evtlog.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32file.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32gui.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32help.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32inet.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32job.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32lz.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32net.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32pdh.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32pipe.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32print.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32process.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32profile.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32ras.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32security.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32service.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32trace.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32transaction.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32ts.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/win32wnet.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32/winxpgui.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/GeneratedSupport.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/PythonCOM.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/QuickStartClientCom.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/QuickStartServerCom.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/docindex.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/image/BTN_HomePage.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/image/BTN_ManualTop.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/image/BTN_NextPage.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/image/BTN_PrevPage.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/image/blank.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/image/pycom_blowing.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/image/pythoncom.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/image/www_icon.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/index.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/misc.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/HTML/package.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/License.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/client/CLSIDToClass.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/client/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/client/build.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/client/combrowse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/client/connect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/client/dynamic.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/client/gencache.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/client/genpy.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/client/makepy.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/client/selecttlb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/client/tlbrowse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/client/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/decimal_23.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/demos/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/demos/connect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/demos/dump_clipboard.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/demos/eventsApartmentThreaded.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/demos/eventsFreeThreaded.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/demos/excelAddin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/demos/excelRTDServer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/demos/iebutton.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/demos/ietoolbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/demos/outlookAddin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/demos/trybag.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/include/PythonCOM.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/include/PythonCOMRegister.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/include/PythonCOMServer.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/libs/axscript.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/libs/pythoncom.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/makegw/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/makegw/makegw.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/makegw/makegwenum.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/makegw/makegwparse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/olectl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/readme.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/server/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/server/connect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/server/dispatcher.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/server/exception.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/server/factory.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/server/localserver.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/server/policy.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/server/register.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/server/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/servers/PythonTools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/servers/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/servers/dictionary.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/servers/interp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/servers/perfmon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/servers/test_pycomtest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/storagecon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/GenTestScripts.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/Testpys.sct"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/daodump.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/errorSemantics.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/pippo.idl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/pippo_server.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/policySemantics.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/readme.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testADOEvents.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testAXScript.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testAccess.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testArrays.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testClipboard.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testCollections.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testDCOM.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testDictionary.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testDictionary.vbs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testDynamic.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testExchange.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testExplorer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testGIT.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testGatewayAddresses.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testInterp.vbs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testIterators.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testMSOffice.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testMSOfficeEvents.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testMarshal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testNetscape.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testPersist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testPippo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testPyComTest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testPyScriptlet.js"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testROT.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testServers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testShell.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testStorage.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testStreams.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testWMI.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testall.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testmakepy.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testvb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testvbscript_regexp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testxslt.js"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testxslt.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/testxslt.xsl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/test/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/universal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32com/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/adsi/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/adsi/adsi.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/adsi/adsicon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/adsi/demos/objectPicker.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/adsi/demos/scp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/adsi/demos/search.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/adsi/demos/test.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/authorization/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/authorization/authorization.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/authorization/demos/EditSecurity.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/authorization/demos/EditServiceSecurity.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axcontrol/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axcontrol/axcontrol.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axdebug/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axdebug/adb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axdebug/axdebug.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axdebug/codecontainer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axdebug/contexts.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axdebug/debugger.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axdebug/documents.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axdebug/dump.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axdebug/expressions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axdebug/gateways.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axdebug/stackframe.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axdebug/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/asp/CreateObject.asp"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/asp/caps.asp"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/asp/interrupt/test.asp"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/asp/interrupt/test.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/asp/interrupt/test1.asp"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/asp/interrupt/test1.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/asp/tut1.asp"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/MarqueeText1.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/calc.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/dbgtest.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/demo.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/demo_check.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/demo_intro.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/demo_menu.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/docwrite.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/foo2.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/form.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/marqueeDemo.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/mousetrack.htm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/ie/pycom_blowing.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/wsh/blank.pys"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/wsh/excel.pys"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/wsh/registry.pys"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/Demos/client/wsh/test.pys"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/asputil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/axscript.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/client/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/client/debug.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/client/error.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/client/framework.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/client/pydumper.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/client/pyscript.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/client/pyscript_rexec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/client/scriptdispatch.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/server/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/server/axsite.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/server/error.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/test/debugTest.pys"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/test/debugTest.vbs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/test/leakTest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/test/test.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/test/testHost.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/axscript/test/testHost4Dbg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/bits/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/bits/bits.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/bits/test/show_all_jobs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/bits/test/test_bits.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/directsound/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/directsound/directsound.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/directsound/test/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/directsound/test/ds_record.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/directsound/test/ds_test.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/ifilter/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/ifilter/demo/filterDemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/ifilter/ifilter.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/ifilter/ifiltercon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/internet/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/internet/inetcon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/internet/internet.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/mapi/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/mapi/demos/mapisend.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/mapi/emsabtags.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/mapi/exchange.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/mapi/exchdapi.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/mapi/mapi.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/mapi/mapitags.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/mapi/mapiutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/propsys/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/propsys/propsys.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/propsys/pscon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/propsys/test/testpropsys.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/IActiveDesktop.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/IShellLinkDataList.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/IUniformResourceLocator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/browse_for_folder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/create_link.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/dump_link.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/explorer_browser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/servers/column_provider.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/servers/context_menu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/servers/copy_hook.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/servers/empty_volume_cache.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/servers/folder_view.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/servers/icon_handler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/servers/shell_view.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/shellexecuteex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/viewstate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/demos/walk_shell_folders.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/knownfolders.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/shell.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/shellcon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/test/testSHFileOperation.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/test/testShellFolder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/shell/test/testShellItem.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/taskscheduler/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/taskscheduler/taskscheduler.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/taskscheduler/test/test_addtask.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/taskscheduler/test/test_addtask_1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/taskscheduler/test/test_addtask_2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/win32comext/taskscheduler/test/test_localsystem.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/docs/CHANGES.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/docs/CHANGES.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/docs/MigrationGuide.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/docs/MigrationGuide.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/docs/README.win32.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/docs/default.css"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/docs/licence/lgpl.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/docs/licence/licence.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/docs/licence/licendoc.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/docs/licence/preamble.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/unins000.dat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/unins000.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/__version__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_activex.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_animate.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_aui.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_calendar.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_combo.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_controls.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_controls_.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_core.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_core_.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_gdi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_gdi_.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_gizmos.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_glcanvas.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_grid.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_html.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_media.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_misc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_misc_.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_richtext.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_stc.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_webkit.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_windows.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_windows_.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_wizard.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/_xrc.pyd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/activex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/animate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/aui.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/build/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/build/build_options.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/build/config.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/calendar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/combo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/gdiplus.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/gizmos.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/glcanvas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/grid.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/html.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/CDate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ClickableHtmlWindow.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/activex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/activexwrapper.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/DownButton.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/UpButton.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/advancedsplash.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/aquabutton.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/artmanager.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/aui/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/aui/aui_constants.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/aui/aui_switcherdialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/aui/aui_utilities.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/aui/auibar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/aui/auibook.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/aui/dockart.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/aui/framemanager.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/aui/tabart.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/aui/tabmdi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/balloontip.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/buttonpanel.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/cubecolourdialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/customtreectrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/flatmenu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/flatnotebook.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/floatspin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/fmcustomizedlg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/fmresources.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/foldpanelbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/fourwaysplitter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/genericmessagedialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/gradientbutton.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/hyperlink.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/hypertreelist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/knobctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/labelbook.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/multidirdialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/peakmeter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/piectrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/pybusyinfo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/pycollapsiblepane.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/pyprogress.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/art.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/art_aui.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/art_default.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/art_internal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/art_msw.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/bar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/buttonbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/control.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/gallery.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/page.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/panel.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ribbon/toolbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/rulerctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/shapedbutton.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/speedmeter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/supertooltip.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/thumbnailctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/toasterbox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/ultimatelistctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/agw/zoombar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/analogclock/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/analogclock/analogclock.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/analogclock/helpers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/analogclock/lib_setup/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/analogclock/lib_setup/buttontreectrlpanel.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/analogclock/lib_setup/colourselect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/analogclock/lib_setup/fontselect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/analogclock/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/analogclock/styles.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/anchors.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/art/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/art/flagart.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/art/img2pyartprov.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/buttonpanel.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/buttons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/calendar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/colourchooser/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/colourchooser/canvas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/colourchooser/intl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/colourchooser/pycolourbox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/colourchooser/pycolourchooser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/colourchooser/pycolourslider.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/colourchooser/pypalette.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/colourdb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/colourselect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/colourutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/combotreebox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/customtreectrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/delayedresult.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/dialogs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/docview.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/dragscroller.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/editor/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/editor/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/editor/editor.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/editor/images.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/editor/selection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/embeddedimage.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/eventStack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/eventwatcher.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/evtmgr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/expando.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/fancytext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/filebrowsebutton.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/flashwin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/flashwin_old.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/flatnotebook.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/floatbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/floatcanvas/FloatCanvas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/floatcanvas/GUIMode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/floatcanvas/NavCanvas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/floatcanvas/Resources.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/floatcanvas/ScreenShot.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/floatcanvas/Utilities/BBox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/floatcanvas/Utilities/BBoxTest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/floatcanvas/Utilities/Colors.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/floatcanvas/Utilities/GUI.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/floatcanvas/Utilities/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/floatcanvas/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/foldmenu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/foldpanelbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/gestures.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/graphics.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/gridmovers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/grids.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/hyperlink.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/iewin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/iewin_old.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/imagebrowser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/imageutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/infoframe.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/inspection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/intctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/langlistctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/layoutf.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/masked/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/masked/combobox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/masked/ctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/masked/ipaddrctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/masked/maskededit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/masked/numctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/masked/textctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/masked/timectrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/mixins/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/mixins/grid.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/mixins/gridlabelrenderer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/mixins/imagelist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/mixins/inspection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/mixins/listctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/mixins/rubberband.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/mixins/treemixin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/msgpanel.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/multisash.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/mvctree.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/myole4ax.idl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/myole4ax.tlb"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/newevent.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ogl/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ogl/_basic.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ogl/_bmpshape.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ogl/_canvas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ogl/_composit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ogl/_diagram.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ogl/_divided.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ogl/_drawn.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ogl/_lines.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ogl/_oglmisc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pdfwin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pdfwin_old.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/platebtn.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/plot.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/popupctl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/printout.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/autosetuppubsubv1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/arg1/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/arg1/listenerimpl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/arg1/publisher.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/arg1/publishermixin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/arg1/topicargspecimpl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/arg1/topicmgrimpl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/callables.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/datamsg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/kwargs/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/kwargs/listenerimpl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/kwargs/publisher.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/kwargs/publishermixin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/kwargs/topicargspecimpl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/kwargs/topicmgrimpl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/listener.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/listenerbase.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/notificationmgr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/notificationmgr_old.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/policies.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/publisherbase.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/topicargspec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/topicdefnprovider.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/topicexc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/topicmgr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/topicobj.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/topictreetraverser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/topicutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/treeconfig.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/validatedefnargs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/core/weakmethod.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/pub.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/pubsub1/pub.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/pubsub1/utils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/pubsub2/pub.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/pubsub2/utils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/pubsubconf.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/setuparg1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/setupkwargs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/setupv1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/setupv2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/utils/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/utils/exchandling.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/utils/globalsettings.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/utils/misc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/utils/notification.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/utils/topictreeprinter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pubsub/utils/topictreevisitor.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pydocview.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/pyshell.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/rcsizer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/resizewidget.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/rightalign.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/rpcMixin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/scrolledpanel.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/sheet.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/shell.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/sized_controls.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/splashscreen.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/splitter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/statbmp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/stattext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/throbber.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ticker.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/ticker_xrc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/utils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/wordwrap.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/wxPlotCanvas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/wxcairo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/lib/wxpTag.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/af/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/ca/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/cs/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/da/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/de/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/el/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/es/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/eu/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/fi/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/fr/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/gl_ES/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/hi/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/hu/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/id/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/it/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/ja/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/lv/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/ms/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/nb/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/nl/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/pl/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/pt/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/pt_BR/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/ru/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/sk/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/sl/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/sq/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/sv/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/tr/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/uk/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/vi/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/zh_CN/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/locale/zh_TW/LC_MESSAGES/wxstd.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/media.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/CHANGES.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/Py.ico"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PyAlaCarte.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PyAlaMode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PyAlaModeTest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PyCrust.ico"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PyCrust.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PyCrust_16.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PyCrust_32.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PyFilling.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PyShell.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PySlices.ico"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PySlices.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PySlicesShell.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PySlices_16.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PySlices_32.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/PyWrap.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/buffer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/crust.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/crustslices.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/dispatcher.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/document.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/editor.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/editwindow.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/filling.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/frame.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/images.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/interpreter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/introspect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/magic.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/parse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/path.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/pseudo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/shell.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/sliceshell.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/tests/test_interpreter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/tests/test_introspect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/tests/test_pseudo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/tests/test_version.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/tests/testall.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/py/version.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/richtext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/stc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/AUTHORS"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/CHANGELOG"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/COPYING"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/Editra"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/FAQ"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/NEWS"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/README"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/THANKS"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/TODO"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/docs/editra_style_sheets.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/launcher.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/ca_ES@valencia/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/cs_CZ/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/da_DK/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/de_DE/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/en_US/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/es_ES/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/fr_FR/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/gl_ES/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/hr_HR/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/hu_HU/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/it_IT/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/ja_JP/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/lv_LV/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/nl_NL/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/nn_NO/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/pl_PL/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/pt_BR/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/ru_RU/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/sk_SK/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/sl_SI/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/sr_RS/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/sv_SE/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/tr_TR/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/uk_UA/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/zh_CN/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/locale/zh_TW/LC_MESSAGES/Editra.mo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/Editra.icns"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/editra.ico"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/editra.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/editra_doc.icns"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/editra_doc.ico"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/editra_doc.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/splashwarn.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Default/README"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/AUTHORS"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/COPYING"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/about.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/advanced.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/attribute.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/backward.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/bin_file.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/bmark_add.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/bmark_next.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/bmark_pre.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/cdrom.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/class.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/computer.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/copy.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/cut.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/delete.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/down.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/element.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/file.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/find.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/findr.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/floppy.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/folder.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/font.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/forward.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/function.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/harddisk.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/html_gen.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/indent.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/log.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/mail.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/method.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/new.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/newwin.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/open.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/outdent.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/paste.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/plugin.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/pref.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/print.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/printpre.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/property.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/pyshell.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/quit.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/readonly.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/redo.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/refresh.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/rtf_gen.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/save.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/saveas.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/selectall.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/stop.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/style_edit.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/tex_gen.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/undo.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/up.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/usb.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/variable.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/menu/web.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/mime/boo.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/mime/css.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/mime/html.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/mime/java.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/mime/php.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/mime/python.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/mime/ruby.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/mime/shell.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/mime/text.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/other/doc_props.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/advanced.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/backward.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/copy.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/cut.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/doc_props.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/find.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/findr.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/forward.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/new.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/open.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/package.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/paste.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/pref.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/print.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/redo.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/save.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/theme.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/undo.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/pixmaps/theme/Tango/toolbar/web.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/plugins/CodeBrowser-1.0-py2.5.egg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/plugins/CodeBrowser-1.0-py2.6.egg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/plugins/FileBrowser-1.5-py2.5.egg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/plugins/FileBrowser-1.5-py2.6.egg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/plugins/Launch-1.6-py2.5.egg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/plugins/Launch-1.6-py2.6.egg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/plugins/PyShell-0.8-py2.5.egg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/plugins/PyShell-0.8-py2.6.egg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/Editra.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/autocomp/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/autocomp/autocomp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/autocomp/completer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/autocomp/csscomp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/autocomp/htmlcomp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/autocomp/pycomp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/autocomp/simplecomp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/dev_tool.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/doctools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ebmlib/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ebmlib/backupmgr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ebmlib/clipboard.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ebmlib/cmenumgr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ebmlib/fchecker.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ebmlib/fileimpl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ebmlib/fileutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ebmlib/histcache.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ebmlib/miscutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ebmlib/searcheng.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ebmlib/txtutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/auinavi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/choicedlg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/colorsetter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/ctrlbox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/ecbasewin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/eclutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/ecpickers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/elistmix.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/encdlg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/errdlg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/filemgrdlg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/filterdlg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/finddlg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/infodlg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/outbuff.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/panelbox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/platebtn.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/pstatbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/segmentbk.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/eclib/txtentry.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_art.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_basestc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_cmdbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_crypt.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_editv.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_event.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_glob.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_i18n.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_ipc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_keyh.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_log.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_main.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_mdlg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_menu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_mpane.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_msg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_pages.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_print.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_script.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_search.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_shelf.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_statbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_stc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_style.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_tab.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_theme.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_toolbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_txt.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/ed_vim.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/edimage.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/README"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/aui/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/aui/aui_constants.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/aui/aui_switcherdialog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/aui/aui_utilities.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/aui/auibar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/aui/auibook.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/aui/dockart.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/aui/framemanager.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/aui/tabart.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/aui/tabmdi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/decorlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/embeddedimage.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/events.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/ez_setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/flatnotebook.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pkg_resources.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pubsub.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/cmdline.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/console.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/filter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/filters/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/formatter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/formatters/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/formatters/_mapping.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/formatters/bbcode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/formatters/html.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/formatters/img.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/formatters/latex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/formatters/other.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/formatters/rtf.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/formatters/svg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/formatters/terminal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/formatters/terminal256.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/_clbuiltins.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/_luabuiltins.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/_mapping.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/_phpbuiltins.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/_vimbuiltins.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/agile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/asm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/compiled.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/dotnet.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/functional.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/math.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/other.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/special.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/templates.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/text.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/lexers/web.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/plugin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/scanner.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/style.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/token.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/unistring.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/pygments/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/stcspellcheck.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/extern/vertedit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/generator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/iface.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/info.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/perspective.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/plugdlg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/plugin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/prefdlg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/profiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/style_editor.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/README"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_actionscript.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_ada.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_apache.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_asm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_asm68k.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_batch.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_boo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_caml.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_cobra.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_cpp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_css.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_d.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_diff.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_django.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_dot.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_editra_ss.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_edje.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_eiffel.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_erlang.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_ferite.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_flagship.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_forth.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_fortran.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_groovy.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_gui4cli.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_haskell.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_haxe.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_html.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_inno.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_issuelist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_java.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_javascript.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_kix.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_latex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_lisp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_lout.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_lua.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_make.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_mako.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_masm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_matlab.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_mssql.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_nasm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_nsis.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_pascal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_perl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_php.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_pike.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_postscript.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_progress.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_props.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_python.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_ruby.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_s.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_sh.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_smalltalk.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_sql.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_squirrel.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_stata.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_tcl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_vbscript.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_verilog.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_vhdl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_visualbasic.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_xml.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_xtext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/_yaml.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/syndata.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/synextreg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/synglob.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/syntax.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/syntax/synxml.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/updater.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/src/wxcompat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/styles/BlackBoard.ess"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/styles/BlueMonday.ess"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/styles/Cream.ess"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/styles/Dessert.ess"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/styles/Guepardo.ess"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/styles/Mocha.ess"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/styles/blue.ess"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/styles/default.ess"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/styles/midnight.ess"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/68k_assembly.68k"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/Xtext.xtext"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/actionscript.as"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/ada.adb"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/apache_conf.conf"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/bash_shell_script.sh"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/boo.boo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/c-shell_script.csh"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/c.c"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/caml.ml"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/cascading_style_sheet.css"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/cobra.cobra"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/coldfusion.cfm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/cpp.cpp"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/csharp.cs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/d.d"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/diff_file.diff"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/django.django"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/dos_batch_script.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/dot.dot"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/editra_style_sheet.ess"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/edje.edc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/eiffel.e"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/erlang.erl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/ferite.fe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/flagship.prg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/forth.4th"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/fortran_77.for"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/fortran_95.f95"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/groovy.groovy"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/gui4cli.gui"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/haskell.hs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/haxe.hx"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/html.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/inno_setup_script.iss"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/issuelist.isl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/java.java"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/javascript.js"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/kix.kix"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/korn_shell_script.ksh"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/latex.tex"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/lisp.lisp"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/lout.lt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/lua.lua"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/makefile.mak"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/mako.mako"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/masm.masm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/matlab.matlab"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/microsoft_sql.sql"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/netwide_assembler.nasm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/newlisp.lsp"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/nullsoft_installer_script.nsi"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/objective_c.mm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/octave.oct"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/pascal.pas"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/perl.pl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/php.php"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/pike.pike"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/pl_sql.plsql"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/plain_text.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/postscript.ps"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/progress_4gl.4gl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/properties.ini"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/python.python"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/r.r"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/ruby.rb"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/s.s"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/scheme.scm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/smalltalk.st"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/sql.sql"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/squirrel.nut"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/stata.do"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/svg.svg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/system_verilog.sv"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/tcl_tk.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/unicode_text.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/vala.vala"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/vbscript.vbs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/verilog.v"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/vhdl.vhdl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/visual_basic.vb"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/xml.xml"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/Editra/tests/syntax/yaml.yaml"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/AttributePanel.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/CHANGES.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/TODO.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/TestWin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/XMLTree.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/XMLTreeMenu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/XRCed_16.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/XRCed_32.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/attribute.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/component.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/encode_bitmaps.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/generate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/globals.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/images.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/images_32x32.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/license.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/listener.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/meta.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/model.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/params.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/_bitmaps.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/DynamicSashWindow.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/EditableListBox.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/LEDNumberCtrl.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/TreeListCtrl.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/separator.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/sizer.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/spacer.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/tool.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxBitmapButton.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxBoxSizer.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxButton.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxCheckBox.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxChoice.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxChoicebook.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxComboBox.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxDialog.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxFilePickerCtrl.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxFlexGridSizer.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxFrame.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxGauge.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxGrid.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxGridBagSizer.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxGridSizer.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxGrid_bad.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxHyperlinkCtrl.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxListBox.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxListCtrl.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxListbook.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxMenu.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxMenuBar.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxMenuItem.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxNotebook.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxRadioButton.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxScrollBar.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxScrolledWindow.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxSlider.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxSpinButton.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxSpinCtrl.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxSplitterWindow.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxStaticBitmap.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxStaticBox.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxStaticBoxSizer.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxStaticLine.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxStaticText.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxStatusBar.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxTextCtrl.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxToggleButton.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxToolBar.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxTreeCtrl.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/bitmaps/wxTreebook.png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/controls.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/core.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/gizmos.crx"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/wxlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/xh_gizmos.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/plugins/xh_wxlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/presenter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/sawfishrc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/tools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/undo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/view.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/xrced.htb"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/xrced.ico"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/xrced.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/xrced.sh"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/XRCed/xrced.xrc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/dbg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/genaxmodule.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/helpviewer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/img2img.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/img2png.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/img2py.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/img2xpm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/tools/pywxrc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/webkit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wizard.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxbase28uh_net_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxbase28uh_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxbase28uh_xml_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxmsw28uh_adv_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxmsw28uh_aui_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxmsw28uh_core_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxmsw28uh_gizmos_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxmsw28uh_gizmos_xrc_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxmsw28uh_gl_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxmsw28uh_html_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxmsw28uh_media_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxmsw28uh_qa_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxmsw28uh_richtext_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxmsw28uh_stc_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/wxmsw28uh_xrc_vc.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wx/xrc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/_controls.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/_core.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/_gdi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/_misc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/_windows.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/_wx.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/activex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/animate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/calendar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/gizmos.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/glcanvas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/grid.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/help.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/html.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/htmlhelp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/iewin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/CDate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/ClickableHtmlWindow.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/ErrorDialogs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/ErrorDialogs_wdr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/PythonBitmaps.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/activexwrapper.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/analogclock.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/anchors.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/buttons.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/calendar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/colourchooser/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/colourchooser/canvas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/colourchooser/intl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/colourchooser/pycolourbox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/colourchooser/pycolourchooser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/colourchooser/pycolourslider.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/colourchooser/pypalette.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/colourdb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/colourselect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/dialogs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/editor/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/editor/editor.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/editor/images.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/editor/selection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/evtmgr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/fancytext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/filebrowsebutton.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/floatbar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/floatcanvas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/foldmenu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/gridmovers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/grids.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/imagebrowser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/imageutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/infoframe.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/intctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/layoutf.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/maskedctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/maskededit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/maskednumctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/mixins/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/mixins/grid.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/mixins/imagelist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/mixins/listctrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/mixins/rubberband.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/multisash.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/mvctree.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/newevent.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/plot.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/popupctl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/printout.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/pubsub.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/pyshell.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/rcsizer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/rightalign.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/scrolledpanel.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/sheet.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/shell.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/splashscreen.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/stattext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/throbber.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/timectrl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/wxPlotCanvas.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/lib/wxpTag.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/media.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/stc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/tools/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/tools/dbg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/tools/helpviewer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/tools/img2img.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/tools/img2png.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/tools/img2py.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/tools/img2xpm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/webkit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/wizard.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx-2.8-msw-unicode/wxPython/xrc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wx.pth"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site-packages/wxversion.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/site.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/smtpd.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/smtplib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sndhdr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/socket.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/dbapi2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/dump.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/test/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/test/dbapi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/test/dump.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/test/factory.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/test/hooks.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/test/py25tests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/test/regression.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/test/transactions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/test/types.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sqlite3/test/userfunctions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sre.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sre_compile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sre_constants.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sre_parse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/ssl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/stat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/statvfs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/string.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/stringold.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/stringprep.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/struct.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/subprocess.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sunau.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/sunaudio.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/symbol.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/symtable.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/tabnanny.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/tarfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/telnetlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/tempfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/185test.db"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/Sine-1000Hz-300ms.aif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/audiotest.au"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/autotest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/bad_coding.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/bad_coding2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/badcert.pem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/badkey.pem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/badsyntax_future3.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/badsyntax_future4.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/badsyntax_future5.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/badsyntax_future6.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/badsyntax_future7.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/badsyntax_future8.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/badsyntax_future9.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/badsyntax_nocaret.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/buffer_tests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/cfgparser.1"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/check_soundcard.vbs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/cjkencodings_test.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/cmath_testcases.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/crashers/bogus_code_obj.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/crashers/bogus_sre_bytecode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/crashers/borrowed_ref_1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/crashers/borrowed_ref_2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/crashers/gc_inspection.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/crashers/infinite_loop_re.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/crashers/iter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/crashers/loosing_mro_ref.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/crashers/mutation_inside_cyclegc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/crashers/nasty_eq_vs_dict.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/crashers/recursion_limit_too_high.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/crashers/recursive_call.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/curses_tests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/abs.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/add.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/and.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/base.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/clamp.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/class.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/compare.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/comparetotal.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/comparetotmag.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/copy.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/copyabs.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/copynegate.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/copysign.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddAbs.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddAdd.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddAnd.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddBase.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddCanonical.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddClass.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddCompare.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddCompareSig.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddCompareTotal.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddCompareTotalMag.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddCopy.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddCopyAbs.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddCopyNegate.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddCopySign.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddDivide.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddDivideInt.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddEncode.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddFMA.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddInvert.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddLogB.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddMax.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddMaxMag.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddMin.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddMinMag.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddMinus.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddMultiply.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddNextMinus.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddNextPlus.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddNextToward.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddOr.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddPlus.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddQuantize.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddReduce.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddRemainder.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddRemainderNear.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddRotate.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddSameQuantum.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddScaleB.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddShift.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddSubtract.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddToIntegral.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ddXor.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/decDouble.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/decQuad.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/decSingle.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/divide.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/divideint.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqAbs.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqAdd.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqAnd.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqBase.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqCanonical.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqClass.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqCompare.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqCompareSig.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqCompareTotal.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqCompareTotalMag.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqCopy.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqCopyAbs.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqCopyNegate.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqCopySign.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqDivide.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqDivideInt.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqEncode.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqFMA.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqInvert.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqLogB.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqMax.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqMaxMag.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqMin.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqMinMag.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqMinus.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqMultiply.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqNextMinus.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqNextPlus.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqNextToward.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqOr.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqPlus.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqQuantize.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqReduce.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqRemainder.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqRemainderNear.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqRotate.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqSameQuantum.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqScaleB.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqShift.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqSubtract.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqToIntegral.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dqXor.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dsBase.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/dsEncode.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/exp.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/extra.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/fma.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/inexact.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/invert.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/ln.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/log10.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/logb.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/max.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/maxmag.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/min.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/minmag.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/minus.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/multiply.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/nextminus.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/nextplus.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/nexttoward.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/or.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/plus.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/power.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/powersqrt.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/quantize.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/randomBound32.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/randoms.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/reduce.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/remainder.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/remainderNear.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/rescale.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/rotate.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/rounding.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/samequantum.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/scaleb.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/shift.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/squareroot.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/subtract.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/testall.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/tointegral.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/tointegralx.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/decimaltestdata/xor.decTest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/doctest_aliases.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/double_const.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/empty.vbs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/exception_hierarchy.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/floating_points.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/fork_wait.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/greyrgb.uue"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/https_svn_python_org_root.pem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/ieee754.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/infinite_reload.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/inspect_fodder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/inspect_fodder2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/keycert.pem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/leakers/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/leakers/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/leakers/test_ctypes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/leakers/test_gestalt.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/leakers/test_selftype.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/list_tests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/lock_tests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/mapping_tests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/nullcert.pem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/outstanding_bugs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/pickletester.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/profilee.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/pyclbr_input.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/pydoc_mod.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/pydocfodder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/pystone.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/randv2_32.pck"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/randv2_64.pck"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/randv3.pck"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/re_tests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/readme.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/regex_tests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/regrtest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/relimport.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/reperf.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/sample_doctest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/seq_tests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/sgml_input.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/sha256.pem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/sortperf.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/ssl_cert.pem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/ssl_key.pem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/string_tests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/svn_python_org_https_cert.pem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test.xml"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test.xml.out"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_MimeWriter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_SimpleHTTPServer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_StringIO.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test___all__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test___future__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test__locale.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_abc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_abstract_numbers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_aepack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_aifc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_al.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_anydbm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_applesingle.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_array.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_ast.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_asynchat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_asyncore.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_atexit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_audioop.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_augassign.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_base64.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_bastion.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_bigaddrspace.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_bigmem.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_binascii.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_binhex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_binop.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_bisect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_bool.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_bsddb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_bsddb185.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_bsddb3.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_buffer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_bufio.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_builtin.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_bytes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_bz2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_calendar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_call.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_capi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_cd.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_cfgparser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_cgi.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_charmapcodec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_cl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_class.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_cmath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_cmd.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_cmd_line.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_cmd_line_script.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_code.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codeccallbacks.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codecencodings_cn.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codecencodings_hk.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codecencodings_jp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codecencodings_kr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codecencodings_tw.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codecmaps_cn.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codecmaps_hk.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codecmaps_jp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codecmaps_kr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codecmaps_tw.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codecs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_codeop.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_coding.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_coercion.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_collections.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_colorsys.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_commands.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_compare.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_compile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_compiler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_complex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_complex_args.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_contains.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_contextlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_cookie.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_cookielib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_copy.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_copy_reg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_cpickle.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_cprofile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_crypt.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_csv.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_ctypes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_curses.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_datetime.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_dbm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_decimal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_decorators.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_defaultdict.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_deque.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_descr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_descrtut.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_dict.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_difflib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_difflib_expect.html"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_dircache.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_dis.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_distutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_dl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_doctest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_doctest.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_doctest2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_doctest2.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_doctest3.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_doctest4.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_docxmlrpc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_dumbdbm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_dummy_thread.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_dummy_threading.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_email.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_email_codecs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_email_renamed.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_enumerate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_eof.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_epoll.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_errno.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_exception_variations.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_exceptions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_extcall.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_fcntl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_file.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_filecmp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_fileinput.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_fileio.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_float.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_fnmatch.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_fork1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_format.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_fpformat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_fractions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_frozen.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_ftplib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_funcattrs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_functools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_future.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_future1.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_future2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_future3.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_future4.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_future5.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_future_builtins.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_gc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_gdbm.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_generators.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_genericpath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_genexps.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_getargs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_getargs2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_getopt.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_gettext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_gl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_glob.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_global.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_grammar.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_grp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_gzip.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_hash.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_hashlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_heapq.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_hmac.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_hotshot.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_htmllib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_htmlparser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_httplib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_httpservers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_imageop.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_imaplib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_imgfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_imp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_import.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_importhooks.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_index.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_inspect.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_int.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_int_literal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_io.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_ioctl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_isinstance.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_iter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_iterlen.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_itertools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_json.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_kqueue.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_largefile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_lib2to3.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_linecache.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_linuxaudiodev.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_list.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_locale.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_logging.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_long.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_long_future.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_longexp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_macos.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_macostools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_macpath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_mailbox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_marshal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_math.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_md5.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_memoryio.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_mhlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_mimetools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_mimetypes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_minidom.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_mmap.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_module.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_modulefinder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_multibytecodec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_multibytecodec_support.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_multifile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_multiprocessing.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_mutants.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_mutex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_netrc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_new.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_nis.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_normalization.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_ntpath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_old_mailbox.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_opcodes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_openpty.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_operator.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_optparse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_os.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_ossaudiodev.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_parser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_peepholer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pep247.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pep263.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pep277.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pep292.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pep352.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pickle.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pickletools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pipes.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pkg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pkgimport.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pkgutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_platform.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_plistlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_poll.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_popen.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_popen2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_poplib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_posix.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_posixpath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pow.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pprint.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_print.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_profile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_profilehooks.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_property.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pstats.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pty.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pwd.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_py3kwarn.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pyclbr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pydoc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_pyexpat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_queue.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_quopri.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_random.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_re.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_readline.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_repr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_resource.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_rfc822.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_richcmp.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_robotparser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_runpy.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_sax.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_scope.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_scriptpackages.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_select.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_set.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_sets.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_sgmllib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_sha.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_shelve.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_shlex.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_shutil.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_signal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_site.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_slice.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_smtplib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_smtpnet.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_socket.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_socketserver.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_softspace.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_sort.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_sqlite.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_ssl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_startfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_str.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_strftime.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_string.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_stringprep.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_strop.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_strptime.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_struct.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_structmembers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_structseq.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_subprocess.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_sunaudiodev.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_sundry.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_support.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_symtable.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_syntax.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_sys.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_tarfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_tcl.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_telnetlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_tempfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_textwrap.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_thread.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_threaded_import.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_threadedtempfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_threading.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_threading_local.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_threadsignals.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_time.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_timeout.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_tokenize.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_trace.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_traceback.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_transformer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_tuple.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_typechecks.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_types.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_ucn.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_unary.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_undocumented_details.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_unicode.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_unicode_file.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_unicodedata.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_unittest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_univnewlines.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_unpack.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_urllib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_urllib2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_urllib2_localnet.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_urllib2net.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_urllibnet.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_urlparse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_userdict.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_userlist.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_userstring.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_uu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_uuid.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_wait3.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_wait4.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_warnings.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_wave.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_weakref.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_whichdb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_winreg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_winsound.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_with.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_wsgiref.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_xdrlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_xml_etree.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_xml_etree_c.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_xmllib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_xmlrpc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_xpickle.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_xrange.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_zipfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_zipfile64.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_zipimport.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_zipimport_support.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/test_zlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/testall.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/testcodec.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/testimg.uue"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/testimgr.uue"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/testrgb.uue"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/testtar.tar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/tf_inherit_check.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/threaded_import_hangers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/time_hashlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/tokenize_tests.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/warning_tests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/wrongcert.pem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/xmltests.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/test/zipdir.zip"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/textwrap.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/this.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/threading.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/timeit.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/toaiff.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/token.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/tokenize.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/trace.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/traceback.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/tty.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/types.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/unittest.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/urllib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/urllib2.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/urlparse.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/user.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/uu.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/uuid.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/warnings.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/wave.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/weakref.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/webbrowser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/whichdb.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/wsgiref/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/wsgiref/handlers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/wsgiref/headers.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/wsgiref/simple_server.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/wsgiref/util.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/wsgiref/validate.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xdrlib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/dom/NodeFilter.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/dom/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/dom/domreg.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/dom/expatbuilder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/dom/minicompat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/dom/minidom.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/dom/pulldom.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/dom/xmlbuilder.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/etree/ElementInclude.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/etree/ElementPath.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/etree/ElementTree.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/etree/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/etree/cElementTree.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/parsers/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/parsers/expat.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/sax/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/sax/_exceptions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/sax/expatreader.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/sax/handler.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/sax/saxutils.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xml/sax/xmlreader.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xmllib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/xmlrpclib.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Lib/zipfile.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/NEWS.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/RemoveMySQL-python.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Removematplotlib.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Removenumpy.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Removepywin32.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/CreateBatchFiles.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/CreateMacScripts.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/editra"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/editra.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/f2py.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/genaxmodule.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/helpviewer"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/helpviewer.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/img2png"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/img2png.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/img2py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/img2py.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/img2xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/img2xpm.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pyalacarte"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pyalacarte.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pyalamode"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pyalamode.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pycrust"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pycrust.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pyshell"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pyshell.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pyslices"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pyslices.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pysliceshell"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pysliceshell.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pywin32_postinstall.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pywrap"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pywrap.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pywxrc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/pywxrc.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/xrced"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Scripts/xrced.bat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/2to3.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/byext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/byteyears.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/checkappend.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/checkpyc.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/classfix.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/cleanfuture.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/combinerefs.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/copytime.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/crlf.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/cvsfiles.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/db2pickle.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/diff.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/dutree.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/eptags.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/finddiv.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/findlinksto.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/findnocoding.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/fixcid.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/fixdiv.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/fixheader.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/fixnotice.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/fixps.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/ftpmirror.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/google.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/gprof2html.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/h2py.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/hotshotmain.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/ifdef.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/lfcr.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/linktree.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/lll.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/logmerge.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/mailerdaemon.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/md5sum.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/methfix.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/mkreal.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/ndiff.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/nm2def.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/objgraph.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/parseentities.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/patchcheck.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/pathfix.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/pdeps.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/pickle2db.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/pindent.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/ptags.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/pydocgui.pyw"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/pysource.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/redemo.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/reindent-rst.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/reindent.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/rgrep.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/setup.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/suff.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/svneol.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/texcheck.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/texi2html.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/treesync.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/untabify.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/which.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/win_add2path.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/Scripts/xxci.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/i18n/makelocalealias.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/i18n/msgfmt.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/i18n/pygettext.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/ChipViewer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/ColorDB.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/DetailsViewer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/ListViewer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/Main.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/PyncheWidget.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/StripViewer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/Switchboard.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/TextViewer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/TypeinViewer.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/X/rgb.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/X/xlicense.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/__init__.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/html40colors.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/namedcolors.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/pyColorChooser.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/pynche.pyw"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/webcolors.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/pynche/websafe.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/versioncheck/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/versioncheck/_checkversion.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/versioncheck/checkversions.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/versioncheck/pyversioncheck.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/webchecker/README.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/webchecker/tktools.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/webchecker/wcgui.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/webchecker/wcmac.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/webchecker/webchecker.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/webchecker/websucker.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/Tools/webchecker/wsgui.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/Python-ast.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/Python.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/abstract.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/asdl.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/ast.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/bitset.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/boolobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/bufferobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/bytearrayobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/bytes_methods.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/bytesobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/cStringIO.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/cellobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/ceval.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/classobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/cobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/code.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/codecs.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/compile.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/complexobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/datetime.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/descrobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/dictobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/enumobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/errcode.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/eval.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/fileobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/floatobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/frameobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/funcobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/genobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/graminit.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/grammar.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/import.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/intobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/intrcheck.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/iterobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/listobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/longintrepr.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/longobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/marshal.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/metagrammar.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/methodobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/modsupport.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/moduleobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/node.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/object.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/objimpl.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/opcode.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/osdefs.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/parsetok.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/patchlevel.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pgen.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pgenheaders.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/py_curses.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pyarena.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pyconfig.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pydebug.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pyerrors.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pyexpat.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pyfpe.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pygetopt.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pymacconfig.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pymactoolbox.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pymath.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pymem.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pyport.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pystate.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pystrcmp.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pystrtod.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pythonrun.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/pythread.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/rangeobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/setobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/sliceobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/stringobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/structmember.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/structseq.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/symtable.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/sysmodule.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/timefuncs.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/token.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/traceback.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/tupleobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/ucnhash.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/unicodeobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/warnings.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/include/weakrefobject.h"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/_bsddb.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/_ctypes.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/_ctypes_test.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/_elementtree.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/_hashlib.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/_msi.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/_multiprocessing.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/_socket.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/_sqlite3.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/_ssl.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/_testcapi.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/_tkinter.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/bz2.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/libpython26.a"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/pyexpat.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/python26.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/select.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/unicodedata.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/libs/winsound.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/python.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/python26.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/pythonw.exe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/dde1.3/pkgIndex.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/dde1.3/tcldde13.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/reg1.2/pkgIndex.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/reg1.2/tclreg12.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/auto.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/clock.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/ascii.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/big5.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp1250.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp1251.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp1252.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp1253.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp1254.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp1255.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp1256.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp1257.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp1258.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp437.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp737.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp775.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp850.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp852.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp855.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp857.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp860.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp861.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp862.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp863.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp864.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp865.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp866.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp869.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp874.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp932.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp936.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp949.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/cp950.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/dingbats.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/ebcdic.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/euc-cn.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/euc-jp.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/euc-kr.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/gb12345.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/gb1988.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/gb2312-raw.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/gb2312.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso2022-jp.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso2022-kr.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso2022.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-1.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-10.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-13.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-14.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-15.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-16.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-2.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-3.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-4.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-5.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-6.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-7.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-8.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/iso8859-9.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/jis0201.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/jis0208.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/jis0212.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/koi8-r.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/koi8-u.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/ksc5601.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/macCentEuro.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/macCroatian.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/macCyrillic.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/macDingbats.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/macGreek.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/macIceland.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/macJapan.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/macRoman.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/macRomania.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/macThai.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/macTurkish.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/macUkraine.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/shiftjis.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/symbol.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/encoding/tis-620.enc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/history.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/http1.0/http.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/http1.0/pkgIndex.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/init.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/af.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/af_za.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ar.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ar_in.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ar_jo.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ar_lb.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ar_sy.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/be.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/bg.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/bn.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/bn_in.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ca.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/cs.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/da.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/de.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/de_at.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/de_be.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/el.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_au.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_be.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_bw.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_ca.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_gb.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_hk.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_ie.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_in.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_nz.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_ph.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_sg.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_za.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/en_zw.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/eo.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_ar.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_bo.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_cl.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_co.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_cr.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_do.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_ec.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_gt.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_hn.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_mx.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_ni.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_pa.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_pe.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_pr.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_py.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_sv.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_uy.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/es_ve.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/et.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/eu.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/eu_es.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/fa.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/fa_in.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/fa_ir.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/fi.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/fo.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/fo_fo.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/fr.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/fr_be.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/fr_ca.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/fr_ch.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ga.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ga_ie.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/gl.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/gl_es.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/gv.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/gv_gb.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/he.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/hi.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/hi_in.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/hr.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/hu.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/id.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/id_id.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/is.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/it.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/it_ch.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ja.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/kl.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/kl_gl.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ko.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ko_kr.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/kok.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/kok_in.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/kw.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/kw_gb.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/lt.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/lv.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/mk.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/mr.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/mr_in.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ms.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ms_my.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/mt.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/nb.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/nl.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/nl_be.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/nn.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/pl.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/pt.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/pt_br.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ro.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ru.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ru_ua.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/sh.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/sk.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/sl.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/sq.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/sr.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/sv.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/sw.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ta.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/ta_in.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/te.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/te_in.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/th.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/tr.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/uk.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/vi.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/zh.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/zh_cn.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/zh_hk.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/zh_sg.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/msgs/zh_tw.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/opt0.4/optparse.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/opt0.4/pkgIndex.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/package.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/parray.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/safe.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tclIndex"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tm.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Abidjan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Accra"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Addis_Ababa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Algiers"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Asmara"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Asmera"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Bamako"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Bangui"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Banjul"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Bissau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Blantyre"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Brazzaville"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Bujumbura"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Cairo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Casablanca"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Ceuta"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Conakry"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Dakar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Dar_es_Salaam"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Djibouti"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Douala"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/El_Aaiun"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Freetown"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Gaborone"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Harare"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Johannesburg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Kampala"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Khartoum"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Kigali"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Kinshasa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Lagos"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Libreville"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Lome"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Luanda"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Lubumbashi"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Lusaka"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Malabo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Maputo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Maseru"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Mbabane"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Mogadishu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Monrovia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Nairobi"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Ndjamena"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Niamey"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Nouakchott"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Ouagadougou"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Porto-Novo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Sao_Tome"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Timbuktu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Tripoli"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Tunis"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Africa/Windhoek"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Adak"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Anchorage"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Anguilla"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Antigua"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Araguaina"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Argentina/Buenos_Aires"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Argentina/Catamarca"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Argentina/ComodRivadavia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Argentina/Cordoba"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Argentina/Jujuy"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Argentina/La_Rioja"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Argentina/Mendoza"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Argentina/Rio_Gallegos"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Argentina/San_Juan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Argentina/San_Luis"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Argentina/Tucuman"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Argentina/Ushuaia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Aruba"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Asuncion"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Atikokan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Atka"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Bahia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Barbados"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Belem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Belize"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Blanc-Sablon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Boa_Vista"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Bogota"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Boise"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Buenos_Aires"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Cambridge_Bay"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Campo_Grande"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Cancun"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Caracas"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Catamarca"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Cayenne"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Cayman"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Chicago"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Chihuahua"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Coral_Harbour"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Cordoba"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Costa_Rica"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Cuiaba"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Curacao"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Danmarkshavn"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Dawson"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Dawson_Creek"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Denver"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Detroit"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Dominica"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Edmonton"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Eirunepe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/El_Salvador"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Ensenada"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Fort_Wayne"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Fortaleza"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Glace_Bay"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Godthab"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Goose_Bay"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Grand_Turk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Grenada"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Guadeloupe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Guatemala"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Guayaquil"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Guyana"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Halifax"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Havana"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Hermosillo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Indiana/Indianapolis"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Indiana/Knox"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Indiana/Marengo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Indiana/Petersburg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Indiana/Tell_City"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Indiana/Vevay"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Indiana/Vincennes"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Indiana/Winamac"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Indianapolis"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Inuvik"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Iqaluit"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Jamaica"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Jujuy"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Juneau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Kentucky/Louisville"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Kentucky/Monticello"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Knox_IN"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/La_Paz"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Lima"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Los_Angeles"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Louisville"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Maceio"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Managua"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Manaus"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Marigot"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Martinique"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Mazatlan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Mendoza"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Menominee"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Merida"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Mexico_City"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Miquelon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Moncton"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Monterrey"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Montevideo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Montreal"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Montserrat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Nassau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/New_York"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Nipigon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Nome"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Noronha"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/North_Dakota/Center"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/North_Dakota/New_Salem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Panama"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Pangnirtung"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Paramaribo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Phoenix"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Port-au-Prince"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Port_of_Spain"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Porto_Acre"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Porto_Velho"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Puerto_Rico"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Rainy_River"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Rankin_Inlet"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Recife"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Regina"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Resolute"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Rio_Branco"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Rosario"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Santiago"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Santo_Domingo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Sao_Paulo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Scoresbysund"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Shiprock"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/St_Barthelemy"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/St_Johns"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/St_Kitts"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/St_Lucia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/St_Thomas"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/St_Vincent"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Swift_Current"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Tegucigalpa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Thule"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Thunder_Bay"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Tijuana"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Toronto"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Tortola"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Vancouver"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Virgin"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Whitehorse"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Winnipeg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Yakutat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/America/Yellowknife"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Antarctica/Casey"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Antarctica/Davis"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Antarctica/DumontDUrville"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Antarctica/Mawson"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Antarctica/McMurdo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Antarctica/Palmer"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Antarctica/Rothera"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Antarctica/South_Pole"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Antarctica/Syowa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Antarctica/Vostok"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Arctic/Longyearbyen"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Aden"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Almaty"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Amman"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Anadyr"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Aqtau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Aqtobe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Ashgabat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Ashkhabad"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Baghdad"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Bahrain"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Baku"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Bangkok"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Beirut"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Bishkek"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Brunei"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Calcutta"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Choibalsan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Chongqing"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Chungking"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Colombo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Dacca"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Damascus"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Dhaka"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Dili"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Dubai"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Dushanbe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Gaza"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Harbin"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Ho_Chi_Minh"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Hong_Kong"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Hovd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Irkutsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Istanbul"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Jakarta"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Jayapura"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Jerusalem"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Kabul"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Kamchatka"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Karachi"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Kashgar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Katmandu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Kolkata"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Krasnoyarsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Kuala_Lumpur"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Kuching"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Kuwait"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Macao"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Macau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Magadan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Makassar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Manila"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Muscat"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Nicosia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Novosibirsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Omsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Oral"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Phnom_Penh"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Pontianak"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Pyongyang"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Qatar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Qyzylorda"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Rangoon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Riyadh"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Saigon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Sakhalin"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Samarkand"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Seoul"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Shanghai"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Singapore"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Taipei"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Tashkent"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Tbilisi"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Tehran"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Tel_Aviv"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Thimbu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Thimphu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Tokyo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Ujung_Pandang"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Ulaanbaatar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Ulan_Bator"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Urumqi"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Vientiane"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Vladivostok"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Yakutsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Yekaterinburg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Asia/Yerevan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Atlantic/Azores"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Atlantic/Bermuda"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Atlantic/Canary"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Atlantic/Cape_Verde"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Atlantic/Faeroe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Atlantic/Faroe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Atlantic/Jan_Mayen"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Atlantic/Madeira"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Atlantic/Reykjavik"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Atlantic/South_Georgia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Atlantic/St_Helena"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Atlantic/Stanley"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/ACT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Adelaide"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Brisbane"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Broken_Hill"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Canberra"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Currie"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Darwin"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Eucla"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Hobart"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/LHI"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Lindeman"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Lord_Howe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Melbourne"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/NSW"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/North"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Perth"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Queensland"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/South"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Sydney"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Tasmania"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Victoria"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/West"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Australia/Yancowinna"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Brazil/Acre"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Brazil/DeNoronha"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Brazil/East"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Brazil/West"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/CET"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/CST6CDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Canada/Atlantic"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Canada/Central"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Canada/East-Saskatchewan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Canada/Eastern"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Canada/Mountain"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Canada/Newfoundland"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Canada/Pacific"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Canada/Saskatchewan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Canada/Yukon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Chile/Continental"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Chile/EasterIsland"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Cuba"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/EET"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/EST"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/EST5EDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Egypt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Eire"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+0"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+1"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+10"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+11"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+12"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+2"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+3"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+4"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+5"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+6"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+7"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+8"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT+9"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-0"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-1"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-10"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-11"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-12"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-13"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-14"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-2"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-3"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-4"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-5"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-6"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-7"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-8"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT-9"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/GMT0"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/Greenwich"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/UCT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/UTC"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/Universal"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Etc/Zulu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Amsterdam"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Andorra"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Athens"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Belfast"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Belgrade"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Berlin"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Bratislava"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Brussels"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Bucharest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Budapest"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Chisinau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Copenhagen"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Dublin"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Gibraltar"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Guernsey"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Helsinki"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Isle_of_Man"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Istanbul"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Jersey"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Kaliningrad"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Kiev"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Lisbon"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Ljubljana"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/London"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Luxembourg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Madrid"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Malta"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Mariehamn"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Minsk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Monaco"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Moscow"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Nicosia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Oslo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Paris"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Podgorica"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Prague"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Riga"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Rome"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Samara"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/San_Marino"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Sarajevo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Simferopol"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Skopje"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Sofia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Stockholm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Tallinn"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Tirane"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Tiraspol"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Uzhgorod"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Vaduz"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Vatican"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Vienna"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Vilnius"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Volgograd"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Warsaw"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Zagreb"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Zaporozhye"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Europe/Zurich"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/GB"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/GB-Eire"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/GMT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/GMT+0"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/GMT-0"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/GMT0"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Greenwich"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/HST"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Hongkong"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Iceland"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Indian/Antananarivo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Indian/Chagos"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Indian/Christmas"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Indian/Cocos"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Indian/Comoro"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Indian/Kerguelen"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Indian/Mahe"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Indian/Maldives"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Indian/Mauritius"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Indian/Mayotte"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Indian/Reunion"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Iran"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Israel"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Jamaica"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Japan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Kwajalein"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Libya"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/MET"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/MST"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/MST7MDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Mexico/BajaNorte"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Mexico/BajaSur"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Mexico/General"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/NZ"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/NZ-CHAT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Navajo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/PRC"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/PST8PDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Apia"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Auckland"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Chatham"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Easter"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Efate"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Enderbury"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Fakaofo"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Fiji"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Funafuti"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Galapagos"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Gambier"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Guadalcanal"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Guam"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Honolulu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Johnston"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Kiritimati"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Kosrae"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Kwajalein"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Majuro"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Marquesas"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Midway"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Nauru"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Niue"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Norfolk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Noumea"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Pago_Pago"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Palau"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Pitcairn"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Ponape"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Port_Moresby"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Rarotonga"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Saipan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Samoa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Tahiti"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Tarawa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Tongatapu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Truk"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Wake"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Wallis"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Pacific/Yap"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Poland"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Portugal"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/ROC"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/ROK"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Singapore"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/AST4"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/AST4ADT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/CST6"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/CST6CDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/EST5"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/EST5EDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/HST10"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/MST7"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/MST7MDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/PST8"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/PST8PDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/YST9"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/SystemV/YST9YDT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Turkey"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/UCT"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/Alaska"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/Aleutian"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/Arizona"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/Central"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/East-Indiana"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/Eastern"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/Hawaii"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/Indiana-Starke"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/Michigan"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/Mountain"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/Pacific"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/Pacific-New"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/US/Samoa"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/UTC"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Universal"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/W-SU"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/WET"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/tzdata/Zulu"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8.5/word.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8/8.4/http-2.7.tm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8/8.4/platform-1.0.3.tm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8/8.4/platform/shell-1.1.3.tm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8/8.5/msgcat-1.4.2.tm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl8/8.5/tcltest-2.3.0.tm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tcl85.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tclConfig.sh"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tclstub85.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Balloon.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/BtnBox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/CObjView.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/ChkList.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/ComboBox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Compat.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Console.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Control.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/DefSchm.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/DialogS.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/DirBox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/DirDlg.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/DirList.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/DirTree.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/DragDrop.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/DtlList.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/EFileBox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/EFileDlg.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Event.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/FileBox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/FileCbx.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/FileDlg.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/FileEnt.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/FloatEnt.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Grid.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/HList.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/HListDD.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/IconView.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Init.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/LabEntry.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/LabFrame.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/LabWidg.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/ListNBk.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Makefile"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Meter.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/MultView.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/NoteBook.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/OldUtil.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/OptMenu.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/PanedWin.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/PopMenu.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Primitiv.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/ResizeH.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/SGrid.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/SHList.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/SListBox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/STList.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/SText.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/SWidget.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/SWindow.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Select.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Shell.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/SimpDlg.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/StackWin.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/StatBar.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/StdBBox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/StdShell.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/TList.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Tix.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Tree.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Utils.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/VResize.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/VStack.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/VTree.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/Variable.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/WInfo.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/act_fold.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/act_fold.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/act_fold.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/balarrow.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/cbxarrow.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/ck_def.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/ck_off.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/ck_on.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/cross.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/decr.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/drop.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/file.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/file.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/file.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/folder.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/folder.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/folder.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/harddisk.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/hourglas.mask"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/hourglas.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/incr.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/info.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/info.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/maximize.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/minimize.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/minus.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/minus.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/minus.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/minusarm.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/minusarm.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/minusarm.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/mktransgif.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/network.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/no_entry.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/no_entry.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/openfile.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/openfold.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/openfold.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/openfold.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/plus.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/plus.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/plus.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/plusarm.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/plusarm.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/plusarm.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/resize1.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/resize2.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/restore.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/srcfile.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/srcfile.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/srcfile.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/system.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/textfile.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/textfile.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/textfile.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/tick.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/warning.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/bitmaps/warning.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/MkChoose.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/MkDirLis.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/MkSample.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/MkScroll.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/about.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/bold.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/capital.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/centerj.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/code.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/combobox.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/combobox.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/drivea.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/drivea.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/exit.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/filebox.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/filebox.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/harddisk.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/harddisk.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/italic.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/justify.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/leftj.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/netw.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/netw.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/network.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/network.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/optmenu.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/rightj.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/select.xpm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/tix.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/bitmaps/underlin.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/AllSampl.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/ArrowBtn.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/Balloon.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/BtnBox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/CObjView.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/ChkList.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/CmpImg.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/CmpImg1.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/CmpImg2.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/CmpImg3.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/CmpImg4.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/ComboBox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/Control.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/DirDlg.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/DirList.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/DirTree.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/DragDrop.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/DynTree.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/EFileDlg.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/EditGrid.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/FileDlg.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/FileEnt.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/HList1.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/LabEntry.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/LabFrame.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/ListNBK.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/Meter.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/NoteBook.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/OptMenu.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/PanedWin.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/PopMenu.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/SGrid0.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/SGrid1.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/SHList.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/SHList2.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/SListBox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/STList1.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/STList2.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/STList3.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/SText.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/SWindow.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/Sample.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/Select.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/StdBBox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/Tree.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/Xpm.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/samples/Xpm1.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/tclIndex"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/tixwidgets.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/demos/widget"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/fs.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pkgIndex.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/10Point.fs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/10Point.fsc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/12Point.fs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/12Point.fsc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/14Point.fs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/14Point.fsc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/Bisque.cs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/Bisque.csc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/Blue.cs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/Blue.csc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/Gray.cs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/Gray.csc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/Makefile"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/Old12Pt.fs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/Old14Pt.fs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/SGIGray.cs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/SGIGray.csc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/TK.cs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/TK.csc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/TK.fs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/TK.fsc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/TixGray.cs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/TixGray.csc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/TkWin.cs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/TkWin.csc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/TkWin.fs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/TkWin.fsc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/WmDefault.cs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/WmDefault.csc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/WmDefault.fs"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/WmDefault.fsc"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/WmDefault.py"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/WmDefault.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/WmDefault.txt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/pkgIndex.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/pref/tixmkpref"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/tix84.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tix8.4.3/tix84.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/bgerror.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/button.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/choosedir.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/clrpick.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/comdlg.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/console.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/README"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/anilabel.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/aniwave.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/arrow.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/bind.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/bitmap.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/browse"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/button.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/check.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/clrpick.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/colors.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/combo.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/cscroll.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/ctext.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/dialog1.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/dialog2.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/en.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/entry1.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/entry2.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/entry3.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/filebox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/floor.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/form.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/goldberg.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/hello"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/hscale.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/icon.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/image1.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/image2.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/images/earth.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/images/earthris.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/images/face.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/images/flagdown.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/images/flagup.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/images/gray25.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/images/letters.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/images/noletter.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/images/pattern.xbm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/images/tcllogo.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/images/teapot.ppm"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/items.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/ixset"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/knightstour.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/label.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/labelframe.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/license.terms"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/mclist.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/menu.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/menubu.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/msgbox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/nl.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/paned1.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/paned2.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/pendulum.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/plot.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/puzzle.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/radio.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/rmt"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/rolodex"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/ruler.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/sayings.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/search.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/spin.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/square"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/states.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/style.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/tclIndex"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/tcolor"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/text.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/textpeer.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/timer"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/toolbar.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/tree.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/ttkbut.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/ttkmenu.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/ttknote.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/ttkpane.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/ttkprogress.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/twind.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/unicodeout.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/vscale.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/demos/widget"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/dialog.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/entry.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/focus.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/README"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/logo.eps"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/logo100.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/logo64.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/logoLarge.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/logoMed.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/pwrdLogo.eps"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/pwrdLogo100.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/pwrdLogo150.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/pwrdLogo175.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/pwrdLogo200.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/pwrdLogo75.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/images/tai-ku.gif"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/license.terms"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/listbox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/menu.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/mkpsenc.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgbox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/cs.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/da.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/de.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/el.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/en.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/en_gb.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/eo.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/es.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/fr.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/hu.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/it.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/nl.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/pl.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/pt.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/ru.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/msgs/sv.msg"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/obsolete.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/optMenu.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/palette.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/panedwindow.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/pkgIndex.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/safetk.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/scale.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/scrlbar.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/spinbox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/tclIndex"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/tearoff.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/text.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/tk.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/tkfbox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/altTheme.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/aquaTheme.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/button.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/clamTheme.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/classicTheme.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/combobox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/cursors.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/defaults.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/entry.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/fonts.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/menubutton.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/notebook.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/panedwindow.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/progress.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/scale.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/scrollbar.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/sizegrip.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/treeview.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/ttk.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/utils.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/winTheme.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/ttk/xpTheme.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/unsupported.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk8.5/xmfbox.tcl"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tk85.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/tcl/tkstub85.lib"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/unicows.dll"
	Delete /REBOOTOK "$INSTDIR\Python26Einstein/w9xpopen.exe"
	Delete /REBOOTOK "$INSTDIR\databases/benchmarks.xml"
	Delete /REBOOTOK "$INSTDIR\databases/boilers.xml"
	Delete /REBOOTOK "$INSTDIR\databases/chp.xml"
	Delete /REBOOTOK "$INSTDIR\databases/electricity_mix.xml"
	Delete /REBOOTOK "$INSTDIR\databases/fluids.xml"
	Delete /REBOOTOK "$INSTDIR\databases/fuels.xml"
	Delete /REBOOTOK "$INSTDIR\databases/heatpumps_and_chillers.xml"
	Delete /REBOOTOK "$INSTDIR\databases/solarthermal.xml"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - BB Module.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - Building Module.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - CCheck.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - CH Module.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - CHP Module.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - EconomicAnalysis.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - HP Module.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - HR Module.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - HTP Module.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - Modules.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - Pipes.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - ProcessModel.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - RT Simulation.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - ST Module.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - SystemSimulation.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - Title.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual - WeatherData.odt"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual.odm"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/EINSTEIN Tool Technical Manual.pdf"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/figures_and_tables/HXDesignModes.odg"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/figures_and_tables/ModuleCC_FlowChart.odg"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/figures_and_tables/ModuleCC_ProbabilityFunction.ods"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/figures_and_tables/ModuleCHP_matching.ods"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/figures_and_tables/ModuleCH_DefaultTemperatures.odg"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/figures_and_tables/ModuleCH_PartLoadCorrection.ods"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/figures_and_tables/ModuleEnergy_SimulationModes.ods"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/figures_and_tables/ModuleHR_Model_HXwithStorage.odg"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/figures_and_tables/ModuleST_AreaRatios.ods"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/figures_and_tables/SystemSimulation_FlowChart.odg"
	Delete /REBOOTOK "$INSTDIR\docs/technical_manual/templates/EINSTEIN Tool Technical Manual - XY Module - Template.odt"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/Makefile"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/.buildinfo"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_downloads/EINSTEIN Show Case DemoTFC - Description.pdf"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_downloads/EINSTEIN Show Case DemoTFC - Materials.zip"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_downloads/EINSTEIN Tool Technical Manual.pdf"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_downloads/EINSTEIN_Parameter_List.xls"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/BM_equation.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/CC_blockscheme.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/CC_screenshotconflicts.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/CC_screenshotstandard.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/DBProfiles-Edit.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/DBProfiles-List.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/DataEntryElectricity.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/DataEntryEquipment_1.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/DataEntryEquipment_2.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/DataEntryHTP_1.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/DataEntryHTP_2.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/DataEntryHTP_3.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/DataEntryHTP_4.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/DataEntryHTP_5.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/ECO_Energy.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/ECO_Investment.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/ECO_Summary.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/EditCon.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/EditHXNet.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/MFEditor_1.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/MFEditor_2.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QHX-Economic.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QHX-EditCon.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QHX-Technical.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QM1.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QM2.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QM3.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QM4.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QPipe.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QProc-Batch.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QProc-Continuous.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QProc-Demand.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QProc-Description.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QProc-DetailedSchedule.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QProc-Residual.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/QProc-Schedule.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/Report_1.JPG"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/Report_2.JPG"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/ScreenShotBB.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/ScreenShotBBPopUp.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/ScreenShotST.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/Screenshot_dbST.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/TCA_DetailOperating1.gif"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/TCA_EnergyandO_new.gif"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/TCA_EnergyandO_old.gif"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/TCA_Investment_25.gif"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/TCA_NonReoccurring.gif"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/TCA_contingencies.gif"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/TCA_main2_25.gif"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/TimeSchedules_1.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/TimeSchedules_2.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/TimeSchedules_3.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/building-parameters1.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/building-parameters2.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/einstein_logo.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/gplv3.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/math/5b4271afe7fc7c0ee84172e0ad19b82caf450c00.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/math/b1bb0c6477a7a66c52ec2317040600a57d9cedbd.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/math/c35df8214cf046662c645bbec76d56659e3c625d.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/math/d76a46db643db999258b871d707298e64080559f.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/process-times.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/screenshot_profiles.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/screenshot_schedules.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/screenshot_simple_schedules.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_images/zunge.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/agogo.css"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/ajax-loader.gif"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/basic.css"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/bgfooter.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/bgtop.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/comment-bright.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/comment-close.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/comment.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/contents.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/default.css"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/dialog-note.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/dialog-seealso.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/dialog-todo.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/dialog-topic.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/dialog-warning.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/doctools.js"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/down-pressed.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/down.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/einstein.css"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/epub.css"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/file.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/footerbg.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/headerbg.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/ie6.css"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/jquery.js"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/middlebg.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/minus.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/navigation.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/plus.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/pygments.css"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/pyramid.css"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/searchtools.js"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/sidebar.js"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/sphinxdoc.css"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/transparent.gif"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/underscore.js"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/up-pressed.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/up.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/_static/websupport.js"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/developers/api.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/genindex.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/gpl3.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/index.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/objects.inv"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/py-modindex.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/redirect.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/search.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/searchindex.js"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/technical/manual.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/HC_BB.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/HC_BIO.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/HC_CH.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/HC_CHP.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/HC_HP.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/HC_ST.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/HC_general.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/HC_storage.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/RTopt.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/RTsim.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/benchmark.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/building.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/comparative_analysis.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/consistency_check.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_bases_sharing.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_buildings.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_control.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_economic_data.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_equipment.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_equipment_HTP.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_equipment_general.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_fuels.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_general_data.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_hx.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_massflow.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_monitoring.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_overview.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_pipes.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_processes.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_renewables.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_res.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_schedules.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_system_scheme.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/data_entry_weather.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/economic_analysis.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/energy_performance.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/energy_statistics.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/faq.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/functions.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/getting_started.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/guide.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/heat_cold_supply.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/heat_recovery.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/installation.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/intro.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/main_menu.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/pinch_analysis.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/process_optimisation.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/questionnaire.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/real_time.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/release_notes.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/report_generation.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/tips_tricks.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_build/html/users/total_cost_analysis.html"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_static/.gitignore"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_themes/einstein/static/einstein.css"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/_themes/einstein/theme.conf"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/conf.py"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/developers/api.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/docs/EINSTEIN Show Case DemoTFC - Description.pdf"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/docs/EINSTEIN Show Case DemoTFC - Materials.zip"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/docs/EINSTEIN_Parameter_List.xls"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/BM_equation.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/CC_blockscheme.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/CC_screenshotconflicts.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/CC_screenshotstandard.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/DBProfiles-Edit.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/DBProfiles-List.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/DataEntryElectricity.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/DataEntryEquipment_1.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/DataEntryEquipment_2.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/DataEntryHTP_1.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/DataEntryHTP_2.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/DataEntryHTP_3.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/ECO_Energy.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/ECO_Investment.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/ECO_Summary.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/EditCon.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/EditHXNet.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/MFEditor_1.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/MFEditor_2.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QHX-Economic.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QHX-EditCon.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QHX-Technical.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QM1.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QM2.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QM3.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QM4.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QPRoc-Description.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QPipe.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QProc-Batch.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QProc-Continuous.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QProc-Demand.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QProc-DetailedSchedule.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QProc-Residual.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/QProc-Schedule.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/Report_1.JPG"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/Report_2.JPG"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/ScreenShotBB.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/ScreenShotBBPopUp.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/ScreenShotST.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/Screenshot_dbST.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/TimeSchedules_1.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/TimeSchedules_2.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/TimeSchedules_3.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/building-parameters1.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/building-parameters2.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/database schema of detailed schedules.svg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/einstein_logo.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/g2876.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/gplv3.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/process-times-batch.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/process-times-batch.svg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/process-times-continuous.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/process-times-continuous.svg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/process-times.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/process-times.svg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/screenshot_simple_schedules.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/screenshot_simple_schedules.xcf"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/simple-process-schedule-fitting.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/simple-process-schedule-fitting.svg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/simple-process-schedule-too-long.png"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/simple-process-schedule-too-long.svg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/images/zunge.jpg"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/index.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/make.bat"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/technical/EINSTEIN Tool Technical Manual.pdf"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/technical/manual.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/HC_BB.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/HC_CH.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/HC_CHP.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/HC_HP.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/HC_ST.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/HC_general.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/HC_storage.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/RTopt.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/RTsim.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/benchmark.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/comparative_analysis.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/consistency_check.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_bases_sharing.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_buildings.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_control.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_equipment.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_equipment_HTP.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_equipment_general.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_fuels.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_general_data.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_hx.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_massflow.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_monitoring.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_overview.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_pipes.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_processes.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_res.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_schedules.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_system_scheme.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/data_entry_weather.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/economic_analysis.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/energy_performance.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/energy_statistics.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/faq.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/functions.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/getting_started.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/guide.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/heat_cold_supply.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/heat_recovery.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/installation.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/intro.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/main_menu.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/pinch_analysis.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/real_time.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/release_notes.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/report_generation.rst"
	Delete /REBOOTOK "$INSTDIR\docs/user_manual/users/tips_tricks.rst"
	Delete /REBOOTOK "$INSTDIR\einstein.bat"
	Delete /REBOOTOK "$INSTDIR\einsteinMVC"
	Delete /REBOOTOK "$INSTDIR\einsteinOPT"
	Delete /REBOOTOK "$INSTDIR\einsteinRT"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_41_AutoPilotResults.xml"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_41_Detailed.xml"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_41_PreliminaryStep.xml"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_41_VisitOnSite.xml"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_42_1a.xml"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_42_1b.xml"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_42_2a.xml"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_42_2b.xml"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_42_2c.xml"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_42_BaseCase.xml"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_43_BaseCase.xml"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_43_HEX example.xml"
	Delete /REBOOTOK "$INSTDIR\projects/audit guide/EINSTEIN_Audit_Guide_43_HREstimate.xml"
	Delete /REBOOTOK "$INSTDIR\questionnaire/EINSTEIN Questionnaire V1.1.ods"
	Delete /REBOOTOK "$INSTDIR\questionnaire/EINSTEIN Questionnaire V1.1.pdf"
	Delete /REBOOTOK "$INSTDIR\questionnaire/EINSTEIN Questionnaire V1.1.xls"
	Delete /REBOOTOK "$INSTDIR\reports/EINSTEIN_report_MASTER_V2.0.ods"
	Delete /REBOOTOK "$INSTDIR\sql/einstein.db"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/DBTitles.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/EINSTEIN Parameters 0.4.csv"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/GUITools.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/HelperClass.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/PreferencesFrame.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/Profiles.xrc"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/Schedule.fbp"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/Schedule.xrc"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/Schedule_xrc-StaticBoxSizer_Names.patch"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/SortListCtrl.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/SortListCtrlXmlHandler.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/Streams.fbp"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/Streams.xrc"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/Streams_xrc-StaticBoxSizer_Names.patch"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/WeatherData.xrc"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/addEquipment_popup.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/conflictFrame.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/conflictSourcesFrame.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/dataEntry/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/dataEntry/editCell.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/dataEntry/tableEntry.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/dialogA.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/dialogDatabase.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/dialogGauge.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/dialogImport.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/dialogLanguage.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/dialogOK.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/dialogPRJ.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/dialogUnits.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/displayCC.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/displayClasses.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/editWindow.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/einsteinGUI.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/einsteinPanel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/flowCharts/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/flowCharts/editCon.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/flowCharts/editHXNet.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/flowCharts/editMF.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/fonts.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/graphics.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/icon.xcf"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/icon_64x64.xcf"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/PanelQ3_Graphics.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/PanelQ5_Graphics_Closed_Circuit.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/PanelQ5_Graphics_Open_Circuit.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/Q3.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/brochureTitlePage.jpg"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/bypass.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/einstein.ico"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/einstein_em.PNG"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/einstein_logo.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/einstein_logo_bg.jpg"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/einstein_logo_small.jpg"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/electricityIcon.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-BB-Boiler.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-BB-Burner.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-BB-Heater.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-BB.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-CH-Abs.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-CH-CT.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-CH-CTDry.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-CH-Comp.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-CH-CompAir.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-CH-FreshWater.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-CH.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-CHP-Engine.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-CHP-GT.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-HP-Comp.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-HP-MVC.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-HTP-RC.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/equipmentIcon-ST.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/fuelIcon.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/gplv3.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/hxIconSnk.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/hxIconSrc.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/icon.xcf"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/icon_64x64.xcf"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/icon_warning.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/martillo.JPG"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/nodeIcon-INT11.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/nodeIcon-INT12.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/nodeIcon-INT13.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/nodeIcon-INT21.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/nodeIcon-INT31.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/parallel.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/pipeIcon-ClosedC.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/pipeIcon-ClosedH.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/pipeIcon-DirectC.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/pipeIcon-DirectH.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/pipeIcon-OpenC.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/pipeIcon-OpenH.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processCIcon-BUI.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processCIcon-I.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processCIcon-IO.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processCIcon-SM.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processCIcon-SMI.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processCIcon-SMO.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processCIcon.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processHCIcon.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processHIcon-BUI.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processHIcon-I.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processHIcon-IO.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processHIcon-SM.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processHIcon-SMI.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processHIcon-SMO.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processHIcon.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/processIcon.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/serial.png"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/summerTime.jpg"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/templates/PanelQ3_Graphics.odg"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/templates/PanelQ5_Graphics_V2.1.odg"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/under_construction.gif"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/img/zunge.jpg"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/bg.po"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/bg/LC_MESSAGES/einstein.mo"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/cat.po"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/cat/LC_MESSAGES/einstein.mo"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/cs_CZ.po"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/cs_CZ/LC_MESSAGES/einstein.mo"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/de.po"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/de/LC_MESSAGES/einstein.mo"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/en.po"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/en/LC_MESSAGES/einstein.mo"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/es.po"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/es/LC_MESSAGES/einstein.mo"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/fr.po"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/fr/LC_MESSAGES/einstein.mo"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/it.po"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/it/LC_MESSAGES/einstein.mo"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/pl.po"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/pl/LC_MESSAGES/einstein.mo"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/si.po"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/si/LC_MESSAGES/einstein.mo"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/sk.po"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/locale/sk/LC_MESSAGES/einstein.mo"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/numCtrl.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelBB.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelBM1.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelBM2.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelBM3.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelCC.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelCH.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelCHP.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelCTRL.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelEA1.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelEA2.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelEA3.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelEA4a.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelEA4c.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelEA5.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelEM1.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelEM2.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelHP.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelHR.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelInfo.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelPA2.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelPRJ.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelQ1.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelQ2_fuel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelQ6.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelQ6B.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelQ7.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelQ8.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelQM.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelReport.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panelST.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/CS/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/CS/panelCS1.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/CS/panelCS2.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/CS/panelCS3.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/CS/panelCS4.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/CS/panelCS5.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/CS/panelCS6.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/CS/panelCS7.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBAuditor.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBAuditorEntry.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBBase.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBBenchmark.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBBenchmarkEntry.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBBoiler.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBBoilerEntry.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBCHP.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBCHPEntry.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBElectricityMix.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBElectricityMixEntry.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBFluid.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBFluidEntry.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBFuel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBFuelEntry.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBHeatPump.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBHeatPumpEntry.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBSolarThermal.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/DB/panelDBSolarThermalEntry.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/ECO/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/ECO/panelECO.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/ECO/panelECO0.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/ECO/panelECO1.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/Energy/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/Energy/panelEA4b.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/Energy/panelEH1.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/Energy/panelEH2.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/Energy/panelEnergy.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/PA/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/PA/panelPA1.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/Q/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/Q/panelQ2_el.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/Q/panelQ3.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/Q/panelQ4.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/Q/panelQ5.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/panelA.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/panels/panelMET.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/plot/XYPlot.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/plot/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/plot/matPanel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/plot/plotPanel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/schedules/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/schedules/profileEditor.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/schedules/profileSelector.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/schedules/scheduleEditor.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/schedules/scheduleViewer.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/status.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/tools/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/tools/dialogOKCancel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/tools/einsteinSimpleDialog.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/tools/filePicker.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/tools/popUp.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/userHelp.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/weatherdata.fbp"
	Delete /REBOOTOK "$INSTDIR\src/einstein/GUI/workfile"
	Delete /REBOOTOK "$INSTDIR\src/einstein/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/auxiliary/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/auxiliary/auxiliary.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/auxiliary/dataTransfer.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/constants.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/control/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/control/default.ini"
	Delete /REBOOTOK "$INSTDIR\src/einstein/control/license.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/database/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/database/dataHub.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/database/dbio.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/einstein.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/globals.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/model/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/model/objects/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/model/objects/elObject.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/model/objects/equipmentObject.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/model/objects/genericObject.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/model/objects/pipeObject.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/model/objects/processObject.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/model/objects/processStreams.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/model/systemConnections.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/model/systemModel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/SpreadsheetProcessing.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/boiler/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/boiler/combustion.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/boiler/moduleBB.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/buildCC.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/buildings.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/calendar.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/ccCheckers.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/ccEquation.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/ccPar.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/ccSchedule.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/ccTracker.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/ccheckFunctions.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkBUI.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkCon.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkECO.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkEq.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkFETel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkFETfuel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkHX.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkMassFlow.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkMatrix.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkPipe.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkProc.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkSchedules.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkTotals.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/checkWHEE.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/dataSet.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/moduleCC.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/ccheck/parameterList.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/checkLicense.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/checkLicense.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/chiller/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/chiller/moduleCH.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/chp/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/chp/moduleCHP.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/compiler.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/control.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/dataHR.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/datectrl.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/economy/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/economy/cashFlow.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/economy/dataECO.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/economy/moduleECO.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/boiler.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/boiler.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/chiller.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/chiller.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/chp.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/chp.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/cyTools.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/cyTools.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/equipment.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/equipment.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/estimateHR.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/estimateHR.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/foreCast.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/heatPump.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/heatPump.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/hx.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/hx.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/moduleEnergy.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/pipe.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/pipe.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/process.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/process.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/realTime.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/simModel.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/simModel.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/simulationNP.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/simulationPlus.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/solar.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/solar.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/storage.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/sunday.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/sunday.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/whee.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energy/whee.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/moduleEA.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/moduleEA1.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/moduleEA2.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/moduleEA3.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/moduleEA4.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/moduleEA5.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/moduleEM1.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/moduleEM2.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/moduleEditCon.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/moduleEditHXNet.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/modulePA1.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/energyStats/modulePA2.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/equipments.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/fluids.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/fluids.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/DB/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/DB/moduleDB.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/ECO/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/ECO/moduleECOPanel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/Energy/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/Energy/moduleEH.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleA.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleEditMF.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/modulePRJ.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/modulePeriodEditor.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleProfileSelector.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleQ2_el.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleQ3.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleQ4.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleQ5.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleQ6.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleQM.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleScheduleEditor.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleScheduleViewer.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleTableEntry.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/guiModules/moduleXY.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatFlow.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatFlow_Tit.pyd"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatFlow_Tit.so"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatPump/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatPump/moduleHP.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatRecovery/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatRecovery/designHXNet.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatRecovery/estimateHR.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatRecovery/heatExchanger.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatRecovery/heatStorage.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatRecovery/hxConnection.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatRecovery/moduleHR.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/heatRecovery/pinchAnalysis.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/hf/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/hf/heatFlowQ_T.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/importHR.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/industry.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/interfaces.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/matPanel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/messageLogger.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/moduleBM.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/moduleCS.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/moduleHC.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/modulePipes.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/modulesClass.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/parseExcel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/parseOO.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/parseSpreadsheet.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/plotPanel.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/processes.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/project.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/schedules/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/schedules/periodSchedule.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/schedules/periodScheduleBuilder.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/schedules/profiles.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/schedules/scheduleTools.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/schedules/schedules.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/solar/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/solar/moduleST.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/solar/sunday.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/spreadsheetIO.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/spreadsheetUtils.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/upgrade.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/weatherdata.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/modules/xmlIO.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/view/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/einstein/view/units.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/dataEntry/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/dataEntry/dataEntry.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/dataEntry/editCell.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/drawing/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/drawing/drawingFrame.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/drawing/drawingPanel.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/drawing/iconGrid.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/drawing/moduleIcon.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/drawing/nodeEditor.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/plot/XYPlot.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/plot/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/plot/barPlot.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/plot/matPanel.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/tools/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/widgets/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/GUI/widgets/rangeSlider.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/auxiliary/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/auxiliary/auxiliary.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/auxiliary/encoding.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/auxiliary/float.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/auxiliary/interpolate.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/control/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/control/messageLogger.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/dateTime/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/dateTime/dateTimeTools.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/fileIO/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/fileIO/config.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/fileIO/fileIO.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/network/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/network/genericNet.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/oo/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/oo/oo.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/optimise/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/optimise/hessian.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/optimise/leastsqTest.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/optimise/leastsqTest_2.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/optimise/optimiser.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/optimise/testFunctions.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/osTools/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/osTools/osTools.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/physics/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/physics/thermo.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/statistics/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/statistics/correlation.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/text/__init__.py"
	Delete /REBOOTOK "$INSTDIR\src/lib/text/textConversion.py"
	Push "$INSTDIR\Prerequisites"

	# remove *.pyc, delete empty dirs
	loop:
		ClearErrors
		Pop $1
		IfErrors done
		Delete /REBOOTOK "$1\*.pyc"
		Delete /REBOOTOK "$1\*.pyo"
		RmDir  /REBOOTOK "$1"
		Goto loop
	done:

	RMDir /r /REBOOTOK $SMPROGRAMS\EINSTEIN

	Delete $DESKTOP\EINSTEIN.lnk
	RMDir /r /REBOOTOK $SMPROGRAMS\EINSTEIN
    
    DeleteRegValue HKLM "${REGKEY}\Components" Main
SectionEnd
#----------------------------------------------------------------------

# Uninstaller functions
Function un.onInit
    ReadRegStr $INSTDIR HKLM "${REGKEY}" Path
    !insertmacro SELECT_UNSECTION Main ${UNSEC0000}
FunctionEnd


#----------------------------------------------------------------------

Section -un.post UNSEC0001
    DeleteRegKey HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)"
    Delete /REBOOTOK $INSTDIR\uninstall.exe
    DeleteRegValue HKLM "${REGKEY}" Path
    DeleteRegKey /IfEmpty HKLM "${REGKEY}\Components"
    DeleteRegKey /IfEmpty HKLM "${REGKEY}"
    RMDir /REBOOTOK $INSTDIR
SectionEnd

#----------------------------------------------------------------------


Function .onSelChange
	GetDlgItem $R1 $HWNDPARENT 1	
FunctionEnd
#----------------------------------------------------------------------
